###################################################################################
# Examples Using PemaByGroup Reference Class
# -- Data must be sorted by grouping variable
# -- Data for each group must fit in memory
# -- Use in distributed computing framework determined by rxDataStep behavior
####################################################################################

options(width = 120)
# A user-defined function to compute the mode
computeMode <- function(x)
{
	if (is.integer(x))
	{
		val <- as.numeric(names(which.max(table(x)))) 
	}
	else if (is.numeric(x))
	{
		meanx <- mean(x, na.rm = TRUE)
		minx <- min(x, na.rm = TRUE)
		if (meanx > minx)
		{
			dd <- density(x, na.rm = TRUE) 
			val <- dd$x[which.max(dd$y)]
		}
		else
		{
			val <- meanx
		}
	}	
	else
	{
		val <- NA
	}
	return( val )
}
	
# Compute the median, min, and mode by group for two variables using a data frame in memory.  Grouping variable is a factor
byGroupPemaObj <- PemaByGroup()
# iris data is already sorted by Species	
groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = iris, groupByVar =  "Species", 
    computeVars = c("Petal.Length", "Sepal.Width"),
		fnList = list(
			median = list(FUN = median, x = NULL, na.rm = TRUE),
			min = list(FUN = min, x = NULL, na.rm = TRUE),
			mode = list(FUN = computeMode, x = NULL)))	
groupVals
		
# Compute by-group median, mean, and mode using integer grouping variable in xdf file
if ("RevoScaleR" %in% .packages())
{
	
xdfFile <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall.xdf")
byGroupPemaObj <- PemaByGroup()
# morDefaultSmall data is already sorted by year
groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, groupByVar =  "year", computeVars = c("creditScore"),
		fnList = list(
			median = list(FUN = median, x = NULL, na.rm = TRUE),
			mean = list(FUN = mean, x = NULL, na.rm = TRUE),
			mode = list(FUN = computeMode, x = NULL)))
groupVals

# Write output to an xdf file
tempOutFile <- tempfile(pattern = "byGroup", fileext = ".xdf")
byGroupPemaObj <- PemaByGroup()
groupValsXdf <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, outData = tempOutFile, 
	    rowsPerWrite = 4, overwrite = TRUE,
	    groupByVar =  "year", computeVars = c("creditScore"),
		fnList = list(
			median = list(FUN = median, x = NULL, na.rm = TRUE),
			mean = list(FUN = mean, x = NULL, na.rm = TRUE),
			mode = list(FUN = computeMode, x = NULL)))
# Read computed summary statistics back into memory to examine results
groupVals <- rxDataStep(inData = tempOutFile)
groupVals
file.remove(tempOutFile)

# Write row-by-row output to an xdf file; output file has same number of rows as input
tempOutFile <- tempfile(pattern = "byGroup", fileext = ".xdf")
byGroupPemaObj <- PemaByGroup()

# This will perform a by-group normalization
myNormalize <- function(x)
{
	ret <- (x - mean(x, na.rm = TRUE))/sd(x, na.rm = TRUE)
	return(ret)
}
normalizeValsXdf <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, outData = tempOutFile, 
	    rowsPerWrite = 4, overwrite = TRUE, otherVars = NULL, # Setting otherVars to NULL will include all vars in output
	    groupByVar =  "year", computeVars = c("creditScore"), 
		fnList = list(
			normalize = list(FUN = myNormalize, x = NULL)))

normalizeVals <- rxDataStep(inData = tempOutFile, numRows = 10)
normalizeVals
file.remove(tempOutFile)

}