# Logistic regression gradient descent
"PemaLogitGD" <- 
	setPemaClass("PemaLogitGD", contains = "PemaGradDescent", 
	
	methods = list(
		"initialize" = function(...) 
		{
			'Initialize object using methods from parent classes'
			callSuper(...)	# always put this first, so that following code can override initializations	
			usingMethods(.pemaMethods) # always call this to provide access to parent methods
		},		
		"fn" = function() 
		{
			'Objective (loss) function for logit gradient descent'
			outputTrace(paste("Entering PemaLogitGD$fn b = ", paste(b, collapse = ", "), "\n"), outTraceLevel = 7)
			z <- X %*% b
			res <- as.numeric(-t(y)%*%z + sum(log(1 + exp(z))))
			outputTrace(paste("Leaving PemaLogitGD$fn fnVal = ", res, "\n"), outTraceLevel = 7)
			res
		},
		"gradientFn" = function() 
		{
			'Gradient function for logit gradient descent'
			outputTrace(paste("Entering PemaLogitGD$gradientFn b = ", paste(b, collapse = ", "), "\n"), outTraceLevel = 7)
            if (require("RevoUtilsMath"))
            {
			    setMKLthreads(1)
            }
			z <- X %*% b
			mu <- plogis(z)
			res <- (t(t(mu - y) %*% X))[,1] # Extract named vector
			outputTrace(paste("Leaving PemaLogitGD$gradientFn gradient = ", paste(res, collapse = ", "), "\n"), outTraceLevel = 7)
			res
		},
		"predict" = function(Xnew, bnew, response = TRUE)
		{
			'Predict response; response = TRUE gives probabilities, else gives log-odds'
			z <- Xnew %*% bnew
			if (response)
				return(plogis(z))
 			else
				return(z)
		}
	
	)
)		
