###################################################################################
# Examples Using PemaWordCount Reference Class
###################################################################################
# Source or run script containing example PemaLogitGD class generator and
# its parent PemaGradDescent

pkgDir <- path.package("RevoPemaR")
gradDescentCode <- file.path(pkgDir, "/demoScripts/PemaGradDescent.R")
source(gradDescentCode)
logitCode <- file.path(pkgDir, "/demoScripts/PemaLogitGD.R")
source(logitCode)

# Generate sample data
"GenerateLogisticData" <- function(numRows, numParams, a, b)
{
	X <- matrix(rnorm(numRows*numParams), numRows, numParams)
	# To generate data, include the intercept
	X1 <- cbind(1, X)
	b1 <- rbind(a, b)
	#b0 <- b1*0.0
	err <- 0.02*matrix(rnorm(numRows), numRows, 1)
	z <- X1 %*% b1 + err
	y <- as.vector((plogis(z) <= runif(numRows)) + 0)
	k = numParams + 1 
	xNames <- paste("X", seq.int(1, numParams), sep="")
	colnames(X) <- xNames
	xy = cbind(X, y)
	dfxy <- as.data.frame(xy)
	return(dfxy)
}

set.seed(12345)
numRows <- 5000
numParams <- 10
a <- -1.0 # intercept
b <- matrix(rnorm(numParams), numParams, 1)

pemaLogitTestData <- GenerateLogisticData(numRows, numParams, a, b)


# Estimate the model
logitGdObj <- PemaLogitGD()
logitFormula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))
result1 <- pemaCompute(pemaObj =logitGdObj, data = pemaLogitTestData, 
	formula = logitFormula, reportProgress = 0)
result1

