#############################################################################################
# Reference Class Generator Function for PemaMean
#############################################################################################

PemaMean <- setPemaClass(
	Class = "PemaMean", 
	contains = "PemaBaseClass",
	# list the fields (member variables)
	fields = list(
		sum = "numeric",
		totalObs = "numeric",
        totalValidObs = "numeric",
		mean = "numeric",
		varName = "character"
		),

	# list the methods (member functions)
	methods = list(
		"initialize" = function(varName = "", ...) 
		{
            'sum, totalValidObs, and mean are all initialized to 0'
            # callSuper calls the method of the parent class
            callSuper(...)			
			usingMethods(.pemaMethods)			
            # Fields are modified in a method by using the non-local assignment operator
			varName <<- varName
			sum <<- 0
			totalObs <<- 0
            totalValidObs <<- 0
			mean <<- 0	
            outputTrace("Leaving PemaMean$initialize\n") # Call after fields are initialized
		},
		"processData" = function(dataList) 
		{
			'Updates the sum and total observations from the current chunk of data.'
			library(methods)
            outputTrace(paste("Starting PemaMean$processData nodename = ", Sys.info()["nodename"], "\n"), outTraceLevel = 7)
			if (is.null(dataList[[varName]]))
				stop( "The variable ", varName, " cannot be found in the data." )
			
            sum <<- sum + sum(as.numeric(dataList[[varName]]), na.rm = TRUE)
            totalObs <<- totalObs + length(dataList[[varName]])
			totalValidObs <<- totalValidObs + sum(!is.na(dataList[[varName]]))
			outputTrace(paste("Leaving PemaMean$processData nodename = ", Sys.info()["nodename"], "sum = ", sum, "\n"), outTraceLevel = 7)
			invisible(NULL)
		},
		"updateResults" = function(pemaMeanObj)
		{
            'Updates the sum and total observations from another PemaMean object.'
			outputTrace(paste("Starting PemaMean$updateResults nodename = ", Sys.info()["nodename"],"\n"), outTraceLevel = 7)
            outputTrace(paste("In PemaMean$updateResults Initial sum:", sum, "pemaMeanObj sum:", pemaMeanObj$sum, "\n" ), outTraceLevel = 5)
			
            sum <<- sum + pemaMeanObj$sum
			totalObs <<- totalObs + pemaMeanObj$totalObs
            totalValidObs <<- totalValidObs + pemaMeanObj$totalValidObs
            outputTrace(paste("In PemaMean$updateResults Updated sum:", sum, "\n" ), outTraceLevel = 5)
			outputTrace(paste("Leaving PemaMean$updateResults nodename = ", Sys.info()["nodename"], "\n"), outTraceLevel = 7)
            invisible(NULL)
		},
		"processResults" = function()
        {
            'Returns the sum divided by the totalValidObs.'
            outputTrace(paste("Starting PemaMean$processResults sum = ", sum, "totalValidObs =", totalValidObs, 
				"nodename = ", Sys.info()["nodename"], "\n"), outTraceLevel = 7)

			if (totalValidObs > 0)
			{
				mean <<- sum/totalValidObs
			}
			else
			{
				mean <<- as.numeric(NA)
			}	
			outputTrace(paste("Leaving PemaMean$processResults mean =", mean, "nodename = ", Sys.info()["nodename"], "\n"), outTraceLevel = 7)
			return( mean )
		},
        "getVarsToUse" = function()
        {
            'Returns the varName.' 
            varName
        }
	) # End of methods
) # End of class generator

