###################################################################################
# Examples Using PemaMean Reference Class
###################################################################################

# The pemaCompute function takes a PemaBaseClass reference class object,
# data, an optional compute context, and any additional variables used in
# the custom analysis object 'initialize' method

#######################################################################################
# Example 1: Using a data frame as data.  The RevoScaleR package is not required.
#######################################################################################

# Show the methdods
PemaMean$methods()

# Get help in the PemaMean initialize method
PemaMean$help(initialize)

# Instantiate an PemaMean reference class
meanPemaObj <- PemaMean()
# See the default values for the fields
meanPemaObj 

# See the source code for initialize
meanPemaObj$initialize

# Call pemaCompute, specifying the custom analyis object, the data, and additional arg
set.seed(67)
pemaCompute(pemaObj = meanPemaObj, 
	data = data.frame(x = rnorm(1000)), varName = "x")

# Note that the reference class object has been updated
meanPemaObj 

# Update meanPemaObj with new data
pemaCompute(pemaObj = meanPemaObj, 
	data = data.frame(x = rnorm(1000)), varName = "x", 
	initPema = FALSE)
meanPemaObj$totalValidObs


#######################################################################################
# Example 1: Using an xdf file as data
#######################################################################################

if ("RevoScaleR" %in% .packages())
{
airXdf <- RxXdfData(file.path(rxGetOption("sampleDataDir"), 
	"AirlineDemoSmall.xdf"))

# Custom Analysis object created above will get re-initialized by pemaCompute
pemaCompute(meanPemaObj, data = airXdf, varName = "ArrDelay", reportProgress = 2)
# Now the values are for the xdf file
meanPemaObj 

# Add more observations to the analysis
pemaCompute(meanPemaObj, data = airXdf, varName = "ArrDelay", initPema = FALSE)
meanPemaObj # Now the values have been updated for the second read of the data
}