###################################################################################
# Examples Using PemaWordCount Reference Class
###################################################################################

# Source or run script containing example PemaLogitGD class generator
pkgDir <- path.package("RevoPemaR")
wordCountCode <- file.path(pkgDir, "/demoScripts/PemaWordCount.R")
source(wordCountCode)

# Create a simple data frame with character data and a factor
myData <- data.frame(comment = 
       c("Today there will be scattered showers", 
         "Looks like mist, followed by drizzle to me", 
         "I see isolated showers to the west",
         "Sprinkles here",
         "No rain today",
         "Nope, don't see rain.",
         "Hardly ever rains this time of year",
         "I'd like a little rain."),
         state = factor(rep(c("Washington", "Arizona"), each = 4)), stringsAsFactors = FALSE)

# Specify a dictionary of words to look for
dictionary = c("rain", "showers", "sprinkles", "mist", "drizzle")

# Generate an PemaWordCount object
PemaWordCount$help(initialize)
myWordCount <- PemaWordCount()

#pemaCompute will initialize myWordCount with ... values
out1 <- pemaCompute(myWordCount, data = myData, varName = "comment", 
    dictionary = dictionary, ratioMultiple = 100)
out1

# Do word counts by group
out2 <- pemaCompute(myWordCount, data = myData, varName = "comment", 
    groupVarName = "state", 
    dictionary = dictionary)
out2


##########################################################################################################
# Data set containing Amazon review summaries of foods
# Requires RevoScaleR package
##########################################################################################################

if ("RevoScaleR" %in% .packages())
{
pkgDir <- path.package("RevoPemaR")
foodRev <- file.path(pkgDir, "/sampleData/foodrev.xdf")
myWordCount <- PemaWordCount()
reviewDictionary = c( "great", "good",  "best", "love", "delicious", "flavor", "excellent", 
"awesome", "fantastic", "tasty", "disappointed", "horrible" )

out1 <- pemaCompute(myWordCount, data = foodRev, varName = "reviewSummary", 
    dictionary = reviewDictionary)
print(out1)
    
out2 <- pemaCompute(myWordCount, data = foodRev, varName = "reviewSummary", 
    groupVarName = "reviewScore", dictionary = reviewDictionary)
print(out2)
}