############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for PemaByGroup - exercising PemaBaseClass and pemaCompute
#
############################################################################################
library(RUnit)
"test.PemaByGroup.aaa.setup" <- function()
{
	

	
    if ("RevoScaleR" %in% .packages())
	{
       	# Create sample data set if RevoScaleR is available
        library(RevoMods)
        
	    set.seed(123)
	    numChunks <- 10
	    numRows <- 10000
	    #pemaTestFileName <<- tempfile(pattern = "PemaByGroup", fileext = ".xdf")
        pemaTestFileName <- file.path(tempdir(), "PemaByGroupTest.xdf")
		append <- "none"
		for (i in 1:numChunks)
		{

			testData <- data.frame(
			   x1 = rnorm(numRows)*10,
			   x2 = runif(numRows, min = 1, max = 10),
			   x3 = rnorm(numRows)*100,
			   xint1 = as.integer(runif(numRows, min = 1, max = 1000)),
			   xint2 = as.integer(runif(numRows, min = 1, max = 2000)),
               xDate = as.Date(runif(numRows, min = 1, max = 3650), origin = "1990-01-01")
			   
			)
			testData$xnum1 <- as.numeric(testData$xint1)
			testData$xfac1 <- factor(testData$xint1, levels = 1:999)
			testData$xchar1 <- sprintf("%04d",testData$xint1) 
			testData$xnum2 <- as.numeric(testData$xint2)
			testData$xfac2 <- factor(testData$xint2, levels = 1:1999)
			miss1 <- as.integer(runif(numRows/10, min = 1, max = numRows))
			testData$xmiss1 <- testData$x1
			testData$xmiss1[miss1] <- NA
			miss2 <- as.integer(runif(numRows/10, min = 1, max = numRows))
			testData$xintmiss1<- testData$xint1
			testData$xintmiss1[miss2] <- NA
			
			rxDataStep(inData = testData, outFile = pemaTestFileName, append = append, overwrite = TRUE)
			append <- "rows"
		}	
	}
}

"test.PemaByGroup.method.initialize" <- function()
{
	#rxOptions(traceLevel = 7)
	analysisObject <- PemaByGroup()

    # Get the values of all the fields at instantiation
    fieldListOrig <- analysisObject$getFieldList()
    
    # Run an pemaCompute to modify fields
    out1 <- pemaCompute(pemaObj = analysisObject, data = iris, groupByVar = "Species", computeVars = c("Petal.Length", "Petal.Width"))
    
    # Re-intialize
    analysisObject$initialize()
    # Get the field values
    fieldListInitialize <- analysisObject$getFieldList()
    
    # Check to make sure everything is reset to initial value
	# Functions may have different environments
	fieldListOrig[["fnList"]] <- NULL
	fieldListInitialize[["fnList"]] <- NULL
    checkIdentical(fieldListOrig, fieldListInitialize)  		
}

"test.PemaByGroup.method.processData" <- function()
{
    byGroupPemaObj <- PemaByGroup()
    byGroupPemaObj$groupByVar = "Species"
	byGroupPemaObj$computeVars = c("Petal.Length", "Sepal.Width")
    byGroupPemaObj$processData( iris )    
	resSetosa <- byGroupPemaObj$resultData[1,]
	setosaData <- iris[iris$Species == "setosa",]
	
	RUnit::checkEquals(resSetosa$Petal.Length.median, median(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.median, median(setosaData$Sepal.Width))
	
    # Get the field values
    fieldList <- byGroupPemaObj$getFieldList()
    
	# Fist two groups should be in result data
    RUnit::checkEquals(fieldList$resultData$Sepal.Width.validObs, as.integer(c(50,50)))
    RUnit::checkEquals(as.character(fieldList$resultData$Species), c("setosa", "versicolor")) 
	
	# Last group should be stored in previousData
    RUnit::checkEquals(fieldList$previousGroupId, "virginica")
	RUnit::checkEquals(length(fieldList$previousData[[1]]), 50)
    
    # Now process again.  This should update the results on this thread
	# but groups will appear twice because the data is not sorted
	# togther
    byGroupPemaObj$processData( iris )    

    # Get the field values
    newFieldList <- byGroupPemaObj$getFieldList()
    
    RUnit::checkEquals(newFieldList$resultData$Sepal.Width.validObs, as.integer(c(50,50,50,50,50)))
    RUnit::checkEquals(as.character(newFieldList$resultData$Species), c("setosa", "versicolor", "virginica", "setosa", "versicolor")) 
	
	# Last group should again be stored in previousData
    RUnit::checkEquals(newFieldList$previousGroupId, "virginica")
	RUnit::checkEquals(length(newFieldList$previousData[[1]]), 50)   
}

"test.PemaByGroup.method.updateResults" <- function()
{
    # Create an PemaByGroup object and process data 
    byGroupPemaObj <- PemaByGroup()
    byGroupPemaObj$groupByVar = "Species"
	byGroupPemaObj$computeVars = c("Petal.Length", "Petal.Width")
	byGroupPemaObj$processData(iris) 

    # Create another PemaMean object and process data    
    byGroupPemaObj2 <- PemaByGroup()
    byGroupPemaObj2$groupByVar = "Species"
	byGroupPemaObj2$computeVars = c("Petal.Length", "Petal.Width")
	byGroupPemaObj2$processData(iris)      
    
    # This is used to update results from one node to another
    byGroupPemaObj$updateResults( byGroupPemaObj2 )    

    # Get the field values
    fieldList <- byGroupPemaObj$getFieldList()
	
	checkTrue(is.null(byGroupPemaObj2$previousData))
    checkTrue(is.null(byGroupPemaObj$previousData))
	
    RUnit::checkEquals(fieldList$resultData$Petal.Length.validObs, as.integer(rep(50, times = 6)))
    RUnit::checkEquals(fieldList$resultData$Species, factor(rep(c("setosa", "versicolor", "virginica"), times = 2))) 	

}

"test.PemaByGroup.method.processResults" <- function()
{
    # Create an PemaByGroup object and process data 
    byGroupPemaObj <- PemaByGroup()
    byGroupPemaObj$groupByVar = "Species"
	byGroupPemaObj$computeVars = c("Petal.Length", "Petal.Width")
	byGroupPemaObj$processData(iris) 

	# Process again
   	byGroupPemaObj$processData(iris) 
        
    # We're done now, so process the final results
    byGroupRes <- byGroupPemaObj$processResults()   

    checkTrue(is.null(byGroupPemaObj$previousData))
	
    RUnit::checkEquals(byGroupRes$Petal.Length.validObs, as.integer(rep(50, times = 6)))
    RUnit::checkEquals(byGroupRes$Species, factor(rep(c("setosa", "versicolor", "virginica"), times = 2))) 	
}
	
"test.PemaByGroup.pemaCompute" <- function()
{
	byGroupPemaObj <- PemaByGroup()
	# Use default fnList
	groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = iris, groupByVar =  "Species", 
		computeVars = c("Petal.Length", "Sepal.Width"))
	
	setosaData <- iris[iris$Species == "setosa",]
	resSetosa <- groupVals[1,]
	RUnit::checkEquals(resSetosa$Petal.Length.median, median(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.median, median(setosaData$Sepal.Width))
    
    # Check for a group with only 1 member
    irisSmall <- iris[1:101,]
	byGroupPemaObj <- PemaByGroup()
	# Use default fnList
	groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = irisSmall, groupByVar =  "Species", 
		computeVars = c("Petal.Length", "Sepal.Width"))
	
	setosaData <- irisSmall[irisSmall$Species == "setosa",]
	resSetosa <- groupVals[1,]
	RUnit::checkEquals(resSetosa$Petal.Length.median, median(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.median, median(setosaData$Sepal.Width))  
    
    resVirginica <- groupVals[3,]
    RUnit::checkEquals(resVirginica$Petal.Length.median, irisSmall[101, "Petal.Length"])
    RUnit::checkEquals(resVirginica$Sepal.Width.median, irisSmall[101, "Sepal.Width"])     
    
	
    if ("RevoScaleR" %in% .packages())
	{
        pemaTestFileName <- file.path(tempdir(), "PemaByGroupTest.xdf")
        rxGetInfo(pemaTestFileName)
		pemaSortedFileName <- tempfile(pattern = "PemaSortByGroup", fileext = ".xdf")
		# Use integer as grouping variable
		tempData <- rxSort(inData = pemaTestFileName, outFile = pemaSortedFileName, sortByVars = "xint1", overwrite = TRUE)
		analysisObj <- PemaByGroup()
		groupMedians1 <- pemaCompute(pemaObj = analysisObj, data = tempData, groupByVar = "xint1", 
			computeVars = c("x1", "xmiss1", "xint2", "xnum2", "xDate"))
        
		# Use factor as grouping variable
		tempData <- rxSort(inData = pemaTestFileName, outFile = pemaSortedFileName, sortByVars = "xfac1", overwrite = TRUE)
		analysisObj <- PemaByGroup()
		groupMedians2 <- pemaCompute(pemaObj = analysisObj, data = tempData, groupByVar = "xfac1", 
			computeVars = c("x1", "xmiss1", "xint2", "xnum2", "xDate"))
        
        # Convert grouping variable to integer so they match
        groupMedians2$xint1 <- as.integer(as.character(groupMedians2$xfac1))
        groupMedians2$xfac1 <- NULL
		RUnit::checkEquals(groupMedians1, groupMedians2)

		# Use character data as grouping variable
		tempData <- rxSort(inData = pemaTestFileName, outFile = pemaSortedFileName, sortByVars = "xchar1", overwrite = TRUE)
		analysisObj <- PemaByGroup()
		groupMedians3 <- pemaCompute(pemaObj = analysisObj, data = tempData, groupByVar = "xchar1", 
			computeVars = c("x1", "xmiss1", "xint2", "xnum2", "xDate"))
		groupMedians3$xint1<- as.integer(groupMedians3$xchar1)
        groupMedians3$xchar1 <- NULL
		RUnit::checkEquals(groupMedians3, groupMedians2)	
		
		file.remove(pemaSortedFileName)
	}
}

"test.PemaByGroup.fields.fnList" <- function()
{
	
	byGroupPemaObj <- PemaByGroup()
	computeMode <- function(x)
	{
		if (is.integer(x))
		{
			val <- as.numeric(names(which.max(table(x)))) 
		}
		else if (is.numeric(x))
		{
			meanx <- mean(x, na.rm = TRUE)
			minx <- min(x, na.rm = TRUE)
			if (meanx > minx)
			{
				dd <- density(x, na.rm = TRUE) 
				val <- dd$x[which.max(dd$y)]
 			}
			else
			{
				val <- meanx
			}
		}	
		else
		{
			val <- NA
		}
		return( val )
	}
	
	groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = iris, groupByVar =  "Species", 
        computeVars = c("Petal.Length", "Sepal.Width"),
		fnList = list(
			median = list(FUN = median, x = NULL, na.rm = TRUE),
			min = list(FUN = min, x = NULL, na.rm = TRUE),
			mode = list(FUN = computeMode, x = NULL)))	
	
	setosaData <- iris[iris$Species == "setosa",]
	resSetosa <- groupVals[1,]
	RUnit::checkEquals(resSetosa$Petal.Length.median, median(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.median, median(setosaData$Sepal.Width))
	
	RUnit::checkEquals(resSetosa$Petal.Length.min, min(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.min, min(setosaData$Sepal.Width))
		
	RUnit::checkEquals(resSetosa$Petal.Length.mode, computeMode(setosaData$Petal.Length))
    RUnit::checkEquals(resSetosa$Sepal.Width.mode, computeMode(setosaData$Sepal.Width))
	
	if ("RevoScaleR" %in% .packages())
	{	
        pemaTestFileName <- file.path(tempdir(), "PemaByGroupTest.xdf")
        rxGetInfo(pemaTestFileName)        

		pemaSortedFileName <- tempfile(pattern = "PemaSortByGroup", fileext = ".xdf")
		
		# Use integer as grouping variable
		tempData <- rxSort(inData = pemaTestFileName, outFile = pemaSortedFileName, sortByVars = "xint1", overwrite = TRUE)
		analysisObj <- PemaByGroup()

		groupValues <- pemaCompute(pemaObj = analysisObj, data = pemaSortedFileName, groupByVar = "xint1", 
			computeVars = c("x1", "xmiss1", "xint2", "xnum2", "xDate"),
			fnList = list(
				median = list(FUN = median, x = NULL, na.rm = TRUE),
				min = list(FUN = min, x = NULL, na.rm = TRUE),
				mode = list(FUN = computeMode, x = NULL)))
		dataGroup1 <- rxDataStep(inData = pemaSortedFileName, varsToKeep = c("x1", "xmiss1", "xint2", "xnum2", "xDate"), rowSelection = xint1 == 1)
		dataGroup288 <- rxDataStep(inData = pemaSortedFileName, varsToKeep = c("x1", "xmiss1", "xint2", "xnum2", "xDate"), rowSelection = xint1 == 288)
		dataGroup999 <- rxDataStep(inData = pemaSortedFileName, varsToKeep = c("x1", "xmiss1", "xint2", "xnum2", "xDate"), rowSelection = xint1 == 999)	
		resGroup1 <- groupValues[1,]
		resGroup288 <- groupValues[288,]
		resGroup999 <- groupValues[999,]
		
		RUnit::checkEquals(resGroup1$x1.median, median(dataGroup1$x1, na.rm = TRUE))
		RUnit::checkEquals(resGroup1$xmiss1.median, median(dataGroup1$xmiss1, na.rm = TRUE))
		RUnit::checkEquals(resGroup1$xint2.median, median(dataGroup1$xint2, na.rm = TRUE))
		RUnit::checkEquals(resGroup1$xnum2.median, median(dataGroup1$xnum2, na.rm = TRUE))
       	RUnit::checkEquals(resGroup1$xDate.median, median(dataGroup1$xDate, na.rm = TRUE))
		
		RUnit::checkEquals(resGroup1$x1.mode, computeMode(dataGroup1$x1))
		RUnit::checkEquals(resGroup1$xmiss1.mode, computeMode(dataGroup1$xmiss1))
		RUnit::checkEquals(resGroup1$xint2.mode, computeMode(dataGroup1$xint2))
		RUnit::checkEquals(resGroup1$xnum2.mode, computeMode(dataGroup1$xnum2))	
		
		RUnit::checkEquals(resGroup288$x1.median, median(dataGroup288$x1, na.rm = TRUE))
		RUnit::checkEquals(resGroup288$xmiss1.median, median(dataGroup288$xmiss1, na.rm = TRUE))
		RUnit::checkEquals(resGroup288$xint2.median, median(dataGroup288$xint2, na.rm = TRUE))
		RUnit::checkEquals(resGroup288$xnum2.median, median(dataGroup288$xnum2, na.rm = TRUE))
		RUnit::checkEquals(resGroup288$xDate.median, median(dataGroup288$xDate, na.rm = TRUE))
        
		RUnit::checkEquals(resGroup288$x1.mode, computeMode(dataGroup288$x1))
		RUnit::checkEquals(resGroup288$xmiss1.mode, computeMode(dataGroup288$xmiss1))
		RUnit::checkEquals(resGroup288$xint2.mode, computeMode(dataGroup288$xint2))
		RUnit::checkEquals(resGroup288$xnum2.mode, computeMode(dataGroup288$xnum2))		
		
		RUnit::checkEquals(resGroup999$x1.median, median(dataGroup999$x1, na.rm = TRUE))
		RUnit::checkEquals(resGroup999$xmiss1.median, median(dataGroup999$xmiss1, na.rm = TRUE))
		RUnit::checkEquals(resGroup999$xint2.median, median(dataGroup999$xint2, na.rm = TRUE))
		RUnit::checkEquals(resGroup999$xnum2.median, median(dataGroup999$xnum2, na.rm = TRUE))
        RUnit::checkEquals(resGroup999$xDate.median, median(dataGroup999$xDate, na.rm = TRUE))
		
		RUnit::checkEquals(resGroup999$x1.mode, computeMode(dataGroup999$x1))
		RUnit::checkEquals(resGroup999$xmiss1.mode, computeMode(dataGroup999$xmiss1))
		RUnit::checkEquals(resGroup999$xint2.mode, computeMode(dataGroup999$xint2))
		RUnit::checkEquals(resGroup999$xnum2.mode, computeMode(dataGroup999$xnum2))		
		
		file.remove(pemaSortedFileName)
	}
}

"test.PemaByGroup.pemaCompute.outFile" <- function()
{
	if ("RevoScaleR" %in% .packages())
	{
		xdfFile <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall.xdf")
		byGroupPemaObj <- PemaByGroup()
		# morDefaultSmall data is already sorted by year
		groupValsDF <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, groupByVar =  "year", computeVars = c("creditScore"),
				fnList = list(
					median = list(FUN = median, x = NULL, na.rm = TRUE),
					mean = list(FUN = mean, x = NULL, na.rm = TRUE)
					))

		# Write output to an xdf file
		tempOutFile <- tempfile(pattern = "byGroup", fileext = ".xdf")
		byGroupPemaObj <- PemaByGroup()
		groupValsXdf <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, outData = tempOutFile, 
				rowsPerWrite = 4, overwrite = TRUE,
				groupByVar =  "year", computeVars = c("creditScore"),
				fnList = list(
					median = list(FUN = median, x = NULL, na.rm = TRUE),
					mean = list(FUN = mean, x = NULL, na.rm = TRUE)
					))
		groupValsRead <- rxDataStep(inData = groupValsXdf)
		RUnit::checkEquals(groupValsDF, groupValsRead)
		
		# Append rows
		groupValsXdf <- pemaCompute(pemaObj = byGroupPemaObj, data = xdfFile, outData = tempOutFile, 
				rowsPerWrite = 4, overwrite = TRUE, append = "rows",
				groupByVar =  "year", computeVars = c("creditScore"),
				fnList = list(
					median = list(FUN = median, x = NULL, na.rm = TRUE),
					mean = list(FUN = mean, x = NULL, na.rm = TRUE)
					))
		groupValsRead2 <- rxDataStep(inData = groupValsXdf)	
		groupValsDF2 <- rbind(groupValsDF, groupValsDF)	
		RUnit::checkEquals(groupValsDF2, groupValsRead2)
		
		file.remove(tempOutFile)
	}
	
}
	
"test.PemaByGroup.pemaCompute.multOutRows" <- function()
{
	if ("RevoScaleR" %in% .packages())
	{	
		# Uneven group sizes
		origFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
		tempInFile <- tempfile(pattern = "byGroupIn", fileext = ".xdf")
		rxSort(inData = origFile, outFile = tempInFile, sortByVars = "xfac2")

		tempOutFile <- tempfile(pattern = "byGroup", fileext = ".xdf")
		byGroupPemaObj <- PemaByGroup()

		# Normalize by group
		myNormalize<- function(x)
		{
			ret <- (x - mean(x, na.rm = TRUE))/sd(x, na.rm = TRUE)
		}

		normalizeValsXdf <- pemaCompute(pemaObj = byGroupPemaObj, data = tempInFile, outData = tempOutFile, 
				rowsPerWrite = 10, overwrite = TRUE, otherVars = c("xint1", "xfac2"),
				groupByVar =  "xfac2", computeVars = c("xnum2", "xnum1"),
				fnList = list(
					normalize = list(FUN = myNormalize, x = NULL)))

		normalizeVals <- rxDataStep(inData = tempOutFile)
		inData <- rxDataStep(tempInFile)
		
		fGroupXnum1 <- as.vector(myNormalize(inData[inData$xfac2 == "f", "xnum1"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "f", "xnum1.normalize"], fGroupXnum1)
		
		gGroupXnum1 <- as.vector(myNormalize(inData[inData$xfac2 == "g", "xnum1"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "g", "xnum1.normalize"],gGroupXnum1)
		
		hGroupXnum1 <- as.vector(myNormalize(inData[inData$xfac2 == "h", "xnum1"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "h", "xnum1.normalize"], hGroupXnum1)
		
		fGroupXnum2 <- as.vector(myNormalize(inData[inData$xfac2 == "f", "xnum2"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "f", "xnum2.normalize"], fGroupXnum2)
		
		gGroupXnum2 <- as.vector(myNormalize(inData[inData$xfac2 == "g", "xnum2"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "g", "xnum2.normalize"],gGroupXnum2)
		
		hGroupXnum2 <- as.vector(myNormalize(inData[inData$xfac2 == "h", "xnum2"]))
		RUnit::checkEquals(normalizeVals[normalizeVals$xfac2 == "h", "xnum2.normalize"], hGroupXnum2)
		
		file.remove(tempInFile)
		file.remove(tempOutFile)
	}
}

"test.PemaByGroup.pemaCompute.multOutFnCols" <- function()
{
    
    inData <- data.frame(
          id = as.integer(rep(1:5, each = 3)),
          income1 = c(500,600,450,   600, 500, 800,    900, 700, 1000,   900, 1200, 1111,  1300, 1200, 1350),
          income2 = c(600,400,400,   650, 400, 750,    950, 800, 1100,   990, 1300, 1100,  1100, 1300, 1400),
          income3 = c(600,400,400,   650, 400, 750,    950, 800, 1100,   990, 1300, 1100,  1100, 1300, 1400))
    
    combineIncomeFn <- function( data )
    {
        minCombineIncome <- min(data[[1]], data[[2]], na.rm = TRUE)
        maxCombineIncome <- max(data[[1]], data[[2]], na.rm = TRUE)
        return(data.frame(minCombineIncome, maxCombineIncome))
    }

	byGroupPemaObj <- PemaByGroup()
	
	groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = inData, groupByVar =  "id", 
        computeVars = c("income1"),
		fnList = list(
			min = list(FUN = min, x = NULL, na.rm = TRUE),
            combineIncome = list(FUN = combineIncomeFn, data = c("income2")),
			max = list(FUN = max, x = NULL, na.rm = TRUE)))	
    RUnit::checkEquals(groupVals$minCombineIncome, c(400, 400, 700, 900, 1100))
    RUnit::checkEquals(groupVals$maxCombineIncome, c(600, 800, 1100, 1300, 1400))
    RUnit::checkEquals(groupVals$income1.min, c( 450, 500, 700, 900, 1200))
    RUnit::checkEquals(groupVals$income1.max, c(600, 800, 1000, 1200, 1350))
    
    inDataSmall <- inData[1:13,]  # Use a group with a single observation
    
	byGroupPemaObj <- PemaByGroup()
	
	groupVals <- pemaCompute(pemaObj = byGroupPemaObj, data = inDataSmall, groupByVar =  "id", 
        computeVars = c("income1"),
		fnList = list(
			min = list(FUN = min, x = NULL, na.rm = TRUE),
            combineIncome = list(FUN = combineIncomeFn, data = c("income2", "income3")),
			max = list(FUN = max, x = NULL, na.rm = TRUE)))	
    RUnit::checkEquals(groupVals$minCombineIncome, c(400, 400, 700, 900, 1100))
    RUnit::checkEquals(groupVals$maxCombineIncome, c(600, 800, 1100, 1300, 1300))
    RUnit::checkEquals(groupVals$income1.min, c( 450, 500, 700, 900, 1300))
    RUnit::checkEquals(groupVals$income1.max, c(600, 800, 1000, 1200, 1300))    
}

"test.PemaByGroup.pemaCompute.outputDifferentRowNums" <- function()
{
    lastRows <- function(dataList, firstRowNum)
    {
        numRows <- length(dataList[[1]])
        if (numRows < firstRowNum)
        {
            return( NULL )
        }
        
        dataList <- lapply(dataList, function(x)x[firstRowNum:numRows])

        dataList
    }
  
    ret <- pemaCompute(pemaObj = PemaByGroup(), data = mtcars, outData = NULL, 
                         groupByVar = "carb", 
                         computeVars = list(lastRows = ".all"),
                         fnList = list(
                           lastRows = list(FUN=lastRows, dataList=NULL, firstRowNum = 6)))
    ret <- ret[order(ret$carb),]
    manualCompute <- sapply(X = 1:8, FUN = function(X, data, varName, firstRow){
            thisData <- data[data[[varName]] == X,]
            numRows <- nrow(thisData)
            if (numRows < firstRow)
            {
                return( NULL )
            }
            return( thisData[firstRow:numRows,])
       }, mtcars, "carb", 6)
    manualDF <- NULL
    for (i in 1:length(manualCompute))
    {
        if (is.null(manualDF))
        {
            manualDF <- manualCompute[[i]]
        }
        else if (!is.null(manualCompute[[i]]))
        {       
            manualDF <- rbind(manualDF, manualCompute[[i]])
        }
    }
    row.names(manualDF) <- NULL
    row.names(ret) <- NULL
    RUnit::checkEquals(manualDF, ret) 
    
	if ("RevoScaleR" %in% .packages())
	{	    
        tempInFile <- tempfile(pattern = "byGroupIn1", fileext = ".xdf")
        tempOutFile <- tempfile(pattern = "byGroupOut1", fileext = ".xdf")
        mtcarsXdf <- rxDataStep(inData = mtcars, outFile = tempInFile, rowsPerRead = 10,
              overwrite = TRUE)
        
        tempInFileXdf <- rxSort(inData = tempInFile, outFile = tempInFile, sortByVars = "carb", overwrite = TRUE)
        retXdf <- pemaCompute(pemaObj = PemaByGroup(), data = tempInFile, outData = tempOutFile, 
                             groupByVar = "carb", 
                             computeVars = list(lastRows = ".all"),
                             fnList = list(
                               lastRows = list(FUN=lastRows, dataList=NULL, firstRowNum = 6)),
                             overwrite = TRUE)
        retDF <- rxDataStep(tempOutFile)
        retDF <- retDF[order(retDF$carb),]  
        row.names(retDF) <- NULL 
        RUnit::checkEquals(manualDF, retDF)  
        unlink(tempInFile)
        unlink(tempOutFile)       
    }
           
}

"test.PemaByGroup.zzz.cleanup" <- function()
{
    pemaTestFileName <- file.path(tempdir(), "PemaByGroupTest.xdf")
	if (file.exists(pemaTestFileName))
	{
		file.remove(pemaTestFileName)
	}
	
	#rm(pemaTestFileName, inherits = TRUE)
}
