############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for PemaMean - exercising PemaBaseClass and pemaCompute
#
############################################################################################
library(RUnit)

"test.PemaMean.method.initialize" <- function()
{
    meanPemaObj <- PemaMean()
    # Get the values of all the fields at instantiation
    fieldListOrig <- meanPemaObj$getFieldList()
    
    # Run an pemaCompute to modify fields
    out1 <- pemaCompute(pemaObj = meanPemaObj, data = iris, varName = "Petal.Length")
    
    # Re-intialize
    meanPemaObj$initialize()
    # Get the field values
    fieldListInitialize <- meanPemaObj$getFieldList()
    
    # Check to make sure everything is reset to initial value
    checkIdentical(fieldListOrig, fieldListInitialize)   
}

"test.PemaMean.method.processData" <- function()
{
    meanPemaObj <- PemaMean()
    meanPemaObj$varName = "Petal.Length"
    meanPemaObj$processData( iris )    

    # Get the field values
    fieldList <- meanPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, nrow(iris))
    RUnit::checkEquals(fieldList$sum, sum(iris$Petal.Length)) 
    RUnit::checkEquals(fieldList$totalValidObs, nrow(iris))
    RUnit::checkEquals(fieldList$mean, 0)  # Mean is not updated in processData
    
    # Now process again.  This should update the results on this thread
    meanPemaObj$processData( iris )    

    # Get the field values
    fieldList <- meanPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(iris))
    RUnit::checkEquals(fieldList$sum, 2*sum(iris$Petal.Length)) 
    RUnit::checkEquals(fieldList$totalValidObs, 2*nrow(iris))
    RUnit::checkEquals(fieldList$mean, 0)  # Mean is not updated in processData    
}

"test.PemaMean.method.updateResults" <- function()
{
    # Create an PemaMean object and process data 
    meanPemaObj <- PemaMean()
    meanPemaObj$varName = "Petal.Length"
    meanPemaObj$processData( iris )    

    # Create another PemaMean object and process data    
    meanPemaObj2 <- PemaMean()
    meanPemaObj2$varName = "Petal.Length"
    meanPemaObj2$processData( iris )        
    
    # This is used to update results from one node to another
    meanPemaObj$updateResults( meanPemaObj2 )    

    # Get the field values
    fieldList <- meanPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(iris))
    RUnit::checkEquals(fieldList$sum, 2*sum(iris$Petal.Length)) 
    RUnit::checkEquals(fieldList$totalValidObs, 2*nrow(iris))
    RUnit::checkEquals(fieldList$mean, 0)  # Mean is not updated in updateResults  
}

"test.PemaMean.method.processResults" <- function()
{
    # Create an PemaMean object and process data 
    meanPemaObj <- PemaMean()
    meanPemaObj$varName = "Petal.Length"
    meanPemaObj$processData( iris )    

    # Create another PemaMean object and process data    
    meanPemaObj2 <- PemaMean()
    meanPemaObj2$varName = "Petal.Length"
    meanPemaObj2$processData( iris )        
    
    # This is used to update results from one node to another
    meanPemaObj$updateResults( meanPemaObj2 ) 
    
    # We're done now, so process the final results
    meanPemaObj$processResults()   

    # Check the mean
    RUnit::checkEquals(meanPemaObj$mean, mean(iris$Petal.Length))  
}

"test.PemaMean.pemaCompute" <- function()
{
    # Instantiate an PemaMean reference class
    meanPemaObj <- PemaMean()

    # Compute the mean of Petal.Length from the iris data set
    # Call pemaCompute, specifying the custom analyis object, the data, and additional arg
    res <- pemaCompute(pemaObj = meanPemaObj, data = iris, varName = "Petal.Length")

    RUnit::checkEquals(meanPemaObj$totalObs, nrow(iris))
    RUnit::checkEquals(meanPemaObj$sum, sum(iris$Petal.Length)) 
    RUnit::checkEquals(meanPemaObj$totalValidObs,nrow(iris))
    RUnit::checkEquals(meanPemaObj$mean, mean(iris$Petal.Length))
 	RUnit::checkEquals(res, meanPemaObj$mean)

}
