############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for PemaPopularWords - exercising PemaBaseClass and pemaCompute
#
############################################################################################
library(RUnit)
"test.PemaPopularWords.aaa.setup" <- function()
{
    # Create a simple data frame with character data and a factor
   pemaTestData <<- data.frame(comment = 
       c("Today there will be scattered showers", 
         "Looks like mist, followed by drizzle to me", 
         "I see isolated showers to the west",
         "Sprinkles here",
         "No rain today",
         "Nope, don't see rain.",
         "Hardly ever get rain this time of year",
         "I'd like a little rain."),
         state = factor(rep(c("Washington", "Arizona"), each = 4)), stringsAsFactors = FALSE)
		
    # Create a simple data frame with character data and a factor
	pemaTestDataNA <<- data.frame(
	   comment =  as.character(c(NA, NA, NA, NA, NA, NA, NA, NA)),
       state = factor(rep(c("Washington", "Arizona"), each = 4)), stringsAsFactors = FALSE)		

	pkgDir <- path.package("RevoPemaR")
	popWordsCode <- file.path(pkgDir, "/demoScripts/PemaPopularWords.R")
	source(popWordsCode)
}

"test.PemaPopularWords.method.initialize" <- function()
{
    popWordsPemaObj <- PemaPopularWords()
    # Get the values of all the fields at instantiation
    fieldListOrig <- popWordsPemaObj$getFieldList()
    
    # Run an pemaCompute to modify fields
    out1 <- pemaCompute(pemaObj = popWordsPemaObj, data = pemaTestData, 
        varName = "comment", minWordSize = 2, minWordCount = 1,
        omitWords = c("the", "to", "be", "of", "by"))
    
    # Re-intialize
    popWordsPemaObj$initialize()
    # Get the field values
    fieldListInitialize <- popWordsPemaObj$getFieldList()
    
    # Check to make sure everything is reset to initial value
    checkIdentical(fieldListOrig, fieldListInitialize)   
}

"test.PemaPopularWords.method.processData" <- function()
{
    popWordsPemaObj <- PemaPopularWords()
    popWordsPemaObj$varName <- "comment"
    popWordsPemaObj$processData( pemaTestData )    

    # Get the field values
    fieldList <- popWordsPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, nrow(pemaTestData))
    RUnit::checkEquals(fieldList$popularWords[1,1],"rain")
    RUnit::checkEquals(fieldList$popularWords[1,2],4)
    RUnit::checkEquals(fieldList$popularWords[3,1],"showers")
    RUnit::checkEquals(fieldList$popularWords[3,2],2)
    
    # Now process again.  This should update the results on this thread
    popWordsPemaObj$processData( pemaTestData)    

    # Get the field values
    fieldList <- popWordsPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$popularWords[1,1],"rain")
    RUnit::checkEquals(fieldList$popularWords[1,2],8)
    RUnit::checkEquals(fieldList$popularWords[3,1],"showers")
    RUnit::checkEquals(fieldList$popularWords[3,2],4)

}

"test.PemaPopularWords.method.updateResults" <- function()
{
    # Create an PemaPopularWords object and process data 
    popWordsPemaObj <- PemaPopularWords()
    popWordsPemaObj$varName <- "comment"
    popWordsPemaObj$processData( pemaTestData )    

    # Create another PemaPopularWords object and process data    
    popWordsPemaObj2 <- PemaPopularWords()
    popWordsPemaObj2$varName <- "comment"
    popWordsPemaObj2$processData( pemaTestData )      
    
    # This is used to update results from one node to another
    popWordsPemaObj$updateResults( popWordsPemaObj2 )    

    # Get the field values  
    fieldList <- popWordsPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$popularWords[1,1],"rain")
    RUnit::checkEquals(fieldList$popularWords[1,2],8)
    RUnit::checkEquals(fieldList$popularWords[3,1],"showers")
    RUnit::checkEquals(fieldList$popularWords[3,2],4)
}

"test.PemaPopularWords.method.updateResults.NAs" <- function()
{
    # Create an PemaPopularWords object and process data 
    popWordsPemaObj <- PemaPopularWords()
    popWordsPemaObj$varName <- "comment"
    popWordsPemaObj$processData( pemaTestData )    

    # Create another PemaPopularWords object and process data    
    popWordsPemaObj2 <- PemaPopularWords()
    popWordsPemaObj2$varName <- "comment"
    popWordsPemaObj2$processData( pemaTestDataNA )      
    
    # This is used to update results from one node to another
    popWordsPemaObj$updateResults( popWordsPemaObj2 )  
  	
    # Get the field values  
    fieldList <- popWordsPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$popularWords[1,1],"rain")
    RUnit::checkEquals(fieldList$popularWords[1,2],4)
    RUnit::checkEquals(fieldList$popularWords[3,1],"showers")
    RUnit::checkEquals(fieldList$popularWords[3,2],2)
	
	# Repeat with all NAs
    # Create an PemaPopularWords object and process data 
    popWordsPemaObj <- PemaPopularWords()
    popWordsPemaObj$varName <- "comment"
    popWordsPemaObj$processData( pemaTestDataNA )    

    # Create another PemaPopularWords object and process data    
    popWordsPemaObj2 <- PemaPopularWords()
    popWordsPemaObj2$varName <- "comment"
    popWordsPemaObj2$processData( pemaTestDataNA )      
    
    # This is used to update results from one node to another
    popWordsPemaObj$updateResults( popWordsPemaObj2 )  
  	
    # Get the field values  
    fieldList <- popWordsPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$popularWords[1,1],NA)	
}	
	

"test.PemaPopularWords.method.processResults" <- function()
{
    # Create an PemaPopularWords object and process data 
    popWordsPemaObj <- PemaPopularWords()
    popWordsPemaObj$varName <- "comment"
    popWordsPemaObj$processData( pemaTestData )    

    # Create another PemaPopularWords object and process data    
    popWordsPemaObj2 <- PemaPopularWords()
    popWordsPemaObj2$varName <- "comment"
    popWordsPemaObj2$processData( pemaTestData )      
    
    # This is used to update results from one node to another
    popWordsPemaObj$updateResults( popWordsPemaObj2 )   
    
    # We're done now, so process the final results
    out1 <- popWordsPemaObj$processResults()   

    # Check the mean
    RUnit::checkEquals(as.integer(out1["rain"]), 8) 
    RUnit::checkEquals(as.integer(out1["showers"]), 4) 
}

"test.PemaPopularWords.pemaCompute" <- function()
{
    # Instantiate an PemaPopularWords reference class
    popWordsPemaObj <- PemaPopularWords()
    
    # Compute the word count for the test data set
    # Call pemaCompute, specifying the custom analyis object, the data, and additional arg
    out1 <- pemaCompute(pemaObj = popWordsPemaObj, data = pemaTestData, varName = "comment",
        minWordSize = 2L, minWordCount = 1L,
        omitWords = c("the", "to", "be", "of", "by"))

    RUnit::checkEquals(popWordsPemaObj$totalObs, nrow(pemaTestData))
    RUnit::checkEquals(as.integer(out1["rain"]), 4) 
    RUnit::checkEquals(as.integer(out1["showers"]), 2) 
    RUnit::checkEquals(as.integer(out1["sprinkles"]), 1) 
    RUnit::checkEquals(as.integer(out1["mist"]), 1) 
    RUnit::checkEquals(popWordsPemaObj$popularWords[1, 1], "rain")
    RUnit::checkEquals(popWordsPemaObj$popularWords[1, 2], 4)
    RUnit::checkEquals(popWordsPemaObj$popularWords[2, 1], "like")
    RUnit::checkEquals(popWordsPemaObj$popularWords[2, 2], 2)
    RUnit::checkEquals(popWordsPemaObj$popularWords[3, 1], "see")
    RUnit::checkEquals(popWordsPemaObj$popularWords[3, 2], 2) 
	
    
    # Compute the word count for the test data set, changing the minWordCount
    # Call pemaCompute, specifying the custom analyis object, the data, and additional arg
    out2 <- pemaCompute(pemaObj = popWordsPemaObj, data = pemaTestData, varName = "comment",
        minWordSize = 2, minWordCount = 2,
        omitWords = c("the", "to", "be", "of", "by"))

    RUnit::checkEquals(popWordsPemaObj$totalObs, nrow(pemaTestData))
    RUnit::checkEquals(as.integer(out2["rain"]), 4) 
    RUnit::checkEquals(as.integer(out2["showers"]), 2) 
    RUnit::checkEquals(as.integer(out2["today"]), 2) 
    RUnit::checkEquals(length(out2), 5)
    RUnit::checkEquals(popWordsPemaObj$popularWords[1, 1], "rain")
    RUnit::checkEquals(popWordsPemaObj$popularWords[1, 2], 4)
    RUnit::checkEquals(popWordsPemaObj$popularWords[2, 1], "like")
    RUnit::checkEquals(popWordsPemaObj$popularWords[2, 2], 2)
    RUnit::checkEquals(popWordsPemaObj$popularWords[3, 1], "see")
    RUnit::checkEquals(popWordsPemaObj$popularWords[3, 2], 2) 	
}

"test.PemaPopularWords.zzz.cleanup" <- function()
{
    rm(pemaTestData, inherits = TRUE)
	rm(pemaTestDataNA, inherits = TRUE)
}
