############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for setPemaClass 
#
############################################################################################
library(RUnit)
"test.setPemaClass.example" <- function()
{
	# A very simple example of computing a sum
	PemaSum <- setPemaClass("PemaSum", 
		contains = "PemaBaseClass",
		
		fields = list( 
			# list the fields (member variables)
			sum = "numeric",
			varName = "character"
			),

		methods = list(
			# list the methods (member functions)
			initialize = function(varName = "", ...) 
			{
				'sum is initialized to 0'  
				# callSuper calls the method of the parent class
				callSuper(...)		        					
				usingMethods(.pemaMethods)	# Will include methods if includeMethods is TRUE		
				# Fields are modified in a method by using the non-local assignment operator
				varName <<- varName
				sum <<- 0
			},
			processData = function(dataList) 
			{
				'Updates the sum  from the current chunk of data.'
				callSuper(dataList)
				sum <<- sum + sum(as.numeric(dataList[[varName]]), na.rm = TRUE)
				invisible(NULL)
			},
			updateResults = function(pemaSumObj)
			{
				'Updates the sum from another PemaSum object.'
				callSuper(pemaSumObj)
				sum <<- sum + pemaSumObj$sum
				invisible(NULL)
			},
			processResults = function()
			{
				'Returns the sum. No further computations required.'
				callSuper()
				sum
			},
			getVarsToUse = function()
			{
				'Returns the varName.' 
				varName
			}
		)
	)

    PemaSum$fields()  # show the field names and types
	PemaSum$methods() # list the methods
	
	pemaSumObj <- PemaSum()
	sumOut <- pemaCompute(pemaSumObj, data = iris, varName = "Sepal.Length")
	RUnit::checkEquals(sumOut, sum(iris$Sepal.Length))
	
	fieldList1 <- pemaSumObj$getFieldList()
	
	# Add more obs
	sumOut2 <- pemaCompute(pemaSumObj, data = iris, varName = "Sepal.Length", initPema = FALSE)
    RUnit::checkEquals(sumOut2, 2*sum(iris$Sepal.Length))
	
	# Use an .xdf file if RevoScaleR is loaded
	if (pemaSumObj$useRevoScaleR == TRUE)
	{
		airXdf <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
		sumOut3 <- pemaCompute(pemaSumObj, data = airXdf, varName = "ArrDelay")
		# rxSumOut <- rxSummary(~ArrDelay, data = airXdf, summaryStats = "Sum")
		RUnit::checkEquals(sumOut3, 6594146)
	}
	
		
}
