# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for Date data type
#
############################################################################################

"test.Date.variable.rxCube.rowSelection" <- function()
{
    form <- ArrDelay ~ UniqueCarrier
    XDF <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    refDate <- as.Date(ISOdate(2008,8,26))
    zXDF <- rxCube(form, XDF, removeZeroCounts = TRUE, 
        rowSelection = Date > refDate,
        transformObjects = list(refDate = refDate),
        returnDataFrame = TRUE)
    DF <- subset(rxReadXdf(XDF), Date > refDate)
    zDF <- rxCube(form, DF, removeZeroCounts = TRUE, returnDataFrame = TRUE)
    checkEquals(as.data.frame(zXDF), as.data.frame(zDF))
}

"test.Date.variable.integer" <- function()
{
    # Do round trip of Data data stored as integer
    
    myData <- data.frame(myDate= as.Date(seq(as.Date("2012/1/1"), by = "day", length.out = 10)))
    testXdf <- tempfile(pattern = ".rxDate1", fileext = ".xdf")
    rxDataStep(myData, outFile = testXdf, overwrite = TRUE)
    myData2 <- rxReadXdf(testXdf)
    checkEquals(myData, myData2)
    file.remove(testXdf)
}
"test.Date.variable.rxGetVarInfo" <- function()
{
    XDF <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    dateVar <- rxGetVarInfo(XDF)[["Date"]]
    checkEquals(dateVar[["varType"]], "Date")
    checkEquals(dateVar[["storage"]], "float64")
}

"test.Date.variable.read.write.read" <- function()
{
    XDF1 <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    XDF2 <- file.path(tempdir(), ".rxDate1Temp.xdf")
    if (file.exists(XDF2)) file.remove(XDF2)
    
    DF1 <- rxReadXdf(XDF1)
    rxDataStep(DF1, XDF2)
    DF2 <- rxReadXdf(XDF2)
    
    checkEquals(DF1, DF2)
    if (file.exists(XDF2)) file.remove(XDF2)
}

# Date variables not supported in rxTextToXdf
# Test left commented out in case it is sometime added
#"test.Date.variable.rxTextToXdf" <- function()
#{
    #DEACTIVATED("Pending fix for bug RA-2874")
    #XDF1 <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    #XDF2 <- file.path(tempdir(), ".rxTemp.xdf")
    #TXT <- file.path(tempdir(), ".rxTemp.txt")
    #if (file.exists(TXT)) file.remove(TXT)
    #if (file.exists(XDF2)) file.remove(XDF2)
    #
    ## there ...
    #rxXdfToText(XDF1, TXT, overwrite = TRUE)
    ##file.show(TXT)
    #
    ## ... and back again    
    #rxTextToXdf(TXT, XDF2, firstRowIsColNames = TRUE, 
        #colClasses = sapply(rxGetVarInfo(XDF1), "[[", "varType"))
    #
    #DF1 <- rxReadXdf(XDF1, varsToKeep = "Date")
    #DF2 <- rxReadXdf(XDF2, varsToKeep = "Date")
    #checkEquals(DF1, DF2)
    #
    #if (file.exists(TXT)) file.remove(TXT)
    #if (file.exists(XDF2)) file.remove(XDF2)
#}

"test.Date.variable.rxTextToXdf.as.Date" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  
  txtFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.csv")
  xdfFile <- file.path(tempdir(), ".rxDate2Temp.xdf")
  
  # Import as character data
  rxTextToXdf( inFile = txtFile, outFile = xdfFile, overwrite=TRUE)
  # rxReadXdf( file = xdfFile, numRows = 5)
  # Example of Date variable is 1996-08-18
  
  # Reimport as date
  rxTextToXdf( inFile = txtFile, outFile = xdfFile,
    transforms = list(Date = as.Date(Date, "%Y-%m-%d")),
    overwrite = TRUE)
  
  varInfo <- rxGetVarInfo( xdfFile )
  checkEquals( varInfo$Date$varType, "Date")
  myData <- rxReadXdf( file=xdfFile, numRows = 1)
  checkEquals( myData$Date[1], as.Date("1996-08-18", "%Y-%m-%d"))
  
  if (file.exists( xdfFile )) file.remove( xdfFile )
}

"test.Date.variable.rxImport" <- function()
{
    XDF1 <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    XDF2 <- file.path(tempdir(), ".rxDate3Temp.xdf")
    TXT <- file.path(tempdir(), ".rxDate3Temp.txt")
    if (file.exists(TXT)) file.remove(TXT)
    if (file.exists(XDF2)) file.remove(XDF2)
    
    # there ...
    rxXdfToText(XDF1, TXT, overwrite = TRUE)
    
    # ... and back again
    TXTDS <- RxTextData(TXT, firstRowIsColNames = TRUE,
    readDateFormat = "%Y-%m-%d",
        colClasses = sapply(rxGetVarInfo(XDF1), "[[", "varType"))
    rxImport(TXTDS, XDF2, overwrite = TRUE)
    
    DF1 <- rxReadXdf(XDF1, varsToKeep = "Date")
    DF2 <- rxReadXdf(XDF2, varsToKeep = "Date")
    checkEquals(DF1, DF2, check.attributes=FALSE)
    
    if (file.exists(TXT)) file.remove(TXT)
    if (file.exists(XDF2)) file.remove(XDF2)
}

"test.Date.variable.import.SAS" <- function()
{
    #DEACTIVATED("Pending example of SAS data set with date data")
    #
    #XDF1 <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    #XDF2 <- file.path(tempdir(), ".rxTemp.xdf")
    #if (file.exists(XDF2)) file.remove(XDF2)
    #
  #SAS <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.sas7bdat")
  #SASDS <- RxSasData(SAS, stringsAsFactors = TRUE)
  #rxImport(inData = SASDS, outFile = XDF2, overwrite = TRUE)
    #
    #DF1 <- rxReadXdf(XDF1, varsToKeep = "Date")
    #DF2 <- rxReadXdf(XDF2, varsToKeep = "Date")
    #checkEquals(DF1, DF2, check.attribues = FALSE)
  
    # if (file.exists(XDF2)) file.remove(XDF2)
  
  # Testing SAS data using transforms with as.Date

  #XDF1 <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
  XDF1 <- file.path(tempdir(), ".rxTemp1.xdf")
    if (file.exists(XDF1)) file.remove(XDF1)
  XDF2 <- file.path(tempdir(), ".rxTemp2.xdf")
    if (file.exists(XDF2)) file.remove(XDF1)

  SAS <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.sas7bdat")
  SASDS <- RxSasData(SAS)
  
  rxImport(inData = SASDS, outFile = XDF1, overwrite = TRUE)
  
  # In SAS file data is strings with format 30-Jul-1998
  # %b abbreviated month name
  # %B full month name
  # %m month as number (1-12)
  # %d day of the month
  # %y year, last two digits
  # %Y year, 
  rxImport(inData = SASDS, outFile = XDF2, 
    transforms = list(Date = as.Date(Date, format="%Y-%m-%d")), 
    overwrite = TRUE)

  DF1 <- rxReadXdf(XDF1)
    DF2 <- rxReadXdf(XDF2)
    checkEquals(as.Date(DF1$Date, format="%Y-%m-%d"), DF2$Date)
  
  file.remove( XDF1 )
  file.remove( XDF2 )
  
}

"test.Date.variable.import.SPSS" <- function()
{

    XDF1 <- file.path(tempdir(), ".rxTemp1.xdf")
  if (file.exists(XDF1)) file.remove(XDF1)
    
  SPSS <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.sav")
  SPSSDS <- RxSpssData(SPSS)
  
  rxImport(inData = SPSSDS, outFile = XDF1,
    rowSelection = Date > as.Date("1999-12-31", format = "%Y-%m-%d"),
    overwrite = TRUE)

  DF1 <- rxReadXdf(XDF1, varsToKeep = "Date")
  centuryVec <- as.integer(substr(DF1$Date, 1, 2))

    checkEquals(centuryVec, rep(20, times=nrow(DF1)))
    
    if (file.exists(XDF1)) file.remove(XDF1)
}

