# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxHpcServer and RxLsfCluster
#
############################################################################################

".rxNodeSetDiff" <- function()
{
    if (exists("rxNodeSetDiff", globalenv()))
        get("rxNodeSetDiff", globalenv())
    else
        RevoScaleR:::rxNodeSetDiff
}

".rxNodeSetIntersect" <- function()
{
    if (exists("rxNodeSetIntersect", globalenv()))
        get("rxNodeSetIntersect", globalenv())
    else
        RevoScaleR:::rxNodeSetIntersect
}

".rxuGetComputeContextDescription" <- function()
{
    if (exists("rxuGetComputeContextDescription", globalenv()))
        get("rxuGetComputeContextDescription", globalenv())
    else
        RevoScaleR:::rxuGetComputeContextDescription
}

.setUp <- function()
{
}

.tearDown <- function()
{
}

"test.rxNodeSetDiff.RxHpcServer.hostnames" <- function()
{
  rxNodeSetDiff <- .rxNodeSetDiff()
  rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  
  checkEquals( character(0), rxNodeSetDiff( character(0), c( "CLASSD" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( "CLASSD", rxNodeSetDiff( c( "CLASSD" ), character(0), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetDiff( c( "classD" ), c( "CLASSD" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( toupper("classD_2"), rxNodeSetDiff( c( "classD_1", "classD_2" ), c( "CLASSD_1" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetDiff( c( "classD" ), c( "CLASSD", "classD_2" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort(toupper(c("classD_2","classd_5"))), sort( rxNodeSetDiff( c( "classD_1", "classD_2","classD_4","ClassD_5" ), c( "CLASSD_1","CLassD_3","CLaSsD_4" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) ) )
}

#"test.rxNodeSetDiff.RxLsfCluster.hostnames" <- function()
#{
  #rxNodeSetDiff <- .rxNodeSetDiff()
  #rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
#
  #checkEquals( character(0), rxNodeSetDiff( character(0), c( "CLASSD" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( "CLASSD", rxNodeSetDiff( c( "CLASSD" ), character(0), rxuGetComputeContextDescription("RxLsfCluster") ) )	
  #checkEquals( character(0), rxNodeSetDiff( c( "classD" ), c( "CLASSD" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( toupper("classD_2"), rxNodeSetDiff( c( "classD_1", "classD_2" ), c( "CLASSD_1" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "classD_1" ), c( "CLASSD_1", "classD_2" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort(toupper(c("classD_2","classd_5"))), sort( rxNodeSetDiff( c( "classD_1", "classD_2","classD_4","ClassD_5" ), c( "CLASSD_1","CLassD_3","CLaSsD_4" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) ) )
#}

"test.rxNodeSetIntersect.RxHpcServer.hostnames" <- function()
{
  rxNodeSetIntersect <- .rxNodeSetIntersect()
  rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  
  checkEquals( character(0), rxNodeSetIntersect( character(0), c( "CLASSD" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetIntersect( c( "CLASSD" ), character(0), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( "CLASSD", rxNodeSetIntersect( c( "classD" ), c( "CLASSD" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( "CLASSD_1", rxNodeSetIntersect( c( "classD_1", "classD_2" ), c( "CLASSD_1" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( "CLASSD_1", rxNodeSetIntersect( c( "classD_1" ), c( "CLASSD_1", "classD_2" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( c("classD_1","classD_4") ) ), sort( rxNodeSetIntersect( c( "classD_1", "classD_2","classD_4" ), c( "CLASSD_1","CLassD_3","CLaSsD_4" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) ) )
}

#"test.rxNodeSetIntersect.RxLsfCluster.hostnames" <- function()
#{
  #rxNodeSetIntersect <- .rxNodeSetIntersect()
  #rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  #
  #checkEquals( character(0), rxNodeSetIntersect( character(0), c( "CLASSD" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetIntersect( c( "CLASSD" ), character(0), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( "CLASSD", rxNodeSetIntersect( c( "classD" ), c( "CLASSD" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( "CLASSD_1", rxNodeSetIntersect( c( "classD_1", "classD_2" ), c( "CLASSD_1" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( "CLASSD_1", rxNodeSetIntersect( c( "classD_1" ), c( "CLASSD_1", "classD_2" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort( toupper( c("classD_1","classD_4") ) ), sort( rxNodeSetIntersect( c( "classD_1", "classD_2","classD_4" ), c( "CLASSD_1","CLassD_3","CLaSsD_4" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) ) )
#}

####################

"test.rxNodeSetDiff.RxHpcServer.domains" <- function()
{
  rxNodeSetDiff <- .rxNodeSetDiff()
  rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  
  checkEquals( sort( toupper( "classD" ) ), rxNodeSetDiff( c( "classD" ), c( "CLASSD.classC" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( "classD.classC" ) ), rxNodeSetDiff( c( "classD.classC" ), c( "CLASSD.classC.classB" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( "classD.classC.classB" ) ), rxNodeSetDiff( c( "classD.classC.classB" ), c( "CLASSD.classC.classB.classA" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( "classD.classC" ) ), rxNodeSetDiff( c( "CLASSD.classC" ), c( "classD" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( "classD.classC.classB" ) ), rxNodeSetDiff( c( "CLASSD.classC.classB" ), c( "classD.classC" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( sort( toupper( "classD.classC.classB.classA" ) ), rxNodeSetDiff( c( "CLASSD.classC.classB.classA" ), c( "classD.classC.classB" ), 
             rxuGetComputeContextDescription("RxHpcServer") ) )
  
}

#"test.rxNodeSetDiff.RxLsfCluster.domains" <- function()
#{
  #rxNodeSetDiff <- .rxNodeSetDiff()
  #rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  #
  #checkEquals( character(0), rxNodeSetDiff( c( "classD" ), c( "CLASSD.classC" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "classD.classC" ), c( "CLASSD.classC.classB" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "classD.classC.classB" ), c( "CLASSD.classC.classB.classA" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "CLASSD.classC" ), c( "classD" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "CLASSD.classC.classB" ), c( "classD.classC" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( character(0), rxNodeSetDiff( c( "CLASSD.classC.classB.classA" ), c( "classD.classC.classB" ), rxuGetComputeContextDescription("RxLsfCluster") ) )
#}

"test.rxNodeSetIntersect.RxHpcServer.domains" <- function()
{
  rxNodeSetIntersect <- .rxNodeSetIntersect()
  rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  
  checkEquals( character(0), rxNodeSetIntersect( c( "classD" ), c( "CLASSD.classC" ), rxuGetComputeContextDescription("RxHpcServer") ) )	
  checkEquals( character(0), rxNodeSetIntersect( c( "classD.classC" ), c( "CLASSD.classC.classB" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetIntersect( c( "classD.classC.classB" ), c( "CLASSD.classC.classB.classA" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetIntersect( c( "CLASSD.classC" ), c( "classD" ), rxuGetComputeContextDescription("RxHpcServer") ) )	
  checkEquals( character(0), rxNodeSetIntersect( c( "CLASSD.classC.classB" ), c( "classD.classC" ), rxuGetComputeContextDescription("RxHpcServer") ) )
  checkEquals( character(0), rxNodeSetIntersect( c( "CLASSD.classC.classB.classA" ), c( "classD.classC.classB" ), rxuGetComputeContextDescription("RxHpcServer") ) )
}

#"test.rxNodeSetIntersect.RxLsfCluster.domains" <- function()
#{
  #rxNodeSetIntersect <- .rxNodeSetIntersect()
  #rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
  #
  #checkEquals( sort( toupper( "classD" ) ), rxNodeSetIntersect( c( "classD" ), c( "CLASSD.classC" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort( toupper( "classD.classC" ) ), rxNodeSetIntersect( c( "classD.classC" ), c( "CLASSD.classC.classB" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort( toupper( "classD.classC.classB" ) ), rxNodeSetIntersect( c( "classD.classC.classB" ), c( "CLASSD.classC.classB.classA" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) )			
  #checkEquals( sort( toupper( c( "CLASSD.classC" ) ) ), rxNodeSetIntersect( c( "CLASSD.classC" ), c( "classD" ),  
             #rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort( toupper( "classD.classC.classB" ) ), rxNodeSetIntersect( c( "classD.classC.classB" ), c( "CLASSD.classC" ), 
             #rxuGetComputeContextDescription("RxLsfCluster") ) )
  #checkEquals( sort( toupper( c( "CLASSD.classC.classB.classA" ) ) ), rxNodeSetIntersect( c( "CLASSD.classC.classB.classA" ), c( "classD.classC.classB" ),  
             #rxuGetComputeContextDescription("RxLsfCluster") ) )
#
#}
