
"test.RxSpark.countMb" <- function()
{
    checkEquals(RevoScaleR:::countMb("123m"), 123)
    checkEquals(RevoScaleR:::countMb("123M"), 123)
    checkEquals(RevoScaleR:::countMb("123g"), 123 * 1024)
    checkEquals(RevoScaleR:::countMb("123G"), 123 * 1024)
    checkEquals(RevoScaleR:::countMb("0.13m"), 0.13)
    checkEquals(RevoScaleR:::countMb("1.3m"), 1.3)
    checkEquals(RevoScaleR:::countMb("13.00m"), 13)
    checkEquals(RevoScaleR:::countMb("0.0032m"), 0.0032)
    checkEquals(RevoScaleR:::countMb("0m"), 0)
    checkEquals(RevoScaleR:::countMb("23.34g"),23.34 * 1024)
    checkEquals(RevoScaleR:::countMb("0.0003g"), 0.0003 * 1024)
}

"test.RxSpark.checkSparkConfigErr.normal" <- function()
{
    RevoScaleR:::checkSparkConfigErr(
        numExecutors=2, executorCores=3, executorMem="2G", driverMem="2g",
        executorOverheadMem="2.3g", extraSparkConfig="--name \"Victor\"" )

    RevoScaleR:::checkSparkConfigErr(
        numExecutors=-1, executorCores=3, executorMem="2000m", driverMem="2000M",
        executorOverheadMem="2000.222m", extraSparkConfig="--name \"Victor\"" )
}

"test.RxSpark.checkSparkConfigErr.exception" <- function()
{
    checkException(RevoScaleR:::checkSparkConfigErr(
        numExecutors="three", executorCores=3,
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of numExecutors must be a positive integer")

    checkException(RevoScaleR:::checkSparkConfigErr(
        numExecutors=2, executorCores="3g",
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorCores must be a positive integer")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=2, executorCores="4",
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="g", 
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorCores must be a positive integer") 

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=2, executorCores=3,
        executorMem="0.01", driverMem="1.43g", executorOverheadMem="2g", 
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorMem must be written in memory format [number][m,M,g,G]")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=1, executorCores=3,
        executorMem="g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorMem must be written in memory format [number][m,M,g,G]") 

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3, executorCores=3,
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="200",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorOverheadMem must be written in memory format [number][m,M,g,G]") 

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3, executorCores=3,
        executorMem="0.01g", driverMem="g45", executorOverheadMem="2g", 
        extraSparkConfig="--name \"Jason\"" ),
        "The value of driverMem must be written in memory format [number][m,M,g,G]")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3, executorCores=3,
        executorMem="0.01g", driverMem="1.43g12", executorOverheadMem="2g", 
        extraSparkConfig="--name \"Jason\"" ),
        "The value of driverMem must be written in memory format [number][m,M,g,G]")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3, executorCores=3,
        executorMem="500.764", driverMem="1.43g", executorOverheadMem="2g", 
        extraSparkConfig="--name \"Jason\"" ), 
        "The value of executorMem must be written in memory format [number][m,M,g,G]")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3.1, executorCores=3,
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of numExecutors must be a positive integer")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=2, executorCores=0.32,
        executorMem="0.01g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorCores must be a positive integer") 

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=-3, executorCores=3,
        executorMem="500.764g", driverMem="1.43g", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of numExecutors must be a positive integer")

    checkException(RevoScaleR:::checkSparkConfigErr(numExecutors=3, executorCores=-0.34,
        executorMem="0.01g", driverMem="1.43g12", executorOverheadMem="2g",
        extraSparkConfig="--name \"Jason\"" ),
        "The value of executorCores must be a positive integer")
}

"test.RxSpark.check.master" <- function() 
{
    #test case 1
    cc <- RxSpark(master="local[3]")
    checkEquals(cc@master, "local[3]")
 
    #test case 2
    cc <- RxSpark(master="local[3,8]")
    checkEquals(cc@master, "local[3,8]")

    #test case 3
    cc <- RxSpark(master="local[*]")
    checkEquals(cc@master, "local[*]")

    #test case 4
    cc <- RxSpark(master="local[*,10]")
    checkEquals(cc@master, "local[*,10]")

    #test case 5
    cc <- RxSpark(master="spark://10.0.0.5:7070")
    checkEquals(cc@master, "spark://10.0.0.5:7070")

    #test case 6
    cc <- RxSpark()
    checkEquals(cc@master, "yarn")
    checkEquals(cc@nameNode, "default")

    #check exceptions
    checkException(RxSpark(master="spark://10.0.0.5"),
                   "Invalid format for the parameter `master`")

    checkException(RxSpark(master="10.0.0.5:7070"),
                   "Invalid format for the parameter `master`")

    checkException(RxSpark(master="spark:10.0.0.5:7070"),
                   "Invalid format for the parameter `master`")

    checkException(RxSpark(master="local[3"),
                   "Invalid format for the parameter `master`")

    checkException(RxSpark(master="local3"),
                   "Invalid format for the parameter `master`")

    checkException(RxSpark(master="local3]"),
                   "Invalid format for the parameter `master`")
 
    
}

"test.RxSpark.check.spark.property.against.master" <- function() {

    warning1 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'yarn', numExecutors = 1, executorCores = 1, executorOverheadMem = '1g')
    checkEquals(warning1, '')

    warning2 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'local', numExecutors = 1, executorCores = 1, executorOverheadMem = '1g')
    checkEquals( warning2, 'The parameters numExecutors, executorCores, executorOverheadMem do not take effect in Spark local mode.')

    warning3 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'spark//:10.0.0.8', numExecutors = 1, executorCores = 1, executorOverheadMem = '1g')
    checkEquals(warning3, 'The parameters numExecutors, executorOverheadMem do not take effect in Spark standalone mode.')

    warning4 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'local[4]', numExecutors = 1)
    checkEquals(warning4, 'The parameter numExecutors does not take effect in Spark local mode.')

    warning5 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'standalone', numExecutors = 1)
    checkEquals(warning5, 'The parameter numExecutors does not take effect in Spark standalone mode.')

    warning6 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'standalone')
    checkEquals(warning6, '')

    warning7 = RevoScaleR:::checkSparkPropertyAgainstMaster(master = 'yarn')
    checkEquals(warning7, '')

    cc <- RxSpark()
    checkEquals(cc@numExecutors, rxGetOption("spark.numExecutors"))
    checkEquals(cc@executorCores, rxGetOption("spark.executorCores"))
    checkEquals(cc@executorMem, rxGetOption("spark.executorMem"))
    checkEquals(cc@driverMem, "4g")
    checkEquals(cc@executorOverheadMem, rxGetOption("spark.executorOverheadMem"))
}

"test.RxSpark.hdfsShareDir.local.Spark" <- function() {
    #test case1
    cc <- RxSpark(master="local[3]")
    checkEquals(cc@hdfsShareDir, cc@shareDir)

    #test case2
    cc <- RxSpark(master="local[*]", hdfsShareDir="/tmp/RevoShare")
    checkEquals(cc@hdfsShareDir, cc@shareDir)

    #test case3
    cc <- RxSpark(master="local[*]", shareDir="/tmp/RevoShare")
    checkEquals(cc@hdfsShareDir, cc@shareDir)

    #test case4
    cc <- RxSpark(master="local", shareDir="/tmp/RevoShare", hdfsShareDir="/user/RevoShare")
    checkEquals(cc@hdfsShareDir, cc@shareDir)
}
