# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxSpssData
#
############################################################################################

"test.RxSpssData.varsToKeep" <- function()
{
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    
    varsToKeep <- c("Y", "XFAC1")
    mySpssData <- RxSpssData( spssFileName, varsToKeep = varsToKeep)    
    myData <- rxImport(inData = mySpssData )
    checkEquals( names(myData), varsToKeep)
    
    # Reverse order  
    varsToKeep <- c("XFAC1", "Y")
    mySpssData <- RxSpssData( spssFileName, varsToKeep = varsToKeep)
    myData <- rxImport(inData = mySpssData )
    checkEquals( names(myData), varsToKeep) 
    
    # Use a transform   
    myData <- rxImport(inData = mySpssData, transforms = list(MYY = Y*2),
        varsToKeep = varsToKeep)
    checkEquals( names(myData), c(varsToKeep, "MYY")) 
     
    # Multiple block
    outFile <- tempfile(pattern = ".rxTempSPSS", fileext = ".xdf") 
    myData <- rxImport(inData = mySpssData, outFile = outFile,
        transforms = list(MYY = Y*2),
        varsToKeep = varsToKeep, rowsPerRead = 30, overwrite=TRUE)
    fileInfo <- rxGetInfo(myData, getVarInfo = TRUE)
    checkEquals( names(fileInfo$varInfo), c(varsToKeep, "MYY")) 
    if (rxIsExpressEdition())
    {
        checkEquals( fileInfo$numBlocks, 1)
    } else
    {
        checkEquals( fileInfo$numBlocks, 4)
    }
    
    file.remove(outFile)    
}

"test.RxSpssData.varsToDrop" <- function()
{
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    
    varsToDrop<- c("Y", "XFAC1")

    mySpssData <- RxSpssData( spssFileName, varsToDrop = varsToDrop)
    
    allVarNames <- rxGetVarNames( RxSpssData(spssFileName) )
    varsToKeep = allVarNames[!(allVarNames %in% varsToDrop)]
     
    myData <- rxImport(inData = mySpssData )
    checkEquals( names(myData), varsToKeep)    
}

# Only can be run if access to gssnet.sav
# Example
# Import an SPSS file, keeping multiple missing value codes as factor levels
## First specify file location
#spssFileName <- file.path(rxGetOption("sampleDataDir"), "gssnet.sav")   
#
## Then create an SPSS data source  
#spssDS <- RxSpssData( spssFileName, varsToKeep = c("age", "sex", "netcat"), 
    #mapMissingCodes = "none", labelsAsLevels = TRUE)  
#
## Import the selected variables into a data frame     
#gssnet <- rxImport(inData = spssDS )

# Plot the counts for each category, including the missings
# rxHistogram(~netcat, data = gssnet)

#"test.RxSpssData.labelsAsInfo" <- function()
#{
#
    #spssFileName <- file.path(rxGetOption("sampleDataDir"), "gssnet.sav") 
    #webHourLabels <- c("Time unknown", "e-mail use only", "Not web user")
    #richWorkLabels <- c("NAP", "CONTINUE WORKING", "STOP WORKING", "DK", "NA")
    #varsToKeep = c("webhrs", "richwork")
          #
    #spssDS1 <- RxSpssData( spssFileName, labelsAsLevels = FALSE, 
        #labelsAsInfo = TRUE, varsToKeep = varsToKeep)      
    #myData1 <- rxImport(inData = spssDS1 )
    #varInfo1 <- rxGetVarInfo(myData1)
    #checkEquals(varInfo1$webhrs$valueInfoLabels, webHourLabels)
    #checkEquals(varInfo1$richwork$valueInfoLabels, richWorkLabels)
#
    #spssDS2 <- RxSpssData( spssFileName, labelsAsLevels = FALSE, labelsAsInfo = TRUE,
        #colClasses = c(webhrs = "integer", richwork = "integer"), varsToKeep = varsToKeep)     
    #myData2 <- rxImport(inData = spssDS2 )
    #varInfo2 <- rxGetVarInfo(myData2)
    #checkEquals(varInfo2$webhrs$valueInfoLabels, webHourLabels) 
    #checkEquals(varInfo2$richwork$valueInfoLabels, richWorkLabels)     
    #
    #spssDS3 <- RxSpssData( spssFileName, labelsAsLevels = FALSE, labelsAsInfo = TRUE,
        #mapMissingCodes = "none", colClasses = c(webhrs = "integer"), varsToKeep =varsToKeep)     
    #myData3 <- rxImport(inData = spssDS3 )
    #varInfo3 <- rxGetVarInfo(myData3)
    #checkEquals(varInfo3$webhrs$valueInfoLabels, webHourLabels) 
    #checkEquals(varInfo3$richwork$valueInfoLabels, richWorkLabels)
      #
    #spssDS4 <- RxSpssData( spssFileName, labelsAsLevels = TRUE, labelsAsInfo = TRUE,
        #mapMissingCodes = "none", colClasses = c(webhrs = "integer"), 
        #varsToKeep = varsToKeep)     
    #myData4 <- rxImport(inData = spssDS4 )
    #varInfo4 <- rxGetVarInfo(myData4)
    #checkEquals(varInfo4$webhrs$valueInfoLabels, webHourLabels)  
    #checkEquals(varInfo4$richwork$levels, richWorkLabels)    
#
#}
    #
#"test.RxSpssData.mapMissingCodes" <- function()
#{
#
    #spssFileName <- file.path(rxGetOption("sampleDataDir"), "gssnet.sav")   
    #spssDSall <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "all", labelsAsLevels = FALSE)
    #spssDSfirst <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "first", labelsAsLevels = FALSE)
    #spssDSnone <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "none", labelsAsLevels = FALSE)
       #
    #dataAll <- rxImport(inData = spssDSall )
    #dataFirst <- rxImport(inData = spssDSfirst)
    #dataNone <- rxImport(inData = spssDSnone)
    #checkEquals(min(dataAll$netcat, na.rm = TRUE), 1)
    #checkEquals(min(dataFirst$netcat, na.rm = FALSE), as.numeric(NA))
    #checkEquals(min(dataFirst$netcat, na.rm = TRUE), -9)    
    #checkEquals(min(dataNone$netcat, na.rm = TRUE), -9)
    #
    #spssDSall <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "all", labelsAsLevels = TRUE)
    #spssDSfirst <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "first", labelsAsLevels = TRUE)
    #spssDSnone <- RxSpssData( spssFileName, varsToKeep = "netcat", mapMissingCodes = "none", labelsAsLevels = TRUE)  
    #dataAll <- rxImport(inData = spssDSall )
    #checkEquals(sum(is.na(dataAll$netcat)), 367)
    #
    #dataFirst <- rxImport(inData = spssDSfirst)
    #checkEquals(sum(is.na(dataFirst$netcat)), 6) 
       #
    #dataNone <- rxImport(inData = spssDSnone)      
    #checkEquals(sum(is.na(dataNone$netcat)), 0)    
#
#}

"test.RxSpssData.method.head" <- function()
{
   spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    
    spssVarsToKeep <- c("Y", "XFAC1")
    
  xdfFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r1b.xdf")
  
  xdfDS <- RxXdfData(xdfFileName)
  headXdf <- head(xdfDS)
  varsToKeep <- c("y", "xfac1")
  headXdf <- subset(headXdf, select = varsToKeep) 
  
    spssDS <- RxSpssData( spssFileName, varsToKeep = spssVarsToKeep, stringsAsFactors = TRUE) 

  headSpss <- head(spssDS)

  # The SPSS data sources will have fewer factor levels and capitalized var names
  checkEquals(headSpss, headXdf, checkNames = FALSE, check.attributes = FALSE, tolerance = 10E-7)
}

"test.RxSpssData.method.names" <- function()
{
   spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")

  spssDS <- RxSpssData( spssFileName) 
  importNames <- names(rxImport(spssDS, numRows = 3))
  methodNames <- names(spssDS)
  checkEquals(importNames, methodNames)
}



