############################################################################################
# Copyright (c) Microsoft Corporation.  All rights reserved.
############################################################################################

#
# Argument validation tests
#
# TODO: add checking for inSqlServer & tableOpClause if needed

# Minimal connection string to be able to instantiate RxInSqlServer and RxSqlServerData
RxSqlServerConnStr <- "uid=u;pwd=p"

test.RxSqlServerData.arg.table <- function()
{
  checkException(RxSqlServerData(table = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(table = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(table = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(table = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  checkTrue(validObject(RxSqlServerData(table = "mytable", connectionString = RxSqlServerConnStr)))
}

test.RxSqlServerData.arg.sqlQuery <- function()
{
  checkException(RxSqlServerData(sqlQuery = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr)))
}

test.RxSqlServerData.arg.server <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = "myserver", connectionString = RxSqlServerConnStr)

  checkTrue(validObject(data))
  checkTrue(grepl("(s|S)(e|E)(r|R)(v|V)(e|E)(r|R)=myserver(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxSqlServerData.arg.dsn <- function()
{
  # We do not provide first class support for DSN because it makes little sense with current design where the R session on
  # the server needs to initiate a connection back to SQL from a different machine and/or user where the same DSN may not exist.
  # However, we do not explicitly prevent someone from manually specifying a DSN in the connection string
  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr)))
  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = paste0(RxSqlServerConnStr,";DSN=MyDSN"))))
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, dsn = "mydsn"), silent = TRUE)
}

test.RxSqlServerData.arg.user <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", user = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", user = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", user = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", user = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxSqlServerData(sqlQuery = "SELECT * FROM mytable", user = "me", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))	
  checkTrue(grepl("(u|U)(i|I)(d|D)=me(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxSqlServerData.arg.password <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", password = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", password = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", password = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", password = 1, connectionString = RxSqlServerConnStr), silent = TRUE)
  
  data <- RxSqlServerData(sqlQuery = "SELECT * FROM mytable", password = "opensesame", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))
  checkTrue(grepl("(p|P)(w|W)(d|D)=opensesame(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxSqlServerData.arg.server <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxSqlServerData(sqlQuery = "SELECT * FROM mytable", server = "MyServer", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))	
  checkTrue(grepl("(S|s)(e|E)(r|R)(v|V)(e|E)(r|R)=MyServer(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxSqlServerData.arg.databaseName <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", databaseName = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", databaseName = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", databaseName = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", databaseName = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxSqlServerData(sqlQuery = "SELECT * FROM mytable", databaseName = "MyDatabase", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))	
  checkTrue(grepl("(D|d)(a|A)(t|T)(a|A)(b|B)(a|A)(s|S)(e|E)=MyDatabase(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxSqlServerData.arg.connectionString <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable",
                connectionString = NA_character_), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable",
                connectionString = ""), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable",
                connectionString = letters), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable",
                connectionString = 1), silent = TRUE)
                
  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable",
                   connectionString = RxSqlServerConnStr)))
}

test.RxSqlServerData.connectionString.DriverAutoAdded <- function()
{
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="Server=x;uid=u;pwd=p")@connectionString, 
    "Driver=SQL Server;Server=x;uid=u;pwd=p")
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="Database=y;Server=x;uid=u;pwd=p")@connectionString,
    "Driver=SQL Server;Database=y;Server=x;uid=u;pwd=p")	
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="Driver=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p")@connectionString,
    "Driver=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p",
    msg="verifying matching Driver.")
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="dRiVeR=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p")@connectionString,
    "dRiVeR=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p",
    msg="keyword is case insensitive.")
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="Database=y;Driver=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p")@connectionString,
    "Database=y;Driver=SQL Server Native Client v69.42;Server=x;uid=u;pwd=p",
    msg="verifying matching Driver in any location.")
  checkEquals(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString="MadmaxDriver=y;Server=x;uid=u;pwd=p")@connectionString,
    "Driver=SQL Server;MadmaxDriver=y;Server=x;uid=u;pwd=p",
    msg="verifying only matching Driver.")
}

test.RxSqlServerData.arg.returnDataFrame <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", returnDataFrame = NULL, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", returnDataFrame = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", returnDataFrame = 1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", returnDataFrame = c(TRUE, FALSE), connectionString = RxSqlServerConnStr), silent = TRUE)

  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, returnDataFrame = TRUE)))
  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, returnDataFrame = FALSE)))
}

test.RxSqlServerData.arg.stringsAsFactors <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = NULL), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = NA), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = 1), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = c(TRUE, FALSE)), silent = TRUE)

  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = TRUE)))
  checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, stringsAsFactors = FALSE)))
}

test.RxSqlServerData.arg.rowsPerRead <- function()
{
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = NULL), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = NA_integer_), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = "a"), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = 0), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = Inf), silent = TRUE)
  checkException(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr, rowsPerRead = 1:2), silent = TRUE)

  for (i in c(1L, 10L, 100L))
  {
    checkTrue(validObject(RxSqlServerData(sqlQuery = "SELECT * FROM mytable", connectionString = RxSqlServerConnStr,
                     rowsPerRead = i), test = TRUE))
  }
}

test.RxInSqlServer.arg.numTasks <- function()
{
  # pass
  checkEquals(RxInSqlServer(numTasks = 0, connectionString = RxSqlServerConnStr)@numTasks, 0)
  checkEquals(RxInSqlServer(numTasks = 1, connectionString = RxSqlServerConnStr)@numTasks, 1)
  checkEquals(RxInSqlServer(numTasks = 100, connectionString = RxSqlServerConnStr)@numTasks, 100)
  # auto-converted to zero
  checkEquals(RxInSqlServer(numTasks = NULL, connectionString = RxSqlServerConnStr)@numTasks, 0)
  # must be valid positive nb
  checkException(RxInSqlServer(numTasks = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(numTasks = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(numTasks = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(numTasks = -1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(numTasks = c(1,2,3), connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.autoCleanup <- function()
{
  # pass
  checkEquals(RxInSqlServer(autoCleanup = TRUE, connectionString = RxSqlServerConnStr)@autoCleanup, TRUE)
  checkEquals(RxInSqlServer(autoCleanup = FALSE, connectionString = RxSqlServerConnStr)@autoCleanup, FALSE)
  # fail
  checkException(RxInSqlServer(autoCleanup = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(autoCleanup = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(autoCleanup = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(autoCleanup = c(1,2,3), connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.trace <- function()
{
  # pass
  checkEquals(RxInSqlServer(traceEnabled = TRUE, connectionString = RxSqlServerConnStr)@traceEnabled, TRUE)
  checkEquals(RxInSqlServer(traceEnabled = FALSE, connectionString = RxSqlServerConnStr)@traceEnabled, FALSE)
  # stays null and resolved at run-time
  cc <- RxInSqlServer(traceEnabled = NULL, connectionString = RxSqlServerConnStr)
  checkEquals(cc@traceEnabled, NULL)

  # TODO: rxComputeContextGetTraceSettings only available when using init.R
  # checkEquals(rxComputeContextGetTraceSettings(cc)$enabled, FALSE)
  # checkEquals(rxComputeContextGetTraceSettings(cc)$level, 0)
  
  checkEquals(RxInSqlServer(traceLevel = 0, connectionString = RxSqlServerConnStr)@traceLevel, 0)
  checkEquals(RxInSqlServer(traceLevel = 100, connectionString = RxSqlServerConnStr)@traceLevel, 100)
  checkEquals(RxInSqlServer(traceLevel = NULL, connectionString = RxSqlServerConnStr)@traceLevel, NULL)
  
  cc <- RxInSqlServer(traceLevel = 7, traceEnabled = TRUE, connectionString = RxSqlServerConnStr)
  # TODO: rxComputeContextGetTraceSettings only available when using init.R
  # checkEquals(rxComputeContextGetTraceSettings(cc)$enabled, TRUE)
  # checkEquals(rxComputeContextGetTraceSettings(cc)$level, 7)
  
  # fail
  checkException(RxInSqlServer(traceEnabled = 1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(traceEnabled = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(traceEnabled = c(TRUE, FALSE), connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(traceLevel = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(traceLevel = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(traceLevel = c(1,2), connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.consoleOutput <- function()
{
  # pass
  checkEquals(RxInSqlServer(consoleOutput = TRUE, connectionString = RxSqlServerConnStr)@consoleOutput, TRUE)
  checkEquals(RxInSqlServer(consoleOutput = FALSE, connectionString = RxSqlServerConnStr)@consoleOutput, FALSE)
  # fail
  checkException(RxInSqlServer(consoleOutput = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(consoleOutput = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(consoleOutput = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(consoleOutput = c(1,2,3), connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.wait <- function()
{
  # pass
  checkEquals(RxInSqlServer(wait = TRUE, connectionString = RxSqlServerConnStr)@wait, TRUE)
  checkEquals(RxInSqlServer(wait = FALSE, connectionString = RxSqlServerConnStr)@wait, FALSE)
  # fail
  checkException(RxInSqlServer(wait = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(wait = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(wait = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(wait = c(1,2,3), connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.executionTimeoutSeconds <- function()
{
  # pass
  checkEquals(RxInSqlServer(executionTimeoutSeconds = 0, connectionString = RxSqlServerConnStr)@executionTimeoutSeconds, 0)
  checkEquals(RxInSqlServer(executionTimeoutSeconds = 1, connectionString = RxSqlServerConnStr)@executionTimeoutSeconds, 1)
  checkEquals(RxInSqlServer(executionTimeoutSeconds = 100, connectionString = RxSqlServerConnStr)@executionTimeoutSeconds, 100)
  # must be valid non null positive nb, not too big
  checkException(RxInSqlServer(executionTimeoutSeconds = NULL, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = -1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = c(1,2,3), connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(executionTimeoutSeconds = 2 ** 20, connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.packagesToLoad <- function()
{
  # pass
  checkEquals(RxInSqlServer(connectionString = RxSqlServerConnStr)@packagesToLoad, NULL)
  checkEquals(RxInSqlServer(packagesToLoad = NULL, connectionString = RxSqlServerConnStr)@packagesToLoad, NULL)
  checkEquals(RxInSqlServer(packagesToLoad = "a", connectionString = RxSqlServerConnStr)@packagesToLoad, "a")
  checkEquals(RxInSqlServer(packagesToLoad=c("a","b","c"), connectionString = RxSqlServerConnStr)@packagesToLoad, c("a", "b", "c"))
  # fail
  checkException(RxInSqlServer(packagesToLoad = 1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(packagesToLoad = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(packagesToLoad = TRUE, connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.shareDir <- function()
{
  # we don't want shareDir to be NULL or empty string by default because this would mean
  # creating folders relative to current working directory by default
  defaultShareDir <- RxInSqlServer(connectionString = RxSqlServerConnStr)@shareDir
  checkTrue(grepl("RevoJobs", defaultShareDir)) # we want all of the Revo job tmp folders to be in 1 revo-specific location
  # Note: not testing that defaultShareDir is under tmp because this is OS and machine specific and we'd need to replicate Revo's logic to figure it out
  
  checkEquals(defaultShareDir, RxInSqlServer(shareDir = NULL, connectionString = RxSqlServerConnStr)@shareDir)
  # we allow empty string, which is create tmp folder relative to current working directory
  checkEquals("", RxInSqlServer(shareDir = "", connectionString = RxSqlServerConnStr)@shareDir)
  checkEquals("/My/Own/Folder", RxInSqlServer(shareDir = "/My/Own/Folder", connectionString = RxSqlServerConnStr)@shareDir)
  
  # fail
  checkException(RxInSqlServer(shareDir = 1, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(shareDir = NA, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(shareDir = TRUE, connectionString = RxSqlServerConnStr), silent = TRUE)
}

test.RxInSqlServer.arg.server <- function()
{
  checkException(RxInSqlServer(server = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(server = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(server = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(server = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxInSqlServer(server = "MyServer", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))	
  checkTrue(grepl("(s|S)(e|E)(r|R)(v|V)(e|E)(r|R)=MyServer(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxInSqlServer.arg.databaseName <- function()
{
  checkException(RxInSqlServer(databaseName = NA_character_, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(databaseName = "", connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(databaseName = letters, connectionString = RxSqlServerConnStr), silent = TRUE)
  checkException(RxInSqlServer(databaseName = 1, connectionString = RxSqlServerConnStr), silent = TRUE)

  data <- RxInSqlServer(databaseName = "MyDatabase", connectionString = RxSqlServerConnStr)
  checkTrue(validObject(data))	
  checkTrue(grepl("(D|d)(a|A)(t|T)(a|A)(b|B)(a|A)(s|S)(e|E)=MyDatabase(;|$)", data@connectionString, ignore.case = FALSE))
}

test.RxInSqlServer.show <- function() 
{
  myOutput <- capture.output(RxInSqlServer(connectionString = "uid=user;pwd=password"))
  
  checkTrue(grepl("SQL Server", myOutput[1])) # header
  checkTrue(grepl("^-+$", myOutput[2])) # dashes underneath the header
  # check that we have 1 line per parameter with good defaults
  checkEquals(1, sum(grepl("^connectionString.*uid=user;pwd=password", myOutput)))
  checkEquals(1, sum(grepl("^numTasks.* 1", myOutput)))
  checkEquals(1, sum(grepl("^executeAsUser.*", myOutput)))
  checkEquals(1, sum(grepl("^executionTimeoutSeconds.* 0", myOutput)))
  checkEquals(1, sum(grepl("^shareDir.*RevoJobs", myOutput)))
  checkEquals(1, sum(grepl("^wait.*TRUE", myOutput)))
  checkEquals(1, sum(grepl("^consoleOutput.*FALSE", myOutput)))
  checkEquals(1, sum(grepl("^autoCleanup.*TRUE", myOutput)))
  checkEquals(1, sum(grepl("^nodes.*NULL", myOutput))) # TODO: what would be a better default?
  checkEquals(1, sum(grepl("^packagesToLoad.*NULL", myOutput)))
  checkEquals(1, sum(grepl("^description.*sqlserver", myOutput)))
  checkEquals(1, sum(grepl("^version", myOutput)))
  checkEquals(14, length(myOutput)) # 2 for header + 1 for each visible field. If this line fails due to legitimate new field, also add the corresponding field check
}

test.RxSqlServerData.show <- function() 
{
  myOutput <- capture.output(RxSqlServerData(connectionString = "uid=MyUser;pwd=MyPassword", table = "MyTable"))
  checkTrue(grepl("RxSqlServerData Source", myOutput[1])) # header
  checkEquals(1, sum(grepl("^connectionString.*uid=MyUser;pwd=MyPassword", myOutput)))
  checkEquals(1, sum(grepl("^table.*MyTable", myOutput)))
  
  myOutput <- capture.output(RxSqlServerData(user = "MyUser", password = "MyPassword", sqlQuery = "SELECT col FROM MyTable"))
  checkTrue(grepl("RxSqlServerData Source", myOutput[1])) # header
  checkEquals(1, sum(grepl("^connectionString.*uid=MyUser", myOutput)))
  checkEquals(1, sum(grepl("^connectionString.*pwd=MyPassword", myOutput)))
  checkEquals(1, sum(grepl("^sqlQuery.*SELECT col FROM MyTable", myOutput)))	
}

test.RxInSqlServer.clone  <- function() 
{
  computeContext <- RxInSqlServer(
    connectionString = RxSqlServerConnStr,
    numTasks = 51,
    autoCleanup = FALSE,
    consoleOutput = TRUE,
    executionTimeoutSeconds = 69,
    wait = FALSE,
    packagesToLoad = c("Some", "Package"),
    shareDir = "Some/Dir",
    dataDistType = "all"
  )

  computeContext2 <-  RxInSqlServer(computeContext)
  checkEquals(computeContext, computeContext2)

  # just to be sure
  checkEquals("Some/Dir", computeContext2@shareDir)

  computeContext3 <- RxInSqlServer(computeContext, shareDir = "Some/Other/Dir")
  checkEquals("Some/Other/Dir", computeContext3@shareDir)
  checkEquals(51, computeContext3@numTasks)
  checkEquals(69, computeContext3@executionTimeoutSeconds)
}

test.RxSqlServerData.MustSpecifyCredentials <- function()
{
  connStr <- RxSqlServerData(user = "MyUser", password = "MyPassword", sqlQuery = "SELECT * FROM MyTable")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))

  connStr <- RxSqlServerData(connectionString = "Server=MyServer;uid=MyUser", password = "MyPassword", sqlQuery = "SELECT * FROM MyTable")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))
  
  connStr <- RxSqlServerData(connectionString = "pwd=MyPassword", user = "MyUser", sqlQuery = "SELECT * FROM MyTable")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))
  
  # password missing is OK
  checkTrue(validObject(RxSqlServerData(user = "MyUser", sqlQuery = "SELECT * FROM MyTable")))
  checkTrue(validObject(RxSqlServerData(connectionString = "Server=MyServer", user = "MyUser", sqlQuery = "SELECT * FROM MyTable")))
  # user missing
  checkException(RxSqlServerData(password = "MyPassword", sqlQuery = "SELECT * FROM MyTable"), silent = TRUE)
  checkException(RxSqlServerData(connectionString = "pwd=MyPassword", sqlQuery = "SELECT * FROM MyTable"), silent = TRUE)
}

test.RxInSqlServer.MustSpecifyCredentials <- function()
{
  connStr <- RxInSqlServer(user = "MyUser", password = "MyPassword")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))

  connStr <- RxInSqlServer(connectionString = "Server=MyServer;uid=MyUser", password = "MyPassword")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))
  
  connStr <- RxInSqlServer(connectionString = "pwd=MyPassword", user = "MyUser")@connectionString
  checkTrue(grepl("(u|U)(i|I)(d|D)=MyUser", connStr, ignore.case = FALSE))
  checkTrue(grepl("(p|P)(w|W)(d|D)=MyPassword", connStr, ignore.case = FALSE))
  
  # password missing is OK
  checkTrue(validObject(RxInSqlServer(user = "MyUser")))
  checkTrue(validObject(RxInSqlServer(connectionString = "Server=MyServer", user = "MyUser")))
  # user missing
  checkException(RxInSqlServer(password = "MyPassword"), silent = TRUE)
  checkException(RxInSqlServer(connectionString = "pwd=MyPassword"), silent = TRUE)
}

test.RxInSqlServer.Negative.SqlAsDataSource <- function()
{
  on.exit(rxSetComputeContext(RxLocalSeq()), add = TRUE )

  claimsCsvPath <- file.path(rxGetOption("sampleDataDir"), "claims.csv")
  claimsXdfPath <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  claimsXdf <- RxXdfData(claimsXdfPath)
  claimsCsvData <- RxTextData(claimsCsvPath)

  sqlCC <- RxInSqlServer(connectionString = RxSqlServerConnStr)

  rxSetComputeContext(sqlCC)

  sqlData <- RxSqlServerData(
      connectionString = RxSqlServerConnStr, 
      table = "TestClaims", 
      rowsPerRead = 5000)

  checkException(rxDataStep(inData=claimsCsvPath, outFile=sqlData, overwrite=TRUE), silent=TRUE)
  checkException(rxDataStep(inData=claimsXdf, outFile=sqlData, overwrite=TRUE), silent=TRUE)
  checkException(rxDataStep(inData=claimsCsvData, outFile=sqlData, overwrite=TRUE), silent=TRUE)

  checkException(rxDataStep(data=claimsXdf, overwrite=TRUE), silent=TRUE)
  checkException(rxDataStep(inData=sqlData, outData=claimsXdf, overwrite=TRUE), silent=TRUE)
  checkException(rxDataStep(inData=sqlData, outFile=claimsXdfPath, overwrite=TRUE), silent=TRUE)
}