# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxTextData
#
############################################################################################

"test.RxTextData.invalid.arg.defaultDecimalColType" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkException(RxTextData(file = tinyFileName, defaultDecimalColType = "character"), silent = TRUE)
    checkException(RxTextData(file = tinyFileName, defaultDecimalColType = rep("float32",2)), silent = TRUE)
}

"test.RxTextData.valid.arg.defaultDecimalColType" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkTrue(validObject(RxTextData(file = tinyFileName, defaultDecimalColType = "float32"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName, defaultDecimalColType = "numeric"),
                          test = TRUE))
}

"test.RxTextData.invalid.arg.defaultMissingColType" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkException(RxTextData(file = tinyFileName, defaultMissingColType = "list"), silent = TRUE)
    checkException(RxTextData(file = tinyFileName, defaultMissingColType = rep("float32",2)), silent = TRUE)
}

"test.RxTextData.valid.arg.defaultMissingColType" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkTrue(validObject(RxTextData(file = tinyFileName, defaultMissingColType = "float32"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName, defaultMissingColType = "numeric"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName, defaultMissingColType = "character"),
                          test = TRUE))
}

"test.RxTextData.invalid.arg.file" <- function()
{
    checkException(RxTextData(), silent = TRUE)
    checkException(RxTextData(file = TRUE), silent = TRUE)
    checkException(RxTextData(file = 1L), silent = TRUE)
    checkException(RxTextData(file = 1.0), silent = TRUE)
    checkException(RxTextData(file = c("a", "b")), silent = TRUE)
}

"test.RxTextData.valid.arg.file" <- function()
{
    checkTrue(validObject(RxTextData(file = "TextFile.txt"), test = TRUE))
    checkTrue(validObject(RxTextData(file = "TextFile"), test = TRUE))
    checkTrue(validObject(RxTextData(file = "Text.File"), test = TRUE))
    checkTrue(validObject(RxTextData(file = "Text.File.txt"), test = TRUE))
}


"test.RxTextData.invalid.arg.varsToKeep" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkException(RxTextData(file = tinyFileName, varsToKeep = NA_character_), silent = TRUE)
    checkException(RxTextData(file = tinyFileName, varsToKeep = 1), silent = TRUE)
    checkException(RxTextData(file = tinyFileName, varsToKeep = "a", varsToDrop = "b"), silent = TRUE)
}

"test.RxTextData.valid.arg.varsToKeep" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkTrue(validObject(RxTextData(file = tinyFileName, varsToKeep = "a"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName, varsToKeep = "b"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName,
                                     varsToKeep = c("a", "b", "c")),
                          test = TRUE))
}

"test.RxTextData.invalid.arg.varsToDrop" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkException(RxTextData(file = tinyFileName, varsToDrop = NA_character_), silent = TRUE)
    checkException(RxTextData(file = tinyFileName, varsToDrop = 1), silent = TRUE)
    # Dropping all variables should fail
    checkException(RxTextData(file = tinyFileName, varsToDrop = c("a", "b", "c")), silent = TRUE)
}

"test.RxTextData.valid.arg.varsToDrop" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    checkTrue(validObject(RxTextData(file = tinyFileName, varsToDrop = "a"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName, varsToDrop = "b"),
                          test = TRUE))
    checkTrue(validObject(RxTextData(file = tinyFileName,
                                     varsToDrop = c("a", "b")),
                          test = TRUE))
}


"test.RxTextData.invalid.arg.rowsToSkip" <- function()
{

    checkException(RxTextData(file = "txt", rowsToSkip = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSkip = NA_integer_), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSkip = "a"), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSkip = -1), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSkip = Inf), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSkip = 0:1), silent = TRUE)
}

"test.RxTextData.valid.arg.rowsToSkip" <- function()
{
    for (i in c(0L, 1L, 10L, 100L))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsToSkip = i),
                              test = TRUE))
    }
    for (i in c(0, 1, 10, 100))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsToSkip = i),
                              test = TRUE))
    }
}


"test.RxTextData.invalid.arg.returnDataFrame" <- function()
{
    checkException(RxTextData(file = "txt", returnDataFrame = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", returnDataFrame = NA), silent = TRUE)
    checkException(RxTextData(file = "txt", returnDataFrame = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", returnDataFrame = c(TRUE, FALSE)), silent = TRUE)
}

"test.RxTextData.valid.arg.combineDelimiters" <- function()
{
    checkTrue(validObject(RxTextData(file = "TextFile.txt", combineDelimiters = TRUE), test = TRUE))
    checkTrue(validObject(RxTextData(file = "TextFile.txt", combineDelimiters = FALSE), test = TRUE))
}

"test.RxTextData.invalid.arg.stringsAsFactors" <- function()
{
    checkException(RxTextData(file = "txt", stringsAsFactors = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", stringsAsFactors = NA), silent = TRUE)
    checkException(RxTextData(file = "txt", stringsAsFactors = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", stringsAsFactors = c(TRUE, FALSE)), silent = TRUE)
}

"test.RxTextData.valid.arg.stringsAsFactors" <- function()
{
    checkTrue(validObject(RxTextData(file = "TextFile.txt", stringsAsFactors = TRUE), test = TRUE))
    checkTrue(validObject(RxTextData(file = "TextFile.txt", stringsAsFactors = FALSE), test = TRUE))
}

"test.RxTextData.invalid.arg.rowsPerRead" <- function()
{
    checkException(RxTextData(file = "txt", rowsPerRead = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsPerRead = NA_integer_), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsPerRead = "a"), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsPerRead = 0), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsPerRead = Inf), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsPerRead = 1:2), silent = TRUE)
}

"test.RxTextData.valid.arg.rowsPerRead" <- function()
{
    for (i in c(1L, 10L, 100L))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsPerRead = i),
                              test = TRUE))
    }
    for (i in c(1, 10, 100))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsPerRead = i),
                              test = TRUE))
    }
}

# This was testing validation code in RxTextData
# that no longer exists, so this test is now obsolete. RBC 08/12/14
#"test.RxTextData.invalid.arg.delimiter" <- function()
#{
#    checkException(RxTextData(file = "txt", delimiter = NA_character_), silent = TRUE)
#    checkException(RxTextData(file = "txt", delimiter = 1), silent = TRUE)
#    checkException(RxTextData(file = "txt", delimiter = ""), silent = TRUE)
#    checkException(RxTextData(file = "txt", delimiter = "ab"), silent = TRUE)
#    checkException(RxTextData(file = "txt", delimiter = c("a", "b")), silent = TRUE)
#}

"test.RxTextData.valid.arg.delimiter" <- function()
{
    for (i in c(",", "\t", "|"))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", delimiter = i),
                              test = TRUE))
    }
}

"test.RxTextData.invalid.arg.combineDelimiters" <- function()
{
    checkException(RxTextData(file = "txt", combineDelimiters = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", combineDelimiters = NA), silent = TRUE)
    checkException(RxTextData(file = "txt", combineDelimiters = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", combineDelimiters = c(TRUE, FALSE)), silent = TRUE)
}

"test.RxTextData.valid.arg.combineDelimiters" <- function()
{
    checkTrue(validObject(RxTextData(file = "TextFile.txt", combineDelimiters = TRUE), test = TRUE))
    checkTrue(validObject(RxTextData(file = "TextFile.txt", combineDelimiters = FALSE), test = TRUE))
}

"test.RxTextData.invalid.arg.quoteMark" <- function()
{
    checkException(RxTextData(file = "txt", quoteMark = NA_character_), silent = TRUE)
    checkException(RxTextData(file = "txt", quoteMark = 1), silent = TRUE)
    ##checkException(RxTextData(file = "txt", quoteMark = ""), silent = TRUE)
    checkException(RxTextData(file = "txt", quoteMark = "ab"), silent = TRUE)
    checkException(RxTextData(file = "txt", quoteMark = c("a", "b")), silent = TRUE)
}

"test.RxTextData.valid.arg.quoteMark" <- function()
{
    for (i in c("\"", "\'", "#"))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", quoteMark = i),
                              test = TRUE))
    }
}

"test.RxTextData.invalid.arg.decimalPoint" <- function()
{
    checkException(RxTextData(file = "txt", decimalPoint = NA_character_), silent = TRUE)
    checkException(RxTextData(file = "txt", decimalPoint = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", decimalPoint = ""), silent = TRUE)
    checkException(RxTextData(file = "txt", decimalPoint = "ab"), silent = TRUE)
    checkException(RxTextData(file = "txt", decimalPoint = c("a", "b")), silent = TRUE)
}

"test.RxTextData.valid.arg.decimalPoint" <- function()
{
    for (i in c(".", ",", ";"))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", decimalPoint = i),
                              test = TRUE))
    }
}

"test.RxTextData.invalid.arg.thousandsSeparator" <- function()
{
    checkException(RxTextData(file = "txt", thousandsSeparator = NA_character_), silent = TRUE)
    checkException(RxTextData(file = "txt", thousandsSeparator = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", thousandsSeparator = ""), silent = TRUE)
    checkException(RxTextData(file = "txt", thousandsSeparator = "ab"), silent = TRUE)
    checkException(RxTextData(file = "txt", thousandsSeparator = c("a", "b")), silent = TRUE)
}

"test.RxTextData.valid.arg.thousandsSeparator" <- function()
{
    for (i in c(".", ",", ";"))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", thousandsSeparator = i),
                              test = TRUE))
    }
}

#test.RxTextData.invalid.arg.currencyMark <- function()
#{
#    checkException(RxTextData(file = "txt", currencyMark = NA_character_), silent = TRUE)
#    checkException(RxTextData(file = "txt", currencyMark = 1), silent = TRUE)
#    checkException(RxTextData(file = "txt", currencyMark = ""), silent = TRUE)
#    checkException(RxTextData(file = "txt", currencyMark = "ab"), silent = TRUE)
#    checkException(RxTextData(file = "txt", currencyMark = c("a", "b")), silent = TRUE)
#}

#test.RxTextData.valid.arg.currencyMark <- function()
#{
#    for (i in c("$", "#", "&"))
#    {
#        checkTrue(validObject(RxTextData(file = "TextFile.txt", currencyMark = i),
#                              test = TRUE))
#    }
#}

"test.RxTextData.invalid.arg.readDateFormat" <- function()
{
    checkException(RxTextData(file = "txt", readDateFormat = NA_character_), silent = TRUE)
    checkException(RxTextData(file = "txt", readDateFormat = 1), silent = TRUE)
    checkException(RxTextData(file = "txt", readDateFormat = c("a", "b")), silent = TRUE)
}

#test.RxTextData.invalid.arg.writeDateFormat <- function()
#{
#    checkException(RxTextData(file = "txt", writeDateFormat = NA_character_), silent = TRUE)
#    checkException(RxTextData(file = "txt", writeDateFormat = 1), silent = TRUE)
#    checkException(RxTextData(file = "txt", writeDateFormat = c("a", "b")), silent = TRUE)
#}

"test.RxTextData.invalid.arg.rowsToSniff" <- function()
{
    checkException(RxTextData(file = "txt", rowsToSniff = NULL), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSniff = NA_integer_), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSniff = "a"), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSniff = 0), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSniff = Inf), silent = TRUE)
    checkException(RxTextData(file = "txt", rowsToSniff = 1:2), silent = TRUE)
}

"test.RxTextData.valid.arg.rowsToSniff" <- function()
{
    for (i in c(1L, 10L, 100L))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsToSniff = i),
                              test = TRUE))
    }
    for (i in c(1, 10, 100))
    {
        checkTrue(validObject(RxTextData(file = "TextFile.txt", rowsToSniff = i),
                              test = TRUE))
    }
}

## Test rxImport (was rxImportToXdf)

"test.rxImportToXdf.RxTextData.delimited" <- function()
{
    expect <-  data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
        c = c("one", "two", "three"),
        stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImportX1", fileext = ".csv")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    xdfFileName <- tempfile( pattern = ".rxImportX1", fileext = ".xdf")

  currentDefaultDecimalColType = rxGetOption("defaultDecimalColType")
  rxOptions(defaultDecimalColType = "numeric")
  on.exit( rxOptions(defaultDecimalColType = currentDefaultDecimalColType), add = TRUE )

    for (tinyDS in list(RxTextData(tinyFileName), tinyFileName))
    {
        for (xdfDS in list(RxXdfData(xdfFileName), xdfFileName))
        {
            rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
            received <- rxReadXdf(xdfFileName, reportProgress = 0)
            checkEquals(received, expect)
            file.remove(xdfFileName)
        }
    }
    file.remove(tinyFileName)
  file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.noextension" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImportD", fileext = "")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImportd", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.differentExtension" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImportT", fileext = ".dat")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImportT", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.autosense" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImportA", fileext = ".txt")

    for (sep in c(",", "\t", ";", " "))
    {
        for (useFastRead in c(TRUE, FALSE))
        {
            write.table(expect, file = tinyFileName, sep = sep, row.names = FALSE)
            tinyDS <- RxTextData(tinyFileName, useFastRead = useFastRead, defaultDecimalColType = "numeric")
            xdfFileName <- tempfile( pattern = ".rxImportA", fileext = ".xdf")
            xdfDS <- RxXdfData(xdfFileName)
            rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
            received <- rxReadXdf(xdfFileName, reportProgress = 0)
            checkEquals(received, expect)
            file.remove(tinyFileName)
            file.remove(xdfFileName)
        }
    }
}


"test.rxImport.RxTextData.delimited.noColNames" <- function()
{

    expect <-
      data.frame(V1 = 1:5, V2 = head(letters, 5), V3 = (101:105)/10,
                 stringsAsFactors = FALSE)
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tinyNoNames.csv"), defaultDecimalColType = "numeric")
  xdfFileName <- tempfile(pattern = ".rxTextData", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)

    # Use StatTransfer import
    outDataFrame <- rxImport(tinyDS, overwrite = TRUE, type = "text", reportProgress = 0)
    checkEquals(outDataFrame, expect)

    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.Date" <- function()
{
  # First row of data
  # 01/02/1962 38.50 1962/01/02  01/01/02 "Jan 01 02"
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.txt")
    expect <- read.table(tinyFileName, header = TRUE, stringsAsFactors = FALSE)
  expect1 <- expect
  expect1[[3L]] <- as.Date(expect1[[3L]], format = "%Y/%m/%d")

    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric", centuryCutoff = 0
        , readDateFormat = "%Y/%m/%d"
    )
    xdfFileName <- tempfile( pattern = ".rxImportDate", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)

    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect1, check.attributes=FALSE)
    file.remove(xdfFileName)

  ## The default read format [\%y[-][/]\%m[-][/]\%d] is not valid anymore
    ## Use default read format [\%y[-][/]\%m[-][/]\%d]	# only handle one format for all column
    ## Column 3 matches this format, but we are overriding with colClasses
  #tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.txt")
    #tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
    #colClasses = c(NewDates = "character"), centuryCutoff = 0 )
    #rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  #varInfo <- rxGetVarInfo( xdfFileName )
  #checkEquals(varInfo$NoYear$varType, "character")
    #received <- rxReadXdf(xdfFileName, reportProgress = 0)
    #expect2 <- expect
  #checkEquals(received$MyDates, expect2$MyDates)
  #checkEquals(received$NewDates[1], "1962-01-02")
    #file.remove(xdfFileName)

    # Set the read format to look for read date data in column 1 & 4
  tinyDS <- RxTextData(tinyFileName, readDateFormat="%m/%d/%Y",
    centuryCutoff = 0, defaultDecimalColType = "numeric")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
  expect1 <- expect
  expect1[[1L]] <- as.Date(expect1[[1L]], format = "%m/%d/%Y")
  #expect1[[4L]] <- as.Date(expect1[[4L]], format = "%m/%d/%y")	## only one format for all columns

    #changeCentury <- expect1[[4L]]  > as.Date("1/1/2000", format = "%m/%d/%Y")
    #expect1[[4L]][ changeCentury] <-
        #(expect1[[4L]][ changeCentury ] - 36525)
  checkEquals(received, expect1, check.attributes=FALSE)
    file.remove(xdfFileName)

  tinyDS <- RxTextData(tinyFileName, readDateFormat="%Y/%m/%d", defaultDecimalColType = "numeric" )
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  expect2 <- expect
  expect2[[3L]] <- as.Date(expect2[[3L]], format = "%Y/%m/%d")
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
  checkEquals(received, expect2, check.attributes=FALSE)
    file.remove(xdfFileName)

}

"test.rxImportToXdf.RxTextData.fixed.Date" <- function()
{
  # First row of data
  # 01/02/1962 38.50 1962/01/02  01/01/02 "Jan 01 02"
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.dat")

    readDateFormat = "%m/%d/%y"
  tinyDS <- RxTextData(tinyFileName, readDateFormat=readDateFormat,
    defaultDecimalColType = "numeric",
        firstRowIsColNames = TRUE,
    useFastRead=FALSE,
        centuryCutoff = 70,
        colInfo = list(
           MyDates = list(type = "Date", start = 1, width = 10),
           Close = list(type = "numeric", start = 12, width = 5),
           NewDates = list(type = "Date", start = 18, width = 10),
           NoYear = list(type = "Date", start = 30, width = 8)
          )
        )

    xdfFileName <- tempfile( pattern = ".rxImportTemp1", fileext = ".xdf")
    rxImport(tinyDS, xdfFileName, overwrite = TRUE, reportProgress = 0, verbose=1)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)

    expect <- read.table(tinyFileName, header = TRUE, stringsAsFactors = FALSE)
  expect1 <- expect[,1:4]
  expect1[[1L]] <- as.Date(expect1[[1L]], format = "%m/%d/%Y")
  expect1[[3L]] <- as.Date(expect1[[3L]], format = "%m/%d/%Y")
    expect1[[4L]] <- as.Date(expect1[[4L]], format = "%m/%d/%y")
  checkEquals(received, expect1, check.attributes=FALSE)

    file.remove(xdfFileName)

}


"test.rxImportToXdf.RxTextData.FastFixed.Date" <- function()
{
  # First row of data
  # 01/02/1962 38.50 1962/01/02  01/01/02 "Jan 01 02"
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.dat")

    expect <- read.table(tinyFileName, header = TRUE, stringsAsFactors = FALSE)
  expect1 <- expect[,1:4]
    expect1[[1L]] <- as.Date(expect1[[1L]], format = "%m/%d/%Y", tz = "GST")
    expect1[[4L]] <- as.Date(expect1[[4L]], format = "%m/%d/%y", tz = "GST")
    print("After as.Date")

    readDateFormat = "%m/%d/%y"

    # All date formats must be the same for useFastRead = TRUE
  tinyDSfast <- RxTextData(tinyFileName, readDateFormat="%m/%d/%Y",
      defaultDecimalColType = "numeric",
      firstRowIsColNames = TRUE,
      centuryCutoff = 70,
      useFastRead=TRUE,
      colInfo = list(
         MyDates = list(type = "Date", start = 1, width = 10)
        )
      )


    xdfFileName <- tempfile( pattern = ".rxImportTemp1", fileext = ".xdf")
    rxImport(tinyDSfast, xdfFileName, overwrite = TRUE, reportProgress = 0, verbose=1)
    received1 <- rxReadXdf(xdfFileName, reportProgress = 0)
  checkEquals(received1[,1], expect1[[1L]], check.attributes=FALSE)

  # Setting useFastRead to FALSE will auto-detect date var formats not in quotes
  # Import type will be fixed format
  tinyDS <- RxTextData(tinyFileName,
    readDateFormat=readDateFormat,
    defaultDecimalColType = "numeric",
    firstRowIsColNames = TRUE,
    centuryCutoff = 70,
        useFastRead = FALSE,
    colInfo = list(
       MyDates = list( start = 1, width = 10),
       Close = list(type = "numeric", start = 12, width = 5),
       NewDates = list( start = 18, width = 10),
       NoYear = list(type = "Date", start = 30, width = 8)
      )
    )

    xdfFileName <- tempfile( pattern = ".rxImportTemp1", fileext = ".xdf")
    rxImport(tinyDS, xdfFileName, overwrite = TRUE, reportProgress = 0, verbose=1)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)

    expect <- read.table(tinyFileName, header = TRUE, stringsAsFactors = FALSE)
  expect1 <- expect[,1:4]

    expect1[[4L]] <- as.Date(expect1[[4L]], format = "%m/%d/%y", tz = "GST")
  checkEquals(received, expect1, check.attributes=FALSE)

    file.remove(xdfFileName)

}

"test.rxImportToXdf.RxTextData.fixed.POSIXct" <- function()
{

    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.dat")

    fixedTextDS <- RxTextData(file=tinyFileName, firstRowIsColNames = TRUE,
    useFastRead=FALSE,
    colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
             readPOSIXctFormat = "%y[/][-]%m[/][-]%d %H:%M[:%S]")

    xdfFileName <- tempfile( pattern = ".rxImportPOSIX", fileext = ".xdf")

    rxImport(fixedTextDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
    rxGetVarInfo(xdfFileName)
    myData <- rxReadXdf(xdfFileName)
    firstVal <- as.numeric(myData[[1]][1])
    secondVal <- as.numeric(myData[[1]][2])
    thirdVal <- as.numeric(myData[[2]][1])

    myTestVal1 <- as.numeric(as.POSIXct("1962/01/02 09:10:11", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))
    myTestVal2 <- as.numeric(as.POSIXct("1963/01/03 10:11:12", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))

    # First value in second column
    myTestVal3 <- as.numeric(as.POSIXct("1962-01-02 09:10", format = "%Y-%m-%d %H:%M", tz = "GMT"))

    checkEquals(firstVal, myTestVal1)
    checkEquals(secondVal, myTestVal2)
    checkEquals(thirdVal, myTestVal3)
    file.remove(xdfFileName)

}


"test.rxImportToXdf.RxTextData.FastFixed.POSIXct" <- function()
{

    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.dat")

  # Make sure we get an appropriate warning if useFastRead is TRUE
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)

    res <- try(
    fixedTextDS <- RxTextData(file=tinyFileName, firstRowIsColNames = TRUE,
      useFastRead=TRUE,
      colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
             readPOSIXctFormat = "%Y/%m/%d %H:%M:%S")
    )

  #checkTrue(inherits(res,"try-error"))
  #checkTrue(grepl("not supported by useFastRead", as.character(res)))
    #
    xdfFileName <- tempfile( pattern = ".rxImportPOSIX", fileext = ".xdf")

  # Should get same warning importing
    res <- rxImport(inData=tinyFileName, outFile = xdfFileName,
      firstRowIsColNames = TRUE,
      type = "fixedFast",
      colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
            readPOSIXctFormat = "%Y/%m/%d %H:%M:%S",
        overwrite = TRUE, reportProgress = 0)


  #checkTrue(inherits(res,"try-error"))
  #checkTrue(grepl("not supported by useFastRead", as.character(res)))

  # Not specifying useFastRead should work
  fixedTextDS <- RxTextData(file=tinyFileName, firstRowIsColNames = TRUE,
      colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
             readPOSIXctFormat = "%Y/%m/%d %H:%M:%S")
  rxImport(fixedTextDS, outFile = xdfFileName, overwrite=TRUE)
    rxGetVarInfo(xdfFileName)
    myData <- rxReadXdf(xdfFileName)
    firstVal <- as.numeric(myData[[1]][1])
    secondVal <- as.numeric(myData[[1]][2])
    thirdVal <- as.numeric(myData[[2]][1])

    myTestVal1 <- as.numeric(as.POSIXct("1962/01/02 09:10:11", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))
    myTestVal2 <- as.numeric(as.POSIXct("1963/01/03 10:11:12", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))

    # First value in second column
    myTestVal3 <- as.numeric(as.POSIXct("1962-01-02 09:10", format = "%Y-%m-%d %H:%M", tz = "GMT"))

    checkEquals(firstVal, myTestVal1)
    checkEquals(secondVal, myTestVal2)
    checkEquals(thirdVal, myTestVal3)
    file.remove(xdfFileName)

}

"test.rxImportToXdf.RxTextData.fixed.POSIXct.GMT" <- function()
{
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.dat")

    fixedTextDS <- RxTextData(file=tinyFileName, firstRowIsColNames = TRUE,
    useFastRead=FALSE,
    colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
       readPOSIXctFormat = "%y[/][-]%m[/][-]%d %H:%M[:%S]")

    xdfFileName <- tempfile( pattern = ".rxImportGMt", fileext = ".xdf")

    rxImport(fixedTextDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo(xdfFileName)
  checkEquals( varInfo$MyDateTime$tzone, "GMT")

    myData <- rxReadXdf(xdfFileName)
    firstVal <- as.numeric(myData[[1]][1])
    secondVal <- as.numeric(myData[[1]][2])
    thirdVal <- as.numeric(myData[[2]][1])

    myTestVal1 <- as.numeric(as.POSIXct("1962/01/02 09:10:11", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))
    myTestVal2 <- as.numeric(as.POSIXct("1963/01/03 10:11:12", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))

    # First value in second column
    myTestVal3 <- as.numeric(as.POSIXct("1962-01-02 09:10", format = "%Y-%m-%d %H:%M", tz = "GMT"))

    checkEquals(firstVal, myTestVal1)
    checkEquals(secondVal, myTestVal2)
    checkEquals(thirdVal, myTestVal3)
    file.remove(xdfFileName)

}


"test.rxImportToXdf.RxTextData.FastFixed.POSIXct.GMT" <- function()
{
  # Check to make sure warning message is given
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.dat")

  # Make sure we get an appropriate warning if useFastRead is TRUE
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)

    res <- try(
    fixedTextDS <- RxTextData(file=tinyFileName,
          useFastRead=TRUE,
          colInfo = list(
                 MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)),
          readPOSIXctFormat = "%Y/%m/%d %H:%M:%S")
    )

  checkEquals(res@colInfo$MyDateTime$type, "POSIXct")
  checkEquals(res@colInfo$MyDateTime2$type, "POSIXct")

}

"test.rxImportToXdf.RxTextData.delimited.varsToKeep" <- function()
{
    tinyFileName <- tempfile( pattern = ".rxImportV", fileext = ".txt")
  rxTiny <-  data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    write.csv(rxTiny, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, varsToKeep = c("a", "c"),
        defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport01Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    expect <- rxTiny[c("a", "c")]
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)

    write.csv(rxTiny, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, varsToKeep = "b",
        defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport02Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    expect <- rxTiny["b"]
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.varsToDrop" <- function()
{
    tinyFileName <- tempfile( pattern = ".rxImportV2", fileext = ".txt")
  rxTiny <-  data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    write.csv(rxTiny, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, varsToDrop = "b",
        defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImportV2", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    expect <- rxTiny[c("a", "c")]
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)


    write.csv(rxTiny, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, varsToDrop = c("a", "c"),
        defaultDecimalColType = "numeric")

    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    expect <- rxTiny["b"]
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)

}

"test.rxImport.RxTextData.delimited.rowsToSkip" <- function()
{
    # Only works for textFast
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tiny3.txt"),
                 rowsToSkip = 2, colClasses = c(V2 = "numeric"))

    received <- rxImport(tinyDS, reportProgress = 0)
    checkEquals(received[[1]], expect[[1]])
    checkEquals(received[[2]], expect[[2]])
    checkEquals(received[[3]], expect[[3]])
}

"test.rxImport.RxTextData.delimited.rowsToSkip.numRows" <- function()
{
  # Rows to skip is greater than rows Per Read
  dataSource <- RxTextData(file = file.path(rxGetOption("unitTestDataDir"),"richTest300rComma.txt"),
  rowsPerRead = 20)

  myData1 <- rxImport( inData = dataSource, numRows = 50, rowsToSkip = 120L, useFastRead = TRUE)
  checkEquals(nrow(myData1), 50)

  # Rows to skip is less than rows Per Read
  dataSource <- RxTextData(file = file.path(rxGetOption("unitTestDataDir"),"richTest300rComma.txt"),
  rowsPerRead = 70)

  myData2 <- rxImport( inData = dataSource, numRows = 50, rowsToSkip = 40L, useFastRead = TRUE)
  checkEquals(nrow(myData2), 50)
}



"test.rxImportToXdf.RxTextData.delimited.stringsAsFactors" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
    c = factor(c("one", "two", "three"), levels = c("one", "two", "three")))
    tinyFileName <- tempfile( pattern = ".rxImport03Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, stringsAsFactors = TRUE, defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport04Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colClasses" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
             c = factor(c("one", "two", "three"), levels = c("one", "two", "three")))
    expect[["a"]] <- as.numeric(expect[["a"]])
    tinyFileName <- tempfile( pattern = ".rxImport05Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colClasses =
                         c(a = "numeric",
                           b = "numeric",
                           c = "factor"))
    xdfFileName <- tempfile( pattern = ".rxImport05Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colClasses.NAs" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = TRUE)
    expect[2, ] <- NA
    expect[["c"]] <- expect[["c"]][drop = TRUE]
    tinyFileName <- tempfile( pattern = ".rxImport06Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDSnotDirect <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colClasses =
                         c(a = "integer",
                           b = "numeric",
                           c = "factor"), useFastRead = FALSE)
    tinyDSdirect <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colClasses =
                         c(a = "integer",
                           b = "numeric",
                           c = "factor"), useFastRead = TRUE)
    xdfFileName <- tempfile( pattern = ".rxImport06Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDSnotDirect, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)

    rxImport(tinyDSdirect, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
    file.remove(tinyFileName)

    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    for (type in c("numeric", "logical", "character"))
    {
        expect[["c"]] <- rep(as(NA, type), nrow(expect))
        tinyFileName <- tempfile( pattern = ".rxImport07Text", fileext = ".txt")
        write.csv(expect, file = tinyFileName, row.names = FALSE)
        tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                             colClasses =
                             c(a = "integer",
                               b = "numeric",
                               c = type), useFastRead=FALSE)
        xdfFileName <- tempfile( pattern = ".rxImport07Text", fileext = ".xdf")
        xdfDS <- RxXdfData(xdfFileName)
        rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- rxReadXdf(xdfFileName, reportProgress = 0)
        checkEquals(received, expect)
        file.remove(tinyFileName)
        file.remove(xdfFileName)
    }
}

"test.rxImportToXdf.RxTextData.delimited.colClasses.characters" <- function()
{
    expect <-
      data.frame(a = letters[1:3], b = LETTERS[1:3], stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport08Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName,  defaultDecimalColType = "numeric",)
    xdfFileName <- tempfile( pattern = ".rxImport08Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colClasses.logicals" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
    c = factor(c("one", "two", "three"), levels = c("one", "two", "three")),
    d = c(TRUE, FALSE, TRUE), e = c(0L, 1L, 1L))

    expect[["a"]] <- as.numeric(expect[["a"]])
    expect[["e"]] <- as.logical(expect[["e"]])
    tinyFileName <- tempfile( pattern = ".rxImport09Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colClasses =
                         c(a = "numeric",
                           b = "numeric",
                           c = "factor",
                           d = "logical",
                           e = "logical"))
    xdfFileName <- tempfile( pattern = ".rxImport09Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colInfo.type" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
    c = factor(c("one", "two", "three"), levels = c("one", "two", "three")))
    expect[["a"]] <- as.numeric(expect[["a"]])
    tinyFileName <- tempfile( pattern = ".rxImport10Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colInfo =
                         list(a = list(type = "numeric"),
                              b = list(type = "numeric"),
                              c = list(type = "factor")))
    xdfFileName <- tempfile( pattern = ".rxImport10Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colInfo.levels" <- function()
{
    extraLevels <- c("one", "two", "three", "four", "five")
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    expect[["c"]] <- factor(expect[["c"]], levels = extraLevels)
    tinyFileName <- tempfile( pattern = ".rxImport11Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colInfo =
                         list(c = list(type = "factor", levels = extraLevels)))
    xdfFileName <- tempfile( pattern = ".rxImport11Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)

    lessLevels <- c("one", "two", "three", "four", "five")
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    expect[["c"]] <- factor(expect[["c"]], levels = lessLevels)
    tinyFileName <- tempfile( pattern = ".rxImport12Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colInfo =
                         list(c = list(type = "factor", levels = extraLevels)))
    xdfFileName <- tempfile( pattern = ".rxImport12Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.colInfo.logicals" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3),
    c = factor(c("one", "two", "three"), levels = c("one", "two", "three")),
    d = c(TRUE, FALSE, TRUE), e = c(0L, 1L, 1L))
    expect[["a"]] <- as.numeric(expect[["a"]])
    expect[["e"]] <- as.logical(expect[["e"]])
    tinyFileName <- tempfile( pattern = ".rxImport13Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
                         colInfo =
                         list(a = list(type = "numeric"),
                              b = list(type = "numeric"),
                              c = list(type = "factor"),
                              d = list(type = "logical"),
                              e = list(type = "logical")))
    xdfFileName <- tempfile( pattern = ".rxImport13Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.missingValueString" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    expect[2,] <- NA
    tinyFileName <- tempfile( pattern = ".rxImport14Text", fileext = ".txt")
    writeLines(c("a,b,c",
                 "1,4.1,one",
                 "NA,NA,NA",
                 "3,6.3,three"), tinyFileName)
    xdfFileName <- tempfile( pattern = ".rxImport14Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
        missingValueString = "NA")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)


    writeLines(c("a,b,c",
                 "1,4.1,one",
                 ".,.,.",
                 "3,6.3,three"), tinyFileName)

    xdfDS <- RxXdfData(xdfFileName)
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
        missingValueString = ".")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.rowsPerRead" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport15Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, rowsPerRead = 2, defaultDecimalColType = "numeric",)
    xdfFileName <- tempfile( pattern = ".rxImport15Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    xdfInfo <- rxGetInfo(xdfFileName, getBlockSizes = TRUE)
    if (rxIsExpressEdition())
    {
        checkEquals(xdfInfo[["numBlocks"]], 1L)
        checkEquals(xdfInfo[["rowsPerBlock"]], 3L)
    } else
    {
        checkEquals(xdfInfo[["numBlocks"]], 2L)
        checkEquals(xdfInfo[["rowsPerBlock"]], c(2L, 1L))
    }

    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.delimiter" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)

   # comma
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tiny.txt"),
                 delimiter = ",", defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport16Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)

    # tab
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tinyTab.txt"),
                 delimiter = "\t", defaultDecimalColType = "numeric")

    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)

    # space
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tinySpace.txt"),
                 delimiter = " ", defaultDecimalColType = "numeric")

    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)

    # pound sign
    tinyDS <-
      RxTextData(file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt"),
                delimiter = "#", defaultDecimalColType = "numeric")

    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImport.RxTextData.delimited.combineDelimiters" <- function()
{
    for (sep in c(",", "\t", ";"))
    {
        expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
        tinyFileName <- tempfile( pattern = ".rxImport01Temp", fileext = ".txt")
        writeLines(gsub("#", sep,
                        c("a#b##c",
                          "1###4.1####one",
                          "2#####5.2######two",
                          "3#######6.3########three")), tinyFileName)
        xdfFileName <- tempfile( pattern = ".rxImport01Temp", fileext = ".xdf")
        xdfDS <- RxXdfData(xdfFileName)

        for (useFastRead in c(TRUE, FALSE))
        {
            ## First show results not right
      if (useFastRead == FALSE)
      {
        # useFastRead set to FALSE imports data, but incorrectly
                tinyDS <- RxTextData(tinyFileName, combineDelimiters = FALSE,
                    defaultDecimalColType = "numeric", useFastRead = useFastRead)
                rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
                received <- rxReadXdf(xdfFileName, reportProgress = 0)
                checkTrue(!isTRUE(all.equal(received, expect)))
      }

            ## Then show argument corrects situation
            tinyDS <- RxTextData(tinyFileName, combineDelimiters = TRUE,
                defaultDecimalColType = "numeric", useFastRead = useFastRead)
            rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
            received <- rxReadXdf(xdfFileName, reportProgress = 0)
            checkEquals(received, expect)
        }


        file.remove(tinyFileName)
        file.remove(xdfFileName)
    }
}

"test.rxImportToXdf.RxTextData.delimited.quoteMark" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport17Text", fileext = ".txt")
    writeLines(c("'a','b','c'",
                 "1,4.1,'one'",
                 "2,5.2,'two'",
                 "3,6.3,'three'"), tinyFileName)
    xdfFileName <- tempfile( pattern = ".rxImport17Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)

    for (useFastRead in c(TRUE, FALSE))
    {
        ## First show results are not right
        tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric",
          useFastRead = useFastRead)
        rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- rxReadXdf(xdfFileName, reportProgress = 0)
        checkTrue(!isTRUE(all.equal(received[[3]], expect[[3]])))

        ## Then show argument corrects situation
        tinyDS <- RxTextData(tinyFileName, quoteMark = "'", defaultDecimalColType = "numeric",
            useFastRead = useFastRead)
        rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- rxReadXdf(xdfFileName, reportProgress = 0)
        checkEquals(received, expect)
    }

    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.decimalPoint" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport18Text", fileext = ".txt")
    writeLines(c("a\tb\tc",
                 "1\t4,1\tone",
                 "2\t5,2\ttwo",
                 "3\t6,3\tthree"), tinyFileName)
    xdfFileName <- tempfile( pattern = ".rxImport18Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)

    ## First show results are not right
    tinyDS <- RxTextData(tinyFileName, delimiter = "\t", defaultDecimalColType = "numeric")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkTrue(!isTRUE(all.equal(received, expect)))

    ## Then show argument corrects situation
    tinyDS <- RxTextData(tinyFileName, delimiter = "\t", decimalPoint = ",",
    defaultDecimalColType = "numeric")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)

    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.thousandsSeparator" <- function()
{
    # Note: will fail if using useFastRead text import (RA-4502)
    expect <-
      data.frame(a = 1:3, b = c(54321.2, 7654321.4, 9876543210.6),
                 c = c("one", "two", "three"), stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport19Text", fileext = ".txt")
    writeLines(c("a\tb\tc",
                 "1\t54,321.2\tone",
                 "2\t7,654,321.4\ttwo",
                 "3\t9,876,543,210.6\tthree"), tinyFileName)
    tinyDS <-
      RxTextData(tinyFileName, colClasses = c(b = "numeric"),
        delimiter = "\t", thousandsSeparator = ",")
    xdfFileName <- tempfile( pattern = ".rxImport19Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImport.RxTextData.delimited.noRowNamesName" <- function()
{

    testDF <- data.frame(x=11:15, y=21:25)
    tinyFileName <- tempfile( pattern = ".rxImport21Text", fileext = ".txt")
    write.table(testDF, tinyFileName, sep=",")
    # By default, R writes a csv file that has a first column of row names
    # This column does not have a name, so we need to give it one: .rxRowNames
    inData1 <- rxImport(inData = tinyFileName)
    checkEquals(names(inData1), c(".rxRowNames", "x", "y"))
    file.remove(tinyFileName)
}

"test.rxImportToXdf.RxTextData.delimited.firstRowIsColNames" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport20Text", fileext = ".txt")
    write.csv(expect, file = tinyFileName, row.names = FALSE)
    tinyDS <- RxTextData(tinyFileName, firstRowIsColNames = TRUE,
    defaultDecimalColType = "numeric",)
    xdfFileName <- tempfile( pattern = ".rxImport20Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)

    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- tempfile( pattern = ".rxImport21Text", fileext = ".txt")
    suppressWarnings(write.table(expect, file = tinyFileName, sep = ",",
                                 row.names = FALSE, col.names = FALSE))
    tinyDS <- RxTextData(tinyFileName, firstRowIsColNames = FALSE,
    defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport21Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    colnames(received) <- colnames(expect)
    checkEquals(received, expect)
    file.remove(tinyFileName)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.rowsToSniff" <- function()
{
    # See RA-2660 and RA-4503
    tinyFileName <- tempfile( pattern = ".rxImport22Text", fileext = ".txt")
    writeLines(c("a,b,c",
                 "1,1,1",
                 "NA,NA,NA",
                 "three, three,three"), tinyFileName)
    xdfFileName <- tempfile( pattern = ".rxImport22Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)

    ## First show results are not right
    tinyDS <- RxTextData(tinyFileName, firstRowIsColNames = TRUE, rowsToSniff = 2,
        defaultDecimalColType = "numeric")
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)

    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(sapply(received, class),
                c(a = "integer", b = "integer", c = "integer"))

    ## Then show argument corrects situation
    # Doesn't work if useFastRead = TRUE
    tinyDS <- RxTextData(tinyFileName, firstRowIsColNames = TRUE, rowsToSniff = 4,
       defaultDecimalColType = "numeric", useFastRead = FALSE)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(sapply(received, class),
                c(a = "character", b = "character", c = "character"))

    file.remove(tinyFileName)
    file.remove(xdfFileName)
}


"test.rxImportToXdf.RxTextData.delimited.defaultDecimalColType" <- function()
{
    expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric")
    xdfFileName <- tempfile( pattern = ".rxImport23Text", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(tinyDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImport.RxTextData.delimited.defaultMissingColType" <- function()
{
    for (type in c("numeric", "character"))
    {
        expect <- data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
        expect[,2] <- rep.int(as(NA, type), nrow(expect))
        tinyFileName <- tempfile( pattern = ".rxImport24Text", fileext = ".txt")
        tinyFileName <- "tiny.txt"
        writeLines(c("a,b,c",
                     "1,,one",
                     "2,,two",
                     "3,,three"), tinyFileName)

        # Only works for useFastRead TRUE
        tinyDS <- RxTextData(tinyFileName, defaultMissingColType = type)
        received <- rxImport(tinyDS, overwrite = TRUE, reportProgress = 0)
        checkEquals(received, expect)
        file.remove(tinyFileName)
    }
}

"test.RxTextData.args.varsToKeep.useFastRead" <- function()
{
    dataFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")

    varsToKeep <- c("y", "xfac1")
    myTextDataNotDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = FALSE)
    myTextDataDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = TRUE)
    myDataNotDirect <- rxImport(inData = myTextDataNotDirect )
    myDataDirect <- rxImport(inData = myTextDataDirect )
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
    checkEquals(myDataNotDirect, myDataDirect)

    # Create multiple blocks in .xdf file
    testXdfDirect <- tempfile(pattern = "testCompressDirect", fileext = ".xdf")
    testXdfNotDirect <- tempfile(pattern = "testCompressNotDirect", fileext = ".xdf")

    varsToKeep <- c("y", "xfac1")
    myTextDataNotDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = FALSE)
    myTextDataDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = TRUE)
    rxImport(inData = myTextDataNotDirect, outFile = testXdfNotDirect, rowsPerRead = 30 , overwrite=TRUE)
    rxImport(inData = myTextDataDirect, outFile = testXdfDirect, rowsPerRead = 30, overwrite=TRUE )
    fileInfoNotDirect <- rxGetInfo( testXdfNotDirect )
    fileInfoDirect <- rxGetInfo( testXdfDirect )
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfoNotDirect$numBlocks, 1 )
        checkEquals(fileInfoDirect$numBlocks, 1 )
    } else
    {
        checkEquals(fileInfoNotDirect$numBlocks, 4 )
        checkEquals(fileInfoDirect$numBlocks, 4 )
    }

    myDataNotDirect <- rxReadXdf(testXdfNotDirect)
    myDataDirect <- rxReadXdf(testXdfDirect)
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
    checkEquals(myDataNotDirect, myDataDirect)
    file.remove(testXdfDirect)
    file.remove(testXdfNotDirect)

    # Reverse the order
    varsToKeep <- c("xfac1", "y")
    myTextDataNotDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = FALSE)
    myTextDataDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = TRUE)
    myDataNotDirect <- rxImport(inData = myTextDataNotDirect)
    myDataDirect <- rxImport(inData = myTextDataDirect )
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
    checkEquals(myDataNotDirect, myDataDirect)

    # Same thing, where one variable name is contained within another
    varsToKeep <- c("y", "ylogical")
    myTextDataNotDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = FALSE)
    myTextDataDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep, useFastRead = TRUE)
    myDataNotDirect <- rxImport(inData = myTextDataNotDirect )
    myDataDirect <- rxImport(inData = myTextDataDirect )
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
    checkEquals(myDataNotDirect$y, myDataDirect$y)
    # StatTran will bring in TRUE/FALSE as character
    checkEquals(as.logical(myDataNotDirect$ylogical), myDataDirect$ylogical)

    # Use a space delimited file (goes through slow parse)
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.dat")
    varsToKeep = c("Close", "MonthStr")

    myTextDataNotDirect <- RxTextData( tinyFileName, varsToKeep = varsToKeep, useFastRead = FALSE,
                                       firstRowIsColNames = TRUE, combineDelimiters = TRUE)
    myTextDataNotDirect <- rxImport(myTextDataNotDirect)


    myDataNotDirect <- rxImport(myTextDataNotDirect)
    myDataDirect <- rxImport(inData = tinyFileName, varsToKeep = varsToKeep,
                             useFastRead = TRUE, readDateFormat = "%Y/%m/%d")
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
    checkEquals(myDataNotDirect, myDataDirect)
}

"test.RxTextData.varsToDrop" <- function()
{
    textFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")

    varsToDrop<- c("y", "xfac1")

    myTextDataNotDirect <- RxTextData( textFileName, varsToDrop = varsToDrop, useFastRead = FALSE)
    myTextDataDirect <- RxTextData( textFileName, varsToDrop = varsToDrop, useFastRead = TRUE)

    allVarNames <- rxGetVarNames( RxTextData(textFileName) )
    varsToKeep = allVarNames[!(allVarNames %in% varsToDrop)]

    myDataNotDirect <- rxImport(inData = myTextDataNotDirect )
    myDataDirect <- rxImport(inData = myTextDataDirect )
    checkEquals( names(myDataNotDirect), varsToKeep)
    checkEquals( names(myDataDirect), varsToKeep)
}

"test.RxTextData.colInfo" <- function()
{

    # The last column has Yes and No
  dsText <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
                         useFastRead = FALSE,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer",
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )

    myData <- rxImport(dsText)
    mySummary <- rxSummary(~., dsText)
    defaultCounts <- rxResultsDF(mySummary, "counts")
    checkEquals(defaultCounts$Counts, c(22,3))

    myLinMod <- rxLinMod(creditScore~default, data=dsText)
  checkEquals(names(coef(myLinMod)), c("(Intercept)", "default=No",  "default=Yes"))

  # Fast delimited text
  dsTextFast <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
                         useFastRead = TRUE,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer",
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )

    myData2 <- rxImport(dsTextFast)
  checkEquals(myData, myData2)
    mySummary <- rxSummary(~., dsTextFast)
    defaultCounts <- rxResultsDF(mySummary, "counts")
    checkEquals(defaultCounts$Counts, c(22,3))

    myLinMod <- rxLinMod(creditScore~default, data=dsTextFast)
  checkEquals(names(coef(myLinMod)), c("(Intercept)", "default=No",  "default=Yes"))

  # Fixed text

  fixedColInfo <- list(
    creditScore = list( start = 1, width = 3, type = "integer"),
    houseAge = list( start = 5, width = 2, type = "integer"),
    yearsEmploy = list( start = 8, width = 1, type = "integer"),
    ccDebt = list(start = 10, width = 4, type = "integer"),
    year = list(start = 15, width = 4, type = "integer"),
    default = list( start = 20, width = 3, type = "factor", levels = c("No", "Yes")))

  fixedFile <-file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.dat")

  # Fast fixed
  dsFixedFast <- RxTextData(fixedFile, colInfo = fixedColInfo,
                         useFastRead = TRUE)

    myData3 <- rxImport(dsFixedFast)
  checkEquals(myData, myData3)
    mySummary <- rxSummary(~., dsFixedFast)
    defaultCounts <- rxResultsDF(mySummary, "counts")
    checkEquals(defaultCounts$Counts, c(22,3))

    myLinMod <- rxLinMod(creditScore~default, data=dsFixedFast)
  checkEquals(names(coef(myLinMod)), c("(Intercept)", "default=No",  "default=Yes"))

  # Standard fixed
  dsFixed <- RxTextData(fixedFile, colInfo = fixedColInfo,
                        firstRowIsColNames = FALSE,	# fixed format
            useFastRead = FALSE)

    myData4 <- rxImport(dsFixed)
  checkEquals(myData, myData4)
    mySummary <- rxSummary(~., dsFixed)
    defaultCounts <- rxResultsDF(mySummary, "counts")
    checkEquals(defaultCounts$Counts, c(22,3))

    myLinMod <- rxLinMod(creditScore~default, data=dsFixed)
  checkEquals(names(coef(myLinMod)), c("(Intercept)", "default=No",  "default=Yes"))

}

"test.RxTextData.args.delimited.missingValueString" <- function()
{
    testFileTxt <- tempfile(pattern = ".rxTD1TempTest1", fileext = ".txt")

    writeLines(c("a,b,c,d",
             "NA, -7, M, -7",
             "-7, NA, -7, M",
             "0.5, 0.5, 0.5, 0.5"), testFileTxt)
    # Use the default of "NA"
    myData1 <- rxImport(testFileTxt, type = "textFast")

    varInfo1 <- rxGetVarInfo(myData1)
    checkEquals(varInfo1$a$varType, "numeric")
    checkEquals(varInfo1$b$varType, "numeric")
    checkEquals(varInfo1$c$varType, "character")
    checkEquals(varInfo1$d$varType, "character")

    myData2 <- rxImport(testFileTxt, type = "text")

    checkEquals(myData1, myData2)

    # Now set the missingValueString to "M"
    myData3 <- rxImport(testFileTxt, type = "textFast", missingValueString = "M")
    varInfo3 <- rxGetVarInfo(myData3)
    checkEquals(varInfo3$a$varType, "character")
    checkEquals(varInfo3$b$varType, "character")
    checkEquals(varInfo3$c$varType, "numeric")
    checkEquals(varInfo3$d$varType, "numeric")

    myData4 <- rxImport(testFileTxt, type = "text", missingValueString = "M")
    checkEquals(myData3, myData4)

    checkEquals(myData1$a, myData3$c)
    checkEquals(myData1$b, myData3$d)

    file.remove(testFileTxt)

}

"test.RxTextData.rxGetVarInfo.low.high" <- function()
{
  textDS <- RxTextData(
                file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
                colClasses = c("V3" = "int16"))
    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))

  textDS <- RxTextData(
            file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
            colClasses = c("V3" = "uint16"))

    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))

  textDS <- RxTextData(
            file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
            colClasses = c("V3" = "uint16"))

    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))

  textDS <- RxTextData(
            file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
            colClasses = c("V3" = "logical"))

    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))

  textDS <- RxTextData(
            file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
            colClasses = c("V3" = "integer"))

    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))

  textDS <- RxTextData(
            file = file.path(rxGetOption("sampleDataDir"), "claims.dat"),
            colClasses = c("V3" = "float32"))

    vInfo <- rxGetVarInfo(textDS)
    checkTrue(is.na(vInfo[["V3"]]$low))
  checkTrue(is.na(vInfo[["V3"]]$high))
}


## Fast Fixed-format test
"test.RxTextData.fixed.args.useFastRead" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  gamesDAT <- file.path(unitTestDataDir, "Games.dat")
  fastGamesXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  slowGamesXDF <- file.path(tempdir(), ".rxOutFileTemp2.xdf")

  colInfo = list(
      Col1 = list(newName = "Month", start=1, width=1, type="integer"),
      Col2 = list(newName = "Day",  start=3, width=2, type="integer"),
      Col3 = list(newName = "Team", start=6, width=19 ),
      Col4 = list(newName = "Hits", start=26, width=2, type="float32"),
      Col5 = list(newName = "Runs", start=29, width=2, type="numeric")
      )

  # Create an data set with two blocks to test cumulative values
  slowGamesDS <- RxTextData(file=gamesDAT,
    missingValueString=".", rowsPerRead = 5, colInfo = colInfo,
    isFixedFormat = TRUE, useFastRead = FALSE)

  fastGamesDS <- RxTextData(file=gamesDAT,
    missingValueString=".", rowsPerRead = 5, colInfo = colInfo,
    isFixedFormat = TRUE, useFastRead = TRUE)

  rxImport(slowGamesDS, outFile = slowGamesXDF, verbose = TRUE, overwrite = TRUE)
  rxImport(fastGamesDS, outFile = fastGamesXDF, verbose = TRUE, overwrite = TRUE)

  varInfoSlow <- rxGetVarInfo(slowGamesXDF)
  varInfoFast <- rxGetVarInfo(fastGamesXDF)

  file.remove(fastGamesXDF)
  file.remove(slowGamesXDF)

  checkEquals(varInfoSlow, varInfoFast)

}


"test.RxTextData.args.rowsPerRead.analysis" <- function()
{
  txtFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")

  # Create a data source that does not require 2 passes
  textDS1 <- RxTextData(file = txtFile, rowsPerRead = 20, colInfo =
    list(xfac1 = list(type = "factor", levels = as.character(1:14))))

  output1 <- capture.output(
    rxCrossTabs(~xfac1, data = textDS1, reportProgress = 2)
    )

    if (rxIsExpressEdition())
    {
        checkTrue(length(grep("100", output1[1])) > 0)
    } else
    {
      checkTrue(length(grep("20", output1[1])) > 0)
      checkTrue(length(grep("40", output1[2])) > 0)
      checkTrue(length(grep("60", output1[3])) > 0)
     }

  # Create a data source that does require 2 passes
  # stringsAsFactors set to TRUE requires 2 passes
  textDS2 <- RxTextData(file = txtFile, stringsAsFactors = TRUE, rowsPerRead = 20)

  output2 <- capture.output(
    rxCrossTabs(~xfac1, data = textDS2, reportProgress = 2)
    )

    if (rxIsExpressEdition())
    {
        checkTrue(length(grep("100", output1[1])) > 0)
    } else
    {
      checkTrue(length(grep("20", output2[1])) > 0)
      checkTrue(length(grep("40", output2[2])) > 0)
      checkTrue(length(grep("60", output2[3])) > 0)
    }
}

"test.RxTextData.args.colClasses.stringToNumeric" <- function()
{
  x <- c(1.1, 2.2, 3.3)
  y <- as.character(x)

  txtFile1 = tempfile(pattern = ".rxSmall", fileext = ".csv")
  write.table(x = data.frame(x=x, y=y), file = txtFile1, row.names = FALSE, sep="," )
  data1 <- rxImport(txtFile1, colClasses = c(y = "numeric"),
    defaultDecimalColType = "numeric", type = "text")
  data2 <- rxImport(txtFile1, colClasses = c(y = "numeric"),
    defaultDecimalColType = "numeric", type = "textFast")
  checkEquals(data1$x, data1$y)
  checkEquals(data2$x, data2$y)

  data1 <- rxImport(txtFile1, colClasses = c(y = "integer"),
    defaultDecimalColType = "numeric", type = "text")
  data2 <- rxImport(txtFile1, colClasses = c(y = "integer"),
    defaultDecimalColType = "numeric", type = "textFast")
  checkEquals(data1$y, data2$y)

  data1 <- rxImport(txtFile1, colClasses = c(x = "character"),
    defaultDecimalColType = "numeric",  type = "text")
  data2 <- rxImport(txtFile1, colClasses = c(x = "character"),
    defaultDecimalColType = "numeric", type = "textFast")
  checkEquals(data1$x, data2$x)
  checkEquals(data1$x, data1$y)

  file.remove(txtFile1)
}

"test.RxTextData.method.show" <- function()
{
  dataFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")
    dataSource <- RxTextData(file = dataFileName,
        stringsAsFactors = TRUE,
        colClasses = c(xint1 = "integer", xnum1 = "numeric"),
        colInfo = list(xfac1 = list(type = "factor", levels = c("One", "Two", "Three"))),
        varsToKeep = c("xint1", "xfac1", "xnum1"),
        missingValueString = "M",
        rowsPerRead = 250000, delimiter = ",", combineDelimiters = FALSE,
        decimalPoint = ",",
        readDateFormat = "[%m[-][/]%d[-][/]%y]",
        centuryCutoff = 20, firstRowIsColNames = TRUE, rowsToSniff = 10000,
        returnDataFrame = TRUE,
        defaultReadBufferSize = 10000,
        defaultDecimalColType = "numeric")

    o1 <- capture.output(show(dataSource))
    checkEquals(o1[1], "RxTextData Source" )
    checkEquals(o1[3], "stringsAsFactors: TRUE")
    checkEquals(o1[4], "colClasses: ")
    checkEquals(o1[5], "   xint1: integer")
    checkEquals(o1[6], "   xnum1: numeric")
    checkEquals(o1[7], "colInfo: ")
    checkEquals(o1[8], "   xfac1: ")
    checkEquals(o1[9], "       type: factor")
    checkEquals(o1[10], "       levels: One Two Three")
    checkEquals(o1[11], "missingValueString: M" )
    checkEquals(o1[12], "delimiter: ," )
    checkEquals(o1[13], "decimalPoint: ,")
    checkEquals(o1[14], "readDateFormat: [%m[-][/]%d[-][/]%y]")
    #checkEquals(o1[15], "readPOSIXctFormat: [%m[-][/]%d[-][/]%y]")
    checkEquals(o1[15], "centuryCutoff: 20")
    checkEquals(o1[16], "firstRowIsColNames: TRUE" )
    checkEquals(o1[17], "rowsToSniff: 10000")
    checkEquals(o1[18], "rowsToSkip: 0")
    checkEquals(o1[19], "defaultReadBufferSize: 10000")
    checkEquals(o1[20], "defaultDecimalColType: numeric")
    checkEquals(o1[21], "isFixedFormat: FALSE" )
    checkEquals(o1[22], "useFastRead: TRUE")
    checkEquals(o1[23], "fileSystem: ")
    checkEquals(o1[24], "    fileSystemType: native")
}

"test.RxTextData.method.head" <- function()
{
  dataFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")

  xdfFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r1b.xdf")

  xdfDS <- RxXdfData(xdfFileName)
  headXdf <- head(xdfDS)
  varsToKeep <- c("y", "xfac1")
  headXdf <- subset(headXdf, select = varsToKeep)

    textDSNotDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep,
    stringsAsFactors = TRUE, firstRowIsColNames = TRUE, useFastRead = FALSE)
    textDSDirect <- RxTextData( dataFileName, varsToKeep = varsToKeep,
    stringsAsFactors = TRUE, useFastRead = TRUE)
  head1 <- head(textDSNotDirect)
  head2 <- head(textDSDirect)
  checkEquals(head1, head2)
  # The text data sources will have fewer factor levels
  checkEquals(head1, headXdf, check.attributes = FALSE)
}

"test.RxTextData.method.names" <- function()
{
   textFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")

  textDS <- RxTextData( textFileName)
  importNames <- names(rxImport(textDS, numRows = 3))
  methodNames <- names(textDS)
  checkEquals(importNames, methodNames)

  textDSslow <- RxTextData( textFileName, firstRowIsColNames = TRUE, useFastRead = FALSE)
  slowNames <- names(textDSslow)
  checkEquals(slowNames, methodNames)
}

"test.RxTextData.fixed.data.blank" <- function()
{
  # Create a text data file that can be treated as
  # fixed format
  x <- data.frame(x = c(NA, NA, 3, 4),
    x1 = c(NA, 1, NA, 2),
    x1a = c(NA, 1, NA, 2),
    x2 = c(NA, 1, 0, NA),
    y = c(1.1, 2.2, NA, 3.3),
    z =  c(1.1, 2.2, NA, 3.3))
  inText <- tempfile(pattern = ".rxBlank", fileext = ".dat")
  write.table(x, file = inText, row.names = FALSE, col.names = FALSE, na = " ")

  textColInfo <- list(
    V1 = list(start = 1, width = 2, type = "integer"),
    V2 = list(start = 3, width = 2, type = "int16"),
    V3 = list(start = 5, width = 2, type = "uint16"),
    V4 = list(start = 7, width = 2, type = "logical"),
    V5 = list(start = 9, width = 3, type = "float32"),
    V6 = list(start = 12, width = 3, type = "numeric"))

  myData1 <- rxImport(inText, colInfo = textColInfo, type = "fixed", firstRowIsColNames = FALSE)
  myData2 <- rxImport(inText, colInfo = textColInfo, type = "fixedFast")
  checkEquals(myData1, myData2)

  file.remove(inText)

}

"test.RxTextData.delimited.data.blank" <- function()
{
  # Create a text data file that can be treated as
  # delimited
  x <- data.frame(x = c(NA, NA, 3, 4),
    x1 = c(NA, 1, NA, 2),
    x1a = c(NA, 1, NA, 2),
    x2 = c(NA, 1, 0, NA),
    y = c(1.1, 2.2, NA, 3.3),
    z =  c(1.1, 2.2, NA, 3.3))
  inText <- tempfile(pattern = ".rxBlank", fileext = ".dat")
  write.table(x, file = inText, row.names = FALSE, col.names = FALSE, sep = ",", na = " ")

  textColInfo <- list(
    list( index = 1, newName = "X1", type = "integer"),
    list( index = 2, newName = "X2", type = "int16"),
    list( index = 3, newName = "X3", type = "uint16"),
    list( index = 4, newName = "X4", type = "logical"),
    list( index = 5, newName = "X5", type = "float32"),
    list( index = 6, newName = "X6", type = "numeric"))

  myData1 <- rxImport(inText, colInfo = textColInfo, type = "text")
  myData2 <- rxImport(inText, colInfo = textColInfo, type = "textFast")
  checkEquals(myData1, myData2)

    # Try simple case as well
    myData1 <- rxImport(inText, type = "text")
  myData2 <- rxImport(inText, type = "textFast")
  checkEquals(myData1, myData2)

  file.remove(inText)

}

"test.RxTextData.args.writePrecision.skipZeros" <- function()
{
  # Create a small xdf from a data frame

  inDF <- data.frame(xInt = as.integer(1:9),
    xNum1 = c(1.1111, 2.2222, 3.3333, 4.4444, 5.5555, 6.6666, 7.7777,8.8888, 9.9999),
        xNum2 = c(1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0))

    inXDF <- tempfile(pattern = ".rxPrecision", fileext = ".xdf")
    rxDataStep(inData = inDF, outFile = inXDF)
    varInfoXDF <- rxGetVarInfo(inXDF)

    outText <- tempfile(pattern = ".rxPrecision", fileext = ".csv")

    outDS <- RxTextData(outText, defaultDecimalColType = "numeric")
    rxDataStep(inXDF, outFile = outDS, overwrite = TRUE)
    outDF1 <- rxImport(outDS)
    checkEquals(inDF, outDF1)

  strippedDF <- data.frame(xInt = as.integer(1:9),
    xNum1 = c(1.11, 2.22, 3.33, 4.44, 5.56, 6.67, 7.78,8.89, 10.0),
        xNum2 = as.integer(c(1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0)))

  outDS2 <- RxTextData(outText, defaultDecimalColType = "numeric",
        writePrecision = 2, stripZeros = TRUE)
    rxDataStep(inXDF, outFile = outDS2, overwrite = TRUE)
    outDF2 <- rxImport(outDS2)
    checkEquals(strippedDF, outDF2)

  file.remove(inXDF)
  file.remove(outText)
}

"test.RxTextData.args.file.directory" <- function()
{
    # Setup directory/file names and data sources
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDS <- RxTextData(claimsTxt, stringsAsFactors = TRUE)
    claimsTxtDir <- file.path(rxGetOption("sampleDataDir"), "claimsTxt")
    claimsTxtDirDS <- RxTextData(claimsTxtDir, stringsAsFactors = TRUE)

    ### Test import of a directory of files ############################
    # Import into a data frame
    claimsDF1 <- rxImport( claimsTxtDS )
    claimsDF2 <- rxImport( claimsTxtDirDS)

    checkEquals(claimsDF1, claimsDF2)

    outXdf1 <- file.path(tempdir(), ".rxOutFileDirTest1.xdf")
    outXdf2 <- file.path(tempdir(), ".rxOutFileDirTest2.xdf")
  if (file.exists(outXdf1)) file.remove(outXdf1)
  if (file.exists(outXdf2)) file.remove(outXdf2)

    # Import into an xdf file
    claimsDS1 <- rxImport(claimsTxtDS, outFile = outXdf1)
    claimsDS2 <- rxImport(claimsTxtDirDS, outFile = outXdf2)

    varInfo1 <- rxGetVarInfo(claimsDS1)
    varInfo2 <- rxGetVarInfo(claimsDS2)

    checkEquals(varInfo1, varInfo2)
    file.remove(outXdf2)

  # Import to composite xdf
  outXdf3 <- file.path(tempdir(), "rxOutDir3Test")
  if (file.exists(outXdf3)) unlink(outXdf3, recursive = TRUE)

  outXdf3DS <- RxXdfData(outXdf3, createCompositeSet = TRUE, blocksPerCompositeFile = 2)
  claimsDS3 <- rxImport(claimsTxtDirDS, outFile = outXdf3DS)
  varInfo3 <- rxGetVarInfo(claimsDS3)

  checkEquals(varInfo1, varInfo3)
  fileInfo3 <- rxGetInfo(claimsDS3)
  checkEquals(fileInfo3$numCompositeFiles, 4)

    file.remove(outXdf1)
  unlink(outXdf3, recursive = TRUE)
    #### rxLocateFile #########

    fullDir <- rxLocateFile(file = "claimsTxt", pathsToSearch = rxGetOption("sampleDataDir"),
        defaultExt = "")
    checkEquals(normalizePath(fullDir), normalizePath(claimsTxtDir))

    ### Test analysis of a directory of files ########################
    # NOT YET IMPLEMENTED: Currently only processes first file

    rxLinMod1 <- rxLinMod(formula = cost~age, data = claimsTxtDS)
    rxLinMod2 <- rxLinMod(formula = cost~age, data = claimsTxtDirDS)

}

"test.RxTextData.data.extra.percent" <- function()
{
    outCsv <- file.path(tempdir(), ".rxOutFile1.csv")
  on.exit(file.remove(outCsv), add = TRUE)

  myData <- data.frame(
    x = c(
  "-7,7.083333,\"Monday\",1",
  "-9,16.00001,\"Monday\"%n,2",
  "-3,11.166667,\"Monday\",3"), stringsAsFactors = FALSE)
  write.table(myData, file = outCsv, row.names = FALSE, col.names = FALSE, quote = FALSE, sep = ",")
  testDataSource <- RxTextData(outCsv, defaultDecimalColType = "numeric");
  testDF <- rxImport(testDataSource)
  testOrigDF <- read.csv(outCsv, header = FALSE)

  checkIdentical(testDF$V1, testOrigDF$V1)
  checkIdentical(testDF$V4, testOrigDF$V4)
}

"test.RxTextData.args.colInfo.factorIndex" <- function()
{
  txtFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  textDS1 <- RxTextData(file = txtFile, rowsPerRead = 40,
        varsToKeep = c("xfac1", "xint1"),
        colInfo =
    list(
            xint1 = list(type = "factorIndex", minLevel = 1, levels = as.character(1:14)),
            xfac1 = list(type = "factorIndex", maxLevel = 14, levels = as.character(0:14))
            ))
    outDataFrame <- rxImport(textDS1)
    checkEquals(levels(outDataFrame$xfac1), as.character(0:14))
    checkEquals(levels(outDataFrame$xint1), c("NA", as.character(1:14)))

    # Make sure variables are treated as factors in analysis
    sumOut <- rxSummary(~xint1, data = textDS1)
    sumOutDF <- rxSummary(~xint1, data = outDataFrame)
    checkEquals(sumOut$categorical$Counts, sumOutDF$categorical$Counts)

    # When being written out, they should be written as integers
    outFileName <- tempfile( pattern = ".rxImportX9", fileext = ".csv")
    textDS2 <- RxTextData(file = outFileName)
    rxDataStep(inData = textDS1, outFile = textDS2, overwrite = TRUE)
    outDataFrame2 <- rxImport(textDS2)

  textDS3 <- RxTextData(file = txtFile, rowsPerRead = 40)
    outDataFrame3 <- rxImport(textDS3, varsToKeep = "xint1")
    checkEquals(outDataFrame2$xint1, outDataFrame3$xint1)
}


