# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxXdfData
#
############################################################################################
"test.RxXdfData.rxIsOpen.read.mode" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    ds <- RxXdfData(fileName)
    checkTrue(!rxIsOpen(ds))
    rxOpen(ds)
    checkTrue(rxIsOpen(ds))
    rxClose(ds)
    checkTrue(!rxIsOpen(ds))
}

"test.RxXdfData.rxIsOpen.write.mode" <- function()
{
    fileName <- paste0(tempfile(), ".xdf")
    ds <- RxXdfData(fileName)
    checkTrue(!rxIsOpen(ds, "w"))
    rxOpen(ds, "w")
    checkTrue(rxIsOpen(ds, "w"))
    rxClose(ds, "w")
    checkTrue(!rxIsOpen(ds, "w"))
}


"test.RxXdfData.rxReadNext" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    varsToKeep <- c("age", "incwage")

    # rxReadNext approach to finding column means
    processData <- function(dframe)
      list(sumCols = colSums(dframe), numRows = nrow(dframe))
    updateResults <- function(x, y)
      list(sumCols = x$sumCols + y$sumCols, numRows = x$numRows + y$numRows)
    ds <- RxXdfData(fileName, varsToKeep = varsToKeep, blocksPerRead = 2)
    rxOpen(ds)
    resList <- processData(rxReadNext(ds))
    while(TRUE)
    {
        df <- rxReadNext(ds)
        if (nrow(df) == 0)
            break
        resList <- updateResults(resList, processData(df))
    }
    rxClose(ds)
    varMeans <- resList$sumCols / resList$numRows

    # colMeans approach to finding column means
    testData <-
      rxReadXdf(file = fileName, varsToKeep = varsToKeep, reportProgress = 0)
    testMeans <- colMeans(testData)

    checkEquals(varMeans[1], testMeans[1])
    checkEquals(varMeans[2], testMeans[2])
}

"test.RxXdfData.rxCrossTabs" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <- rxCrossTabs(~ sex:state, data = fileName, reportProgress = 0)
    usingDataSource <-
      rxCrossTabs(~ sex:state, data = RxXdfData(fileName), reportProgress = 0)
  usingStringList <- usingString[!(names(usingString) %in% "call")]
  usingStringList$params$fileSystem <- NULL
    usingStringList$params$DataSource <- NULL
    usingStringList$params$File <- NULL
  usingDataSourceList <- usingDataSource[!(names(usingDataSource) %in% "call")]
  usingDataSourceList$params$fileSystem <- NULL
    usingDataSourceList$params$DataSource <- NULL
    usingDataSourceList$params$File <- NULL
    checkEquals(usingStringList,
                usingDataSourceList, checkAttributes = FALSE)
}

"test.RxXdfData.rxCube" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <-
      attributes(rxCube(~ sex:state, data = fileName, reportProgress = 0))
    usingDataSource <-
      attributes(rxCube(~ sex:state, data = RxXdfData(fileName),
                        reportProgress = 0))
  usingStringList <- usingString[!(names(usingString) %in% "call")]
  usingStringList$params$fileSystem <- NULL
    usingStringList$params$DataSource <- NULL
     usingStringList$params$File <- NULL
  usingDataSourceList <- usingDataSource[!(names(usingDataSource) %in% "call")]
  usingDataSourceList$params$fileSystem <- NULL
    usingDataSourceList$params$DataSource <- NULL
    usingDataSourceList$params$File <- NULL
    checkEquals(usingStringList,
                usingDataSourceList)					
}

"test.RxXdfData.rxDataStep.data.frame" <- function()
{
    myDF <- data.frame(a = 1:10, b = 11:20)
    fileName <- tempfile( pattern = ".rxXDF01Text", fileext = ".xdf")
    rxDataStep(myDF, RxXdfData(fileName))
    checkEquals(rxReadXdf(fileName, reportProgress = 0), myDF)
    file.remove(fileName)
}

"test.RxXdfData.rxDataStep" <- function()
{
    inFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    outFile <-  fileName <- tempfile( pattern = ".rxXDF02Text", fileext = ".xdf")
    rxDataStep(RxXdfData(inFile, varsToKeep = c("sex","state")),
                  RxXdfData(outFile), reportProgress = 0)
    checkEquals(rxReadXdf(outFile, reportProgress = 0),
                rxReadXdf(inFile, varsToKeep = c("sex","state"),
                          reportProgress = 0))
    file.remove(outFile)
}

"test.RxXdfData.rxGetInfo" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    checkEquals(rxGetInfo(fileName, varsToKeep = c("sex","state")),
                rxGetInfo(RxXdfData(fileName, varsToKeep = c("sex","state"))))
}

"test.RxXdfData.rxGetVarInfo" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    checkEquals(rxGetVarInfo(fileName, varsToKeep = c("sex","state")),
                rxGetVarInfo(RxXdfData(fileName, varsToKeep = c("sex","state"))))
}

"test.RxXdfData.rxGetVarNames" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    checkEquals(rxGetVarNames(fileName), rxGetVarNames(RxXdfData(fileName)))
}

"test.RxXdfData.rxLinMod" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <-
      rxLinMod(wkswork1 ~ sex, data = fileName, reportProgress = 0)
    usingDataSource <-
      rxLinMod(wkswork1 ~ sex, data = RxXdfData(fileName),
               reportProgress = 0)
  usingStringList <- usingString[!(names(usingString) %in% "call")]
  usingStringList$params$fileSystem <- NULL
    usingStringList$params$DataSource <- NULL
    usingStringList$params$File <- NULL
  usingDataSourceList <- usingDataSource[!(names(usingDataSource) %in% "call")]
  usingDataSourceList$params$fileSystem <- NULL
    usingDataSourceList$params$DataSource <- NULL
    usingDataSourceList$params$File <- NULL
    checkEquals(usingStringList,
                usingDataSourceList)
}

"test.RxXdfData.rxLogit" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <- rxLinMod(N(sex) ~ state, data = fileName, reportProgress = 0)
    usingDataSource <-
      rxLinMod(N(sex) ~ state, data = RxXdfData(fileName), reportProgress = 0)
  usingStringList <- usingString[!(names(usingString) %in% "call")]
  usingStringList$params$fileSystem <- NULL
    usingStringList$params$File <- NULL
    usingStringList$params$DataSource <- NULL
  usingDataSourceList <- usingDataSource[!(names(usingDataSource) %in% "call")]
  usingDataSourceList$params$fileSystem <- NULL
    usingDataSourceList$params$File <- NULL
    usingDataSourceList$params$DataSource <- NULL
    checkEquals(usingStringList,
                usingDataSourceList)
}

"test.RxXdfData.rxPredict" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    predictFile <- tempfile( pattern = ".rxXDF03", fileext = ".xdf")
    rxDataStep(fileName, predictFile, reportProgress = 0)
    model <- rxLinMod(wkswork1 ~ sex, data = fileName, reportProgress = 0)
    rxPredict(model, RxXdfData(predictFile),  reportProgress = 0)
    info1 <- rxGetInfo(fileName)
    info2 <- rxGetInfo(predictFile)
    checkEquals( info1$numVars+1, info2$numVars)
    file.remove(predictFile)
}

"test.RxXdfData.rxReadXdf" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <-
      rxReadXdf(fileName, varsToKeep = c("sex", "state"), reportProgress = 0)
    usingDataSource <-
      rxReadXdf(RxXdfData(fileName, varsToKeep = c("sex", "state")),
                reportProgress = 0)
    checkEquals(usingString, usingDataSource)
}

"test.RxXdfData.rxSetVarInfo" <- function()
{
    myDF <- data.frame(a = 1:10, b = 11:20)
    fileName <- tempfile( pattern = ".rxXDF04", fileext = ".xdf")
    rxDataStep(myDF, fileName)
    aColDescript <- "DESCRIPTION OF A"
    rxSetVarInfo(list(a = list(description = aColDescript)),
                    RxXdfData(fileName))
    checkEquals(rxGetVarInfo(fileName)[["a"]][["description"]], aColDescript)
    file.remove(fileName)
}

"test.RxXdfData.rxSummary" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <-
      rxSummary(wkswork1 ~ sex, data = fileName, reportProgress = 0)
    usingDataSource <-
      rxSummary(wkswork1 ~ sex, data = RxXdfData(fileName), reportProgress = 0)
  usingStringList <- usingString[!(names(usingString) %in% "call")]
  usingStringList$params$fileSystem <- NULL
    usingStringList$params$File <- NULL
    usingStringList$params$DataSource <- NULL
  usingDataSourceList <- usingDataSource[!(names(usingDataSource) %in% "call")]
  usingDataSourceList$params$fileSystem <- NULL
    usingDataSourceList$params$File <- NULL
    usingDataSourceList$params$DataSource <- NULL
    checkEquals(usingStringList,
                usingDataSourceList)
}

"test.RxXdfData.rxTextToXdf" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    stringOutFile <- tempfile( pattern = ".rxXDF05", fileext = ".xdf")
    dataSourceOutFile <- tempfile( pattern = ".rxXDF06", fileext = ".xdf")
    usingString <- rxTextToXdf(fileName, stringOutFile, reportProgress = 0)
    usingDataSource <-
      rxTextToXdf(fileName, RxXdfData(dataSourceOutFile), reportProgress = 0)
    checkEquals(rxReadXdf(stringOutFile, reportProgress = 0),
                rxReadXdf(dataSourceOutFile, reportProgress = 0))
    file.remove(stringOutFile)
    file.remove(dataSourceOutFile)
}

"test.RxXdfData.rxXdfToDataFrame.deprecated" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    usingString <-
      rxDataStep(fileName, varsToKeep = c("sex", "state"),
                       reportProgress = 0)
    usingDataSource <-
      rxDataStep(RxXdfData(fileName, varsToKeep = c("sex", "state")),
                       reportProgress = 0)
    checkEquals(usingString, usingDataSource)
}

"test.RxXdfData.rxXdfToText.replace" <- function()
{
    DEACTIVATED("Pending investigation on build system")
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    stringOutFile <- tempfile( pattern = ".rxXDF07", fileext = ".xdf")
    dataSourceOutFile <- tempfile( pattern = ".rxXDF08", fileext = ".xdf")
    usingString <-
      rxDataStep(fileName, varsToKeep = c("sex", "state"), stringOutFile,
                  reportProgress = 0)
    usingDataSource <-
      rxDataStep(RxXdfData(fileName, varsToKeep = c("sex", "state")),
                  dataSourceOutFile, reportProgress = 0)
    checkEquals(read.table(stringOutFile), read.table(dataSourceOutFile))
    file.remove(stringOutFile)
    file.remove(dataSourceOutFile)
}

#####
# S3/S4 Methods

"test.RxXdfData.method.head" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  numRows <- 6L
  z1 <- head(DS, n = numRows)
  z2 <- rxReadXdf(DS, numRows = numRows)
  checkEquals(z1, z2)
  checkEquals(nrow(z1), numRows)
}

"test.RxXdfData.method.tail" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  numRows <- 6L
  z1 <- tail(DS, n = numRows)
  numRowsTotal <- rxGetInfo(DS)[["numRows"]]
  z2 <- tail(rxReadXdf(DS), n = numRows)
  checkEquals(z1, z2)
  checkEquals(nrow(z1), numRows)
  z3 <- tail(DS, n = numRows, addrownums = FALSE)
  checkEquals(row.names(z3), as.character(1:numRows))
}

"test.RxXdfData.method.names" <- function()
{
    inFile <-file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf")
  DS1 <- RxXdfData(file = inFile)
    varNames <- rxGetVarNames(DS1)
  checkEquals(names(DS1), varNames)
    checkEquals(names(DS1), names(rxGetVarInfo(DS1)))
    
    varsToDrop <- c("height", "male")
    DS2 <- RxXdfData(file = inFile, varsToDrop = varsToDrop)
    subVarNames <- varNames[!(varNames %in% varsToDrop)]
    checkEquals(names(DS2), subVarNames)
    checkEquals(rxGetVarNames(DS2), subVarNames)
    
}

"test.RxXdfData.method.names.replacement" <- function()
{
  XDF <- file.path(tempdir(), "iris.xdf")
  rxDataStep(iris, XDF, overwrite = TRUE)
  irisDS <- RxXdfData(XDF)
  newNames <- c("cow","horse","chicken","goat","squirrel")
  names(irisDS) <- newNames
  checkEquals(names(irisDS), newNames)
  if (file.exists(XDF)) file.remove(XDF)
}

"test.RxXdfData.method.dim" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  checkEquals(dim(DS), as.vector(unlist(rxGetInfo(DS)[c("numRows","numVars")])))
}

"test.RxXdfData.method.dimnames" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  checkEquals(dimnames(DS), list(NULL, rxGetVarNames(DS)))
}

"test.RxXdfData.method.formula" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  nms <- names(DS)
  
  form <- formula(paste("~", paste(nms, collapse = "+")))
  checkEquals(formula(DS),form)
  
  form <- formula(paste("male ~", paste(nms[-grep("male", nms)], collapse = "+")))
  checkEquals(formula(DS, depVars = "male"), form)
  
  form <- formula(paste("male ~", paste(nms[-grep(c("male|reps"), nms)], collapse = "+")))
  checkEquals(formula(DS, depVars = "male", varsToDrop = "reps"), form)

  form <- ~ eyecolor
  checkEquals(formula(DS, varsToKeep = "eyecolor"), form)

  form <- ~ height + eyecolor:male
  checkEquals(formula(DS, varsToKeep = "height", interactions = list(c("eyecolor", "male"))), form)

  form <- ~ height + eyecolor:male + height:reps
  checkEquals(formula(DS, varsToKeep = "height", interactions = list(c("eyecolor", "male"), c("height","reps"))), form)
  
  form <- ~ height + eyecolor:male:height:reps
  checkEquals(formula(DS, varsToKeep = "height", interactions = list(c("eyecolor", "male","height","reps"))), form)

  form <- ~ eyecolor:male
  checkEquals(formula(DS, varsToDrop = nms, interactions = list(c("eyecolor", "male"))), form)
  
  checkException(formula(DS, varsToDrop = nms), silent = TRUE)
}

"test.RxXdfData.method.nrow" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  nr <- rxGetInfo(DS)[["numRows"]]
  checkEquals(nrow(DS), nr)
}

"test.RxXdfData.method.ncol" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  checkEquals(ncol(DS), rxGetInfo(DS)[["numVars"]])
}

"test.RxXdfData.method.length" <- function()
{
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  checkEquals(length(DS), rxGetInfo(DS)[["numVars"]])
}

# No longer have special str method for RxXdfData
#"test.RxXdfData.method.str" <- function()
#{
  #DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))
  #output <- capture.output(str(DS))
  #checkTrue(length(grep("'RxXdfData'", output[1L], fixed = TRUE)) > 0L)
  #checkTrue(length(grep("fourthgraders.xdf", output[5L], fixed = TRUE)) > 0L)  
#}

"test.RxXdfData.method.summary" <- function()
{
  # check a sampling of the output
  DS <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "fourthgraders.xdf"))	
  output <- capture.output(summary(DS))
  checkTrue(length(grep("Summary Statistics Results for: ~height + eyecolor + male + reps", output, fixed = TRUE)) > 0L)
  checkTrue(length(grep("Category Counts for eyecolor", output, fixed = TRUE)) > 0L)
  checkTrue(length(grep("Number of categories: 4", output, fixed = TRUE)) > 0L)
  checkTrue(length(grep("Number of valid observations: 44", output, fixed = TRUE)) > 0L)
  checkTrue(length(grep("Number of missing observations: 0", output, fixed = TRUE)) > 0L)
}


"test.RxXdfData.args.createCompositeSet" <- function()
{
    myData <- data.frame(x = 1:100, y = 1:100, z = 1:100)
    outFileCompositeDir <- tempfile(pattern = "rxSmallComposite", fileext = "")
    outFileSingle <- tempfile(pattern = "rxSingle", fileext = ".xdf")
    csvFile <- tempfile(pattern = "rxCompositeCsv", fileext = ".csv")
    rxDataStep(inData = myData, outFile = csvFile)

    importIn <- RxTextData(file = csvFile, rowsPerRead = 10)
    compositeXdfDS <- RxXdfData(file = outFileCompositeDir, createCompositeSet = TRUE)
    singleXdfDS <- RxXdfData(file = outFileSingle)
    rxImport(importIn, singleXdfDS, overwrite = TRUE)
    rxImport(importIn, compositeXdfDS, overwrite = TRUE)
    file.remove(csvFile)
    
    # Test rxGetInfo
    fileInfoSingle <- rxGetInfo(singleXdfDS)
    fileInfoComposite <- rxGetInfo(compositeXdfDS)
    checkEquals(fileInfoSingle$numRows, fileInfoComposite$numRows)
    checkEquals(fileInfoSingle$numVars, fileInfoComposite$numVars)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfoComposite$numCompositeFiles, 1L)
    } else
    {
        checkEquals(fileInfoComposite$numCompositeFiles, 4L)
    }   

    # Test rxGetVarInfo
    varInfoComposite <- rxGetVarInfo(compositeXdfDS)
    varInfoSingle <- rxGetVarInfo(singleXdfDS)
    checkIdentical(varInfoComposite, varInfoSingle) 
    
    # Test rxReadXdf
    myReadDataComposite <- rxReadXdf(compositeXdfDS, numRows = 83, varsToKeep = c("x", "z"))
    myReadDataSingle <- rxReadXdf(singleXdfDS, numRows = 83, varsToKeep = c("x", "z"))
    checkIdentical(myReadDataComposite, myReadDataSingle)
    
    # Test rxDataStep
    
    # Clean up
    if (file.exists(outFileCompositeDir)) unlink(outFileCompositeDir, recursive = TRUE) 
    if (file.exists(outFileSingle)) unlink(outFileSingle, recursive = TRUE) 
}

"test.RxXdfData.args.createCompositeSet.data.frame" <- function()
{
    #library(RevoScaleR)
    dataColInfo <- list(
        RowNum =list(type = "integer"),
        age = list( type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")))

    inData <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    inDataDS <- RxTextData(inData, colInfo = dataColInfo)

    ## output data.frame
    outDfF <- rxImport(inDataDS, createCompositeSet = FALSE)    #OK
    rxGetInfo(outDfF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outDfT <- rxImport(inDataDS, createCompositeSet = TRUE)     #crash
    rxGetInfo(outDfT, getVarInfo = TRUE, getBlockSizes = TRUE)

    checkEquals(outDfF, outDfT)
}

"test.RxXdfData.args.createCompositeSet.outFile" <- function()
{
    #library(RevoScaleR)
    dataColInfo <- list(
        RowNum =list(type = "integer"),
        age = list( type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")))

    inData <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    inDataDS <- RxTextData(inData, colInfo = dataColInfo)

    ###########################################################################
    ## file name contains neither "." nor "_"
    outFileStr <- tempfile(pattern = "fileOutFile", tmpdir = tempdir(), fileext = "")

    outImpStrT <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = TRUE) #composite
    rxGetInfo(outImpStrT, getVarInfo = TRUE, getBlockSizes = TRUE)

    outImpStrF <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = FALSE) #non-composite overwrites existing composite!
    rxGetInfo(outImpStrF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outStepStr <- rxDataStep(inDataDS, outFile = outFileStr, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepStr, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = TRUE
    outFileDsT <- RxXdfData(outFileStr, createCompositeSet = TRUE)

    outImpDsTT <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = TRUE) #composite
    rxGetInfo(outImpDsTT, getVarInfo = TRUE, getBlockSizes = TRUE)

    outImpDsTF <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = FALSE) #composite
    rxGetInfo(outImpDsTF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outStepDsT <- rxDataStep(inDataDS, outFile = outFileDsT, overwrite = TRUE) #composite
    rxGetInfo(outStepDsT, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = FALSE
    outFileDsF <- RxXdfData(outFileStr, createCompositeSet = FALSE)

    outImpDsFF <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = FALSE) #non-composite
    rxGetInfo(outImpDsFF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outImpDsFT <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = TRUE) #composite
    rxGetInfo(outImpDsFT, getVarInfo = TRUE, getBlockSizes = TRUE)

    outStepDsF <- rxDataStep(inDataDS, outFile = outFileDsF, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepDsF, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## rxTextToXdf seems to ignore createCompositeSet and returns NULL

    rxTextToXdf(inData, outFileStr, overwrite = TRUE, createCompositeSet = TRUE) #non-composite!
    rxGetInfo(outFileStr, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    rxTextToXdf(inData, outFileDsT, overwrite = TRUE, createCompositeSet = TRUE) #non-composite!
    rxGetInfo(outFileDsT, getVarInfo = TRUE, getBlockSizes = TRUE)

    rxTextToXdf(inData, outFileDsF, overwrite = TRUE, createCompositeSet = TRUE) #non-composite
    rxGetInfo(outFileDsF, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    checkTrue(TRUE)
}

"test.RxXdfData.args.createCompositeSet.outFile.dot" <- function()
{
    #library(RevoScaleR)

    dataColInfo <- list(
        RowNum =list(type = "integer"),
        age = list( type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")))

    inData <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    inDataDS <- RxTextData(inData, colInfo = dataColInfo)

    ###########################################################################
    ## file name contains "."
    outFileStr <- tempfile(pattern = "fileOutFileDot", tmpdir = tempdir(), fileext = ".xdf")

    checkException(outImpStrT <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpStrT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    outImpStrF <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = FALSE) #non-composite overwrites existing composite!
    rxGetInfo(outImpStrF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outStepStr <- rxDataStep(inDataDS, outFile = outFileStr, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepStr, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- tools::file_path_sans_ext(outFileStr)
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = TRUE
    outFileDsT <- RxXdfData(outFileStr, createCompositeSet = TRUE)

    checkException(outImpDsTT <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsTT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    checkException(outImpDsTF <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = FALSE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsTF, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    checkException(outStepDsT <- rxDataStep(inDataDS, outFile = outFileDsT, overwrite = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outStepDsT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- tools::file_path_sans_ext(outFileStr)
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = FALSE
    outFileDsF <- RxXdfData(outFileStr, createCompositeSet = FALSE)

    outImpDsFF <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = FALSE) #non-composite
    rxGetInfo(outImpDsFF, getVarInfo = TRUE, getBlockSizes = TRUE)

    checkException(outImpDsFT <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsFT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    outStepDsF <- rxDataStep(inDataDS, outFile = outFileDsF, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepDsF, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- tools::file_path_sans_ext(outFileStr)
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    checkTrue(TRUE)
}

"test.RxXdfData.args.createCompositeSet.outFile.underscore" <- function()
{
    #library(RevoScaleR)

    dataColInfo <- list(
        RowNum =list(type = "integer"),
        age = list( type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")))

    inData <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    inDataDS <- RxTextData(inData, colInfo = dataColInfo)

    ###########################################################################
    ## file name contains "_"
    outFileStr <- tempfile(pattern = "fileOutFileUnderscore", tmpdir = tempdir(), fileext = "_xdf")
    
    checkException(outImpStrT <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpStrT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    outImpStrF <- rxImport(inDataDS, outFile = outFileStr, overwrite = TRUE, createCompositeSet = FALSE) #non-composite overwrites existing composite!
    rxGetInfo(outImpStrF, getVarInfo = TRUE, getBlockSizes = TRUE)

    outStepStr <- rxDataStep(inDataDS, outFile = outFileStr, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepStr, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = TRUE
    outFileDsT <- RxXdfData(outFileStr, createCompositeSet = TRUE)

    checkException(outImpDsTT <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsTT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    checkException(outImpDsTF <- rxImport(inDataDS, outFile = outFileDsT, overwrite = TRUE, createCompositeSet = FALSE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsTF, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    checkException(outStepDsT <- rxDataStep(inDataDS, outFile = outFileDsT, overwrite = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outStepDsT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    ## data source with createCompositeSet = FALSE
    outFileDsF <- RxXdfData(outFileStr, createCompositeSet = FALSE)

    outImpDsFF <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = FALSE) #non-composite
    rxGetInfo(outImpDsFF, getVarInfo = TRUE, getBlockSizes = TRUE)

    checkException(outImpDsFT <- rxImport(inDataDS, outFile = outFileDsF, overwrite = TRUE, createCompositeSet = TRUE), silent = TRUE) #composite
    checkException(rxGetInfo(outImpDsFT, getVarInfo = TRUE, getBlockSizes = TRUE), silent = TRUE)

    outStepDsF <- rxDataStep(inDataDS, outFile = outFileDsF, overwrite = TRUE) #non-composite overwrites existing composite!
    rxGetInfo(outStepDsF, getVarInfo = TRUE, getBlockSizes = TRUE)

    if (file.exists(outFileStr)) unlink(outFileStr, recursive = TRUE)
    outFileStrExt <- paste(outFileStr, "xdf", sep = ".")
    if (file.exists(outFileStrExt)) unlink(outFileStrExt, recursive = TRUE)

    ###########################################################################
    checkTrue(TRUE)
}

"test.RxXdfData.args.blocksPerCompositeFile" <- function()
{
  inDataFile <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    xdfFile <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    outFileDir <- tempfile(pattern = "rxComposite", fileext = "")
    rxImport(inData = inDataFile, outFile = outFileDir, rowsPerRead = 20, 
    createCompositeSet = TRUE, overwrite = TRUE)
    fileInfo1 <- rxGetInfo(outFileDir)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo1$numCompositeFiles, 1)
        checkEquals(fileInfo1$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo1$numCompositeFiles, 3)
        checkEquals(fileInfo1$numBlocks, 7)
     }
        
    
    rxImport(inData = inDataFile, outFile = RxXdfData(outFileDir, createCompositeSet = TRUE),
        rowsPerRead = 20, overwrite = TRUE)
    fileInfo2 <- rxGetInfo(outFileDir)
    
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo2$numCompositeFiles, 1)
        checkEquals(fileInfo2$numBlocks, 1)
    } else
    {    
        checkEquals(fileInfo2$numCompositeFiles, 3)
        checkEquals(fileInfo2$numBlocks, 7) 
    }
       
    rxImport(inData = inDataFile, outFile = 
        RxXdfData(outFileDir, createCompositeSet = TRUE, blocksPerCompositeFile = 2),
        rowsPerRead = 20,  overwrite = TRUE)
    fileInfo3 = rxGetInfo(outFileDir, getBlockSizes = TRUE)
    
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo3$numCompositeFiles, 1)
        checkEquals(fileInfo3$numBlocks, 1)
        checkEquals(fileInfo3$rowsPerBlock, 128)
    } else
    {
        checkEquals(fileInfo3$numCompositeFiles, 4)
        checkEquals(fileInfo3$numBlocks, 7)
        checkEquals(fileInfo3$rowsPerBlock, c(20,20))
    }
    
    
    rxImport(inData = inDataFile, outFile = 
        RxXdfData(outFileDir, createCompositeSet = TRUE),
        rowsPerRead = 20, blocksPerCompositeFile = 2, overwrite = TRUE)
    fileInfo4 = rxGetInfo(outFileDir, getBlockSizes = TRUE) 
    
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo4$numCompositeFiles, 1)
        checkEquals(fileInfo4$numBlocks, 1)
        checkEquals(fileInfo4$rowsPerBlock, 128)
    } else
    {    
        checkEquals(fileInfo4$numCompositeFiles, 4)
        checkEquals(fileInfo4$numBlocks, 7)
        checkEquals(fileInfo4$rowsPerBlock, c(20,20))  
    }

    myData <- rxReadXdf(outFileDir)  
    rxSummary(~cost, outFileDir)  
  myData1 <- rxReadXdf(xdfFile)
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  myData2 <- rxReadXdf(test100r5b)
    
    if (file.exists(outFileDir)) unlink(outFileDir, recursive = TRUE)
}
