# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for prune.rxDTree
#
############################################################################################

.setUp <- function()
{
    require(rpart)
}

.tearDown <- function()
{
}

test.prune.rxDTree.dispatch <- function()
{
  require(rpart)
  kyph.rxDTree <- rxDTree(Kyphosis ~ Age + Start + Number, data=kyphosis)
  kyph.rpart <- rxAddInheritance(kyph.rxDTree)
  kyph.rpart.prune <- prune(kyph.rpart, cp=0.1)
  kyph.rxDTree.prune <- prune(kyph.rxDTree, cp=0.1)
  checkEquals(kyph.rpart.prune$frame, kyph.rxDTree.prune$frame)
  checkEquals(kyph.rpart.prune$cptable, kyph.rxDTree.prune$cptable)
  checkEquals(kyph.rpart.prune$splits, kyph.rxDTree.prune$splits)
  checkEquals(kyph.rpart.prune$where, kyph.rxDTree.prune$where)
}

test.prune.rxDTree.extrema <- function()
{
  require(rpart)
  kyph.rxDTree <- rxDTree(Kyphosis ~ Age + Start + Number, data=kyphosis)
  kyph.rxDTree.prune <- prune(kyph.rxDTree, cp=0)
  checkEquals(kyph.rxDTree.prune, kyph.rxDTree)
  kyph.rxDTree.prune <- prune(kyph.rxDTree, cp=1)
  checkEquals(length(kyph.rxDTree.prune$splits), 0)
  checkEquals(nrow(kyph.rxDTree.prune$frame), 1)
  checkEquals(nrow(kyph.rxDTree.prune$cptable),1)
  checkEquals(kyph.rxDTree.prune$where, rep(1,81))
}

test.prune.rxDTree.examples <- function()
{
  require(rpart)
  z.auto <- rpart(Mileage ~ Weight, car.test.frame)
  zp <- prune(z.auto, cp=0.1)
  zprx <- prune.rxDTree(z.auto, cp=0.1)
  checkEquals(zp$frame, zprx$frame)
  checkEquals(zp$cptable, zprx$cptable)
  checkEquals(zp$splits, zp$splits)
  checkEquals(zp$where, zp$where)
}

test.prune.rxDTree.airlineDemo <- function()
{
  require(rpart)
  airlineDemo <- file.path(system.file("SampleData", package="RevoScaleR"), "AirlineDemoSmall.xdf")
  airlineTree <- rxDTree(ArrDelay ~ CRSDepTime + DayOfWeek, data=airlineDemo, cp=1e-5)
  airlineRpart <- rxAddInheritance(airlineTree)
  airlineTreePruned <- prune(airlineTree, cp=.01)
  airlineRpartPruned <- prune(airlineRpart, cp=.01)
  checkEquals(airlineTreePruned$frame, airlineRpartPruned$frame)
  checkTrue(is.null(airlineTreePruned$csplit))
  checkEquals(airlineTreePruned$cptable, airlineRpartPruned$cptable)
  checkEquals(airlineTreePruned$splits, airlineRpartPruned$splits)
  airlineTreePruned <- prune(airlineTree, cp=1e-4)
  airlineRpartPruned <- prune(airlineRpart, cp=1e-4)
  checkEquals(airlineTreePruned$frame, airlineRpartPruned$frame)
  checkEquals(airlineTreePruned$csplit, airlineRpartPruned$csplit)
  checkEquals(airlineTreePruned$cptable, airlineRpartPruned$cptable)
  checkEquals(airlineTreePruned$splits, airlineRpartPruned$splits)
}