# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxClusterHadoop.R and logics of spark backend entry point
#
############################################################################################

"test.initAndCheckRxRemoteSparkHpaCallInput" <- function()
{
  if (.Platform$OS.type != "unix") {
    DEACTIVATED("ONLY FOR UNIX.")
  }
  mockCall <- function(func, args) {
    checkEquals(func, "RxUtil")
    checkEquals(args$USE, "InitializeLogging")
    checkEquals(args$traceLevel, 1)
  }
  info.list <- RevoScaleR:::.initAndCheckRxRemoteSparkHpaCallInput(list(rxNetworkShareJobDirectory = "/tmp/test", rxTraceLevel = 1, rxJobRData = "fileName", rxFroRDataPath = "/path", rxHPAJobDir = "/hpa"), mockCall)
  #.initAndCheckRxRemoteSparkHpaCallInput should be successfully called once
  checkEquals(info.list$guidDirOnHeadNode, "/tmp/test")
  checkEquals(info.list$inFileName, "fileName")
  checkEquals(info.list$froDataPath, "/path")
  checkEquals(info.list$hpaJobDir, "/hpa")
}

"test.initAndCheckRxRemoteSparkHpaCallInputData" <- function()
{
  if (.Platform$OS.type != "unix") {
    DEACTIVATED("ONLY FOR UNIX.")
  }
  root.dir <- tempdir()
  guidDirOnHeadNode <- file.path(root.dir,"test.initAndCheckRxRemoteSparkHpaCallInputData")
  dir.create(guidDirOnHeadNode)
  computeContext <- RxSpark()
  inputObject <- list(ARGS=list(computeContext=computeContext))
  inFileName <- "job.RInData"
  save(inputObject, file=file.path(guidDirOnHeadNode, inFileName))
  info.list <- RevoScaleR:::.initAndCheckRxRemoteSparkHpaCallInputData(guidDirOnHeadNode, inFileName)
  checkEquals(info.list$computeContext@appId, computeContext@appId)
}

"test.buildFifoPath" <- function()
{
  if (.Platform$OS.type != "unix") {
    DEACTIVATED("ONLY FOR UNIX.")
  }
  computeContext <- RxSpark()
  computeContext@wait <- FALSE
  fifo <- RevoScaleR:::.buildFifoPath(computeContext, "RevoFifoFromSpark")
  fifoExpect <- paste(
    computeContext@shareDir,
    paste(
      "RevoFifoFromSpark",
      computeContext@user,
      computeContext@rSessionPid,
      computeContext@appId,
      computeContext@jobId,
      sep="-"),
    sep="/")
  checkEquals(fifo, fifoExpect)
}

"test.executeRxRemoteSparkCallHpaJob" <- function()
{
  if (.Platform$OS.type != "unix") {
    DEACTIVATED("ONLY FOR UNIX.")
  }
  assign("call.count", 0, envir = .GlobalEnv)
  mockCall <- function(func, args) {
    assign("call.count", call.count + 1, envir = .GlobalEnv)
    checkTrue(func %in% list("RxUtil","test"))
    if (func == "RxUtil") {
      checkTrue(args$USE %in% list("InitializeHadoop","UninitializeHadoop"))
    }
    return("done")
  }
  trace(mockCall, tracer=function() .count <- .count + 1)
  # borrow .nativeCallFun(mockCall) in .executeRxRemoteSparkCallHpaJob to test do.call function. 
  inputObject <- list(FUN=".nativeCallFun", ARGS=list(func="test", args=list()))
  returnObject <- RevoScaleR:::.executeRxRemoteSparkCallHpaJob(inputObject, "inputObjectPath", "finalIroPath", "hdfsShareDirPath", "cmdToLaunchJob", mockCall)
  checkEquals(returnObject, "done")
  checkEquals(call.count, 3)
}
