# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxCompareContexts
#
############################################################################################

".rxuGetComputeContextDescription" <- function()
{
    if (exists("rxuGetComputeContextDescription", globalenv()))
        get("rxuGetComputeContextDescription", globalenv())
    else
        RevoScaleR:::rxuGetComputeContextDescription
}

".rxuCreatePlatformContext" <- function()
{
    if (exists("rxuCreatePlatformContext", globalenv()))
        get("rxuCreatePlatformContext", globalenv())
    else
        RevoScaleR:::rxuCreatePlatformContext
}


".rxNodeSetDiff" <- function()
{
    if (exists("rxNodeSetDiff", globalenv()))
        get("rxNodeSetDiff", globalenv())
    else
        RevoScaleR:::rxNodeSetDiff
}


rxuGetComputeContextDescription <- .rxuGetComputeContextDescription()
rxuCreatePlatformContext <- .rxuCreatePlatformContext()
rxNodeSetDiff <- .rxNodeSetDiff()

"singleSlotHelper" <- function( context, slotName, value1, value2, exactMatch )
{
  val1 <- list( newval = value1 )
  names( val1 ) <- slotName
  params <- list( platform = context@description, object = context )
  params <- c( params, val1 )
  context1 <- do.call( "rxuCreatePlatformContext", params )
  

  val2 <- list( newval = value2 )
  names( val2 ) <- slotName	
  params <- list( platform = context@description, object = context )
  params <- c( params, val2 )
  context2 <- do.call( "rxuCreatePlatformContext", params )
  
  retVal <- rxCompareContexts( context1, context2, exactMatch )
  retVal <- retVal && ( rxCompareContexts( context2, context1, exactMatch ) )
  
  return( retVal )
}


# helper to test all the fields that live in the RxDistributedHpa class
"rxDistributedHpaTestHelper" <- function( context )
{

  # FIELDS IN NON-EXACT MATCH (always checked)
        
  # shareDir
  checkTrue( singleSlotHelper( context, "shareDir", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "shareDir", "foo", "foo", TRUE ) )
  checkTrue( !singleSlotHelper( context, "shareDir", "foo", "bar", FALSE ) )
  checkTrue( !singleSlotHelper( context, "shareDir", "foo", "bar", TRUE ) )
  
  # FIELDS IN EXACT MATCH  (only checked when exactMatch == TRUE )

  # revoPath
  checkTrue( singleSlotHelper( context, "revoPath", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "revoPath", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "revoPath", "foo", "bar", FALSE ) )
  checkTrue( !singleSlotHelper( context, "revoPath", "foo", "bar", TRUE ) )

  checkTrue( singleSlotHelper( context, "revoPath", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "revoPath", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "revoPath", NULL, "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "revoPath", NULL, "foo", TRUE ) )
  
  # wait
  checkTrue( singleSlotHelper( context, "wait", TRUE, TRUE, FALSE ) )
  checkTrue( singleSlotHelper( context, "wait", TRUE, TRUE, TRUE ) )
  checkTrue( singleSlotHelper( context, "wait", TRUE, FALSE, FALSE ) )
  checkTrue( !singleSlotHelper( context, "wait", TRUE, FALSE, TRUE ) )
    
  # wait
  checkTrue( singleSlotHelper( context, "consoleOutput", TRUE, TRUE, FALSE ) )
  checkTrue( singleSlotHelper( context, "consoleOutput", TRUE, TRUE, TRUE ) )
  checkTrue( singleSlotHelper( context, "consoleOutput", TRUE, FALSE, FALSE ) )
  checkTrue( !singleSlotHelper( context, "consoleOutput", TRUE, FALSE, TRUE ) )

  # wait
  checkTrue( singleSlotHelper( context, "autoCleanup", TRUE, TRUE, FALSE ) )
  checkTrue( singleSlotHelper( context, "autoCleanup", TRUE, TRUE, TRUE ) )
  checkTrue( singleSlotHelper( context, "autoCleanup", TRUE, FALSE, FALSE ) )
  checkTrue( !singleSlotHelper( context, "autoCleanup", TRUE, FALSE, TRUE ) )

  # FIXME: CONFIG FILE checks on client at instantiation to see if it exists or not; this is probably a bug
  ## configFile
  
  checkTrue( singleSlotHelper( context, "configFile", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "configFile", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "configFile", NULL, context@configFile, FALSE ) )
  # note that the following test is resetting NULL to the actual configFile...
  checkTrue( singleSlotHelper( context, "configFile", NULL, context@configFile, TRUE ) )
            
  # workingDir
  checkTrue( singleSlotHelper( context, "workingDir", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "workingDir", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "workingDir", "foo", "bar", FALSE ) )
  checkTrue( !singleSlotHelper( context, "workingDir", "foo", "bar", TRUE ) )

  checkTrue( singleSlotHelper( context, "workingDir", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "workingDir", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "workingDir", NULL, "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "workingDir", NULL, "foo", TRUE ) )

  # dataPath
  checkTrue( singleSlotHelper( context, "dataPath", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "dataPath", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "dataPath", "foo", "bar", FALSE ) )
  checkTrue( !singleSlotHelper( context, "dataPath", "foo", "bar", TRUE ) )

  checkTrue( singleSlotHelper( context, "dataPath", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "dataPath", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "dataPath", NULL, "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "dataPath", NULL, "foo", TRUE ) )
  
  checkTrue( singleSlotHelper( context, "dataPath", c("bar","foo"), c("bar","foo"), FALSE ) )
  checkTrue( singleSlotHelper( context, "dataPath", c("bar","foo"), c("bar","foo"), TRUE ) )
  checkTrue( singleSlotHelper( context, "dataPath", c("bar","foo"), "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "dataPath", c("bar","foo"), "foo", TRUE ) )
  
  # minElems
  checkTrue( singleSlotHelper( context, "minElems", 3, 3, FALSE ) )
  checkTrue( singleSlotHelper( context, "minElems", 3, 3, TRUE ) )
  checkTrue( singleSlotHelper( context, "minElems", 3, 4, FALSE ) )
  checkTrue( !singleSlotHelper( context, "minElems", 3, 4, TRUE ) )
            
  # maxElems
  checkTrue( singleSlotHelper( context, "maxElems", 3, 3, FALSE ) )
  checkTrue( singleSlotHelper( context, "maxElems", 3, 3, TRUE ) )
  checkTrue( singleSlotHelper( context, "maxElems", 3, 4, FALSE ) )
  checkTrue( !singleSlotHelper( context, "maxElems", 3, 4, TRUE ) )

  # exclusive
  checkTrue( singleSlotHelper( context, "exclusive", TRUE, TRUE, FALSE ) )
  checkTrue( singleSlotHelper( context, "exclusive", TRUE, TRUE, TRUE ) )
  checkTrue( singleSlotHelper( context, "exclusive", TRUE, FALSE, FALSE ) )
  checkTrue( !singleSlotHelper( context, "exclusive", TRUE, FALSE, TRUE ) )
                    
  # nodes	- anything compared to NULL should be true				
  checkTrue( singleSlotHelper( context, "nodes", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "nodes", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "nodes", NULL, "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "nodes", NULL, "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "nodes", NULL, c("foo","bar"), FALSE ) )
  checkTrue( singleSlotHelper( context, "nodes", NULL, c("foo","bar"), TRUE ) )
  
  checkTrue( singleSlotHelper( context, "nodes", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "nodes", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "nodes", "bar", "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "nodes", "bar", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "nodes", c("foo","bar"), c("foo","bar"), FALSE ) )
  checkTrue( singleSlotHelper( context, "nodes", c("foo","bar"), c("foo","bar"), TRUE ) )	
  checkTrue( singleSlotHelper( context, "nodes", c("foo","bar","baz"), c("foo","bar"), FALSE ) )
  checkTrue( !singleSlotHelper( context, "nodes", c("foo","bar","baz"), c("foo","bar"), TRUE ) )	
  
  expectedRet <- FALSE
  if ( inherits( context, "RxLsfCluster" ) )
  {
    expectedRet <- TRUE
  }

  checkTrue( singleSlotHelper( context, "nodes", c("foo","bar"), c("foo.baz","bar.baz.flue"), FALSE ) )
  checkTrue( expectedRet == ( singleSlotHelper( context, "nodes", c("foo","bar"), c("foo.baz","bar.baz.flue"), TRUE ) ) )
  
  
  # dataDistType
  checkTrue( singleSlotHelper( context, "dataDistType", "split", "split", FALSE ) )
  checkTrue( singleSlotHelper( context, "dataDistType", "split", "split", TRUE ) )
  checkTrue( singleSlotHelper( context, "dataDistType", "split", "all", FALSE ) )
  checkTrue( !singleSlotHelper( context, "dataDistType", "split", "all", TRUE ) )
  
  # packagesToLoad			
  checkTrue( singleSlotHelper( context, "packagesToLoad", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", NULL, "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "packagesToLoad", NULL, "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", NULL, c("foo","bar"), FALSE ) )
  checkTrue( !singleSlotHelper( context, "packagesToLoad", NULL, c("foo","bar"), TRUE ) )
  
  checkTrue( singleSlotHelper( context, "packagesToLoad", "foo", "foo", FALSE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", "bar", "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "packagesToLoad", "bar", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", c("foo","bar"), c("foo","bar"), FALSE ) )
  checkTrue( singleSlotHelper( context, "packagesToLoad", c("foo","bar"), c("bar","foo"), TRUE ) )	
  checkTrue( singleSlotHelper( context, "packagesToLoad", c("foo","bar","baz"), c("foo","bar"), FALSE ) )
  checkTrue( !singleSlotHelper( context, "packagesToLoad", c("foo","bar","baz"), c("foo","bar"), TRUE ) )	
    
  # email
  checkTrue( singleSlotHelper( context, "email", NULL, NULL, FALSE ) )
  checkTrue( singleSlotHelper( context, "email", NULL, NULL, TRUE ) )
  checkTrue( singleSlotHelper( context, "email", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "email", "foo", "foo", TRUE ) )
  checkTrue( singleSlotHelper( context, "email", NULL, "foo", FALSE ) )
  checkTrue( !singleSlotHelper( context, "email", NULL, "foo", TRUE ) )
          
  # resultsTimeout
  checkTrue( singleSlotHelper( context, "resultsTimeout", 3, 3, FALSE ) )
  checkTrue( singleSlotHelper( context, "resultsTimeout", 3, 3, TRUE ) )
  checkTrue( singleSlotHelper( context, "resultsTimeout", 3, 4, FALSE ) )
  checkTrue( !singleSlotHelper( context, "resultsTimeout", 3, 4, TRUE ) )
}


#"rxHpcServerTestHelper" <- function( context )
#{
  ## FIELDS IN NON-EXACT MATCH (always checked)
                  #
  ## headNode 
  #checkTrue( singleSlotHelper( context, "headNode", "bar", "bar", FALSE ) )
  #checkTrue( singleSlotHelper( context, "headNode", "bar", "bar", TRUE ) )
  #checkTrue( !singleSlotHelper( context, "headNode", "bar", "foo", FALSE ) )
  #checkTrue( !singleSlotHelper( context, "headNode", "bar", "foo", TRUE ) )
  #
  ## computeOnHeadNode
  #checkTrue( singleSlotHelper( context, "computeOnHeadNode", TRUE, TRUE, FALSE ) )
  #checkTrue( singleSlotHelper( context, "computeOnHeadNode", TRUE, TRUE, TRUE ) )
  #checkTrue( singleSlotHelper( context, "computeOnHeadNode", TRUE, FALSE, FALSE ) )
  #checkTrue( !singleSlotHelper( context, "computeOnHeadNode", TRUE, FALSE, TRUE ) )
  #
  ## FIELDS IN EXACT MATCH  (only checked when exactMatch == TRUE )
  #
  ## priority
  #checkTrue( singleSlotHelper( context, "priority", 3, 3, FALSE ) )
  #checkTrue( singleSlotHelper( context, "priority", 3, 3, TRUE ) )
  #checkTrue( singleSlotHelper( context, "priority", 3, 4, FALSE ) )
  #checkTrue( !singleSlotHelper( context, "priority", 3, 4, TRUE ) )	
  #
  ## groups
  #checkTrue( singleSlotHelper( context, "groups", NULL, NULL, FALSE ) )
  #checkTrue( singleSlotHelper( context, "groups", NULL, NULL, TRUE ) )
  #checkTrue( singleSlotHelper( context, "groups", NULL, "foo", FALSE ) )
  #checkTrue( !singleSlotHelper( context, "groups", NULL, "foo", TRUE ) )
  #checkTrue( singleSlotHelper( context, "groups", NULL, c("foo","bar"), FALSE ) )
  #checkTrue( !singleSlotHelper( context, "groups", NULL, c("foo","bar"), TRUE ) )
#
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), "foo", FALSE ) )
  #checkTrue( !singleSlotHelper( context, "groups", c("foo","bar"), "foo", TRUE ) )
#
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("foo","bar"), FALSE ) )
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("foo","bar"), TRUE ) )
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("bar","foo"), FALSE ) )
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("bar","foo"), TRUE ) )
  #
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("foo","bar","baz"), FALSE ) )
  #checkTrue( !singleSlotHelper( context, "groups", c("foo","bar"), c("foo","bar","baz"), TRUE ) )
  #checkTrue( singleSlotHelper( context, "groups", c("foo","bar"), c("asdf","foo"), FALSE ) )
  #checkTrue( !singleSlotHelper( context, "groups", c("foo","bar"), c("asdf","foo"), TRUE ) )	
#}


  
##########################################################################################	

# base testing by type of simple types, exact match is default (FALSE)
"test.rxCompareContexts.baseContexts"
{
  myRxLocalSeq1 <- RxLocalSeq()
  myRxLocalSeq2 <- RxLocalSeq()
  
  myRxLocalParallel1 <- RxLocalParallel()
  myRxLocalParallel2 <- RxLocalParallel()
  
  myRxForeachDoPar1 <- RxForeachDoPar()
  myRxForeachDoPar2 <- RxForeachDoPar()
  
  checkTrue( rxCompareContexts( myRxLocalSeq1, myRxLocalSeq2 ) )
  checkTrue( rxCompareContexts( myRxLocalParallel1, myRxLocalParallel2 ) )
  checkTrue( rxCompareContexts( myRxForeachDoPar1, myRxForeachDoPar2 ) )
  
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxLocalParallel1 ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxForeachDoPar1 ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxLocalSeq1 ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxForeachDoPar1 ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalSeq1 ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalParallel1 ) )
}


# base testing by type of simple types, exact match set to TRUE
"test.rxCompareContexts.baseContexts.exactMatch" <- function()
{
  myRxLocalSeq1 <- RxLocalSeq()
  myRxLocalSeq2 <- RxLocalSeq()
  
  myRxLocalParallel1 <- RxLocalParallel()
  myRxLocalParallel2 <- RxLocalParallel()
  
  myRxForeachDoPar1 <- RxForeachDoPar()
  myRxForeachDoPar2 <- RxForeachDoPar()
  
  checkTrue( rxCompareContexts( myRxLocalSeq1, myRxLocalSeq2, exactMatch=TRUE ) )
  checkTrue( rxCompareContexts( myRxLocalParallel1, myRxLocalParallel2, exactMatch=TRUE ) )
  checkTrue( rxCompareContexts( myRxForeachDoPar1, myRxForeachDoPar2, exactMatch=TRUE ) )
  
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxLocalParallel1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxForeachDoPar1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxLocalSeq1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxForeachDoPar1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalSeq1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalParallel1, exactMatch=TRUE ) )
}


# base testing by type of basic and advanced types, exact match is default (FALSE)
"test.rxCompareContexts.baseContexts.clusterContexts" <- function()
{
  myRxLocalSeq1 <- RxLocalSeq()
  myRxLocalParallel1 <- RxLocalParallel()	
  myRxForeachDoPar1 <- RxForeachDoPar()
    
  checkTrue( rxCompareContexts( myRxLocalSeq1, myRxLocalSeq1 ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxLocalParallel1 ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxForeachDoPar1 ) )
  
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxLocalSeq1 ) )
  checkTrue( rxCompareContexts( myRxLocalParallel1, myRxLocalParallel1 ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxForeachDoPar1 ) )
  
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalSeq1 ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalParallel1 ) )
  checkTrue( rxCompareContexts( myRxForeachDoPar1, myRxForeachDoPar1 ) )

  
}


# base testing by type of basic and advanced types, exact match is set to TRUE
"test.rxCompareContexts.baseContexts.clusterContexts.exactMatch" <- function()
{
  myRxLocalSeq1 <- RxLocalSeq()
  myRxLocalParallel1 <- RxLocalParallel()	
  myRxForeachDoPar1 <- RxForeachDoPar()
  

  checkTrue( rxCompareContexts( myRxLocalSeq1, myRxLocalSeq1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxLocalParallel1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalSeq1, myRxForeachDoPar1, exactMatch=TRUE ) )
  
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxLocalSeq1, exactMatch=TRUE ) )
  checkTrue( rxCompareContexts( myRxLocalParallel1, myRxLocalParallel1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxLocalParallel1, myRxForeachDoPar1, exactMatch=TRUE ) )
  
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalSeq1, exactMatch=TRUE ) )
  checkTrue( !rxCompareContexts( myRxForeachDoPar1, myRxLocalParallel1, exactMatch=TRUE ) )
  checkTrue( rxCompareContexts( myRxForeachDoPar1, myRxForeachDoPar1, exactMatch=TRUE ) )
  
}
    

















