# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for to verify that rxDataStep has functionality of 
#    deprecated rxDataFrameToXdf
#
############################################################################################

"test.rxDataFrameToXdf.args.data.invalid" <- function()
{
  fileName1 <- tempfile(pattern = ".rxDFtoXdf", fileext = ".xdf")
  fileName2 <- tempfile(pattern = ".rxDFtoXdf", fileext = ".xdf")
    checkException(rxDataStep(new.env(hash = TRUE),
                                    fileName1),
                   silent = TRUE)
    checkException(rxDataStep(data.frame(),fileName2),
                   silent = TRUE)
}

"test.rxDataFrameToXdf.args.data.logical" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = c(NA, TRUE, NA, FALSE, NA))
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxDataStep(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.integer" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = c(NA, 7L, NA, 921L, NA))
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxReadXdf(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.numeric" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = c(NA, 1.4, NA, 3.7, NA))
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxDataStep(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.character" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = c(NA, "a", NA, "b", NA), stringsAsFactors = FALSE)
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxDataStep(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.factor" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = factor(c(NA, "a", NA, "b", NA), levels = letters))
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxDataStep(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.ordered.factor" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    myData <- data.frame(x = c(1,2,1,3,1), xfac = factor(c("low", "med", "low", "high", "low"), 
    levels = c("low", "med", "high"), ordered=TRUE))
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
  myNewData <- rxDataStep(outFile, reportProgress = 0)
    checkEquals(myNewData, myData)
  fileVarInfo <- rxGetVarInfo( outFile )
  dfVarInfo <- rxGetVarInfo( myNewData )
  checkEquals(fileVarInfo$xfac$varType, "ordered factor")
  checkEquals(dfVarInfo$xfac$varType, "ordered factor")
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.Date" <- function()
{
    myData <- data.frame(x = as.Date(.leap.seconds))
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    checkEquals(rxDataStep(outFile, reportProgress = 0), myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.data.POSIXct" <- function()
{
    myData <- data.frame(x = as.POSIXct(.leap.seconds))
    outFile <- tempfile(pattern = ".rxTemp", fileext = ".xdf")
    rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    myData1 <- rxDataStep( outFile, reportProgress = 0 )
    checkEquals(myData1, myData)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.outFile.invalid" <- function()
{
    testData <- data.frame(x = 1:10)
    checkException(rxDataStep(testData, outFile = TRUE), silent = TRUE)
    checkException(rxDataStep(testData, outFile = 1L), silent = TRUE)
    checkException(rxDataStep(testData, outFile = 1.0), silent = TRUE)
    checkException(rxDataStep(testData, outFile = c("a", "b")),
                   silent = TRUE)
}

"test.rxDataFrameToXdf.args.varsToKeepDrop.invalid" <- function()
{
    testData <- data.frame(x = 1:10)
    oldWarn <- options(warn = 2)
    on.exit(options(oldWarn), add = TRUE)
    checkException(rxDataStep(testData, tempfile(pattern = ".rxDF1V", fileext = ".xdf"),
                                    varsToKeep = NA_character_), silent = TRUE)
    checkException(rxDataStep(testData, tempfile(pattern = ".rxDF2V", fileext = ".xdf"),
                                    varsToKeep = 1), silent = TRUE)
    checkException(rxDataStep(testData, tempfile(pattern = ".rxDF3V", fileext = ".xdf"),
                                    varsToKeep = "a", varsToDrop = "b"),
                   silent = TRUE)

    checkException(rxDataStep(testData, tempfile(pattern = ".rxDF4V", fileext = ".xdf"),
                                    varsToDrop = NA_character_), silent = TRUE)
    checkException(rxDataStep(testData, tempfile(pattern = ".rxDF5V", fileext = ".xdf"),
                                    varsToDrop = 1), silent = TRUE)
}

"test.rxDataFrameToXdf.args.varsToKeepDrop" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")

    # Create a data frame with 100 rows
    myExtraData <- data.frame(xextra = 1:100,
                              yextra = rep(1:25, times = 4),
                              zextra = rep(1:25, each = 4))

    # Put a selection of columns into an xdf file
    varsToKeep <- c("xextra", "zextra")
    rxDataStep(myExtraData, outFile = outFile, varsToKeep = varsToKeep)

    fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, 2)
    checkEquals(names(fileInfo$varInfo), varsToKeep)
    file.remove(outFile)
  
  # Put just one columns into an xdf file [RA-2884]
    varsToKeep1 <- c("xextra")
    rxDataStep(myExtraData, outFile = outFile, varsToKeep = varsToKeep1)

    fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, 1)
    checkEquals(names(fileInfo$varInfo), varsToKeep1)
    file.remove(outFile)

    # Put a selection of columns into an xdf file, this time using varsToDrop
    varsToDrop <- "yextra"
    rxDataStep(myExtraData, outFile = outFile, varsToDrop = varsToDrop)
    fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, 2)
    checkEquals(names(fileInfo$varInfo), varsToKeep)
    file.remove(outFile)

    if (!rxIsExpressEdition())
    {
        # Copy 5 block base file to working directory
        file.copy(from = test100r5b, to = outFile, overwrite = TRUE)
        origFileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)

        # Append to columns from data frame
        rxDataStep(myExtraData, outFile = outFile, varsToKeep = varsToKeep,
                         append = "cols")
        fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
        checkEquals(fileInfo$numRows, 100)
        checkEquals(fileInfo$numVars, (origFileInfo$numVars + 2)) 
        checkEquals(names(fileInfo$varInfo), c(names(origFileInfo$varInfo), varsToKeep))
        file.remove(outFile)

        # Repeat using varsToDrop
        file.copy(from = test100r5b, to = outFile, overwrite = TRUE)

        rxDataStep(myExtraData, outFile = outFile, varsToDrop = varsToDrop,
                         append = "cols")
        fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
        checkEquals(fileInfo$numRows, 100)
        checkEquals(fileInfo$numVars, (origFileInfo$numVars + 2)) 
        checkEquals(names(fileInfo$varInfo), c(names(origFileInfo$varInfo), varsToKeep))
        file.remove(outFile)
    }
}

"test.rxDataFrameToXdf.args.rowVarName" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")

    # Create a data frame with 100 rows
    myExtraData <- data.frame(xextra = 1:100, 
                              yextra = rep(1:25, times = 4),
                              zextra = rep(1:25, each = 4))

    # Put a selection of columns into an xdf file
    varsToKeep <- c("xextra", "zextra", "myRowNames")
    rxDataStep(myExtraData, outFile = outFile, varsToKeep = varsToKeep,
                     rowVarName = "myRowNames")

    fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, 3)
    checkEquals(names(fileInfo$varInfo), varsToKeep)
    myNewData <- rxDataStep(outFile, reportProgress = 0)
    checkEquals(myNewData$myRowNames[1:10], as.character(1:10))
    file.remove(outFile)

    # Do the same thing, but appending columns to an existing data file
    if (!rxIsExpressEdition()) # Appending not supported
    {
        file.copy(from = test100r5b, to = outFile, overwrite = TRUE)
        rxDataStep(myExtraData, outFile = outFile, varsToKeep = varsToKeep,
                         rowVarName = "myRowNames", append = "cols")
        fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
        checkEquals(fileInfo$numRows, 100)
        checkEquals(fileInfo$numVars, 12)
        myNewData <- rxDataStep(outFile, reportProgress = 0)
        checkEquals(myNewData$myRowNames[1:10], as.character(1:10))
        file.remove(outFile)
    }

    # Test using row names that are character data
    myLittleDF <- data.frame(x = 1:3, y = c(TRUE, FALSE, TRUE))
    rowNameVec <- c("a", "b", "c")
    row.names(myLittleDF) <- rowNameVec
    rxDataStep(myLittleDF, outFile = outFile, rowVarName = "CharRowNames")
    fileInfo <- rxGetInfo(outFile, getVarInfo = TRUE)
    myNewData <- rxDataStep(outFile, stringsAsFactors = FALSE, reportProgress = 0)
    checkEquals(myNewData$CharRowNames, rowNameVec)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.append" <- function()
{
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }
    
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")

    # Create 2 original data frames with the same data but different variable names
    myData1 <- data.frame(x = 1:10, y = rep(c("a", "b", "c", "d", "e"), 2))
    myData2 <- myData1
    #names(myData2) <- c("x1", "y1")

    # Put the first data frame in an .xdf file
    rxDataStep(myData1, outFile = outFile)

    # Append the rows of the second data frame to the .xdf file
    rxDataStep(myData2, outFile = outFile, append = "rows")

    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 20)
    myNewData <- rxDataStep(outFile, reportProgress = 0)
    checkEquals(myNewData$x[1:10], myNewData$x[11:20])
    file.remove(outFile)

    # Put the first data frame in an .xdf file again
    rxDataStep(myData1, outFile = outFile)
    # Change the names of the variables in the data frame
    names(myData2) <- c("x1", "y1")
    # Now append columns to the xdf file
    rxDataStep(myData2, outFile = outFile, append = "cols")
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numVars, 4) 
    checkEquals(fileInfo$numRows, 10)
    myNewData <- rxDataStep(outFile, reportProgress = 0)
    checkEquals(myData1$x, myNewData$x1)

    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.append.noOutFile" <- function()
{
    
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }
    # RA-4510   
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")
    
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)
    
    warningExpr <- "A new file will be created." 
    # Note: rxDataStep will fail 
    # Append rows
    res <- try(
        rxDataStep(inData = iris, outFile = outFile, append = "rows", overwrite = T)
        )
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep("cannot be found", as.character(res))) > 0)
    if (file.exists(outFile)) file.remove(outFile)

    # Append cols
    res <- try(
        rxDataStep(inData = iris, outFile = outFile, append = "cols", overwrite = T)
        )
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep("cannot be found", as.character(res))) > 0)
    if (file.exists(outFile)) file.remove(outFile) 
    options(oldWarn)
}


"test.rxDataFrameToXdf.args.append.multiblock" <- function()
{
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }    
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")

    # Copy 1 block base file to working directory
    outFile <- tempfile(pattern = ".rxDFTempTest", fileext = ".xdf") 
    file.copy(from = test100r1b, to = outFile, overwrite = TRUE)
    origFileInfo <- rxGetInfo(outFile)

    # Create a data frame with 100 rows
    myExtraData <- data.frame(xextra = 1:100, yextra = rep(1:25, times = 4))

    # Add the columns of the data frame to an .xdf file
    rxDataStep(myExtraData, outFile = outFile, append = "cols")

    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, (origFileInfo$numVars + 2)) 

    file.remove(outFile)

    # Copy the multiblock base file to temp directory
    file.copy(from = test100r5b, to = outFile, overwrite = TRUE)
    origFileInfo <- rxGetInfo(outFile)

    # Add the columns of the data frame to an .xdf file
    rxDataStep(myExtraData, outFile = outFile, append = "cols")

    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 100)
    checkEquals(fileInfo$numVars, (origFileInfo$numVars + 2)) 

    myNewData <- rxDataStep(outFile, reportProgress = 0)
    checkEquals(myNewData$xextra, myExtraData$xextra)
    checkEquals(myNewData$yextra, myExtraData$yextra)
    file.remove(outFile)
}

"test.rxDataFrameToXdf.args.append.compXdf.dotInFileName" <- function()
{
    DEACTIVATED("Pending fix for Defect 5858780")
    
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }    
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
  
    claimsXdfd <- tempfile( pattern = ".claimsXdf", fileext = "")
    on.exit(if(file.exists(claimsXdfd)) unlink(claimsXdfd, recursive = TRUE, force = TRUE), add = TRUE)
    claimsXdfdDs <- RxXdfData(claimsXdfd, createCompositeSet = TRUE)
    rxImport(inData = claimsTxt, outFile = claimsXdfdDs, overwrite = TRUE,
      rowsPerRead = blockSize, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    rxGetInfo(claimsXdfdDs, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    rxDataStep(inData = setNames(claimsDf, paste(colnames(claimsDf), "2", sep = "")), 
        outFile = claimsXdfdDs, append = "cols", overwrite = FALSE, computeLowHigh = TRUE)
}

"test.rxDataFrameToXdf.args.append.compXdf.cols" <- function()
{
    DEACTIVATED("Pending fix for Bug 4636")
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }    
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
  
    claimsXdfd <- tempfile( pattern = "claimsXdf", fileext = "")
    on.exit(if(file.exists(claimsXdfd)) unlink(claimsXdfd, recursive = TRUE, force = TRUE), add = TRUE)
    claimsXdfdDs <- RxXdfData(claimsXdfd, createCompositeSet = TRUE)
    rxImport(inData = claimsTxt, outFile = claimsXdfdDs, overwrite = TRUE,
      rowsPerRead = blockSize, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    rxGetInfo(claimsXdfdDs, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    rxDataStep(inData = setNames(claimsDf, paste(colnames(claimsDf), "2", sep = "")), 
        outFile = claimsXdfdDs, append = "cols", overwrite = FALSE, computeLowHigh = TRUE)
}

"test.rxDataFrameToXdf.args.append.compXdf.rows" <- function()
{
    DEACTIVATED("Pending fix for Defect 5865690")
    if (rxIsExpressEdition())
    {
        # Append not supported in Standard Edition
        return( NULL )
    }    
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))

    claimsXdfd <- tempfile( pattern = "claimsXdf", fileext = "")
    on.exit(if(file.exists(claimsXdfd)) unlink(claimsXdfd, recursive = TRUE, force = TRUE), add = TRUE)
    claimsXdfdDs <- RxXdfData(claimsXdfd, createCompositeSet = TRUE)
    rxImport(inData = claimsTxt, outFile = claimsXdfdDs, overwrite = TRUE,
      rowsPerRead = blockSize, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    rxGetInfo(claimsXdfdDs, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    rxDataStep(inData = setNames(claimsDf, paste(colnames(claimsDf), "2", sep = "")), 
        outFile = claimsXdfdDs, append = "rows", overwrite = FALSE, computeLowHigh = TRUE)
}

"test.rxDataFrameToXdf.args.xdfCompressionLevel" <- function()
{
  x <- rep((1:1000)/5, times = 10)
  y <- rep(1:10, times = 1000)
  z <- rep(letters[1:25], times = 4000)
  tempFile0 <- tempfile(pattern = ".rxCompress0", fileext = ".xdf") 
  tempFile0a <- tempfile(pattern = ".rxCompress0a", fileext = ".xdf") 
  tempFile1 <- tempfile(pattern = ".rxCompress1", fileext = ".xdf") 
  tempFile1a <- tempfile(pattern = ".rxCompress1a", fileext = ".xdf") 
  tempFile5 <- tempfile(pattern = ".rxCompress5", fileext = ".xdf") 
  myData <- data.frame(x, y ,z)
  rxDataStep(inData = myData, outFile = tempFile0, xdfCompressionLevel = 0)
  rxDataStep(inData = myData, outFile = tempFile0a, xdfCompressionLevel = 0)
  rxDataStep(inData = myData, outFile = tempFile1, xdfCompressionLevel = 1)
  rxDataStep(inData = myData, outFile = tempFile1a, xdfCompressionLevel = 1)
  rxDataStep(inData = myData, outFile = tempFile5, xdfCompressionLevel = 5)
  
  compressedSize0 <- file.info(tempFile0)$size
  compressedSize1 <- file.info(tempFile1)$size
  compressedSize5 <- file.info(tempFile5)$size
  checkTrue( compressedSize0 > compressedSize1)
  checkTrue( compressedSize1 > compressedSize5)
  
    warningExpr <- "Cannot write compressed data to an uncompressed file." 		

    if (!rxIsExpressEdition())
    {
      # Append rows
      rxDataStep(inData = myData, outFile = tempFile0, xdfCompressionLevel = 0, append = "rows")
      
      # Try to append compressed data to uncompressed file
      # This should work, but give a warning

      out1 <- capture.output(
          rxDataStep(inData = myData, outFile = tempFile0a, xdfCompressionLevel = 1, append = "rows")
          )

      checkTrue(grepl(warningExpr, out1[2]))
      compressedSize0 <- file.info(tempFile0)$size
      compressedSize0a <- file.info(tempFile0a)$size
      checkEquals( compressedSize0, compressedSize0a)
      
      # Append rows with different compression level to compressed file
      rxDataStep(inData = myData, outFile = tempFile1, xdfCompressionLevel = 1, append = "rows")
      rxDataStep(inData = myData, outFile = tempFile1a, xdfCompressionLevel = 3, append = "rows")
      compressedSize1 <- file.info(tempFile1)$size
      compressedSize1a <- file.info(tempFile1a)$size
      checkTrue( compressedSize1 > compressedSize1a)

      # Append columns
      file.remove(tempFile0)
      file.remove(tempFile0a)
      rxDataStep(inData = myData, outFile = tempFile0, xdfCompressionLevel = 0)
      rxDataStep(inData = myData, outFile = tempFile0a, xdfCompressionLevel = 0)
      myData1 <- myData
      names(myData1) <- c("xa", "ya", "za")
      rxDataStep(inData = myData1, outFile = tempFile0, xdfCompressionLevel = 0, append = "cols")
      # Try to append compressed data to uncompressed file
      # This should work but give warning
      out2 <- capture.output(
        rxDataStep(inData = myData1, outFile = tempFile0a, xdfCompressionLevel = 1, append = "cols")
        )
      checkTrue(grepl(warningExpr, out2[2]))
      compressedSize0 <- file.info(tempFile0)$size
      compressedSize0a <- file.info(tempFile0a)$size
      checkEquals( compressedSize0, compressedSize0a)	
      
      file.remove(tempFile1)
      file.remove(tempFile1a)
      rxDataStep(inData = myData, outFile = tempFile1, xdfCompressionLevel = 1)
      rxDataStep(inData = myData, outFile = tempFile1a, xdfCompressionLevel = 1)
        rxDataStep(inData = myData1, outFile = tempFile1, xdfCompressionLevel = 2, append = "cols")
      rxDataStep(inData = myData1, outFile = tempFile1a, xdfCompressionLevel = 4, append = "cols")
      compressedSize1 <- file.info(tempFile1)$size
      compressedSize1a <- file.info(tempFile1a)$size
      checkTrue(compressedSize1 > compressedSize1a)
  }	
  if (file.exists(tempFile0)) file.remove(tempFile0)
  if (file.exists(tempFile0a)) file.remove(tempFile0a)
  if (file.exists(tempFile1)) file.remove(tempFile1)
  if (file.exists(tempFile1a)) file.remove(tempFile1a)
  if (file.exists(tempFile5)) file.remove(tempFile5)	
}

"test.rxDataFrameToXdf.value" <- function()
{
    outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")

    # Create 2 original data frames with the same data but different variable names
    myData1 <- data.frame(x = 1:10, y = rep(c("a", "b", "c", "d", "e"), 2))
    myData2 <- myData1

    # Put the first data frame in an .xdf file
    out1 <- rxDataStep(myData1, outFile = outFile)
    checkTrue(inherits(out1, "RxXdfData"))

    if (!rxIsExpressEdition())
    {
        # Append the rows of the second data frame to the .xdf file
        out2 <- rxDataStep(myData2, outFile = outFile, append = "rows")
        checkTrue(inherits(out2, "RxXdfData"))
 
        file.remove(outFile)

        # Put the first data frame in an .xdf file again
        out3 <- rxDataStep(myData1, outFile = outFile)
        names(myData2) <- c("x1", "y1")
        # Now append columns to the xdf file
        out4 <- rxDataStep(myData2, outFile = outFile, append = "cols")
        checkTrue(inherits(out4, "RxXdfData"))
        fileInfo <- rxGetInfo(out4)
        checkEquals(fileInfo$numVars, 4) 
        checkEquals(fileInfo$numRows, 10)
        myNewData <- rxDataStep(out4, reportProgress = 0)
        checkEquals(myData1$x, myNewData$x1)
   }
    file.remove(outFile)
}

"test.rxDataFrameToXdf.utf8" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74535: SQLPAL: unsupported conversion from 'UTF-8' to 'latin1' in codepage 1252")
  
  outFile <- tempfile( pattern = ".rxDF", fileext = ".xdf")

  latinCopyChar <- iconv("\u00a9", "UTF-8", "latin1") # create a latin1 copyright character, regardless of current local encoding
  utf8CopyChar <- "\u00a9" # creates a UTF-8 copyright character

  latinDf<-data.frame(a=latinCopyChar, stringsAsFactors=FALSE) # use latin1 copyright for df contents
  colnames(latinDf)<-latinCopyChar # and df column name
  checkEquals("latin1", Encoding(latinDf[1,1])) # confirm encoding
  checkEquals("latin1", Encoding(colnames(latinDf)))

  rxDataStep(latinDf, outFile, overwrite=TRUE) # write to XDF
  latinDf2 <- rxDataStep(outFile) # and get back
  checkEquals("UTF-8", Encoding(latinDf2[1,1])) # confirm it is now UTF-8
  checkEquals(utf8CopyChar, latinDf2[1,1]) # and has correct contents
  checkEquals("UTF-8", Encoding(colnames(latinDf2))) # do the same for column name
  checkEquals(utf8CopyChar, colnames(latinDf2))

  asianMoonChar <- "\u6708" # unicode moon character (creates a UTF-8 character)
  
  asianDf<-data.frame(a=asianMoonChar, stringsAsFactors=FALSE) # use moon character for df contents
  colnames(asianDf)<-asianMoonChar # and column name also
  checkEquals("UTF-8", Encoding(asianDf[1,1])) # confirm encoding
  checkEquals("UTF-8", Encoding(colnames(asianDf)))

  rxDataStep(asianDf, outFile, overwrite=TRUE) #write to XDF
  asianDf2 <- rxDataStep(outFile) # and get back
  checkEquals("UTF-8", Encoding(asianDf2[1,1])) # confirm it is still UTF-8
  checkEquals(asianMoonChar, asianDf2[1,1]) # and has correct contents
  checkEquals("UTF-8", Encoding(colnames(asianDf2))) # do the same for column name
  checkEquals(asianMoonChar, colnames(asianDf2))

  file.remove(outFile)
}

"test.rxDataFrameToXdf.help.file" <- function()
{
    # Convert included 'cars' data frame to .xdf file
    rxDataStep(inData = cars, outFile = file.path(tempdir(), "cars.xdf"), overwrite = TRUE)

    # Add columns from a data frame to an existing .xdf file
    # Create a data frame with 100 rows
    myExtraData <- data.frame(xextra = 1:100, yextra = rep(1:25, times = 4),
                          zextra = rep(1:25, each = 4))

    # Make a copy of an existing .xdf file that has 100 rows
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outputFile <- file.path(tempdir(), "rxTestFile100.xdf")
    file.copy(from = test100r5b, to = outputFile, overwrite = TRUE)

    # Put a selection of columns from the data frame into the xdf file
    # Include the row names in a column called "myRowNames"
    varsToKeep <- c("xextra", "zextra", "myRowNames")
    rxDataStep(inData = myExtraData, outFile = outputFile,
                 varsToKeep = varsToKeep, rowVarName = "myRowNames",
                 overwrite = TRUE)
    varInfo <- rxGetVarInfo(outputFile)
    checkEquals(names(varInfo), varsToKeep)
    file.remove(outputFile)
}