# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxDataStep
#
############################################################################################

".rxIsWindowsTest" <- if(exists(".rxIsWindows", globalenv())) get(".rxIsWindows", globalenv()) else RevoScaleR:::.rxIsWindows

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"test.rxDataStep.args.rowSelection.multvars" <- function()
{

  # x is 1, 2, 3, 4 repeated 4 times
  # y is 1, 1, 1, 1, 2, 2, etc
  # z is 1, 2, 1, 2...
  # w is 1, 1,... followed by 2s
    myData <- data.frame(x=rep(1:4, times=4), y=rep(1:4, each=4),
    z=rep(1:2, times=8), w=rep(1:2, each=8))

    testFile <- file.path(tempdir(),".rxDataStepTest.xdf")
  outFile <- file.path(tempdir(),".rxDataStepTestOut.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(outFile)) file.remove(outFile)
    
    # Create Xdf file
    rxDataStep(myData, testFile)

  rxDataStep(inData=testFile, outFile=outFile, rowSelection = x > 1)
  myXdfData <- rxDataStep( outFile )
  myOutData <- rxDataStep(inData = myData, rowSelection = x > 1)

  # Compare results using data frame and .xdf file
  checkEquals(myOutData, myXdfData)
  rm(myOutData)
  rm(myXdfData)
  file.remove(outFile)
  
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=x > w)
  myXdfData <- rxDataStep( outFile )
  myOutData <- rxDataStep(inData = myData, rowSelection = x > w)
  
  # Compare results using data frame and .xdf file
  checkEquals(myOutData, myXdfData)
  rm(myOutData)
  rm(myXdfData)
  file.remove(outFile)
    
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=(x > w) & (z > y))
  myXdfData <- rxDataStep( outFile )
  myOutData <- rxDataStep(inData = myData, rowSelection = (x > w) & (z > y) )
  
  checkEquals(myOutData, myXdfData)
  rm(myOutData)
  rm(myXdfData)
  file.remove(outFile)
  file.remove(testFile)
}

"test.rxDataStep.args.rowSelection.expressions" <- function()
{
  # Create a small example data frame and .xdf file for comparison
  factorVar <- c("a", "b", "c", "a", "b", "c", "a", "b", "c", "a")
  numericVar1 <- rep(1:5, times=2)
  numericVar2 <- 1:10
  charVar <- c("TOK1", "AAA1", "TOK2", "AAA2", "TOK3", "AAA3", "TOK4", "AAA4", "TOK5", "AAA5")
  myDF <- data.frame(factorVar, numericVar1, numericVar2, charVar)
  
  # Example 1
  # Extract a subset of rows from a data frame
  subDF <- myDF[which(as.character(myDF$factorVar) %in% c("a","b")),]
    row.names(subDF) <- NULL

  # Extract the same rows using the rowSelection argument
  dataStepDF <- rxDataStep(myDF,
    rowSelection=as.character(factorVar) %in% c("a", "b"))

  checkEquals(subDF, dataStepDF)
  rm(subDF)
  rm(dataStepDF)

  # Example 2
  subDF <- myDF[intersect(which(myDF$numericVar1 > 3), which(myDF$numericVar2 <= 5)) ,]
    row.names(subDF) <- NULL
  dataStepDF <- rxDataStep(myDF,
    rowSelection=numericVar1 > 3 & numericVar2 <=5)
  checkEquals(subDF, dataStepDF)
  rm(subDF)
  rm(dataStepDF)

  # Example 3
  subDF <- subset(myDF, numericVar1 == 3 | factorVar == "a")
    row.names(subDF) <- NULL
  dataStepDF <- rxDataStep(myDF,
    rowSelection= numericVar1 == 3 | factorVar == "a")
  checkEquals(subDF, dataStepDF)
  rm(subDF)
  rm(dataStepDF)

  # Example 4
  subDF <- myDF[-which(substr(myDF$charVar,1,3)=="TOK") ,]
    row.names(subDF) <- NULL
  dataStepDF <- rxDataStep(myDF,
    rowSelection= substr(charVar, 1, 3) != "TOK")
    checkEquals(subDF, dataStepDF)
  rm(subDF)
  rm(dataStepDF)
}

"test.rxDataStep.args.transformFunc.simpleData" <- function()
{
  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))

    testFile <- file.path(tempdir(),".rxDS1TempTest.xdf")
    testFile1 <- file.path(tempdir(),".rxDS1TempTest1.xdf")
    testFile2 <- file.path(tempdir(), ".rxDS1TempTest2.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  
    rxDataStep(myData, testFile)

    trans <- function(data)
    {
      data$a <- data$w > 0
      data$b <- data$z*100	
      return(data)								
    }

    # Pass all data to transformation function
    rxDataStep(inData=testFile, outFile=testFile1,  transformFunc=trans)
    dataOutXdf1 <- rxReadXdf( testFile1 )
  dataStepOut <- rxDataStep(inData = myData, transformFunc=trans)
  checkEquals(dataOutXdf1, dataStepOut)

  # Clean up
  file.remove(testFile)
  file.remove(testFile1)
  
} 

"test.rxDataStep.transformVars.dropVar" <- function()
{
    inFileC <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
  inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmallUC.xdf")
    outFile <- tempfile( pattern = ".rxDataStep", fileext = ".xdf")

    xform <- function(dataList)
    {
        # remove a variable
        dataList$CRSDepTime <- NULL
        dataList
    }
    
    rxDataStep(inFile, outFile, transformFunc = xform,
                  transformVars = "CRSDepTime", reportProgress = 0)
  xdfDataOut <- rxReadXdf(outFile)
  dataStepOut <- rxDataStep(rxReadXdf(inFile), transformFunc = xform)
  
    checkIdentical(xdfDataOut, dataStepOut)
  
  # Redo with compressed file
    rxDataStep(inFileC, outFile, transformFunc = xform,
                  transformVars = "CRSDepTime", reportProgress = 0, overwrite = TRUE)
  xdfDataOut <- rxReadXdf(outFile)
  checkIdentical(xdfDataOut, dataStepOut)	
  
    file.remove(outFile)
}

"test.rxDataStep.args.varsToKeep" <- function()
{
    set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), 
        z=rnorm(100), w=runif(100), s = 1:100, t = 1:100)
    testFile <- file.path(tempdir(), "rxTempTest.xdf")
    testFile1 <- file.path(tempdir(), "rxTempTest1.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  
    rxDataStep(myData, testFile)
    
    myTranforms <- function( dataList)
    {
        dataList$x2 = dataList$x*2
        return(dataList)
    }
    
    rxDataStep(inData = testFile, outFile = testFile1, 
       transformFunc = myTranforms, transformVars = "x",
       transforms = list(s2 = s*2),
       varsToKeep = c("y", "z"), rowSelection = w > 0,
       overwrite = TRUE)

    fileInfo <- rxGetInfo(testFile1, getVarInfo = TRUE)
    checkEquals(names(fileInfo$varInfo), c("y",  "z",  "x" , "s",  "w",  "s2", "x2"))
    
    # Repeat with data source
    testDS <- RxXdfData(testFile, varsToKeep = c("y", "z"))
    rxDataStep(inData = testDS, outFile = testFile1, 
       transformFunc = myTranforms, transformVars = "x",
       transforms = list(s2 = s*2),
       rowSelection = w > 0,
       overwrite = TRUE) 

    fileInfo1 <- rxGetInfo(testFile1, getVarInfo = TRUE)
    checkEquals(names(fileInfo1$varInfo), c("y",  "z",  "x" , "s",  "w",  "s2", "x2"))  
    checkEquals(fileInfo$numRows, fileInfo1$numRows) 
    
    if (file.exists(testFile)) file.remove(testFile)
    if (file.exists(testFile1)) file.remove(testFile1)
  
  # Use newName with a text data source
  claimsTxtDS <- RxTextData(file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
    colInfo = list(age = list(newName = "newAge"), car.age = list(newName = "newCarAge"),
    cost = list(newName = "newCost")))

  myData1 <- rxDataStep(claimsTxtDS, varsToKeep ="newCost")
  checkEquals(names(myData1), "newCost")
  myData2 <- rxDataStep(claimsTxtDS, varsToDrop ="newCarAge", transforms = list(
    newVar = newCost*2))
  checkEquals(names(myData2), c("RowNum", "newAge", "type", "newCost", "number", "newVar"))
  
  # Get error message for dtaa sources that don't respect varsToKeep
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()

    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE)

    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
    odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)	
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)			
  
  res <- try(myData <- rxDataStep(inData = odbcDS, varsToKeep = c("RowNum", "type", "cost")))
  
    warningExpr <- "'varsToDrop' not supported"
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep(warningExpr, as.character(res))) > 0) 
  
}

"test.rxDataStep.args.varsToDrop" <- function()
{
  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    testFile <- file.path(tempdir(), "rxTempTest.xdf")
    testFile1 <- file.path(tempdir(), "rxTempTest1.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  
    rxDataStep(myData, testFile)

    # Specify which variables to keep in the new data file
    varsToKeep <- c("x", "z", "w")
    rxDataStep(inData=testFile, outFile=testFile1,  varsToKeep=varsToKeep)
  xdfOutDF <- rxReadXdf(testFile1)
    checkEquals(names(xdfOutDF), varsToKeep)
    dataStepOutDF <- rxDataStep(inData=myData, varsToKeep = varsToKeep)
  checkIdentical( xdfOutDF, dataStepOutDF )
  file.remove(testFile1)
  rm(dataStepOutDF)
  rm(xdfOutDF)
  
    # Alternatively specify which variables to drop
    varsToDrop <- c("y") 
    rxDataStep(inData=testFile, outFile=testFile1,  varsToDrop=varsToDrop)
  xdfOutDF <- rxReadXdf(testFile1)
    checkEquals(names(xdfOutDF), varsToKeep)
    dataStepOutDF <- rxDataStep(inData=myData, varsToDrop=varsToDrop)
  checkIdentical( xdfOutDF, dataStepOutDF )
    file.remove(testFile1)
  
    # Drop 2 variables 
    varsToDrop <- c("x","y") 
    rxDataStep(inData=testFile, outFile=testFile1,  varsToDrop=varsToDrop)
  xdfOutDF <- rxReadXdf(testFile1)
    checkEquals(names(xdfOutDF), c("z", "w"))
    dataStepOutDF <- rxDataStep(inData=myData, varsToDrop=varsToDrop)
  checkIdentical( xdfOutDF, dataStepOutDF )
    file.remove(testFile1)
    
    # Drop all variables (RA-4387)
    varsToDrop <- c("x","y", "z", "w") 
    
    #oldWarn <- options(warn=2)
   # on.exit(options(oldWarn), add = TRUE)
    res <- try(
         rxDataStep(inData=testFile, outFile=testFile1,  varsToDrop=varsToDrop)
        ) 
    checkTrue(inherits(res,"try-error"))
    #warnRegExpr <- "All variables have been dropped from the data set"
    #checkTrue(length(grep(warnRegExpr, as.character(res))) > 0)
    #options(oldWarn)
    
  if (file.exists(testFile1)) file.remove(testFile1)
  file.remove(testFile)    
}

"test.rxDataStep.args.varsToDrop.data.frame" <- function()
{
    #RA-10424
    set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    newData <- rxDataStep(inData = myData, varsToDrop = c("y", "w"), 
        transforms = list(x10 = x*10))
    checkEquals(names(newData), c("x", "z", "x10"))
    
    # Compare with varsToKeep
    newData1 <- rxDataStep(inData = myData, varsToKeep = c("x", "z"), 
        transforms = list(x10 = x*10))
    checkEquals(names(newData), names(newData1))  
    
    res <- try(
         newData2 <- rxDataStep(inData=myData, varsToDrp= c("y", "w"))
        ) 
    checkTrue(inherits(res,"try-error"))
    errorExpr <- "not a valid argument name"
    checkTrue(length(grep(errorExpr, as.character(res))) > 0)
   
}


"test.rxDataStep.args.varsToDrop.dataSource" <- function()
{
  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    testFile <- file.path(tempdir(), "rxTempTest.xdf")
    testFile1 <- file.path(tempdir(), "rxTempTest1.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  
    rxDataStep(myData, testFile)

    # Specify which variables to keep in the new data file
    varsToKeep <- c("x", "w", "z")
    rxDataStep(inData=testFile, outFile=RxXdfData(testFile1),  varsToKeep=varsToKeep)
  xdfOutDF <- rxReadXdf(testFile1)
    dataStepOutDF <- rxDataStep(inData=myData, varsToKeep = varsToKeep)
  checkIdentical( xdfOutDF, dataStepOutDF )
  file.remove(testFile1)
  rm(dataStepOutDF)
  rm(xdfOutDF)
  
    # Alternatively specify which variables to drop
    varsToDrop <- c("y") 
    rxDataStep(inData=testFile, outFile=RxXdfData(testFile1),  varsToDrop=varsToDrop)
  xdfOutDF <- rxReadXdf(testFile1)
    dataStepOutDF <- rxDataStep(inData=myData, varsToDrop=varsToDrop)
  checkIdentical( xdfOutDF, dataStepOutDF )
    file.remove(testFile1)
  
    # Drop 2 variables 
    varsToDrop <- c("x","y") 
    rxDataStep(inData=testFile, outFile=RxXdfData(testFile1),  varsToDrop=varsToDrop)
  xdfOutDF <- rxReadXdf(testFile1)
    dataStepOutDF <- rxDataStep(inData=myData, varsToDrop=varsToDrop)
  checkIdentical( xdfOutDF, dataStepOutDF )
    file.remove(testFile1)
    
    # Drop all variables (RA-4387)
    varsToDrop <- c("x","y", "z", "w") 
    
    #oldWarn <- options(warn=2)
    #on.exit(options(oldWarn), add = TRUE)
    res <- try(
         rxDataStep(inData=testFile, outFile=RxXdfData(testFile1),  varsToDrop=varsToDrop)
        ) 
    checkTrue(inherits(res,"try-error"))
    #warnRegExpr <- "All variables have been dropped from the data set"
    #checkTrue(length(grep(warnRegExpr, as.character(res))) > 0)
    #options(oldWarn)
    
  if (file.exists(testFile1)) file.remove(testFile1)
  file.remove(testFile)    
}


"test.rxDataStep.args.removeMissings.xdf" <- function()
{
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
    
    # Base file has 1 block with missings removed before writing
    test77NoMiss <- rxReadXdf(test77r1bNoMiss)
  test100WithMiss <- rxReadXdf(test100r1b)
  # These 2 files have factor levels in different order
  # Clean that up first
  test77NoMiss$xfac1 <- factor(as.integer(as.character(test77NoMiss$xfac1)))
  test100WithMiss$xfac1 <- factor(as.integer(as.character(test100WithMiss$xfac1)))
  
  test77NoMiss$xfac2 <- factor(as.character(test77NoMiss$xfac2))
  test100WithMiss$xfac2 <- factor(as.character(test100WithMiss$xfac2))
  
    # Remove missings from data set with 1 block and compare with base
    dataStepDF <- rxDataStep(inData=test100WithMiss,  removeMissings=TRUE)
  checkIdentical(test77NoMiss, dataStepDF)
}

"test.rxDataStep.args.varsToKeep" <- function()
{
    xform <- function(data)
    {
      # alter a variable
      data$x <- data$x * 10
      
      # remove a variable
      data$z <- NULL
      
      # return the altered list of variables
      return(data)
    }

  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    varInfo1 <- rxGetVarInfo(myData)
    
    # Check for 4 variables in first data set
    checkEquals(length(varInfo1), 4)

    # Remove z in transform function variables to keep in the new data file
    outData <- rxDataStep(inData = myData,  transformFunc=xform)
    varInfo2 <- rxGetVarInfo(outData) 
    
    # Check for 3 variables in second data set
    checkEquals(length(varInfo2), 3)
    checkTrue(all(sort(names(varInfo2)) %in% c("x", "y", "w")))
    
    # Check the x variable was changed properly
    checkTrue(all.equal(outData$x, seq(from=10, to=1000, by=10), check.attributes=FALSE))
  
  ## Do the same thing, this time removing vars using varsToKeep

  outData2 <- rxDataStep(inData=myData, transforms = list(x = x*10), varsToKeep=c("x", "y", "w"))
  varInfo3 <- rxGetVarInfo(outData2) 
  # Check for 3 variables in second data set
    checkEquals(length(varInfo3), 3)
    checkTrue(all(sort(names(varInfo3)) %in% c("x", "y", "w")))
    #
    # Check the x variable was changed properly
    checkTrue(all.equal(outData2$x, seq(from=10, to=1000, by=10), check.attributes=FALSE))
    
    # Try keeping a variable that doesn't exist
    
    res <- try(outData3 <- rxDataStep(inData = myData, varsToKeep = c("x", "y", "a", "b")))
  checkTrue(is(res, "try-error"))
}
 
"test.rxDataStep.args.reportProgress.xdf" <- function()
{
    # tiny data set. 30 rows, 10 rows/block, 3 blocks.
    inFile   <-  file.path(rxGetOption("unitTestDataDir"), "testSubset.xdf")       # Input
    
  outFile <- tempfile(pattern = ".rxDS2TempTest", fileext = ".xdf") # Output
    
  if (file.exists(outFile)) file.remove(outFile)
    o1 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=0))
    o2 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=1, overwrite=TRUE))
  if (file.exists(outFile)) file.remove(outFile)
    o2a <- capture.output(rxDataStep(inData=inFile,outFile=outFile, reportProgress=1, blocksPerRead=3))
  if (file.exists(outFile)) file.remove(outFile)
    o3 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=2))
  if (file.exists(outFile)) file.remove(outFile)
    o4 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=2, blocksPerRead=3))
  if (file.exists(outFile)) file.remove(outFile)
    # reportProgress = 0 should print no output
    checkEquals(length(o1), 0)
    checkEquals(length(o2), 1)
    if (rxIsExpressEdition())
    {
        checkEquals(o2, "\rRows Processed: 30 ")
        checkTrue(length(o3) == 1 )
        checkTrue(length(o3) == length(o4))
    } else
    {
        checkEquals(o2, "\rRows Processed: 10\rRows Processed: 20\rRows Processed: 30 ")
        checkTrue(length(o3) > 1 )
        # We should get shorter output when reading all the data in one block
        checkTrue(length(o3) > length(o4))
    }
    
    # If blocks per read is 3, should process all rows in one pass
    checkEquals(o2a, "\rRows Processed: 30 ")
    checkTrue(length(o4) > 0 )
}

"test.rxDataStep.args.append" <- function()
{
    testFile <- tempfile(pattern = ".rxDS3TempTest", fileext = ".xdf")
  testFile1 <- tempfile(pattern = ".rxDS3TempTest1", fileext = ".xdf")
  outFile1 <- tempfile(pattern = ".rxDS3TempOut1", fileext = ".xdf")
    
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  if (file.exists(outFile1)) file.remove(outFile1)
  
  # Create 2 original .xdf file, with the same data but different variable names
  myData <- data.frame(x=1:10, y=rep(c("a", "b", "c", "d", "e"),2))
    rxDataStep(inData = myData, outFile = testFile)

  names(myData) <- c("x1", "y1")
    rxDataStep(inData = myData, outFile = testFile1)
  
  # Just copy the data from one file to another using rxDataStep
  rxDataStep(inData=testFile, outFile=outFile1)
  
  # Try repeating this; it should fail because the outFile now exists
  res <- try(	rxDataStep(inData=testFile, outFile=outFile1))
  checkTrue(is(res,"try-error"))
  
  # Try one more time, this time specifying overwrite
  rxDataStep(inData=testFile, outFile=outFile1, overwrite=TRUE)
  
  # Now append rows to the newly created output file
  rxDataStep(inData=testFile, outFile=outFile1, append="rows")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 20)
  myNewData <- rxReadXdf(file=outFile1)
  checkEquals(myNewData$x[1:10], myNewData$x[11:20])
    
    # Now append rows again - for uneven number of rows
  rxDataStep(inData=testFile, outFile=outFile1, append="rows")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 30)
  myNewData <- rxReadXdf(file=outFile1)
  checkEquals(myNewData$x[1:10], myNewData$x[21:30])
      
  # Now append columns to the original xdf file
  rxDataStep(inData = testFile1, outFile = testFile, append="cols")
  fileInfo <- rxGetInfo(testFile)
  checkEquals(fileInfo$numVars, 4) 
  checkEquals(fileInfo$numRows, 10)
  myNewData <- rxReadXdf(file=testFile)
  checkEquals(myData$x1, myNewData$x1)
  
  file.remove(testFile)
  file.remove(testFile1)
  file.remove(outFile1)
}

"test.rxDataStep.args.append.multiblock" <- function()
{
    test100r5bC <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5bUC.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to temp directory
    outFile1 <- tempfile(pattern = ".rxTestFile1", fileext = ".xdf")
  outFile2 <- tempfile(pattern = ".rxTestFile2", fileext = ".xdf")
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Append rows from the original file to the copy
  rxDataStep(inData = test100r5b, outFile=outFile1, append="rows")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 200)
  file.remove(outFile1)
  
  # Make a copy again
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Try to append columns with the same name; should fail
  res <- try(rxDataStep(inData=test100r5b, outFile=outFile1, append="cols"))
  checkTrue(is(res,"try-error"))
  
  # Try again, this time requesting overwrite
  rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", overwrite=TRUE)
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 100)
  checkEquals(fileInfo$numVars, 9)  	
    
  # Rename the columns in outFile1
  varInfo <- as.list(NULL)
    checkNames <- vector();
  for (i in 1:fileInfo$numVars)
  {
    varInfo[[i]] <- list(position=i, newName=paste("xxx", i, sep=""))
        checkNames[i] <- paste("xxx", i, sep="")
  }
  rxSetVarInfo(varInfo, data=outFile1)
    newVarInfo <- rxGetVarInfo(outFile1)
    checkEquals(names(newVarInfo), checkNames)
  rm(fileInfo)
  
  file.copy(from=outFile1, to=outFile2, overwrite=TRUE)
  # Append columns with a different block size
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 18)
    checkEquals(fileInfo$numRows, 100)
    checkDF <- rxReadXdf(file=outFile1)
    checkEquals(checkDF$xxx1, checkDF$y)

  file.remove(outFile1)
    rm(checkDF)
  
  # Append columns with the same block size
  rxDataStep(inData=test100r5b, outFile=outFile2, append="cols")
  fileInfo <- rxGetInfo(outFile2)
  checkEquals(fileInfo$numVars, 18)
    checkEquals(fileInfo$numRows, 100)
    checkDF <- rxReadXdf(file=outFile2)
    checkEquals(checkDF$xxx1, checkDF$y)
   
  file.remove(outFile2)
}

"test.rxDataStep.args.append.multiblock.varsToKeep" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to working directory
    outFile1 <- tempfile(pattern = ".rxTestFile1", fileext = ".xdf")
  outFile2 <- tempfile(pattern = ".rxTestFile2", fileext = ".xdf")
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  fileInfo <- rxGetInfo(outFile1)
  
  # Rename the columns in outFile1
  varInfo <- as.list(NULL)
  for (i in 1:fileInfo$numVars)
  {
    varInfo[[i]] <- list(position=i, newName=paste("xxx", i, sep=""))
  }
  rxSetVarInfo(varInfo, data=outFile1)
  rm(fileInfo)
  
  # Make another copy of the multi-block outFile
  file.copy(from=outFile1, to=outFile2, overwrite=TRUE)
  
  # Append columns with a different block size, using varsToKeep 
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols", varsToKeep=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 12)
  # The values of the renamed first column in the original outFile should be the same as y in the inFile
  myData <- rxReadXdf(file=outFile1, varsToKeep=c("y", "xxx1"))
  checkEquals(myData[,1], myData[,2])
  file.remove(outFile1)
  
  file.copy(from=outFile2, to=outFile1, overwrite=TRUE)
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols", varsToDrop=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 15)
  file.remove(outFile1)
  
  # Append columns with the same block size
  rxDataStep(inData=test100r5b, outFile=outFile2, append="cols", varsToKeep=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo( outFile2)
  checkEquals(fileInfo$numVars, 12)
  
  file.remove(outFile2)
}

"test.rxDataStep.args.append.differentNumCols" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74579: Unit test test.rxDataStep.args.append.differentNumCols fails to return ERROR in output when running inside SPEES")

    myData1 <- data.frame(x=1:5, y=1:5)
    myData2 <- data.frame(x=6:10)

    xdfFile1 <- tempfile(pattern = ".rxTest1", fileext = ".xdf")
    xdfFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")
    
    rxDataStep(inData = myData1, outFile = xdfFile1, overwrite = TRUE)
    rxDataStep(inData = myData2, outFile = xdfFile2, overwrite = TRUE)
          
    # This should fail because of different number of columns	
    o1 <- capture.output(
        res <- try(
            rxDataStep(inData=xdfFile2, outFile=xdfFile1, append="rows", reportProgress = 2)
        ))
    checkTrue(is(res, "try-error"))

    #Bug 76414: capture.output fails to capture entire output when run inside SPEES
    if(!(  RevoScaleR:::rxInDatabaseHostPlatform())) checkTrue(grepl( "data file being appended must have the same", o1[2]))
    
    # Check to make sure the original data is OK
    myData2a <- rxReadXdf(xdfFile2)
    checkEquals(myData2, myData2a)
    
    myData1a <- rxReadXdf(xdfFile1)
    checkEquals(myData1, myData1a)

    # This should work
    rxDataStep(inData= xdfFile2, outFile = xdfFile1, append="rows", transforms=list(y=x))
    myData3 <- rxReadXdf(xdfFile1)
    checkEquals(nrow(myData3), 10)
    checkEquals(myData3$x, myData3$y)
    
    file.remove(xdfFile1)
    file.remove(xdfFile2)
}

"test.rxDataStep.args.append.rowSelection" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74579: Unit test test.rxDataStep.args.append.differentNumCols fails to return ERROR in output when running inside SPEES")
    
    myData1 <- data.frame(x=1:50, y=1:50)

    xdfFile1 <- tempfile(pattern = ".rxTest1", fileext = ".xdf")
    xdfFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")
    xdfFile3 <- tempfile(pattern = ".rxTest3", fileext = ".xdf")
    
    rxDataStep(inData = myData1, outFile = xdfFile1, rowsPerRead = 20, overwrite = TRUE)
    rxDataStep(inData = myData1, outFile = xdfFile2, 
    transforms = list(x1 = x, y1 = y, x = NULL, y = NULL), rowsPerRead = 20, overwrite = TRUE)
    # xdfFile3 has is identical to xdfFile1
    rxDataStep(inData = myData1, outFile = xdfFile3, rowsPerRead = 20, overwrite = TRUE)     
     
    # This should fail because of different number of rows with rowSelection	
    o1 <- capture.output(
        res <- try(
            rxDataStep(inData=xdfFile1, outFile=xdfFile2, append="cols", rowSelection = x > 25)
        ))
    checkTrue(is(res, "try-error"))	

    #Bug 76414: capture.output fails to capture entire output when run inside SPEES
    #checkTrue(grepl( "Cannot use 'rowSelection' when appending columns", o1[2]))
  
    # Appending rows to the same file should fail
    o1 <- capture.output(
        res <- try(
          rxDataStep(inData=xdfFile1, outFile=xdfFile1, append="rows", rowSelection = x > 25)
        ))
    checkTrue(is(res, "try-error"))	
    checkTrue(grepl( "Cannot append rows if input and output data sources are the same", o1[2]))    
        
    # But appending rows to a different existing file should work with rowSelection
    rxDataStep(inData=xdfFile1, outFile=xdfFile3, append="rows", rowSelection = x > 25)
  
    fileInfo3 <- rxGetInfo(xdfFile3)
    checkEquals(fileInfo3$numRows, 75)
  
    # Check to make sure the data is OK
    myData3 <- rxReadXdf(xdfFile3)
    checkEquals(myData3[26:50,]$x, myData3[51:75,]$x, checkAttributes = FALSE)
    
    file.remove(xdfFile1)
    file.remove(xdfFile2)
    file.remove(xdfFile3)
}

"test.rxDataStep.args.append.compXdf.cols" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
    
    #DEACTIVATED("Pending fix for Defect 5866567")  
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
    offset <- 2 # to make the row distribution uneven across the blocks
    #rxOptions("xdfCompressionLevel" = 0)	#default is 1.
    createCompositeSet <- TRUE   #FALSE #    

    compXdf1 <- tempfile( pattern = "compXdf1", fileext = "")
    on.exit(if(file.exists(compXdf1)) unlink(compXdf1, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf1.ds <- RxXdfData(compXdf1, createCompositeSet = createCompositeSet)
    rxImport(inData = claimsTxt, outFile = compXdf1.ds, overwrite = TRUE, stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf1.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    head(compXdf1.ds)
    
    compXdf2 <- tempfile( pattern = "compXdf2", fileext = "")
    on.exit(if(file.exists(compXdf2)) unlink(compXdf2, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf2.ds <- RxXdfData(compXdf2, createCompositeSet = createCompositeSet)
    #NOTE": the second composite XDF file has to match the first one structurely
    # i.e., in terms of numCompositeFiles, numRows, numBlocks, and rowsPerBlock.
    rxImport(inData = claimsTxt, outFile = compXdf2.ds, overwrite = TRUE, 
        transforms = list(cost2 = cost * 2, number2 = number * 2, RowNum2 = ceiling(RowNum / 32)), stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf2.info <- rxGetInfo(compXdf2.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    head(compXdf2.ds)
    
    rxDataStep(inData = compXdf2.ds, outFile = compXdf1.ds, 
        varsToKeep = c("cost2", "number2", "RowNum2"),
        append = "cols", overwrite = FALSE, computeLowHigh = TRUE)
    compXdf.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    head(compXdf1.ds)   #failed with Error: Call to 'uncompress' failed. Input data was corrupted or incomplete.

    filenameId <- match("fileName", names(compXdf.info))
    checkIdentical(compXdf.info[-filenameId], compXdf2.info[-filenameId])
    
    ## append the same columns again but with different values
    #compXdf3 <- tempfile( pattern = "compXdf3", fileext = "")
    #on.exit(if(file.exists(compXdf3)) unlink(compXdf3, recursive = TRUE, force = TRUE), add = TRUE)
    #compXdf3.ds <- RxXdfData(compXdf3, createCompositeSet = createCompositeSet)
    ##NOTE": the second composite XDF file has to match the first one structurely
    ## i.e., in terms of numCompositeFiles, numRows, numBlocks, and rowsPerBlock.
    #rxImport(inData = claimsTxt, outFile = compXdf3.ds, overwrite = TRUE, 
        #transforms = list(cost2 = cost * 3, number2 = number * 3, RowNum2 = ceiling(RowNum / 16)), stringsAsFactors = TRUE,
        #rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        #)
    #compXdf3.info <- rxGetInfo(compXdf3.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    #head(compXdf3.ds)
    #
    #rxDataStep(inData = compXdf3.ds, outFile = compXdf1.ds, 
        #varsToKeep = c("cost2", "number2", "RowNum2"),
        #append = "cols", overwrite = TRUE, computeLowHigh = TRUE)
    #compXdf.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    #head(compXdf1.ds)   #failed with Error: Call to 'uncompress' failed. Input data was corrupted or incomplete.
    #
    #filenameId <- match("fileName", names(compXdf.info))
    #checkIdentical(compXdf.info[-filenameId], compXdf3.info[-filenameId])
    #
    ####temp
    #compXdf2 <- rxDataStep(compXdf2.ds)
    #compXdf3 <- rxDataStep(compXdf3.ds)
    #compXdf1 <- rxDataStep(compXdf1.ds)
    #all.equal(compXdf3[1:6], compXdf1[1:6])
    #
    #col <- 7:9    #9  #8  #
    #all.equal(compXdf3, compXdf1)
    #cbind(compXdf3[col], compXdf1[col], compXdf3[col] == compXdf1[col])
}

"test.rxDataStep.args.append.compXdf.cols.transforms" <- function()
{
    DEACTIVATED("Pending fix for appending cols to composite xdf with transforms")
  #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
    offset <- 2 # to make the row distribution uneven across the blocks
    createCompositeSet <- TRUE  #FALSE  #
  
    compXdf1 <- tempfile( pattern = "compXdf1", fileext = "")
    on.exit(if(file.exists(compXdf1)) unlink(compXdf1, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf1.ds <- RxXdfData(compXdf1, createCompositeSet = createCompositeSet)
    rxImport(inData = claimsTxt, outFile = compXdf1.ds, overwrite = TRUE, stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf1.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    compXdf2 <- tempfile( pattern = "compXdf2", fileext = "")
    on.exit(if(file.exists(compXdf2)) unlink(compXdf2, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf2.ds <- RxXdfData(compXdf2, createCompositeSet = createCompositeSet)
    #NOTE": the second composite XDF file has to match the first one structurely
    # i.e., in terms of numCompositeFiles, numRows, numBlocks, and rowsPerBlock.
    rxImport(inData = claimsTxt, outFile = compXdf2.ds, overwrite = TRUE, 
        transforms = list(cost2 = cost * 2, number2 = number * 2), stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf2.info <- rxGetInfo(compXdf2.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
  ## use transforms
  xform <- expression(list(cost2 = cost * 2, number2 = number * 2))
    rxDataStep(inData = compXdf2.ds, outFile = compXdf1.ds, 
        #varsToKeep = c("cost", "number"),  #didn't help
    #varsToDrop = c("RowNum", "age", "car.age", "type", "cost", "number"),  #didn't help
    transforms = xform, append = "cols", overwrite = FALSE, computeLowHigh = TRUE)  #failed
    compXdf.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    head(compXdf1.ds)
    
    filenameId <- match("fileName", names(compXdf.info))
    checkIdentical(compXdf.info[-filenameId], compXdf2.info[-filenameId])
}

"test.rxDataStep.args.append.compXdf.cols.transformFunc" <- function()
{
    DEACTIVATED("Pending fix for appending cols to composite xdf with transforms")
  #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
    offset <- 2 # to make the row distribution uneven across the blocks
    createCompositeSet <- TRUE  #FALSE  #
  
    compXdf1 <- tempfile( pattern = "compXdf1", fileext = "")
    on.exit(if(file.exists(compXdf1)) unlink(compXdf1, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf1.ds <- RxXdfData(compXdf1, createCompositeSet = createCompositeSet)
    rxImport(inData = claimsTxt, outFile = compXdf1.ds, overwrite = TRUE, stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf1.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    compXdf2 <- tempfile( pattern = "compXdf2", fileext = "")
    on.exit(if(file.exists(compXdf2)) unlink(compXdf2, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf2.ds <- RxXdfData(compXdf2, createCompositeSet = createCompositeSet)
    #NOTE": the second composite XDF file has to match the first one structurely
    # i.e., in terms of numCompositeFiles, numRows, numBlocks, and rowsPerBlock.
    rxImport(inData = claimsTxt, outFile = compXdf2.ds, overwrite = TRUE, 
        #transforms = list(cost2 = cost * 2, number2 = number * 2), 
    stringsAsFactors = TRUE,
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf2.info <- rxGetInfo(compXdf2.ds, getVarInfo = TRUE, getBlockSizes = TRUE)

  ## use transformFunc
  xform <- function(dataList)
  {
    outList <- list()
    outList$cost2 <- dataList$cost * 2
    outList$number2 <- dataList$number * 2
    outList
  }
  
    rxDataStep(inData = compXdf1.ds, outFile = compXdf1.ds, 
    #varsToKeep = c("cost", "number"),  #didn't help
        #transformVars = c("cost", "number"),
    transformFunc = xform, append = "cols", 
        overwrite = FALSE, computeLowHigh = TRUE)  #failed with overwrite = FALSE
    compXdf.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    filenameId <- match("fileName", names(compXdf.info))
    checkIdentical(compXdf.info[-filenameId], compXdf2.info[-filenameId])
}

"test.rxDataStep.args.append.compXdf.rows" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
    
    if ( !.rxIsWindowsTest() ) DEACTIVATED("Pending fix for Defect 6226774")  
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(claimsTxt,  stringsAsFactors = TRUE)
    claimsDf <- rxImport(claimsTxtDs)
    
    numObs <- nrow(claimsDf)
    numXdfd <- 4L
    blocksPerCompositeFile <- 4L
    (blockSize <- as.integer(numObs / (numXdfd * blocksPerCompositeFile)))
    offset <- 2 # to make the row distribution uneven across the blocks
    
    compXdf1 <- tempfile( pattern = "compXdf1", fileext = "")
    on.exit(if(file.exists(compXdf1)) unlink(compXdf1, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf1.ds <- RxXdfData(compXdf1, createCompositeSet = TRUE, blocksPerCompositeFile = blocksPerCompositeFile)
    rxImport(inData = claimsTxt, outFile = compXdf1.ds, overwrite = TRUE,
        stringsAsFactors = TRUE,
        colClasses = c(cost = "numeric", number = "numeric"),   #by default floating point data are stored as "float32"
        rowsPerRead = blockSize#, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf1.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    vars <- colnames(claimsDf)
    varTypes <- sapply(claimsDf, class)
    varsToAdd <- paste(vars, "2", sep = "")
    xf <- ifelse(varTypes == "numeric", paste(vars, 2, sep = " * "), vars)
    xform <- paste(paste(varsToAdd, xf, sep = " = "), collapse = ", ")    
    transforms <- parse(text = paste("list(", xform, ")"))
    
    compXdf2 <- tempfile( pattern = "compXdf2", fileext = "")
    on.exit(if(file.exists(compXdf2)) unlink(compXdf2, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf2.ds <- RxXdfData(compXdf2, createCompositeSet = TRUE)
    rxImport(inData = claimsTxt, outFile = compXdf2.ds, overwrite = TRUE, 
        stringsAsFactors = TRUE, transforms = transforms,
        #colClasses = c(cost2 = "float32", number2 = "float32"),   #transformed floating point data are 'always' stored as "numeric"
        rowsPerRead = blockSize + offset, blocksPerCompositeFile = blocksPerCompositeFile    #createCompositeSet = TRUE,
        )
    compXdf2.info <- rxGetInfo(compXdf2.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    rxDataStep(inData = compXdf2.ds, outFile = compXdf1.ds, 
        varsToKeep = varsToAdd,
        append = "rows", overwrite = FALSE, computeLowHigh = TRUE)
    compXdf.info <- rxGetInfo(compXdf1.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    checkEquals(compXdf.info$numCompositeFiles, compXdf1.info$numCompositeFiles + compXdf2.info$numCompositeFiles)
    checkEquals(compXdf.info$numRows, compXdf1.info$numRows + compXdf2.info$numRows)
    checkEquals(compXdf.info$numBlocks, compXdf1.info$numBlocks + compXdf2.info$numBlocks)
    checkEquals(compXdf.info$varInfo[varsToAdd], compXdf2.info$varInfo[varsToAdd])
}

"test.rxDataStep.args.append.rows" <- function()
{
  if(rxIsExpressEdition()) 
  { 
    DEACTIVATED("This test is DEACTIVATED due to failure in 3.3.2")
  }
    
  ####
  ## xdf to xdf
  xdf1 <- file.path(tempdir(), "temp1.xdf")
  on.exit(if(file.exists(xdf1)) unlink(xdf1, recursive = TRUE, force = TRUE), add = TRUE)
    xdf1ds <-  RxXdfData(xdf1, createCompositeSet = FALSE)
    rxDataStep(iris, xdf1ds, overwrite = TRUE, rowsPerRead = 50)
  xdf1.info <- rxGetInfo(xdf1ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
  xdf2 <- file.path(tempdir(), "temp2.xdf")
  on.exit(if(file.exists(xdf2)) unlink(xdf2, recursive = TRUE, force = TRUE), add = TRUE)
    xdf2ds <- RxXdfData(xdf2, createCompositeSet = FALSE)
  rxDataStep(iris, xdf2ds, overwrite = TRUE, rowsPerRead = 25)
  xdf2.info <- rxGetInfo(xdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
    checkException(rxDataStep(xdf1ds, xdf2ds, append = "cols", rowsPerRead = 50, overwrite = FALSE))
    rxDataStep(xdf1ds, xdf2ds, append = "cols", rowsPerRead = 50, overwrite = TRUE)
  xdf21.info <- rxGetInfo(xdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    rxDataStep(xdf1ds, xdf2ds, append = "rows", rowsPerRead = 50)
  xdf22.info <- rxGetInfo(xdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    checkEquals(xdf21.info[["numRows"]] * 2, xdf22.info[["numRows"]])
  
  ####
  ## cxdf to cxdf
  cxdf1 <- file.path(tempdir(), "temp1Xdf")
  on.exit(if(file.exists(cxdf1)) unlink(cxdf1, recursive = TRUE, force = TRUE), add = TRUE)
    cxdf1ds <- RxXdfData(cxdf1, createCompositeSet = TRUE, blocksPerCompositeFile = 3)
    rxDataStep(iris, cxdf1ds, overwrite = TRUE, rowsPerRead = 25)
  cxdf1.info <- rxGetInfo(cxdf1ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
  cxdf2 <- file.path(tempdir(), "temp2Xdf")
  on.exit(if(file.exists(cxdf2)) unlink(cxdf2, recursive = TRUE, force = TRUE), add = TRUE)
    cxdf2ds <- RxXdfData(cxdf2, createCompositeSet = TRUE, blocksPerCompositeFile = 3)
  rxDataStep(iris, cxdf2ds, overwrite = TRUE, rowsPerRead = 25)
  cxdf2.info <- rxGetInfo(cxdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
    checkException(rxDataStep(cxdf1ds, cxdf2ds, append = "cols", rowsPerRead = 25, overwrite = FALSE))
    #rxDataStep(cxdf1ds, cxdf2ds, append = "cols", rowsPerRead = 25, overwrite = TRUE)  #bug?
  #cxdf21.info <- rxGetInfo(cxdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    rxDataStep(cxdf1ds, cxdf2ds, append = "rows", rowsPerRead = 50)
  cxdf22.info <- rxGetInfo(cxdf2ds, getVarInfo = TRUE, getBlockSizes = TRUE)
    #checkEquals(cxdf2.info[["numRows"]] * 2, cxdf22.info[["numRows"]]) #bug?

  ####
  ## xdf to cxdf  #NOTE: mixing xdf and cxdf is not currently supported.
  xdf <-  file.path(tempdir(), "temp.xdf")
  on.exit(if(file.exists(xdf)) unlink(xdf, recursive = TRUE, force = TRUE), add = TRUE)
    xdfds <-  RxXdfData(xdf, createCompositeSet = FALSE)
    rxDataStep(iris, xdfds, overwrite = TRUE, rowsPerRead = 50)
  xdf.info <- rxGetInfo(xdfds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
  cxdf <- file.path(tempdir(), "tempXdf")
  on.exit(if(file.exists(cxdf)) unlink(cxdf, recursive = TRUE, force = TRUE), add = TRUE)
    cxdfds <- RxXdfData(cxdf, createCompositeSet = TRUE, blocksPerCompositeFile = 3)
  rxDataStep(iris, cxdfds, overwrite = TRUE, rowsPerRead = 25)
  cxdf.info <- rxGetInfo(cxdfds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
    checkException(rxDataStep(xdfds, cxdfds, append = "cols", rowsPerRead = 50, overwrite = FALSE))
    #rxDataStep(xdfds, cxdfds, append = "cols", rowsPerRead = 50, overwrite = TRUE) #bug?
  #cxdf1.info <- rxGetInfo(cxdfds, getVarInfo = TRUE, getBlockSizes = TRUE)
    #rxDataStep(xdfds, cxdfds, append = "rows", rowsPerRead = 50)
  #cxdf2.info <- rxGetInfo(cxdfds, getVarInfo = TRUE, getBlockSizes = TRUE)
    #checkEquals(cxdf1.info[["numRows"]] * 2, cxdf2.info[["numRows"]])
}

"test.rxDataStep.args.append.rows.filesize" <- function()
{
    # DEACTIVATED("Bug 24722: rxDataStep(inDF, outXdf, append = "rows") creates crazy big XDF file when called in a loop")
    #library("RevoScaleR")

    nrows <- 20 #2000 #
    ncols <- 5 #50 #5000 #
    rowsPerBlock <- 10
    xdfCompressionLevel <- 0

    ## generate data
    set.seed(123)
    vec <- runif(nrows * ncols)
    size.vec <- object.size(vec)
    mat <- matrix(vec, nrow = nrows)
    size.mat <- object.size(mat)
    inDF = data.frame(mat)
    size.df <- object.size(inDF)

    ## call rxDataStep once (single block)
    outXdfSingleBlock <- tempfile(pattern = "outXdfSingleBlock", fileext = ".xdf")
    on.exit(if (file.exists(outXdfSingleBlock)) unlink(outXdfSingleBlock, recursive = TRUE, force = TRUE), add = TRUE)
    rxDataStep(inDF, outXdfSingleBlock, overwrite = TRUE, 
        xdfCompressionLevel = xdfCompressionLevel, rowsPerRead = nrows)

    size.outXdfSingleBlock <- file.info(outXdfSingleBlock)$size
    #rxGetInfo(outXdfSingleBlock, getVarInfo = TRUE, getBlockSizes = TRUE)
    outDfSingleBlock <- rxDataStep(outXdfSingleBlock, maxRowsByCols = 1e9)
    size.df.outDfSingleBlock <- object.size(outDfSingleBlock)

    ## call rxDataStep once (multiple blocks)
    outXdfMultipleBlock <- tempfile(pattern = "outXdfMultipleBlock", fileext = ".xdf")
    on.exit(if (file.exists(outXdfMultipleBlock)) unlink(outXdfMultipleBlock, recursive = TRUE, force = TRUE), add = TRUE)
    rxDataStep(inDF, outXdfMultipleBlock, overwrite = TRUE, 
        xdfCompressionLevel = xdfCompressionLevel, rowsPerRead = rowsPerBlock)

    size.outXdfMultipleBlock <- file.info(outXdfMultipleBlock)$size
    #rxGetInfo(outXdfMultipleBlock, getVarInfo = TRUE, getBlockSizes = TRUE)
    outDfMultipleBlock <- rxDataStep(outXdfMultipleBlock, maxRowsByCols = 1e9)
    size.df.outDfMultipleBlock <- object.size(outDfMultipleBlock)

    ## call rxDataStep repeatedly to append
    outXdfAppend <- tempfile(pattern = "outXdfAppend", fileext = ".xdf")
    on.exit(if (file.exists(outXdfAppend)) unlink(outXdfAppend, recursive = TRUE, force = TRUE), add = TRUE)
    nblocks <- seq(nrows / rowsPerBlock)
    for (i in nblocks) {
        rowsToWrite <- ((i - 1) * rowsPerBlock + 1):(i * rowsPerBlock)
        inDFsub <- inDF[rowsToWrite,]
        if (i == 1) {
            rxDataStep(inDFsub, outXdfAppend, overwrite = TRUE, 
                xdfCompressionLevel = xdfCompressionLevel, rowsPerRead = rowsPerBlock)
        } else {
            rxDataStep(inDFsub, outXdfAppend, append = "rows", 
                xdfCompressionLevel = xdfCompressionLevel, rowsPerRead = rowsPerBlock)
        }
    }

    size.outXdfAppend <- file.info(outXdfAppend)$size
    #rxGetInfo(outXdfAppend, getVarInfo = TRUE, getBlockSizes = TRUE)
    outDfAppend <- rxDataStep(outXdfAppend, maxRowsByCols = 1e9)
    size.df.outDfAppend <- object.size(outDfAppend)

    ## check results
    print(c(df = size.df, 
        outXdfSingleBlock = size.outXdfSingleBlock,
        outXdfMultipleBlock = size.outXdfMultipleBlock, 
        outXdfAppend = size.outXdfAppend))
    checkTrue(abs(size.outXdfAppend - size.outXdfMultipleBlock) <=
        min(size.outXdfAppend, size.outXdfMultipleBlock) * 0.1) #the difference is 212 to be exact but should it be 0 instead?
    checkTrue(abs(size.outXdfSingleBlock - size.outXdfMultipleBlock) <=
        min(size.outXdfSingleBlock, size.outXdfMultipleBlock) * 0.1) #the difference is 124 to be exact but why size.outXdfSingleBlock is larger?

    checkIdentical(size.df, size.df.outDfSingleBlock)
    checkIdentical(size.df, size.df.outDfMultipleBlock)
    checkIdentical(size.df, size.df.outDfAppend)
}

"test.rxDataStep.args.overwrite" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to working directory
    outFile1 <- tempfile(pattern = ".rxTest1", fileext = ".xdf")
    outFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")

  file.copy(test100r5b, outFile1, overwrite=TRUE)
  if (file.exists(outFile2)) file.remove(outFile2)
  
  # Specify overwrite = true with no append and no existing outFile
  
  rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", overwrite=TRUE)
  fileInfo <- rxGetInfo(data = outFile2)
  checkEquals(fileInfo$numVars, 8)
  
  # Specify overwrite = true with an existing outfile
  rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", overwrite=TRUE)
  fileInfo <- rxGetInfo(data = outFile2)
  checkEquals(fileInfo$numVars, 8)
  
  # Specify overwrite = false with an existing outfile: should fail
  res <- try(rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", 
    overwrite=FALSE))
  checkTrue(is(res, "try-error"))
  
  # Specify no overwrite with an existing outfile: should fail
  res <- try(rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y"))
  checkTrue(is(res, "try-error"))
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  # If appending cols, specify overwrite = TRUE should overwrite columns
  rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", 
    varsToKeep=c("y", "xnum1", "xfac2"), overwrite=TRUE)
  fileInfo <- rxGetInfo(data=outFile1)
  checkEquals(fileInfo$numVars, 9)
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Specify overwrite = FALSE - should fail because of the same column names
  res <- try(rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", 
    varsToKeep=c("y", "xnum1", "xfac2"), overwrite=FALSE))
  checkTrue(is(res, "try-error"))
  file.remove(outFile1)
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  # overwrite is ignored when appending rows
  rxDataStep(inData=test100r1b, outFile=outFile1, append="rows", 
   overwrite=TRUE)
  fileInfo <- rxGetInfo(data=outFile1)
  checkEquals(fileInfo$numRows, 200)
    file.remove(outFile1)
  
  if (file.exists(outFile2)) file.remove(outFile2)
  
}

"test.rxDataStep.args.transformFunc.combo" <- function()
{

    inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
    outFile1 <- tempfile(pattern = ".rxKyphosis1", fileext = ".xdf")
    outFile2 <- tempfile(pattern = ".rxKyphosis2", fileext = ".xdf")

    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
    
    xform <- function(data) 
    { 
        data$Number1 <- data$Number + 1 
        return(data) 
     } 

    # With transform
  rxDataStep(inData=inFile, outFile=outFile1, rowSelection = Age > 30, 
        varsToKeep="Start", transforms=list(Number1=Number + 1))

    # With transformFunc
    rxDataStep(inData=inFile, outFile=outFile2, transformVars=c("Number"), 
        rowSelection = Age > 30, 
        varsToKeep="Start", transformFunc=xform, overwrite=TRUE) 

    fileInfo1 <- rxGetInfo(outFile1)
    fileInfo2 <- rxGetInfo(outFile2)
    
    checkEquals(fileInfo1$numRows, fileInfo2$numRows)
    checkEquals(fileInfo1$numVars, fileInfo2$numVars)
  
  myDF1 <- rxReadXdf(outFile1)
  myDF2 <- rxReadXdf(outFile2)
  checkEquals(myDF1, myDF2)
    
    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
}

"test.rxDataStep.args.rowSelection.combo" <- function()
{

  inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
    outFile1 <- tempfile(pattern = ".rxKyphosis1", fileext = ".xdf")
    outFile2 <- tempfile(pattern = ".rxKyphosis2", fileext = ".xdf")
    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
  
  xform <- function(data) 
  { 
    data$Age <- data$Age*2. 
    data$Number <- as.integer(data$Number) 
    data$Start <- as.factor(data$Start) 
    return(data) 
  } 

  rxDataStep(inData=inFile, outFile=outFile1,transformFunc=xform,overwrite=TRUE) 

  rxDataStep(inData=inFile, outFile=outFile2, transformFunc=xform, 
        rowSelection= Kyphosis=="absent", overwrite=TRUE) 
  
  myDF1 <- rxReadXdf(outFile1)
  myDF1 <- subset(myDF1, Kyphosis=="absent")
  myDF2 <- rxReadXdf(outFile2)
  # Row names will be different
  checkEquals(myDF1, myDF2, check.attributes=FALSE)
  rm(myDF1)
  
    if (file.exists(outFile1)) file.remove(outFile1)
  
  rxDataStep(inData=inFile, outFile=outFile1, 
        transforms=list(Age = Age*2., 
            Number=as.integer(Number), Start=as.factor(Start)),
    rowSelection= Kyphosis=="absent", overwrite=TRUE) 
  
  myDF1 <- rxReadXdf(outFile1)
  checkEquals(myDF1, myDF2)
  
  if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
}

"test.rxDataStep.args.transformFunc.rxStartRow" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    tempFile <- tempfile(pattern = ".rxTemp", fileext = ".xdf")
  if (file.exists(tempFile)) file.remove(tempFile)
    
  myTransformFunc <- function( dataList)
  {
    rxDataSource <- .rxReadFileName
    rxStartRow <- .rxStartRow
    rxChunkNum <- .rxChunkNum
    rxIsTestChunk <- .rxIsTestChunk
    
    nrows <- length(dataList[[1]])
    dataList$startRow <- rep(.rxStartRow, nrows)
    dataList$chunkNum <- rep(.rxChunkNum, nrows)
    return ( dataList)
  }
  rxDataStep(inData=test100r5b, outFile=tempFile, transformFunc=myTransformFunc, overwrite=TRUE)
  myData <- rxReadXdf(tempFile)
    if (rxIsExpressEdition())
    {
      testStartRow <- rep(1, each=100)
      testChunkNum <- testStartRow
    } else
    {
      testStartRow <- rep(seq(1,by=20, length.out=5), each=20)
      testChunkNum <- rep(seq(1, by=1, length.out=5), each=20)
    }
       
    
  checkEquals(myData$startRow, testStartRow)
  checkEquals(myData$chunkNum, testChunkNum)
  if (file.exists(tempFile)) file.remove(tempFile)
  rm(myData)
  
  # Repeat with numCoresToUse set to 1
    numCoresOld <- rxGetOption("numCoresToUse")
    
  rxOptions(numCoresToUse=1)
  rxDataStep(inData=test100r5b, outFile=tempFile, transformFunc=myTransformFunc, overwrite=TRUE)
  myData <- rxReadXdf(tempFile)
  #testStartRow <- rep(seq(1,by=20, length.out=5), each=20)
  checkEquals(myData$startRow, testStartRow)
  checkEquals(myData$chunkNum, testChunkNum)
    
  rxOptions(numCoresToUse=numCoresOld)
  if (file.exists(tempFile)) file.remove(tempFile)
    
}

"test.rxDataStep.transformFunc.negative" <- function(){
  
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxDataStep(inData=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxDataStep"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxDataStep"))
    }
  })
  
}

"test.rxDataStep.examples" <- function()
{
  set.seed(100)
  myData <- data.frame(x = 1:100, y = rep(c("a", "b", "c", "d"), 25),
    z = rnorm(100), w = runif(100))
      
  # Get a subset rows and columns from the data frame   
  varsToKeep <- c("x", "w", "y")
  myDataSubset <- rxDataStep(inData = myData,
    varsToKeep = varsToKeep, rowSelection = y != "a") 
  checkEquals(names(myDataSubset), varsToKeep)
  checkEquals(nrow(myDataSubset), 75)  

  # Create a simple .xdf file from the data frame
  inputFile <- file.path(tempdir(), ".rxTestInput.xdf")
  if (file.exists(inputFile)) file.remove(inputFile)
  rxDataStep(myData, inputFile, overwrite = TRUE)

  # Subset rows and columns, creating a new .xdf file
  outputFile <- file.path(tempdir(), "testOutput.xdf")
  rxDataStep(inData = inputFile, outFile = outputFile,
    varsToKeep = varsToKeep, rowSelection = y != "a",
    overwrite = TRUE)
  checkEquals(rxReadXdf(outputFile), myDataSubset, check.attributes=FALSE)

  # Use transforms list with a data frame
  myNewData <- rxDataStep(inData = myData, 
    transforms = list(a = w > 0, b = 100 * z))
  checkEquals(names(myNewData), c("x", "y", "z", "w", "a", "b"))

  # Specify which variables to keep in a new data file
  varsToKeep <- c("x", "w", "z")
  rxDataStep(inData = inputFile, outFile = outputFile, varsToKeep = varsToKeep,
    transforms = list(a = w > 0, b = 100 * z), overwrite = TRUE)
  varInfo <- rxGetVarInfo(outputFile)
  checkEquals(length(varInfo), 5)

  # Alternatively specify which variables to drop
  varsToDrop <- "y"
  rxDataStep(inData = inputFile, outFile = outputFile, varsToDrop = varsToDrop, 
    transforms = list(a = w > 0, b = 100 * z), overwrite = TRUE)
  varInfo1 <- rxGetVarInfo(outputFile)
  checkEquals(c("x", "z", "w", "a", "b"), names(varInfo1))

  # Create a selection variable to take a roughly 25\% random sample
  # of each block of data read (in this case 1).

  set.seed(75)
  rxDataStep(inData = inputFile, outFile = outputFile, rowSelection = selVar,
    transforms = list(
      selVar = as.logical(rbinom(.rxNumRows, 1, .25))
    ), overwrite = TRUE)
  
  fileInfo <-	rxGetInfo(outputFile)
  checkTrue( (fileInfo$numRows < 30) && (fileInfo$numRows > 15))
    mySampledXdfData <- rxReadXdf(outputFile)

  file.remove(inputFile)
  file.remove(outputFile)
    
    set.seed(75)
  mySampledData <- rxDataStep(inData = myData,  rowSelection = selVar,
    transforms = list(
      selVar = as.logical(rbinom(.rxNumRows, 1, .25))
    ))
    checkEquals( mySampledXdfData, mySampledData)
}

"test.rxDataStep.args.rowsPerRead" <- function()
{
  tempFile <- file.path(tempdir(),".rxDS4TempTest.xdf")
  if (file.exists(tempFile)) file.remove(tempFile)
  
  # Write a data frame to an .xdf file using rowsPerRead, so block size is 50		
  rxDataStep(inData=iris, outFile=tempFile, rowsPerRead=50, overwrite=TRUE)
  fileInfo <- rxGetInfo(tempFile)
  
  if (rxIsExpressEdition())
  {
    checkEquals( fileInfo$numBlocks, 1)
    } else
    {
      checkEquals( fileInfo$numBlocks, nrow(iris)/50)
  }
  

  testDF <- rxReadXdf(tempFile)
  checkEquals(iris, testDF)
  
  # Try reblocking the new file and writing back to itself
  # This should fail
  res <- try(rxDataStep(inData=tempFile, outFile=tempFile, rowsPerRead=20, overwrite=TRUE))
  checkTrue(is(res,"try-error"))

  if (file.exists(tempFile)) file.remove(tempFile)	
}

"test.rxDataStep.args.rowSelection.ordered" <- function()
{
    sampleDataDir <- rxGetOption("sampleDataDir")
    inputFile <- file.path(sampleDataDir, "claims.txt")
    outputFile <- file.path(tempdir(), "claimsFactorExploration.xdf")
    rxTextToXdf(inputFile, outputFile,
                colInfo = list(car.age = list(type = "ordered", 
                levels = c("0-3", "4-7", "8-9", "10+"))),
                overwrite = TRUE)
    oldCars <- rxDataStep(inData = outputFile, rowSelection = car.age > "0-3")
    checkEquals( sum(oldCars$car.age == "0-3"), 0)
    file.remove(outputFile)
}

"test.rxDataStep.args.outFile.samefile" <- function()
{
  XDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  inFileNew <- tempfile(pattern = ".rxTempIn", fileext = ".xdf")
  
  if (file.exists(inFileNew)) file.remove(inFileNew)
  
  file.copy(from = XDF, to = inFileNew, overwrite = TRUE)
  
    rxDataStep(inData = inFileNew, outFile = inFileNew, 
    transforms = list(xint1 = xint1*2L),
    overwrite = TRUE, reportProgress = 4)
  file.remove(inFileNew)
}


"test.rxDataStep.args.rowSelection.samefile" <- function()
{

    # RA-4190
  # x is 1, 2, 3, 4 repeated 4 times
  # y is 1, 1, 1, 1, 2, 2, etc
  # z is 1, 2, 1, 2...
  # w is 1, 1,... followed by 2s
    myData <- data.frame(x=rep(1:4, times=4), y=rep(1:4, each=4),
    z=rep(1:2, times=8), w=rep(1:2, each=8))

    testFile <- tempfile(pattern = ".rxDS5TempTest", fileext = ".xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
    
    # Create Xdf file
    rxDataStep(myData, testFile, overwrite=TRUE)
  
    rxDataStep(inData=testFile, outFile=testFile, rowSelection=x > 1, overwrite=TRUE)
  
    testData <- rxDataStep(testFile)
    checkEquals(sum(testData$x <= 1), 0L)

  file.remove(testFile)
    
}	

"test.rxDataStep.args.startRow" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile(pattern = ".rxDS6TempTest", fileext = ".xdf")
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 15, overwrite=TRUE)
    myData1 <- rxReadXdf(test100r5b)
    myData2 <- rxReadXdf(outFile)
    checkEquals(myData1[15:100,], myData2, check.attributes = FALSE)

    # Try skipping the first block entirely
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 25, overwrite=TRUE)
    myData2 <- rxReadXdf(outFile)
    checkEquals(myData1[25:100,], myData2, check.attributes = FALSE)

    # Now try combining with a total number of rows
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 25, 
        numRows = 35, overwrite=TRUE)
    myData2 <- rxReadXdf(outFile)
    checkEquals(myData1[25:59,], myData2, check.attributes = FALSE)

    # Again, but less than 1 block
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 25, 
        numRows = 7, overwrite=TRUE)
    myData2 <- rxReadXdf(outFile)
    checkEquals(myData1[25:31,], myData2, check.attributes = FALSE)

    # What happens if we have rowSelection  - it will happen after the read
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 25, 
        numRows = 40, rowSelection = ylogical == TRUE, overwrite=TRUE)
    myData1small <- myData1[25:64,]
    myData2 <- rxReadXdf(outFile)
    checkEquals(subset(myData1small, ylogical == TRUE), myData2, check.attributes = FALSE)

    # Also check remove missings
    rxDataStep(inData = test100r5b, outFile = outFile, startRow = 25, 
        numRows = 40, removeMissings = TRUE, overwrite=TRUE)
    myData1small <- myData1[25:64,]
    myData2 <- rxReadXdf(outFile)
    checkEquals(na.omit(myData1small), myData2, check.attributes = FALSE)

    file.remove(outFile)

    # Now some examples with data frames
    myData2 <- rxDataStep(inData = myData1,  startRow = 15)
    checkEquals(myData1[15:100,], myData2, check.attributes = FALSE)

    # Try skipping the first block entirely
    myData2 <- rxDataStep(inData = myData1, startRow = 25)
    checkEquals(myData1[25:100,], myData2, check.attributes = FALSE)

    # Now try combining with a total number of rows
    myData2 <- rxDataStep(inData = myData1, startRow = 25, 
        numRows = 35)
    checkEquals(myData1[25:59,], myData2, check.attributes = FALSE)

    # Again, but less than 1 block
    myData2 <- rxDataStep(inData = myData1,  startRow = 25, 
        numRows = 7)
    checkEquals(myData1[25:31,], myData2, check.attributes = FALSE)

    # What happens if we have rowSelection  - it will happen after the read
    myData2 <- rxDataStep(inData = myData1, , startRow = 25, 
        numRows = 40, rowSelection = ylogical == TRUE)
    myData1small <- myData1[25:64,]
    checkEquals(subset(myData1small, ylogical == TRUE), myData2, check.attributes = FALSE)

    # Also check remove missings
    myData2 <- rxDataStep(inData = myData1, startRow = 25, 
        numRows = 40, removeMissings = TRUE )
    myData1small <- myData1[25:64,]
    checkEquals(na.omit(myData1small), myData2, check.attributes = FALSE)

}

"test.rxDataStep.args.startBlock.rowsPerRead" <- function()
{
    tempFile <- file.path(tempdir(),".rxDS7TempTest.xdf")
  if (file.exists(tempFile)) file.remove(tempFile)
        
    rxDataStep(inData = iris, outFile = tempFile,
      startBlock = -1, rowsPerRead = 50)
    
    fileInfo <- rxGetInfo( tempFile )
  if (rxIsExpressEdition())
  {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
      checkEquals(fileInfo$numBlocks, 3)
  }

  if (file.exists(tempFile)) file.remove(tempFile)	
}

"test.rxDataStep.args.transforms.constant"<-function(){

    inFile <- file.path(system.file("SampleData", package = "RevoScaleR"), "claims.xdf")
    df <- rxDataStep(inFile) 
    
    dfTransforms1 <- rxDataStep(inFile, transforms = list( foo = 0 )) 
    varNames <- names(df)
    varDim <- length(df[[varNames[1]]])
    df$foo <- rep(0, varDim)
    checkEquals(df, dfTransforms1)

    dfTransforms2 <- rxDataStep(inFile, transforms = list( foo = 0, eagle = 38 * cost ))  
    df$eagle <- 38*df$cost
    checkEquals(df, dfTransforms2)

}

"test.rxDataStep.args.transforms.NULL" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile(pattern = ".rxDS8TempTest", fileext = ".xdf")
  origNames <- rxGetVarNames(test100r5b)
    
    outData1 <- rxDataStep(inData = test100r5b, outFile = outFile,
        transforms = list(xnum1 = NULL), 
    varsToKeep = c("xnum1", "xnum2","xint1", "xfac1"), overwrite=TRUE)
  
    outData2 <- rxDataStep(inData = test100r5b, 
        transforms = list(xnum1 = NULL) 
    )	
  
  varNames1 <- rxGetVarNames(outData1)
  varNames2 <- rxGetVarNames(outData2)
  checkEquals(varNames1, c("xnum2", "xint1", "xfac1"))
  
  newNames <- setdiff(origNames, "xnum1")
  checkEquals(varNames2, newNames)
    
  myTransformFunc <- function( dataList )
  {
    dataList$xnum1 <- NULL
    return( dataList )
  }
  file.remove(outFile)
    outData3 <- rxDataStep(inData = test100r5b, outFile = outFile,
       transformFunc = myTransformFunc, overwrite=TRUE)	
    
  checkEquals(rxGetVarNames(outData3), newNames)
    file.remove( outFile)
}

"test.rxDataStep.args.transforms.rxNumRows" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile(pattern = ".rxDS9TempTest", fileext = ".xdf")
    
    rxDataStep(inData = test100r5b, outFile = outFile, 
        transforms = list(
            rowNum = .rxStartRow : (.rxStartRow +.rxNumRows - 1)),
        overwrite = TRUE)
    myData <- rxReadXdf(outFile)
    checkEquals(myData$rowNum, 1:100)
    
    file.remove( outFile)
}

"test.rxDataStep.value" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")
    outFile3 <- tempfile(pattern = ".rxTest3", fileext = ".xdf")
    outDS3 <- RxXdfData( outFile3 )
    
    # Output data frame
    outData1 <- rxDataStep( inData = inFile, transforms = list(xnew = xnum1*2))   
    # Output xdf
    outData2 <- rxDataStep( inData = inFile, outFile = outFile2, 
        transforms = list(xnew = xnum1*2), overwrite = TRUE)
    # Output xdf data source
    outData3 <- rxDataStep( inData = inFile, outFile = outDS3, 
        transforms = list(xnew = xnum1*2), rowsPerRead = 30, overwrite = TRUE )

    sumFormula <-  ~y + ybinint + xnum2miss + xint1 + xnew
    outSum1 <- rxResultsDF(rxSummary(sumFormula, data = outData1))
    
    outSum2 <- rxResultsDF(rxSummary(sumFormula, data = outData2))
    
    outSum3 <- rxResultsDF(rxSummary(sumFormula, data = outData3))
    
    checkEquals(outSum1, outSum2)
    checkEquals(outSum1, outSum3)
    
    file.remove(outFile2)
    file.remove(outFile3)
}

 "test.rxDataStep.args.inData.missing" <- function()
{
    outFile <- tempfile(pattern = ".rxTest1", fileext = ".xdf")
    if (file.exists(outFile))
  {
    file.remove(outFile)
  }
      
    # Output data frame
    outData1 <- rxDataStep( numRows = 50, rowsPerRead = 10,
        transforms = list(xnew = .rxStartRow:(.rxStartRow + .rxNumRows - 1)))   
    # Output xdf   
    outData2 <- rxDataStep( outFile = outFile, numRows = 50, rowsPerRead = 10,
        transforms = list(xnew = .rxStartRow:(.rxStartRow + .rxNumRows - 1)))
  
    # Check to make sure the automatic variable contains row numbers
  outData3 <- rxDataStep(inData = outData2)
  checkEquals(outData3$.rxRowNums, outData3$xnew)
  checkEquals(outData1$.rxRowNums, outData3$.rxRowNums)
  checkEquals(outData1$.rxRowNums, outData1$xnew)
  
  file.remove(outFile)
}

# Test fails on older Windows platforms RA-11437
#"test.rxDataStep.args.inData.otherDataSources" <- function()
#{
    #
    #xdfOutFileName <- tempfile(pattern = "rxDataStepNonXdf", fileext = ".xdf")
    #
    #xdfFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
    #rxDataStep(inData = xdfFileName, outFile = xdfOutFileName, overwrite = TRUE,
         #transforms = list(xintmult = xint1*xint1))
    #varInfoXdf <- rxGetVarInfo(xdfOutFileName)
    #print("varInfoXdf:")
    #print(varInfoXdf)
    #
    ## Text data source
    #textFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")
    #
    #haveFile <- file.exists(textFileName)
    #print("Have text file?")
    #print( haveFile)
    #
    #myTextDS <- RxTextData(file = textFileName,
      #colClasses = c(ylogical = "logical", ybinint ="integer", xint1 = "integer", xfac1 = "factor",
         #xfac2 = "factor", zfac = "factor"), rowsPerRead = 20, useFastRead = FALSE)  
    #rxDataStep(inData=myTextDS, outFile = xdfOutFileName, overwrite = TRUE,
        #transforms = list(xintmult = xint1*xint1)) 
    #varInfoText <- rxGetVarInfo(xdfOutFileName)
#
    #print("varInfoText")
    #print(varInfoText)
    ##checkEquals(varInfoXdf, varInfoText)
    #
    #sasFileName <-
      #file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")    
    #mySasDS <- RxSasData(file = sasFileName, 
         #colClasses = c(YLOGICAL = "logical", YBININT ="integer", XINT1 = "integer", XFAC1 = "factor",
         #XFAC2 = "factor", ZFAC = "factor", Y = "float32", Y2 = "float32", XNUM1 = "float32",
         #XNUM2 = "float32", XNUM2MISS = "float32"), 
         #rowsPerRead = 20)  
    #
    #rxDataStep(inData=mySasDS, outFile = xdfOutFileName, overwrite = TRUE,
        #transforms = list(xintmult = XINT1*XINT1)) 
    #
    #varInfoSas <- rxGetVarInfo(xdfOutFileName)
    #newNames <- names(varInfoSas)
    #newNames <- tolower(newNames)
    #newNames[11] <- "z.char1"
    #names(varInfoSas) <- newNames
    #
    #print("varInfoSas:")
    #print(varInfoSas)
    ##checkEquals(varInfoXdf, varInfoSas)
    #
    #unlink(xdfOutFileName)
#}


 "test.rxDataStep.args.inData.missing.example" <- function()
{
    outFile <- tempfile(pattern = ".rxTest1", fileext = ".xdf")
    if (file.exists(outFile))
  {
    file.remove(outFile)
  }
      
    # Output data source
    outDS <- rxDataStep( outFile = outFile, numRows = 100, rowsPerRead = 20,
        transforms = list(
            x = (.rxRowNums - 50)/25,
            pnormx = pnorm(x),
            dnormx = dnorm(x))) 
        
    outData1 <- rxDataStep(outDS)
    
    .rxRowNums = 1:100
    x = (.rxRowNums - 50)/25
    pnormx = pnorm(x)
    dnormx = dnorm(x)
    outData2 <- data.frame(.rxRowNums, x, pnormx, dnormx)
  
    checkEquals(outData1, outData2)
  
  file.remove(outFile)
}

"test.rxDataStep.args.inData.dataframe.list" <- function()
{
    model1 <- charToRaw(paste(letters[1:2], collapse=""))
    model2 <- charToRaw(paste(LETTERS[24:26], collapse=""))

    dfList <- data.frame(
        key = c("model1", "model2"), 
        value = I(list(m1 = model1, m2 = model2)))
    
    dfListOut <- rxDataStep(dfList) 
    checkEquals(dfList, dfListOut, check.attributes = FALSE)
    
    outFile <- tempfile(pattern = ".rxDataStepTest", fileext = ".xdf")
    on.exit(file.remove(outFile), add = TRUE)
    xdfOut <- rxDataStep(dfList, outFile, overwrite = TRUE)
    dfListIn <- rxDataStep(outFile)
    checkEquals(dfList, dfListIn, check.attributes = FALSE)
    
    dfList2 <- data.frame(
        key = c("model1", "model2"), 
        value = I(list(m1 = model1, m2 = "model2")))
    checkException(dfListOut2 <- rxDataStep(dfList2))
    
    dfList3 <- data.frame(
        key = c("model1", "model2"), 
        value = I(list(m1 = "model1", m2 = model2)))
    checkException(dfListOut3 <- rxDataStep(dfList3))
    
    ###################################
    dfList4 <- data.frame(key = c("model1", "model2"))
    dfList4$value <- list(m1 = model1, m2 = model2)
    
    dfListOut4 <- rxDataStep(dfList4) 
    checkEquals(dfList4, dfListOut4, check.attributes = FALSE)
    
    xdfOut4 <- rxDataStep(dfList4, outFile, overwrite = TRUE)
    dfListIn4 <- rxDataStep(outFile)
    checkEquals(dfList4, dfListIn4, check.attributes = FALSE)
}

"test.rxDataStep.args.inData.colnames" <- function()
{
    #Bug 7114:rxDataStep failed for some inData data sources with specific column names
       
    #library(RevoScaleR)
    dat <- data.frame(Summary = 1:3, Score = 4:6)
    res <- try(rxDataStep(dat))
    
    checkTrue(!inherits(res,"try-error"))
}

 "test.rxDataSet.args.returnTransformObjects" <- function()
{
    
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  inData <- rxReadXdf(inFile)  

  myFun <- function( dataList )
  {
        chunkSum <- sum(dataList$xint1)
        if (!.rxIsTestChunk)
        {
        sumx <<- sumx + chunkSum
        } 
        else
        {
            print("Processing test chunk")
        }
        
        print(sumx)
    return( NULL )
  }
    
    outInfo <- capture.output(newTransformObjects <- rxDataStep( inData = inFile, 
        transformFunc = myFun, returnTransformObjects = TRUE,
      transformVars = c("xint1"), transformObjects = list(sumx = 0), reportProgress = 0))

    checkEquals(newTransformObjects[["sumx"]], sum(inData$xint1))
    
    # Should have output for each block; test block not sent
    if (rxIsExpressEdition())
    {
        checkEquals(length(outInfo), 1) 
    } else
    {   
        checkEquals(length(outInfo), 5)
    }
    
    # A slightly more interesting example
    
    myCenter <- 40
    myFun <- function( dataList )
    {
        chunkSumDev <- sum((dataList$y - toMyCenter)^2,  na.rm = TRUE)
        print(chunkSumDev)
        toTotalSumDev <<- toTotalSumDev + chunkSumDev
        return(NULL)
    }
    newTransformVals <- rxDataStep(inData = inFile, transformFunc = myFun,
        transformObjects = list( 
            toMyCenter = myCenter,
            toTotalSumDev = 0),
        transformVars = "y", returnTransformObjects = TRUE)
    
    myData <- rxReadXdf(inFile)
    checkEquals( newTransformVals[["toTotalSumDev"]], sum( (myData$y - 40)^2, na.rm = TRUE))
}

"test.rxDataStep.args.transformFunc.rowSelection" <- function()
{
  # Fix to RA-5344
  myData <- data.frame(x = 1:20, y = 1:20) 
  myTransform <- function( dataList ) 
  { 
    dataList$z <- 1:length(dataList[[1]]) 
    return( dataList) 
  } 

  myData1 <- rxDataStep(inData = myData, varsToKeep = "x", 
    transformFunc = myTransform, 
    rowSelection = z < 8) 
  checkEquals(myData1$x, myData1$z)
  checkEquals(nrow(myData1), 7)
  
  # Note that because z is used in rowSelection, it
  # is kept in the data set
  checkEquals(ncol(myData1), 2)
    
}
"test.rxDataStep.args.outFile.text" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74579: Unit test test.rxDataStep.args.append.differentNumCols fails to return ERROR in output when running inside SPEES")
    
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile(pattern = ".rxDSTempTest1", fileext = ".csv")
    
    outDS <- RxTextData(outFile, delimiter = ",", useFastRead = TRUE)
    
    rxDataStep(inData = test100r5b, outFile = outDS)
    testInDF <- rxReadXdf(test100r5b)
    testOutDF <- rxImport(outDS, stringsAsFactors = TRUE)
    checkEquals(testInDF, testOutDF, tolerance=1e-6)
    
    # Check for overwrite
    # This should fail because file exists
    o1 <- capture.output( res <- try(   
        rxDataStep(inData = test100r5b, outFile = outDS)
        ))

    checkTrue(inherits(res,"try-error"))
    errorMsg <- "Specify 'overwrite = TRUE' to overwrite"
    #Bug 76414: capture.output fails to capture entire output when run inside SPEES
    #checkTrue(grepl( errorMsg, o1[2]))
    
    # Specifying overwrite should work 
    rxDataStep(inData = test100r5b, outFile = outDS, overwrite = TRUE)    
    if (file.exists(outFile)) file.remove(outFile)
    
    # Try outDataPath
    origOutDataPath <- rxGetOption("outDataPath")
    rxOptions(outDataPath = tempdir())
    on.exit(rxOptions(outDataPath = origOutDataPath), add = TRUE)
    outFile <- ".rxDSTempTest2.csv"
    fullOutFile <- file.path(tempdir(), outFile)
    outDS <- RxTextData(outFile, delimiter = ",", useFastRead = TRUE)

    rxDataStep(inData = test100r5b, outFile = outDS)
    checkTrue(file.exists(fullOutFile))
        
    if (file.exists(fullOutFile)) file.remove(fullOutFile)
    
}

"test.rxDataStep.args.varsToKeep.outFile.text" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    testTxt <-  tempfile(pattern = ".rxTempXdfToText", fileext = ".txt")
    testXdfNew <- tempfile(pattern = ".rxTempXdfToText", fileext = ".xdf")

    if (file.exists(testTxt)) file.remove(testTxt)
    if (file.exists(testXdfNew)) file.remove(testXdfNew)
    
    # Test keeping first, middle, and last variable
    testTxtDS <- RxTextData(testTxt, delimiter = ",")
    rxDataStep(inData = testXdf, outFile = testTxtDS,
                varsToKeep = c("y", "xnum2", "xfac2"), blocksPerRead = 1,
                reportProgress = 0)
    # Read it back in and compare
    rxImport(inData = testTxt, outFile = testXdfNew, stringsAsFactors = TRUE,
                reportProgress = 0)

    infoObj <- rxGetInfo(testXdf, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdf)
  checkEquals(infoObj$fileName, normalizePath(testXdf))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 5)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(testXdfNew, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdfNew)
  checkEquals(infoObj$fileName, normalizePath(testXdfNew))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"),  tolerance=1e-6 )
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"), tolerance=1e-6 )
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    # Cleanup
    file.remove(testTxt)
    file.remove(testXdfNew) 
}

"test.rxDataStep.args.outFile.NULL.empty" <- function()
{
  # Test the return of a NULL data frame because of no rows
  dat <- data.frame(
    x = letters,
    d = as.Date("2010-01-01") + 1:26
  )

  outData <- rxDataStep(dat, rowSelection = d > as.Date("2010-12-15"))
  # Should give warning, but not error
  checkTrue(is.null(outData))

  #tempOutFile <- tempfile(pattern = "rxEmpty", fileext = ".xdf")
  #rxDataStep(dat, rowSelection = d > as.Date("2010-12-15"), outFile = tempOutFile)
}

"test.rxDataStep.args.transforms.variable" <- function(){

  inFile <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  df <- rxDataStep(inFile)
  
  constant <- 38
  myTransform <- function(cost){
    constant <- get("constant", parent.frame())
    constant * cost
  }
  
  xForm <- function(dataList) {
    dataList $eagle = constant * dataList$cost
    return(dataList)
  }
  
  dfTransforms <- rxDataStep(inData = inFile, 
                             overwrite = TRUE,
                             transformObjects = list(constant = constant, myTransform = myTransform),
                             transforms = list(eagle = myTransform(cost)) 
                  )
  
  dfTransformFunc <- rxDataStep(inData = inFile,
                                overwrite = TRUE,
                                transformObjects = list(constant = constant),
                                transformFunc = xForm)
  
  df$eagle <- constant * df$cost
  checkEquals(df, dfTransforms)
  checkEquals(df, dfTransformFunc)
  checkException(result <- rxDataStep(inData = inFile, 
                                      overwrite = TRUE,
                                      transformObjects = list(myTransform = myTransform),
                                      transforms = list(eagle = myTransform(cost)))
  )
  checkException(result <- rxDataStep(inData = inFile, 
                                      overwrite = TRUE,
                                      transformObjects = list(constant = constant),
                                      transforms = list(eagle = myTransform(cost)))
  )
}

"test.rxDataStep.args.transformFunc.varType.single" <- function() {

  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  inData <- rxDataStep(inFile)
  convertToNumeric <- function(dataList)
  {
    dataList$ybinint = as.numeric(dataList$ybinint)
    return(dataList)
  }
  
  resData1 <- rxDataStep(inData = inData, 
                         transformFunc = convertToNumeric, overwrite = TRUE)
  
  resData2 <- rxDataStep(inData = inData,
                         transforms = list(ybinint = as.numeric(ybinint)), overwrite =TRUE)
  
  varInfo1 <- rxGetVarInfo(resData1)
  checkEquals(varInfo1$ybinint$storage, "float64")
  varInfo2 <- rxGetVarInfo(resData2)
  checkEquals(varInfo2$ybinint$storage, "float64")
  inData$ybinint <- as.numeric(inData$ybinint)
  checkEquals(resData1$ybinint, inData$ybinint)
  checkEquals(resData2$ybinint, inData$ybinint)
}

"test.rxDataStep.args.transformFunc.new.data" <- function(){

  inputFile <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  fileInfo <- rxGetVarInfo(inputFile)
  myTransformFunc <- function(dataList) {
    outList <- list()
    outList$cost <- max(na.omit(dataList$cost))
    outList$number <- max(dataList$number)
    return(outList)
  }
  
  data <- rxDataStep(inData = inputFile,
                     transformFunc = myTransformFunc)
  checkEquals(data$cost, fileInfo$cost$high)
  checkEquals(data$number, fileInfo$number$high)
}

"test.rxDataStep.args.transformFunc.varType" <- function()
{
  
  myInt <- as.integer(c(5,4,3))
  myString <- c("5.123456789", "4.123456789", "3.123456789")
  myData <- data.frame(myInt = myInt, myString = myString, stringsAsFactors = FALSE)
  tempFile <- tempfile(pattern = "rxDataStepT", fileext = ".xdf")
  convertToNumeric <- function(dataList)
  {
    newData <- lapply(dataList, as.numeric)
    return(newData)
  }
  
  
  resFile <- rxDataStep(inData = myData, outFile = tempFile,
                        transformFunc = convertToNumeric, overwrite = TRUE)
  
  varInfo <- rxGetVarInfo(tempFile)
  checkEquals(varInfo$myInt$storage, "float64")
  checkEquals(varInfo$myString$storage, "float64")
  resData <- rxDataStep(tempFile)
  baser <- lapply(myData, as.numeric)
  checkEquals(resData$myInt, baser$myInt)
  checkEquals(resData$myString, baser$myString)
  file.remove(tempFile)
  
}


"test.rxDataStep.args.outFile.composite" <- function()
{
  # See TFS 6292882
    #DEACTIVATED("Awaiting final implementation of append rows with composite files on Linux.")
    # Setup directory/file names and data sources
    #claimsSingle <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    #claimsComposite <- file.path(rxGetOption("sampleDataDir"), "claimsXdf")
    #
    #claimsOutComposite <- tempfile(pattern = "rxCompositeOut", fileext = "")
    #claimsOutCompositeDS <- RxXdfData(file = claimsOutComposite, createCompositeSet = TRUE, 
        #blocksPerCompositeFile = 2L)
    #
    ## Input data is single xdf, outFile is composite
    #rxDataStep(inData = claimsSingle, outFile = claimsOutCompositeDS)
    #singleInfo <- rxGetInfo(claimsSingle)
    #outCompositeInfo <- rxGetInfo(claimsOutComposite)
    #checkEquals(singleInfo$numRows, outCompositeInfo$numRows)
    #checkEquals(outCompositeInfo$numCompositeFiles, 1L)
  #outSingleDF <- rxReadXdf(claimsSingle)
  #outCompositeDF <- rxReadXdf(claimsOutCompositeDS)
  #checkEquals(outSingleDF, outCompositeDF)
    #unlink(claimsOutComposite, recursive = TRUE)
    #
    ## Input data and output are both composite
    #rxDataStep(inData = claimsComposite, outFile = claimsOutCompositeDS)
    #inCompositeInfo <- rxGetInfo(claimsComposite)
    #outCompositeInfo <- rxGetInfo(claimsOutComposite)
    #checkEquals(inCompositeInfo$numRows, outCompositeInfo$numRows)
    #checkEquals(inCompositeInfo$numBlocks, outCompositeInfo$numBlocks)
    #checkEquals(outCompositeInfo$numCompositeFiles, outCompositeInfo$numBlocks/2L) # blocksPerComposite
      #
    ## Append rows
    #rxDataStep(inData = claimsComposite, outFile = claimsOutCompositeDS, 
        #append = "rows", transforms = list(RowNum = as.integer(RowNum + 128)))
    #fileInfo <- rxGetInfo(claimsOutComposite)
    #checkEquals(fileInfo$numCompositeFiles, 16)
    #checkEquals(fileInfo$numRows, 256)
    #checkEquals(fileInfo$numBlocks, 32)
    #checkEquals(fileInfo$numVars, 6)
    #varInfo <- rxGetVarInfo(claimsOutComposite)
    #checkEquals(varInfo$RowNum$low, 1)
    #checkEquals(varInfo$RowNum$high, 256)
    #
    #claimsAppend1DF <- rxReadXdf(claimsOutCompositeDS)
    #checkEquals(nrow(claimsAppend1DF), 256L)
    #
    #unlink(claimsOutComposite, recursive = TRUE)
    #
    ## Data step with transforms and varsToKeep
    #
    #rxDataStep(inData = claimsComposite, 
        #transforms = list(cost10 = cost*10, RowNum = as.integer(RowNum)),
        #varsToKeep = c("RowNum", "cost", "age", "type"), rowSelection = cost > 12, 
        #outFile = claimsOutCompositeDS)
    #
    #claimsDF <- rxDataStep(inData = claimsSingle, 
        #transforms = list(cost10 = cost*10, RowNum = as.integer(RowNum)),
        #varsToKeep = c("RowNum", "cost", "age", "type"), rowSelection = cost > 12 )
    #
    #inCompositeInfo <- rxGetInfo(claimsDF)
    #outCompositeInfo <- rxGetInfo(claimsOutComposite)
    #checkEquals(inCompositeInfo$numRows, outCompositeInfo$numRows)
    #checkEquals(inCompositeInfo$numVars, outCompositeInfo$numVars)
    #checkEquals(outCompositeInfo$numCompositeFiles, 8L) 
    #
    ## Append rows
    ##  Check to make sure error is given if variables don't match up when appending rows
    #res <- try(
        #rxDataStep(inData = claimsComposite, outFile = claimsOutCompositeDS,
            #append = "rows")
        #)
    #checkTrue(is(res, "try-error")) 
    ## Make sure original file isn't corrupted
    #outClaimsDF <- rxReadXdf(claimsOutCompositeDS) 
    ## On Linux, get
    ## The file '/tmp/RtmpZp5js4/rxCompositeOut319a7d8e9b5e/data/rxCompositeOut319a7d8e9b5e_9.xdfd' is empty.
#
    #checkEquals(claimsDF, outClaimsDF)
            #
    #rxDataStep(inData = claimsComposite, outFile = claimsOutCompositeDS,
        #append = "rows",
        #transforms = list(cost10 = cost*10, RowNum = as.integer(RowNum + 128) ),
        #varsToKeep = c("RowNum","cost", "age", "type"), rowSelection = cost > 12)    
    #
    #appendFileInfo <- rxGetInfo(claimsOutComposite)
    #checkEquals(appendFileInfo$numCompositeFiles, outCompositeInfo$numCompositeFiles * 2)
    #checkEquals(appendFileInfo$numRows, outCompositeInfo$numRows * 2)
    #checkEquals(appendFileInfo$numBlocks, outCompositeInfo$numBlocks * 2)
    #checkEquals(appendFileInfo$numVars, outCompositeInfo$numVars)
    #appendVarInfo <- rxGetVarInfo(claimsOutComposite)
    #checkEquals(appendVarInfo$RowNum$low, 1)
    #checkEquals(appendVarInfo$RowNum$high, 256)
#
    #claimsOutAppendDF <- rxReadXdf(claimsOutCompositeDS)
    #checkEquals(claimsOutAppendDF[123:244, "cost"], claimsDF[,"cost"])
    #
    #unlink(claimsOutComposite, recursive = TRUE)    
}

"test.rxDataStep.RxSpssData.args.RxDataSource" <- function()
{

  # Test that information in data source object is being respected
  spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
    spssDS <- RxSpssData(spssFileName)    
    testData <- rxDataStep(spssDS, overwrite = TRUE, reportProgress = 4, labelsAsLevels = FALSE)
    
  varInfo <- rxGetVarInfo( testData )
  checkEquals(varInfo$age$varType, "numeric")
  checkEquals(varInfo$type$varType, "numeric")
    
    # Set inside of a function
    myTestFunction <- function( useLabelsAsLevels, spssDS )
    {
        rxDataStep(spssDS, overwrite = TRUE, reportProgress = 0, labelsAsLevels = useLabelsAsLevels)
    }
    
    myLabels <- FALSE
    testData <- myTestFunction(useLabelsAsLevels = myLabels, spssDS)
  varInfo <- rxGetVarInfo( testData )
  checkEquals(varInfo$age$varType, "numeric")
  checkEquals(varInfo$type$varType, "numeric")    
        
  
  # Since labelsAsInfo is by default true, that information should be in varInfo
  checkEquals(varInfo$age$valueInfoCodes, as.character(1:4))
  ageLabels <- c("17-20", "21-24", "25-29", "30-34")
  checkEquals(varInfo$age$valueInfoLabels, ageLabels)

    # Create an XDF data file path
    xdfFileName <- file.path(tempdir(), "claimsIntImport2.xdf")
    if (file.exists(xdfFileName)) file.remove(xdfFileName)

    # Use rxDataStep; notice that factor variables are created for any
    # SPSS variables with value labels
    
    rxDataStep(spssDS, xdfFileName, overwrite = TRUE, reportProgress = 4)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$cost$varType, "numeric")
    checkEquals(varInfo$type$varType, "factor")

    # Redo, storing some variables as integer instead of
    # numeric

    spssColClasses <- c(RowNum = "integer", cost = "integer", 
      number = "integer")
  spssDS <- RxSpssData( spssFileName, colClasses = spssColClasses)
    rxDataStep(spssDS, xdfFileName, 
       overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNum$varType, "integer")
    checkEquals(varInfo$cost$varType, "integer")

    # Redo, changing re-specifing the strings used
    # for factor levels

    spssColInfo <- list(
      RowNum = list(type="integer", newName = "RowNumber"),
      cost = list(type="integer", newName = "AutoCost", description = "Cost of Auto"),
      number = list(type="integer", newName = "Number", description = "Number of cars"),
      age = list(
      levels = c("1", "2", "3", "4"),
      newLevels = c("New", "Old", "Older", "Oldest")))
  spssDS <- RxSpssData(spssFileName, colInfo = spssColInfo)
    rxDataStep(spssDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNumber$varType, "integer")
    checkEquals(varInfo$AutoCost$varType, "integer")
    checkEquals(varInfo$AutoCost$description, "Cost of Auto")
    checkEquals(varInfo$age$levels, c("New", "Old", "Older", "Oldest"))
  
  renamedVarsToKeepVector <- c("RowNumber","AutoCost","Number", "age", "type") 
    spssDS <- RxSpssData(spssFileName, colInfo = spssColInfo)
    rxDataStep(spssDS, xdfFileName, varsToKeep = renamedVarsToKeepVector,
    overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )

    checkEquals(names(varInfo), renamedVarsToKeepVector)
    checkEquals(varInfo$type$varType, "factor")
    checkEquals(varInfo$RowNumber$varType, "integer")
    checkEquals(varInfo$AutoCost$varType, "integer")
    checkEquals(varInfo$AutoCost$description, "Cost of Auto")
    checkEquals(varInfo$age$levels, c("New", "Old", "Older", "Oldest"))
  
    if (file.exists(xdfFileName)) file.remove(xdfFileName)
}


"test.rxDataStep.args.xdfCompressionLevel" <- function()
{
    origXdfFile <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall.xdf")
    compressedXdf <- tempfile(pattern = "compressed", fileext = ".xdf")
  myCompressedData5 <- rxDataStep(inData = origXdfFile, outFile = compressedXdf, xdfCompressionLevel = 5, overwrite = TRUE)
    
    origSize <- file.info(origXdfFile)$size 
    compressedSize5 <- file.info(compressedXdf)$size
    
    checkTrue(origSize > compressedSize5)
    
    myCompressedData4 <- rxDataStep(inData = origXdfFile, outFile = compressedXdf, xdfCompressionLevel = 4, overwrite = TRUE)
    compressedSize4 <- file.info(compressedXdf)$size
    
    checkTrue(compressedSize4 > compressedSize5)
    
    file.remove(compressedXdf)
}



"test.rxDataStep.data.specialChars" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74674: SQLPAL: unit tests test.rxDataStep.data.specialChars fails in sqlpar when sourcing file with special characters")

  if(Sys.info()["nodename"] == "AT03-cent5")  DEACTIVATED("Pending fix of RA-11003")
  
  if (isTRUE(file.exists("/etc/lsb-release"))) {
        DEACTIVATED("Test fails because of file encoding problems on Ubuntu - Bug 40062.")
  }

  source(file.path(system.file("unitTests", package="RevoScaleR"), "common", "specialChars.R"), encoding=if(.Platform$OS.type=="unix")"latin1" else "")
  rxDataStep.data.specialChars()
}

"test.rxDataStep.args.transforms.highlow" <- function()
{
  # See Issue 6208264
  tempXdf <- tempfile(pattern = "highlow", fileext = ".xdf")
  myData <- data.frame(x = 1:100, y = as.numeric(1:100))
  rxDataStep(inData = myData, outFile = tempXdf)
  # Two blocks
  rxDataStep(inData = myData, outFile = tempXdf, append = "rows")

  tranFunc <- function( dataList )
  {
    dataList$x5 <- dataList$x1
    attr(dataList$x5, ".rxSetLowHigh") <- c(-100, 200)
    return ( dataList)
  }
  
  rxDataStep(inData = tempXdf, outFile = tempXdf, transforms = 
    list(x1 = as.numeric(x + 10),
             x2 = as.integer(x+10),
       x3 = as.factor(x+10),
       x4 = as.character(x),
        y1 = 10 + y/2),
         transformFunc = tranFunc, overwrite = TRUE)
  varInfo <- rxGetVarInfo(tempXdf)
  checkEquals(varInfo$x$low, 1)
  checkEquals(varInfo$x$high, 100)
  checkEquals(varInfo$x1$low, 11)
  checkEquals(varInfo$x1$high, 110)
  checkEquals(varInfo$x2$low, 11)
  checkEquals(varInfo$x2$high, 110)
  checkEquals(varInfo$x3$low, 1)
  checkEquals(varInfo$x3$high, 100)
  checkEquals(varInfo$y1$low, 10.5)
  checkEquals(varInfo$y1$high, 60)
    fileInfo <- rxGetInfo(tempXdf)
    checkEquals(fileInfo$numRows, 200)
  file.remove(tempXdf)
    
    # Different output file
  tempXdfOut <- tempfile(pattern = "highlowOut", fileext = ".xdf")
    # Recreate input data set
  rxDataStep(inData = myData, outFile = tempXdf, overwrite = TRUE)
  # Two blocks
  rxDataStep(inData = myData, outFile = tempXdf, append = "rows")   
    
  rxDataStep(inData = tempXdf, outFile = tempXdfOut, transforms = 
    list(x1 = as.numeric(x + 10),
             x2 = as.integer(x+10),
       x3 = as.factor(x+10),
       x4 = as.character(x),
        y1 = 10 + y/2),
         transformFunc = tranFunc, overwrite = TRUE)
  varInfo <- rxGetVarInfo(tempXdfOut)
  checkEquals(varInfo$x$low, 1)
  checkEquals(varInfo$x$high, 100)
  checkEquals(varInfo$x1$low, 11)
  checkEquals(varInfo$x1$high, 110)
  checkEquals(varInfo$x2$low, 11)
  checkEquals(varInfo$x2$high, 110)
  checkEquals(varInfo$x3$low, 1)
  checkEquals(varInfo$x3$high, 100)
  checkEquals(varInfo$y1$low, 10.5)
  checkEquals(varInfo$y1$high, 60)
    fileInfo <- rxGetInfo(tempXdfOut)
    checkEquals(fileInfo$numRows, 200)    
  file.remove(tempXdf)     
  
}

"test.rxDataStep.args.append.rows.composite" <- function()
{
# See TFS 6298213

    compXdf1 <- tempfile( pattern = "compAppendXdf1", fileext = "")
    on.exit(if(file.exists(compXdf1)) unlink(compXdf1, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf1DataSource <- RxXdfData(compXdf1, createCompositeSet = TRUE)

  # Create a composite xdf using append rows
  # convert the first datafile
  sampleDir <- rxGetOption("sampleDataDir")
  csvFile <- RxTextData(paste0(sampleDir, "/mortDefaultSmall", 2000, ".csv"), delimiter = ",")
  rxDataStep(csvFile, compXdf1DataSource, overwrite = TRUE)

  # now append on the remainder
  for(i in 2001: 2005)
  {
    csvFile <- RxTextData(paste0(sampleDir, "/mortDefaultSmall", i, ".csv"), delimiter = ",")
    rxDataStep(csvFile, compXdf1DataSource, append = "rows", overwrite = TRUE)
  }

    # Create a composite xdf using 1 call to rxDataStep
  compXdf2 <- tempfile( pattern = "compAppendXdf2", fileext = "")
    on.exit(if(file.exists(compXdf2)) unlink(compXdf2, recursive = TRUE, force = TRUE), add = TRUE)
    compXdf2DataSource <- RxXdfData(compXdf2, createCompositeSet = TRUE)
   

  csvPath <- paste0(getwd(), "/testCsv", sep = "")
  dir.create(csvPath)
  on.exit(if(file.exists(csvPath)) unlink(csvPath, recursive = TRUE, force = TRUE), add = TRUE)
  for(i in 2000: 2005)
  {	
    csvFileName <- paste0(sampleDir, "/mortDefaultSmall", i, ".csv")
    file.copy(csvFileName, csvPath)
  }
    #csvPath <- "C:/TEMP/mortDefaultSmall" # folder contains 10 files of .Csv

    csvFile <- RxTextData(csvPath, rowsPerRead=10000, delimiter = ",")

    rxDataStep(csvFile, compXdf2DataSource, overwrite = TRUE)

    info1 <- rxGetInfo(compXdf1DataSource)
  info2 <- rxGetInfo(compXdf2DataSource)
  varInfo1 <- rxGetVarInfo(compXdf1DataSource)
  varInfo2 <- rxGetVarInfo(compXdf2DataSource)
  
  linModObj <- rxLinMod(default~creditScore, data = compXdf1DataSource)

  # This works
    tempXdf1 <- tempfile( pattern = "tempXdf1", fileext = ".xdf")
    rxPredict(linModObj, data = compXdf2DataSource, 
    outData = tempXdf1, overwrite = TRUE)
  varInfo1 <- rxGetVarInfo(tempXdf1)
  
  # The file created with append used to get extra variables
  rxPredict(linModObj, data = compXdf1DataSource, 
    outData = tempXdf1, overwrite = TRUE)
  varInfo2 <- rxGetVarInfo(tempXdf1)
  
    checkEquals(varInfo1, varInfo2)
    file.remove(tempXdf1)
  
 }

#
#7190313: rxDataStep/RxTextData options to set NA to ‘’, not output double-quotes, and not output headers
# Convert from XDF to CSV with rxDataStep using 3 parameters
"test.rxDataStep.args.firstRowIsColNames.missingValueString.quoteMark" <- function()
{
  myData <- data.frame(textVar = c("Hello", "hi", "goodby", "hello"),
              intVar = as.integer(c(1:2, NA, 4)))
          
  dataXdf <- tempfile( pattern = "textToXdf_", fileext = ".xdf")
  rxDataStep(inData = myData, outFile = dataXdf, overwrite = TRUE)

  outCsvFile <- tempfile( pattern = "outCsvFile_", fileext = ".csv")
  myTextDS <- RxTextData(file = outCsvFile, 
    missingValueString = "", 
    firstRowIsColNames = FALSE,
    quoteMark = "")
  rxDataStep(inData = dataXdf, outFile = myTextDS, overwrite = TRUE)
  dataCsv <- read.csv(outCsvFile, header=FALSE)
    checkEquals(myData$textVar, dataCsv$V1)
    checkEquals(myData$intVar, dataCsv$V2)
  
    file.remove(dataXdf)
    file.remove(outCsvFile)
}

"test.rxDataStep.args.inData.composite.outFile.singleXdf.rowsPerRead.numRows" <- function()
{
    sampleDir <- rxGetOption("sampleDataDir")
    inData  <- RxXdfData(file.path(sampleDir, "AirlineDemoSmallComposite"))
    tempXdf1 <- tempfile( pattern = "tempXdf1", fileext = ".xdf")
    tempXdf2 <- tempfile( pattern = "tempXdf2", fileext = ".xdf")
    rxDataStep(inData, outFile = tempXdf1, overwrite = TRUE)
    # Use rxDataStep to read data with rowsPerRead and numRows.
    # rowsPerRead and numRows are big number so it should not affect the behavior of rxDataStep.
    # it should generate the same xdf file as the case without rowsPerRead and numRows
    rxDataStep(inData, outFile = tempXdf2, rowsPerRead=5e+06, numRows=1.5e+07, overwrite = TRUE)
    checkEquals(summary(RxXdfData(tempXdf1))$nobs.valid, summary(RxXdfData(tempXdf2))$nobs.valid)
    checkEquals(summary(RxXdfData(tempXdf1))$nobs.valid, 6e+05)
    file.remove(tempXdf1)
    file.remove(tempXdf2)
}

"test.rxDataStep.description" <- function() 
{
    #DEACTIVATED("Bug 11674: rxDataStep is not keeping description attribute for data frame") 

    # Specify name and location of output file
    outFile <- tempfile(pattern = ".rxTempFile", fileext = ".xdf")
    on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)
    myAttr <- ".rxDescription"
    numRow <- 10

    # Create a sample data frame with a description
    origData <- data.frame(x = seq.int(numRow))
    myDescription <- "Hello Data File!"
    attr(origData, myAttr) <- myDescription

    # Does the description go from the data frame to the .xdf?
    rxDataStep(inData = origData, outFile = outFile, overwrite = TRUE, rowsPerRead = numRow / 2)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$description, myDescription)

    # Does the description get passed to the data frame?
    myData <- rxDataStep(outFile)
    checkEquals(myData$x, seq.int(numRow))
    checkEquals(attr(myData, myAttr), myDescription)
}

"test.rxDataStep.numRows" <- function()
{
    xdf.path <- file.path(tempdir(), "temp1.xdf")
  on.exit(if(file.exists(xdf.path)) unlink(xdf.path, recursive = TRUE, force = TRUE), add = TRUE)
    xdf.ds <-  RxXdfData(xdf.path, createCompositeSet = FALSE)
    rxDataStep(iris, xdf.ds, overwrite = TRUE, rowsPerRead = 50)
    
    checkEquals(NROW(iris), 150)
    
    # Read the entire data set
    out.df <- rxDataStep(xdf.path, rowsPerRead = 1e5)
    checkIdentical(out.df, iris)
    
    # Read the entire data set in a loop
    out.df <- rxDataStep(xdf.path, rowsPerRead = 31)
    checkIdentical(out.df, iris)
    
    # Read a subset in a single shot
    out.df <- rxDataStep(xdf.path, rowsPerRead = 1e5, numRows = 42)
    checkIdentical(out.df, iris[1:42,])
    
    # Read a subset in a loop
    out.df <- rxDataStep(xdf.path, rowsPerRead = 11, numRows = 42)
    checkIdentical(out.df, iris[1:42,])
}
