# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxDataStep, testing functionality provided by deprecated rxDataStepXdf
#
############################################################################################
#===============================================================
# Sub-sample input file and create an output file
#

# Sub-sample rows from the 'Airline' dataset.
"test.rxDataStepXdf.args.numBlocks.AirlineArrDelayGT240" <- function()
{
    inFile    = file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayGT240.xdf")   # Input
    outFile <- tempfile(pattern = ".rxTempAirline1", fileext = ".xdf")

  if (file.exists(outFile)) file.remove(outFile)
  infoObj1 <- rxGetInfo(inFile, getBlockSizes=TRUE, numRows=10)
      
    rxDataStep(inData=inFile, outFile=outFile, startBlock=1, numBlocks=1)
  infoObj2 <- rxGetInfo(outFile, getBlockSizes=TRUE, numRows=10)
  # Check size of first block in inFile is same size as block in outFile
    checkEquals(infoObj1$rowsPerBlock[1], infoObj2$rowsPerBlock[1])
     
    # Check for equal values in DataSet for column info and first 10 rows
  checkEquals(infoObj1$data, infoObj2$data)
  file.remove(outFile)
}

"test.rxDataStepXdf.args.startBlock" <- function()
{
    # tiny data set. 30 rows, 10 rows/block, 3 blocks.
    inFile <- file.path(rxGetOption("unitTestDataDir"), "testSubset.xdf")       # Input
    outFile <- tempfile(pattern = ".rxTempDataStep", fileext = ".xdf")   # Output	
  if (file.exists(outFile)) file.remove(outFile)

    # Extract the second block, from rows 11 through 20 
    rxDataStep(inData=inFile, outFile=outFile, startBlock=2, numBlocks=1)
    infoObj <- rxGetInfo(inFile, numRows = 10, startRow = 11, getBlockSizes = TRUE)
  varInfoObj <- rxGetVarInfo(inFile)
  checkEquals(infoObj$fileName, normalizePath(inFile))
  checkEquals(infoObj$numRows, 30)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 3)
  checkEquals(infoObj$rowsPerBlock, c(10, 10, 10))
  checkEquals(infoObj$data[["Age"]], c(1L, 3L, 76L, 34L, 23L, 12L, 23L, 43L, 43L, 23L))
  checkEquals(infoObj$data[["Sex"]], structure(c(1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 2L), .Label = c("M", "F"), class = "factor"))
  checkEquals(infoObj$data[["State"]], c(2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L))
  checkEquals(varInfoObj[["Age"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 78), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["Sex"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("M", "F")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["State"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 1, getBlockSizes = TRUE)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(infoObj$rowsPerBlock, 10)
  checkEquals(infoObj$data[["Age"]], c(1L, 3L, 76L, 34L, 23L, 12L, 23L, 43L, 43L, 23L))
  checkEquals(infoObj$data[["Sex"]], structure(c(1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 2L), .Label = c("M", "F"), class = "factor"))
  checkEquals(infoObj$data[["State"]], c(2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L))
  checkEquals(varInfoObj[["Age"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 76), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["Sex"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("M", "F")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["State"]], structure(list(varType = "integer", storage = "int32", low = 2, high = 2), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  file.remove(outFile)
}


"test.rxDataStepXdf.args.rowSelection.multvars" <- function()
{

  # x is 1, 2, 3, 4 repeated 4 times
  # y is 1, 1, 1, 1, 2, 2, etc
  # z is 1, 2, 1, 2...
  # w is 1, 1,... followed by 2s
    myData <- data.frame(x=rep(1:4, times=4), y=rep(1:4, each=4),
    z=rep(1:2, times=8), w=rep(1:2, each=8))

    testFile <- tempfile(pattern = ".rxDSX1TempTest", fileext = ".xdf")
    outFile <- tempfile(pattern = ".rxDSX1TempTestOut", fileext = ".xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(outFile)) file.remove(outFile)
    
    # Create Xdf file
    rxDataStep(myData, testFile)
    
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=x > 1)
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 12)
  rm(outInfo)
  file.remove(outFile)
  
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=x > w)
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 10)
  rm(outInfo)
  file.remove(outFile)
  
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=x > w & z > 1)
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 6)
  rm(outInfo)
  file.remove(outFile)	
  
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=(x > w) & (z > 1))
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 6)
  rm(outInfo)
  file.remove(outFile)	
  
  rxDataStep(inData=testFile, outFile=outFile, rowSelection=(x > w) & (z > y))
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 2)
  rm(outInfo)
  file.remove(outFile)
  file.remove(testFile)
}

"test.rxDataStepXdf.args.rowSelection.expressions" <- function()
{
  # Create a small example data frame and .xdf file for comparison
  factorVar <- c("a", "b", "c", "a", "b", "c", "a", "b", "c", "a")
  numericVar1 <- rep(1:5, times=2)
  numericVar2 <- 1:10
  charVar <- c("TOK1", "AAA1", "TOK2", "AAA2", "TOK3", "AAA3", "TOK4", "AAA4", "TOK5", "AAA5")
  myDF <- data.frame(factorVar, numericVar1, numericVar2, charVar)
  
  sourceFile <- ".rxTempSource.xdf"
  subFile <- ".rxTempSubsetFile.xdf"
  rxDataStep(inData=myDF, outFile=sourceFile, overwrite=TRUE)

  # Example 1
  # Extract a subset of rows from a data frame
  subDF <- myDF[which(as.character(myDF$factorVar) %in% c("a","b")),]

  # Extract the same rows from an .xdf file and put into a new .xdf file
  #  using the rowSelection argument
  rxDataStep(sourceFile, subFile, overwrite=TRUE,
    rowSelection=as.character(factorVar) %in% c("a", "b"))
  # Read the data out of the new file in order to compare it with subDF
  subXdfDF <- rxReadXdf(subFile)
  # Row names attribute is different
  checkEquals(subDF, subXdfDF,check.attributes=FALSE)

  # Example 2
  subDF <- myDF[intersect(which(myDF$numericVar1 > 3), which(myDF$numericVar2 <= 5)) ,]
  rxDataStep(sourceFile, subFile, overwrite=TRUE,
    rowSelection=numericVar1 > 3 & numericVar2 <=5)
  subXdfDF <- rxReadXdf( subFile)
  checkEquals(subDF, subXdfDF,check.attributes=FALSE)

  # Example 3
  subDF <- myDF[union(which(myDF$numericVar1 == 3), which(myDF$factorVar == "a")) ,]

  rxDataStep(sourceFile, subFile,overwrite=TRUE,
    rowSelection= numericVar1 == 3 | factorVar == "a")
  subXdfDF <- rxReadXdf(subFile)
    # The data will not be in the same order
  checkEquals(subDF[order(subDF$numericVar2),], 
    subXdfDF[order(subXdfDF$numericVar2),],check.attributes=FALSE)

  # Example 4
  subDF <- myDF[-which(substr(myDF$charVar,1,3)=="TOK") ,]
  rxDataStep(sourceFile, subFile,
    rowSelection= substr(charVar, 1, 3) != "TOK", overwrite=TRUE)
  subXdfDF <- rxReadXdf(subFile) 
    checkEquals(subDF, subXdfDF,check.attributes=FALSE)
  
  file.remove(sourceFile)
  file.remove(subFile)
}

"test.rxDataStepXdf.args.transformFunc.simpleData" <- function()
{
    set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    testFile <- tempfile(pattern = ".rxDSX2TempTest", fileext = ".xdf")
    testFile1 <- tempfile(pattern = ".rxDSX2TempTest1", fileext = ".xdf")
    testFile2 <- tempfile(pattern = ".rxDSX2TempTest2", fileext = ".xdf")

  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  if (file.exists(testFile2)) file.remove(testFile2)
  
    rxDataStep(myData, testFile)

    trans <- function(data)
    {
      data$a <- data$w > 0
      data$b <- data$z*100	
      return(data)								
    }

    # Pass all data to transformation function
    rxDataStep(inData=testFile, outFile=testFile1,  transformFunc=trans)
    infoObj1 <- rxGetInfo(testFile1, numRows=5)
    
    # Improve efficiency by specifying which variables are used in the transformation function
    transformVars <- c("w", "z")
    rxDataStep(inData=testFile, outFile=testFile2,  transformFunc=trans, transformVars=transformVars)
    infoObj2 <- rxGetInfo(testFile2, numRows=5)
  
    # First 5 rows of data should be the same, returned as dataframe in infoObj
  checkEquals(infoObj1$data, infoObj2$data)
  file.remove(testFile2)
  
  # This should fail because transformVars are insufficient
  res <- try(rxDataStep(inData=testFile, outFile=testFile2,  transformFunc=trans, 
    transformVars=c("w")))
  checkTrue(is(res, "try-error"))

  # Clean up
  file.remove(testFile)
  file.remove(testFile1)
  
} 

"test.rxDataStepXdf.transformVars.dropVar" <- function()
{
    # RA-893: rxDataStep with transformFunc does not work to drop a variable
    #         when you specify variable in transformVars

    inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
    outFile <- tempfile( pattern = ".rxDataStepDV", fileext = ".xdf")

    xform <- function(dataList)
    {
        # remove a variable
        dataList$CRSDepTime <- NULL
        dataList
    }
    
    rxDataStep(inFile, outFile, transformFunc = xform,
                  transformVars = "CRSDepTime", reportProgress = 0)
    checkIdentical(rxReadXdf(inFile, reportProgress = 0)[-2L],
                   rxReadXdf(outFile, reportProgress = 0))
    file.remove(outFile)
}

"test.rxDataStepXdf.args.varsToDrop" <- function()
{
  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    testFile <- "rxTempTest.xdf"
    testFile1 <- "rxTempTest1.xdf"
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  
    rxDataStep(myData, testFile)
    varInfo1 <- rxGetVarInfo(data=testFile)
    
    # Check for 4 variables in first data set
    checkEquals(length(varInfo1), 4)

    # Specify which variables to keep in the new data file
    varsToKeep <- c("x", "w", "z")
    rxDataStep(inData=testFile, outFile=testFile1,  varsToKeep=varsToKeep)
    varInfo2 <- rxGetVarInfo(data=testFile1)

    # Check for 3 variables in second data set
    checkEquals(length(varInfo2), 3)
    checkTrue(all(sort(names(varInfo2)) %in% varsToKeep)) 
    file.remove(testFile1)
  
    # Alternatively specify which variables to drop
    varsToDrop <- c("y") 
    rxDataStep(inData=testFile, outFile=testFile1,  varsToDrop=varsToDrop)
    varInfo2 <- rxGetVarInfo(data=testFile1)    
    checkEquals(length(varInfo2), 3)
    checkTrue(all(sort(names(varInfo2)) %in% varsToKeep))
    file.remove(testFile1)
  
    # Drop 2 variables 
    varsToDrop <- c("x","y") # You cannot drop variables used in the transformation function
    rxDataStep(inData=testFile, outFile=testFile1,  varsToDrop=varsToDrop)
    metaInfo2 <- rxGetVarInfo(data=testFile1)    
    checkEquals(length(metaInfo2), 2)
    checkTrue(all(sort(names(metaInfo2)) %in% c("z", "w")))
  
  file.remove(testFile1)
  file.remove(testFile)
    
}
 
"test.rxDataStepXdf.data.removeMissings.xdf" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
    test100r5bStartMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bStartMiss")
    test100r5bEndMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bEndMiss")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
    
    testFile1 <- tempfile(pattern = ".rxDSX3TempTestFile1", fileext = ".xdf")
    testFile2 <- tempfile(pattern = ".rxDSX3TempTestFile2", fileext = ".xdf")
    testFile3 <- tempfile(pattern = ".rxDSX4TempTestFile3", fileext = ".xdf")
    testFile4 <- tempfile(pattern = ".rxDSX5TempTestFile4", fileext = ".xdf")
  
  if (file.exists(testFile1)) file.remove(testFile1)
  if (file.exists(testFile2)) file.remove(testFile2)
  if (file.exists(testFile3)) file.remove(testFile3)
  if (file.exists(testFile3)) file.remove(testFile4)
    
    # Base file has 1 block with missings removed before writing
    infoObj <- rxGetInfo(test77r1bNoMiss)
    
    # Remove missings from data set with 1 block and compare with base
    rxDataStep(inData=test100r1b, outFile=testFile1,  removeMissings=TRUE)
    infoObj1 <- rxGetInfo(testFile1)
  # Check the numRows are the same
  checkEquals(infoObj$numRows, infoObj1$numRows)
    
    # Remove missings from data with multiple blocks and compare with base
    rxDataStep(inData=test100r5b, outFile=testFile2,  removeMissings=TRUE)
    infoObj2 <-rxGetInfo(testFile2)
    checkEquals(infoObj$numRows, infoObj2$numRows)
    
    # Remove missings from data set with multiple blocks, all missing in first
    rxDataStep(inData=test100r5bStartMiss, outFile=testFile3,  removeMissings=TRUE)
    infoObj3 <- rxGetInfo(testFile3)
  checkEquals(infoObj$numRows, infoObj3$numRows)
    
    # Remove missings from data set with multiple blocks, all missing at end
    rxDataStep(inData=test100r5bEndMiss, outFile=testFile4,  removeMissings=TRUE)
    infoObj4 <- rxGetInfo(testFile4)
  checkEquals(infoObj$numRows, infoObj4$numRows)
    
  if (file.exists(testFile1)) file.remove(testFile1)
  if (file.exists(testFile2)) file.remove(testFile2)
  if (file.exists(testFile3)) file.remove(testFile3)
  if (file.exists(testFile4)) file.remove(testFile4)
}

"test.rxDataStepXdf.args.varsToKeep.xdf" <- function()
{
    xform <- function(data)
    {
      # alter a variable
      data$x <- data$x * 10
      
      # remove a variable
      data$z <- NULL
      
      # return the altered list of variables
      return(data)
    }

  set.seed(100)
    myData <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    testFile <- tempfile(pattern = ".rxDSX6TempTest", fileext = ".xdf")
    testFile1 <- tempfile(pattern = ".rxDSX6TempTest1", fileext = ".xdf")

    rxDataStep(myData, testFile)
    varInfo1 <- rxGetVarInfo(data=testFile)
    
    # Check for 4 variables in first data set
    checkEquals(length(varInfo1), 4)

    # Remove z in transform function variables to keep in the new data file
    rxDataStep(inData=testFile, outFile=testFile1,  transformFunc=xform)
    varInfo2 <- rxGetVarInfo(data=testFile1) 
    
    # Check for 3 variables in second data set
    checkEquals(length(varInfo2), 3)
    checkTrue(all(sort(names(varInfo2)) %in% c("x", "y", "w")))
    
    # Check the x variable was changed properly
    checkTrue(all.equal(rxReadXdf(testFile1)$x, seq(from=10, to=1000, by=10), check.attributes=FALSE))
  
  file.remove(testFile1)
  # Do the same thing, this time removing vars using varsToKeep
  xform1 <- function(data)
    {
      # alter a variable
      data$x <- data$x * 10     
      # return the altered list of variables
      return(data)
    }
  rxDataStep(inData=testFile, outFile=testFile1,  transformFunc=xform1, varsToKeep=c("x", "y", "w"))
  varInfo3 <- rxGetVarInfo(data=testFile1) 
  # Check for 3 variables in second data set
    checkEquals(length(varInfo3), 3)
    checkTrue(all(sort(names(varInfo3)) %in% c("x", "y", "w")))
    
    # Check the x variable was changed properly
    checkTrue(all.equal(rxReadXdf(testFile1)$x, seq(from=10, to=1000, by=10), check.attributes=FALSE))
  
  # Clean up
  file.remove(testFile)
  file.remove(testFile1)
}
 
"test.rxDataStepXdf.args.reportProgress.xdf" <- function()
{
    # tiny data set. 30 rows, 10 rows/block, 3 blocks.
    inFile  <- file.path(rxGetOption("unitTestDataDir"), "testSubset.xdf")       # Input
    outFile <- tempfile(pattern = ".rxDSX7TempTest", fileext = ".xdf")
  
  if (file.exists(outFile)) file.remove(outFile)
    o1 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=0))
    o2 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=1, overwrite=TRUE))
  if (file.exists(outFile)) file.remove(outFile)
    o2a <- capture.output(rxDataStep(inData=inFile,outFile=outFile, reportProgress=1, blocksPerRead=3))
  if (file.exists(outFile)) file.remove(outFile)
    o3 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=2))
  if (file.exists(outFile)) file.remove(outFile)
    o4 <- capture.output(rxDataStep(inData=inFile, outFile=outFile, reportProgress=2, blocksPerRead=3))
  if (file.exists(outFile)) file.remove(outFile)
    # reportProgress = 0 should print no output
    checkEquals(length(o1), 0)
    checkEquals(length(o2), 1)
    if (rxIsExpressEdition())
    {
        checkEquals(o2, "\rRows Processed: 30 ")
        checkTrue(length(o3) == 1 )
        checkTrue(length(o3) == length(o4))
    } else
    {
        checkEquals(o2, "\rRows Processed: 10\rRows Processed: 20\rRows Processed: 30 ")
        # If blocks per read is 3, should process all rows in one pass
        checkEquals(o2a, "\rRows Processed: 30 ")
        checkTrue(length(o3) > 1 )
        # We should get shorter output when reading all the data in one block
        checkTrue(length(o3) > length(o4))		
    }
    checkTrue(length(o4) > 0 )

}

"test.rxDataStepXdf.args.append" <- function()
{
    testFile <- tempfile(pattern = ".rxDSX8TempTest", fileext = ".xdf")
    testFile1 <- tempfile(pattern = ".rxDSX8TempTest1", fileext = ".xdf")
    outFile1 <- tempfile(pattern = ".rxDSX8TempOut1", fileext = ".xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(testFile1)) file.remove(testFile1)
  if (file.exists(outFile1)) file.remove(outFile1)
  
  # Create 2 original .xdf file, with the same data but different variable names
  myData <- data.frame(x=1:10, y=rep(c("a", "b", "c", "d", "e"),2))
  rxDataStep(inData=myData, outFile=testFile)
  names(myData) <- c("x1", "y1")
  rxDataStep(inData=myData, outFile=testFile1)
  
  # Just copy the data from one file to another using rxDataStep
  rxDataStep(inData=testFile, outFile=outFile1)
  
  # Try repeating this; it should fail because the outFile now exists
  res <- try(	rxDataStep(inData=testFile, outFile=outFile1))
  checkTrue(is(res,"try-error"))
  
  # Try one more time, this time specifying overwrite
  rxDataStep(inData=testFile, outFile=outFile1, overwrite=TRUE)
  
  # Now append rows to the newly created output file
  rxDataStep(inData=testFile, outFile=outFile1, append="rows")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 20)
  myNewData <- rxReadXdf(file=outFile1)
  checkEquals(myNewData$x[1:10], myNewData$x[11:20])
  
  # Now append columns to the original xdf file
  rxDataStep(inData=testFile1, outFile=testFile, append="cols")
  fileInfo <- rxGetInfo(testFile)
  checkEquals(fileInfo$numVars, 4) 
  checkEquals(fileInfo$numRows, 10)
  myNewData <- rxReadXdf(file=testFile)
  checkEquals(myData$x1, myNewData$x1)
  
  file.remove(testFile)
  file.remove(testFile1)
  file.remove(outFile1)
}

"test.rxDataStepXdf.args.append.multiblock" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to working directory
    outFile1 <- tempfile(pattern = ".rxTestFile1", fileext = ".xdf")
  outFile2 <- tempfile(pattern = ".rxTestFile2", fileext = ".xdf")
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Append rows from the original file to the copy
  rxDataStep(inData=test100r5b, outFile=outFile1, append="rows")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 200)
  file.remove(outFile1)
  
  # Make a copy again
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Try to append columns with the same name; should fail
  res <- try(rxDataStep(inData=test100r5b, outFile=outFile1, append="cols"))
  checkTrue(is(res,"try-error"))
  
  # Try again, this time requesting overwrite
  rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", overwrite=TRUE)
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numRows, 100)
  checkEquals(fileInfo$numVars, 9)  	
    
  # Rename the columns in outFile1
  varInfo <- as.list(NULL)
  for (i in 1:fileInfo$numVars)
  {
    varInfo[[i]] <- list(position=i, newName=paste("xxx", i, sep=""))
  }
  rxSetVarInfoXdf(varInfo, file=outFile1)
  rm(fileInfo)
  
  file.copy(from=outFile1, to=outFile2, overwrite=TRUE)
  # Append columns with a different block size
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols")
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 18)
    checkEquals(fileInfo$numRows, 100)
    checkDF <- rxReadXdf(file=outFile1)
    checkEquals(checkDF$xxx1, checkDF$y)
  file.remove(outFile1)
    rm(checkDF)
  
  # Append columns with the same block size
  rxDataStep(inData=test100r5b, outFile=outFile2, append="cols")
  fileInfo <- rxGetInfo(outFile2)
  checkEquals(fileInfo$numVars, 18)
    checkEquals(fileInfo$numRows, 100)
    checkDF <- rxReadXdf(file=outFile2)
    checkEquals(checkDF$xxx1, checkDF$y)
   
  file.remove(outFile2)
}

"test.rxDataStepXdf.args.append.multiblock.varsToKeep" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to working directory
    outFile1 <- tempfile(pattern = ".rxTestFile1", fileext = ".xdf")
  outFile2 <- tempfile(pattern = ".rxTestFile2", fileext = ".xdf")
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  fileInfo <- rxGetInfo(outFile1)
  
  # Rename the columns in outFile1
  varInfo <- as.list(NULL)
  for (i in 1:fileInfo$numVars)
  {
    varInfo[[i]] <- list(position=i, newName=paste("xxx", i, sep=""))
  }
  rxSetVarInfoXdf(varInfo, file=outFile1)
  rm(fileInfo)
  
  # Make another copy of the multi-block outFile
  file.copy(from=outFile1, to=outFile2, overwrite=TRUE)
  
  # Append columns with a different block size, using varsToKeep 
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols", varsToKeep=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 12)
  # The values of the renamed first column in the original outFile should be the same as y in the inFile
  myData <- rxReadXdf(file=outFile1, varsToKeep=c("y", "xxx1"))
  checkEquals(myData[,1], myData[,2])
  file.remove(outFile1)
  
  file.copy(from=outFile2, to=outFile1, overwrite=TRUE)
  rxDataStep(inData=test100r1b, outFile=outFile1, append="cols", varsToDrop=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo(outFile1)
  checkEquals(fileInfo$numVars, 15)
  file.remove(outFile1)
  
  # Append columns with the same block size
  rxDataStep(inData=test100r5b, outFile=outFile2, append="cols", varsToKeep=c("y", "xnum1", "xfac2"))
  fileInfo <- rxGetInfo(data=outFile2)
  checkEquals(fileInfo$numVars, 12)
  
  file.remove(outFile2)
}

"test.rxDataStepXdf.args.append.differentNumCols" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
    
    myData1 <- data.frame(x=1:5, y=1:5)
    myData2 <- data.frame(x=6:10)
    
    testFileXdf <- tempfile(pattern = ".rxDSX9TempTest", fileext = ".xdf")
    testCsv1 <- tempfile(pattern = ".rxDSX9TempTest1", fileext = ".csv")
    testCsv2 <- tempfile(pattern = ".rxDSX9TempTest2", fileext = ".csv")    
    write.csv(myData1, file=testCsv1, row.names=FALSE)
    write.csv(myData2, file=testCsv2, row.names=FALSE)
    rxTextToXdf(inFile=testCsv1, outFile=testFileXdf, overwrite=TRUE)
    
    # This should fail because of different number of columns	
    res <- try(rxTextToXdf(inFile=testCsv2, outFile=testFileXdf, append="rows"))
    checkTrue(is(res, "try-error"))

    # This should work
    rxTextToXdf(inFile=testCsv2, outFile=testFileXdf, append="rows", transforms=list(y=x))
    myData3 <- rxReadXdf(testFileXdf)
    checkEquals(nrow(myData3), 10)
    checkEquals(myData3$x, myData3$y)
    
    file.remove(testCsv1)
    file.remove(testCsv2)
    file.remove(testFileXdf)
}


"test.rxDataStepXdf.args.overwrite" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
  
  # Copy base file to working directory
    outFile1 <- tempfile(pattern = ".rxTestFile1", fileext = ".xdf")
    outFile2 <- tempfile(pattern = ".rxTestFile2", fileext = ".xdf")

  file.copy(test100r5b, outFile1, overwrite=TRUE)
  if (file.exists(outFile2)) file.remove(outFile2)
  
  # Specify overwrite = true with no append and no existing outFile
  
  rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", overwrite=TRUE)
  fileInfo <- rxGetInfo(data = outFile2)
  checkEquals(fileInfo$numVars, 8)
  
  # Specify overwrite = true with an existing outfile
  rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", overwrite=TRUE)
  fileInfo <- rxGetInfo(data = outFile2)
  checkEquals(fileInfo$numVars, 8)
  
  # Specify overwrite = false with an existing outfile: should fail
  res <- try(rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y", 
    overwrite=FALSE))
  checkTrue(is(res, "try-error"))
  
  # Specify no overwrite with an existing outfile: should fail
  res <- try(rxDataStep(inData=outFile1, outFile=outFile2, varsToDrop="y"))
  checkTrue(is(res, "try-error"))
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  # If appending cols, specify overwrite = TRUE should overwrite columns
  rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", 
    varsToKeep=c("y", "xnum1", "xfac2"), overwrite=TRUE)
  fileInfo <- rxGetInfo(data=outFile1)
  checkEquals(fileInfo$numVars, 9)
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  
  # Specify overwrite = FALSE - should fail because of the same column names
  res <- try(rxDataStep(inData=test100r5b, outFile=outFile1, append="cols", 
    varsToKeep=c("y", "xnum1", "xfac2"), overwrite=FALSE))
  checkTrue(is(res, "try-error"))
  file.remove(outFile1)
  
  file.copy(from=test100r5b, to=outFile1, overwrite=TRUE)
  # overwrite is ignored when appending rows
  rxDataStep(inData=test100r1b, outFile=outFile1, append="rows", 
   overwrite=TRUE)
  fileInfo <- rxGetInfo(data=outFile1)
  checkEquals(fileInfo$numRows, 200)
    file.remove(outFile1)
  
  if (file.exists(outFile2)) file.remove(outFile2)
  
}

"test.rxDataStepXdf.args.transformFunc.combo" <- function()
{

    inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
    outFile1 <- ".rxTempKyphosis1.xdf"
    outFile2 <- ".rxTempKyphosis2.xdf"
    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
    # Number is already an integer, so this should not change anything
    xform <- function(data) 
    { 
        data$Number <- as.integer(data$Number) 
        return(data) 
     } 

    # With transform
  rxDataStep(inData=inFile, outFile=outFile1, rowSelection=Age>30, varsToKeep="Start", transforms=list(Number=as.integer(Number)))

    # With transformFunc
    rxDataStep(inData=inFile, outFile=outFile2, transformVars=c("Number"), rowSelection=Age>30, 
    varsToKeep="Start", transformFunc=xform, overwrite=TRUE) 

    fileInfo1 <- rxGetInfo(outFile1)
    fileInfo2 <- rxGetInfo(outFile2)
    
    checkEquals(fileInfo1$numRows, fileInfo2$numRows)
    checkEquals(fileInfo1$numVars, fileInfo2$numVars)
  
  myDF1 <- rxReadXdf(outFile1)
  myDF2 <- rxReadXdf(outFile2)
  checkEquals(myDF1, myDF2)
    
    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
}

"test.rxDataStepXdf.args.rowSelection.combo" <- function()
{
  # See RA-1720
  inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
    outFile1 <- ".rxTempKyphosis1.xdf"
    outFile2 <- ".rxTempKyphosis2.xdf"
    if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
  
  xform <- function(data) 
  { 
    data$Age <- data$Age*2. 
    data$Number <- as.integer(data$Number) 
    data$Start <- as.factor(data$Start) 
    return(data) 
  } 

  rxDataStep(inData=inFile, outFile=outFile1,transformFunc=xform,overwrite=TRUE) 

  rxDataStep(inData=inFile, outFile=outFile2, transformFunc=xform, rowSelection= Kyphosis=="absent", overwrite=TRUE) 
  
  myDF1 <- rxReadXdf(outFile1)
  myDF1 <- subset(myDF1, Kyphosis=="absent")
  myDF2 <- rxReadXdf(outFile2)
  # Row names will be different
  checkEquals(myDF1, myDF2, check.attributes=FALSE)
  rm(myDF1)
  
    if (file.exists(outFile1)) file.remove(outFile1)
  
  rxDataStep(inData=inFile, outFile=outFile1, transforms=list(Age=Age*2., Number=as.integer(Number), Start=as.factor(Start)),
     rowSelection= Kyphosis=="absent", overwrite=TRUE) 
  
  myDF1 <- rxReadXdf(outFile1)
  checkEquals(myDF1, myDF2)
  
  if (file.exists(outFile1)) file.remove(outFile1)
    if (file.exists(outFile2)) file.remove(outFile2)
}

"test.rxDataStepXdf.args.transformFunc.rxStartRow" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    tempFile <- tempfile(pattern = ".rxDSX10TempTest", fileext = ".xdf")

  if (file.exists(tempFile)) file.remove(tempFile)
    
  myTransformFunc <- function( dataList)
  {
    rxDataSource <- .rxReadFileName
    rxStartRow <- .rxStartRow
    rxChunkNum <- .rxChunkNum
    rxIsTestChunk <- .rxIsTestChunk
        rxNumRows     <- .rxNumRows
    
    #nrows <- length(dataList[[1]])
    dataList$startRow <- rep(.rxStartRow, rxNumRows)
    dataList$chunkNum <- rep(.rxChunkNum, rxNumRows)
    return ( dataList)
  }
  rxDataStep(inData=test100r5b, outFile=tempFile, transformFunc=myTransformFunc, overwrite=TRUE)
  myData <- rxReadXdf(tempFile)
  if (rxIsExpressEdition())
  {
      testStartRow <- rep(1, each=100)
      testChunkNum <- testStartRow
  } else
  {
    testStartRow <- rep(seq(1,by=20, length.out=5), each=20)
      testChunkNum <- rep(seq(1, by=1, length.out=5), each=20)
  }
  
  checkEquals(myData$startRow, testStartRow)
  checkEquals(myData$chunkNum, testChunkNum)
  if (file.exists(tempFile)) file.remove(tempFile)
  rm(myData)
    
    # Try with data frame
    myInData <- rxReadXdf(test100r5b)
    myData <- rxDataStep(inData=myInData,transformFunc=myTransformFunc, overwrite=TRUE)
    checkEquals(myData$startRow, rep(1, 100))
  checkEquals(myData$chunkNum, rep(1,100))
    rm(myData)
  
  # Repeat with numCoresToUse
    numCoresOld <- rxGetOption("numCoresToUse")
  rxOptions(numCoresToUse=1)
  rxDataStep(inData=test100r5b, outFile=tempFile, transformFunc=myTransformFunc, overwrite=TRUE)
  myData <- rxReadXdf(tempFile)
  #testStartRow <- rep(seq(1,by=20, length.out=5), each=20)
  checkEquals(myData$startRow, testStartRow)
  checkEquals(myData$chunkNum, testChunkNum)
  rxOptions(numCoresToUse=numCoresOld)
  if (file.exists(tempFile)) file.remove(tempFile)
    
}

"test.rxDataStepXdf.args.rowsPerRead" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    tempFile <- tempfile(pattern = ".rxDSX11TempTest", fileext = ".xdf")

  if (file.exists(tempFile)) file.remove(tempFile)
    
  rxDataStep(inData=test100r5b, outFile=tempFile, rowsPerRead=10, overwrite=TRUE)
  fileInfo <- rxGetInfo(tempFile)
  if (rxIsExpressEdition())
  {
    checkEquals( fileInfo$numBlocks, 1)
  } else
  {
      checkEquals( fileInfo$numBlocks, fileInfo$numRows/10)
  }
  
  testDF1 <- rxReadXdf(test100r5b)
  testDF2 <- rxReadXdf(tempFile)
  checkEquals(testDF1, testDF2)

  if (file.exists(tempFile)) file.remove(tempFile)	
  
  # Write a data frame to a file, using rowsPerRead
  rxDataStep(inData=test100r5b, outFile=tempFile, rowsPerRead=50, overwrite=TRUE)
  fileInfo <- rxGetInfo(tempFile)
  if (rxIsExpressEdition())
  {
    checkEquals( fileInfo$numBlocks, 1)
  } else
  {
      checkEquals( fileInfo$numBlocks, 2)
  }
  
  testDF2 <- rxReadXdf(tempFile)
  checkEquals(testDF1, testDF2)	
  
  file.remove(tempFile)
  
}

"test.rxDataStepXdf.args.rowsSelection.ordered" <- function()
{
     
    x <- rep(1:4, times=4)
    xfac <- factor(x, labels = c("Low", "Med", "High", "Highest"))
    xofac <- factor(x, labels = c("Low", "Med", "High", "Highest"), ordered=TRUE)
    myData <- data.frame(x, xfac, xofac)

    testFile <- file.path(tempdir(), ".rxDSX12TempTest.xdf")
  outFile <- file.path(tempdir(), ".rxDSX12TempTestOut.xdf")
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(outFile)) file.remove(outFile)
    
    # Create Xdf file
    rxDataStep(myData, testFile)
    
  rxDataStep(inData=testFile, outFile=outFile, rowSelection = xofac < "Highest")
  outInfo <- rxGetInfo(outFile)
  checkEquals(outInfo$numRows, 12)
    myNewData <- rxReadXdf(outFile)
    checkEquals( sum(myNewData$xfac == "Highest"), 0)
  rm(outInfo)
  file.remove(outFile)
  file.remove(testFile)
}


