# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    RUnit test file for arithmetic formula expressions in RevoScaleR
#
############################################################################################

###
# pow()
###

test.expression.pow <- function()
{
    # define local data.frames
    DF <- as.data.frame(UCBAdmissions)
    DF2 <- DF
    DF2$Freq <- DF2$Freq^2

    # cross-tabulation
    z1.rxCrossTabs <- rxCrossTabs(pow(Freq,2) ~ Gender : Admit, DF, transformVars="Freq")
    z2.rxCrossTabs <- rxCrossTabs(Freq ~ Gender : Admit, DF2, transformVars="Freq")
    checkTrue(all.equal(z1.rxCrossTabs[c("sums", "counts")], z2.rxCrossTabs[c("sums", "counts")], check.attributes=FALSE))

    # linear modeling
    z1.rxLinMod <- rxLinMod(pow(Freq,2) ~ Gender : Admit, DF, transformVars="Freq")
    z2.rxLinMod <- rxLinMod(Freq ~ Gender : Admit, DF2, transformVars="Freq")
    checkTrue(all.equal(coef(z1.rxLinMod), coef(z2.rxLinMod), check.attributes=FALSE))

    # model summarization
    z1.rxSummary <- rxSummary(~ pow(Freq,2), DF, transformVars="Freq")
    z2.rxSummary <- rxSummary(~Freq, DF2, transformVars="Freq")
    checkTrue(all.equal(z1.rxSummary$sDataFrame[c("Mean","StdDev","Min","Max","ValidObs")], 
    z2.rxSummary$sDataFrame[c("Mean","StdDev","Min","Max","ValidObs")]))
}

#NEED TO CONVERT TO NOT RELY ON RevoScaleR::
#test.rxFormula.I.expressions <- function()
#{
  ## need to remove 	
  #deblank <- function(x) gsub("[ ]*","", x)
  #DF <- data.frame(x=rnorm(5), y=rnorm(5), cow=rnorm(5), milk=rnorm(5), foo=1:5)
  #z <- rxCrossTabs(milk ~ I(y/x^2), data=DF)
  #checkTrue(z$expr == "Iyx2")
  #checkTrue(deblank(z$newVars) == "Iyx2=y/pow(x,2)")
  #
  #z <- rxUtilsFixExpr((~I(x^3.14159))[[2]])
  #checkTrue(z$expr == "Ix3.14159")
  #checkTrue(deblank(z$newVars) == "Ix3.14159=pow(x,3.14159)")
#
  #z <- rxUtilsFixExpr((~I(1 + cow/milk^3))[[2]])
  #checkTrue(z$expr == "I1cowmilk3")
  #checkTrue(deblank(z$newVars) == "I1cowmilk3=1+cow/pow(milk,3)")
#
  #z <- rxUtilsFixExpr((~I(10 * cow/milk^3))[[2]])
  #checkTrue(z$expr == "I10cowmilk3")
  #checkTrue(deblank(z$newVars) == "I10cowmilk3=10*cow/pow(milk,3)")
#
  #z <- rxUtilsFixExpr((~I(y/(foo)^10))[[2]])
  #checkTrue(z$expr == "Iyfoo10")
  #checkTrue(deblank(z$newVars) == "Iyfoo10=y/pow((foo),10)")
#}

"test.formula.multiple.depvars" <- function()
{
  # define data source and formula variations
  sampleDataDir <- rxGetOption("sampleDataDir")
    XDF <- file.path(sampleDataDir, "AirlineDemoSmall")
  form <- formula(cbind(ArrDelay, CRSDepTime) ~ DayOfWeek)
  formStr <- "ArrDelay, CRSDepTime ~ DayOfWeek"
  formStr2 <- "ArrDelay ~ DayOfWeek;CRSDepTime ~ DayOfWeek"
  
  # rxLinMod
    zLMCbind <- rxLinMod(form, data=XDF)
    zLMStr <- rxLinMod(formStr, data=XDF)
  noCompare <- which(names(zLMCbind) %in% c("call","formula","params"))
  checkEquals(zLMCbind[-noCompare], zLMStr[-noCompare])
  
  # rxCrossTabs
    zCTCbind <- rxCrossTabs(form, data=XDF)
    zCTStr <- rxCrossTabs(formStr2, data=XDF)
  noCompare <- which(names(zCTCbind) %in% c("call","formula","params"))
  checkEquals(zCTCbind[-noCompare], zCTStr[-noCompare])
}

"test.expression.args.F" <- function()
{
  # define local functions
  "deblank" <- function(x) gsub(" ", "", x)
  
  "verifyFormula" <- function(form, data, formulaStr, FUN=rxCube)
  {
    exaStatFormula <- deblank(attr(FUN(form, data=data),"params")$Formula)
    correctFormula <- deblank(formulaStr)
    checkEquals(correctFormula, exaStatFormula, msg=paste(" ", 
            exaStatFormula, "not equal to", correctFormula))
  }

  "FStressTest" <- function(FUN, formula, data, ...)
  {		
    # run the crosstab
    res <- try(do.call(FUN, list(formula=formula, data=data, ...)))
      
    # return TRUE for no errors encountered during execution
    return (!is(res, "try-error"))
  }

  # create test data sources	
  set.seed(100)
  N <- 10
  DF <- data.frame(c1=seq(-100, 100, length.out=N), c2=(1:N)*0.1, 
    f1=factor(1:N), f2=factor(sample(c("Male","Female"), 10, replace=TRUE)))
  DF2 <- data.frame(x=1:10, y=(1:10)*2, z=rnorm(10), xf=factor(1:10))
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
  airlineDF <- rxReadXdf(file = inFile)
    airline2DF <- cbind(airlineDF, ArrOnTime=((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))

  # check argument permutations: output names and values
  z1 <- rxCube(c1 ~ F(c1, low=10, high=80, exclude=TRUE), data=DF, removeZeroCounts=TRUE)
  z2 <- rxCube(c1 ~ F(c1, high=80, low=10, exclude=TRUE), data=DF, removeZeroCounts=TRUE)
  z3 <- rxCube(c1 ~ F(c1, high=80, low=10), data=DF, removeZeroCounts=TRUE)
  z4 <- rxCube(c1 ~ F(c1, exclude=TRUE, high=80, low=10), data=DF, removeZeroCounts=TRUE)
  nms <- c("F_c1_10_80_T", "c1", "Counts")
  checkTrue(all(names(z1) == nms))
  checkTrue(all(names(z2) == nms))
  checkTrue(all(names(z3) == nms))
  checkTrue(all(names(z4) == nms))
  checkEquals(z1, z2, check.attributes=FALSE)
  checkEquals(z2, z3, check.attributes=FALSE)
  checkEquals(z3, z4, check.attributes=FALSE)
  c1cut <- DF$c1[6:9]
  checkEquals(z1, data.frame(factor(floor(c1cut)), c1cut, rep(1,4)), check.attributes=FALSE)
  
  # argument specification by order 
  z1 <- rxCube(c1 ~ F(c1, exclude=FALSE, high=80, low=10), data=DF)
  z2 <- rxCube(c1 ~ F(c1, 10, 80, FALSE), data=DF)
  checkEquals(z1, z2, check.attributes=FALSE)
    
  # check to make sure that a user hasn't errantly used F() on a dependent variable
  checkException(rxCube(F(c1, low=10, high=80, exclude=TRUE) ~ f1, data=DF), 
        msg="Caught exception: dependent variables cannot be factors")
    
  # test various formulations of F(...) input arguments to standard
  verifyFormula(y ~ F(x), DF2, "y ~ F(x)")
  verifyFormula(z ~ F(x) : F(y), DF2, "z ~ F(x) : F(y)")
  verifyFormula(z ~ xf : F(y), DF2, "z ~ xf : F(y)")
  verifyFormula(z ~ F(x,4) : F(y), DF2, "z ~ F(x,4,NA,T) : F(y)")
  verifyFormula(z ~ F(x,1,12,FALSE) : F(y,1,4,TRUE), DF2, "z ~ F(x,1,12,F) : F(y,1,4,T)")
  verifyFormula(z ~ xf : F(y, high=round(pi,3), exclude=FALSE, low=2.1), DF2, "z ~ xf : F(y,2.1,3.142,F)")
  verifyFormula(z ~ xf : F(y, high=4, exclude=FALSE), DF2, "z ~ xf : F(y,NA,4,F)")
  #verifyFormula(z ~ x + F(y, high=3, length=100, exclude=FALSE), DF2, "z ~ x : F(y,NA,3,100,F,T,T,F)")
  #verifyFormula(z ~ x + F(y, high=2, by=3.14159, exclude=FALSE), DF2, "z ~ x : F(y,NA,2,3.14159,T,T,T,F)")
  #checkException(rxCube(z ~ x + F(y, low=2, high=50.1, by=3.14159, length=100), DF2))
    
  # negative low and/or high values
  zMinusFormula <- rxCube(~ F(c1, low=-80, high=-60, exclude=TRUE), data=DF)
  zMinusStr <- rxCube("F(c1,rxMinus80,rxMinus60,T)", data=DF)
  checkTrue(all.equal(zMinusFormula, zMinusStr, check.attributes=FALSE))
    
  # check exclusion argument ...
  low <- 10
  high <- 80
  formExcludeFalse <- formula(paste("c2 ~ F(c1, low=", low, ", high=", high, ", exclude=FALSE) : f2", sep=""))
  formExcludeTrue <- formula(paste("c2 ~ F(c1, low=", low, ", high=", high, ", exclude=TRUE) : f2", sep=""))
  zExcludeTrue <- rxCube(formExcludeTrue, data=DF, returnDataFrame=TRUE, removeZeroCounts=TRUE)
  zExcludeFalse <- rxCube(formExcludeFalse, data=DF, returnDataFrame=TRUE, removeZeroCounts=TRUE)
  
  # ... check rxCube output data.frame names
  nms <- c(paste("F_c1_", low, "_", high, "_T", sep=""), "f2", "c2", "Counts")
  checkTrue(all(names(zExcludeTrue) == nms))
  nms <- c(paste("F_c1_", low, "_", high, "_F", sep=""), "f2", "c2", "Counts")
  checkTrue(all(names(zExcludeFalse) == nms))

  # ... check in-range category values (means and counts) for both sexes
  inRangeDF <- subset(DF, c1 >= low & c1 <= high)
  inRangeDFC2Male <- subset(inRangeDF, f2 == "Male")$c2
  inRangeDFC2Female <- subset(inRangeDF, f2 == "Female")$c2
  zExcludeTrueMissingFemale <- subset(as.data.frame(zExcludeTrue), f2 == "Female")
  zExcludeTrueMissingMale <- subset(as.data.frame(zExcludeTrue), f2 == "Male")
  checkEqualsNumeric(zExcludeTrueMissingFemale$c2, inRangeDFC2Female)
  checkTrue(all(zExcludeTrueMissingFemale$Counts == 1))
  checkEqualsNumeric(zExcludeTrueMissingMale$c2, inRangeDFC2Male)
  checkTrue(all(zExcludeTrueMissingMale$Counts == 1))
    
  # ... check out-of-range category values (means and counts) for both sexes
  outRangeDF <- subset(DF, c1 < low | c1 > high)
  outRangeDFC2Male <- subset(outRangeDF, f2 == "Male")$c2
  outRangeDFC2Female <- subset(outRangeDF, f2 == "Female")$c2
  zExcludeFalseMissingFemale <- subset(zExcludeFalse, F_c1_10_80_F == ".rxOther" & f2 == "Female")
  zExcludeFalseMissingMale <- subset(zExcludeFalse, F_c1_10_80_F == ".rxOther" & f2 == "Male")
  checkEqualsNumeric(zExcludeFalseMissingFemale$c2, mean(outRangeDFC2Female))
  checkEqualsNumeric(zExcludeFalseMissingFemale$Counts, length(outRangeDFC2Female))
  checkEqualsNumeric(zExcludeFalseMissingMale$c2, mean(outRangeDFC2Male))
  checkEqualsNumeric(zExcludeFalseMissingMale$Counts, length(outRangeDFC2Male))
  
  # check for exceptions ...
  # ... reversal of high-low
  checkException(rxCube(~ F(c1, 80, 10, exclude=TRUE), data=DF), msg="Caught exception: reversal of high-low values in F() call")
  
  # ... bad argument types
  checkException(rxCube(~ F(c1, TRUE, 10, exclude=TRUE), data=DF), msg="Caught exception: bad lower bound type")
  checkException(rxCube(~ F(c1, 10, TRUE, exclude=TRUE), data=DF), msg="Caught exception: bad upper bound type")
  checkException(rxCube(~ F(c1, 10, 80, exclude=pi), data=DF), msg="Caught exception: bad exclude type")

  # ... bad argument lengths
  checkException(rxCube(~ F(c1, rep(10,2), 10, exclude=TRUE), data=DF), msg="Caught exception: bad lower bound length")
  checkException(rxCube(~ F(c1, 10, rep(80,2), exclude=TRUE), data=DF), msg="Caught exception: bad upper bound length")
  checkException(rxCube(~ F(c1, 10, 80, exclude=rep(TRUE,2)), data=DF), msg="Caught exception: bad exclude length")
  
  # ... missing variable name
  checkException(rxCube(~ F(, low=10, high=80, exclude=TRUE), data=DF), msg="Caught exception: missing variable argument")
  checkException(rxCube(~ F(low=10, high=80, exclude=TRUE), data=DF), msg="Caught exception: missing variable argument")

  # variable name does not exist
  checkException(rxCube(~ F(myNonexistentVariableName,low=10, high=80, exclude=TRUE), data=DF), 
    msg="Caught exception: specified variable does not exist in data source")
  
  # check that we get nothing back when low=high and exclude=TRUE (this should not throw an error)
  checkTrue(length(rxCube(~ F(c1, 10, 10, exclude=TRUE), data=DF)) == 0)
  
  # ensure that we have all missings when low=high and exclude=FALSE (this should not throw an error)
  zAllMissings <- rxCube(~ F(c1, 10, 10, exclude=FALSE), data=DF, returnDataFrame=TRUE)
  checkTrue(all(names(zAllMissings) == c("F_c1_10_10_F","Counts")))
  convertedFcolumn <- zAllMissings$F_c1_10_10_F
  checkTrue(is.factor(convertedFcolumn))
  checkTrue(all(levels(convertedFcolumn) == c("10",".rxOther")))
  checkEqualsNumeric(zAllMissings$Counts, c(0,NROW(DF)))
  
  # do stress tests on various primary package functions with F()
  checkTrue(FStressTest(rxLinMod, c2 ~ F(c1, low=70, high=90, exclude=TRUE) - 1, DF))
  checkTrue(FStressTest(rxCrossTabs, c2 ~ F(c1, low=70, high=90, exclude=TRUE) : f1 : f2, DF))
  checkTrue(FStressTest(rxSummary,~ F(c1, low=70, high=90, exclude=TRUE), DF))
  # high, low not currently supported for rxLogit
  #checkTrue(FStressTest(rxLogit, ArrOnTime ~ CRSDepTime + F(ArrDelay, low=0, high=3, exclude=FALSE), 
    #data=airline2DF, maxIterations=5))
}

"test.expression.F.names" <- function()
{
  # tests the consistency of the converted F(...) names for relevant functions
  DF <- data.frame(sex=c("Male","Male","Female", "Male"), age=c(20,20,12,15), 
    score=(1:4)+0.1, isScore=c(0,1,1,0))
  
  # rxCube
  zCube1 <- rxCube(score ~ sex : F(age), DF)
  zCube2 <- rxCube(score ~ sex : F(age,low=14,high=20,exclude=TRUE), DF)
  checkTrue("F_age" %in% names(zCube1))
  checkTrue("F_age_14_20_T" %in% names(zCube2))
  
  # rxLinMod
  zLinMod <- rxLinMod(score ~ sex + F(age, -1, 20, TRUE), DF)
  checkTrue(all(paste("F_age_-1_20_T", -1:20, sep="=") %in% names(coef(zLinMod))))
  
  # rxLogit
  # high, low not currently supported for rxLogit
  #zLogit <- rxLogit(isScore ~ F(age,14,18,FALSE), DF)
  #checkTrue(all(paste("F_age_14_18_F", 14:18, sep="=") %in% names(coef(zLogit))))
  
  # rxSummary
  zSummary <- rxSummary(~ F(age), DF)
  checkTrue(zSummary$sDataFrame$Name == "F_age")
  checkTrue("F_age" %in% names(zSummary$categorical[[1]]))
  
  # rxCrossTabs
  zCrossTabs <- rxCrossTabs(score ~ sex : F(age,10,20,T), DF, mean=TRUE)
  checkTrue("F_age_10_20_T" %in% names(dimnames(zCrossTabs$sums$score)))
  checkTrue("F_age_10_20_T" %in% names(dimnames(zCrossTabs$counts$score)))
  checkTrue("F_age_10_20_T" %in% names(dimnames(zCrossTabs$means$score)))
}