# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#	rxFindFileInPath
#
#	Remember that all directories and files should always be deleted in the reverse order
#	that they were created.  Also, unlink andon NULL without choking (but are no-ops);
#	not so on file.remove.  unlink should be used for directories; file.remove for files.
#
############################################################################################


myL1_1dirName <- NULL
myL1_2dirName <- NULL
myL2_1_1dirName <- NULL
myL2_1_2dirName <- NULL
myL1_2_1dirName <- NULL
myL1_2_2dirName <- NULL

.rxFindFileFile1 <- NULL
.rxFindFileFile2 <- NULL
.rxFindFileFile3 <- NULL

.rxFindFilePathDelim = if ( identical(.Platform$OS.type , "windows" ) ) "\\" else "/"
.rxFindFilePathSeperatorDelim = if ( identical(.Platform$OS.type , "windows" ) ) ";" else ":"

.rxFindFileInPathInitSucceeded <- TRUE

".setUp" <- function()
{
  myTempDir <- normalizePath(tempdir())
  myL1_1dirName <<- paste( myTempDir, "L1_1dir", sep=.rxFindFilePathDelim )
  myL1_2dirName <<- paste( myTempDir, "L1_2dir", sep=.rxFindFilePathDelim )
  myL2_1_1dirName <<- paste( myL1_1dirName, "L1_1_1dir", sep=.rxFindFilePathDelim )
  myL2_1_2dirName <<- paste( myL1_1dirName, "L1_1_2dir", sep=.rxFindFilePathDelim )
  myL1_2_1dirName <<- paste( myL1_2dirName, "L1_2_1dir", sep=.rxFindFilePathDelim )
  myL1_2_2dirName <<- paste( myL1_2dirName, "L1_2_2dir", sep=.rxFindFilePathDelim )

  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL1_1dirName, showWarnings = TRUE )
  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL1_2dirName, showWarnings = TRUE )
  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL2_1_1dirName, showWarnings = TRUE )
  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL2_1_2dirName, showWarnings = TRUE )
  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL1_2_1dirName, showWarnings = TRUE )
  .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && dir.create( myL1_2_2dirName, showWarnings = TRUE )
  
  if ( .rxFindFileInPathInitSucceeded ) 
  {
    .rxFindFileFile1 <<- paste( myL2_1_2dirName, "foo", sep=.rxFindFilePathDelim )
    .rxFindFileFile2 <<- paste( myL1_2_2dirName, "foo", sep=.rxFindFilePathDelim )
    .rxFindFileFile3 <<- paste( myL1_1dirName, "foo", sep=.rxFindFilePathDelim )
    
    .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && file.create( .rxFindFileFile1, showWarnings = TRUE )
    .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && file.create( .rxFindFileFile2, showWarnings = TRUE )
    .rxFindFileInPathInitSucceeded <<- .rxFindFileInPathInitSucceeded && file.create( .rxFindFileFile3, showWarnings = TRUE )
  }
}

".tearDown" <- function()
{
  if ( !is.null( .rxFindFileFile3 ) )
    file.remove( .rxFindFileFile3 )
  if ( !is.null( .rxFindFileFile2 ) )		
    file.remove( .rxFindFileFile2 )
  if ( !is.null( .rxFindFileFile1 ) )		
    file.remove( .rxFindFileFile1 )
  
  unlink( myL1_2_2dirName, recursive=TRUE, force=TRUE )
  unlink( myL1_2_1dirName, recursive=TRUE, force=TRUE )
  unlink( myL2_1_2dirName, recursive=TRUE, force=TRUE )
  unlink( myL2_1_1dirName, recursive=TRUE, force=TRUE )
  unlink( myL1_2dirName, recursive=TRUE, force=TRUE )
  unlink( myL1_1dirName, recursive=TRUE, force=TRUE )
}


# check for NULL
"test.rxFindFileInPath.NULL" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()
    
    myPath <- NULL
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkTrue( is.null( myFoundPath ) )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkTrue( is.null( myFoundPath ) )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkTrue( is.null( myFoundPath ) )
    
    myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    checkTrue( is.null( myFoundPath ) )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkTrue( is.null( myFoundPath ) )
  }
}

# check for empty string (acceptable for flattened character vector, not for dataPath itself, though, which must be NULL)
"test.rxFindFileInPath.emptyString" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myPath <- ""
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkTrue( is.null( myFoundPath ) )
  }
}
        
# test for single path
"test.rxFindFileInPath.singlePath" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()
    
    myPath <- myL1_1dirName
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    #myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    #myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )
    #
    #hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    #myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )
  }
}
  
# test for case of first element in a multiple path with only one file anywhere in path		
"test.rxFindFileInPath.firstAndOnlyInPath" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )

    # test for first entry in a character scalar with multiple path's seperated by OS style delim
    myPath <- paste( myL1_1dirName, myL1_2dirName, sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    # test for first entry in a character vector
    myPath <- c( myL1_1dirName, myL1_2dirName )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )		
  }
}				


# test for case of non-first element in a multiple path with only one file anywhere in path		
"test.rxFindFileInPath.notFirstButOnlyInPath" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()

    # check for valid middle entry in a character scalar, only one file to be found, OS style delim
    myPath <- paste( myL1_2_1dirName, myL2_1_1dirName, myL1_1dirName, myL1_2dirName, sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    # check for middle entry in a character vector
    myPath <- c( myL1_2_1dirName, myL2_1_1dirName, myL1_1dirName, myL1_2dirName )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )

    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )		

    #myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    #myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )		
            
    #hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    #myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )
  }
}
    

# test for case of non-first element in a multiple path with more than one file somewhere in path		
"test.rxFindFileInPath.NotfirstNotOnlyInPath" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()

    # check for first of multiple entries in a OS style system path in a character scalar
    # TEST 1
    myPath <- paste( myL1_1dirName, myL1_2_1dirName, myL2_1_1dirName, myL2_1_2dirName, myL1_2dirName, myL1_2_2dirName, sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )

    # TEST 2
    myPath <- paste( myL1_2_1dirName, myL2_1_1dirName, myL2_1_2dirName, myL1_2dirName, myL1_2_2dirName, myL1_1dirName, sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL2_1_2dirName, myFoundPath )
    
    # TEST 1 for first of multiple entries in a character vector
    myPath <- c( myL1_1dirName, myL1_2_1dirName, myL2_1_1dirName, myL2_1_2dirName, myL1_2dirName, myL1_2_2dirName )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )

    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkEquals( myL1_1dirName, myFoundPath )		
#
    #myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    #myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )		
    #
    #hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    #myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    #checkEquals( myL1_1dirName, myFoundPath )

    # TEST 2 for first of multiple entries in a character vector (reordered)
    myPath <- c( myL1_2_1dirName, myL2_1_1dirName, myL2_1_2dirName, myL1_2dirName, myL1_2_2dirName, myL1_1dirName )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkEquals( myL2_1_2dirName, myFoundPath )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkEquals( myL2_1_2dirName, myFoundPath )

    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkEquals( myL2_1_2dirName, myFoundPath )		

    #myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    #myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    #checkEquals( myL2_1_2dirName, myFoundPath )		

    #hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    #myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    #checkEquals( myL2_1_2dirName, myFoundPath )
  }
}

    
# test of non-existent directories
"test.rxFindFileInPath.noSuchDirectory" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()

    # check for a path that doesn't exist - character scalar
    myPath <- paste( "NoSuchPath", "NotHereEither", "nope", sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkTrue( is.null( myFoundPath ) )

    # check for a path that doesn't exist - character vector
    myPath <- c( "NoSuchPath", "NotHereEither", "nope" )
    myFoundPath <- rxFindFileInPath( myPath, "foo" )
    checkTrue( is.null( myFoundPath ) )		
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo" )
    checkTrue( is.null( myFoundPath ) )
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkTrue( is.null( myFoundPath ) )				

    myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxSpark, "foo" )
    checkTrue( is.null( myFoundPath ) )				
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo" )
    checkTrue( is.null( myFoundPath ) )		
  }
}

    
# test of non-existent files
"test.rxFindFileInPath.noSuchFile" <- function()
{
  checkTrue( .rxFindFileInPathInitSucceeded )
  
  if ( .rxFindFileInPathInitSucceeded )
  {
    myRxHpcServer <- RxHpcServer( headNode = "somewhere" )
    myRxSpark <- RxSpark()

    # check for a file that doesn't exist
    myPath <- paste( myL2_1_2dirName, sep = .rxFindFilePathSeperatorDelim )
    myFoundPath <- rxFindFileInPath( myPath, "foo1" )
    checkTrue( is.null( myFoundPath ) )
    
    myRxHpcServer <- RxHpcServer( myRxHpcServer, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxHpcServer, "foo1" )
    checkTrue( is.null( myFoundPath ) )

    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxHpcServer)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo1" )
    checkTrue( is.null( myFoundPath ) )	
        
    myRxSpark <- RxSpark( myRxSpark, dataPath=myPath )
    myFoundPath <- rxFindFileInPath( myRxSpark, "foo1" )
    checkTrue( is.null( myFoundPath ) )				
    
    hpcServerJob <- new("RxDistributedJob", hpcServer=myRxSpark)
    myFoundPath <- rxFindFileInPath( hpcServerJob, "foo1" )
    checkTrue( is.null( myFoundPath ) )				
  }		
}

