# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxFindPackage and rxInstalledPackages
#
############################################################################################

#rxFindPackage(package, computeContext = NULL, allNodes = FALSE, lib.loc = NULL, quiet = TRUE,
             #verbose = getOption("verbose"))
      
"test.rxFindPackage.args.package" <- function()
{
  packageLoc <- rxFindPackage(package = "lattice")
  checkTrue(grepl("lattice", packageLoc))	
  
  packageLoc <- rxFindPackage(package = c("lattice", "stats"))
  checkTrue(grepl("lattice", packageLoc[1]))	
  checkTrue(grepl("stats", packageLoc[2]))		
}

"test.rxFindPackage.args.computeContext" <- function()
{
  fakeComputeContext <- RxSpark()
  origContext <- rxSetComputeContext(fakeComputeContext)
  on.exit(rxSetComputeContext(origContext), add = TRUE)
  packageLoc <- rxFindPackage(package = "lattice", computeContext = "local")
  checkTrue(grepl("lattice", packageLoc))	
  
  packageLoc <- rxFindPackage(package = "lattice", computeContext = origContext)
  checkTrue(grepl("lattice", packageLoc))	
  
  rxSetComputeContext(origContext)	
}

"test.rxFindPackage.args.lib.loc" <- function()
{
  myLoc <- R.home(component = "library")
  packageLoc <- rxFindPackage(package = "stats", lib.loc = myLoc)
  checkTrue(grepl("stats", packageLoc))	
}

"test.rxFindPackage.args.quiet" <- function()
{
  packageLoc <- rxFindPackage(package = "mySillyPackage", quiet = TRUE)
  checkEquals(character(0), packageLoc)
  
  res <- try(
    packageLoc <- rxFindPackage(package = "mySillyPackage", quiet = FALSE)
    )
  checkTrue(is(res, "try-error"))
}

#rxInstalledPackages(computeContext = NULL, allNodes = FALSE, lib.loc = NULL,
                    #priority = NULL, noCache = FALSE, fields = NULL,
                    #subarch = NULL)
          
"test.rxInstalledPackages.args.computeContext" <- function()
{
  fakeComputeContext <- RxSpark()
  origContext <- rxSetComputeContext(fakeComputeContext)
  on.exit(rxSetComputeContext(origContext), add = TRUE)
  
  isInstalled <- rxInstalledPackages(computeContext = "local", priority = "base")
  basePackages <- sort(c( "base",  "compiler",  "datasets",  "graphics", "grDevices", "grid",
    "methods",  "parallel",   "splines",     "stats",    "stats4",   "tcltk", 
    "tools",  "utils" ))
    print(isInstalled)
    
  checkEquals( sort(isInstalled), basePackages)

  isInstalled <- rxInstalledPackages(computeContext = origContext, priority = "base")
  checkEquals( sort(isInstalled), basePackages)
  rxSetComputeContext(origContext)	
}		

"test.rxInstalledPackages.args.fields" <- function()
{
  
    # By default, get just a character vector of packages
  isInstalled <- rxInstalledPackages( priority = "base")

  basePackages <- sort(c( "base",  "compiler",  "datasets",  "graphics", "grDevices", "grid",
    "methods",  "parallel",   "splines",     "stats",    "stats4",   "tcltk", 
    "tools",  "utils" ))
    print(isInstalled)
  checkEquals( sort(isInstalled), basePackages, checkNames = FALSE)
    
    # Test R equivalent default
    isInstalled <- rxInstalledPackages( priority = "base", fields = NULL)
   
    defaultFields <- sort(c("Package", "LibPath", "Version", 
        "Priority", "Depends", "Imports", "LinkingTo",
        "Suggests", "Enhances", "License", "License_is_FOSS",
        "License_restricts_use", "OS_type", "MD5sum", "NeedsCompilation",  
        "Built"))
  checkEquals( sort(isInstalled[,"Package"]), basePackages, checkNames = FALSE)
    checkEquals(sort(dimnames(isInstalled)[[2]]), defaultFields)
    
    # Test user-specified 
    
    userFields <- c("Package", "LibPath", "Version", "Built")
    isInstalled <- rxInstalledPackages( priority = "base", fields = userFields)
  checkEquals( sort(isInstalled[,"Package"]), basePackages, checkNames = FALSE)
    checkEquals(dimnames(isInstalled)[[2]], userFields)
}				
