# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxGetVarInfo 
#
############################################################################################

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}
                
"test.rxGetVarInfo.args.varsToKeep" <- function()
{
  # Specify name and location of test data file
   testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")

  # Read the varInfo using varsToKeep
  varInfo1 <- rxGetVarInfo(testXdf, varsToKeep=c("xnum2", "xfac2"))
  varNames1 <- names(varInfo1)
  checkEquals(length(varNames1), 2)
  
  # Use a text data source with new names
  claimsTxtDS <- RxTextData(file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
    colInfo = list(age = list(newName = "newAge"), car.age = list(newName = "newCarAge"),
    cost = list(newName = "newCost")))
  varsToKeep2 <- c("RowNum", "newAge", "newCarAge", "newCost", "number")
  varInfo2 <- rxGetVarInfo(claimsTxtDS, varsToKeep = varsToKeep2, computeInfo = TRUE)
  checkEquals(names(varInfo2), varsToKeep2)

}                

"test.rxGetVarInfo.args.varsToDrop" <- function()
{
  # Specify name and location of sample data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")

  # Read the varInfo
  varInfo <- rxGetVarInfo(testXdf)
  # Get a list of the variable names
  varNames <- names(varInfo)
  checkEquals(length(varNames), 9)
  
  # Read the varInfo using varsToDrop
  
  varInfo1 <- rxGetVarInfo(testXdf, varsToDrop=c("xnum2", "xfac2"))
  varNames1 <- names(varInfo1)
  checkEquals(length(varNames1), 7)
  
}

"test.rxGetVarInfo.args.getValueLabels" <- function()
{
  # Specify name and location of test data file
   testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")

  # Read the varInfo using getValueLabels
  varInfo1 <- rxGetVarInfo(testXdf, getValueLabels=TRUE)
  varInfo2 <- rxGetVarInfo(testXdf, getValueLabels=FALSE)
  checkEquals(varInfo1$xfac2$levels, c("f","c","d","e","i","g","b","j","a","h"))
  checkEquals(is.null(varInfo2$xfac2$levels), TRUE)
}

"test.rxGetVarInfo.data" <- function()
{
   # Compare results of rxGetInfo with data frame and .xdf file
   # using a variety of data types, including data and datetime
    
   dates <- c("02/27/92", "02/27/92", "01/14/92", "02/28/92", "02/01/92",
                "03/27/92", "04/27/02", "06/14/03", "02/28/98", "02/01/04")
            
   datetimes <- c("1999-03-01 01:30:00", "2000-03-02 01:30:00", 
    "2001-03-03 01:30:00", "2002-03-04 01:30:03", "2003-03-27 01:30:00",
    "2004-03-27 01:30:00", "2005-03-27 01:30:00", "2006-03-14 12:30:00", 
    "2007-03-27 23:30:00", "2008-03-15 01:25:00")            

    myDF <- data.frame(
        x=1:10, 
        y=21:30, 
        z =factor(rep(1:2, times=5), labels = c("A", "B")),
    w = as.logical(rep(0:1, times=5)),
        d = as.Date(dates, "%m/%d/%y"),
        dt = as.POSIXct(strptime(datetimes, '%Y-%m-%d %H:%M:%S')),
        dtz = as.POSIXct(strptime(datetimes, '%Y-%m-%d %H:%M:%S', tz="EST"), tz="EST"))
    varInfo <- rxGetVarInfo(myDF)
  
  
  # Create .xdf file and use rxGetVarInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    varInfoXdf <- rxGetVarInfo(myFileXdf)
    
  checkEquals(varInfo, varInfoXdf)
  file.remove(myFileXdf)
}

"test.rxGetVarInfo.data.extraFactorLevels" <- function()
{
    # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    xdfFileName <- tempfile( pattern = ".rxGetVarInfo01Temp", fileext = ".xdf")
    
  myData <- rxImport(inData=inFile, varsToKeep = c("xint1"),
        colInfo = list(xint1 = list(type = "factor", levels = as.character( 1:15))))
    
    rxImport(inData = inFile, outFile = xdfFileName, 
        varsToKeep = c("xint1"),
        colInfo = list(xint1 = list(type = "factor", levels = as.character( 1:15))))
        
  # Read the varInfo
  varInfoDF <- rxGetVarInfo(myData)
  # See RA-4611
    varInfoDF$xint1$low <-  varInfoDF$xint1$low 
    varInfoDF$xint1$high <- varInfoDF$xint1$high 
    varInfoXdf <- rxGetVarInfo( xdfFileName)
    
    checkEquals(varInfoDF, varInfoXdf)
  file.remove(xdfFileName)

}

"test.rxGetVarInfo.args.varsToKeep" <- function()
{
  # Specify name and location of test data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testDF <- rxReadXdf(testXdf)
  
  varsToKeep <- c("xnum2", "xfac2")
  
  varInfo <- rxGetVarInfo(testDF, varsToKeep=varsToKeep)
  varNames <- names(varInfo)
  checkEquals(varNames, varsToKeep)
  
  # Read the varInfo using varsToKeep
  varInfoXdf <- rxGetVarInfo(testXdf, varsToKeep=varsToKeep)
  
  checkEquals(varInfo$xfac2, varInfoXdf$xfac2)
  
  # Note that storage types will differ for xnum2
  checkEquals(varInfo$xnum2$varType, varInfoXdf$xnum2$varType)
  checkEquals(varInfo$xnum2$low, varInfoXdf$xnum2$low)
  checkEquals(varInfo$xnum2$high, varInfoXdf$xnum2$high)
  
  # Try an Xdf data source
  varInfoDS <- rxGetVarInfo(RxXdfData(testXdf), varsToKeep = varsToKeep)
  checkEquals(varInfoDS$xnum2$varType, varInfoXdf$xnum2$varType)
  checkEquals(varInfoDS$xnum2$low, varInfoXdf$xnum2$low)
  checkEquals(varInfoDS$xnum2$high, varInfoXdf$xnum2$high)

}   

"test.rxGetVarInfo.args.varsToDrop" <- function()
{
  # Specify name and location of test data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testDF <- rxReadXdf(testXdf)
  
  varsToDrop <- c("y", "ylogical","ybinint", "xnum1", "xnum2miss", "xint1", "xfac1")
  remainingVars <- c("xnum2", "xfac2")
    
  varInfo <- rxGetVarInfo(testDF, varsToDrop=varsToDrop)
  varNames <- names(varInfo)
  checkEquals(varNames, remainingVars)
  
  # Read the varInfo using varsToKeep
  varInfoXdf <- rxGetVarInfo(testXdf, varsToDrop=varsToDrop)
  
  # Compare
  # .xdf files have zero based factors, so adjust output; no longer 4/4/2014
  #varInfo$xfac2$low <- varInfo$xfac2$low - 1
  #varInfo$xfac2$high <- varInfo$xfac2$high - 1
  checkEquals(varInfo$xfac2, varInfoXdf$xfac2)
  
  # Note that storage types will differ for xnum2
  checkEquals(varInfo$xnum2$varType, varInfoXdf$xnum2$varType)
  checkEquals(varInfo$xnum2$low, varInfoXdf$xnum2$low)
  checkEquals(varInfo$xnum2$high, varInfoXdf$xnum2$high)

} 

"test.rxGetVarInfo.args.getValueLabels" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=21:30, z =factor(rep(1:2, times=5), labels = c("A", "B")),
    w = as.logical(rep(0:1, times=5)))
    varInfo <- rxGetVarInfo(myDF, getValueLabels = FALSE)
  
  # .xdf files have 0 based factors; subtract one for comparison: no longer 4/4/2014
  #varInfo$z$low <- varInfo$z$low - 1
  #varInfo$z$high <- varInfo$z$high - 1
  
  # Create .xdf file and use rxGetVarInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    varInfoXdf <- rxGetVarInfo(myFileXdf, getValueLabels = FALSE)
  
  checkEquals(varInfo, varInfoXdf)
  file.remove(myFileXdf)
}

"test.rxGetVarInfo.datatypes" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=seq(from=21, to=30, by=1))
    infoObj <- rxGetVarInfo(myDF)

  checkEquals(infoObj$x$varType, "integer")
  checkEquals(infoObj$x$storage, "int32")
  checkEquals(infoObj$x$low, 1)
  checkEquals(infoObj$x$high, 10)

  checkEquals(infoObj$y$varType, "numeric")
  checkEquals(infoObj$y$storage, "float64")

  # Create .xdf file and use rxGetVarInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    infoObjXdf <- rxGetVarInfo(myFileXdf)
  
  checkEquals(infoObj, infoObjXdf)

  # Create .xdf data source
  myXdfDS <- RxXdfData( myFileXdf)
  infoObjDS <- rxGetVarInfo( myXdfDS )
  checkEquals(infoObjXdf, infoObjDS)
  
  file.remove(myFileXdf)
  
  # Try a list
  myList <- list(item1 = c(1:10), item2 = "hello", item3 = list(a=1, b=2))
  infoObjList <- rxGetVarInfo(myList)
  checkEquals(infoObjList$item1$varType, "integer")
  checkEquals(infoObjList$item2$varType, "character")
  checkEquals(infoObjList$item3$varType, "list")
  
  # Matrix row and column names
  mdat <- matrix(c(1,2,3, 11,12,13), nrow = 2, ncol=3, byrow=TRUE,
             dimnames = list(c("row1", "row2"),
                               c("C.1", "C.2", "C.3")))
  infoObjMatrix <- rxGetVarInfo(mdat)	
  checkEquals(length(infoObjMatrix), 3)
  checkEquals(infoObjMatrix$C.1$low, 1)
  checkEquals(infoObjMatrix$C.1$high, 11)
  checkEquals(infoObjMatrix$C.2$low, 2)
  checkEquals(infoObjMatrix$C.2$high, 12)
  checkEquals(infoObjMatrix$C.3$low, 3)
  checkEquals(infoObjMatrix$C.3$high, 13)
  
  mdat <- matrix(c("a", "b", "c", "x", "y", "z"), nrow = 2, ncol=3, byrow=TRUE,
             dimnames = list(c("row1", "row2"),
                               c("C.1", "C.2", "C.3")))	
  infoObjMatrix <- rxGetVarInfo(mdat)	
  checkEquals(length(infoObjMatrix), 3)
  infoObjMatrix <- rxGetVarInfo(mdat, varsToDrop = c("C.2"))
  checkEquals(length(infoObjMatrix), 2)
  
  mdat <- matrix(c(1,2,3, 11,12,13), nrow = 2, ncol=3, byrow=TRUE)				
  infoObjMatrix <- rxGetVarInfo(mdat)
  checkEquals(length(infoObjMatrix), 3)
  checkEquals(infoObjMatrix[[1]]$low, 1)
  checkEquals(infoObjMatrix[[1]]$high, 11)
  checkEquals(infoObjMatrix[[2]]$low, 2)
  checkEquals(infoObjMatrix[[2]]$high, 12)
  checkEquals(infoObjMatrix[[3]]$low, 3)
  checkEquals(infoObjMatrix[[3]]$high, 13)			
}

"test.rxGetVarInfo.args.data.NAs" <- function()
{
  myNAs <- rep(NA, 20)
  
  testData <- data.frame(
    xInt = as.integer(myNAs),
    xDouble = as.numeric(myNAs),
    xLogical = as.logical(myNAs)
    )
  testInfo <- rxGetVarInfo(testData)
  checkEquals(testInfo$xInt$low, NA)
  checkEquals(testInfo$xInt$high, NA)
  
  checkEquals(testInfo$xDouble$low, NA)
  checkEquals(testInfo$xDouble$high, NA)
  
  checkEquals(testInfo$xLogical$low, NA)
  checkEquals(testInfo$xLogical$high, NA)
  
  # Now Xdf
  outFile <- tempfile(pattern = ".rxGetVarInfo", fileext = ".xdf")
  on.exit(if(file.exists(outFile)) unlink(outFile), add = TRUE)
    
  rxDataStep(inData = testData, outFile = outFile, rowsPerRead = 8)
  testInfoXdf <- rxGetVarInfo(outFile)
  
  checkEquals(testInfoXdf$xInt$low, as.numeric(NA))
  checkEquals(testInfoXdf$xInt$high, as.numeric(NA))
  
  checkEquals(testInfoXdf$xDouble$low, as.numeric(NA))
  checkEquals(testInfoXdf$xDouble$high, as.numeric(NA))
  
  checkEquals(testInfoXdf$xLogical$low, as.numeric(NA))
  checkEquals(testInfoXdf$xLogical$high, as.numeric(NA))
  
  file.remove(outFile)
}

"test.rxGetVarInfo.args.data" <- function()
{
    # SAS file
    claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsSasSource <- RxSasData(claimsSasFileName)

    varInfoDS <- rxGetVarInfo( claimsSasSource )
    
  # Import the data into a data frame in memory
  claimsIn <- rxImport(claimsSasSource)
  varInfoDF <- rxGetVarInfo(claimsIn)

    compareVarInfos <- function( varInfoDF, varInfoDS )
    {
        varNames <- names(varInfoDF)
        checkEquals(names(varInfoDS), varNames )
        for (i in seq_along( varNames))
        {
            checkEquals(varInfoDS[[ varNames[i] ]]$varType, varInfoDF[[ varNames[i] ]]$varType)     
        }
     }
     compareVarInfos( varInfoDF, varInfoDS)
    
    # Try an SPSS file
    claimsSpssFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
  claimsSpssSource <- RxSpssData(claimsSpssFileName)

    varInfoDS <- rxGetVarInfo( claimsSpssSource )
    compareVarInfos( varInfoDF, varInfoDS)
    
    # Fixed format
    # Has . instead of _ in name
    newDF <- claimsIn
    newDF$RowNum <- as.numeric(newDF$RowNum)
    names(newDF)[3] <- "car.age"
  varInfoDF <- rxGetVarInfo(newDF)
    claimsFixedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sts")
  claimsFixedSource <- RxTextData(claimsFixedFileName)
    varInfoDS <- rxGetVarInfo( claimsFixedSource )
    compareVarInfos( varInfoDF, varInfoDS)
  
    # Fast Fixed format
  claimsFFixedSource <- RxTextData(claimsFixedFileName, useFastRead = TRUE)
    varInfoFFDS <- rxGetVarInfo( claimsFFixedSource )
    compareVarInfos( varInfoDF, varInfoFFDS)
    
    # Delimited format
    newDF$RowNum <- as.integer(newDF$RowNum)
    varInfoDF <- rxGetVarInfo(newDF)
    claimsDelimitedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  claimsDelimitedSource <- RxTextData(claimsDelimitedFileName)
    varInfoDS <- rxGetVarInfo(claimsDelimitedSource)
    compareVarInfos( varInfoDF, varInfoDS)
    
    # SQLITE - useFastRead
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE) 
    
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    odbcDS <-  RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)
    names(newDF)[3] <- "car_age"
    varInfoDF <- rxGetVarInfo(newDF)            
    varInfoDS <- rxGetVarInfo(odbcDS)
    compareVarInfos( varInfoDF, varInfoDS)
    
    # SQLITE - useFastRead
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                          "claims.sqlite"), useFastRead = FALSE),
          silent = TRUE) 
    
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    odbcDS <-  RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString, useFastRead = FALSE)           
    varInfoDS <- rxGetVarInfo(odbcDS)
    compareVarInfos( varInfoDF, varInfoDS)    
    
}

"test.rxGetVarInfo.args.data.datasource.colInfo" <- function()
{
    
    sasDataFile <- file.path(rxGetOption("sampleDataDir"),"claims.sas7bdat")
    typeLevels <- c("C","A","B","E")
    sasDS <- RxSasData(sasDataFile, colClasses=c(RowNum="integer"),
        colInfo=list(
            cost=list(type="numeric"),
            age=list(type="factor"), 
            car_age=list(type="factor"),type=list(type="factor",levels=typeLevels)
        )
    ) 
    myVarInfo <- rxGetVarInfo(sasDS) 
    checkEquals(myVarInfo$RowNum$varType, "integer")
    checkEquals(myVarInfo$cost$varType, "numeric")
    checkEquals(myVarInfo$age$varType, "factor")
    checkEquals(myVarInfo$car_age$varType, "factor")
    checkEquals(myVarInfo$type$varType, "factor")
    checkEquals(myVarInfo$type$levels, typeLevels)
}

###
# Multiple data sources

"test.rxGetVarInfo.args.data.multiple.sources" <- function()
{   
    DEACTIVATED("Does not run in parallel on ShipR. See tfs bug 1745")
  #####
  # Combinations of data sources using iris, airquality, and attitude data frames
  
  # list of data frames
  df.only.list <- rxGetVarInfo(list(iris, airquality, attitude))

  # list of .xdf paths
  irisXDF <- tempfile(pattern = "iris", fileext = ".xdf")
  airXDF <- tempfile(pattern = "airquality", fileext = ".xdf")
  attXDF <- tempfile(pattern = "attitude", fileext = ".xdf")
  irisXDF <- "iris.xdf"
  airXDF <- "air.xdf"
  attXDF <- "att.xdf"
  rxDataStep(iris, irisXDF, overwrite = TRUE)
  rxDataStep(airquality, airXDF, overwrite = TRUE)
  rxDataStep(attitude, attXDF, overwrite = TRUE)

  xdf.only.list <- rxGetVarInfo(list(irisXDF, airXDF, attXDF))
  xdf.only.c <- rxGetVarInfo(c(irisXDF, airXDF, attXDF))

  # list of RxXdfData objects
  irisDS <- RxXdfData(irisXDF)
  airDS <- RxXdfData(airXDF)
  attDS <- RxXdfData(attXDF)

  ds.only.list <- rxGetVarInfo(list(irisDS, airDS, attDS))
  ds.only.c <- rxGetVarInfo(c(irisDS, airDS, attDS))

  # list of heterogenous data sources
  ds.xdf.df <- rxGetVarInfo(list(irisDS, airXDF, attitude))

  # variable assignment calls
  x <- iris
  y <- airquality
  z <- attitude
  df.only.list.assigned.vars <- rxGetVarInfo(list(x, y, z))

  x <- irisDS
  y <- airXDF
  z <- attitude
  ds.xdf.df.assigned.vars <- rxGetVarInfo(list(x, y, z))
  
  # define standards for comparison
  iris.df.std <- rxGetVarInfo(iris)
  air.df.std <- rxGetVarInfo(airquality)
  att.df.std <- rxGetVarInfo(attitude)

  iris.xdf.std <- rxGetVarInfo(irisXDF)
  air.xdf.std <- rxGetVarInfo(airXDF)
  att.xdf.std <- rxGetVarInfo(attXDF)
    
  iris.ds.std <- rxGetVarInfo(irisDS)
  air.ds.std <- rxGetVarInfo(airDS)
  att.ds.std <- rxGetVarInfo(attDS)

  # Test and compare the results
  checkEquals(iris.df.std, df.only.list[[1L]])
  checkEquals(air.df.std, df.only.list[[2L]])
  checkEquals(att.df.std, df.only.list[[3L]])
  checkEquals(names(df.only.list), c("iris","airquality","attitude"))
    
  checkEquals(iris.xdf.std, xdf.only.list[[1L]])
  checkEquals(air.xdf.std, xdf.only.list[[2L]])
  checkEquals(att.xdf.std, xdf.only.list[[3L]])
  checkEquals(names(xdf.only.list), c("irisXDF","airXDF","attXDF"))
  
  checkEquals(iris.xdf.std, xdf.only.c[[1L]])
  checkEquals(air.xdf.std, xdf.only.c[[2L]])
  checkEquals(att.xdf.std, xdf.only.c[[3L]])
  checkEquals(names(xdf.only.c), c("iris.xdf","air.xdf","att.xdf"))

  checkEquals(iris.ds.std, xdf.only.list[[1L]])
  checkEquals(air.ds.std, xdf.only.list[[2L]])
  checkEquals(att.ds.std, xdf.only.list[[3L]])
  checkEquals(names(ds.only.list), c("irisDS","airDS","attDS"))
  
  checkEquals(iris.ds.std, xdf.only.c[[1L]])
  checkEquals(air.ds.std, xdf.only.c[[2L]])
  checkEquals(att.ds.std, xdf.only.c[[3L]])
  checkEquals(names(ds.only.c), c("irisDS","airDS","attDS"))

  checkEquals(iris.ds.std, ds.xdf.df[[1L]])
  checkEquals(air.xdf.std, ds.xdf.df[[2L]])
  checkEquals(att.df.std, ds.xdf.df[[3L]])
  checkEquals(names(ds.xdf.df), c("irisDS","airXDF","attitude"))

  nocomp <- which(names(iris.df.std) %in% c("objName"))
  checkEquals(iris.df.std[-nocomp], df.only.list.assigned.vars[[1L]][-nocomp])
  checkEquals(air.df.std[-nocomp], df.only.list.assigned.vars[[2L]][-nocomp])
  checkEquals(att.df.std[-nocomp], df.only.list.assigned.vars[[3L]][-nocomp])
  checkEquals(names(df.only.list.assigned.vars), c("x","y","z"))

  checkEquals(iris.ds.std, ds.xdf.df.assigned.vars[[1L]])
  checkEquals(air.xdf.std, ds.xdf.df.assigned.vars[[2L]])
  nocomp <- which(names(att.df.std) %in% c("objName"))
  checkEquals(att.df.std[-nocomp], ds.xdf.df.assigned.vars[[3L]][-nocomp])
  checkEquals(names(ds.xdf.df.assigned.vars), c("x","y","z"))
  
  #####
  # rxSplitXdf case
  splitPaths <- rxSplit(iris, splitByFactor = "Species", outFilesBase = file.path(tempdir(),"iris1"), 
      reportProgress = 0, verbose = 0, overwrite = TRUE)

  rxSplit.df <- rxGetVarInfo(splitPaths)
  
  # check values
  checkEquals(rxSplit.df[[1L]], rxSplit.df[[2L]])
  checkEquals(rxSplit.df[[2L]], rxSplit.df[[3L]])
  checkEquals(names(rxSplit.df), as.vector(sapply(splitPaths, slot, "file")))
  
  #####
  # Direct inline definition
  
  direct.list <- rxGetVarInfo(list(data.frame(cow = 1:5, horse = 2:6), 
                  data.frame(pig = letters, goat = 1:26, dog = 26:1)))
  
  # check values
    checkEquals(direct.list[[1L]][["cow"]][["low"]], 1L)
    checkEquals(direct.list[[1L]][["cow"]][["high"]], 5L)
    checkEquals(direct.list[[1L]][["cow"]][["varType"]], "integer")
    checkEquals(direct.list[[1L]][["horse"]][["low"]], 2L)
    checkEquals(direct.list[[1L]][["horse"]][["high"]], 6L)
    checkEquals(direct.list[[1L]][["horse"]][["varType"]], "integer")
  
    checkEquals(direct.list[[2L]][["pig"]][["levels"]], letters)
    checkEquals(direct.list[[2L]][["pig"]][["varType"]], "factor")
    checkEquals(direct.list[[2L]][["goat"]][["low"]], 1L)
    checkEquals(direct.list[[2L]][["goat"]][["high"]], 26L)
    checkEquals(direct.list[[2L]][["goat"]][["varType"]], "integer")
    checkEquals(direct.list[[2L]][["dog"]][["low"]], 1L)
    checkEquals(direct.list[[2L]][["dog"]][["high"]], 26L)
    checkEquals(direct.list[[2L]][["dog"]][["varType"]], "integer")

  #####
  # Clean up
  if (file.exists(irisXDF)) file.remove(irisXDF)
  if (file.exists(airXDF)) file.remove(airXDF)
  if (file.exists(attXDF)) file.remove(attXDF)
  invisible(sapply(splitPaths, function(x) if (file.exists(x@file)) file.remove(x@file)))
}

"test.rxGetVarInfo.args.computeInfo" <- function()
{
  
  claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsXdfFileName <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  claimsSasSource <- RxSasData(claimsSasFileName, rowsPerRead = 20)

    varInfoDS1 <- rxGetVarInfo( claimsSasSource, computeInfo = TRUE )
  
  varInfoDS2 <- rxGetVarInfo( claimsSasSource, computeInfo = FALSE )
  
  varInfoXdf <- rxGetVarInfo( claimsXdfFileName)
  checkEquals(varInfoDS1$cost$low, varInfoXdf$cost$low)
  checkEquals(varInfoDS1$cost$high, varInfoXdf$cost$high)
  checkEquals(varInfoDS1$number$low, varInfoXdf$number$low)
  checkEquals(varInfoDS1$number$high, varInfoXdf$number$high)
  checkTrue(is.na(varInfoDS2$cost$low))	
}


"test.rxGetVarInfo.args.data.multiple.sources" <- function()
{   
  #####
  # Combinations of data sources using iris, airquality, and attitude data frames
  
  # list of .xdf paths
  irisXDF <- tempfile(pattern = "iris", fileext = ".xdf")
  airXDF <- tempfile(pattern = "airquality", fileext = ".xdf")
  attXDF <- tempfile(pattern = "attitude", fileext = ".xdf")
  irisXDF <- "iris.xdf"
  airXDF <- "air.xdf"
  attXDF <- "att.xdf"
  rxDataStep(iris, irisXDF, overwrite = TRUE)
  rxDataStep(airquality, airXDF, overwrite = TRUE)
  rxDataStep(attitude, attXDF, overwrite = TRUE)

  xdf.only.list <- rxGetVarInfo(list(irisXDF, airXDF, attXDF))
  xdf.only.c <- rxGetVarInfo(c(irisXDF, airXDF, attXDF))

  # list of RxXdfData objects
  irisDS <- RxXdfData(irisXDF)
  airDS <- RxXdfData(airXDF)
  attDS <- RxXdfData(attXDF)

  ds.only.list <- rxGetVarInfo(list(irisDS, airDS, attDS))
  ds.only.c <- rxGetVarInfo(c(irisDS, airDS, attDS))

  # list of heterogenous data sources
  ds.xdf.ds <- rxGetVarInfo(list(irisDS, airXDF, attDS))

  # variable assignment calls
  x <- irisDS
  y <- airXDF
  z <- attDS
  ds.xdf.df.assigned.vars <- rxGetVarInfo(list(x, y, z))
  
  # define standards for comparison
  iris.xdf.std <- rxGetVarInfo(irisXDF)
  air.xdf.std <- rxGetVarInfo(airXDF)
  att.xdf.std <- rxGetVarInfo(attXDF)
    
  iris.ds.std <- rxGetVarInfo(irisDS)
  air.ds.std <- rxGetVarInfo(airDS)
  att.ds.std <- rxGetVarInfo(attDS)

  # Test and compare the results
  checkEquals(iris.xdf.std, xdf.only.list[[1L]])
  checkEquals(air.xdf.std, xdf.only.list[[2L]])
  checkEquals(att.xdf.std, xdf.only.list[[3L]])
  checkEquals(names(xdf.only.list), c("irisXDF","airXDF","attXDF"))
  
  checkEquals(iris.xdf.std, xdf.only.c[[1L]])
  checkEquals(air.xdf.std, xdf.only.c[[2L]])
  checkEquals(att.xdf.std, xdf.only.c[[3L]])
  checkEquals(names(xdf.only.c), c("iris.xdf","air.xdf","att.xdf"))

  checkEquals(iris.ds.std, xdf.only.list[[1L]])
  checkEquals(air.ds.std, xdf.only.list[[2L]])
  checkEquals(att.ds.std, xdf.only.list[[3L]])
  checkEquals(names(ds.only.list), c("irisDS","airDS","attDS"))
  
  checkEquals(iris.ds.std, xdf.only.c[[1L]])
  checkEquals(air.ds.std, xdf.only.c[[2L]])
  checkEquals(att.ds.std, xdf.only.c[[3L]])
  checkEquals(names(ds.only.c), c("irisDS","airDS","attDS"))

  checkEquals(iris.ds.std, ds.xdf.ds[[1L]])
  checkEquals(air.xdf.std, ds.xdf.ds[[2L]])
  checkEquals(att.ds.std, ds.xdf.ds[[3L]])
  checkEquals(names(ds.xdf.ds), c("irisDS","airXDF","attDS"))
  
  #####
  # rxSplitXdf case
  splitPaths <- rxSplit(iris, splitByFactor = "Species", outFilesBase = file.path(tempdir(),"iris2"), 
      reportProgress = 0, verbose = 0, overwrite = TRUE)

  rxSplit.df <- rxGetVarInfo(splitPaths)
  
  # check values
  checkEquals(rxSplit.df[[1L]], rxSplit.df[[2L]])
  checkEquals(rxSplit.df[[2L]], rxSplit.df[[3L]])
  checkEquals(names(rxSplit.df), as.vector(sapply(splitPaths, slot, "file")))
  
  # check names
  checkEquals(names(rxGetInfo(list(foo = iris, airquality, bar = attitude))), c("foo","airquality","bar"))
  checkEquals(names(rxGetInfo(list(iris, airquality, bar = attitude))), c("iris","airquality","bar"))
  checkEquals(names(rxGetInfo(list(one = iris, one = airquality, two = attitude))), c("one","one","two"))
  
  #####
  # Clean up
  if (file.exists(irisXDF)) file.remove(irisXDF)
  if (file.exists(airXDF)) file.remove(airXDF)
  if (file.exists(attXDF)) file.remove(attXDF)
  invisible(sapply(splitPaths, function(x) if (file.exists(x@file)) file.remove(x@file)))
}
