# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxHistogram
#
############################################################################################
.setUp <- function()
{

}

.tearDown <- function()
{
}

"test.rxHistogram.args.formula" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
  # These should all work
    rxHistogram(~ age, data = censusWorkers, main="Single Histogram")
    rxHistogram(~ age | state, data = censusWorkers, main="Conditioned by State")
    rxHistogram(~ age | sex + state, data = censusWorkers, main="Conditioned by State and Sex")
    rxHistogram(~ F(age) | sex + state, data = censusWorkers, main="F(age) Conditioned by State and Sex")
    rxHistogram(~ F ( age) | sex + state, data = censusWorkers, main="F ( age) Conditioned by State and Sex")
  rxHistogram(~ state, data = censusWorkers, main="Single Histogram: Factor")
  
  # Having a dependent variable should fail
  res <- try(rxHistogram(age~state, data=censusWorkers), silent=TRUE)
    checkTrue(is(res,"try-error"))
  
  #Multiple independent variables should fail
  res <- try(rxHistogram(~age+state, data=censusWorkers), silent=TRUE)
    checkTrue(is(res,"try-error"))
  
  # Interacted independent variables should fail
  res <- try(rxHistogram(~age:state, data=censusWorkers), silent=TRUE)
    checkTrue(is(res,"try-error"))
   
}

"test.rxHistogram.args.data.nonxdf" <- function()
{
    # SAS file
    claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsSasSource <- RxSasData(claimsSasFileName)
    rxHistogram(~cost, data = claimsSasSource, title = "SAS data") 
   
    # SPSS file
    claimsSpssFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
  claimsSpssSource <- RxSpssData(claimsSpssFileName)
    rxHistogram(~cost, data=claimsSpssSource, title = "SPSS data")
    
    # Fixed format
    claimsFixedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sts")
  claimsFixedSource <- RxTextData(claimsFixedFileName)
    rxHistogram(~cost, data=claimsFixedSource, title = "Fixed Format data")
    
    # Fast Fixed format
    claimsFixedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sts")
  claimsFixedSource <- RxTextData(claimsFixedFileName, 
        useFastRead = TRUE)
    rxHistogram(~cost, data=claimsFixedSource, title = "Fast Fixed Format data")
  
    # Delimited format
    claimsDelimitedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  claimsDelimitedSource <- RxTextData(claimsDelimitedFileName)
    rxHistogram(~cost, data=claimsFixedSource, title = "Delimited Text data")
}

"test.rxHistogram.args.blocksPerRead" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    rxHistogram(~ age, data = censusWorkers, blocksPerRead=2, main="Single Histogram: blocksPerRead=2")

}

"test.rxHistogram.args.numBreaks" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    rxHistogram(~ age | state + sex, data = censusWorkers, main="Default numBreaks")
    rxHistogram(~ age | state + sex, data = censusWorkers, numBreaks=25, main="numBreaks = 25") 
    rxHistogram( ~ weight | feed, data = chickwts, numBreaks = 6) 
  if ("lattice" %in% .packages())	
  {
    # Uses data from lattice package
    rxHistogram( ~ height | voice.part, data = singer)
    rxHistogram( ~ height | voice.part, data = singer, numBreaks=4) 
  }
 }

"test.rxHistogram.args.startVal" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    rxHistogram(~ age | state + sex, data = censusWorkers, numBreaks=20, 
        main="Default startVal, numBreaks=20")
    rxHistogram(~ age | state + sex, data = censusWorkers, numBreaks=20, startVal=40, 
        main="startVal=40, numBreaks=20")
    rxHistogram(~ age | state + sex, data = censusWorkers, startVal=40, 
        main="startVal=40")  
    rxHistogram(~ age | state + sex, data = censusWorkers, startVal=40, endVal=50, 
        main="startVal=40, endVal=50")  
    
}

"test.rxHistogram.args.endVal" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")  
    rxHistogram(~ age | state + sex, data = censusWorkers, numBreaks=11, endVal=50,
        main="endVal=50, numBreaks=11") 
    
}

"test.rxHistogram.data.smallDF" <- function()
{
    myData <- data.frame(x=1:5)
    rxHistogram(~x, data=myData)
    myData <- data.frame(x=1:10)
    rxHistogram(~x, data=myData)
    myData <- data.frame(x=1:1000)
    rxHistogram(~x, data=myData)
    myData <- data.frame(x=1:10000)
    rxHistogram(~x, data=myData)

}


"test.rxHistogram.args.levelsToDrop" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data=censusWorkers, main="All levels") 
    rxHistogram(~state, data = censusWorkers, levelsToDrop=c("Indiana"), 
        main="levelsToDrop - Indiana")  
    rxHistogram(~state, data = censusWorkers, levelsToDrop=c("Indiana"), 
        title="levelsToDrop - Indiana", yAxisMinMax=c(1000, 8000), subtitle="RevoScaleR")  
    rxHistogram(~state, data = censusWorkers, levelsToDrop=c("Indiana", "Washington"), 
        main="levelsToDrop - Indiana & Washington")  
    
}

"test.rxHistogram.args.levelsToKeep" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data=censusWorkers, main="All levels") 
    rxHistogram(~state, data = censusWorkers, levelsToKeep=c("Indiana"), 
        main="levelsToKeep - Indiana")  
    rxHistogram(~state, data = censusWorkers, levelsToKeep=c("Indiana", "Washington"), 
        main="levelsToKeep - Indiana & Washington")  
    rxHistogram(~state| sex, data = censusWorkers,  
        main="Conditioned by sex, All levels")  
    rxHistogram(~state| sex, data = censusWorkers, levelsToKeep=c("Indiana", "Washington"), 
        main="levelsToKeep - Indiana & Washington")  
     
}

"test.rxHistogram.args.pweights" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  main="No weights") 
    rxHistogram(~state, data= censusWorkers, pweights="perwt", main="pweights=perwt") 
    
}

"test.rxHistogram.args.histType" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  main="Default Type (Counts)") 
    rxHistogram(~state, data= censusWorkers, histType="Percent", main="Type = Percent") 
    rxHistogram(~age , data= censusWorkers, histType="Percent", main="Type = Percent") 
    rxHistogram(~age , data= censusWorkers, numBreaks=10, histType="Percent", main="Type = Percent") 
    rxHistogram(~age |state, data= censusWorkers, histType="Percent", main="Type = Percent")
    rxHistogram(~F(age) |state, data= censusWorkers, histType="Percent", main="Type = Percent")
    
}

"test.rxHistogram.args.fillColor" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  fillColor="red", main="fillColor = Red (factor)") 
    rxHistogram(~age, data= censusWorkers, fillColor="red", main="fillColor = Red (numeric)") 
}

"test.rxHistogram.args.lineColor" <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  lineColor="red", main="lineColor = Red (factor)") 
    rxHistogram(~age, data= censusWorkers, lineColor="red", main="lineColor = Red (numeric)")
   
}

test.rxHistogram.args.lineWidth <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  lineColor="red", lineWidth=5, 
         main="lineColor = red, lineWidth=5: (factor)") 
    rxHistogram(~age, data= censusWorkers, lineColor="red", lineWidth=5,
        main="lineColor = red, lineWidth=5, (numeric)") 
    rxHistogram(~age, data= censusWorkers, lineColor="red", lwd=5,
        main="lineColor = red, lwd=5, (numeric)") 
    
}

test.rxHistogram.args.lineStyle <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers,  lineColor="steelblue1", lineWidth=2, 
        lineStyle = "dashed",
         main="lineColor = steelblue1, lineWidth=2, lineStyle=dashed: (factor)") 
    rxHistogram(~age, data= censusWorkers, lineColor="steelblue1", lineWidth=2,
        lineStyle = "dotted",
        main="lineColor = steelblue1, lineWidth=2, lineStyle=dotted (numeric)") 
    rxHistogram(~age, data= censusWorkers, lineColor="steelblue1", lineWidth=2,
        lineStyle = "blank",
        main="blank lineStyle (numeric)") 
     rxHistogram(~age, data= censusWorkers, lineColor="steelblue1", lineWidth=2,
        lty = "dotted",
        main="blank lineStyle (numeric), lty=dotted") 
    
}

test.rxHistogram.args.plotAreaColor <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers, plotAreaColor="black",
         main="plotAreaColor = black (factor)") 
    rxHistogram(~age, data= censusWorkers, plotAreaColor="mistyrose1",
        main="plotAreaColor = mistyrose1 (numeric)") 
    
}

test.rxHistogram.args.gridColor <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
         main="plotAreaColor=skyblue1, gridColor=royalblue4 (factor)") 
    rxHistogram(~age, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
        main="plotAreaColor=skyblue1, gridColor=royalblue4 (numeric)") 
    
}

test.rxHistogram.args.gridLineWidth <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
        gridLineWidth=5,
         main="plotAreaColor=skyblue1, gridColor=royalblue4, gridLineWidth=5 (factor)") 
    rxHistogram(~age, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
        gridLineWidth=5,
        main="plotAreaColor=skyblue1, gridColor=royalblue4, gridLineWidth=5 (numeric)")
     
}

test.rxHistogram.args.gridLineStyle <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
        gridLineStyle="dotted",
         main="plotAreaColor=skyblue1, gridColor=royalblue4, gridLineStyle=dotted (factor)") 
    rxHistogram(~age, data= censusWorkers, plotAreaColor="skyblue1", gridColor="royalblue4",
      gridLineStyle="dashed",
        main="plotAreaColor=skyblue1, gridColor=royalblue4, gridLineStyle=dashed (numeric)") 
  
}

test.rxHistogram.args.xNumTicks <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~age, data= censusWorkers, xNumTicks=5,
        main="xNumTicks=5") 
    
}

test.rxHistogram.args.yNumTicks <- function()
{
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    rxHistogram(~state, data= censusWorkers, yNumTicks=15,
        main="yNumTicks=15 (factor)") 
    rxHistogram(~age, data= censusWorkers, yNumTicks=5,
        main="yNumTicks=5 (numeric)") 
}

test.rxHistogram.args.xTitle <- function()
{
    testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    rxHistogram(~xfac2, data=testData)
    rxHistogram(~xfac2, data=testData, xTitle="Second Factor Variable", subtitle="My subtitle")
}

test.rxHistogram.args.par <- function()
{
     testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
     rxHistogram(~xint1, data=testData, ylab="My ylab title", xlab="My xlab title", lwd=5, lty="dotted",
        main="Main Title", sub="Sub title")
}

test.rxHistogram.args.yTitle <- function()
{
    testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    rxHistogram(~xint1, data=testData)
    rxHistogram(~xint1, data=testData, yTitle="How Many Are There?")
}

"test.rxHistogram.args.rowSelection" <- function()
{  
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    rxHistogram(~ age, data = censusWorkers, fillColor="red", main="Single Histogram")
    rxHistogram(~ age, data = censusWorkers, fillColor="blue", rowSelection=state=="Washington", main="Just Washington")
    rxHistogram(~ age, data = censusWorkers, fillColor="pink", rowSelection=state=="Washington" & sex=="Female", main="Just Washington Females")
  
  if ("lattice" %in% .packages())	
  {
    testData <- data.frame(x = c(0, 0, 0, 0, .1, .2, .3, .4, 6, 7, 8, 9))
    histOut1 <- rxHistogram(~x, data = testData)
    checkEquals(sum(histOut1$panel.args[[1]]$y), 12)
    
    histOut2 <- rxHistogram(~x, data = testData, rowSelection = x > 0)
    checkEquals(sum(histOut2$panel.args[[1]]$y), 8)
    
    histOut3 <- rxHistogram(~F(x), data = testData)
    checkEquals(sum(histOut3$panel.args[[1]]$y), 12)
  }
}


"test.rxHistogram.data.dataframe" <- function()
{
  censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
  censusWorkersDF <- rxReadXdf(censusWorkers, returnDataFrame=TRUE)
  rxHistogram(~wkswork1|state, data=censusWorkersDF)
  rxHistogram(~incwage|sex,data=censusWorkersDF) 
  if ("lattice" %in% .packages())	
  {
    rxHistogram(~incwage|sex, histType="Percent", endVal=200000, numBreaks=15, 
      layout=c(1,2), data=censusWorkersDF) 
  }
  rxHistogram(~wkswork1|sex,data=censusWorkersDF) 
    rxHistogram(~F(age)|sex, data=censusWorkersDF)
}

"test.rxHistogram.data.smallValues" <- function()
{  
    set.seed(100)
    pid <- 1:100
    dist1 <- rnorm(100,0.0003,0.00001)
    data1 <- as.data.frame(cbind(pid,dist1))
    rxHistogram(~dist1, data=data1, lineColor="black")
}

"test.rxHistogram.args.maxNumPanels" <- function()
{  
    testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  # This should work with 10 levels by default
    rxHistogram(~xnum1|xfac1, data=testData)
  
  if ("lattice" %in% .packages())	
  {
    # This should fail
    res <- try(rxHistogram(~xnum1|xfac1, maxNumPanels=5, data=testData), silent=TRUE)
    checkTrue(is(res,"try-error"))
    
    # This should fail: each factor has 10 levels
    res <- try(rxHistogram(~xnum1|xfac1+ xfac2, maxNumPanels=20, data=testData), silent=TRUE)
    checkTrue(is(res,"try-error"))	
    
    numCols <- 5
    numRows <- 4
    rxHistogram(~xnum1|xfac1+ xfac2, maxNumPanels=NULL, data=testData, 
      layout=c(numCols, numRows), blocksPerRead=3)
  }
  
}

"test.rxHistogram.args.inFunction" <- function()
{
  set.seed(100)
  myDF <- data.frame(x=rnorm(100))
  getLineColor <- function(i)
  {
    return(i+1)
  }
  myHistogram <- function(myDF, lineColor)
  {
    rxHistogram(~x, data=myDF, lineColor=lineColor)
  }
  
  myFunction <- function(myDF)
  {
    for (i in 1:2)
    {
      myHistogram(myDF, lineColor=getLineColor(i))	
    }
  }
  myFunction(myDF)
}

"test.rxHistogram.data.factorLabels" <- function()
{
  xFac1 <- factor(c(1,2,4,3,2,3,1,2,1,4), levels = 1:4, 
    c("This is a Really Long Factor Label 1", 
      "This is a Really Long Factor Label 2",
      "This is a Really Long Factor Label 3", 
      "This is a Really Long Factor Label 4"))
  myData <- data.frame(x=1:10, xFac1=xFac1)
  # Check for angled labels
  rxHistogram(~xFac1, data=myData)

}

"test.rxHistogram.data.logical" <- function()
{
  xLogical <- as.logical(c(1,0,1,1,1,1,0,1,0,1))
  xFactor <- factor(rep(c(1,2), times=10), labels=c("FALSE", "TRUE"))
  xInt <- as.integer(rep(c(1,2), times=10))
  xLogical2 <-as.logical(rep(c(0,1), times=10))
  xLogical3 <-as.logical(rep(c(0,1), each=10))
  myData <- data.frame(xFactor = xFactor, xLogical = xLogical, xLogical2 = xLogical2, 
    xLogical3=xLogical3, xInt=xInt)
  rxHistogram(~xLogical, data=myData)
  rxHistogram(~xLogical|xFactor, data=myData)
  rxHistogram(~xLogical|xInt, data=myData)
  rxHistogram(~xLogical|F(xLogical2), data=myData)
  rxHistogram(~xLogical|xLogical2, data=myData)
  rxHistogram(~xLogical2|xLogical, data=myData, histType="Percent")
}

"test.rxHistogram.data.F.low" <- function()
{
  testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    rxHistogram(~xnum1|F(xint1,low=7,high=12), data=testData)
}

"test.rxHistogram.args.transforms" <- function()
{
  testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    rxHistogram(~xnew, data = testData,
         transforms = list(xnew = xnum1 + xnum2),
        title = "Histogram with transforms")
    
    rxHistogram(~xnew, data = testData, 
        transforms = list(xnew = xnum1 + xnum2),
        rowSelection = ylogical == TRUE,
        title = "Histogram with transforms and rowSelection")    
    
    rxHistogram(~xnew, data = testData, pweights = "xint1",
        transforms = list(xnew = xnum1 + xnum2),
        title = "Histogram with transforms and pweights")
}

"test.rxHistogram.args.transformFunc" <- function()
{
  testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    myFunc <- function( dataList)
    {
        dataList$xnew = (dataList$xnum1 + dataList$xnum2)^2
        return( dataList)
    }
    
    rxHistogram(~xnew, data = testData,
        transformFunc = myFunc, transformVars = c("xnum1", "xnum2"),
        title = "Histogram with transformFunc")
}

"test.rxHistogram.args.scales" <- function()
{
  testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
    rxHistogram(~xnum1, data = testData,
    scales = list(x = list(rot = 30), y = list(rot = 30)),
        title = "Histogram with Rotated Axis Labels")
  
  rxHistogram(~xfac2, data = testData,
    scales = list(x = list(rot = 30), y = list(rot = 30)),
        title = "Factor Histogram with Rotated Axis Labels")
}

"test.rxHistogram.args.panel" <- function()
{
  testData <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
  if ("lattice" %in% .packages())	
  {	
    rxHistogram(~xnum1, data = testData,
      panel = function(x, y, ...)
      {
        panel.spline(x,y)

      },		
      title = "Histogram with spline")
  
    rxHistogram(~xfac2, data = testData,
      panel = function(x, y, ...)
      {
        splineVals <- spline(x,y)
        panel.xyplot(splineVals$x,splineVals$y, type = "l")
      },	
      title = "Factor Histogram with Spline")
  }
}


"test.rxHistogram.args.print" <- function()
{
  testData <- data.frame(x = 1:10,  xfac = factor(rep(1:5, times = 2)))
  testFile <- tempfile(pattern = ".rxHistogram", fileext = ".pdf")
  if (file.exists(testFile)) (file.remove(testFile))
  pdf(file = testFile)
  testPlotObj <- rxHistogram(~x, data = testData, print = FALSE)
  dev.off()
  fileSizePrintFalse <- file.info(testFile)$size
  file.remove(testFile)
  
  pdf(file = testFile)
  testPlotObj <- rxHistogram(~xfac, data = testData, print = TRUE)
  dev.off()
  fileSizePrintTrue <- file.info(testFile)$size
  checkTrue(fileSizePrintTrue > fileSizePrintFalse)
  file.remove(testFile)
  
  # Repeat with factor
  pdf(file = testFile)
  testPlotObj <- rxHistogram(~xfac, data = testData, print = FALSE)
  dev.off()
  fileSizePrintFalse <- file.info(testFile)$size
  file.remove(testFile)
  
  pdf(file = testFile)
  testPlotObj <- rxHistogram(~xfac, data = testData, print = TRUE)
  dev.off()
  fileSizePrintTrue <- file.info(testFile)$size
  checkTrue(fileSizePrintTrue > fileSizePrintFalse)
  file.remove(testFile)
    
  # Known bug in R (14991) that temporary files are left behind when 
  # creating pdf
  
  unlink(list.files(path=tempdir(), pattern="^pdf.", full.names=TRUE))

}

 
  
    
  