# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxImport, testing for functionality of deprecated rxImportToXdf
#
############################################################################################

.setUp <- function()
{

}

.tearDown <- function()
{
}

.rxSafeReadXdf <- function(file, ...)
{
    ans <- rxReadXdf(file = file, ...)
    whichSinglePrecision <-
      which(sapply(rxGetVarInfo(file),
                   function(x) x[["storage"]] == "float32"))
    for (i in whichSinglePrecision)
    {
        ans[[i]] <- signif(ans[[i]], 4)
    }
    ans
}


### SAS DATA SOURCE
"test.rxImportToXdf.RxSasData" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    for (sasDS in list(RxSasData(sasFileName), sasFileName))
    {
        for (xdfDS in list(RxXdfData(xdfFileName), xdfFileName))
        {
            rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
            checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                           c(Y = "float64", Y2 ="float64", YLOGICAL = "string",
                             YBININT = "float64", XNUM1 = "float64",
                             XNUM2 = "float64", XNUM2MISS = "float64",
                             XINT1 = "float64", XFAC1 = "string",
                             XFAC2 = "string", "Z_CHAR1" = "string",
                             ZFAC = "string"))
            received <- rxReadXdf(xdfFileName, reportProgress = 0)
            checkEquals(received, expect)
            file.remove(xdfFileName)
        }
    }
}

test.rxImportToXdf.RxSasData.args.stringsAsFactors <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = TRUE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[11]] <-
      factor(expect[[11]], levels = c("Washington", "North Carolina", "Alaska"))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.factor(expect[[i]])
    expect[[12]][expect[[12]] == " "] <- NA
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    sasDS <- RxSasData(sasFileName, stringsAsFactors = TRUE)
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float64", Y2 ="float64", YLOGICAL = "uint32",
                     YBININT = "float64", XNUM1 = "float64", XNUM2 = "float64",
                     XNUM2MISS = "float64", XINT1 = "float64", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "uint32", ZFAC = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSasData.args.colClasses" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    sasDS <-
      RxSasData(sasFileName,
                colClasses =
                c(Y = "float32", Y2 = "float32", YLOGICAL = "logical",
                  YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                  XNUM2MISS = "float32", XINT1 = "integer", XFAC1 = "factor",
                  XFAC2 = "factor", ZFAC = "factor"))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSasData.args.colInfo" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-
      factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    sasDS <-
      RxSasData(sasFileName,
                colInfo =
                list(Y = list(type = "float32"), Y2 = list(type = "float32"),
                     YLOGICAL = list(type = "logical"),
                     YBININT = list(type = "uint16"),
                     XNUM1 = list(type = "float32"),
                     XNUM2 = list(type = "float32"),
                     XNUM2MISS = list(type = "float32"),
                     XINT1 = list(type = "integer"),
                     XFAC1 = list(type = "factor", levels = as.character(5:14)),
                     XFAC2 = list(type = "factor", levels = head(letters, 10)),
                     ZFAC =
                     list(type = "factor",
                          levels = c("Alaska", "North Carolina", "Washington"))))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "uint16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSasData.args.colClasses.colInfo" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-
      factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    sasDS <-
      RxSasData(sasFileName,
                colClasses =
                c(Y = "float32", Y2 = "float32", YLOGICAL = "logical",
                  YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                  XNUM2MISS = "float32", XINT1 = "integer", XFAC1 = "factor",
                  XFAC2 = "factor", ZFAC = "factor"),
                colInfo =
                list(XFAC1 = list(type = "factor", levels = as.character(5:14)),
                     XFAC2 = list(type = "factor", levels = head(letters, 10)),
                     ZFAC =
                     list(type = "factor",
                          levels = c("Alaska", "North Carolina", "Washington"))))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSasData.args.colClasses.numericToFactor" <- function()
{
  # RA-2503 regression test
  # Create a SAS data source. Specify variable YBININT to be imported as character: 
  testSAS <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat") 
  xdfFileName <- file.path(tempdir(), "richTest.xdf")

  # For factor import, use: 
  sourceDataSAS <- RxSasData(testSAS, colClasses = c(YBININT= "factor")) 
  xdfFileSAS <- "testSAS.xdf" 

  # XDF data source 
  xdfSourceSAS <- RxXdfData(xdfFileName) 
  rxImport(sourceDataSAS, xdfSourceSAS, overwrite=TRUE, reportProgress=0) 
  myDF <- rxReadXdf(xdfFileName)
  varInfo <- rxGetVarInfo( xdfFileName )
  # YBININT should contain factor levels 0 and 1 (no missings). 
  checkEquals(varInfo$YBININT$low, 1)  # 1-based levels
  checkEquals(varInfo$YBININT$high, 2)
  checkEquals(varInfo$YBININT$levels, c("0", "1"))
  
  if (file.exists( xdfFileName)) file.remove( xdfFileName)
}

"test.rxImportToXdf.RxSasData.args.colClasses.numericToCharacter" <- function()
{
  # Regression test for RA-2499
  # Create a SAS data source 
  claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat") 
  claimsSasSource <- 
    RxSasData(claimsSasFileName, 
        colClasses = c(RowNum = "character", age = "character", "car_age" = "character", 
                 type = "character", cost = "character", number = "character")) 

  # Create an xdf data source 
  claimsXdfFileName <- file.path(tempdir(), "importedClaims.xdf") 
  claimsXdfSource <- RxXdfData(claimsXdfFileName) 

  # Import the data into the xdf file 
  rxImport(claimsSasSource, claimsXdfSource, overwrite = TRUE, reportProgress = 0) 

  # Read xdf file into a data frame 
  claimsIn <- rxReadXdf(claimsXdfFileName, numRows = 3, reportProgress = 0) 
  checkEquals( claimsIn$cost, c("289", "282", "133"), checkNames = FALSE)
  checkEquals( claimsIn$number, c("8", "8", "4"), checkNames = FALSE)

  if (file.exists( claimsXdfFileName )) file.remove( claimsXdfFileName)
}

"test.rxImportToXdf.RxSasData.args.rowsPerRead" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    for (i in c(1L, 49L, 99L))
    {
        sasDS <- RxSasData(sasFileName, rowsPerRead = i)
        xdfFileName <- file.path(tempdir(), "richTest.xdf")
        xdfDS <- RxXdfData(xdfFileName)
        rxImport(sasDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- rxReadXdf(xdfFileName, reportProgress = 0)
        checkEquals(received, expect)
        xdfInfo <- rxGetInfo(xdfFileName, getBlockSizes = TRUE)
    if (rxIsExpressEdition())
    {
      checkEquals(xdfInfo[["numBlocks"]], 1L)
            checkEquals(xdfInfo[["rowsPerBlock"]], 100)
    } else
    {
            checkEquals(xdfInfo[["numBlocks"]], as.integer(ceiling(100/i)))
            checkEquals(xdfInfo[["rowsPerBlock"]],
                    c(rep.int(i, floor(100/i)), if (100 %% i != 0) 100 %% i))
    }
      
    }
    file.remove(xdfFileName)
}


### SPSS DATA SOURCE
"test.rxImportToXdf.RxSpssData" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    for (spssDS in list(RxSpssData(spssFileName), spssFileName))
    {
        for (xdfDS in list(RxXdfData(xdfFileName), xdfFileName))
        {
            rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
            checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                           c(Y = "float64", Y2 ="float64", YLOGICAL = "string",
                             YBININT = "float64", XNUM1 = "float64",
                             XNUM2 = "float64", XNUM2MISS = "float64",
                             XINT1 = "float64", XFAC1 = "string",
                             XFAC2 = "string", "Z_CHAR1" = "string",
                             ZFAC = "string"))
            received <- rxReadXdf(xdfFileName, reportProgress = 0)
            checkEquals(received, expect)
            file.remove(xdfFileName)
        }
    }
}

"test.rxImportToXdf.RxSpssData.args.stringsAsFactors" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = TRUE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[11]] <-
      factor(expect[[11]], levels = c("Washington", "North Carolina", "Alaska"))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.factor(expect[[i]])
    expect[[12]][expect[[12]] == " "] <- NA
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    spssDS <- RxSpssData(spssFileName, stringsAsFactors = TRUE)
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float64", Y2 ="float64", YLOGICAL = "uint32",
                     YBININT = "float64", XNUM1 = "float64", XNUM2 = "float64",
                     XNUM2MISS = "float64", XINT1 = "float64", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "uint32", ZFAC = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.colClasses" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    spssDS <-
      RxSpssData(spssFileName,
                 colClasses =
                 c(Y = "float32", Y2 = "float32", YLOGICAL = "logical",
                   YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                   XNUM2MISS = "float32", XINT1 = "integer", XFAC1 = "factor",
                   XFAC2 = "factor", ZFAC = "factor"))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.colInfo" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-
      factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    spssDS <-
      RxSpssData(spssFileName,
                 colInfo =
                 list(Y = list(type = "float32"), Y2 = list(type = "float32"),
                      YLOGICAL = list(type = "logical"),
                      YBININT = list(type = "uint16"),
                      XNUM1 = list(type = "float32"),
                      XNUM2 = list(type = "float32"),
                      XNUM2MISS = list(type = "float32"),
                      XINT1 = list(type = "integer"),
                      XFAC1 = list(type = "factor", levels = as.character(5:14)),
                      XFAC2 = list(type = "factor", levels = head(letters, 10)),
                      ZFAC =
                      list(type = "factor",
                           levels = c("Alaska", "North Carolina", "Washington"))))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "uint16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.colClasses.colInfo" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-
      factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    spssDS <-
      RxSpssData(spssFileName,
                 colClasses =
                 c(Y = "float32", Y2 = "float32", YLOGICAL = "logical",
                   YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                   XNUM2MISS = "float32", XINT1 = "integer", XFAC1 = "factor",
                   XFAC2 = "factor", ZFAC = "factor"),
                 colInfo =
                 list(XFAC1 = list(type = "factor", levels = as.character(5:14)),
                      XFAC2 = list(type = "factor", levels = head(letters, 10)),
                      ZFAC =
                      list(type = "factor",
                           levels = c("Alaska", "North Carolina", "Washington"))))
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(Y = "float32", Y2 ="float32", YLOGICAL = "uchar",
                     YBININT = "int16", XNUM1 = "float32", XNUM2 = "float32",
                     XNUM2MISS = "float32", XINT1 = "int32", XFAC1 = "uint32",
                     XFAC2 = "uint32", "Z_CHAR1" = "string", ZFAC = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.colClasses.factors" <- function()
{
  # Create SPSS data source
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
  spssDS <- RxSpssData(spssFileName)

  # Creates XDF data source
  xdfFileName <- file.path(tempdir(), "ClaimsIntImport3.xdf")
  xdfDS <- RxXdfData(xdfFileName)
  spssColClasses <- c(RowNum = "integer", number = "integer", 
    cost = "integer")
  spssDS <- RxSpssData(spssFileName, colClasses = spssColClasses)
  rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfo <- rxGetVarInfo( xdfFileName )
  checkEquals(varInfo$RowNum$varType, "integer")
  checkEquals(varInfo$cost$varType, "integer")
  checkEquals(varInfo$number$varType, "integer")
  checkEquals(varInfo$cost$low, 11)
  checkEquals(varInfo$cost$high, 850)
  file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.factors.default" <- function()
{
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
  spssDS <- RxSpssData(spssFileName)
    xdfFileName <- file.path(tempdir(), ".tempClaimsInt.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfo <- rxGetVarInfo( xdfFileName )

  
  cubeDF <- rxCube(~age:type:car_age, data=xdfFileName, returnDataFrame=TRUE)
  checkEquals(subset(cubeDF, age=="17-20" & type == "A" & car_age == "0-3")$Counts, 1)
    checkEquals(subset(cubeDF, age=="30-34" & type == "A" & car_age == "0-3")$Counts, 1)
    checkEquals(subset(cubeDF, age=="17-20" & type == "B" & car_age == "4-7")$Counts, 1)
    checkEquals(subset(cubeDF, age=="17-20" & type == "D" & car_age == "10+")$Counts, 0)
  
}

"test.rxImportToXdf.RxSpssData.args.labelsAsLevels" <- function()
{ 
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
  spssDS <- RxSpssData(spssFileName, labelsAsLevels=FALSE)
    xdfFileName <- file.path(tempdir(), ".tempClaimsInt.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfo <- rxGetVarInfo( xdfFileName )
  checkEquals(varInfo$RowNum$varType, "numeric")
  checkEquals(varInfo$number$varType, "numeric")
  # Since labelsAsInfo is by default true, that information should be in varInfo
  checkEquals(varInfo$type$valueInfoCodes, as.character(1:4))
  typeLabels <- LETTERS[1:4]
  checkEquals(varInfo$type$valueInfoLabels, typeLabels)
  
  file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.labelsAsInfo" <- function()
{  
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")    
  spssDS <- RxSpssData(spssFileName, labelsAsLevels=FALSE, labelsAsInfo=FALSE)
    xdfFileName <- file.path(tempdir(), ".tempClaimsInt.xdf")
  xdfDS <- RxXdfData(xdfFileName)
    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfo <- rxGetVarInfo( xdfFileName )
  
  checkEquals(varInfo$type$varType, "numeric")
  checkEquals(varInfo$car_age$varType, "numeric")
  checkEquals(varInfo$type$valueInfoCodes, NULL)
  checkEquals(varInfo$type$valueInfoLabels, NULL)
  file.remove(xdfFileName)
}

"test.rxImportToXdf.RxSpssData.args.numRows" <- function()
{
  xdfFileName <- file.path(tempdir(), ".tempClaimsInt.xdf")
    xdfDS <- RxXdfData( xdfFileName )
    
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")  
  spssDS <- RxSpssData( spssFileName )
    
  rxImport(spssDS, xdfDS, overwrite = TRUE, numRows = 5, reportProgress = 0)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals(fileInfo$numRows, 5)
  
  # Set rowsPerRead to less than numRows
  spssDS <- RxSpssData( spssFileName, rowsPerRead = 5 )
  rxImport(spssDS, xdfDS, overwrite = TRUE, numRows = 7, reportProgress = 0)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals(fileInfo$numRows, 7)
    if (rxIsExpressEdition())
    {
      checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 2)
    }
    
  spssDS <- RxSpssData( spssFileName, rowsPerRead = 10 )
  rxImport(spssDS, xdfDS, overwrite = TRUE,  reportProgress = 0)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals(fileInfo$numRows, 120)
  
  if (file.exists( xdfFileName)) file.remove( xdfFileName )
}

"test.rxImportToXdf.RxSpssData.args.newLevels" <- function()
{

  xdfFileName <- file.path(tempdir(), ".tempClaimsInt.xdf")
    xdfDS <- RxXdfData(xdfFileName)

    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")  
  
  existingTypeLevels = LETTERS[1:4]
  newTypeLevels = c("AA", "BB", "CC", "DD")
  spssDS <- RxSpssData(spssFileName, 
      colInfo = list(RowNum = list(type="integer"), 
    type = list(
      levels = existingTypeLevels,
      newLevels = newTypeLevels )))

    rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfo <- rxGetVarInfo( xdfFileName )
  checkEquals(varInfo$type$levels, newTypeLevels)
  checkEquals(varInfo$RowNum$varType, "integer")
  myDF <- rxReadXdf(xdfFileName, numRows = 5)
    
  # Original data obs 2
  # 2,"17-20","4-7","A",282.00,8.00

  checkEquals(myDF$RowNum[2], 2)
  checkEquals(as.character(myDF$age[2]), "17-20")
  checkEquals(as.character(myDF$type[2]), "AA") 
  checkEquals(myDF$cost[2], 282)
    checkEquals(myDF$number[2], 8)
  checkEquals(as.character(myDF$car_age[2]), "4-7") 
  
  if (file.exists(xdfFileName)) file.remove(xdfFileName)

}


"test.rxImportToXdf.RxSpssData.args.rowsPerRead" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    spssFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    for (i in c(1L, 49L, 99L))
    {
        spssDS <- RxSpssData(spssFileName, rowsPerRead = i)
        xdfFileName <- file.path(tempdir(), "richTest.xdf")
        xdfDS <- RxXdfData(xdfFileName)
        rxImport(spssDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- rxReadXdf(xdfFileName, reportProgress = 0)
        checkEquals(received, expect)
        xdfInfo <- rxGetInfo(xdfFileName, getBlockSizes = TRUE)
    if (rxIsExpressEdition())
    {
      checkEquals(xdfInfo[["numBlocks"]], 1L)
            checkEquals(xdfInfo[["rowsPerBlock"]], 100)
    } else
    {
            checkEquals(xdfInfo[["numBlocks"]], as.integer(ceiling(100/i)))
            checkEquals(xdfInfo[["rowsPerBlock"]],
                    c(rep.int(i, floor(100/i)), if (100 %% i != 0) 100 %% i))
    }
      
    }
    file.remove(xdfFileName)
}

### FIXED FORMAT DATA SOURCE
"test.rxImportToXdf.RxTextData.fixedFormat.schema" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <- RxTextData(ffFileName, missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float32", y2 ="float32", ylogical = "fixed",
                     ybinint = "float32", xnum1 = "float32", xnum2 = "float32",
                     xnum2miss = "float32", xint1 = "float32", xfac1 = "fixed",
                     xfac2 = "fixed", z.char1 = "fixed", zfac = "fixed"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)	
  
    checkEquals(received[-3], expect[-3], check.attributes = FALSE) # compare without 3rd column	
  # remove spaces from the 3rd column	
  exp <- lapply(expect[3], FUN=function(X){gsub(" ","",X)} )
  rec <- lapply(received[3], FUN=function(X){gsub(" ","",X)} )
  checkEquals(exp, rec, check.attributes = FALSE)	 # compare only 3rd column
  
  
    file.remove(xdfFileName)
}

### Fast FIXED FORMAT DATA SOURCE
"test.rxImportToXdf.RxTextData.FastFixedFormat.schema" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <- RxTextData(ffFileName, missingValueString = "", useFastRead = TRUE)
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float32", y2 ="float32", ylogical = "fixed",
                     ybinint = "float32", xnum1 = "float32", xnum2 = "float32",
                     xnum2miss = "float32", xint1 = "float32", xfac1 = "fixed",
                     xfac2 = "fixed", z.char1 = "fixed", zfac = "fixed"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)	
  
    checkEquals(received[-3], expect[-3], check.attributes = FALSE) # compare without 3rd column	
  # remove spaces from the 3rd column	
  exp <- lapply(expect[3], FUN=function(X){gsub(" ","",X)} )
  rec <- lapply(received[3], FUN=function(X){gsub(" ","",X)} )
  checkEquals(exp, rec, check.attributes = FALSE)	 # compare only 3rd column
  
  
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.fixedFormat.stringsAsFactors" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.factor(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[11]] <-
      factor(expect[[11]], levels = c("Washington", "North Carolina", "Alaska"))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <-
      RxTextData(ffFileName, stringsAsFactors = TRUE, missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float32", y2 ="float32", ylogical = "uint32",
                     ybinint = "float32", xnum1 = "float32", xnum2 = "float32",
                     xnum2miss = "float32", xint1 = "float32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "uint32", zfac = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
    
    checkEquals(received, expect)
  
    file.remove(xdfFileName)
}
##  Fast Fixed-format test
"test.rxImportToXdf.RxTextData.FastFixedFormat.stringsAsFactors" <- function()
{
    expect <- read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.factor(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[11]] <-  factor(expect[[11]], levels = c("Washington", "North Carolina", "Alaska"))
    expect[[12]] <-  factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    ffFileName <-  file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <- RxTextData(ffFileName, stringsAsFactors = TRUE, 
            useFastRead=TRUE, 
            missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float32", y2 ="float32", ylogical = "uint32",
                     ybinint = "float32", xnum1 = "float32", xnum2 = "float32",
                     xnum2miss = "float32", xint1 = "float32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "uint32", zfac = "uint32"))
    received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)    
    checkEquals(received, expect)
  
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.fixedFormat.args.colClasses" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    for (i in c(1, 2))
        expect[[i]] <- round(expect[[i]], 5)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <-
      RxTextData(ffFileName,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "int16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect, check.attributes = FALSE)
    file.remove(xdfFileName)
}

##  Fast Fixed-format test
"test.rxImportToXdf.RxTextData.FastFixedFormat.args.colClasses" <- function()
{
    expect <- read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    for (i in c(1, 2))
        expect[[i]] <- round(expect[[i]], 5)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[12]] <- factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))
    ffFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <- RxTextData(ffFileName,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
         useFastRead=TRUE, 
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  
  
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "int16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect, check.attributes = FALSE)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.fixedFormat.args.colInfo" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    for (i in c(1, 2))
        expect[[i]] <- round(expect[[i]], 5)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-
      factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.dat")
    ffDS <-
      RxTextData(ffFileName,
                 colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8),
                      y2 = list(type = "numeric", start = 9, width = 9),
                      ylogical = list(type = "logical", start = 18, width = 7),
                      ybinint = list(type = "uint16", start = 25, width = 2),
                      xnum1 = list(type = "numeric", start = 27, width = 12),
                      xnum2 = list(type = "numeric", start = 39, width = 13),
                      xnum2miss = list(type = "numeric", start = 52, width = 13),
                      xint1 = list(type = "integer", start = 65, width = 3),
                      xfac1 = list(type = "factor", levels = as.character(5:14), start = 68, width = 3),
                      xfac2 = list(type = "factor", levels = head(letters, 10), start = 71, width = 2),
                      z.char1 = list(type = "character", start = 73, width = 15),
                      zfac =
                      list(type = "factor",
                           levels = c("Alaska", "North Carolina", "Washington"),
                           start = 88, width = 15)),
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "uint16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect, check.attributes = FALSE)
    file.remove(xdfFileName)
}

## Fast Fixed-format test
"test.rxImportToXdf.RxTextData.FastFixedFormat.args.colInfo" <- function()
{
    expect <-  read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    for (i in c(1, 2))
        expect[[i]] <- round(expect[[i]], 5)
    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]])
    expect[[12]] <-    factor(expect[[12]], levels = c("Alaska", "North Carolina", "Washington"))
    ffFileName <-      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.dat")
    ffDS <-    RxTextData(ffFileName,
                 colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8),
                      y2 = list(type = "numeric", start = 9, width = 9),
                      ylogical = list(type = "logical", start = 18, width = 7),
                      ybinint = list(type = "uint16", start = 25, width = 2),
                      xnum1 = list(type = "numeric", start = 27, width = 12),
                      xnum2 = list(type = "numeric", start = 39, width = 13),
                      xnum2miss = list(type = "numeric", start = 52, width = 13),
                      xint1 = list(type = "integer", start = 65, width = 3),
                      xfac1 = list(type = "factor", levels = as.character(5:14), start = 68, width = 3),
                      xfac2 = list(type = "factor", levels = head(letters, 10), start = 71, width = 2),
                      z.char1 = list(type = "character", start = 73, width = 15),
                      zfac =     list(type = "factor",
                     levels = c("Alaska", "North Carolina", "Washington"),
                     start = 88, width = 15)),
            useFastRead=TRUE, 
            missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(y = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "uint16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))
    received <- rxReadXdf(xdfFileName, reportProgress = 0)
    checkEquals(received, expect, check.attributes = FALSE)
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.fixedFormat.args.colInfo.newName" <- function()
{
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.dat")
    ffDS <-
      RxTextData(ffFileName,
                 colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8, newName="NEWy"),
                      y2 = list(type = "numeric", start = 9, width = 9),
                      ylogical = list(type = "logical", start = 18, width = 7),
                      ybinint = list(type = "uint16", start = 25, width = 2),
                      xnum1 = list(type = "numeric", start = 27, width = 12),
                      xnum2 = list(type = "numeric", start = 39, width = 13),
                      xnum2miss = list(type = "numeric", start = 52, width = 13),
                      xint1 = list(type = "integer", start = 65, width = 3),
                      xfac1 = list(type = "factor", levels = as.character(5:14), start = 68, width = 3),
                      xfac2 = list(type = "factor", levels = head(letters, 10), start = 71, width = 2),
                      z.char1 = list(type = "character", start = 73, width = 15),
                      zfac =
                      list(type = "factor",
                           levels = c("Alaska", "North Carolina", "Washington"),
                           start = 88, width = 15)),
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(NEWy = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "uint16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))

    file.remove(xdfFileName)
}

## Fast Fixed-format test
"test.rxImportToXdf.RxTextData.FastFixedFormat.args.colInfo.newName" <- function()
{
    ffFileName <-      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.dat")
    ffDS <-   RxTextData(ffFileName,
                 colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8, newName="NEWy"),
                      y2 = list(type = "numeric", start = 9, width = 9),
                      ylogical = list(type = "logical", start = 18, width = 7),
                      ybinint = list(type = "uint16", start = 25, width = 2),
                      xnum1 = list(type = "numeric", start = 27, width = 12),
                      xnum2 = list(type = "numeric", start = 39, width = 13),
                      xnum2miss = list(type = "numeric", start = 52, width = 13),
                      xint1 = list(type = "integer", start = 65, width = 3),
                      xfac1 = list(type = "factor", levels = as.character(5:14), start = 68, width = 3),
                      xfac2 = list(type = "factor", levels = head(letters, 10), start = 71, width = 2),
                      z.char1 = list(type = "character", start = 73, width = 15),
                      zfac =
                      list(type = "factor",
                           levels = c("Alaska", "North Carolina", "Washington"),
                           start = 88, width = 15)),
         useFastRead=TRUE, 
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
    rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
    checkIdentical(sapply(rxGetVarInfo(xdfFileName), "[[", "storage"),
                   c(NEWy = "float64", y2 ="float64", ylogical = "uchar",
                     ybinint = "uint16", xnum1 = "float64", xnum2 = "float64",
                     xnum2miss = "float64", xint1 = "int32", xfac1 = "uint32",
                     xfac2 = "uint32", z.char1 = "fixed", zfac = "uint32"))

    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.delimited.args.colInfo.newLevels" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    outFile <- file.path(tempdir(), ".test100r.xdf")
    if (file.exists(outFile)) file.remove(outFile)
      
  origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
  newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
  testColInfo <- list(xfac2=list(type="factor",
    levels=origLevels,
        newLevels=newLevels))
  
    delDS <- RxTextData(inFile, colInfo=testColInfo)
    xdfDS <- RxXdfData(outFile)
    rxImport(delDS, xdfDS, overwrite = TRUE, reportProgress = 0)

  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$xfac2$levels, newLevels)
  rm(varInfoObj)
  
  # Combine with newName and multiple blocks
  testColInfo <- list(xfac2=list(newName="MyFactor", type="factor",
    levels=origLevels,
        newLevels=newLevels), xfac1=list(newName="MyOtherFactor"))
  
  delDS <- RxTextData(inFile, colInfo=testColInfo, rowsPerRead=50)
    xdfDS <- RxXdfData(outFile)
    rxImport(delDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$MyFactor$levels, newLevels)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[8], "MyOtherFactor")
  checkEquals(varNames[9], "MyFactor")
  
  if (file.exists(outFile)) file.remove(outFile)  
}

"test.rxImportToXdf.RxTextData.delimited.args.colInfo.ordered" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    outFile <- file.path(tempdir(), ".test100r.xdf")
    if (file.exists(outFile)) file.remove(outFile)
      
  origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
  newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
  testColInfo <- list(xfac2=list(type="ordered",
    levels=origLevels,
        newLevels=newLevels))
  
    delDS <- RxTextData(inFile, colInfo=testColInfo)
    xdfDS <- RxXdfData(outFile)
    rxImport(delDS, xdfDS, overwrite = TRUE, reportProgress = 0)

  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$xfac2$levels, newLevels)
  rm(varInfoObj)
  
  # Combine with newName and multiple blocks
  testColInfo <- list(xfac2=list(newName="MyFactor", type="ordered",
    levels=origLevels,
        newLevels=newLevels), xfac1=list(newName="MyOtherFactor"))
  
  delDS <- RxTextData(inFile, colInfo=testColInfo, rowsPerRead=50)
    xdfDS <- RxXdfData(outFile)
    rxImport(delDS, xdfDS, overwrite = TRUE, reportProgress = 0)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$MyFactor$levels, newLevels)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[8], "MyOtherFactor")
  checkEquals(varNames[9], "MyFactor")
    
  topAlphabet <- rxDataStep(inData = outFile, rowSelection = MyFactor < "dd")
  checkEquals(sum(topAlphabet$MyFactor == "dd"), 0)
  checkEquals(sum(topAlphabet$MyFactor == "jj"), 0)
  if (file.exists(outFile)) file.remove(outFile) 
}

test.rxImportToXdf.RxTextData.fixedFormat.args.rowsPerRead <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    for (i in c(1L, 49L, 99L))
    {
        ffDS <- RxTextData(ffFileName, rowsPerRead = i, missingValueString = "")
        xdfFileName <- file.path(tempdir(), "richTest.xdf")
        xdfDS <- RxXdfData(xdfFileName)
        rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
        # Data frame character columns will have .rxColWidth attribute
        checkEquals(received[-3], expect[-3], check.attributes=FALSE) # compare without 3rd column	
    # remove spaces from the 3rd column	
    exp <- lapply(expect[3], FUN=function(X){gsub(" ","",X)} )
    rec <- lapply(received[3], FUN=function(X){gsub(" ","",X)} )
    checkEquals(exp, rec, check.attributes=FALSE)	 # compare only 3rd column
    
        xdfInfo <- rxGetInfo(xdfFileName, getBlockSizes = TRUE)
        if (rxIsExpressEdition())
        {
            checkEquals(xdfInfo[["numBlocks"]], 1)
            checkEquals(xdfInfo[["rowsPerBlock"]], 100)
        }
        else
        {
            checkEquals(xdfInfo[["numBlocks"]], as.integer(ceiling(100/i)))
            checkEquals(xdfInfo[["rowsPerBlock"]],
                        c(rep.int(i, floor(100/i)), if (100 %% i != 0) 100 %% i))
         }
            
    }
    file.remove(xdfFileName)
}
###  Fast Fixed-Format	- FFFMissing.log
test.rxImportToXdf.RxTextData.FastFixedFormat.args.rowsPerRead <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    whichNumeric <- which(sapply(expect, is.numeric))
    for (i in whichNumeric)
        expect[[i]] <- signif(expect[[i]], 4)
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
  #i <- 49L
    for (i in c(1L, 49L, 99L))
    {
        ffDS <- RxTextData(ffFileName, rowsPerRead = i, 
              useFastRead = TRUE,
              missingValueString = "")
        xdfFileName <- file.path(tempdir(), "richTest.xdf")
        xdfDS <- RxXdfData(xdfFileName)
        rxImport(ffDS, xdfDS, overwrite = TRUE, reportProgress = 0)
        received <- .rxSafeReadXdf(xdfFileName, reportProgress = 0)
        # Data frame character columns will have .rxColWidth attribute
        checkEquals(received[-3], expect[-3], check.attributes=FALSE) # compare without 3rd column	tolerance=1E-6,
    
    # remove spaces from the 3rd column	
    exp <- lapply(expect[3], FUN=function(X){gsub(" ","",X)} )
    rec <- lapply(received[3], FUN=function(X){gsub(" ","",X)} )
    checkEquals(exp, rec, check.attributes=FALSE)	 # compare only 3rd column
    
        xdfInfo <- rxGetInfo(xdfFileName, getBlockSizes = TRUE)
        if (rxIsExpressEdition())
        {
            checkEquals(xdfInfo[["numBlocks"]], 1)
            checkEquals(xdfInfo[["rowsPerBlock"]], 100)
        }
        else{
            checkEquals(xdfInfo[["numBlocks"]], as.integer(ceiling(100/i)))
            checkEquals(xdfInfo[["rowsPerBlock"]],
                    c(rep.int(i, floor(100/i)), if (100 %% i != 0) 100 %% i))            
        }
    }
    file.remove(xdfFileName)
}

"test.rxImportToXdf.RxTextData.fixedFormat.args.numRows" <- function()
{
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <-
      RxTextData(ffFileName,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
  
  rxImport( inData = ffDS, outFile = xdfDS, numRows = 5)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 5 )
  
  # Try with multiple blocks
   ffDS <-
      RxTextData(ffFileName, rowsPerRead = 5,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
                 missingValueString = "")
      
  rxImport( inData = ffDS, outFile = xdfDS, numRows = 7, overwrite=TRUE)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 7)
    if (rxIsExpressEdition())
    {
        checkEquals( fileInfo$numBlocks, 1)
    }    else
    {
      checkEquals( fileInfo$numBlocks, 2)
    }
    
  
  if (file.exists( xdfFileName)) file.remove( xdfFileName )
  
}
### Fast Fixed-Format
"test.rxImportToXdf.RxTextData.FastFixedFormat.args.numRows" <- function()
{
    ffFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.sts")
    ffDS <-
      RxTextData(ffFileName,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
         useFastRead = TRUE,
                 missingValueString = "")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
  
  rxImport( inData = ffDS, outFile = xdfDS, numRows = 5,overwrite=TRUE )
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 5 )
  
  # Try with multiple blocks
   ffDS <-
      RxTextData(ffFileName, rowsPerRead = 5,
                 colClasses =
                 c(y = "numeric", y2 = "numeric", ylogical = "logical",
                   ybinint = "int16", xnum1 = "numeric", xnum2 = "numeric",
                   xnum2miss = "numeric", xint1 = "integer", xfac1 = "factor",
                   xfac2 = "factor", zfac = "factor"),
         useFastRead = TRUE,
                 missingValueString = "")
      
  rxImport( inData = ffDS, outFile = xdfDS, numRows = 7, overwrite=TRUE)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 7)
    if (rxIsExpressEdition())
    {
        checkEquals( fileInfo$numBlocks, 1)
    }
    else{
      checkEquals( fileInfo$numBlocks, 2)
    }
    
  if (file.exists( xdfFileName)) file.remove( xdfFileName )
  
}


### DELIMITED TEXT DATA SOURCE

"test.rxImportToXdf.RxTextData.delimited.args.numRows" <- function()
{
    dFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")
    dDS <-
      RxTextData(dFileName)
    xdfFileName <- file.path(tempdir(), "richTest.xdf")
    xdfDS <- RxXdfData(xdfFileName)
  
  rxImport( inData = dDS, outFile = xdfDS, numRows = 5, overwrite=TRUE)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 5 )
  
  # Try with multiple blocks
   dDS <-
      RxTextData(dFileName, rowsPerRead = 5 )	
  rxImport( inData = dDS, outFile = xdfDS, numRows = 12, overwrite=TRUE)
  fileInfo <- rxGetInfo( xdfFileName )
  checkEquals( fileInfo$numRows, 12)
    if (rxIsExpressEdition())
    {
        checkEquals( fileInfo$numBlocks, 1)
    } else
    {
      checkEquals( fileInfo$numBlocks, 3)
    }
      
  if (file.exists( xdfFileName)) file.remove( xdfFileName )
  
}

"test.rxImportToXdf.RxTextData.delimited.data.logicals" <- function()
{
    xdfFile <- file.path(tempdir(), ".rxTemp.xdf")
    txtFile <- file.path(tempdir(), ".rxTemp.csv")

    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)

    myData1 <-
    data.frame(rowNum = 1:4, 
               logicalMiss = rep(NA, times = 4),
               logicalStr = c("FALSE", "TRUE", "FALSE", "TRUE"),
               logicalInt = c(0, 1, 0, 1),
               logicalStrShort = c("F", "T", "F", "T"),
               logicalStrM = c("FALSE", NA, "FALSE", "TRUE"),
               logicalIntM = c(0, NA, 0, 1),
               logicalStrShortM = c("F", NA, "F", "T"))

    write.csv(myData1, file=txtFile, row.names=FALSE)

    myColClasses <-
      c(logicalMis = "logical", logicalStr = "logical", logicalInt = "logical",
        logicalStrShort = "logical", logicalStrM = "logical",
        logicalIntM = "logical", logicalStrShortM = "logical")

    textDS <- RxTextData(txtFile, colClasses = myColClasses)
    xdfDS <- RxXdfData(xdfFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,
                  reportProgress = 0)

    varSum <- rxSummary(~ logicalMiss, data = xdfFile, reportProgress = 0, byTerm=FALSE)
    checkEquals(varSum$nobs.valid, 0)
    varSum <- rxSummary(~ logicalStr, data = xdfFile, reportProgress = 0, byTerm=FALSE)
    checkEquals(varSum$nobs.valid, 4)
    myData2 <- rxReadXdf(file = xdfFile, reportProgress = 0)
    checkEquals(myData2$logicalStr, myData2$logicalInt)
    checkEquals(myData2$logicalStr, myData2$logicalStrShort)
    checkEquals(myData2$logicalStrM, myData2$logicalIntM)
    checkEquals(myData2$logicalStrM, myData2$logicalStrShortM)

    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)
}


"test.rxImportToXdf.RxTextData.delimited.args.colInfo.newName" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
  textDS <- RxTextData(inFile)
  xdfDS <- RxXdfData(outFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,
                  reportProgress = 0)
  varInfoObj <- rxGetVarInfo(outFile)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("a", "b", "c"))
      
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), 
    c=list(newName="Cleveland"))
  textDS <- RxTextData(inFile, colInfo=testColInfo)
  xdfDS <- RxXdfData(outFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,
                  reportProgress = 0)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  rm(varNames)
  
  # Test with additional colInfo
  testColInfo <- list(a=list(newName="Atlanta", type="numeric"), b=list(newName="Boston"), 
    c=list(newName="Cleveland", type="factor", levels=c("One", ".rxOther")))
  textDS <- RxTextData(inFile, colInfo=testColInfo)
  xdfDS <- RxXdfData(outFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,
                  reportProgress = 0)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Atlanta$varType, "numeric")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  rm(varInfoObj)
  
  # Test with colClasses
  testColClasses <- c(a="numeric")
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), 
  c=list(newName="Cleveland", type="factor", levels=c("One", "Two")))
  textDS <- RxTextData(inFile, colInfo=testColInfo, colClasses=testColClasses)
  xdfDS <- RxXdfData(outFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,  reportProgress = 0)

  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Atlanta$varType, "numeric")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  
  # Test with colClasses - different variable
  testColClasses <- c(b="integer")
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), 
     c=list(newName="Cleveland", type="factor", levels=c("One", "Two")))

  textDS <- RxTextData(inFile, colInfo=testColInfo, colClasses=testColClasses)
  xdfDS <- RxXdfData(outFile)
    rxImport(inData = textDS, outFile = xdfDS, overwrite = TRUE,  reportProgress = 0)
  
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Boston$varType, "integer")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxImportToXdf.args.transformFunc.rxStartRow" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  
  # See related RA-2878
    # Test for comma delimited
  richDatatxt <- file.path(rxGetOption("unitTestDataDir"),"richTest200rComma.txt") 
  richData <- file.path(tempdir(),"richTest200r5b.xdf") 

  transformFunc <- function(datalist)
  { 
        numRowsInChunk <- length(datalist$zfac) 
    datalist$Index <- .rxStartRow:(.rxStartRow + numRowsInChunk - 1) 
    datalist$Chunk <- rep(.rxChunkNum, numRowsInChunk)
    datalist$File <-  rep(.rxReadFileName, numRowsInChunk)
        datalist 
  } 

  textDS <- RxTextData(richDatatxt, colClasses=c(z.char1="character"), rowsPerRead=40 )
  xdfDS <- RxXdfData(richData)
    rxImport(inData = textDS, outFile = xdfDS, 
    transformFunc=transformFunc, 
    transformVars=c("zfac"),
    overwrite = TRUE,  reportProgress = 0)

  myData <- rxReadXdf( richData)
  checkEquals(myData$Index, 1:200)
    if (rxIsExpressEdition())
    {
        checkEquals(myData$Chunk, rep(1, each=200))
    } else
    {
      checkEquals(myData$Chunk, rep(1:5, each=40))
    }
    
  checkEquals(myData$File, rep(normalizePath(richDatatxt), 200))
  file.remove( richData)
  
  # Repeat using SPSS file

  richDataSpss <- file.path(rxGetOption("unitTestDataDir"),"richTest100r.sav") 
  richData <- file.path(tempdir(),"richTest100r5b.xdf") 

  transformFunc <- function(datalist)
  { 
        numRowsInChunk <- length(datalist$ZFAC) 
    datalist$Index <- .rxStartRow:(.rxStartRow + numRowsInChunk - 1) 
    datalist$Chunk <- rep(.rxChunkNum, numRowsInChunk)
    datalist$File <-  rep(.rxReadFileName, numRowsInChunk)
        datalist 
  } 

  spssDS <- RxSpssData(richDataSpss, rowsPerRead=20 )
  xdfDS <- RxXdfData(richData)
  #rxImport( inData=spssDS, outFile = xdfDS, overwrite=TRUE)
  
    rxImport(inData = spssDS, outFile = xdfDS, 
    transformFunc=transformFunc, 
    transformVars=c("ZFAC"),
    overwrite = TRUE,  reportProgress = 0)

  myData <- rxReadXdf( richData)
  checkEquals(myData$Index, 1:100)
    if (rxIsExpressEdition())
    {
      checkEquals(myData$Chunk, rep(1, each=100))
    } else
    {
        checkEquals(myData$Chunk, rep(1:5, each=20))
    }

    
  checkEquals(normalizePath(myData$File), rep(normalizePath(richDataSpss), 100))
  file.remove(richData)
}

## Fixed-format test
"test.rxImportToXdf.data.DAT" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkindat <- file.path(unitTestDataDir, "Pumpkin.dat")
  pumpkinDATupper <- file.path(tempdir(), "Pumpkin.DAT")
    if (file.exists(pumpkinDATupper)) file.remove(pumpkinDATupper)

  cat("test.rxImport.data.DAT Copying", pumpkindat, pumpkinDATupper)
    
    file.copy(from = pumpkindat, to=pumpkinDATupper)
  pumpkinXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  
  pumpkinColInfo = list(
      Col1 = list(newName = "Name", start=1, width=17),
      Col2 = list(newName = "Age",  type="integer", start=18, width=2),
      Col3 = list(newName = "Type", start=21, width=1),
      Col4 = list(newName = "Date", type="Date", start=23, width=10),
      Col5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      Col6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      Col7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      Col8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      Col9 = list(newName = "Scr5", type="numeric", start=49, width=4)
      )
  
  pumpkinDS1 <- RxTextData(file=pumpkindat, firstRowIsColNames = FALSE,
    missingValueString=".", 
    readDateFormat = "%m-%d-%Y", colInfo = pumpkinColInfo )
  
  pumpkinDS2 <- RxTextData(file=pumpkinDATupper, firstRowIsColNames = FALSE,
    missingValueString="." , readDateFormat = "%m-%d-%Y", colInfo = pumpkinColInfo)
    
    print("test.rxImport.data.DAT Before first import")
  rxImport(inData=pumpkinDS2, outFile=pumpkinXDF, overwrite=TRUE )
  pumpkinData2 <- rxReadXdf( pumpkinXDF )
  
    print("test.rxImport.data.DAT Before second import")
  rxImport(inData=pumpkinDS1, outFile=pumpkinXDF, overwrite=TRUE )
  pumpkinData1 <- rxReadXdf( pumpkinXDF)
  
  checkEquals(pumpkinData1, pumpkinData2)
  
  file.remove(pumpkinDATupper)
  file.remove(pumpkinXDF)
  
}

"test.rxImportToXdf.data.DAT.FastFixed" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkindat <- file.path(unitTestDataDir, "Pumpkin.dat")
  
    # Make sure we get an appropriate warning if useFastRead is TRUE
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)	
  
  pumpkinColInfo = list(
      Col1 = list(newName = "Name", start=1, width=17),
      Col2 = list(newName = "Age",  type="integer", start=18, width=2),
      Col3 = list(newName = "Type", start=21, width=1),
      Col4 = list(newName = "Date", type="Date", start=23, width=10),
      Col5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      Col6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      Col7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      Col8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      Col9 = list(newName = "Scr5", type="numeric", start=49, width=4)
      )
  
  res <- try(
    pumpkinDS1 <- RxTextData(file=pumpkindat, firstRowIsColNames = FALSE,
      useFastRead = TRUE,
      missingValueString=".", 
      readDateFormat = "%m-%d-%Y", colInfo = pumpkinColInfo )
  )
  checkEquals(inherits(res,"try-error"), FALSE )
  #checkTrue(grepl("not supported by useFastRead", as.character(res)))		
  
    
}

## Fast Fixed-format test
"test.rxImportToXdf.data.DAT.CheckException" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkindat <- file.path(unitTestDataDir, "Pumpkin.dat")
  #pumpkinDATupper <- file.path(tempdir(), "Pumpkin.DAT")
    #if (file.exists(pumpkinDATupper)) file.remove(pumpkinDATupper)
#
  ##cat("test.rxImport.data.DAT Copying", pumpkindat, pumpkinDATupper)
    #
    #file.copy(from = pumpkindat, to=pumpkinDATupper)
  pumpkinXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  
  pumpkinColInfo = list(
      Col1 = list(newName = "Name", start=1, width=17),
      Col2 = list(newName = "Age",  type="integer", start=18, width=2),
      Col3 = list(newName = "Type", start=21, width=1),
      Col4 = list(newName = "Date", type="Date", start=23, width=10),
      Col5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      Col6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      Col7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      Col8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      Col9 = list(newName = "Scr5", type="numeric", start=49, width=4)
      )
  
  oldWarn <- options(warn = 2L)
  on.exit(options(oldWarn), add = TRUE)
  
  #checkException(
    pumpkinDS1 <- RxTextData(file=pumpkindat, firstRowIsColNames = FALSE,
    useFastRead = TRUE,
    missingValueString=".", 
    readDateFormat = "%m-%d-%Y", colInfo = pumpkinColInfo )
  #,"Error in RxTextData(file = pumpkindat, firstRowIsColNames = FALSE, useFastRead = TRUE,  : (converted from warning) 'readDateFormat' not supported by useFastRead text import.  'useFastRead' set to FALSE.")
      
    print("test.rxImport.data.DAT Before second import")
  rxImport(inData=pumpkinDS1, outFile=pumpkinXDF, overwrite=TRUE )

  if (file.exists(pumpkinXDF)) file.remove(pumpkinXDF)
  #file.remove(pumpkinXDF)	
  
}

#test.rxImportToXdf.RxSpssData.args.varsToKeep <- function()
#{
  # Work in progress
    #expect <-
      #read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               #stringsAsFactors = FALSE)
    #colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    #whichLogical <- which(sapply(expect, is.logical))
    #for (i in whichLogical)
        #expect[[i]] <- as.character(expect[[i]])
    #expect[[9]] <- as.character(expect[[9]])
    #expect[[12]][expect[[12]] == " "] <- NA
  #
  ## Data file has variables: y, y2, ylogical, ybinint, xnum1,
  ## xnum2, xnum2miss, xint1, xfac1, xfac2, z.char1, zfac (in CAPS)
  #
    #spssFileName <-
      #file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
    #xdfFileName <- file.path(tempdir(), "richTest.xdf")
  #
  #spssDS <- RxSpssData( spssFileName, varsToKeep = c("Y", "YLOGICAL", "XFAC1"))
  #
  #rxImport(spssDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
  #
  #varInfo <- rxGetVarInfo( xdfFileName )
#}


"test.rxImportToXdf.RxTextData.args.xdfCompressionLevel.noAppend" <- function()
{
  claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  txtSource <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block size	
    
  claimsXdf <- file.path(tempdir(), "ClaimsNonCompress.xdf")		
  xdfSource <- RxXdfData(claimsXdf) 			
  rxImport(txtSource, xdfSource,	overwrite=TRUE) 	
  data <- rxReadXdf( claimsXdf)
  
  claimsCompressedXdf <- file.path(tempdir(), "ClaimsCompressed.xdf")
  xdfSourceCompressed <- RxXdfData(claimsCompressedXdf) 			
  rxImport(txtSource, xdfSourceCompressed,	overwrite=TRUE, xdfCompressionLevel=-1)		
  dataCompressed <- rxReadXdf( claimsCompressedXdf)
  
  checkEquals(dataCompressed, data)
  if(file.exists(claimsXdf)) file.remove(claimsXdf)
  if(file.exists(claimsCompressedXdf)) file.remove(claimsCompressedXdf)
}

"test.rxImportToXdf.RxTextData.args.xdfCompressionLevel.append" <- function()
{
  claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  txtSource <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block size	
      
  claimsCompressedXdf <- file.path(tempdir(), "ClaimsCompressedAppendCompressed.xdf")
  xdfSourceCompressed <- RxXdfData(claimsCompressedXdf) 			
  rxImport(txtSource, xdfSourceCompressed,	overwrite=TRUE, xdfCompressionLevel=-1)	
  
  
  txtSource2 <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block 	
  rxImport(txtSource2, xdfSourceCompressed, append = "rows",	overwrite=TRUE, xdfCompressionLevel=-1)	
  
  checkEquals(nrow(xdfSourceCompressed), 256)	
  if(file.exists(claimsCompressedXdf)) file.remove(claimsCompressedXdf)
}

"test.rxImportToXdf.RxTextData.args.xdfCompressionLevel.appendNonCompress" <- function()
{
  claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  txtSource <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block size	
      
  claimsCompressedXdf <- file.path(tempdir(), "ClaimsCompressedAppendNonCompressed.xdf")
  xdfSourceCompressed <- RxXdfData(claimsCompressedXdf) 			
  rxImport(txtSource, xdfSourceCompressed,	overwrite=TRUE, xdfCompressionLevel=-1)	
  
  
  txtSource2 <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block 	
  rxImport(txtSource2, xdfSourceCompressed, append = "rows",	overwrite=TRUE, xdfCompressionLevel=0)		
  dataCompressed <- rxReadXdf( claimsCompressedXdf)
  
  checkEquals(nrow(dataCompressed), 256)
  if(file.exists(claimsCompressedXdf)) file.remove(claimsCompressedXdf)
}


"test.rxImportToXdf.RxTextData.args.xdfCompressionLevel.nonCompressAppendCompressCheckFailure" <- function()
{
  claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  txtSource <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block size	
      
  claimsCompressedXdf <- file.path(tempdir(), "ClaimsNonCompressedAppendCompressed.xdf")
  xdfSourceCompressed <- RxXdfData(claimsCompressedXdf) 			
  rxImport(txtSource, xdfSourceCompressed,	overwrite=TRUE, xdfCompressionLevel=0)		
  
  txtSource2 <- RxTextData(claimsFF, rowsPerRead = 65) # rowsPerRead will determine block 	
  
  # We expect a warning here ignoring compression level
    warningExpr <- "Cannot write compressed data to an uncompressed file." 	
  out1 <- capture.output(
        rxImport(txtSource2, xdfSourceCompressed, append = "rows",	overwrite=TRUE, xdfCompressionLevel=-1)	
          )
    checkTrue(grepl(warningExpr, out1[2]))
  if(file.exists(claimsCompressedXdf)) file.remove(claimsCompressedXdf)
}
