# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLinePlot
#
############################################################################################

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

test.rxLinePlot.args.lineStyle <- function()
{
  censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 
    # Compute the weighted counts for age and sex 
    ageSex <- rxCube(~ F(age) : sex, data = censusWorkers, pweights = "perwt", 
                     returnDataFrame = TRUE) 
    # Convert the age result back to an integer 
    ageSex$age <- as.integer(as.character(ageSex$F_age)) 
    # Draw a line plot of the results 

    rxLinePlot(Counts ~ age, groups = sex, data = ageSex, lineStyle = 1:2,
               main = "Different line styles") 
        
    rxLinePlot(Counts ~ age, groups = sex, data = ageSex, type="b", lineStyle = c("solid", "dashed"),
               main = "Both symbols and lines; different line styles") 
    set.seed(100)
    myData <- data.frame(x=1:10, y=rnorm(10))   
    rxLinePlot(y~x, data=myData, lineStyle="solid", title="Solid line")
    rxLinePlot(y~x, data=myData, lineStyle="dashed", title="Dashed line")
    rxLinePlot(y~x, data=myData, lineStyle="dotted", title="Dotted line")
    rxLinePlot(y~x, data=myData, lineStyle="dotdash", title="Dotdash line")
    rxLinePlot(y~x, data=myData, lineStyle="longdash", title="Longdash line")
    rxLinePlot(y~x, data=myData, lineStyle="twodash", title="Twodash line")
    rxLinePlot(y~x, data=myData, lineStyle="blank", title="Blank line")
    
}
test.rxLinePlot.args.type <- function()
{
    set.seed(100)
    myData <- data.frame(x=1:20, y=runif(20)) 
    rxLinePlot(y~x, data=myData, title="Default plot type")
    rxLinePlot(y~x, data=myData, type="p", title="Points")
    rxLinePlot(y~x, data=myData, type="l", title="Line")
    rxLinePlot(y~x, data=myData, type="b", lineColor="green", title="Both with green line")
    rxLinePlot(y~x, data=myData, type="smooth", title="Smooth")
    rxLinePlot(y~x, data=myData, type=c("smooth", "p"), title="Smooth and Points")
    rxLinePlot(y~x, data=myData, type="s", title="Step")
    rxLinePlot(y~x, data=myData, type="r", title="Regression")
    rxLinePlot(y~x, data=myData, type=c("p","r"), 
        symbolColor="red", symbolSize=1.5, title="Regression and Red Points")
    
}

test.rxLinePlot.args.symbolSize <- function()
{
    
    rxLinePlot(Ozone ~ Temp, data=airquality, type="p", title="Default symbol size") 
    rxLinePlot(Ozone ~ Temp, data=airquality, type="p", symbolSize=.5, title="Symbol size = .5") 
    rxLinePlot(Ozone ~ Temp, data=airquality, type="p", symbolSize=2, 
        symbolColor="springgreen4", title="Symbol size = 2, symbolColor = springgreen4") 
}

test.rxLinePlot.args.symbolColor <- function()
{
    
    set.seed(100)
    myData <- data.frame(x=1:20, y=runif(20)) 
    rxLinePlot(y~x, data=myData, type="b", title="Default symbol color")
    rxLinePlot(y~x, data=myData, type="b", symbolColor="violetred", title=" violetred symbols")
    rxLinePlot(y~x, data=myData, type="b", symbolSize=1.5, symbolColor="violetred", lineColor="turquoise2",
        title=" violetred symbols and turqoise2 lines")
}

test.rxLinePlot.args.symbolStyle <- function()
{
    set.seed(100)
    myData <- data.frame(x=1:20, y=runif(20), z=runif(20)) 
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, title="Default symbol style")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="circle", title="circle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="solid circle", title="Solid circle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="square", title="square")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="solid square", title="Solid square")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="diamond", title="diamond")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="solid diamond", title="Solid diamond")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="triangle", title="triangle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="solid triangle", title="Solid triangle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="plus", title="plus")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="cross", title="cross")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="down triangle", title="down triangle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="square x", title="square x")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="star", title="star")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="diamond +", title="diamond +")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="circle +", title="circle +")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="up down triangle", title="up down triangle")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="square +", title="square +")
    rxLinePlot(y~x, data=myData, type="p", symbolSize=2, symbolStyle="circle x", title="circle x")
    rxLinePlot(y+z~x, data=myData, type="p", symbolSize=2, symbolStyle=c("solid circle", "triangle"), title="Solid circle and triangle")
    rxLinePlot(y+z~x, data=myData, type="p", symbolStyle=c("M", "F"), 
        title="Letters M and F")
   rxLinePlot(y+z~x, data=myData, type="b", symbolSize=1.1, symbolColor=c("violetred", "blue"), 
        lineColor=c("turquoise2","green"), symbolStyle=c("solid circle", "solid square"),
        lineStyle=c("solid", "dashed"),
        title="Different colors and styles")
}

"test.rxLinePlot.args.rowSelection" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  myDF <- rxReadXdf(file=test100r5b)
  myDF1 <- subset(myDF, xint1 > 7)
  latticeLoaded <- "lattice" %in% .packages()

    plotObj1 <- rxLinePlot(xnum1~xnum2, data=test100r5b, type="p", rowSelection = xint1 > 7, 
    title="test100r5b xdf file rowSelection = xint1 > 7 ")
  if (latticeLoaded)	
  {
    checkEquals(plotObj1$packet.sizes, nrow(myDF1))
  }
    
  plotObj1a <- rxLinePlot(xnum1~xnum2, data=myDF, type="p", rowSelection = xint1 > 7, 
    title="test100r5b data frame rowSelection = xint1 > 7 ")
  if (latticeLoaded)	
  {
    checkEquals(plotObj1a$packet.sizes, nrow(myDF1))
  }

  myDF2 <- subset(myDF, xnum1 > 0)
    plotObj2 <- rxLinePlot(xnum1~xfac1, data=test100r5b, type="p", rowSelection = xnum1 > 0,
    title="test100r5b rowSelection = xnum1 > 0")
  if (latticeLoaded)	
  {
    checkEquals(plotObj2$packet.sizes, nrow(myDF2))
  }
  
    # Try groups
  plotObj3 <- rxLinePlot(xnum1~xnum2, data=test100r5b, type="p", groups=ylogical,
    rowSelection = xint1 > 7, 
    title="test100r5b xdf file rowSelection = xint1 > 7 with groups")
  if (latticeLoaded)
  {
    checkEquals(plotObj3$packet.sizes, nrow(myDF1))
  }

  plotObj3a <- rxLinePlot(xnum1~xnum2, data=myDF, type="p", groups=xfac2,
    rowSelection = xint1 > 7, 
    title="test100r5b data frame rowSelection = xint1 > 7 with groups ")
  if (latticeLoaded)
  {
    checkEquals(plotObj3a$packet.sizes, nrow(myDF1))
  }
  # Row selection using data frame from rxCube
  # Regression test for RA-2494
  admissDF <- as.data.frame(UCBAdmissions)
  cube1 <- rxCube(Freq~Dept, data=admissDF,returnDataFrame=TRUE)
  plotObj4 <- rxLinePlot(Freq~Dept, data=cube1, rowSelection = Dept != "A")
  if (latticeLoaded)
  {
    checkEquals(plotObj4$packet.sizes, nrow(cube1)-1)
  }
}

"test.rxLinePlot.args.transforms" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  myDF <- rxReadXdf(file=test100r5b)
  myDF1 <- subset(myDF, xint1 > 7)
  latticeLoaded <- "lattice" %in% .packages()
  
  # With rowSelection
    plotObj1 <- rxLinePlot(xnum3~xnum2, data=test100r5b, type=c("p","r"),
    transforms=list(xnum3=xnum1+xnum2, xint2 = xint1+1), rowSelection = xint2 > 8, 
    title="test100r5b xdf rowSelection and transforms")
  if (latticeLoaded)
  {
    checkEquals(plotObj1$packet.sizes, nrow(myDF1))
  }

  # With data frame
  plotObj1a <- rxLinePlot(xnum3~xnum2, data=myDF, type=c("p","r"),
    transforms=list(xnum3=xnum1+xnum2, xint2 = xint1+1), rowSelection = xint2 > 8, 
    title="test100r5b data frame rowSelection and transforms")
  if (latticeLoaded)
  {
    checkEquals(plotObj1a$packet.sizes, nrow(myDF1))
  }
  myTransforms <- function( dataList )
  {
    dataList$xnum3 <- dataList$xnum1 + dataList$xnum2
    dataList$xint2 <- dataList$xint1 + 1
    return (dataList)
  }

    plotObj1b <- rxLinePlot(xnum3~xnum2, data=test100r5b, type=c("p","r"),
    transformFunc=myTransforms, transformVars=c("xnum1", "xnum2", "xint1"), rowSelection = xint2 > 8, 
    title="test100r5b rowSelection and transformFunc")
  
  plotObj1c <- rxLinePlot(xnum3~xnum2, data=myDF, type=c("p","r"),
    transformFunc=myTransforms, transformVars=c("xnum1", "xnum2", "xint1"), rowSelection = xint2 > 8, 
    title="test100r5b data frame rowSelection and transformFunc")
  
  # Check to make sure the same data is being plotted using transforms
  # and transformFunc
  if (latticeLoaded)
  {
    checkEquals(plotObj1$panel.args[[1]]$x, plotObj1a$panel.args[[1]]$x)
    checkEquals(plotObj1$panel.args[[1]]$y, plotObj1a$panel.args[[1]]$y)
    checkEquals(plotObj1$panel.args[[1]]$x, plotObj1b$panel.args[[1]]$x)
    checkEquals(plotObj1$panel.args[[1]]$y, plotObj1b$panel.args[[1]]$y)
    checkEquals(plotObj1$panel.args[[1]]$x, plotObj1c$panel.args[[1]]$x)
    checkEquals(plotObj1$panel.args[[1]]$y, plotObj1c$panel.args[[1]]$y)
  }
  # Without rowSelection
    plotObj2 <- rxLinePlot(xnum3~xnum2, data=test100r5b, type=c("p","r"),
    transforms=list(xnum3=xnum1+xnum2), symbolColor="red", lineStyle="dotted",
    title="test100r5b transforms")
  if (latticeLoaded)
  {
    checkEquals(plotObj2$packet.sizes, 100)
  }
}

"test.rxLinePlot.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  # Only used if rowSelection or transform
    co1 <- capture.output(rxLinePlot(xnum1~xnum2, data=test100r5b, rowSelection = xint1 > 7,
    type="p", blocksPerRead =2, reportProgress=1,
    title="test100r5b blocksPerRead = 2"))
    if (!rxIsExpressEdition())
    {
        checkTrue(length(grep("Rows Processed: 40", co1, fixed=TRUE)) > 0)
        checkTrue(length(grep("Rows Processed: 80", co1, fixed=TRUE)) > 0)
    }
    
    checkTrue(length(grep("Rows Processed: 100", co1, fixed=TRUE)) > 0)
}

"test.rxLinePlot.data.xdf.RxTextData" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    xdfPlot1 <- rxLinePlot(xnum1~xnum2, data=test100r5b, type="p", title="test100r5b xdf file")
    xdfPlot2 <- rxLinePlot(xnum1~xfac1, data=test100r5b, type="p", title="test100r5b xdf file with factor")
    xdfPlot3 <-rxLinePlot(xnum1~xnum2miss, data=test100r5b, type="p", 
        symbolSize=1.2, symbolColor="papayawhip", plotAreaColor="steelblue3",
        title="test100r5b xdf file with missings and papayawhip")
    
    textFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    textDS <- RxTextData(textFile, stringsAsFactors = TRUE)
    dsPlot1 <- rxLinePlot(xnum1~xnum2, data=textDS, type="p", title="test100r Text Data Source")
    dsPlot2 <- rxLinePlot(xnum1~xfac1, data=textDS, type="p", title="test100r5b Text Data Source with Factor")
    dsPlot3 <- rxLinePlot(xnum1~xnum2miss, data=textDS, type="p", 
        symbolSize=1.2, symbolColor="olivedrab", plotAreaColor="mistyrose",
        title="test100r Text Data Source with Missings in olivedrab") 
    
    checkEquals(xdfPlot1$panel.args, dsPlot1$panel.args)
    checkEquals(xdfPlot2$panel.args, dsPlot2$panel.args)
    checkEquals(xdfPlot3$panel.args, dsPlot3$panel.args)  
}

test.rxLinePlot.data.missingVar <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    # Try to do a line plot using a variable that isn't there
    # xdf file
    res <- try(
        rxLinePlot(xnum1~xnum3, data=test100r5b)
    )
    checkTrue(is(res, "try-error"))
    checkTrue(grepl("rxLinePlot", res[[1]]))
    checkTrue(grepl("xnum3", res[[1]]))
    
    # data frame
    myData <- rxReadXdf(test100r5b)
    res <- try(
        rxLinePlot(xnum1~xnum3, data=myData)
    )
    checkTrue(is(res, "try-error"))
    checkTrue(grepl("rxLinePlot", res[[1]]))
    checkTrue(grepl("xnum3", res[[1]])) 
    
    # text data source
    textDS <- RxTextData(file.path(rxGetOption("unitTestDataDir"), "test100r.txt"))
    res <- try(
        rxLinePlot(xnum1~xnum3, data=textDS)
    )
    checkTrue(is(res, "try-error"))
    checkTrue(grepl("rxLinePlot", res[[1]]))
    checkTrue(grepl("xnum3", res[[1]]))        
    
}

"test.rxLinePlot.args.panel" <- function()
{
  ageSex <- data.frame(
    age = c(1:10, 1:10),
    sex = factor(rep(c("M", "F"), each = 10)),
    Counts = c( 5,7,9,3,4, 5,6,7,8,2, 3,4,6,8,7, 9,4,5,9,3)
  )
    
  latticeLoaded <- "lattice" %in% .packages()
    plotObj <- rxLinePlot(Counts ~ age| sex, data = ageSex, 
    panel = function(x, y, ...)
    {
      panel.spline(x,y, lwd = 2, col = 1)
    },
        main = "Spline panel function") 
  
  if (latticeLoaded)
  {
    checkTrue( grepl("panel.spline", paste(deparse(plotObj$panel.args.common$panel.user), collapse = " ")))
  }
    
  # Check to make sure it works within a function
  myPlot <- function(formula, data)
  {
    plotObj <- rxLinePlot(formula, data, 
    panel = function(x, y, ...)
    {
      panel.spline(x,y, lwd = 2, col = 1)
    },
        main = "Spline panel function") 
  }
  
  if (latticeLoaded)	
  {
    plotObj2 <- myPlot(Counts ~ age|sex, data = ageSex)
    checkTrue( grepl("panel.spline", paste(deparse(plotObj2$panel.args.common$panel.user), collapse = " ")))
  }
  
}

"test.rxLinePlot.args.print" <- function()
{
  testData <- data.frame(x = 1:10, y = 1:10)
  testFile <- tempfile(pattern = ".rxLinMod", fileext = ".pdf")
  if (file.exists(testFile)) (file.remove(testFile))
  pdf(file = testFile)
  testPlotObj <- rxLinePlot(y~x, data = testData, print = FALSE)
  dev.off()
  fileSizePrintFalse <- file.info(testFile)$size
  file.remove(testFile)
  
  pdf(file = testFile)
  testPlotObj <- rxLinePlot(y~x, data = testData, print = TRUE)
  dev.off()
  fileSizePrintTrue <- file.info(testFile)$size
  checkTrue(fileSizePrintTrue > fileSizePrintFalse)
  file.remove(testFile)
  # Known bug in R (14991) that temporary files are left behind when 
  # creating pdf
  unlink(list.files(path=tempdir(), pattern="^pdf.", full.names=TRUE))

}
"test.rxLinePlot.args.groups.axisrange" <- function()
{
  testData <- data.frame(x = 1:20, y = 11:30, groupVar = factor(rep(1:2, each = 10)))
  rxLinePlot(y~x, groups = groupVar, data = testData, type = "b", lineColor = c("red", "blue"))
  testData2 <- data.frame(y1 = 1:10, y2 = 11:20, y3 = 21:30, x = 15:24)
  rxLinePlot(y1+y2+y3~x, data = testData2) 
}

"test.rxLinePlot.data.RxOdbcData" <- function()
{
   rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), 
                useFastRead = FALSE),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    #conString <- "Driver=SQLite3 ODBC Driver;Database=C:\\Users\\sue\\SVN\\bigAnalytics\\trunk/revoAnalytics/inst/SampleData/claims.sqlite"        
  odbcDS <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                useFastRead = FALSE,
                colClasses = c("RowNum" = "integer", "age" = "character",
                 "car_age" = "character", "type" = "character",
                 "cost" = "integer", "number" = "integer"))
  result = rxLinePlot(cost~number, data=odbcDS, title = "ODBC data")
    # There should be no warnings
 }