# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLocateFile
#
############################################################################################


"test.rxLocateFile.args.file" <- function()
{
    # Full path
    testFileName <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
    fullFileName <- normalizePath(testFileName)
    locatedFileName <- rxLocateFile( testFileName )
    checkEquals(fullFileName, locatedFileName)
    
    # Full path, no extension
    testFileName <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall")
    locatedFileName <- rxLocateFile( testFileName )
    checkEquals(fullFileName, locatedFileName)
    
    # No path, with extension
    testFileName <- "AirlineDemoSmall.xdf"
    pathsToSearch <- rxGetOption("sampleDataDir")
    locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)
    checkEquals(fullFileName, locatedFileName)
    
    # No path, no extension
    testFileName <- "AirlineDemoSmall"
    pathsToSearch <- rxGetOption("sampleDataDir")
    locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)
    checkEquals(fullFileName, locatedFileName)  
    
    # Xdf data source
    testFileName <- RxXdfData("AirlineDemoSmall")
    pathsToSearch <- rxGetOption("sampleDataDir")
    locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)
    checkEquals(fullFileName, locatedFileName)   
    
    # Non-existant file should throw an error
    locatedFileName <- try(
        rxLocateFile(file = "nosuchfile.xdf")
        )
    checkTrue(is(locatedFileName, "try-error")) 
    
    # Partial path
    #testFileName <- "\\SampleData\\AirlineDemoSmall.xdf"
    #pathsToSearch <- sub("SampleData", "", pathsToSearch)
    #locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)  
    #checkEquals(fullFileName, locatedFileName)  
}

"test.rxLocateFile.args.pathsToSearch" <- function()
{
    # test for multiple paths
    testFileName <- "AirlineDemoSmall.xdf"
    fullFileName <- normalizePath(file.path(rxGetOption("sampleDataDir"), testFileName))
    pathsToSearch <- c(rxGetOption("unitTestDataDir"), rxGetOption("sampleDataDir"))
    locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)
    checkEquals(fullFileName, locatedFileName)
    
    # Reverse the order
    pathsToSearch <- c(rxGetOption("sampleDataDir"), rxGetOption("unitTestDataDir"))
    locatedFileName <- rxLocateFile(file = testFileName, pathsToSearch = pathsToSearch)
    checkEquals(fullFileName, locatedFileName)
}