# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLorenz and related methods
#
############################################################################################

"test.rxLorenz.data.dataframe" <- function()
{
  set.seed(59)
  myData <- data.frame(income = 1000 + rnorm(10000)* 100 + 1:10000)
  dataOut <- rxLorenz("income", data = myData, numBreaks = 100000)
  plot(dataOut)
  rxGini( dataOut)
}

"test.rxLorenz.args.valueVarName" <- function()
{
  set.seed(59)
  myData <- data.frame(income = 1000 + rnorm(10000)*100 + (1:10000)/5,
    wealth =  1000 + rnorm(10000)* 100 + (1:10000) )
  dataOut <- rxLorenz("income", valueVarName = "wealth",
    data = myData, numBreaks = 10)
  plot(dataOut)
  rxGini( dataOut)
}

"test.rxLorenz.data.small" <- function()
{
  #set.seed(59)
  #testData <- data.frame(income = 200 + rnorm(10)* 100 + 1:10)
  
  testData <- data.frame(income = c(14.78663, 315.98069, 234.76267, 343.98593, 428.76260,
    149.00910, 149.09506, 93.05054, 288.67544, 250.34600))
  # Results from Lc function in ineq package
  # compareRes <- Lc(testData$income)
  # compareGini <- Gini(testData$income)
  p <- c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0)
  L <- c(0.000000000,0.006518371, 0.047537723, 0.113225214, 0.178950599, 0.282440732,
    0.392800444, 0.520056874, 0.659350244, 0.810989126,	1.000000000)
  compareOut <- data.frame(p = p*100, L = L*100)	
  compareGini <- 0.2976261
  
  lorenzOut <- rxLorenz("income", data = testData, numBreaks = 100)
  
  checkEquals(lorenzOut$cumVals, compareOut$L, tolerance = 1E-6)
  checkEquals(compareGini, rxGini(lorenzOut), tolerance = 1E-6)
}


"test.rxLorenz.data.negative" <- function()
{

  testData <- data.frame(income = c(-200, -100, -50, 0, 50, 100, 200, 300, 400, 600))
  # Results from Lc function in ineq package
  # compareRes <- Lc(testData$income, plot=TRUE)
  # compareGini <- Gini(testData$income)
  p <- c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0)
  L <- c(0.000000000, -0.1538462, -0.2307692, -0.2692308, -0.2692308, -0.2307692,
     -0.1538462,  0.0000000,  0.2307692,  0.5384615,	1.000000000)
  compareOut <- data.frame(p = p*100, L = L*100)	
  compareGini <- 1.007692
  
  lorenzOut <- rxLorenz("income", data = testData, numBreaks = 100)
  # TO DO: HOW TO HANDLE NEGATIVE VALUES
  checkEquals(lorenzOut$cumVals, compareOut$L, tolerance = 1E-6)
  #checkEquals(compareGini, rxGini(lorenzOut))
}

"test.rxLorenz.data.scale" <- function()
{
  
  # Lorenz curve computations should be indpendent of multiple
  testData <- data.frame(income = c(100, 200, 300, 400, 600, 800, 1000, 1100, 1500, 2000))
  
  a <- 0
  b <- 10
  testData$income2 <- a + testData$income*b
  a <- 100
  b <- 0
  testData$income3 <- a + testData$income*b
  lorenzOut1 <- rxLorenz("income", data = testData, numBreaks = 100)
  lorenzOut2 <- rxLorenz("income2", data = testData, numBreaks = 100)
  lorenzOut3 <- rxLorenz("income3", data = testData, numBreaks = 100)

  checkEquals(lorenzOut1$cumVals, lorenzOut2$cumVals)
}

"test.rxLorenz.data.extremes" <- function()
{
  # Lorenz curve for complete equality
  testData <- data.frame(income = rep(100, times=10))
  lorenzOut1 <- rxLorenz("income", data = testData, numBreaks = 100)
  plot(lorenzOut1)
  rxGini(lorenzOut1)
  
  # Extreme inequality
  
  testData <- data.frame(income = c(rep(0, times=99), 100))
  lorenzOut2 <- rxLorenz("income", data = testData, numBreaks = 100)
  plot(lorenzOut2, equalityWidth = 3, equalityColor = "black")
  
  
}
"test.rxLorenz.plot.args.lines" <- function()
{
  testData <- data.frame(income = c(rep(0, times=99), 100))
  lorenzOut <- rxLorenz("income", data = testData, numBreaks = 100)
  
  # This requires a visual check
  lorenzPlot <- plot(lorenzOut, equalityWidth = 3, 
    equalityColor = "black", equalityStyle = "dotted",
    lineStyle = "dashed", lineColor = "red", lineWidth = 4,
    gridColor = "blue")
  myGridColor = "red"
  myLineColor = "green"
  lorenzPlot <- plot(lorenzOut, gridColor = myGridColor, lineColor = myLineColor)
  
}

"test.rxLorenz.args.pweight" <- function()
{

    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers") 

  lorenzOut <- rxLorenz(orderVarName = "incwage", data = censusWorkers,
    pweights = "perwt")
  
  lorenzPlot <- plot(lorenzOut, equalityWidth = 1, 
    title = "Lorenz Curve for Workers from Three States",
    subtitle = "Data Source: 5% Sample of U.S. 2000 Census",
    equalityColor = "black", equalityStyle = "longdash", lineWidth = 3)
  
  # Compute Gini
  giniCoef <- rxGini(lorenzOut)
}


"test.rxLorenz.args.valueVarName" <- function()
{
  actual <- 101:200
  pred <- actual + rep(c(15, 12, -17, 8, -30), times = 20)
  
  testData <- data.frame(actual, pred)

  lorenzOut <- rxLorenz(valueVarName = "actual", orderVarName = "pred", data = testData)

  lorenzOut2 <- rxLorenz(valueVarName = "actual", orderVarName = "actual", data = testData)
  # Compute normalized Gini
  rxGini(lorenzOut)/rxGini(lorenzOut2)
  # TODO: Provide check
}




