# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxMerge and rxMergeXdf
#
############################################################################################

"test.rxMergeXdf.args.inData.basic" <- function()
{
  x <- 1:20
  y <- 20:1
  df <- data.frame(x=x, y=y)
  # Create two identical xdf files sorted on x
  
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxDataStep(inData =df, outFile = inFile1)
  rxDataStep(inData =df, outFile = inFile2)
  
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "x", overwrite=TRUE)
  varInfo <- rxGetVarInfo( outXDF )
  checkEquals(length(varInfo), 3)
  df1 <- rxReadXdf( file=outXDF)
  checkEquals(df1[[2]], y)
  checkEquals(df1[[3]], y)
  
  if (file.exists( outXDF )) file.remove( outXDF )
  if (file.exists( inFile1 )) file.remove( inFile1 )
  if (file.exists( inFile2 )) file.remove( inFile2 )	
  
}

"test.rxMergeXdf.args.xdfCompressionLevel" <- function()
{
  x <- 1:30
  y <- 30:1
  df <- data.frame(x=x, y=y)
  # Create two identical xdf files sorted on x, but one compressed
  
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxDataStep(inData = df, outFile = inFile1, rowsPerRead = 12, xdfCompressionLevel = 0)
  rxDataStep(inData = df, outFile = inFile2, rowsPerRead = 12, xdfCompressionLevel = 1)
  
  rxMerge(inData1 = inFile1, inData2 = inFile2, outFile=outXDF, matchVars = "x", 
    overwrite=TRUE, xdfCompressionLevel = 0)
  
  varInfo <- rxGetVarInfo( outXDF )
  checkEquals(length(varInfo), 3)
  df1 <- rxReadXdf( file=outXDF)
  checkEquals(df1[[2]], y)
  checkEquals(df1[[3]], y)
  mergeFileInfo1 <- file.info(outXDF)
  mergeFileSize1 <- mergeFileInfo1$size
  
  if (file.exists( outXDF )) file.remove( outXDF )
  
  rxMerge(inData1 = inFile1, inData2 = inFile2, outFile=outXDF, matchVars = "x", 
    overwrite=TRUE, xdfCompressionLevel = 1)
  
  varInfo <- rxGetVarInfo( outXDF )
  checkEquals(length(varInfo), 3)
  df1 <- rxReadXdf( file=outXDF)
  checkEquals(df1[[2]], y)
  checkEquals(df1[[3]], y)
  mergeFileInfo2 <- file.info(outXDF)
  mergeFileSize2 <- mergeFileInfo2$size
  
  checkTrue(mergeFileSize2 < mergeFileSize1)
  
  if (file.exists( outXDF )) file.remove( outXDF )
  if (file.exists( inFile1 )) file.remove( inFile1 )
  if (file.exists( inFile2 )) file.remove( inFile2 )	
  
}

"test.rxMergeXdf.args.newVarNames" <- function()
{
  x <- 1:20
  y <- 20:1
  df <- data.frame(x=x, y=y)
  
  # Create two identical xdf files sorted on x
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxDataStep(inData =df, outFile = inFile1)
  rxDataStep(inData =df, outFile = inFile2)
  
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "x",
    newVarNames1 = c(y = "y1"), newVarNames2 = c(y = "y2"), overwrite=TRUE)
  varInfo <- rxGetVarInfo( outXDF )
  checkEquals(length(varInfo), 3)
    varNames <- names(varInfo)
  checkEquals(varNames[2], "y1")
  checkEquals(varNames[3], "y2")
  
  if (file.exists( outXDF )) file.remove( outXDF )
  if (file.exists( inFile1 )) file.remove( inFile1 )
  if (file.exists( inFile2 )) file.remove( inFile2 )
}
    
"test.rxMerge.args.newVarNames" <- function() 
{   
    
  # BUG RA-4225
  # Use data frames
    x1 <- 1:20
    y1 <- 20:1
    df1 <- data.frame(x1=x1, y1=y1)
       
    x2 <- 1:20
    y2 <- 25:6
    df2 <- data.frame(x2=x2, y2=y2)
       
    dfOut <- rxMerge(inData1 = df1, inData2 = df2, matchVars = "y2", 
             newVarNames1 = c(y1 = "y2") )

  varInfo <- rxGetVarInfo( dfOut )
  checkEquals(length(varInfo), 3)
    varNames <- names(varInfo)
  checkEquals(varNames[2], "y2")
  checkEquals(varNames[3], "x2")  
    
    df1Names <- names(df1)
    df1Names[2] <- "y2"
    names(df1) <- df1Names
    dfTest <- merge(df1, df2)

    dfTest <- dfTest[order(dfTest$y2),] 
    checkEquals(dfTest$x1, dfOut$x1)
    checkEquals(dfTest$x2, dfOut$x2)
    checkEquals(dfTest$y2, dfOut$y2)
    
    # RA-4496
    
    X1 <- factor(c("a","b","a","d", "G"), levels=letters) 
    X2 <- 5:1 

    Y1 <- factor(c("a","e","f","a","k"),levels=letters) 
    Y2 <- 1:5 

    DF1 <- data.frame(X1, X2) 
    DF2 <- data.frame(Y1, Y2) 

    DFOut <- rxMerge(inData1 = DF1, inData2 = DF2, matchVars = "X1", newVarNames2 = c(Y1="X1"),
        type = "left", autoSort = TRUE)
     rxGetVarInfo(DFOut) 

    inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
    rxDataStep(DF1, inFile1, overwrite=TRUE) 
    rxDataStep(DF2, inFile2, overwrite=TRUE) 
     
    DFOut2 <- rxMerge(inData1 = inFile1, inData2 = inFile2, matchVars = "X1", newVarNames2 = c(Y1="X1"),
        type = "left", autoSort = TRUE)
    
    checkEquals( names(DFOut), c("X1", "X2", "Y2"))
    checkEquals(DFOut, DFOut2)

    file.remove(inFile1)
    file.remove(inFile2)
  
}
"test.rxMergeXdf.args.type.inner" <- function()
{
  #RA-3677 merging character data
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  lastName <- c("Rafferty", "Jones", "Steinberg", "Robinson", "Smith", "John")
  deptID <- as.integer(c(31,33,33,34,34,NA))
  employee <- data.frame(lastName, deptID)
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  rxMergeXdf(inFile1 = employeeXDF, inFile2 = deptXDF, outFile=outXDF, matchVars = "deptID", overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  
  # Employee without ID is dropped
  checkEquals( nrow(newData), nrow(employee)-1)
  # Column is added
  checkEquals( ncol(newData), 3)
  checkEquals( subset(newData, deptID == 31, select=deptName)[1,1], "Sales")
  checkEquals( subset(newData, deptID == 33, select=deptName)[1,1], "Engineering")
  checkEquals( subset(newData, deptID == 34, select=deptName)[1,1], "Clerical")

  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove( outXDF )

}

"test.rxMergeXdf.args.newVarNames.matchVars.inner" <- function()
{
  x1 <- 1:20
  y1 <- 20:1
  df1 <- data.frame(x1=x1, y1=y1)
  
  x2 <- 1:20
  y2 <- 25:6
  df2 <- data.frame(x2=x2, y2=y2)
  
  # Create two identical xdf files sorted on y1 in first file and y2 in second file
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxDataStep(inData =df1, outFile = inFile1)
  rxDataStep(inData =df2, outFile = inFile2)
  
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "y2", decreasing=TRUE,
    newVarNames1 = c(y1 = "y2"))
  info <- rxGetInfo( outXDF,numRows=20,getVarInfo=TRUE )
  checkEquals(length(info$varInfo), 3)
  checkEquals(nrow(info$data), 15)
    varNames <- names(info$varInfo)
  checkEquals(varNames[1], "x1")
  checkEquals(varNames[2], "y2")
  checkEquals(varNames[3], "x2")
  checkEquals(info$data$x1,1:15)
  checkEquals(info$data$x2,6:20)
  checkEquals(info$data$y2,20:6)
  
  if (file.exists( outXDF )) file.remove( outXDF )
  if (file.exists( inFile1 )) file.remove( inFile1 )
  if (file.exists( inFile2 )) file.remove( inFile2 )
  
}

"test.rxMerge.args.matchVars.union" <- function()
{
  x1 <- 1:20
  y1 <- 20:1
  df1 <- data.frame(x1=x1, y1=y1)
  
  x2 <- 1:20
  y2 <- 25:6
  df2 <- data.frame(x2=x2, y2=y2)
  
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Matchvars not needed for "union" - it just append the rows of one to the other
  rxMerge(inData1 = df1, inData2 = df2, outFile=outXDF, type="union")
  
  info <- rxGetInfo( outXDF )
  checkEquals( info$numRows, 40)
  checkEquals( info$numVars, 2)
  
  if (file.exists( outXDF )) file.remove( outXDF )
  
}
"test.rxMergeXdf.data.mathcVars.differentVarTypes.error" <- function()
{
  df1 <- data.frame(animal=c("ape","bird","cat","dog","fish","lizard"), class=c("mammal","bird","mammal","mammal","amphibian","reptile"),
    numLegs=c(2,0,4,4,NA,4), numArms=c(2,NA,0,0,NA,0), 
    flies=c(FALSE,TRUE,FALSE,FALSE,FALSE,FALSE),
    stringsAsFactors=FALSE)
  df2 <- 	data.frame(class=c("arthropod","reptile","bird","mammal", "amphibian"), 
    animal=c("fly","frog","parrot","rat","snake"), 
    stringsAsFactors=TRUE)
  xdfFile1 <- tempfile( pattern = "rxMerge1", fileext = ".xdf")
    xdfFile2 <- tempfile( pattern = "rxMerge2", fileext = ".xdf")
  xdfSort1 <- tempfile( pattern = "rxSort1", fileext = ".xdf")
    xdfSort2 <- tempfile( pattern = "rxSort2", fileext = ".xdf")
    xdfOut <- tempfile( pattern = "rxOut", fileext = ".xdf")
        
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = rbind(df1, df1), outFile = xdfFile1, overwrite = TRUE)
        rxDataStep(inData = rbind(df2, df2), outFile = xdfFile2, overwrite = TRUE)
    } else
    {
      rxDataStep(df1, xdfFile1, overwrite=TRUE)
      rxDataStep(df2, xdfFile2, overwrite=TRUE)
      # append another copy to get two blocks
      rxDataStep(df1, xdfFile1, append= "rows")
      rxDataStep(df2, xdfFile2, append= "rows")
    }
    
  
  rxSortXdf(xdfFile1, xdfSort1, sortByVars=c("class","animal"), overwrite=TRUE)
  rxSortXdf(xdfFile2, xdfSort2, sortByVars=c("class","animal"), overwrite=TRUE)
  
  # This should fail since in one file the class is a character variable and in the other it is a factor
  checkException(rxMergeXdf(inFile1=xdfSort1,inFile2=xdfSort2,outFile=xdfOut, matchVars="class",overwrite=TRUE))

  file.remove(xdfFile1)
  file.remove(xdfFile2)
  file.remove(xdfSort1)
  file.remove(xdfSort2)
  if(file.exists(xdfOut)) file.remove(xdfOut)
  
}
"test.rxMergeXdf.args.matchVars.two" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkinDAT <- file.path(unitTestDataDir, "Pumpkin.dat")
  pumpkinXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  pumpkinXDF2s <- file.path(tempdir(), ".rxOutFileTemp2sorted.xdf")
  pumpkinXDFs <- file.path(tempdir(), ".rxOutFileTemp1sorted.xdf")
  pumpkinOutXdf <- file.path(tempdir(), ".rxOutFileTempOut.xdf")	
  
  pumpkinDS <- RxTextData(file=pumpkinDAT, firstRowIsColNames = FALSE,
    missingValueString=".", 
    readDateFormat = "%m-%d-%Y",
    colInfo = list(
      Col1 = list(newName = "Name", start=1, width=17),
      Col2 = list(newName = "Age",  type="integer", start=18, width=2),
      Col3 = list(newName = "Type", start=21, width=1),
      Col4 = list(newName = "Date", type="Date", start=23, width=10),
      Col5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      Col6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      Col7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      Col8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      Col9 = list(newName = "Scr5", type="numeric", start=49, width=4)) )
    
  rxImport(inData=pumpkinDS, outFile=pumpkinXDF, overwrite=TRUE)

  rxSortXdf(pumpkinXDF, pumpkinXDFs, sortByVars=c("Name","Age"), overwrite=TRUE)

  pumpkinDF1 <- rxDataStep(pumpkinXDFs)
  pumpkinDF2s <- data.frame(pumpkinDF1[,c(1,4)],scr6=c(3.2, 4.5, 6.0, 5.5, 6.6, 4.3), pumpkinDF1["Age"])
  
  rxSort(pumpkinDF2s, pumpkinXDF2s, sortByVars=c("Name","Age"), overwrite=TRUE)
  
  # merge 2 xdf files
  rxMergeXdf(inFile1=pumpkinXDFs, inFile2=pumpkinXDF2s, outFile=pumpkinOutXdf, 
    matchVars=c("Name", "Age"), varsToDrop2=c("Date"), overwrite=TRUE)
  
  # output
  infoOut <- rxGetInfo(pumpkinOutXdf, getVarInfo=TRUE, numRows=20)
  
  expect <- data.frame(pumpkinDF1, Src6=pumpkinDF2s[,3])
  
    checkEquals(expect, infoOut$data, checkNames=FALSE)
  
  file.remove(pumpkinXDF)
  file.remove(pumpkinXDFs)
  file.remove(pumpkinXDF2s)
  file.remove(pumpkinOutXdf)
}

"test.rxMergeXdf.args.matchVars.date" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkinDAT <- file.path(unitTestDataDir, "Pumpkin.dat")
  pumpkinXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  pumpkinXDF2s <- file.path(tempdir(), ".rxOutFileTemp2sorted.xdf")
  pumpkinXDFs <- file.path(tempdir(), ".rxOutFileTemp1sorted.xdf")
  pumpkinOutXdf <- file.path(tempdir(), ".rxOutFileTempOut.xdf")	
  pumpkinOutXdf1 <- file.path(tempdir(), ".rxOutFileTempOut1.xdf")
    
  pumpkinDS <- RxTextData(file=pumpkinDAT, firstRowIsColNames = FALSE,
    useFastRead=FALSE,
    missingValueString=".", 
    readDateFormat = "%m-%d-%Y",
    colInfo = list(
      Col1 = list(newName = "Name", start=1, width=17),
      Col2 = list(newName = "Age",  type="integer", start=18, width=2),
      Col3 = list(newName = "Type", start=21, width=1),
      Col4 = list(newName = "Date", type="Date", start=23, width=10),
      Col5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      Col6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      Col7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      Col8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      Col9 = list(newName = "Scr5", type="numeric", start=49, width=4)) )
    
  rxImport(inData=pumpkinDS, outFile=pumpkinXDF, overwrite=TRUE)
  
  rxSortXdf(pumpkinXDF, pumpkinXDFs, sortByVars=c("Date","Name"), overwrite=TRUE)
  
  pumpkinDF1 <- rxDataStep(pumpkinXDFs)
  pumpkinDF2s <- data.frame(pumpkinDF1[,c(1,4)],scr6=c(3.2, 4.5, 6.0, 5.5, 6.6, 4.3), Age=c(13,11,10,6,9,7))
  
  #  rxMerge (df and xdf file)
  rxMerge(inData1=pumpkinXDFs, inData2=pumpkinDF2s, outFile=pumpkinOutXdf1, 
    matchVars=c("Date","Name"), overwrite=TRUE)
  
  #rxReadXdf(pumpkinOutXdf1)	
  
  # merge 2 xdf files
  rxDataStep(pumpkinDF2s, pumpkinXDF2s, overwrite=TRUE)
  rxMergeXdf(inFile1=pumpkinXDFs, inFile2=pumpkinXDF2s, outFile=pumpkinOutXdf, 
    matchVars=c("Date","Name"), overwrite=TRUE)		
  
  # check that using DF and XDF file give same result:
  checkEquals(rxGetInfo(pumpkinOutXdf,numRows=10)$data, rxGetInfo(pumpkinOutXdf1,numRows=10)$data, checkNames=FALSE)
  # so far so good
  infoOut <- rxGetInfo(pumpkinOutXdf, getVarInfo=TRUE, numRows=20)
  
  expect <- data.frame(pumpkinDF1, Src6=pumpkinDF2s[,3], pumpkinDF1["Age"])

  checkEquals(expect, infoOut$data, checkNames=FALSE)
  
  file.remove(pumpkinXDF)
  file.remove(pumpkinXDFs)
  file.remove(pumpkinXDF2s)
  file.remove(pumpkinOutXdf1)
}

"test.rxMergeXdf.data.matchVars.differentVarTypes2.errors" <- function()
{
  unitTestDataDir <- rxGetOption("unitTestDataDir")
  pumpkinDAT <- file.path(unitTestDataDir, "Pumpkin.dat")
  pumpkinXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  pumpkinXDF2s <- file.path(tempdir(), ".rxOutFileTemp2sorted.xdf")
  pumpkinXDFs <- file.path(tempdir(), ".rxOutFileTemp1sorted.xdf")
  pumpkinOutXDF <- file.path(tempdir(), ".rxOutFileTempOut.xdf")	
  
  pumpkinDS <- RxTextData(file=pumpkinDAT, firstRowIsColNames = FALSE,
    missingValueString=".", 
    readDateFormat = "%m-%d-%Y",
    colInfo = list(
      V1 = list(newName = "Name", start=1, width=17),
      V2 = list(newName = "Age",  type="integer", start=18, width=2),
      V3 = list(newName = "Type", start=21, width=1),
      V4 = list(newName = "Date", type="Date", start=23, width=10),
      V5 = list(newName = "Scr1", type="numeric", start=33, width=4),
      V6 = list(newName = "Scr2", type="numeric", start=37, width=4),
      V7 = list(newName = "Scr3", type="numeric", start=41, width=4),
      V8 = list(newName = "Scr4", type="numeric", start=45, width=4),
      V9 = list(newName = "Scr5", type="numeric", start=49, width=4)) )
    
  rxImport(inData=pumpkinDS, outFile=pumpkinXDF, overwrite=TRUE)

  rxSortXdf(pumpkinXDF, pumpkinXDFs, sortByVars="Date", overwrite=TRUE)

  pumpkinDF1 <- rxDataStep(pumpkinXDFs)
  pumpkinDF2s <- data.frame(Name=pumpkinDF1[,1], 
    Date=as.double(3.2, 4.5, 6.0, 5.5, 6.6, 4.3), scr6=c(3.2, 4.5, 6.0, 5.5, 6.6, 4.3), 
    Age=c("13","10","11","9","6","7"))
  rxSort(pumpkinDF2s, pumpkinXDF2s, sortByVars="Date", overwrite=TRUE)
  
  # merge 2 xdf files on key with different types in Date variable, then Age

  checkException(
    rxMergeXdf(inFile1=pumpkinXDFs, inFile2=pumpkinXDF2s, outFile=pumpkinOutXDF, 
    matchVars=c("Date"), overwrite=TRUE)
  )
  rxSortXdf(pumpkinXDF, pumpkinXDFs, sortByVars="Age", overwrite=TRUE)
  rxSort(pumpkinDF2s, pumpkinXDF2s, sortByVars="Age", overwrite=TRUE)
  checkException(
    rxMergeXdf(inFile1=pumpkinXDFs, inFile2=pumpkinXDF2s, outFile=pumpkinOutXDF, 
    matchVars=c("Age"), overwrite=TRUE)
  )

  file.remove(pumpkinXDF)
  file.remove(pumpkinXDFs)
  file.remove(pumpkinXDF2s)
  if (file.exists(pumpkinOutXDF)) file.remove(pumpkinOutXDF)
}
"test.rxMergeXdf.args.multipleMatchVars.inner" <- function()
{
  x1 <- 1:20
  y1 <- 20:1
  z1 <- c(rep(5,5),6:15,rep(16,5))
  df1 <- data.frame(x1=x1, y1=y1, z1=z1)

  renameExtension1 <- ".rxTempIn1"
  renameExtension2 <- ".rxTempIn2"
  
  # Create two identical xdf files sorted on y1 in first file and y2 in second file
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxDataStep(inData =df1, outFile = inFile1)
  rxDataStep(inData =df1, outFile = inFile2)
  
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("x1","z1"), decreasing=FALSE)
  info <- rxGetInfo( outXDF,numRows=20,getVarInfo=TRUE )
  checkEquals(length(info$varInfo), 4)
  checkEquals(nrow(info$data), 20)
    varNames <- names(info$varInfo)
  checkEquals(varNames[1], "x1")
  checkEquals(varNames[2], paste("y1", renameExtension1, sep="."))
  checkEquals(varNames[3], "z1")
  checkEquals(varNames[4], paste("y1",renameExtension2,sep="."))
  checkEquals(info$data$x1,x1)
  checkEquals(info$data$y1..rxTempIn1,y1)
  checkEquals(info$data$z1,z1)
  
  if (file.exists( outXDF )) file.remove( outXDF )
  if (file.exists( inFile1 )) file.remove( inFile1 )
  if (file.exists( inFile2 )) file.remove( inFile2 )
  
}

"test.rxMergeXdf.args.decreasing" <- function()
{
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  lastName <- c("Rafferty", "Jones", "Steinberg", "Robinson", "Smith", "John")
  deptID <- as.integer(c(31,33,33,34,34,NA))
  employee <- data.frame(lastName, deptID) 
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  # use decreasing opposite to actual data:
  if(file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = employeeXDF, inFile2 = deptXDF, outFile=outXDF, matchVars = "deptID", decreasing=TRUE)
  newData <- rxReadXdf( file=outXDF )
  
  # only one row merged since data is opposite of setting for decreasing
  checkEquals( nrow(newData),1)
  # Column is added
  checkEquals( ncol(newData), 3)

  checkEquals( subset(newData, deptID == 31, select=deptName)[1,1], "Sales")


  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove( outXDF )

}

  
"test.rxMergeXdf.args.inData.differentBlockSize" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  
    if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"))
  info <- rxGetInfo( outXDF, getVarInfo=TRUE, getBlockSize=TRUE)
  checkEquals(info$data$yfac1, info$data$yfac1.R1)
  checkEquals( info$numVars, (rxGetInfo(testFile1)$numVars)*2 -1) 
  
  # check block sizes, by default it should be ave of 20 and 100
  checkEquals(info$numBlocks, 2)
  checkEquals(info$rowsPerBlock,c(60,40))
  
  #validate data is same as in orginal file
  sum.orig <- rxSummary(~y + xnum1, byTerm=TRUE, data=testFile1)
  sum.out <- rxSummary(~y..rxTempIn1 + xnum1..rxTempIn1, byTerm=TRUE, data=outXDF)
  sDataFrameOrig <- sum.orig$sDataFrame
  sDataFrameOut <- sum.out$sDataFrame
  # Remove the name column
  sDataFrameOrig$Name <- NULL
  sDataFrameOut$Name <- NULL
  checkEquals(sDataFrameOrig, sDataFrameOut, checkNames = FALSE)
  
  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
}
"test.rxMergeXdf.args.rowsPerOutputBlock" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  
    if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"), rowsPerOutputBlock=30)
  info <- rxGetInfo( outXDF, getVarInfo=TRUE, getBlockSize=TRUE)
  checkEquals(info$data$yfac1, info$data$yfac1.R1)
  checkEquals( info$numVars, (rxGetInfo(testFile1)$numVars)*2 -1) 
  
  checkEquals(info$numBlocks, 4)
  checkEquals(info$rowsPerBlock,c(30,30,30,10))

  if (file.exists(inFile1)) file.remove(inFile1)
  if (file.exists(inFile2)) file.remove(inFile2)
  if (file.exists(outXDF)) file.remove(outXDF)
  
}
"test.rxMergeXdf.args.rowsPerOutputBlock.2" <- function()
{
  df <- data.frame(ylogical = c(rep(TRUE,2),rep(FALSE,2)), x = 1:4, y = letters[1:4])
  #file1 <- "rxFile1.xdf"
    file1 <- tempfile(pattern = "rxFile1", fileext = ".xdf")
    file2 <- tempfile(pattern = ".rxOut", fileext = ".xdf")
  rxDataStep(inData =df, outFile=file1, overwrite=TRUE)
  rxMergeXdf(inFile1=file1, inFile2=file1, outFile=file2,matchVars="ylogical", decreasing=TRUE, 
    rowsPerOutputBlock=2, overwrite=TRUE)
  info <- rxGetInfo(file2,  getBlockSizes=TRUE)
  checkEquals(info$numRows, 8)
  checkEquals(info$numBlocks,4)
  checkEquals(info$rowsPerBlock,c(2,2,2,2))

  file.remove(file2)
  file.remove(file1)
}
"test.rxMergeXdf.args.reportProgress.verbose" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  
    if (file.exists(outXDF)) file.remove(outXDF)
  
  out0 <- capture.output(rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"), 
    reportProgress=0, verbose=0, overwrite=TRUE))
  outVerbose <- capture.output(rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"), 
    reportProgress=0, verbose=1, overwrite=TRUE))
  outProgress1 <- capture.output(rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"), 
    reportProgress=1, verbose=0, overwrite=TRUE))
  outProgress2 <- capture.output(rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"), 
    reportProgress=2, verbose=0, overwrite=TRUE))
  
  checkEquals(length(out0), 0)
  checkEquals(length(outVerbose), 4)
  checkTrue(length(outProgress1) > 0)
  checkTrue(length(outProgress2) > length(outProgress1))
  
  file.remove(inFile1)
  file.remove(inFile2)
  file.remove(outXDF)
}
"test.rxMergeXdf.args.varsToKeep" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  renameExtension1 <- ".rxTempIn1"
  renameExtension2 <- ".rxTempIn2"

  
    if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars=c("xnum2"), varsToKeep1=c("xnum2","xnum2miss","y"), varsToKeep2=c("ybinint","y","xnum2"))
  names.out <- names(rxGetVarInfo(outXDF))
  names.expect <- c("xnum2","xnum2miss",paste("y", renameExtension1, sep="."),"ybinint",paste("y",renameExtension2,sep="."))
  
  checkEquals(names.out,names.expect)
  info <- rxGetInfo(outXDF, getVarInfo=TRUE)
  
  file.remove(inFile1)
  file.remove(inFile2)
  file.remove(outXDF)
}
"test.rxMergeXdf.args.varsToDrop" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1Name <- ".rxTempIn1"
  inFile2Name <- ".rxTempIn2"
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf" )
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  renameExtension1 <- inFile1Name
  renameExtension2 <- inFile2Name

  
    if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars=c("xnum2"), varsToDrop1=c("xnum2miss","y"), varsToKeep2=c("ybinint","y","xnum2"))
  names.out <- names(rxGetVarInfo(outXDF))
  names.expect <- c("ylogical", paste("ybinint", renameExtension1, sep="."),"xnum1","xnum2", "xint1", 
    "xfac1", "xfac2",paste("ybinint",renameExtension2,sep="."), "y")
  
  checkEquals(names.out,names.expect)
  info <- rxGetInfo(outXDF, getVarInfo=TRUE)
  
  file.remove(inFile1)
  file.remove(inFile2)
  file.remove(outXDF)
}

"test.rxMergeXdf.duplicateVarRenaming" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testFile2 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  renameExtension1 <- ".rxTempIn1"
  renameExtension2 <- ".rxTempIn2"
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xnum2", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="xnum2", overwrite=TRUE)
  names1.in <- names(rxGetVarInfo(inFile1))
  names2.in <- names(rxGetVarInfo(inFile2))
  
    if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = c("xnum2"))
  names.out <- names(rxGetVarInfo(outXDF))
  checkTrue(length(unique(names.out))==(length(names1.in)+length(names2.in)-1))
  
  #Rename duplicate variable names in second file
  names1.out <- as.vector(sapply(names1.in[], paste,renameExtension1,sep=".")) 
  names1.out[5] <- "xnum2"
  names2.out <- as.vector(sapply(names2.in[], paste,renameExtension2,sep="."))[-5] # remove xnum2
  
  checkEquals(names.out, c(names1.out,names2.out))
  
  file.remove(inFile1)
  file.remove(inFile2)
  file.remove(outXDF)
}
#
#"test.rxMergeXdf.args.inData.sameBlockSize" <- function()
#{
  #testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  #
  #inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  #inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  #outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  #
  #rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xint1", overwrite=TRUE)
  #rxSortXdf(inFile=testFile2, outFile = inFile1, sortByVars="xint1", overwrite=TRUE)
  #
    #if (file.exists(outXDF)) file.remove(outXDF)
  #rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "xint1")
  #varInfo <- rxGetVarInfo( outXDF )
  ## Do a simple check to start
  #checkEquals(length(varInfoOut) > length(varInfoIn1))
  #
  #testData <- rxReadXdf(outXDF, varsToKeep = c("xnum1", "xnum1.1"))
  #checkEquals(testData$xnum1, testData$xnum1.1, checkNames=FALSE)
  #
  ## If you compare the contents, the results are not yet correct
  #
  #if (file.exists(inFile1)) file.remove(inFile1)
  #if (file.exists(inFile2)) file.remove(inFile2)
  #if (file.exists(outXDF)) file.remove(outXDF)
  #
#}
#
#"test.rxMergeXdf.args.inData.oneBlock" <- function()
#{
  #testFile1 <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
  #
  #inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  #inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  #outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  #
  #rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="xint1", overwrite=TRUE)
  #rxSortXdf(inFile=testFile2, outFile = inFile1, sortByVars="xint1", overwrite=TRUE)
  #
    #if (file.exists(outXDF)) file.remove(outXDF)
  #rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "xint1")
  #varInfo <- rxGetVarInfo( outXDF )
  ## Do a simple check to start
  #checkEquals(length(varInfoOut) > length(varInfoIn1))
  #
  #testData <- rxReadXdf(outXDF, varsToKeep = c("xnum1", "xnum1.1"))
  #checkEquals(testData$xnum1, testData$xnum1.1, checkNames=FALSE)
  #
  ## If you compare the contents, the results are not yet correct
  #
  #if (file.exists(inFile1)) file.remove(inFile1)
  #if (file.exists(inFile2)) file.remove(inFile2)
  #if (file.exists(outXDF)) file.remove(outXDF)
  #
#}

"test.rxMerge.args.inData.data.frame" <- function()
{
  x <- 1:20
  y <- 20:1
  df <- data.frame(x=x, y=y)
  
  # Merge two identical data frames sorted on x	
  mergedDF <- rxMerge(inData1 = df, inData2 = df, matchVars = "x")
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)	
  
  # Merge into an .xdf file
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  rxMerge(inData1 = df, inData2 = df, outFile = outXDF, matchVars = "x", overwrite=TRUE)
  mergedXdfDF <- rxReadXdf( outXDF )
  varInfo1 <- rxGetVarInfo( outXDF )
  checkEquals(varInfo, varInfo1, checkNames=FALSE)
  checkEquals(mergedDF, mergedXdfDF, checkNames=FALSE)
  
  file.remove(outXDF)
}
"test.rxMerge.args.inData.multiple" <- function()
{
  outDir <- tempdir()
    outFilesBase <- paste(outDir, "/mySplitMergeFiles", sep = "")
  on.exit(unlink(outFilesBase, recursive = TRUE), add = TRUE)
  
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf") 
  splitOut <- rxSplit(inData = testXdf, splitByFactor = "xfac1", 
    outFilesBase = outFilesBase, overwrite = TRUE) 
  # splitOut is a list of data sources 
  print(splitOut)
  tempXdf <- tempfile(pattern = "mergeSplitXdf", fileext = ".xdf")
  # Now merge them back to an xdf file
  newTestXDF <- rxMerge(inData1 = splitOut, outFile = tempXdf, type= "union", overwrite = TRUE)
  
  oldInfo <- rxGetInfo(testXdf)
  newInfo <- rxGetInfo(newTestXDF)
  checkEquals(oldInfo$numRows, newInfo$numRows)
    checkEquals(oldInfo$numVars, oldInfo$numVars)
  oldVarInfo <- rxGetVarInfo(testXdf)
  newVarInfo <- rxGetVarInfo(newTestXDF)
  checkEquals(oldVarInfo$y$low, newVarInfo$y$low)
  checkEquals(oldVarInfo$y$high, newVarInfo$y$high)
  file.remove(tempXdf)	
  
  # Do the same, but return a data frame
  newTestDF <- rxMerge(inData1 = splitOut, type= "union")
  newInfo <- rxGetInfo(newTestDF)
  checkEquals(oldInfo$numRows, newInfo$numRows)
    checkEquals(oldInfo$numVars, oldInfo$numVars)
  newVarInfo <- rxGetVarInfo(newTestDF)
  checkEquals(oldVarInfo$y$low, newVarInfo$y$low)
  checkEquals(oldVarInfo$y$high, newVarInfo$y$high)	
}

"test.rxMergeXdf.data.factors" <- function()
{
  testFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  sortedFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  outFile <- file.path(tempdir(), ".rxTempOut.xdf")
  if (file.exists(outFile)) file.remove(outFile)
  
  rxSortXdf(inFile = testFile, outFile = sortedFile1, sortByVars = "ylogical", overwrite = TRUE)
  sortedFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  rxSortXdf(inFile = testFile, outFile = sortedFile2, sortByVars = "ylogical", overwrite = TRUE)
  
  rxMergeXdf(inFile1 = sortedFile1, inFile2 = sortedFile2, outFile = outFile, matchVars = "ylogical")
  
  infoIn <- rxGetVarInfo(sortedFile1)
  infoOut <- rxGetVarInfo(outFile)
  for (i in 1:9 )
  {
    print(checkEquals(infoIn[[i]], infoOut[[i]], checkNames = FALSE))
  }
      
  file.remove( sortedFile1 )
  file.remove( sortedFile2 )
  file.remove( outFile )
}

"test.rxMergeXdf.data.string" <- function()
{
  #Test for fix to RA-3677 
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  empID <- 1:6
  deptID <- as.integer(c(31,33,33,34,34,NA))
  employee <- data.frame(empID, deptID)
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  # Note data is already sorted on deptID
  rxMergeXdf(inFile1 = employeeXDF, inFile2 = deptXDF, outFile=outXDF, matchVars = "deptID", overwrite=TRUE)
  # In the input xdf file, deptName is character
  infoIn <- rxGetVarInfo(deptXDF)
  
  # In the output xdf file, deptName should be the same
  infoOut <- rxGetVarInfo(outXDF)
  checkEquals(infoIn$deptName, infoOut$deptName)
  
  file.remove(employeeXDF)
  file.remove(deptXDF)
  file.remove(outXDF)
}
  
"test.rxMergeXdf.args.matchVars.logical" <- function()
{
  df <- data.frame(ylogical = c(rep(TRUE,2),rep(FALSE,2)), x = 1:4, y = letters[1:4])
  
  file1 <- file.path(tempdir(), ".rxFile1.xdf")
  outFile <- file.path(tempdir(), ".rxOut.xdf")
  if (file.exists(file1)) file.remove( file1 )
  if (file.exists(outFile)) file.remove( outFile )
  
  rxDataStep(inData =df,outFile=file1, overwrite=TRUE)
  rxMergeXdf(inFile1=file1, inFile2=file1, outFile=outFile ,matchVars="ylogical", decreasing=TRUE, overwrite=TRUE)
  infoIn <- rxGetVarInfo( file1 )
  infoOut <- rxGetVarInfo(outFile)
  checkEquals( infoIn$y$levels, infoOut$y$levels)
  
  newData <- rxReadXdf( outFile )
  checkEquals(nrow(newData), 8)
    checkEquals(c("logical", "integer", "factor","integer","factor"), as.character(lapply(newData,class)))
  expect <- merge(df, df, by="ylogical", sort=FALSE)
  checkEquals(expect, newData, checkNames=FALSE)

  file.remove( file1 )
  file.remove( outFile )
}
"test.rxMergeXdf.args.matchVar.logical2" <- function()
{
  testFile1 <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
  testFile2 <- testFile1	
  inFile1 <- file.path(tempdir(), ".rxTempIn1.xdf")
  inFile2 <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  rxSortXdf(inFile=testFile1, outFile = inFile1, sortByVars="ylogical", overwrite=TRUE)
  rxSortXdf(inFile=testFile2, outFile = inFile2, sortByVars="ylogical", overwrite=TRUE)
  
  InfoIn1 <- rxGetInfo(inFile1, numRows=100)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxMergeXdf(inFile1 = inFile1, inFile2 = inFile2, outFile=outXDF, matchVars = "ylogical")
  testData <- rxReadXdf(outXDF)

  # create expected output data frame
  testdf <- rxReadXdf(inFile1)
  
  # Pending fix for factors to integer problem
  #df <- data.frame(stringsAsFactors=FALSE)
  #for (i in 1:35) {
    #for (j in 1:35)
    #{
      #df <- rbind(df,testdf[i,])
    #}
  #}
  #for (i in 1:65) {
    #for (j in 1:65)
    #{
      #df <- rbind(df,testdf[i,])
    #}
  #}
  #df <- cbind(df,df)
  #testData2 <- cbind(testData[,1:14],testData[3],testData[,15:23])
  #checkEquals(dim(df), dim(testData2))
  #dfColClasses <- lapply(df,class)
  #testColClasses <- lapply(testData2, class)
  #
  #checkEquals(lapply(df,class),lapply(testData2,class), checkNames=FALSE)
  #all.equal(testData2[1:100,1:12], df[1:100,1:12], checkNames=FALSE,check.attributes=FALSE)

  file.remove(inFile1)
  file.remove(inFile2)
  file.remove(outXDF)	
}
"test.rxMergeXdf.args.matchVars.factor" <- function()
{
  
  # Factors must have same levels in same order
  # Data must be sorted on merge variable
  y1 <- factor(letters[1:4], levels=letters[1:7])
  y2 <- factor(letters[2:7], levels=letters[1:7])
  df1 <- data.frame(ylogical = c(rep(TRUE,2),rep(FALSE,2)), x = 1:4, y = y1 )
  df2 <- data.frame(z=c(40,44,33,22, 15,13), y = y2)
  
  file1 <- file.path(tempdir(), ".rxFile1.xdf")
  file2 <- file.path(tempdir(), ".rxFile2.xdf")
  outFile <- file.path(tempdir(), ".rxOutFile.xdf")

  rxDataStep(inData =df1, outFile=file1, overwrite=TRUE)
  rxDataStep(inData =df2, outFile=file2, overwrite=TRUE)
  rxMergeXdf(inFile1=file1, inFile2=file2, outFile=outFile, matchVars="y", overwrite=TRUE)
  mergeDF <- rxReadXdf( outFile )

  expect <- merge(df1,df2, by="y", sort=FALSE)
  
  checkEquals(expect$y, mergeDF$y)
  checkEquals(expect$ylogical, mergeDF$ylogical)
  checkEquals(expect$x, mergeDF$x)
  checkEquals(expect$z, mergeDF$z)
  
  file.remove( file1 )
  file.remove( file2 )
  file.remove( outFile )

}

"test.rxMerge.args.duplicateVarExt" <- function()
{
  x <- 1:20
  y <- 20:1
  df <- data.frame(x=x, y=y)
  
  # Merge two identical data frames sorted on x with the same name	

  mergedDF <- rxMerge(inData1 = df, inData2 = df, matchVars = "x", 
    overwrite=TRUE)
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.1", "y.2"))	
  rm(mergedDF)
  
  # Merge two identical data frames sorted on x with the same name
  # specifying an extension for the second	

  mergedDF <- rxMerge(inData1 = df, inData2 = df, matchVars = "x", 
    overwrite=TRUE, duplicateVarExt = c("", "copy"))
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y", "y.copy"))	
  rm(mergedDF)
  
  # Merge two identical data frames sorted on x with the same name
  # specifying an extension for the first	

  mergedDF <- rxMerge(inData1 = df, inData2 = df, matchVars = "x", 
    overwrite=TRUE, duplicateVarExt = c("copy", ""))
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.copy", "y"))	
  rm(mergedDF)
  
  # Merge data frames with different names
  df1 <- df
  mergedDF <- rxMerge(inData1 = df, inData2 = df1, matchVars = "x", 
  overwrite=TRUE)
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.df", "y.df1"))	
  rm(mergedDF)
  
  # Merge data frames with different names; try to omit extension
  mergedDF <- rxMerge(inData1 = df, inData2 = df1, matchVars = "x", 
    duplicateVarExt = c("", ""))
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.1", "y.2"))	
  rm(mergedDF)
  
  # Specify duplicate var extension
  mergedDF <- rxMerge(inData1 = df, inData2 = df, matchVars = "x", 
  duplicateVarExt = c("file1", "file2"), overwrite=TRUE)
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.file1", "y.file2"))
  
  # Merge into an .xdf file
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  rxMerge(inData1 = df, inData2 = df, outFile = outXDF, matchVars = "x", overwrite=TRUE)
  mergedXdfDF <- rxReadXdf( outXDF )
  
  varInfo1 <- rxGetVarInfo( outXDF )
  checkEquals(varInfo, varInfo1, checkNames=FALSE)
  checkEquals(mergedDF, mergedXdfDF, checkNames=FALSE)
  checkEquals( names(varInfo1), c("x", "y.1", "y.2"))
  
  # Merge 1 xdf file and 1 data frame with same name
  inXDF1 <- file.path(tempdir(), "df.xdf")
  rxDataStep( inData = df, outFile = inXDF1)
  mergedDF <- rxMerge(inData1 = df, inData2 = inXDF1, matchVars = "x")
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.1", "y.2"))	
  rm(mergeDF)
  
  # Reverse the order
  mergedDF <- rxMerge(inData1 = inXDF1, inData2 = df, matchVars = "x", 
    overwrite=TRUE)
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.1", "y.2"))	
  rm(mergeDF)
  
  # Merge 1 xdf file and 1 data frame with different names
  file.remove(inXDF1)
  inXDF1 <- file.path(tempdir(), "xdfnew.xdf")
  if (file.exists(inXDF1)) file.remove(inXDF1)
  rxDataStep( data = df, outFile = inXDF1)
  mergedDF <- rxMerge(inData1 = df, inData2 = inXDF1, matchVars = "x")
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.df", "y.xdfnew"))	
  rm(mergedDF)
  
  # Reverse the order
  mergedDF <- rxMerge(inData1 = inXDF1, inData2 = df, matchVars = "x")
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.xdfnew", "y.df"))	
  rm(mergedDF)
  
  # Merge 2 xdf files with the different names

  inXDF2 <- file.path(tempdir(), "xdfnew1.xdf")
  if (file.exists(inXDF2)) file.remove(inXDF2)
  rxDataStep( data = df, outFile = inXDF2)
  mergedDF <- rxMerge(inData1 = inXDF1, inData2 = inXDF2, matchVars = "x")
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.xdfnew", "y.xdfnew1"))	
  rm(mergedDF)
  
  # Merge 2 xdf files, trying to use no extension
  mergedDF <- rxMerge(inData1 = inXDF1, inData2 = inXDF2, matchVars = "x",
    duplicateVarExt = c("", ""))
  varInfo <- rxGetVarInfo( mergedDF)
  checkEquals(length(varInfo), 3)
  checkEquals(mergedDF[[2]], y)
  checkEquals(mergedDF[[3]], y)
  checkEquals(names(mergedDF), c("x", "y.1", "y.2"))	
  rm(mergedDF)
  
  file.remove(inXDF1)
  file.remove(inXDF2)	
  file.remove(outXDF)
  
}	

"test.rxMergeXdf.args.type.left" <- function()
{
  #
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  lastName <- c("Rafferty", "Jones", "Steinberg", "Robinson", "Smith", "John")
  deptID <- as.integer(c(31,33,33,34,34,NA))
  employee <- data.frame(lastName, deptID)
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  rxMergeXdf(inFile1 = deptXDF, inFile2 = employeeXDF, type = "left", outFile=outXDF, matchVars = "deptID", overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  

  # Column is added
  checkEquals( ncol(newData), 3)
  checkEquals( subset(newData, deptID == 31, select=deptName)[1,1], "Sales")
  checkEquals( subset(newData, deptID == 33, select=deptName)[1,1], "Engineering")
  checkEquals( subset(newData, deptID == 34, select=deptName)[1,1], "Clerical")

  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove( outXDF )

}

"test.rxMergeXdf.args.type.right" <- function()
{
  #
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  lastName <- c("TBD", "Rafferty", "Jones", "Steinberg", "Robinson", "Smith", "John")
  deptID <- as.integer(c(NA, 31,33,33,34,34,NA))
  employee <- data.frame(lastName, deptID)
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("Research", "Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(NA,31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  rxMergeXdf(inFile1 = deptXDF, inFile2 = employeeXDF, type = "right", outFile=outXDF, matchVars = "deptID", overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )

  # Right merge 
  checkEquals( nrow(newData), nrow(employee))	

  # Column is added
  checkEquals( ncol(newData), 3)
  checkEquals( subset(newData, deptID == 31, select=deptName)[1,1], "Sales")
  checkEquals( subset(newData, deptID == 33, select=deptName)[1,1], "Engineering")
  checkEquals( subset(newData, deptID == 34, select=deptName)[1,1], "Clerical")

  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove( outXDF )
}

"test.rxMergeXdf.args.type.full" <- function()
{
  # Full merge
  # NA does not match with NA
  
  employeeXDF <- file.path(tempdir(), ".rxTempIn1.xdf")
  deptXDF <- file.path(tempdir(), ".rxTempIn2.xdf")
  outXDF <- file.path(tempdir(), ".rxTempOut.xdf")
  
  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create employee data set
  lastName <- c("TBD", "Rafferty", "Jones", "Steinberg", "Robinson", "Smith", "John")
  deptID <- as.integer(c(NA, 31,33,33,34,34,NA))
  employee <- data.frame(lastName, deptID)
  rxDataStep(inData =employee, outFile=employeeXDF, overwrite=TRUE)
  
  # Create department data set
  deptName <- c("NewDept", "Research", "Sales", "Engineering", "Clerical", "Marketing")
  deptID <- as.integer(c(NA, NA,31,33:35))
  dept <- data.frame(deptName, deptID, stringsAsFactors=FALSE)
  rxDataStep(inData =dept, outFile=deptXDF, overwrite=TRUE)
  
  rxMergeXdf(inFile1 = deptXDF, inFile2 = employeeXDF, type = "full", outFile=outXDF, matchVars = "deptID", overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )

  # Full merge always has more rows than either infile.
  checkEquals( nrow(newData), 10)	

  # Column is added
  checkEquals( ncol(newData), 3)
  checkEquals( subset(newData, deptID == 31, select=deptName)[1,1], "Sales")
  checkEquals( subset(newData, deptID == 33, select=deptName)[1,1], "Engineering")
  checkEquals( subset(newData, deptID == 34, select=deptName)[1,1], "Clerical")
  checkEquals( newData$deptID, c(NA, NA, NA, 31, 33, 33, 34, 34, NA, 35))
  checkEquals( newData$lastName, as.factor(c("TBD",NA,NA,"Rafferty","Jones","Steinberg","Robinson","Smith","John",NA) ))
  checkEquals( newData$deptName, c( NA,"NewDept","Research" ,"Sales", "Engineering","Engineering", "Clerical", "Clerical", NA, "Marketing"  ))

  if (file.exists(employeeXDF)) file.remove(employeeXDF)
  if (file.exists(deptXDF)) file.remove(deptXDF)
  if (file.exists(outXDF)) file.remove( outXDF )
}

"test.rxMergeXdf.PromoteKey" <- function()
{
  # Full merge
  # Promote key type for matching
  
  # Prepare data files	
  inFileOne <- file.path(tempdir(), ".Int2Double1_48_3_1.xdf")
  inFileTwo <- file.path(tempdir(), ".Int2Double1_40_3_1.xdf")
  outOldXDF <- file.path(tempdir(), ".Merged48_3_1x40_3_1Old.xdf")
  outXDF <- file.path(tempdir(), ".Merged48_3_1x40_3_1New.xdf")
  
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(outOldXDF)) file.remove(outOldXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create in file data set
  v0 <- as.integer(c( 34,7,32,15,46,13,36,9,27,26,12,5,30,2,43,19,11,22,37,23,45,41,25,38,20,4,42,17,8,6,28,14,33,3,31,48,16,39,18,21,10,29,44,35,47,1,40,24 ))
  v1 <- as.integer(c( 1,0,0,5,4,3,4,5,3,5,3,0,4,5,5,2,3,4,3,5,2,2,2,3,3,1,1,0,0,2,4,1,5,3,3,1,0,3,4,1,1,3,4,4,2,1,2,1 ))
  v2 <- as.double(c( -2.34026744325950,-2.20500150497635,-1.81266937831136,-1.07675022346768,-1.00939900960202,-0.838159196997566,-0.726302090043302,-0.714634258090251,-0.710548675442457,-0.691527130306433,-0.683624173445497,-0.642994699823499,-0.453960016396264,-0.357307056618992,-0.338729195713885,-0.306638581855948,-0.291412681282623,-0.206738134550251,-0.107960959120525,0.0176818338490002,0.0762579388241012,0.14624764279819,0.153615652239011,0.158186109133152,0.17806353088908,0.333215876685053,0.363403894475203,0.531107902918381,0.584035471872512,0.586241095292548,0.592275589686163,0.73061810725832,0.754109920076463,0.777569908867605,0.795441041029793,0.80794007064579,0.862250935984705,1.03868488046633,1.14547102888304,1.18466424685926,1.28914342515682,1.37494785375538,1.38379923883905,1.38730697523133,1.97471850740613,2.18821439028082,2.22012489436868,2.47157884321200 ))
  inDataFrame1 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame1, outFile=inFileOne, overwrite=TRUE)
    
  v0 <- as.integer(c( 10,11,12,20,23,26,28,34,39,40,8,13,17,19,30,33,14,15,16,21,31,32,1,2,6,18,22,24,25,27,29,35,3,5,7,37,4,9,36,38 ))
  v1 <- as.integer(c( 0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5 ))
  v2 <- as.double(c( -0.566262628041856,-0.41105086909616,0.424430976591925,-0.601437254593254,-1.46767672074149,1.09294767254428,-1.55064167859839,-0.584087884093103,-0.00105428857235583,-0.63209776783239,0.67502024723581,-1.08539983620172,1.28831366128514,-0.184078385209438,0.180812921059514,-0.659899445423548,-0.802741160254401,0.0876556207195027,1.25945056168564,-0.309232604798752,-0.346808084162287,0.107729365407721,-1.75505909165829,-0.636810419521642,0.785895414101782,0.859100094167071,0.216135963077191,0.0852932038148529,-1.90987049431761,-0.0572398602860544,-1.63037005977505,-0.874986010753547,-0.61775922792921,0.280616779377390,-0.0546753905855378,0.21052696576626,1.03117240340477,0.253247680644996,-0.208387670219793,-0.0068394934841558 ))
  inDataFrame2 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame2, outFile=inFileTwo, overwrite=TRUE)
  
  v0 <- as.integer(c( 34,7,32,15,46,13,36,9,27,26,12,5,30,2,43,19,11,22,37,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,23,45,41,25,38,20,4,42,17,8,6,28,14,33,3,31,48,16,NA,NA,NA,NA,NA,NA,39,18,21,10,29,44,35,47,NA,NA,NA,NA,NA,NA,1,40,24,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA ))
  v1 <- as.integer(c( 1,0,0,5,4,3,4,5,3,5,3,0,4,5,5,2,3,4,3,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,5,2,2,2,3,3,1,1,0,0,2,4,1,5,3,3,1,0,NA,NA,NA,NA,NA,NA,3,4,1,1,3,4,4,2,NA,NA,NA,NA,NA,NA,1,2,1,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA ))
  Key <- as.double(c( -2.34026744325950,-2.20500150497635,-1.81266937831136,-1.07675022346768,-1.00939900960202,-0.838159196997566,-0.726302090043302,-0.714634258090251,-0.710548675442457,-0.691527130306433,-0.683624173445497,-0.642994699823499,-0.453960016396264,-0.357307056618992,-0.338729195713885,-0.306638581855948,-0.291412681282623,-0.206738134550251,-0.107960959120525,0,0,0,0,0,0,0,0,0,0,0.0176818338490002,0.0762579388241012,0.14624764279819,0.153615652239011,0.158186109133152,0.17806353088908,0.333215876685053,0.363403894475203,0.531107902918381,0.584035471872512,0.586241095292548,0.592275589686163,0.73061810725832,0.754109920076463,0.777569908867605,0.795441041029793,0.80794007064579,0.862250935984705,1,1,1,1,1,1,1.03868488046633,1.14547102888304,1.18466424685926,1.28914342515682,1.37494785375538,1.38379923883905,1.38730697523133,1.97471850740613,2,2,2,2,2,2,2.18821439028082,2.22012489436868,2.47157884321200,3,3,3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5 ))
  col0 <- as.integer(c( NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,10,11,12,20,23,26,28,34,39,40,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,8,13,17,19,30,33,NA,NA,NA,NA,NA,NA,NA,NA,14,15,16,21,31,32,NA,NA,NA,1,2,6,18,22,24,25,27,29,35,3,5,7,37,4,9,36,38 ))
  col2 <- as.double(c( NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,-0.566262628041856,-0.41105086909616,0.424430976591925,-0.601437254593254,-1.46767672074149,1.09294767254428,-1.55064167859839,-0.584087884093103,-0.00105428857235583,-0.63209776783239,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,0.67502024723581,-1.08539983620172,1.28831366128514,-0.184078385209438,0.180812921059514,-0.659899445423548,NA,NA,NA,NA,NA,NA,NA,NA,-0.802741160254401,0.0876556207195027,1.25945056168564,-0.309232604798752,-0.346808084162287,0.107729365407721,NA,NA,NA,-1.75505909165829,-0.636810419521642,0.785895414101782,0.859100094167071,0.216135963077191,0.0852932038148529,-1.90987049431761,-0.0572398602860544,-1.63037005977505,-0.874986010753547,-0.61775922792921,0.280616779377390,-0.0546753905855378,0.21052696576626,1.03117240340477,0.253247680644996,-0.208387670219793,-0.0068394934841558 ))
  outOldFrame <- data.frame(v0, v1, Key, col0, col2)
  rxDataStep(inData =outOldFrame, outFile=outOldXDF, overwrite=TRUE)
      
  if (file.exists(outXDF)) file.remove(outXDF)
    
  rxMergeXdf(inFile1 = inFileOne, inFile2 = inFileTwo, type = "full", # autoSort = FALSE, 
    outFile=outXDF, matchVars = "Key", 
    newVarNames1 = c(v0="v0", v1="v1", v2 = "Key"), newVarNames2 = c(v0 = "col0", v1 = "Key", v2="col2"), overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  oldData <- rxReadXdf( file=outOldXDF )		
  checkEquals( newData, oldData)
  if (file.exists(outXDF)) file.remove( outXDF )
    if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(outOldXDF)) file.remove(outOldXDF)

}


"test.rxMergeXdf.Union" <- function()
{
  # Union merge
  
  # Prepare data files	
  inFileOne <- file.path(tempdir(), "Data32_4_1.xdf")
  inFileTwo <- file.path(tempdir(), "Data16_4_2.xdf")
  oldXDF <- file.path(tempdir(), "Data32_4_1x16_4_2Old.xdf")
  outXDF <- file.path(tempdir(), "Data32_4_1x16_4_2New.xdf")
  
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create in file data set
  v0 <- as.integer(c( 2,18,32,14,5,16,9,29,17,13,19,25,6,8,28,23,10,12,26,1,20,15,27,3,24,30,21,7,11,31,4,22 ))
  v1 <- as.integer(c( 0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,3,3,3,4,4,4,4,5,5,5,5,5,5,5 ))
  v2 <- as.integer(c( 0,0,1,2,3,3,4,4,5,1,1,2,3,4,4,0,3,5,1,4,4,1,2,3,3,2,3,4,4,4,5,5 ))	
  inDataFrame1 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame1, outFile=inFileOne, overwrite=TRUE)
    
  v0 <- as.integer(c( 11,27,23,15,9,24,28,13,19,20,5,26,1,10,32,31,6,30,8,17,25,2,4,18,21,16,3,7,14,12,22,29 ))
  v1 <- as.integer(c( 0,0,0,1,2,2,2,3,3,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5 ))
  v2 <- as.integer(c( 1,1,5,0,1,3,3,3,3,3,4,4,5,5,0,1,2,2,3,3,5,0,0,0,0,1,2,2,2,5,5,5 ))
  inDataFrame2 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame2, outFile=inFileTwo, overwrite=TRUE)
  
  v0 <- as.integer(c( 2,18,32,14,5,16,9,29,17,13,19,25,6,8,28,23,10,12,26,1,20,15,27,3,24,30,21,7,11,31,4,22,11,27,23,15,9,24,28,13,19,20,5,26,1,10,32,31,6,30,8,17,25,2,4,18,21,16,3,7,14,12,22,29 ))
  v1 <- as.integer(c( 0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,3,3,3,4,4,4,4,5,5,5,5,5,5,5,0,0,0,1,2,2,2,3,3,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5 ))
  v2 <- as.integer(c( 0,0,1,2,3,3,4,4,5,1,1,2,3,4,4,0,3,5,1,4,4,1,2,3,3,2,3,4,4,4,5,5,1,1,5,0,1,3,3,3,3,3,4,4,5,5,0,1,2,2,3,3,5,0,0,0,0,1,2,2,2,5,5,5 ))
  outOldFrame <- data.frame(v0, v1, v2)
  rxDataStep(inData =outOldFrame, outFile=oldXDF, overwrite=TRUE)
    
  if (file.exists(outXDF)) file.remove(outXDF)
    
  rxMergeXdf(inFile1 = inFileOne, inFile2 = inFileTwo, type = "union", outFile=outXDF, overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  #newData
  oldData <- rxReadXdf( file=oldXDF )		
  checkEquals( newData, oldData)
    
  if (file.exists(outXDF)) file.remove( outXDF )
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)
   
}

"test.rxMergeXdf.OneToOne" <- function()
{
  # OneToOne merge
  #  Not change the 2nd file col names (same as the 1st file), they will be rename with suffix ".Data16_4_2"
  
  
  # Prepare data files	
  inFileOne <- file.path(tempdir(), "Data32_4_1.xdf")
  inFileTwo <- file.path(tempdir(), "Data16_4_2.xdf")
  oldXDF <- file.path(tempdir(), "Data32_4_1x16_4_2Old.xdf")
  outXDF <- file.path(tempdir(), "Data32_4_1x16_4_2New.xdf")
  
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create in file data set
  v0 <- as.integer(c( 2,18,32,14,5,16,9,29,17,13,19,25,6,8,28,23,10,12,26,1,20,15,27,3,24,30,21,7,11,31,4,22 ))
  v1 <- as.integer(c( 0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,3,3,3,4,4,4,4,5,5,5,5,5,5,5 ))
  v2 <- as.integer(c( 0,0,1,2,3,3,4,4,5,1,1,2,3,4,4,0,3,5,1,4,4,1,2,3,3,2,3,4,4,4,5,5 ))	
  inDataFrame1 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame1, outFile=inFileOne, overwrite=TRUE)
    
  v0 <- as.integer(c( 11,27,23,15,9,24,28,13,19,20,5,26,1,10,32,31,6,30,8,17,25,2,4,18,21,16,3,7,14,12,22,29 ))
  v1 <- as.integer(c( 0,0,0,1,2,2,2,3,3,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5 ))
  v2 <- as.integer(c( 1,1,5,0,1,3,3,3,3,3,4,4,5,5,0,1,2,2,3,3,5,0,0,0,0,1,2,2,2,5,5,5 ))
  inDataFrame2 <- data.frame(v0, v1, v2)
  rxDataStep(inData=inDataFrame2, outFile=inFileTwo, overwrite=TRUE)
  
  v0.Data32_4_1 <- as.integer(c( 2,18,32,14,5,16,9,29,17,13,19,25,6,8,28,23,10,12,26,1,20,15,27,3,24,30,21,7,11,31,4,22 ))
  v1.Data32_4_1 <- as.integer(c( 0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,3,3,3,4,4,4,4,5,5,5,5,5,5,5 ))
  v2.Data32_4_1 <- as.integer(c( 0,0,1,2,3,3,4,4,5,1,1,2,3,4,4,0,3,5,1,4,4,1,2,3,3,2,3,4,4,4,5,5 ))	
  v0.Data16_4_2 <- as.integer(c( 11,27,23,15,9,24,28,13,19,20,5,26,1,10,32,31,6,30,8,17,25,2,4,18,21,16,3,7,14,12,22,29 ))
  v1.Data16_4_2 <- as.integer(c( 0,0,0,1,2,2,2,3,3,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5 ))
  v2.Data16_4_2 <- as.integer(c( 1,1,5,0,1,3,3,3,3,3,4,4,5,5,0,1,2,2,3,3,5,0,0,0,0,1,2,2,2,5,5,5 ))
  outOldFrame <- data.frame(v0.Data32_4_1, v1.Data32_4_1, v2.Data32_4_1, v0.Data16_4_2, v1.Data16_4_2, v2.Data16_4_2)
  rxDataStep(inData=outOldFrame, outFile=oldXDF, overwrite=TRUE)
  
  #unitTestDataDir <- rxGetOption("unitTestDataDir")
  #infile1 <- file.path(unitTestDataDir, "Data32_4_1.xdf")
  #infile2 <- file.path(unitTestDataDir, "Data16_4_2.xdf")
  #outXDF <- file.path(unitTestDataDir, "OneToOne32_4_1x16_4_2New.xdf")
  #oldXDF <- file.path(unitTestDataDir, "OneToOne32_4_1x16_4_2.xdf")
    
  if (file.exists(outXDF)) file.remove(outXDF)
    
  rxMergeXdf(inFile1 = inFileOne, inFile2 = inFileTwo, type = "oneToOne", outFile=outXDF, overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  #newData
  oldData <- rxReadXdf( file=oldXDF )		
  checkEquals( newData, oldData)
  if (file.exists(outXDF)) file.remove( outXDF )
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)
  
}

"test.rxMergeXdf.Right.Rename" <- function()
{
  # OneToOne merge
  #  Change the 2nd file col names (same as the 1st file), they will be rename with suffix ".Data16_4_2"
    
  # Prepare data files	
  inFileOne <- file.path(tempdir(), "Data4_4_2.xdf")
  inFileTwo <- file.path(tempdir(), "Data8_4_2.xdf")
  oldXDF <- file.path(tempdir(), "Right4_4_2x8_4_2Old.xdf")
  outXDF <- file.path(tempdir(), "Right4_4_2x8_4_2New.xdf")
  
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # Create in file data set
  v0 <- as.integer(c( 4,8,1,5,2,6,7,3 ))
  v1 <- as.integer(c( 1,3,4,1,2,2,5,3 ))
  v2 <- as.integer(c( 0,0,1,1,2,2,2,3 ))	
  v3 <- as.integer(c( 66,2,40,48,8,45,56,58 ))	
  inDataFrame1 <- data.frame(v0, v1, v2, v3)
  rxDataStep(inData =inDataFrame1, outFile=inFileOne, overwrite=TRUE)		

  v0 <- as.integer(c( 11,4,6,8,2,13,10,1,16,5,9,14,7,15,3,12 ))
  v1 <- as.integer(c( 1,1,1,1,2,2,2,2,2,3,3,3,4,4,5,5 ))
  v2 <- as.integer(c( 0,4,5,5,0,1,2,4,4,0,1,1,4,4,1,2 ))	
  v3 <- as.integer(c( 8,66,7,92,62,69,89,91,10,95,53,61,15,66,35,13 ))	
  inDataFrame2 <- data.frame(v0, v1, v2, v3)
  rxDataStep(inData =inDataFrame2, outFile=inFileTwo, overwrite=TRUE)
  
  v0.Data4_4_2 <- as.integer(c( 1,1,1,1,5,5,5,5,2,2,2,2,2,6,6,6,6,6,7,7,7,7,7,3,3,3,NA,NA,NA,NA ))
  v1 <- as.integer(c( 4,4,4,4,1,1,1,1,2,2,2,2,2,2,2,2,2,2,5,5,5,5,5,3,3,3,NA,NA,NA,NA ))
  KEY <- as.integer(c( 1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,4,4,5,5 ))	
  v3.Data4_4_2 <- as.integer(c( 40,40,40,40,48,48,48,48,8,8,8,8,8,45,45,45,45,45,56,56,56,56,56,58,58,58,NA,NA,NA,NA ))
  v0.Data8_4_2 <- as.integer(c( 11,4,6,8,11,4,6,8,2,13,10,1,16,2,13,10,1,16,2,13,10,1,16,5,9,14,7,15,3,12 ))
  v2 <- as.integer(c( 0,4,5,5,0,4,5,5,0,1,2,4,4,0,1,2,4,4,0,1,2,4,4,0,1,1,4,4,1,2 ))
  v3.Data8_4_2 <- as.integer(c( 8,66,7,92,8,66,7,92,62,69,89,91,10,62,69,89,91,10,62,69,89,91,10,95,53,61,15,66,35,13 ))
  outOldFrame <- data.frame(v0.Data4_4_2, v1, KEY, v3.Data4_4_2, v0.Data8_4_2, v2, v3.Data8_4_2)
  rxDataStep(inData =outOldFrame, outFile=oldXDF, overwrite=TRUE)
  
  
  #infile1 <- file.path(unitTestDataDir, "Data4_4_2.xdf")
  #inData1 <- rxReadXdf( file=infile1 )
  #write.table(t(inData1), "C:/TEMP/junky1.txt",sep=",")		
  
  #unitTestDataDir <- rxGetOption("unitTestDataDir")
  #infile1 <- file.path(unitTestDataDir, "Data4_4_2.xdf")
  #infile2 <- file.path(unitTestDataDir, "Data8_4_2.xdf")
  #outXDF <- file.path(unitTestDataDir, "Right4_4_2x8_4_2New.xdf")
  #oldXDF <- file.path(unitTestDataDir, "Right4_4_2x8_4_2.xdf")
    
  if (file.exists(outXDF)) file.remove(outXDF)
    
  rxMergeXdf(inFile1 = inFileOne, inFile2 = inFileTwo, type = "right", matchVars = "KEY", 
    newVarNames1 = c(v0="v0", v1="v1", v2 = "KEY"), newVarNames2 = c(v0 = "v0", v1 = "KEY", v2="v2"),
    outFile=outXDF, overwrite=TRUE)
  newData <- rxReadXdf( file=outXDF )
  #newData
  oldData <- rxReadXdf( file=oldXDF )		
  checkEquals( newData, oldData)
    
  if (file.exists(outXDF)) file.remove( outXDF )
    if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  if (file.exists(oldXDF)) file.remove(oldXDF)

}

test.rxMergeXdf.args.bufferLimit <- function()
{
  # Prepare data files	
  inFileOne <- file.path(tempdir(), "Int2Double1_2M_3_15.xdf")
  inFileTwo <- file.path(tempdir(), "Int2Double1_1M_3_30.xdf")
  
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)
  
  # Create in file data set
  v0 <- as.integer(rbinom(3000000, 100, .5))
  v1 <- as.integer(rbinom(3000000, 5, .5))
  v2 <- as.double(runif(3000000, -5, 5))	
  inDataFrame1 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame1, outFile=inFileOne, overwrite=TRUE)		

  v0 <- as.integer(rbinom(3000000, 100, .5))
  v1 <- as.integer(rbinom(3000000, 5, .5))
  v2 <- as.double(runif(3000000, -5, 5))	
  inDataFrame2 <- data.frame(v0, v1, v2)
  rxDataStep(inData =inDataFrame2, outFile=inFileTwo, overwrite=TRUE)
            
    rxGetInfo(inFileOne,numRows=15)
    rxGetInfo(inFileTwo,numRows=15)
   
    if(file.exists("rxOut.xdf")) file.remove("rxOut.xdf")
    # The buffer limit is smaller than a block of data
    # need to be able to fit in 3 blocks of data (smallest block size is used to read and write out): 
    #  For this case it would be:
    #    16MB(input) *3 (2 for read and one for output) = 48 MB
    # A buffer limit of 15 should give an error:
    checkException(rxMergeXdf(inFile1=inFileOne, newVarNames1=c(v0="v0", v1="Key", v2="v2"),
               inFile=inFileTwo, newVarNames2=c(v0="col10",v1="Key",v2="col2"),
               matchVars=c("Key"), type="inner",decreasing=TRUE, rowsPerOutputBlock =1e6, bufferLimit=2,
               outFile="rxOut.xdf"),"Error: Buffer memory size is too small to hold a block of 2000000 rows.")
        
  if (file.exists(inFileOne)) file.remove(inFileOne)
  if (file.exists(inFileTwo)) file.remove(inFileTwo)        
   
}

"test.rxMerge.value" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile2 <- tempfile(pattern = ".rxMerge2t", fileext = ".xdf")
    outFile3 <- tempfile(pattern = ".rxMerge3t", fileext = ".xdf")
    outDS3 <- RxXdfData( outFile3 )
    inData2 <- data.frame(xint1 = rep(5:14, times = 10), xintnew = rep(1:10, times=10))
    inData1 <- rxReadXdf( inFile )
    
    # Output data frame
    outData1 <- rxMerge( inData1 = inFile, inData2 = inData2, 
        matchVars = "xint1",  reportProgress = 0)   
    # Output xdf
    outData2 <- rxMerge( inData1 = inFile, inData2 = inData2,
        outFile = outFile2,  matchVars = "xint1", 
        overwrite = TRUE, reportProgress = 0)
    # Output xdf data source
    outData3 <- rxMerge( inData1 = inFile, inData2 = inData2,
        outFile = outDS3, matchVars = "xint1", 
        reportProgress = 0, overwrite = TRUE )
    
    outDataShort1 <- rxDataStep(outData1, numRows = 5)
    outDataShort2 <- rxDataStep(outData2, numRows = 5)
    outDataShort3 <- rxDataStep(outData3, numRows = 5)
    
    checkEquals(outDataShort1, outDataShort2)
    checkEquals(outDataShort1, outDataShort3)

    sumFormula <-  ~y + ybinint + xnum2miss + xint1 + xintnew
    outSum1 <- rxResultsDF(rxSummary(sumFormula, data = outData1))
    
    outSum2 <- rxResultsDF(rxSummary(sumFormula, data = outData2))
    
    outSum3 <- rxResultsDF(rxSummary(sumFormula, data = outData3))
    
    checkEquals(outSum1, outSum2)
    checkEquals(outSum1, outSum3)
    
    file.remove(outFile2)
    file.remove(outFile3)
}
