# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxNaiveBayes
#
############################################################################################

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

.setUp <- function()
{
    .deactivate.rxNaiveBayes.tests <<- FALSE
}

.tearDown <- function()
{
    
}

# package e1071
### S3 method for class 'formula'
#naiveBayes(formula, data, laplace = 0, ..., subset, na.action = na.pass)
### Default S3 method:
#naiveBayes(x, y, laplace = 0, ...)
#
### S3 method for class 'naiveBayes'
#predict(object, newdata,
  #type = c("class", "raw"), threshold = 0.001, eps = 0, ...)

#######################################################################
## classification

###################################
## rxNaiveBayes arguments

"test.rxNaiveBayes.args.default" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }
}

"test.rxNaiveBayes.args.formula.dot" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF   <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    
    claimsColInfo.txt = list(
        age     = list(type = "factor"), 
        car.age = list(type = "factor"),
        type    = list(type = "factor")
    )
  claimsTxt   <- RxTextData(file = file.path(rxGetOption("sampleDataDir"), "claims.txt"), colInfo = claimsColInfo.txt)    
    
    claimsColInfo.sas = list(
        RowNum  = list(type = "integer"),
        age     = list(type = "factor"), 
        car_age = list(type = "factor"),
        type    = list(type = "factor")
    )
    claimsSas   <- RxSasData(file = file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat"), colInfo = claimsColInfo.sas)

    claimsColInfo.odbc <- list(
        age     = list(type = "factor"),
        car_age = list(type = "factor"),
        type    = list(type = "factor")
    )
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite")))
    if (inherits(conString, "try-error")) DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  claimsOdbc <- RxOdbcData(sqlQuery = "SELECT * FROM claims", connectionString = conString, colInfo = claimsColInfo.odbc)
  claimsOdbcDF <- rxImport(claimsOdbc)
  res <- try(checkEquals(nrow(claimsOdbcDF), 128))
  if (inherits(res, "try-error"))
  {
    message("Incorrect number of rows returned. Verify that you have a compatible SQLite ODBC driver (!=.92).")
  }
    
    #rxGetVarInfo(claimsSas)
    #data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ . #multi-class response with categorical and numeric predictors
    
    fit.nbrx.xdf    <- rxNaiveBayes(formula = form, data = claimsXDF)
    fit.nbrx.df     <- rxNaiveBayes(formula = form, data = claimsDF)
    fit.nbrx.txt    <- rxNaiveBayes(formula = form, data = claimsTxt)
    fit.nbrx.sas    <- rxNaiveBayes(formula = form, data = claimsSas)
    fit.nbrx.odbc   <- rxNaiveBayes(formula = form, data = claimsOdbc)

    domatch <- c("apriori", "tables", "levels", "nobs.by.class", "nobs.valid", "nobs.missing", "new.transformed.vars")
    checkEquals(fit.nbrx.df[domatch], fit.nbrx.xdf[domatch])
    checkEquals(fit.nbrx.df[domatch], fit.nbrx.txt[domatch])
    checkEquals(fit.nbrx.df[domatch], fit.nbrx.sas[domatch], check.attributes = FALSE) # car.age vs. car_age
    checkEquals(fit.nbrx.df[domatch], fit.nbrx.odbc[domatch], check.attributes = FALSE) # car.age vs. car_age
}

"test.rxNaiveBayes.args.smoothingFactor" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model    
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data, 
        smoothingFactor = laplace)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data, 
            laplace = laplace)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }
}

"test.rxNaiveBayes.args.useSparseCube.removeZeroCounts" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF[!(claimsDF$type == "D" & claimsDF$age == "60+"), ]  
    #rxSummary(~ type:(age + cost + number), data)
        
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx.ff <- rxNaiveBayes(formula = form, data = data, useSparseCube = FALSE, removeZeroCounts = FALSE)
    fit.nbrx.ft <- rxNaiveBayes(formula = form, data = data, useSparseCube = FALSE, removeZeroCounts = TRUE)
    fit.nbrx.tt <- rxNaiveBayes(formula = form, data = data, useSparseCube = TRUE, removeZeroCounts = TRUE)
    fit.nbrx.tf <- rxNaiveBayes(formula = form, data = data, useSparseCube = TRUE, removeZeroCounts = FALSE)
    
    domatch <- c("apriori", "tables", "levels")
    checkEquals(fit.nbrx.ft[domatch], fit.nbrx.ff[domatch])
    checkEquals(fit.nbrx.tf[domatch], fit.nbrx.tt[domatch])
    checkEquals(fit.nbrx.ff[domatch], fit.nbrx.tt[domatch])
    
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        fit.nbrx <- fit.nbrx.ff
        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }
}

"test.rxNaiveBayes.args.pweights.fweights" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    summ.pw <- rxSummary(~ type:(type + age + cost), data, pweights = "number")
    summ.fw <- rxSummary(~ type:(type + age + cost), data, fweights = "number")
    #checkEquals(summ.pw$categorical, summ.fw$categorical)
    ##Component "": Component "StdDev": Mean relative difference: 0.01614665
    
    ## set up arguments
    form        <- type ~ age + cost    # + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx.pw <- rxNaiveBayes(formula = form, data = data, pweights = "number")
    fit.nbrx.fw <- rxNaiveBayes(formula = form, data = data, fweights = "number")
    
    domatch <- c("apriori", "tables", "levels")
    #checkEquals(fit.nbrx.pw[domatch], fit.nbrx.fw[domatch])
    ##Component "tables": Component "cost": Mean relative difference: 0.01614665
}

"test.rxNaiveBayes.args.rowSelection" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number > 3)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = data, 
        smoothingFactor = laplace,
        rowSelection = sub
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = data, 
            laplace = laplace,
            subset = eval(sub)
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.args.byTerm" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxSummary(~., claimsDF)
    data        <- claimsDF
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    #sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = data, 
        smoothingFactor = laplace,
        #rowSelection = sub,
        byTerm = FALSE
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = data, 
            laplace = laplace,
            #subset = eval(sub),
            na.action = na.omit
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.args.transforms" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost2 + number #multi-class response with categorical and numeric predictors
 
    transforms <- expression(list(cost2 = cost * 2))
    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsDF,
        transforms = transforms, #transformVars = c("cost")
        )
    
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = cbind(claimsDF, cost2 = claimsDF$cost * 2)
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsXDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsXDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsXDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2))
        pred.nbro.clss <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2), type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2), type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.args.transformFunc" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost + number2 #multi-class response with categorical and numeric predictors
 
    myTransform <- function(dataList) 
    { 
        dataList$number2 <- dataList$number * 2
        return(dataList) 
    }
    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsDF,
        transformFunc = myTransform, transformVars = c("number")
        )
    
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = cbind(claimsDF, number2 = claimsDF$number * 2)
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsXDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsXDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsXDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = cbind(claimsDF, number2 = claimsDF$number * 2))
        pred.nbro.clss <- predict(fit.nbro, newdata = cbind(claimsDF, number2 = claimsDF$number * 2), type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = cbind(claimsDF, number2 = claimsDF$number * 2), type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.args.transforms.transformFunc" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost2 + number2 #multi-class response with categorical and numeric predictors
 
    myTransform <- function(dataList) 
    { 
        dataList$number2 <- dataList$number * 2
        return(dataList) 
    }
    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsDF,
        transforms = list(cost2 = cost * 2), 
        transformFunc = myTransform, transformVars = c("number")
        )
    
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = cbind(claimsDF, cost2 = claimsDF$cost * 2, number2 = claimsDF$number * 2)
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsXDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsXDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsXDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2, number2 = claimsDF$number * 2))
        pred.nbro.clss <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2, number2 = claimsDF$number * 2), type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = cbind(claimsDF, cost2 = claimsDF$cost * 2, number2 = claimsDF$number * 2), type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

###################################
## rxNaiveBayes data sources

"test.rxNaiveBayes.data.xdf" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = claimsXDF, 
        smoothingFactor = laplace,
        rowSelection = sub,
        byTerm = FALSE
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = claimsDF, 
            laplace = laplace,
            subset = eval(sub),
            na.action = na.omit
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsXDF)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsXDF, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsXDF, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.data.sas" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    claimsSas   <- RxSasData(file = file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat"), stringsAsFactors = TRUE)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = claimsSas, 
        smoothingFactor = laplace,
        rowSelection = sub,
        byTerm = FALSE
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = claimsDF, 
            laplace = laplace,
            subset = eval(sub),
            na.action = na.omit
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsSas)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsSas, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsSas, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.data.odbc" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
        rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), useFastRead = TRUE),
        silent = TRUE)
    if (inherits(conString, "try-error")) DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  # Temporary conString
    #conString <- paste("Driver=SQLite3 ODBC Driver;Database=", file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), sep = "")
    claimsColInfo <- list( 
        "type" = list(type = "factor", levels = c("A", "B", "C", "D")),	
        "age" = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")))		
  claimsOdbc <- RxOdbcData(
      sqlQuery = "SELECT * FROM claims",
        useFastRead = TRUE,
        connectionString = conString,
        colInfo = claimsColInfo,
        rowsPerRead = 32)
  claimsOdbcDF <- rxImport(claimsOdbc)
  res <- try(checkEquals(nrow(claimsOdbcDF), 128))
  if (inherits(res, "try-error"))
  {
    message("Incorrect number of rows returned. Verify that you have a compatible SQLite ODBC driver (!=.92).")
  }

    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = claimsOdbc, 
        smoothingFactor = laplace,
        rowSelection = sub,
        byTerm = FALSE
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = claimsDF, 
            laplace = laplace,
            subset = eval(sub),
            na.action = na.omit
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    yvar <- "type"
    ylevels <- levels(claimsDF[[yvar]])
    
    ## NOTE: there are extra vars in the prediction output for odbc data source!
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsOdbc)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsOdbc, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsOdbc, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob[paste(ylevels, "prob", sep = "_")]), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxNaiveBayes.data.txt" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    claimsColInfo <- list( 
        "type" = list(type = "factor", levels = c("A", "B", "C", "D")),	
        "age" = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")))		
  claimsText <- RxTextData(
    file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
        useFastRead = TRUE,
        colInfo = claimsColInfo,
        rowsPerRead = 32)

    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(
        formula = form, data = claimsText, 
        smoothingFactor = laplace,
        rowSelection = sub,
        byTerm = FALSE
        #reportProgress = 1, verbose = 2
        )
    if (e1071.installed) {
        fit.nbro <- naiveBayes(
            formula = form, data = claimsDF, 
            laplace = laplace,
            subset = eval(sub),
            na.action = na.omit
            )

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction

    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = claimsText)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = claimsText, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = claimsText, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

###################################
## rxPredict.rxNaiveBayes arguments

"test.rxPredict.rxNaiveBayes.args.type" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class")
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob")
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = data)
        pred.nbro.clss <- predict(fit.nbro, newdata = data, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = data, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)    
    }
}

"test.rxPredict.rxNaiveBayes.args.prior" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    ylevels <- levels(claimsDF$type)
    nylevels <- length(ylevels)
    prior <- rep(1/nylevels, nylevels)
    
    checkException(rxPredict(fit.nbrx, data = data, prior = prior[-1]))
    checkException(rxPredict(fit.nbrx, data = data, prior = prior / 2))
    checkException(rxPredict(fit.nbrx, data = data, prior = prior + seq(from = -1, to = 1, length.out = nylevels)))
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data, prior = prior)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class", prior = prior)
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob", prior = prior)
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = data)
        pred.nbro.clss <- predict(fit.nbro, newdata = data, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = data, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)    
    }
}

"test.rxPredict.rxNaiveBayes.args.predVarNames" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    yvar <- "type"
    ylevels <- levels(claimsDF[[yvar]])
    prefix <- "pred"
    predVarNames.clss <- paste(prefix, yvar, sep = "_")
    predVarNames.prob <- paste(prefix, ylevels, sep = "_")
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data, predVarNames = predVarNames.clss)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class", predVarNames = predVarNames.clss)
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob", predVarNames = predVarNames.prob)
    
    checkTrue(all(predVarNames.clss %in% colnames(pred.nbrx.dflt)))
    checkTrue(all(predVarNames.clss %in% colnames(pred.nbrx.clss)))
    checkTrue(all(predVarNames.prob %in% colnames(pred.nbrx.prob)))
}

"test.rxPredict.rxNaiveBayes.args.writeModelVars" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    writeModelVars <- TRUE
    modelVars <- names(fit.nbrx$tables)
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data, writeModelVars = writeModelVars)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class", writeModelVars = writeModelVars)
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob", writeModelVars = writeModelVars)
    
    checkTrue(all(modelVars %in% colnames(pred.nbrx.dflt)))
    checkTrue(all(modelVars %in% colnames(pred.nbrx.clss)))
    checkTrue(all(modelVars %in% colnames(pred.nbrx.prob)))
}

"test.rxPredict.rxNaiveBayes.args.extraVarsToWrite" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost    # + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    extraVarsToWrite <- "number"
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data, extraVarsToWrite = extraVarsToWrite)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class", extraVarsToWrite = extraVarsToWrite)
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob", extraVarsToWrite = extraVarsToWrite)
    
    checkTrue(all(extraVarsToWrite %in% colnames(pred.nbrx.dflt)))
    checkTrue(all(extraVarsToWrite %in% colnames(pred.nbrx.clss)))
    checkTrue(all(extraVarsToWrite %in% colnames(pred.nbrx.prob)))
}

"test.rxPredict.rxNaiveBayes.args.checkFactorLevels" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    checkFactorLevels <- TRUE
    datapred <- data
    levels(datapred$age) <- paste(levels(datapred$age), "extra", sep = "")
    
    checkException(pred.nbrx.dflt <- rxPredict(fit.nbrx, data = datapred, checkFactorLevels = checkFactorLevels))
    checkException(pred.nbrx.clss <- rxPredict(fit.nbrx, data = datapred, type = "class", checkFactorLevels = checkFactorLevels))
    checkException(pred.nbrx.prob <- rxPredict(fit.nbrx, data = datapred, type = "prob", checkFactorLevels = checkFactorLevels))
}

"test.rxPredict.rxNaiveBayes.args.removeMissingsOnRead" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    #rxGetVarInfo(claimsDF)
    data        <- claimsDF    
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    
    fit.nbrx <- rxNaiveBayes(formula = form, data = data)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = data)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    removeMissingsOnRead <- TRUE
    num.complete.cases <- sum(complete.cases(claimsDF))
    
    pred.nbrx.dflt <- rxPredict(fit.nbrx, data = data, removeMissingsOnRead = removeMissingsOnRead)
    pred.nbrx.clss <- rxPredict(fit.nbrx, data = data, type = "class", removeMissingsOnRead = removeMissingsOnRead)
    pred.nbrx.prob <- rxPredict(fit.nbrx, data = data, type = "prob", removeMissingsOnRead = removeMissingsOnRead)
    
    checkEquals(nrow(pred.nbrx.dflt), num.complete.cases)
    checkEquals(nrow(pred.nbrx.clss), num.complete.cases)
    checkEquals(nrow(pred.nbrx.prob), num.complete.cases)
}

###################################
## rxPredict.rxNaiveBayes outData data sources

"test.rxPredict.rxNaiveBayes.outData.xdf" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
        
    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsXDF)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = claimsDF)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction
    
    overwrite <- TRUE
    outData <- tempfile( pattern = ".rxNaiveBayes", fileext = ".xdf")
    on.exit(if (file.exists(outData)) file.remove(outData), add = TRUE)
    
    rxPredict(fit.nbrx, data = claimsXDF, outData = outData, overwrite = overwrite)
    pred.nbrx.dflt <- rxReadXdf(outData)
    rxPredict(fit.nbrx, data = claimsXDF, type = "class", outData = outData, overwrite = overwrite)
    pred.nbrx.clss <- rxReadXdf(outData)
    rxPredict(fit.nbrx, data = claimsXDF, type = "prob", outData = outData, overwrite = overwrite)
    pred.nbrx.prob <- rxReadXdf(outData)
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], pred.nbro.dflt)
        checkEquals(pred.nbrx.clss[["type_Pred"]], pred.nbro.clss)
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE)
    }
}

"test.rxPredict.rxNaiveBayes.outData.txt" <- function()
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")
    
    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    suppressWarnings(e1071.installed <- require(e1071, quietly = TRUE))

    ###################################################################
    ## build model
    claimsXDF    <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF    <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)
    
    claimsColInfo <- list( 
        "type" = list(type = "factor", levels = c("A", "B", "C", "D")),	
        "age" = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")))		
  claimsText <- RxTextData(
    file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
        useFastRead = TRUE,
        colInfo = claimsColInfo,
        rowsPerRead = 32)

    ## set up arguments
    form        <- type ~ age + cost + number #multi-class response with categorical and numeric predictors
    laplace     <- 1    
    sub         <- expression(number < 100)
        
    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsText)
    if (e1071.installed) {
        fit.nbro <- naiveBayes(formula = form, data = claimsDF)

        checkEquals(fit.nbrx$apriori, fit.nbro$apriori / fit.nbrx$nobs.valid, check.attributes = FALSE)
        checkEquals(fit.nbrx$tables, fit.nbro$tables, check.attributes = FALSE)
        checkEquals(fit.nbrx$levels, fit.nbro$levels)
    }

    ###################################################################
    ## do prediction

    overwrite <- TRUE
    outData <- tempfile( pattern = ".rxNaiveBayes", fileext = ".txt")
    on.exit(if (file.exists(outData)) file.remove(outData), add = TRUE)
    
    rxPredict(fit.nbrx, data = claimsXDF, outData = outData, overwrite = overwrite)
    pred.nbrx.dflt <- rxImport(outData)
    rxPredict(fit.nbrx, data = claimsXDF, type = "class", outData = outData, overwrite = overwrite)
    pred.nbrx.clss <- rxImport(outData)
    rxPredict(fit.nbrx, data = claimsXDF, type = "prob", outData = outData, overwrite = overwrite)
    pred.nbrx.prob <- rxImport(outData)
    
    if (e1071.installed) {
        pred.nbro.dflt <- predict(fit.nbro, newdata = claimsDF)
        pred.nbro.clss <- predict(fit.nbro, newdata = claimsDF, type = "class")
        pred.nbro.prob <- predict(fit.nbro, newdata = claimsDF, type = "raw")
    
        checkEquals(pred.nbrx.dflt[["type_Pred"]], as.character(pred.nbro.dflt))
        checkEquals(pred.nbrx.clss[["type_Pred"]], as.character(pred.nbro.clss))
        checkEquals(as.matrix(pred.nbrx.prob), pred.nbro.prob, check.attributes = FALSE, tolerance = 1e-7)
    }
}

###################################
## rxPredict.rxNaiveBayes data data sources

"test.rxPredict.rxNaiveBayes.data.df.only1obs" <- function() 
{
    if (exists(".deactivate.rxNaiveBayes.tests") && .deactivate.rxNaiveBayes.tests) DEACTIVATED("rxNaiveBayes is still in progress")

    #library(RevoScaleR)
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

    ###################################################################
    ## build model
    claimsXDF <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF <- rxReadXdf(claimsXDF)
    #rxSummary(~., claimsDF)

    ## set up arguments
    form <- type ~ age + cost + number #multi-class response with categorical and numeric predictors

    fit.nbrx <- rxNaiveBayes(formula = form, data = claimsXDF)

    ###################################################################
    ## do prediction

    overwrite <- TRUE
    outData <- tempfile(pattern = ".rxNaiveBayes", fileext = ".xdf")
    on.exit(if (file.exists(outData)) file.remove(outData), add = TRUE)

    pred.nbrx.obsa <- rxPredict(fit.nbrx, data = claimsDF)
    pred.nbrx.obs1 <- rxPredict(fit.nbrx, data = claimsDF[1, ])
    checkIdentical(pred.nbrx.obsa[1, , drop = FALSE], pred.nbrx.obs1)
}
