# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxOptions and rxGetOption
#
############################################################################################

"test.rxOptions.args.initialize" <- function()
{
    # RA-3365
    numCoresToUse <- rxGetOption("numCoresToUse") 
    rxOptions(numCoresToUse = 2)
  
    # This should reset options to defaults
    oldOptions <- rxOptions(TRUE) 
    defaultNumCoresToUse <- rxGetOption("numCoresToUse")
    checkEquals(defaultNumCoresToUse, 4)
    
    # Reset to original value
    rxOptions(numCoresToUse = numCoresToUse)
}

"test.rxOptions.value" <- function()
{
    # RA-4333
    numCoresToUse <- rxGetOption("numCoresToUse") 
    oldOptions <- rxOptions( numCoresToUse = 3) 
    
    # oldOptions should have original value
    checkEquals(oldOptions$numCoresToUse, numCoresToUse)
    
    newNumCoresToUse <- rxGetOption("numCoresToUse") 
    checkEquals(newNumCoresToUse, 3)
    
    # Reset to original value
    rxOptions(numCoresToUse = numCoresToUse)
}

"test.rxOptions.args.rowDisplayMax" <- function()
{
    rowDisplayMax <- rxGetOption("rowDisplayMax")
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
     
    rxOptions( rowDisplayMax = 2)
    on.exit(rxOptions(rowDisplayMax = rowDisplayMax), add = TRUE)
    out1 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b, verbose = 1, reportProgress = 0))
    checkEquals(length(out1), 6)
    
    rxOptions( rowDisplayMax = 5)
    out2 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b, verbose = 1, reportProgress = 0))
    checkEquals(length(out2), 9)
  
    # Make sure option does not get altered by setting another option
  numCoresToUse <- rxGetOption("numCoresToUse")
  rxOptions( numCoresToUse = 3)
  on.exit(rxOptions(numCoresToUse = numCoresToUse), add = TRUE)
  out2 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b, verbose = 1, reportProgress = 0))
    checkEquals(length(out2), 9)
     
    # Reset to original value
    rxOptions(rowDisplayMax = rowDisplayMax)
    out3 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b, verbose = 1, reportProgress = 0))
    checkEquals(length(out3), 14)
  
  rxOptions(numCoresToUse = numCoresToUse)
  
}

"test.rxOptions.args.reportProgress" <- function()
{
    origReportProgress <- rxGetOption("reportProgress")
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b") 
    
    rxOptions( reportProgress = 2)
    on.exit(rxOptions(reportProgress = origReportProgress), add = TRUE)
    out1 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b))
    if (rxIsExpressEdition())
    {
        checkEquals(length(out1), 2)
    } else
    {  
        checkEquals(length(out1), 6)
    }
    
    rxOptions( reportProgress = 1)
    out2 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b))  
    checkEquals(length(out2), 1)
    
    rxOptions(reportProgress = 0) # by default, don't report progress
    out3 <- capture.output(myCube <- rxCube( ~xfac1:ylogical, data=test100r5b))  
    checkEquals(length(out3), 0)  

    rxOptions(reportProgress = origReportProgress)
}

"test.rxOptions.args.dataPath" <- function()
{
  origDataPath <- rxGetOption("dataPath")
    on.exit(rxOptions(dataPath = origDataPath), add= TRUE)
    
  origOutDataPath <- rxGetOption("outDataPath")
    on.exit(rxOptions(outDataPath = origOutDataPath), add= TRUE)    
    
    rxOptions(dataPath = rxGetOption("sampleDataDir"))
    rxOptions(outDataPath = '.')

  linMod1 <- rxLinMod(cost ~ number, data = "claims.xdf")	
  
  linMod2 <- rxLinMod(cost ~ number, data = RxTextData("claims.txt"))
  
  checkEquals(coef(linMod1), coef(linMod2))

  varInfo1 <- rxGetVarInfo("claims.xdf")
  varInfo2 <- rxGetVarInfo(RxTextData("claims.txt"))
  checkEquals(names(varInfo1), names(varInfo2))

  DS.xdf <- RxXdfData("claims.xdf")
  DS.txt <- RxTextData("claims.txt")

  linMod3 <- rxLinMod(cost ~ number, data = DS.xdf)
  linMod4 <- rxLinMod(cost ~ number, data = DS.txt)
  
  linMod1$call <- NULL
  linMod3$call <- NULL
  linMod1$params <- NULL
  linMod3$params <- NULL
  
  checkEquals(linMod1, linMod3)
  
  linMod2$call <- NULL
  linMod4$call <- NULL
  linMod2$params <- NULL
  linMod4$params <- NULL	

  checkEquals(linMod4, linMod2)

  varInfo3 <- rxGetVarInfo(DS.xdf)
  varInfo4 <- rxGetVarInfo(DS.txt)

  checkEquals(varInfo1, varInfo3)
  checkEquals(varInfo2, varInfo4)
  
    # Test writing to the working directory different from data path
    # Note that the outDataPath is set to '.'
    rxOptions(dataPath = rxGetOption("unitTestDataDir"))
    origWD <- getwd()
    on.exit(setwd(origWD), add = TRUE)
    
    setwd(tempdir())
    inFile <- "test100r5b.xdf"
    outFile <- ".rxPathTest.xdf"
    fullOutFile <- file.path(tempdir(), outFile)
    
    if (file.exists(fullOutFile)) file.remove(fullOutFile)
        
    rxDataStep(inData = inFile, outFile = outFile)
    checkTrue(file.exists(fullOutFile))
    file.remove(fullOutFile)
    setwd( origWD )
    rxOptions(dataPath = origDataPath)
    
    # Use multiple paths
    rxOptions(dataPath = c(rxGetOption("sampleDataDir"), rxGetOption("unitTestDataDir")))
    
    # Use data from the sample data dir
    logitObj <- rxLogit(Kyphosis ~ Age + Number + Start, data = "kyphosis")
    checkEquals(logitObj$nValidObs, 81)
    
    # Use data from the unit test data dir
    form <- ybinint ~ xnum2miss
    glmObj <- rxGlm(form, data="test100r5b", blocksPerRead=-1)    
    checkEquals(glmObj$nValidObs, 77)
    
    rxOptions(dataPath = origDataPath)
        
}

"test.rxOptions.args.outDataPath" <- function()
{
  origDataPath <- rxGetOption("dataPath")
    on.exit(rxOptions(dataPath = origDataPath), add= TRUE)    
    
    origOutDataPath <- rxGetOption("outDataPath")
    on.exit(rxOptions(outDataPath = origOutDataPath), add= TRUE)  
  
    rxOptions(dataPath = rxGetOption("unitTestDataDir"))
    rxOptions(outDataPath = tempdir())

    inFile <- "test100r5b.xdf"
    outFile <- ".rxPathTest.xdf"
    fullOutFile <- file.path(tempdir(), outFile)
    
    if (file.exists(fullOutFile)) file.remove(fullOutFile)
        
    rxDataStep(inData = inFile, outFile = outFile)
    checkTrue(file.exists(fullOutFile))
    file.remove(fullOutFile)
  
  # Test with rxDataFrameToXdf
    myData1 <- data.frame(x = 1:10, y = rep(c("a", "b", "c", "d", "e"), 2))

    rxDataStep(myData1, outFile = outFile)
    checkTrue(file.exists(fullOutFile))
    file.remove(fullOutFile)	
    
    # Test with rxXdfToText
    outTextFile <- ".rxPathTest.csv"
    fullOutTextFile <- file.path(tempdir(), outTextFile)
  rxXdfToText(inFile = inFile, outFile = outTextFile)
    checkTrue(file.exists(fullOutTextFile))
    file.remove(fullOutTextFile)
    
    # Summary out file
    censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
  sumOut <- rxSummary(~incwage:F(age) + F(age) + wkswork1:F(age), data = censusWorkers, blocksPerRead = 3,
    byGroupOutFile = outFile, rowsPerBlock = 10, summaryStats = c("Mean", "Min", "ValidObs"))
  checkTrue(file.exists(fullOutFile))
  fileInfo <- rxGetInfo(fullOutFile)
  checkEquals(fileInfo$numBlocks, 5 )
  checkEquals(fileInfo$numVars, 8 )  
    file.remove(fullOutFile)
      
    # rxSplit
    outDataSources <- rxSplit(inData = inFile, numOut = 2)   
    fullOutFile1 <- file.path(tempdir(), "test100r5b1.xdf")
    fullOutFile2 <- file.path(tempdir(), "test100r5b2.xdf")
    checkTrue(file.exists(fullOutFile1))
    checkTrue(file.exists(fullOutFile2))
    file.remove(fullOutFile1)	
    file.remove(fullOutFile2)
      
    # Test for sort
    rxSort(inData = inFile, outFile = outFile, sortByVars = "xint1")
    checkTrue(file.exists(fullOutFile))

    file.remove(fullOutFile) 
    
    workingDir <- getwd()
    on.exit(setwd(workingDir), add = TRUE)
    setwd(tempdir())
    
    rxOptions(dataPath = rxGetOption("unitTestDataDir"), outDataPath = '.')
    rxSort(inData = inFile, outFile = outFile, sortByVars = "xint1",
      type = "varByVar" ) 
    checkTrue(file.exists(fullOutFile))
    inVarInfo <- rxGetVarInfo(inFile)
    outVarInfo <- rxGetVarInfo(fullOutFile)
    checkEquals(sort(names(inVarInfo)), sort(names(outVarInfo)))    
    
    rxOptions(dataPath = '.', outDataPath = '.')
   
    rxSort(inData = outFile, outFile = fullOutFile, sortByVars = "xnum1",
       type = "varByVar", overwrite = TRUE)
     
    checkTrue(file.exists(fullOutFile))
    outVarInfo <- rxGetVarInfo(fullOutFile)
    checkEquals(sort(names(inVarInfo)), sort(names(outVarInfo)))      
    file.remove(fullOutFile)  
    
    # Two paths
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- ".rxPathTest2.xdf"
    outPath1 <- file.path(tempdir(), "dir1")
    outPath2 <- file.path(tempdir(), "dir2")
    if (!file.exists(outPath1))
  {
    dir.create(outPath1)
  }
    
    if (!file.exists(outPath2))
  {
    dir.create(outPath2)
  }
    rxOptions(outDataPath = c(outPath1, outPath2))
    fullOutFile <- file.path(outPath1, outFile)
    
    if (file.exists(fullOutFile)) file.remove(fullOutFile)
        
    rxDataStep(inData = inFile, outFile = outFile)
    checkTrue(file.exists(fullOutFile))
    file.remove(fullOutFile)
    unlink(outPath1, recursive = TRUE)
    unlink(outPath2, recursive = TRUE)
     
    rxOptions(dataPath = origDataPath, outDataPath = origOutDataPath)
}

"test.rxOptions.args.isExpress" <- function()
{
    if (Sys.info()["sysname"] != "Windows") DEACTIVATED("Deactivated for platforms other than Windows.")
    
    if (rxIsExpressEdition())
  {
    on.exit(rxOptions(isExpress = TRUE), add = TRUE)
    
    rxOptions(isExpress = FALSE)
    checkTrue(rxIsExpressEdition())
  }
  else
  {
    on.exit(rxOptions(isExpress = FALSE), add = TRUE)
    
    rxOptions(isExpress = TRUE)
    checkTrue(rxIsExpressEdition())
    
    rxOptions(isExpress = FALSE)
    checkTrue(!rxIsExpressEdition())
  }
}
