# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxRoc and related methods
#
############################################################################################

"test.rxRoc.args.actualVarName" <- function()
{
    # Check to make sure we get a proper error if variable name isn't specified in quotes
    # RA-12338
  rxLogitOut1 <- rxLogit(case ~ spontaneous+induced, data=infert )
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = infert, 
    writeModelVars = TRUE, predVarNames = "casePred1")
    case <- 7
  
    res1 <- try(rxRocCurve(actualVarName = case, predVarNames = c("casePred1"), 
    numBreaks = 20, data = rxPredOut))
    checkTrue(is(res1, "try-error"))
  checkTrue(grepl("must be a character scalar", attr(res1, "condition")$message))  
    
    casePred1 <- NULL
    res2 <- try(rxRocCurve(actualVarName = "case", predVarNames = casePred1, 
    numBreaks = 20, data = rxPredOut))
    checkTrue(is(res2, "try-error"))
  checkTrue(grepl("must be a character vector", attr(res2, "condition")$message))        
}


"test.rxRoc.data.infert" <- function()
{
  # Compare with results from epicalc
  # model1 <- glm(case ~ induced + spontaneous, data=infert, family=binomial)
  # lrocOut <- lroc(model1, title=TRUE, table = TRUE)
  # lrocOut$diagnostic.table # results in:
  
  OneMSpecificity <- c(1.000000000,  0.636363636, 0.436363636, 0.315151515, 
    0.163636364, 0.096969697, 0.072727273, 0.006060606, 0.000000000 )
  
  Sensitivity <- c(1.00000000, 0.91566265, 0.77108434,  0.66265060, 0.39759036,
    0.33734940, 0.28915663, 0.07228916, 0.00000000 )
  
  compareResults <- data.frame(Sensitivity, OneMSpecificity)

  rxLogitOut1 <- rxLogit(case ~ spontaneous+induced, data=infert )
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = infert, writeModelVars = TRUE, predVarNames = "casePred1")
  rxRocResults <- rxRoc(actualVarName = "case", predVarNames = c("casePred1"), numBreaks = 20, data = rxPredOut)

  testResults <- data.frame(Sensitivity = rxRocResults$sensitivity, 
    OneMSpecificity = 1 - rxRocResults$specificity)
  
  checkEquals(compareResults, testResults)
  rm(testResults)
    
    # Add some missings to confirm that results still are the same
    extraMissRows <- data.frame(casePred1 = c(.7, .4, NA, .3, NA),
        case = c(NA, NA, 1, NA, 1),
        spontaneous = c(2,1,2,1,0),
        induced = c(1,1,2,2,0) )
    rxPredOutNew <- rbind(rxPredOut, extraMissRows)
    rxRocResults <- rxRoc(actualVarName = "case", predVarNames = c("casePred1"), numBreaks = 20, data = rxPredOutNew)
    testResults <- data.frame(Sensitivity = rxRocResults$sensitivity, 
    OneMSpecificity = 1 - rxRocResults$specificity)   
  checkEquals(compareResults, testResults)
  rm(testResults)       
        
  
  # Redo with a multi-block .xdf file
  testFile <- tempfile(pattern = ".rxRoc1TempTest", fileext = ".xdf") 
  rxDataStep(inData = infert, outFile = testFile, rowsPerRead = 20)
  rxLogitOut1 <- rxLogit(case ~ spontaneous+induced, data=testFile )
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = testFile, outData = testFile,
     predVarNames = "casePred1")
  rxRocResults <- rxRoc(actualVarName = "case", predVarNames = c("casePred1"), 
    numBreaks = 20, data = testFile)

  testResults <- data.frame(Sensitivity = rxRocResults$sensitivity, 
    OneMSpecificity = 1 - rxRocResults$specificity)
  
  checkEquals(compareResults, testResults)
  file.remove(testFile)

}

"test.rxAuc.rxRoc.data.infert" <- function()
{
  testCalc <- function( oneMinusSpec, sens)
  {
    auc <- 0
    for (i in 2:(length(oneMinusSpec))) 
    {
      auc <- auc + (oneMinusSpec[i-1] - oneMinusSpec[i]) * 
      (sens[i] + sens[i - 1])/2
    }
    return( auc )
  }
  # Result from epicalc
  compareAuc <- 0.7285506
  rxLogitOut1 <- rxLogit(case ~ spontaneous+induced, data=infert )
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = infert, writeModelVars = TRUE, predVarNames = "casePred1")
  rxRocResults <- rxRoc(actualVarName = "case", predVarNames = c("casePred1"), numBreaks = 20, data = rxPredOut)
  
  testAuc <- rxAuc(rxRocResults)
  checkEquals(compareAuc, testAuc, tolerance = 10e-6)
  
  # Add another predictor with the same results
  rxPredOut1 <- rxPredict(modelObject = rxLogitOut1, data = infert,
    outData = rxPredOut, predVarNames = "casePred2")
  rxRocResults <- rxRoc(actualVarName = "case", predVarNames = c("casePred1", "casePred2"), 
    numBreaks = 20, data = rxPredOut1)
  
  testAuc <- rxAuc(rxRocResults)
  checkEquals(rep(compareAuc,2), testAuc, tolerance = 10e-6, checkNames = FALSE)
  
  # Add another predictor with different results
  # To get value from epicalc
  # model1 <- glm(case ~ induced + spontaneous + parity, data=infert, family=binomial)
  # lrocOut <- lroc(model1, title=TRUE, graph = FALSE)
  # lrocOut$auc # results in:  0.7702081
  oneMinusSpec <- c( 1.00000000 , 0.97575758, 0.95151515, 0.84848485, 0.83030303,  
    0.81818182, 0.58181818, 0.57575758, 0.56969697 , 0.56363636, 0.47272727,  
    0.45454545, 0.41212121, 0.38787879, 0.38181818,  0.29090909, 0.24242424,  
    0.23636364, 0.23636364, 0.18787879, 0.16969697,  0.15151515, 0.07878788, 
    0.06666667, 0.06060606, 0.04242424, 0.02424242,  0.01818182, 0.01818182,  
    0.00000000, 0.00000000 )
  sens <-  c( 1.00000000, 0.98795181, 0.97590361, 0.95180723, 0.95180723,
    0.95180723, 0.90361446, 0.86746988,  0.86746988, 0.85542169, 0.80722892,
    0.80722892, 0.79518072, 0.79518072,  0.79518072, 0.69879518,  0.68674699,
    0.67469880, 0.66265060, 0.61445783,  0.59036145,  0.59036145,  0.33734940,
    0.31325301, 0.30120482,  0.26506024, 0.21686747,  0.20481928,  0.19277108,
    0.03614458,  0.00000000)
  compareRocOut <- data.frame(sens,oneMinusSpec)

  compareAuc2 <- 0.7702081
  rxLogitOut2 <- rxLogit(case ~ spontaneous + induced + parity, data=infert )
  rxPredOut2 <- rxPredict(modelObject = rxLogitOut2, data = infert,
    outData = rxPredOut, predVarNames = "casePred2")
  
  # Second model alone
  rxRocResults1 <- rxRoc(actualVarName = "case", predVarNames = c("casePred2"), 
    numBreaks = 1000, data = rxPredOut2)
  
  checkEquals(rxRocResults1$sensitivity, compareRocOut$sens)
  testAuc2 <- rxAuc(rxRocResults1)
  checkEquals(compareAuc2, testAuc2, tolerance = 10e-2, checkNames = FALSE)	
    
  
  rxRocResults2 <- rxRoc(actualVarName = "case", predVarNames = c("casePred1", "casePred2"), 
    numBreaks = 1000, data = rxPredOut2)
  
  testAuc3 <- rxAuc(rxRocResults2)
  checkEquals(c(compareAuc,compareAuc2), testAuc3, tolerance = 10e-2, checkNames = FALSE)	
}

"test.rxRocCurve.data.infert" <- function()
{
  # Check to make sure we get a plot without error
  rxLogitOut1 <- rxLogit(case ~ spontaneous+induced, data=infert )
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = infert, 
    writeModelVars = TRUE, predVarNames = "casePred1")
  rxRocCurve(actualVarName = "case", predVarNames = c("casePred1"), 
    numBreaks = 20, data = rxPredOut)
  
  # Add a second model
  rxLogitOut2 <- rxLogit(case ~ spontaneous + induced + parity, data=infert )
  rxPredOut2 <- rxPredict(modelObject = rxLogitOut2, data = infert, 
    outData = rxPredOut, predVarNames = "casePred2")
  
  rxRocCurve(actualVarName = "case", predVarNames = c("casePred1", "casePred2"), 
    numBreaks = 20, data = rxPredOut2, legend = FALSE)
}


"test.rxRocCurve.data.mortDefaultSmall" <- function()
{
  mortXdf <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall")
  logitOut1 <- rxLogit(default ~ creditScore + yearsEmploy + ccDebt, 
    data = mortXdf,	blocksPerRead = 5)
  
  predFile <- tempfile(pattern = ".rxPred", fileext = ".xdf")
  
  predOutXdf <- rxPredict(modelObject = logitOut1, data = mortXdf, 
    writeModelVars = TRUE, predVarNames = "Model1", outData = predFile)
  
  # Estimate a second model without ccDebt
  logitOut2 <- rxLogit(default ~ creditScore + yearsEmploy, 
    data = predOutXdf, blocksPerRead = 5)
  
  # Add preditions to prediction data file
  predOutXdf <- rxPredict(modelObject = logitOut2, data = predOutXdf, 
    predVarNames = "Model2")
  
  curveOut <- rxRocCurve(actualVarName = "default", 
    predVarNames = c("Model1", "Model2"), 
    data = predOutXdf)
  
  latticeLoaded <- "lattice" %in% .packages()
  if (latticeLoaded)
  {
    legendText <- c("Model1 (AUC = 0.89)", "Model2 (AUC = 0.53)")
    checkEquals(curveOut$legend$top$args$text, legendText)
    }
    # Make sure we get an appropriate warning if no observations have TRUE
    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)	
  
    falseData <- rxDataStep(predOutXdf, rowSelection = default == FALSE)

    res <- try(curveOut1 <- rxRocCurve(actualVarName = "default", 
    predVarNames = c("Model1", "Model2"), 
    data = falseData)
  )
  checkTrue(inherits(res,"try-error"))
  checkTrue(grepl("no observations", as.character(res)))
  
  
  # Same with no observations with FALSE
  trueData <- rxDataStep(predOutXdf, rowSelection = default == TRUE)
  
    res <- try(
    curveOut2 <- rxRocCurve(actualVarName = "default", 
      predVarNames = c("Model1", "Model2"), 
      data = trueData)	
    )
  
    checkTrue(inherits(res,"try-error"))
  checkTrue(grepl("no observations", as.character(res)))
  file.remove(predFile)
}

"test.rxRocCurve.data.simple" <- function()
{

  sampleDF <- data.frame(actual = c(0,0,0,0,0, 1,1,1,1,1))
  sampleDF$pred1 <- c(.6, .5, .4, .3, .2, .8, .7, .6, .5, .4)
  
  rxRocOut <- rxRoc(actualVarName = "actual", predVarNames = "pred1", 
    data = sampleDF, numBreaks = 10, removeDups = FALSE)
  
  calculatedSensitivity <- c(1, 1, 1, 1, 1, .8, .6, .4, .2, 0, 0)
  calculatedSpecificity <- c(0, 0, 0, .2, .4, .6, .8, 1, 1, 1, 1)

  checkEquals(rxRocOut$sensitivity, calculatedSensitivity)
  checkEquals(rxRocOut$specificity, calculatedSpecificity)
  
  sampleDF$badPred <- c(.99, .99, .99, .99, .99, .01, .01, .01, .01, .01)
  rxRocOutBad <- rxRoc(actualVarName = "actual", predVarNames = "badPred", 
    data = sampleDF, numBreaks = 10)
  checkEquals(rxRocOutBad$sensitivity, c(1, 0, 0))
  checkEquals(rxRocOutBad$specificity, c(0, 0, 1))
        
  sampleDF$goodPred <- c( .01, .01, .01, .01, .01,.99, .99, .99, .99, .99)
  rxRocOutGood <- rxRoc(actualVarName = "actual", predVarNames = "goodPred", 
    data = sampleDF, numBreaks = 10)
  checkEquals(rxRocOutGood$sensitivity, c(1, 1, 0))
  checkEquals(rxRocOutGood$specificity, c(0, 1, 1))	
    
}