# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxSort and rxSortXdf
#
############################################################################################

"test.rxSortXdf.args.inFile" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  inXDFDS <- RxXdfData(inXDF)
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  outXDF2 <- file.path(tempdir(), ".rxOutFileTemp2.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  if (file.exists(outXDF2)) file.remove(outXDF2)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  noDataInFile <- tempfile( pattern = ".rxSort4Temp", fileext = ".xdf")
  
  sortByVars <- names(iris)[1:2]
  decreasing <- FALSE
  
  # illegal values
  checkException(rxSortXdf(inFile = iris, outFile = outXDF, sortByVars = sortByVars), silent = TRUE)
  checkException(rxSortXdf(inFile = "", outFile = outXDF, sortByVars = sortByVars), silent = TRUE)
  checkException(rxSortXdf(inFile = NULL, outFile = outXDF, sortByVars = sortByVars), silent = TRUE)
  checkException(rxSortXdf(inFile = noDataInFile, outFile = outXDF, sortByVars = sortByVars), silent = TRUE)
  
  # legal values
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars))
  checkTrue(rxSortXdf(inFile = inXDFDS, outFile = outXDF2, sortByVars = sortByVars))
    z <- rxDataStep(outXDF1)
    z2 <- rxDataStep(outXDF2)
    checkEquals(z, z2)
    checkEquals(names(z), names(iris))
  
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  if (file.exists(outXDF2)) file.remove(outXDF2)
}

"test.rxSortXdf.args.outFile" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  outXDFDS <- RxXdfData(outXDF)
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxDataStep(infert, inXDF, overwrite = TRUE)
  
  sortByVars <- "education"
  decreasing <- TRUE
  
  # illegal values
  checkException(rxSortXdf(inFile = inXDF, outFile = "", sortByVars = sortByVars), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = NULL, sortByVars = sortByVars), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = c("cat","dog"), sortByVars = sortByVars), silent = TRUE)
    
    # legal values
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF, sortByVars = sortByVars, decreasing = decreasing))
    z1 <- rxDataStep(outXDF)
    checkEquals(z1[["education"]], sort(infert[["education"]], decreasing = decreasing), check.attributes = FALSE) 
    
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDFDS, sortByVars = sortByVars, decreasing = decreasing, overwrite = TRUE))
    z2 <- rxDataStep(outXDFDS)
    checkEquals(z2[["education"]], sort(infert[["education"]], decreasing = decreasing), check.attributes = FALSE) 

    # outfile has same name as infile with varByVar sort
    rxSortXdf(inFile = outXDF, outFile = outXDF, sortByVars = "age", type = "varByVar", overwrite = TRUE)
    z2 <- rxDataStep(outXDF)
    checkEquals(z2[["age"]], sort(infert[["age"]], decreasing = FALSE), check.attributes = FALSE) 

  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)    
}

"test.rxSortXdf.args.sortByVars" <- function()
{  
  # initialize variables
    sampleDataDir <- rxGetOption("sampleDataDir")
    CensusPath <- file.path(sampleDataDir, "CensusWorkers.xdf")
  outXDF1 <- file.path(tempdir(), ".rxCensusSorted1.xdf")
  outXDF2 <- file.path(tempdir(), ".rxCensusSorted2.xdf")
  outXDF3 <- file.path(tempdir(), ".rxCensusSorted3.xdf")
  if (file.exists(outXDF1)) file.remove(outXDF1)
  if (file.exists(outXDF2)) file.remove(outXDF2)
  if (file.exists(outXDF3)) file.remove(outXDF3)
  
  sortByVars <- rxGetVarNames(CensusPath)
    keys1 <- sortByVars[1L]
    keys2 <- sortByVars[1L:2L]
  keys3 <- sortByVars[1L:3L]
    decreasing <- TRUE
  
  # illegal values
  checkException(rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = ""), silent = TRUE)
  checkException(rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = NULL), silent = TRUE)
  checkException(rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = letters[1:2]), silent = TRUE)
  checkException(rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = "notInSet"), silent = TRUE)
   
    # check multiple sort key configurations
  checkTrue(rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys1, decreasing = decreasing))
  checkTrue(rxSortXdf(inFile = CensusPath, outFile = outXDF2, sortByVars = keys2, decreasing = decreasing))
  checkTrue(rxSortXdf(inFile = CensusPath, outFile = outXDF3, sortByVars = keys3, decreasing = decreasing))
    
    # check one key sorting
    zNative <- rxReadXdf(CensusPath, varsToKeep = keys1)
    zSortXdf <- rxDataStep(outXDF1, varsToKeep = keys1)
    sortedNative <- sort(zNative[[keys1]], decreasing = decreasing)
    sortedXdf <- zSortXdf[[keys1]]
    checkEquals(sortedNative, sortedXdf, check.attributes = FALSE)
   
  if (file.exists(outXDF1)) file.remove(outXDF1)
  if (file.exists(outXDF2)) file.remove(outXDF2)
  if (file.exists(outXDF3)) file.remove(outXDF3)
}

"test.rxSortXdf.args.decreasing" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  rxDataStep(iris, inXDF, overwrite = TRUE)	
  sortByVars <- names(iris)
  
  # illegal values
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, decreasing = NULL), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, decreasing = 1L), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars[1L], decreasing = c(TRUE, FALSE)), silent = TRUE)

  # single key
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars[1L], decreasing = TRUE))
    zDecreasing <- rxDataStep(outXDF1, varsToKeep = sortByVars[1L])
    checkEquals(sort(iris[[sortByVars[1L]]], decreasing = TRUE), zDecreasing[[sortByVars[1L]]])
  if (file.exists(outXDF1)) file.remove(outXDF1)

  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars[1L], decreasing = FALSE))
    zDecreasing <- rxDataStep(outXDF1, varsToKeep = sortByVars[1L])
    checkEquals(sort(iris[[sortByVars[1L]]], decreasing = FALSE), zDecreasing[[sortByVars[1L]]])
  if (file.exists(outXDF1)) file.remove(outXDF1)

  # multiple key
    set.seed(10)
    decreasing <- sample(c(TRUE, FALSE), length(sortByVars), replace = TRUE)   
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, decreasing = decreasing))
    
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
}

"test.rxSortXdf.args.missingsLow" <- function()
{
  # initialize variables
  inXDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  # check illegal values
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, 
       missingsLow = pi), silent = TRUE)
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, 
       missingsLow = c(TRUE, FALSE)), silent = TRUE)
    
    # check whether missingsLow = TRUE, maintainOrder = TRUE puts NA values up front
    sortByVars <- "xnum2miss"
    decreasing <- FALSE
    DF <- rxReadXdf(inXDF)
    iMissing <- is.na(DF[[sortByVars]])
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF, sortByVars = sortByVars, 
        missingsLow = TRUE, maintainOrder = TRUE, decreasing = decreasing, 
    type = "mergeSort", overwrite = TRUE))   
    z <- rxReadXdf(outXDF)
    numMissing <- length(which(iMissing))
    firstRows <- seq.int(from = 1, length.out = numMissing)
    checkEquals(z[firstRows,], DF[which(iMissing),], check.attributes = FALSE)
    
    # check whether missingsLow = FALSE, maintainOrder = TRUE puts NA values up front
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF, sortByVars = sortByVars, 
        missingsLow = FALSE, maintainOrder = TRUE, decreasing = decreasing, 
    type = "mergeSort", overwrite = TRUE))   
    z <- rxReadXdf(outXDF)
    lastRows <- seq.int(to = NROW(DF), length.out = numMissing)
    checkEquals(z[lastRows,], DF[which(iMissing),], check.attributes = FALSE)
    
  if (file.exists(outXDF)) file.remove(outXDF)
}

"test.rxSortXdf.args.overwrite" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  rxDataStep(iris, inXDF, overwrite = TRUE)	
  sortByVars <- names(iris)
  
  # check illegal values
    checkException(rxSortXdf(inFile = inXDF, outFile = inXDF, sortByVars = sortByVars, overwrite = "a"), silent = TRUE)
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, overwrite = "a"), silent = TRUE)
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, overwrite = c(TRUE, FALSE)), silent = TRUE)
    
    # check legal values
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars))
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, overwrite = FALSE), silent = TRUE)
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, overwrite = TRUE))
        
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
}

"test.rxSortXdf.args.type" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  rxDataStep(iris, inXDF, overwrite = TRUE)	
  sortByVars <- names(iris)
  
  # check illegal values
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, type = "unknown"), silent = TRUE)
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, type = c("mergeSort", "varByVar")), silent = TRUE)
    checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, type = pi), silent = TRUE)
    
    # check legal values
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, 
        decreasing = FALSE, type = "mergeSort", overwrite = TRUE))
    z1 <- rxReadXdf(outXDF1, stringsAsFactors = FALSE)
  checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, 
        decreasing = FALSE, type = "varByVar", overwrite = TRUE))
    z2 <- rxReadXdf(outXDF1, stringsAsFactors = FALSE)
    checkEquals(z1, z2)
    
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
}

"test.rxSortXdf.args.varsToKeep" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  rxDataStep(iris, inXDF, overwrite = TRUE)	
  sortByVars <- names(iris)
  
  # illegal values
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, varsToKeep = 1L), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = sortByVars, varsToKeep = ""), silent = TRUE)
 
    # legal values
    varsToKeep <- sortByVars[1L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = varsToKeep, varsToKeep = varsToKeep))
    checkEquals(names(rxDataStep(outXDF1)), varsToKeep)
 
    varsToKeep <- sortByVars[1L:2L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = varsToKeep, varsToKeep = varsToKeep, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), varsToKeep)

    varsToKeep <- sortByVars[1L:3L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = varsToKeep, varsToKeep = varsToKeep, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), varsToKeep)

    varsToKeep <- sortByVars[1L:4L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = varsToKeep, varsToKeep = varsToKeep, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), varsToKeep)

    varsToKeep <- sortByVars[1L:5L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = varsToKeep, varsToKeep = varsToKeep, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), varsToKeep)

  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)       
}

"test.rxSortXdf.args.varsToDrop" <- function()
{
  # initialize variables
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  rxDataStep(iris, inXDF, overwrite = TRUE)	
    allVars <- names(iris)
  
  # illegal values
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars, varsToDrop = 1L), silent = TRUE)
  checkException(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars, varsToDrop = ""), silent = TRUE)
 
    # legal values
    varsToDrop <- allVars[1L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars[2L], varsToDrop = varsToDrop))
    checkEquals(names(rxDataStep(outXDF1)), setdiff(allVars, varsToDrop))
 
    varsToDrop <- allVars[1L:2L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars[3L], varsToDrop = varsToDrop, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), setdiff(allVars, varsToDrop))
 
    varsToDrop <- allVars[1L:3L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars[4L], varsToDrop = varsToDrop, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), setdiff(allVars, varsToDrop))

    varsToDrop <- allVars[1L:4L]
    checkTrue(rxSortXdf(inFile = inXDF, outFile = outXDF1, sortByVars = allVars[5L], varsToDrop = varsToDrop, overwrite = TRUE))
    checkEquals(names(rxDataStep(outXDF1)), setdiff(allVars, varsToDrop))

  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)      
}

"test.rxSortXdf.args.blocksPerRead" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(outXDF1)) file.remove(outXDF1)
  sortByVars <- c("xnum2", "xfac1")
  
    # mergeSort
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead= -1L, 
        type = "mergeSort"))
  zBlocksDefault <- rxDataStep(outXDF1)
  
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead = 1L, 
        type = "mergeSort", overwrite = TRUE))
  zBlocks1 <- rxDataStep(outXDF1)
  checkEquals(zBlocksDefault, zBlocks1)
    
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead = 5L, 
        type = "mergeSort", overwrite = TRUE))
  zBlocks5 <- rxDataStep(outXDF1)
  checkEquals(zBlocksDefault, zBlocks5)

    # varByVar
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead= -1L, 
        type = "varByVar", overwrite = TRUE))
  zBlocksDefault <- rxDataStep(outXDF1)
  
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead = 1L, 
        type = "varByVar", overwrite = TRUE))
  zBlocks1 <- rxDataStep(outXDF1)
  checkEquals(zBlocksDefault, zBlocks1)
    
    checkTrue(rxSortXdf(inFile = test100r5b, outFile = outXDF1, sortByVars =  sortByVars, blocksPerRead = 5L, 
        type = "varByVar", overwrite = TRUE))
  zBlocks5 <- rxDataStep(outXDF1)
  checkEquals(zBlocksDefault, zBlocks5)
    
    if (file.exists(outXDF1)) file.remove(outXDF1)
}


#####

"test.rxSortXdf.multiple.key.ouput.small" <- function()
{
    inXDF <- file.path(tempdir(), ".rxIn.xdf")
    outXDF <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
    DF <- data.frame(x=c(1,2,2,1,4,3), y = letters[1:6], z = 1:6)
    rxDataStep(DF,inXDF)

    # one sort key
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "x", decreasing = FALSE))
    z1 <- rxReadXdf(outXDF)
    iSort1<- order(DF[["x"]], decreasing = FALSE)
    checkEquals(DF[iSort1,], z1, check.attributes = FALSE)
        
    # two sort keys
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = c("x","y"), 
                    decreasing = c(FALSE, TRUE), overwrite = TRUE))
    z2 <- rxReadXdf(outXDF)
    iSort2 <- c(4L, 1L, 3L, 2L, 6L, 5L)
    checkEquals(DF[iSort2,], z2, check.attributes = FALSE)

    # three sort keys
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = c("x","y","z"), 
                    decreasing = c(FALSE, TRUE, TRUE), overwrite = TRUE))
    z3 <- rxReadXdf(outXDF)
    checkEquals(DF[iSort2,], z3, check.attributes = FALSE)

    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
}

"test.rxSortXdf.single.key.strings" <- function()
{
    # initialize variables
    inXDF <- file.path(tempdir(), ".rxIn.xdf")
    outXDF <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
    nStates <- length(state.name)
    iOrder <- seq_along(state.name)
    set.seed(100)
    
    # test already sorted strings
    DF <- data.frame(states = state.name, order = seq_along(state.name), stringsAsFactors = FALSE)
    rxDataStep(DF, inXDF)
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "states", decreasing = FALSE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(state.name, z[["states"]])
    checkEquals(iOrder, z[["order"]])
    
    # test permuted strings with no repeats
    iSample <- sample.int(nStates, nStates, replace = FALSE)
    DF <- data.frame(states = state.name[iSample], order = iSample, stringsAsFactors = FALSE)
    rxDataStep(DF, inXDF, overwrite = TRUE)
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "states", decreasing = FALSE, overwrite = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(state.name, z[["states"]])
    checkEquals(iOrder, z[["order"]])
    
    # test permuted strings with repeats
    iSample <- sample.int(nStates, nStates, replace = TRUE)
    DF <- data.frame(states = state.name[iSample], order = iSample, stringsAsFactors = FALSE)
    rxDataStep(DF, inXDF, overwrite = TRUE)
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "states", decreasing = FALSE, 
        overwrite = TRUE, maintainOrder = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(sort(state.name[iSample], decreasing = FALSE), z[["states"]])
    checkEquals(sort(iSample, decreasing = FALSE), z[["order"]])

    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
}
    
"test.rxSortXdf.single.key.long.strings" <- function()
{    
    inXDF <- file.path(tempdir(), ".rxIn.xdf")
    outXDF <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
    longName <- paste(rep(letters, 1000L), collapse="")
    longName2 <- longName
    substring(longName2, nchar(longName2) - 1L) <- "zy"
    DF <- data.frame(nms = c("a", longName2, "b", longName), index = 1:4, stringsAsFactors = FALSE)
    rxDataStep(DF, inXDF, overwrite = TRUE)
    
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "nms", decreasing = TRUE, overwrite = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(c(3L, 2L, 4L, 1L), z[["index"]])
    
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "nms", decreasing = FALSE, overwrite = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(c(1L, 4L, 2L, 3L), z[["index"]])

    if (file.exists(inXDF)) file.remove(inXDF)
    if (file.exists(outXDF)) file.remove(outXDF)
}

"test.rxSortXdf.Date.variables" <- function()
{
    inXDF <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    DF <- rxReadXdf(inXDF)    
    outXDF <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(outXDF)) file.remove(outXDF)
    
    # sort dates increasing
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "Date", decreasing = FALSE,
    type = "mergeSort", overwrite = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(DF[order(DF[["Date"]], decreasing = FALSE),], z, check.attributes = FALSE)
    
    # sort dates decreasing
    checkTrue(rxSortXdf(inXDF, outXDF, sortByVars = "Date", decreasing = TRUE, 
    type = "mergeSort", overwrite = TRUE))
    z <- rxReadXdf(outXDF, stringsAsFactors = FALSE)
    checkEquals(DF[order(DF[["Date"]], decreasing = TRUE),], z, check.attributes = FALSE)
   
    if (file.exists(outXDF)) file.remove(outXDF)    
}

"test.rxSortXdf.args.verbose.bufferLimit" <- function()
{
  # initialize variables
    sampleDataDir <- rxGetOption("sampleDataDir")
    CensusPath <- file.path(sampleDataDir, "CensusWorkers.xdf")
  outXDF1 <- file.path(tempdir(), ".rxCensusSorted1.xdf")

  if (file.exists(outXDF1)) file.remove(outXDF1)
  
  sortByVars <- rxGetVarNames(CensusPath)
    keys <- sortByVars[1L:2L]
    decreasing <- TRUE
  
    # Crank down memory limit to force mergeSort
  output1 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 2, verbose=1, reportProgress = 0,
      overwrite=TRUE)
  )
  #checkEquals( output1[2], "Auto selected sort type: Merge Sort" )
  print( output1[2])

    # Do a varByVar sort
  output2 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, type = "varByVar", verbose=1, reportProgress = 0,
      overwrite=TRUE)
  )
  #checkEquals( output2[1], "VarByVar/Mixed Sort: keys being presorted." )
  print( output2[1])

    # Set memory limit to get varByVar sort
  output3 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 4, verbose=1, reportProgress = 0,
      overwrite=TRUE)
  )
  #checkEquals( output3[2], "Auto selected sort type: Mixed Sort (VarByVar with Merge Sort for keys)." )
  print(output3[2])
  # Up the buffer limit so keys can be sorted in memory
  output4 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 6, verbose=1, reportProgress = 0,
      overwrite=TRUE)
  )
  #checkEquals( output4[2], "Auto selected sort type: VarByVar Sort (sort keys with InMemory Sort)." )
  print( output4[2] )
    # Auto sort should all be in memory
  output5 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, verbose=1, reportProgress = 0,
      overwrite=TRUE)
  )
  #checkEquals( output5[2], "Auto selected sort type: InMemory Sort." )
  print( output5[2] )

  if (file.exists(outXDF1)) file.remove(outXDF1)
}

"test.rxSortXdf.args.reportProgress" <- function()
{
  # initialize variables
    sampleDataDir <- rxGetOption("sampleDataDir")
    CensusPath <- file.path(sampleDataDir, "CensusWorkers.xdf")
  outXDF1 <- file.path(tempdir(), ".rxCensusSorted1.xdf")

  if (file.exists(outXDF1)) file.remove(outXDF1)
  
  sortByVars <- rxGetVarNames(CensusPath)
    keys <- sortByVars[1L:2L]
    decreasing <- TRUE
  
    # Crank down memory limit to force mergeSort
  #### Never use capture.output for check, it is changing depends on memory available.
  output1 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 2, verbose=0, reportProgress = 1,
      overwrite=TRUE)
  )

  checkEquals( output1[13],
    "Number of rows written to file: 1597, Number of columns: 6, Total number of rows in file: 351121"  )

  # Combine with verbose
  #### Never use capture.output for check, it is changing depends on memory available.
  output2 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 2, verbose=1, reportProgress = 1,
      overwrite=TRUE)
  )

  checkTrue(length(output2) > length(output1))
  
  # Get timings
  #### Never use capture.output for check, it is changing depends on memory available.
  output3 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 2, verbose=0, reportProgress = 2,
      overwrite=TRUE)
  )

  checkTrue(length(output3) > length(output1))

  # Get detailed timings 
  #### Never use capture.output for check, it is changing depends on memory available.
  output4 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 2, verbose=0, reportProgress = 3,
      overwrite=TRUE)
  )
  checkTrue(length(output4) > length(output3))
  
    # Do a varByVar sort
  #### Never use capture.output for check, it is changing depends on memory available.
  output5 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, type = "varByVar", verbose=0, reportProgress = 1,
      overwrite=TRUE)
  )
  checkEquals( output5[1], 
    "Number of rows written to file: 58521, Variable(s): age, incwage, Total number of rows in file: 58521" )

    # Set memory limit to get varByVar sort
  #### Never use capture.output for check, it is changing depends on memory available.
  output6 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 4, verbose=0, reportProgress = 1,
      overwrite=TRUE)
  )
  checkEquals( output6[1],
"Number of rows written to file: 58521, Variable(s): age, incwage, rxOrigRowIdx, Total number of rows in file: 58521")
  
  # Up the buffer limit so keys can be sorted in memory
  #### Never use capture.output for check, it is changing depends on memory available.
  output7 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, bufferLimit = 6, verbose=0, reportProgress = 1,
      overwrite=TRUE)
  )
    # Should have 5 write operations for 6 blocks
  # The keys written out together, then each other variable
  checkEquals( length(output7), 6 )

    # Auto sort should all be in memory
  #### Never use capture.output for check, it is changing depends on memory available.
  output8 <- capture.output(
    rxSortXdf(inFile = CensusPath, outFile = outXDF1, sortByVars = keys, 
      decreasing = decreasing, verbose=0, reportProgress = 1,
      overwrite=TRUE)
  )
    # Should just write out 6 blocks
  checkEquals( length(output8), 6)

  if (file.exists(outXDF1)) file.remove(outXDF1)
}

"test.rxSort.args.inData" <- function()
{
  # Sort data frame and return data frame
  sortByVars <- names(iris)[1:2]
  
  sortedIris <- rxSort(inData = iris, sortByVars = sortByVars)
  orderedIris <- iris[ order(iris[[1]], iris[[2]]) ,]
  checkEquals( sortedIris, orderedIris, check.attributes = FALSE)
  
  # Sort data frame and put in .xdf file

  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
  if (file.exists(outXDF)) file.remove(outXDF)
  
  rxSort(inData = iris, outFile = outXDF, sortByVars = sortByVars)
  sortedIrisXdf <- rxReadXdf(outXDF)
  checkEquals(sortedIris, sortedIrisXdf)
  
  # Clean up 
  file.remove(outXDF)

}

"test.rxSort.args.outFile" <- function()
{
  # Sort .xdf file and return data frame 

  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  
  sortByVars <- names(iris)[1:2]
  
  sortedIris <- rxSort(inData = inXDF, sortByVars = sortByVars)
  orderedIris <- iris[ order(iris[[1]], iris[[2]]) ,]
  checkEquals( sortedIris, orderedIris, check.attributes = FALSE)
  
  # Sub-selection of variables
  varsToKeep <- c("Sepal.Length", "Sepal.Width","Species")
  sortedIrisSub <- rxSort(inData = inXDF, sortByVars = sortByVars, 
    varsToKeep = varsToKeep)
  checkEquals( sortedIrisSub, orderedIris[, varsToKeep], check.attributes=FALSE)
  # Clean up 
  file.remove(inXDF)
}


"test.rxSort.args.xdfCompressionLevel" <- function()
{
  # Sort .xdf file put in new file

  inXDF <- file.path(tempdir(), ".rxInFileSort.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileSort.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  
  sortByVars <- names(iris)[1:2]
  
  sortDS1 <- rxSort(inData = inXDF, outFile = outXDF, xdfCompressionLevel = 0, sortByVars = sortByVars)
  sortedIris <- rxReadXdf(outXDF)
  orderedIris <- iris[ order(iris[[1]], iris[[2]]) ,]
  checkEquals( sortedIris, orderedIris, check.attributes = FALSE)

  sortFileInfo1 <- file.info(outXDF)
  sortFileSize1 <- sortFileInfo1$size
  file.remove(outXDF)

    sortCompressed <- function(type) {
        sortDS2 <- rxSort(inData = inXDF, outFile = outXDF, xdfCompressionLevel = 2, sortByVars = sortByVars,
            type = type)
        sortedIris <- rxReadXdf(outXDF)
        orderedIris <- iris[order(iris[[1]], iris[[2]]),]
        checkEquals(sortedIris, orderedIris, check.attributes = FALSE)

        sortFileInfo2 <- file.info(outXDF)
        sortFileSize2 <- sortFileInfo2$size

        file.remove(outXDF)
        checkTrue(sortFileSize2 < sortFileSize1)
    }

    sortCompressed("auto")
    sortCompressed("mergeSort")
    sortCompressed("varByVar")

    # Clean up 
  file.remove(inXDF)
}

"test.rxSort.args.decreasing" <- function()
{
  # Sort .xdf file and return data frame 
  
  sortByVars <- c("Sepal.Length", "Sepal.Width")
  decreasing <- c(TRUE, FALSE)
  sortedIris <- rxSort( inData = iris, sortByVars = sortByVars,
    decreasing = decreasing)

  orderedIris <- iris[ order(-iris[[1]], iris[[2]]) ,]
  checkEquals( sortedIris, orderedIris, check.attributes = FALSE)
  
  # Reverse
  decreasing <- c(FALSE, TRUE)
  sortedIris <- rxSort( inData = iris, sortByVars = sortByVars,
    decreasing = decreasing)

  orderedIris <- iris[ order(iris[[1]], -iris[[2]]) ,]
  checkEquals( sortedIris, orderedIris, check.attributes = FALSE)

}

"test.rxSort.args.type" <- function()
{
  # Sort .xdf file and return data frame 
  
  sortByVars <- c("Sepal.Length", "Sepal.Width")
  sortedIris1 <- rxSort( inData = iris, sortByVars = sortByVars,
    type = "varByVar")

  orderedIris <- iris[ order(iris[[1]], iris[[2]]) ,]
  checkEquals( sortedIris1, orderedIris, check.attributes = FALSE)
  
  sortedIris2 <- rxSort( inData = iris, sortByVars = sortByVars,
    type = "mergeSort")

  checkEquals( sortedIris1, sortedIris2)
  
  # Use "auto" default
  sortedIris3 <- rxSort( inData = iris, sortByVars = sortByVars)
  checkEquals( sortedIris1, sortedIris3)

}


"test.rxSort.value" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")
    outFile3 <- tempfile(pattern = ".rxTest3", fileext = ".xdf")
    outDS3 <- RxXdfData( outFile3 )
    
    # Output data frame
    outData1 <- rxSort( inData = inFile, sortByVars = "xint1")   
    # Output xdf
    outData2 <- rxSort( inData = inFile, outFile = outFile2, 
        sortByVars = "xint1", overwrite = TRUE)
    # Output xdf data source
    outData3 <- rxSort( inData = inFile, outFile = outDS3, 
        sortByVars = "xint1", overwrite = TRUE )
    
    outDataShort1 <- rxDataStep(outData1, numRows = 5)
    outDataShort2 <- rxDataStep(outData2, numRows = 5)
    outDataShort3 <- rxDataStep(outData3, numRows = 5)
    
    checkEquals(outDataShort1, outDataShort2)
    checkEquals(outDataShort1, outDataShort3)

    sumFormula <-  ~y + ybinint + xnum2miss + xint1
    outSum1 <- rxResultsDF(rxSummary(sumFormula, data = outData1))
    
    outSum2 <- rxResultsDF(rxSummary(sumFormula, data = outData2))
    
    outSum3 <- rxResultsDF(rxSummary(sumFormula, data = outData3))
    
    checkEquals(outSum1, outSum2)
    checkEquals(outSum1, outSum3)
    
    file.remove(outFile2)
    file.remove(outFile3)
}

"test.rxSort.args.caseSensitive" <- function()
{
  testData <- data.frame(x = c("Z", "z", "c", "C", "A", "a"), stringsAsFactors = FALSE)
  sortedData1 <- rxSort(inData = testData, sortByVars = "x")
  
  notCaseSensitiveSort <- c("A", "a",  "c", "C","Z", "z" )
  checkEquals(notCaseSensitiveSort, sortedData1$x)
    
  sortedData2 <- rxSort(inData = testData, sortByVars = "x", caseSensitive = TRUE)
  caseSensitiveSort <- c("A", "C", "Z", "a", "c", "z" )
  checkEquals(caseSensitiveSort, sortedData2$x)
  
}

"test.rxSort.data.fixedWidth" <- function()
{
  testData <- data.frame(x = c(
        "abaaa", 
        "aaaba", 
        "aaaaa", 
        "aabaa", 
        "babab", 
        "aaaab",
        "abaaa", 
        "aaaba", 
        "aaaaa", 
        "babaa", 
        "baaab", 
        "aaaab",
        "abcde",   
        "cdefg",
        "hijkl",
        "mnopq", 
        "aaaaa"    
        ), 
        stringsAsFactors = FALSE)
    attr(testData$x, ".rxColWidth") <- as.integer(5)
    outFile <- tempfile(pattern = ".rxSort", fileext = ".xdf")
    outFileTemp <- tempfile(pattern = ".rxSortTemp", fileext = ".xdf")
    rxDataStep(testData, outFile = outFileTemp)
    test1 <- rxReadXdf(outFileTemp)
    checkEquals(testData, test1)
    
    # Now make it multi-block
    rxDataStep(inData = outFileTemp, outFile =  outFile, rowsPerRead = 11, overwrite = TRUE)
    
    # Make sure we have fixed width data
    varInfo <- rxGetVarInfo(outFile)
    checkEquals(varInfo$x$storage, "fixed")
    testData1 <- rxReadXdf(outFile)
    checkEquals(testData, testData1)
    
  sortedData <- rxSort(inData = outFile, sortByVars = "x")
    expectData <- sort(testData$x)
    checkEquals(as.vector(sortedData$x), expectData)
    
    file.remove(outFile)	
  file.remove(outFileTemp)
}

"test.rxSort.args.removeDupKeys" <- function()
{

  # Create data on 3 families and replicate for different states	
  family1 <- data.frame(
    MemberId = as.integer(1),
    FirstName = c("Joe"),
    LastName = c("Smith"),
    Age = c(40), stringsAsFactors = FALSE
    )
  
  family2 <- data.frame(
    MemberId = as.integer(1:6),
    FirstName = c("John", "Mary", "Scott", "Judy", "Tom", "Peter"),
    LastName = rep("Jones", times = 6),
    Age = c(40,  38, 2.5, 4, 6, 8), stringsAsFactors = FALSE
    )
  
  family3 <- data.frame(
    MemberId = as.integer(1:3),
    FirstName = c("Jack", "Jerry", "Laura"),
    LastName = c("Elderkin", "Williams", "Willkin"),
    Age = c( 35, 35, 2.5 ), stringsAsFactors = FALSE
    )
  
  states <- c("WA", "VA", "NM", "OH", "CA", "MA", "TX", "NC", "CO", "NH",
    "VT", "WI", "ND", "SD", "UT", "WA", "OH", "WV", "MN", "NY", "DL", "AK",
    "AZ", "FL" )
  
  families <- rbind(family1, family2, family3)
  for (i in 1:length(states))
  {
    thisState <- families
    thisState$State <- states[i]
    if (i == 1)
    {
      census <- thisState
    }
    else
    {
      census <- rbind(census, thisState)
    }	
  }
  census$State <- as.factor(census$State)
  
  origFile <- tempfile( pattern = ".rxSortDedup", fileext = ".xdf")
  sortedFile <- tempfile(pattern = ".rxSortDedup2", fileext = ".xdf")
  rxDataStep(inData = census, outFile = origFile, rowsPerRead = 20)
  
  on.exit(
    if (file.exists(origFile)) file.remove(origFile), add = TRUE
    )
  
  on.exit(
    if (file.exists(sortedFile)) file.remove(sortedFile), add = TRUE
    )	
  
  # Start tests for removing duplicates here
  rxSort(inData = origFile, outFile = sortedFile,
       sortByVars = c("LastName", "FirstName", "Age", "State"),
     removeDupKeys = TRUE)

  fileInfoOrig <- rxGetInfo(origFile)
  fileInfoSorted <- rxGetInfo(sortedFile)

    cubeOut <- rxResultsDF(rxCube(~State, data = origFile))
  # Duplicate states will have 10 extra observations
  numNoDups <- length(cubeOut$Counts) * 10
  checkEquals( fileInfoSorted$numRows, numNoDups)
  
  # Make sure we have all the variables
  checkEquals(fileInfoOrig$numVars, fileInfoSorted$numVars)
  
  # Redo, with dupFreqVar

  file.remove(sortedFile)		
  rxSort(inData = origFile, outFile = sortedFile,
       sortByVars = c("LastName", "FirstName", "Age", "State"),
     removeDupKeys = TRUE, dupFreqVar = "FreqWt")

  fileInfoOrig <- rxGetInfo(origFile)
  fileInfoSorted <- rxGetInfo(sortedFile)

    cubeOut <- rxResultsDF(rxCube(~State, data = origFile))
  # Duplicate states will have 10 extra observations
  numNoDups <- length(cubeOut$Counts) * 10
  checkEquals( fileInfoSorted$numRows, numNoDups)	
  
  # Make sure we have all the variables
  checkEquals(fileInfoOrig$numVars + 1, fileInfoSorted$numVars)
  
  # Confirm that the new variable is there, and has the correct sum
  sumOut <- rxSummary(~FreqWt, data = sortedFile, summaryStats = "Sum")	
  checkEquals(sumOut$sDataFrame$Sum[1], fileInfoOrig$numRows)	
  
  # Check for error if duplicate variable names unless 
  
  file.remove(sortedFile)		
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)
    
  res <- try(
      rxSort(inData = origFile, outFile = sortedFile,
        sortByVars = c("LastName", "FirstName", "Age", "State"),
          removeDupKeys = TRUE, dupFreqVar = "Age")
    )
  checkTrue(inherits(res,"try-error"))

}








