# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for things related to rxSparkCacheData
#
###########################################################################################
"test.rxSparkCacheData.hive" <- function()
{
  hiveData <- RxHiveData(table = "table")
  hiveData <- rxSparkCacheData(hiveData)
  checkEquals(hiveData@cache, TRUE)
  hiveData <- rxSparkCacheData(hiveData, FALSE)
  checkEquals(hiveData@cache, FALSE)
}

"test.rxSparkCacheData.Xdf" <- function()
{
 fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
 xdfData <- RxXdfData(fileName)
 xdfData <- rxSparkCacheData(xdfData)
 checkEquals(xdfData@cache, TRUE)
 xdfData <- rxSparkCacheData(xdfData, FALSE)
 checkEquals(xdfData@cache, FALSE)
}


"test.rxSparkCacheData.Parquet" <- function()
{
  parquetData <- RxParquetData(file = "/testpath")
  parquetData <- rxSparkCacheData(parquetData)
  checkEquals(parquetData@cache, TRUE)
  parquetData <- rxSparkCacheData(parquetData, FALSE)
  checkEquals(parquetData@cache, FALSE)
}

"test.rxSparkCacheData.Orc" <- function()
{
  orcData <- RxOrcData(file = "/testpath")
  orcData <- rxSparkCacheData(orcData)
  checkEquals(orcData@cache, TRUE)
  orcData <- rxSparkCacheData(orcData, FALSE)
  checkEquals(orcData@cache, FALSE)
}

"test.rxSparkRemoveData.invalid.cc" <- function()
{
  local <- RxLocalSeq()
  input <- RxOrcData(file = "/testpath")
  checkException(rxSparkRemoveData(input, local), "Should not accept local cc.")
}

"test.rxSparkRemoveData.sparkCC.persistentRun.FALSE" <- function()
{
  cc <- RxSpark()
  input <- RxOrcData(file = "/testpath")
  checkException(rxSparkRemoveData(cc, local), "Should not accept spark cc with persistentRun to be FALSE.")
}

"test.rxSparkRemoveData.input.empty.list" <- function()
{
  cc <- RxSpark()
  cc@persistentRun <- TRUE
  checkException(rxSparkRemoveData(list(), cc), "Input list cannot be empty.")
}

"test.rxSparkRemoveData.input.NULL" <- function()
{
  cc <- RxSpark()
  cc@persistentRun <- TRUE
  checkException(rxSparkRemoveData(NULL, cc), "Input cannot be NULL.")
}

"test.prepareRemoveDataForSpark.invalid.input.data" <- function()
{
  input <- "invalid"
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(input), "Input must be RxHiveData/RxParquetData/RxOrcData/RxXdfData object or list of these objects.")

  input <- list("invalid")
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(input), "Input must be RxHiveData/RxParquetData/RxOrcData/RxXdfData object or list of these objects.")

  input <- RxOrcData(file = "/testpath")
  input@dfName <- NULL
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(input), "Input RxHiveData/RxParquetData/RxOrcData/RxXdfData object must have not null dfName.")

  input <- RxOrcData(file = "/testpath")
  input@xdfUuid <- NULL
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(input), "Input RxHiveData/RxParquetData/RxOrcData/RxXdfData object must have not null xdfUuid.")

  input <- RxOrcData(file = "/testpath")
  input@dfName <- NULL
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(list(input)), "Input RxHiveData/RxParquetData/RxOrcData/RxXdfData object must have not null dfName.")

  input <- RxOrcData(file = "/testpath")
  input@xdfUuid <- NULL
  checkException(RevoScaleR:::.prepareRemoveDataForSpark(list(input)), "Input RxHiveData/RxParquetData/RxOrcData/RxXdfData object must have not null xdfUuid.")
}

"test.prepareRemoveDataForSpark.valid.input.data" <- function()
{
  input <- RxOrcData(file = "/testpath")
  metadata <- RevoScaleR:::.prepareRemoveDataForSpark(input)
  checkEquals(
      lapply(
          metadata[["args"]][["removeDataList"]], function(x) {return(x[["dfName"]])}),
      list(input@dfName))
  checkEquals(
      lapply(
          metadata[["args"]][["removeDataList"]], function(x) {return(x[["xdfId"]])}),
      list(input@xdfUuid))

  input <- RxOrcData(file = "/testpath")
  input2 <- RxParquetData(file = "/testpath2")
  metadata <- RevoScaleR:::.prepareRemoveDataForSpark(list(input, input2))
  checkEquals(
      lapply(
          metadata[["args"]][["removeDataList"]], function(x) {return(x[["dfName"]])}),
      list(input@dfName, input2@dfName))
  checkEquals(
      lapply(
          metadata[["args"]][["removeDataList"]], function(x) {return(x[["xdfId"]])}),
      list(input@xdfUuid, input2@xdfUuid))
}
