# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxSplit and rxSplitXdf
#
############################################################################################

"test.rxSplitXdf.args.spltByFactor.with.transforms" <- function()
{
    assign("OPTS", options(), .GlobalEnv)
    DF <- data.frame(alphabet = letters[1:12], 
                   state = state.name[1:12],
             color = c("black","red","black","red","blue","red","red",
                  "blue", "red", "blue","blue","red"),
             x = rnorm(12),
             y = runif(12, 20, 30),
               splitVar = factor(c(1,1,1,1,2,2,2,2,3,3,3,3)))

    splitDF <- lapply(split(DF, DF[["splitVar"]]), "[", -6L)

    XDF <- tempfile( pattern = ".rxSplitF", fileext = ".xdf")

    for (i in seq_along(splitDF))
    {
      rxDataStep(splitDF[[i]], XDF, append = if (i == 1L) "none" else "rows", overwrite = TRUE)
    }

    transforms <- expression(list(colorUpper = factor(toupper(color), levels = toupper(colorLevels))))
    xform <- function(dataList)
    {
      dataList[["colorCode"]] <- toupper(substring(dataList[["color"]], 1L, 1L))
      dataList
    }

    rowSelection <- expression(colorCode == "B")

    #> XDF
    # alphabet       state color           x        y 
    # 1         a     Alabama black  0.87033302 20.19974
    # 2         b      Alaska   red  0.90337475 26.61231
    # 3         c     Arizona black -0.73451096 28.69978
    # 4         d    Arkansas   red  0.35590520 29.12514
    #---------------------------------------------------
    # 5         e  California  blue  1.66062376 24.42044
    # 6         f    Colorado   red -1.11864006 25.66632
    # 7         g Connecticut   red -1.12591963 25.03329
    # 8         h    Delaware  blue -0.03031119 23.85914
    #---------------------------------------------------
    # 9         i     Florida   red  0.46016741 25.34201
    # 10        j     Georgia  blue -0.76203934 25.09110
    # 11        k      Hawaii  blue  0.36683224 28.22891
    # 12        l       Idaho   red  1.10631387 29.03328
    
    ###
    # No row selection
    outFiles <- rxSplitXdf(XDF, splitByFactor = "color", verbose = 0L, reportProgress = 0L,
      transforms = transforms,
      transformFunc = xform,
        transformObjects = list(colorLevels = levels(DF[["color"]])),
      overwrite = TRUE,
        updateLowHigh = TRUE)
    
    outDF <- lapply(outFiles, rxDataStep, reportProgress = 0L)
    varInfo <- lapply(outFiles, rxGetVarInfo)
    
    # check transformed values and split
    checkEquals(length(outDF), 3L)
    checkEquals(as.vector(outDF[[1L]][["colorUpper"]]), rep("BLACK", 2L))
    checkEquals(as.vector(outDF[[2L]][["colorUpper"]]), rep("BLUE", 4L))
    checkEquals(as.vector(outDF[[3L]][["colorUpper"]]), rep("RED", 6L))
    checkEquals(as.vector(outDF[[1L]][["colorCode"]]), rep("B", 2L))
    checkEquals(as.vector(outDF[[2L]][["colorCode"]]), rep("B", 4L))
    checkEquals(as.vector(outDF[[3L]][["colorCode"]]), rep("R", 6L))
    
    # check low-high values of continuous variables
    blackDF <- subset(DF, color == "black")
    checkEqualsNumeric(unlist(varInfo[[1L]][["x"]][c("low", "high")]), range(blackDF[["x"]]))
    checkEqualsNumeric(unlist(varInfo[[1L]][["y"]][c("low", "high")]), range(blackDF[["y"]]))
    
    blueDF <- subset(DF, color == "blue")
    checkEqualsNumeric(unlist(varInfo[[2L]][["x"]][c("low", "high")]), range(blueDF[["x"]]))
    checkEqualsNumeric(unlist(varInfo[[2L]][["y"]][c("low", "high")]), range(blueDF[["y"]]))
    
    redDF <- subset(DF, color == "red")
    checkEqualsNumeric(unlist(varInfo[[3L]][["x"]][c("low", "high")]), range(redDF[["x"]]))
    checkEqualsNumeric(unlist(varInfo[[3L]][["y"]][c("low", "high")]), range(redDF[["y"]]))
    
    # remove output files
    invisible(sapply(outFiles, function(x) if (file.exists(x@file)) unlink(x@file)))
    
    ###
    # Row selection
    outFiles <- rxSplitXdf(XDF, splitByFactor = "color", verbose = 0L, reportProgress = 0L,
        rowSelection = rowSelection,
      transforms = transforms,
      transformFunc = xform,
        transformObjects = list(colorLevels = levels(DF[["color"]])),
      overwrite = TRUE,
        updateLowHigh = TRUE)
    
    outDF <- lapply(outFiles, rxDataStep, reportProgress = 0L)
    varInfo <- lapply(outFiles, rxGetVarInfo)
    
    # check transformed values and split
    checkEquals(length(outDF), 2L)
    checkEquals(as.vector(outDF[[1L]][["colorUpper"]]), rep("BLACK", 2L))
    checkEquals(as.vector(outDF[[2L]][["colorUpper"]]), rep("BLUE", 4L))
    checkEquals(as.vector(outDF[[1L]][["colorCode"]]), rep("B", 2L))
    checkEquals(as.vector(outDF[[2L]][["colorCode"]]), rep("B", 4L))
    
    # check low-high values of continuous variables
    blackDF <- subset(DF, color == "black")
    checkEqualsNumeric(unlist(varInfo[[1L]][["x"]][c("low", "high")]), range(blackDF[["x"]]))
    checkEqualsNumeric(unlist(varInfo[[1L]][["y"]][c("low", "high")]), range(blackDF[["y"]]))
    
    blueDF <- subset(DF, color == "blue")
    checkEqualsNumeric(unlist(varInfo[[2L]][["x"]][c("low", "high")]), range(blueDF[["x"]]))
    checkEqualsNumeric(unlist(varInfo[[2L]][["y"]][c("low", "high")]), range(blueDF[["y"]]))

    # remove output files
    invisible(sapply(outFiles, function(x) if (file.exists(x@file)) unlink(x@file)))

    ###
    # No row selection, updateLowHigh = FALSE
    outFiles <- rxSplitXdf(XDF, splitByFactor = "color", verbose = 0L, reportProgress = 0L,
      transforms = transforms,
      transformFunc = xform,
        transformObjects = list(colorLevels = levels(DF[["color"]])),
      overwrite = TRUE,
        updateLowHigh = FALSE)
    
    outDF <- lapply(outFiles, rxDataStep, reportProgress = 0L)
    varInfo <- lapply(outFiles, rxGetVarInfo)
    
    # check transformed values and split
    checkEquals(length(outDF), 3L)
    
    # check low-high values of continuous variables
    xrange <- range(DF[["x"]])
    yrange <- range(DF[["y"]])
    checkEqualsNumeric(unlist(varInfo[[1L]][["x"]][c("low", "high")]), xrange)
    checkEqualsNumeric(unlist(varInfo[[1L]][["y"]][c("low", "high")]), yrange)
    
    checkEqualsNumeric(unlist(varInfo[[2L]][["x"]][c("low", "high")]), xrange)
    checkEqualsNumeric(unlist(varInfo[[2L]][["y"]][c("low", "high")]), yrange)
    
    checkEqualsNumeric(unlist(varInfo[[3L]][["x"]][c("low", "high")]), xrange)
    checkEqualsNumeric(unlist(varInfo[[3L]][["y"]][c("low", "high")]), yrange)
    
    invisible(sapply(outFiles, function(x) if (file.exists(x@file)) unlink(x@file)))
  file.remove(XDF)
}

"test.rxSplit.args.splitByFactors.blocks" <- function()
{
  outDir <- tempdir()
  outFile <- file.path(outDir, "testSplit.xdf")
  outFilesBase = file.path(outDir, "splitOut")
  
  myData <- data.frame(x = 1:100, y = rep(c(0,1), each = 50))
  myData <- data.frame(x = 1:100, y = rep(1, times = 100))
  myData$y[1:10] <- rep(0, times = 10)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
    } else
    {
      rxDataStep(inData = myData[1:50,], outFile = outFile, overwrite = TRUE)
      rxDataStep(inData = myData[51:100,], outFile = outFile, append = "rows")
    } 

  newFiles <- rxSplit( inData = outFile, outFilesBase = outFilesBase,
    outFileSuffixes = c("Group0", "Group1"),
    splitByFactor = "testSplitVar", overwrite = TRUE,
    transforms = list(testSplitVar = factor( y, levels = c(0, 1))))	
  
  myData0 <- rxDataStep(newFiles[[1]])
  myData1 <- rxDataStep(newFiles[[2]])
  
  checkEquals( myData0$y, rep(0, times = 10))
  checkEquals( myData1$y, rep(1, times = 90))
  
  file.remove(outFile)
  lapply(newFiles, function(x){ file.remove(x@file) })
}


"test.rxSplitXdf.args.inFile.outFilesBase.outFileSuffixes.numOutFiles" <- function()
{
    #####
    # Write files to different directories
    inFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    numOutFiles <- 5
    outFileSuffixes <- "Census-byRows"
    paths <- sapply(seq(numOutFiles), function(x) tempfile( pattern = ".rxSplitSuf"))

    # Method 1: outFilesBase only
    outFilesBase <- file.path(paths, paste("Census-byRows", seq(numOutFiles), ".xdf", sep= ""))
    outFiles1 <- rxSplitXdf(inFile, outFilesBase = outFilesBase, verbose = 1, overwrite = TRUE)
  outFiles1 <- sapply(outFiles1, slot, "file")
    info1 <- sapply(outFiles1, rxGetInfo)
    standardNames <- normalizePath(file.path(paths, paste("Census-byRows", seq(numOutFiles), ".xdf", sep = "")))
    sapply(paths, unlink, recursive = TRUE)
    
    # Method 2: outFileBase with numOutFiles
    outFilesBase <- file.path(paths, "Census-byRows")
    outFiles2 <- rxSplitXdf(inFile, outFilesBase = outFilesBase, numOutFiles = numOutFiles,
                           splitBy = "rows", verbose = 1, overwrite = TRUE)
  outFiles2 <- sapply(outFiles2, slot, "file")
    info2 <- sapply(outFiles2, rxGetInfo)
    sapply(paths, unlink, recursive = TRUE)
    
    # Method 3: outFileBase with outFileSuffixes and numOutFiles
    outFilesBase <- paste(paths, .Platform$file.sep, sep = "")
    outFiles3 <- rxSplitXdf(inFile, outFilesBase = outFilesBase, outFileSuffixes = outFileSuffixes, 
                            numOutFiles = numOutFiles, splitBy = "rows", verbose = 1, overwrite = TRUE)
  outFiles3 <- sapply(outFiles3, slot, "file")
    info3 <- sapply(outFiles3, rxGetInfo)
    sapply(paths, unlink, recursive = TRUE)
    
    checkEquals(outFiles1, outFiles2)
    checkEquals(outFiles2, outFiles3)
    checkEquals(info1, info2)
    checkEquals(info2, info3)
    
    checkEquals(as.vector(outFiles1), standardNames)
}

"test.rxSplitXdf.args.inFile" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit2", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for nonexistent file
    checkException(rxSplitXdf(inFile = tempfile(), numOutFiles = 2L), silent = TRUE)
    
    # check for illegal length and types
    checkException(rxSplitXdf(inFile = DF, numOutFiles = 2L), silent = TRUE)
    checkException(rxSplitXdf(inFile = c(XDF, XDF), numOutFiles = 2L), silent = TRUE)
    checkException(rxSplitXdf(inFile = pi, numOutFiles = 2L), silent = TRUE)
    checkException(rxSplitXdf(inFile = character(0), numOutFiles = 2L), silent = TRUE)
    checkException(rxSplitXdf(inFile = NULL, numOutFiles = 2L), silent = TRUE)
    checkException(rxSplitXdf(inFile = "", numOutFiles = 2L), silent = TRUE)
    
    # check for inFile name as outFile base when only nuOutFiles is specified
    outFilePaths <- rxSplitXdf(inFile = XDF, numOutFiles = 2L, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(grep(tools:::file_path_sans_ext(basename(XDF)), outFilePaths), seq(2L))
    
    # remove files
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.outFilesBase" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit3", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = pi), silent = TRUE)
    
    # check illegal number of output files: one and more than the number of rows
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = letters[1L]), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = letters[seq(nrow(DF) + 1L)]), silent = TRUE)

    # relative paths
    outFilesBase <- c("cat","dog")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste(outFilesBase, "xdf", sep = "."))
    checkEquals(normalizePath(dirname(outFilePaths))[1L], normalizePath(getwd())) 
    
    outFilesBase <- c("cat.XdF","dog.XDf")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste(tools:::file_path_sans_ext(outFilesBase), "xdf", sep = "."))
    
    outFilesBase <- c("cat","dog.XDf")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste(tools:::file_path_sans_ext(outFilesBase), "xdf", sep = "."))
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))
 
    # Mixed absolute and relative paths
 
  outDir <- tempdir()
  outFileDir <- paste(outDir, ".rxSplitCat", sep = .Platform$file.sep)
    outFilesBase <- c(paste(outFileDir, "cow", sep = .Platform$file.sep), "dog.XDf")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    # clean up
  on.exit(
    invisible(sapply(c(XDF, outFilePaths, outFileDir), function(x) if (file.exists(x)) unlink(x, recursive = TRUE))),
      add = TRUE    
    )
    checkEquals(normalizePath(dirname(outFilePaths[1L])), normalizePath(file.path(outDir, ".rxSplitCat")))
    checkEquals(basename(outFilePaths[1L]), "cow.xdf")
    checkEquals(normalizePath(dirname(outFilePaths[2L])), normalizePath(getwd()))
    checkEquals(basename(outFilePaths[2L]), "dog.xdf")


}

"test.rxSplitXdf.args.outFileSuffixes" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit4", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, outFileSuffixes = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFileSuffixes = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFileSuffixes = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, outFileSuffixes = pi), silent = TRUE)
    
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilesSuffixes <- c(2, 5, 7)
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, outFileSuffixes = outFilesSuffixes, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste("foo", outFilesSuffixes, ".xdf", sep = ""))
    checkEquals(unique(normalizePath(dirname(outFilePaths))), normalizePath(dirname(outFilesBase)))

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
    outPath <- file.path(outDir, "myFiles")
    if (file.exists(outPath)) unlink(outPath, recursive = TRUE)
}

"test.rxSplitXdf.args.numOutFiles" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit5", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 1L), silent = TRUE)
    
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste("foo", seq(2L), ".xdf", sep = ""))
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))
    
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 5L, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    checkEquals(basename(outFilePaths), paste("foo", seq(5L), ".xdf", sep = ""))

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.splitBy" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit6", fileext = ".xdf")
    
    #if (rxIsExpressEdition())
    #{
        #rxDataStep(inData = DF, outFile = XDF)
    #} else
    #{
        rxDataStep(DF[1:2,], XDF)
        rxDataStep(DF[3:5,], XDF, append = "rows")
        rxDataStep(DF[6:7,], XDF, append = "rows")
        rxDataStep(DF[8:10,], XDF, append = "rows")
    #}
 
    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = c("rows", "rows")), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, splitBy = "unsupported"), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 11L, splitBy = "rows"), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 5L, splitBy = "blocks"), silent = TRUE)
    
    # split by rows
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, splitBy = "rows", overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkEquals(DFS[[1L]], DF[1:5,], check.attributes = FALSE)
    checkEquals(DFS[[2L]], DF[6:10,], check.attributes = FALSE)
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))

    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 3L, splitBy = "rows", overwrite = TRUE, fill = "right")
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkEquals(DFS[[1L]], DF[1:3,], check.attributes = FALSE)
    checkEquals(DFS[[2L]], DF[4:6,], check.attributes = FALSE)
    checkEquals(DFS[[3L]], DF[7:10,], check.attributes = FALSE)
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))
    
    # split by blocks
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 4L, splitBy = "blocks", overwrite = TRUE, fill = "right")
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkEquals(DFS[[1L]], DF[1:2,], check.attributes = FALSE)
    checkEquals(DFS[[2L]], DF[3:5,], check.attributes = FALSE)
    checkEquals(DFS[[3L]], DF[6:7,], check.attributes = FALSE)
    checkEquals(DFS[[4L]], DF[8:10,], check.attributes = FALSE)
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.splitByFactor" <- function()
{
    animal <- c("goat", "cow", "goat", "pig", "horse", "horse", "pig", "orangutan", "goat", "pig")
    DF <- data.frame(x = 1:10, y = 10:1, animal = animal)
    XDF <- tempfile( pattern = ".rxSplit7", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = c("animal", "animal")), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = "notAVariable"), silent = TRUE)
    
    # split by rows
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/mySplitByFactorFiles", sep = "")
  on.exit(unlink(outFilesBase, recursive = TRUE), add = TRUE)
  
    outFilePaths1 <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase,
                               splitByFactor = "animal", overwrite = TRUE)
  outFilePaths1 <- sapply(outFilePaths1, slot, "file")
    DFS <- lapply(outFilePaths1, rxDataStep)
    
    # ensure that each file contains data corresponing to a single animal level
    checkEquals(length(DFS), length(levels(DF[["animal"]])))
    checkTrue(all(sapply(DFS, function(x) "animal" %in% names(x))))
    checkEquals(as.vector(sapply(DFS, function(x) unique(as.vector(x[["animal"]])))), levels(DF[["animal"]]))
    
    animalCounts <- rxCube(~ animal, DF)[["Counts"]]
    checkEquals(animalCounts, as.vector(sapply(DFS, nrow)))
    
    # check error for when variable exists in original data but is not a factor
    checkException(rxSplitXdf(inFile = XDF, splitByFactor = "x"), silent = TRUE)
    
    # variable is created by a transform and does not exist in the original data
    "xform" <- function(dataList)
    {
        dataList[["myNewVar"]] <- factor(sample.int(5, length(dataList[[1L]]), replace = TRUE), levels = myLevels)
        dataList
    }
    
    myLevels <- as.character(1:5)
    outFilesBase <- paste(outDir, "/mySplitByFactorFiles/transformedFactor", sep = "")
    outFilePaths2 <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase,
                                splitByFactor = "myNewVar",  overwrite = TRUE,
                                transformFunc = xform, transformObjects = list(myLevels = myLevels))
  outFilePaths2 <- sapply(outFilePaths2, slot, "file")
    DFS <- lapply(outFilePaths2, rxDataStep)
    checkTrue(all(sapply(DFS, function(x) "myNewVar" %in% names(x))))
    checkTrue(all(sapply(DFS, function(x) unique(as.vector(x[["myNewVar"]]))) %in% myLevels))
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths1, outFilePaths2), function(x) if (file.exists(x)) unlink(x)))
    
    outPath <- file.path(outDir, "mySplitByFactorFiles")
    if (file.exists(outPath)) unlink(outPath, recursive = TRUE)    
}

"test.rxSplitXdf.args.varsToKeep" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit8", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToKeep = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToKeep = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToKeep = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToKeep = 1L), silent = TRUE)

    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToKeep = NULL, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], names(DF))
    checkEquals(varsKept[[2L]], names(DF))
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))
    
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToKeep = "x", overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], "x")
    checkEquals(varsKept[[2L]], "x")
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))

    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToKeep = "y", overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], "y")
    checkEquals(varsKept[[2L]], "y")
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))

    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToKeep = c("y","x"), overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], c("y","x"))
    checkEquals(varsKept[[2L]], c("y","x"))
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
  
  # This tests bug RA-5800
  XDF <- tempfile(pattern = "iris", fileext = ".xdf") 
  if (file.exists(XDF)) file.remove(XDF)
  rxDataStep(iris, XDF) 
  z1 <- rxSplitXdf(XDF, splitByFactor = "Species", varsToKeep = c("Sepal.Length", "Petal.Length", "Species"), 
    overwrite = TRUE, updateHighLow = FALSE)  
  z2 <- rxSplitXdf(XDF, splitByFactor = "Species", varsToKeep = c("Sepal.Length", "Petal.Length", "Species"), 
    overwrite = TRUE, updateHighLow = TRUE) 
  checkEquals(sapply(z1, slot, "file"), sapply(z2, slot, "file"))
  if (file.exists(XDF)) file.remove(XDF)
  invisible(sapply(z1, function(x) if (file.exists(x@file)) unlink(x@file)))
}

"test.rxSplitXdf.args.varsToDrop" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit9", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToDrop = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToDrop = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToDrop = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, varsToDrop = 1L), silent = TRUE)

    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToDrop = NULL, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], names(DF))
    checkEquals(varsKept[[2L]], names(DF))
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))
    
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToDrop = "x", overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], "y")
    checkEquals(varsKept[[2L]], "y")
    invisible(sapply(outFilePaths, function(x) if (file.exists(x)) unlink(x)))

    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, varsToDrop = "y", overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    varsKept <- lapply(outFilePaths, rxGetVarNames)
    checkEquals(varsKept[[1L]], "x")
    checkEquals(varsKept[[2L]], "x")
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.varsToDrop.all" <- function()
{
    # Currently, setting varsToDrop to all variables in the .xdf file results in all the variables
    # in the .xdf file being written to the output .xdf file, the opposite behavior from what is expected.
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit10", fileext = ".xdf")
    rxDataStep(DF, XDF)

    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
    varsToDrop = c("y","x"), overwrite = TRUE), silent = TRUE)
  file.remove(XDF)
}

"test.rxSplitXdf.args.rowSelection" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplitRW01Temp", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowSelection = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowSelection = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowSelection = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowSelection = 1L), silent = TRUE)
    
    # check number of rows after legal rowSelection
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    rowSelection <- expression(x > 3 & y > 3)
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        rowSelection = rowSelection, overwrite = TRUE, verbose = 1)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkEquals(DFS[[1L]], subset(DF[1:5,], eval(rowSelection)), check.attributes = FALSE) 
    checkEquals(DFS[[2L]], subset(DF[6:10,], eval(rowSelection)), check.attributes = FALSE) 
 
    rowSelection <- expression(x > 3 & y < 5)
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        rowSelection = rowSelection, overwrite = TRUE, verbose = 1)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.transforms" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit11", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transforms = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transforms = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transforms = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transforms = 1L), silent = TRUE)
    
    # check legal values
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    transforms <- expression(list(z = x * y))
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, transforms = transforms, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkTrue("z" %in% names(DFS[[1L]]))
    checkTrue("z" %in% names(DFS[[2L]]))
    checkEquals(DFS[[1L]][["z"]], DFS[[1L]][["x"]] * DFS[[1L]][["y"]]) 
    checkEquals(DFS[[2L]][["z"]], DFS[[2L]][["x"]] * DFS[[2L]][["y"]]) 
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.transformFunc" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit12", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformFunc = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformFunc = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformFunc = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformFunc = 1L), silent = TRUE)
    
    # check legal values
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    transformFunc <- function(dataList)
    {
        dataList[["z"]] <- dataList[["x"]] * dataList[["y"]]
        dataList
    }
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, transformFunc = transformFunc, overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkTrue("z" %in% names(DFS[[1L]]))
    checkTrue("z" %in% names(DFS[[2L]]))
    checkEquals(DFS[[1L]][["z"]], DFS[[1L]][["x"]] * DFS[[1L]][["y"]]) 
    checkEquals(DFS[[2L]][["z"]], DFS[[2L]][["x"]] * DFS[[2L]][["y"]]) 
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.transformVars" <- function()
{
  set.seed(100)
    DF <- data.frame(x=1:100, y=rep(c("a", "b", "c", "d"),25), z=rnorm(100), w=runif(100))
    XDF <- tempfile( pattern = ".rxSplit13", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformVars = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformVars = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformVars = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformVars = 1L), silent = TRUE)
    
    # check legal values
    trans <- function(data)
    {
      data$a <- data$w > 0
      data$b <- data$z*100	
      return(data)								
    }

    # test legal values
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
                               transformFunc = trans, transformVars = c("w", "z"), overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkTrue(all(c("a", "b") %in% names(DFS[[1L]])))
    checkTrue(all(c("a", "b") %in% names(DFS[[2L]])))
    checkEquals(DFS[[1L]][["a"]], DF[1:50,"w"] > 0) 
    checkEquals(DFS[[2L]][["a"]], DF[51:100,"w"] > 0)
    checkEquals(DFS[[1L]][["b"]], DF[1:50, "z"] * 100)
    checkEquals(DFS[[2L]][["b"]], DF[51:100,"z"] * 100) 
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}
  
"test.rxSplitXdf.args.transformEnvir" <- function()
{
    transformEnvir <- new.env(hash = TRUE, parent = baseenv()) 
    assign("doubleValue", function(x) x * 2, envir = transformEnvir)

    inFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    transforms <- expression(list(myAge = doubleValue(age)))
    outFilesBase <- file.path(tempdir(), "byRowsDir", "Census-byRows")
    byRowFiles <- rxSplitXdf(inFile, outFilesBase = outFilesBase,
               numOutFiles = 5, splitBy = "rows", verbose = 1,
               transforms = transforms, transformEnvir = transformEnvir, overwrite = TRUE)
  byRowFiles <- sapply(byRowFiles, slot, "file")
  
    infoByRows <- sapply(byRowFiles, rxGetVarInfo, simplify = FALSE)
    checkTrue(all(as.vector(sapply(infoByRows, function(x) "myAge" %in% names(x)))))
    
    # clean up
    invisible(sapply(byRowFiles, function(x) if (file.exists(x)) unlink(x)))
  if (file.exists(dirname(outFilesBase))) unlink(dirname(outFilesBase), recursive = TRUE)
}

"test.rxSplitXdf.args.transformPackages" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit14", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformPackages = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformPackages = character(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformPackages = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformPackages = 1L), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, transformPackages = "myNonExistentPackageNumber1"), silent = TRUE)
    
    # clean up
    invisible(sapply(XDF, function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.transformObjects" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit16", fileext = ".xdf")
    rxDataStep(DF, XDF)
    
    # check legal values
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    transforms <- expression(list(z = myFavoriteFunction(x, y)))
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, transforms = transforms, 
        transformObjects = list(myFavoriteFunction = function(x, y){x * y}), overwrite = TRUE)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    DFS <- lapply(outFilePaths, rxDataStep)
    checkTrue("z" %in% names(DFS[[1L]]))
    checkTrue("z" %in% names(DFS[[2L]]))
    checkEquals(DFS[[1L]][["z"]], DFS[[1L]][["x"]] * DFS[[1L]][["y"]]) 
    checkEquals(DFS[[2L]][["z"]], DFS[[2L]][["x"]] * DFS[[2L]][["y"]]) 
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.overwrite" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit17", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, overwrite = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, overwrite = logical(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, overwrite = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, overwrite = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, overwrite = 1L), silent = TRUE)
    
    # create output files
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/myFiles/foo", sep = "")
    outFilePaths1 <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE)
  outFilePaths1 <- sapply(outFilePaths1, slot, "file")

    # try the same test but with overwrite = FALSE: expect an error
    checkException(rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = FALSE), silent = TRUE)
    
    # try the same test but with overwrite = TRUE: expect the return paths to be the same as in the first run
    outFilePaths2 <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE)
  outFilePaths2 <- sapply(outFilePaths2, slot, "file")
    checkEquals(outFilePaths1, outFilePaths2)
    
    # clean up
    invisible(sapply(c(XDF, outFilePaths1, outFilePaths2), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.removeMissings" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    DF[c(1,5:6,10),] <- NA
    DF[3,"y"] <- NA
    XDF <- tempfile( pattern = ".rxSplit18", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, removeMissings = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, removeMissings = logical(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, removeMissings = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, removeMissings = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, removeMissings = 1L), silent = TRUE)
    
    # remove missings
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/noMissings/foo", sep = "")
    outFilePathsNoMissings <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE, removeMissings = TRUE)
  outFilePathsNoMissings <- sapply(outFilePathsNoMissings, slot, "file")
 
    # keep missings
    outFilesBase <- paste(outDir, "/Missings/foo", sep = "")
    outFilePathsMissings <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE, removeMissings = FALSE)
  outFilePathsMissings <- sapply(outFilePathsMissings, slot, "file")

    # gather and check results
    missingsDFS <- lapply(outFilePathsMissings, rxDataStep)
    noMissingsDFS <- lapply(outFilePathsNoMissings, rxDataStep)
    DF1 <- DF[1:5,]
    DF2 <- DF[6:10,]
    
    iMissing1 <- apply(DF1, MARGIN = 1L, function(x) any(is.na(x)))
    iMissing2 <- apply(DF2, MARGIN = 1L, function(x) any(is.na(x)))
    
    checkEquals(missingsDFS[[1L]], DF1, check.attributes = FALSE)
    checkEquals(missingsDFS[[2L]], DF2, check.attributes = FALSE)
    
    checkEquals(noMissingsDFS[[1L]], DF1[!iMissing1,], check.attributes = FALSE)
    checkEquals(noMissingsDFS[[2L]], DF2[!iMissing2,], check.attributes = FALSE)

  # check remove missings using splitByFactor
  irisMissing <- iris 
  irisMissing[29, 3] <- NA 
  irisMissing[53, 2] <- NA 

  irisMissingSplit <- rxSplit(irisMissing, splitByFactor = "Species", removeMissings = TRUE)
  checkEquals(nrow(irisMissingSplit[[1L]]), 49L)
  checkEquals(nrow(irisMissingSplit[[2L]]), 49L)
  checkEquals(nrow(irisMissingSplit[[3L]]), 50L)
  checkTrue(!any(is.na(unlist(irisMissingSplit))))
  
    # clean up
    invisible(sapply(c(XDF, outFilePathsMissings, outFilePathsNoMissings), function(x) if (file.exists(x)) unlink(x)))
  outNoMissingsDir <- file.path(tempdir(), "noMissings")
  outMissingsDir <- file.path(tempdir(), "Missings")
  if (file.exists(outMissingsDir)) unlink(outMissingsDir, recursive = TRUE)
  if (file.exists(outNoMissingsDir)) unlink(outNoMissingsDir, recursive = TRUE)
}

"test.rxSplitXdf.args.rowsPerRead" <- function()
{
    # create a four block data set
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit19", fileext = ".xdf")
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF)
    } else
    {
        rxDataStep(DF[1:2,], XDF)
        rxDataStep(DF[3:5,], XDF, append = "rows")
        rxDataStep(DF[6:7,], XDF, append = "rows")
        rxDataStep(DF[8:10,], XDF, append = "rows")
    }
    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowsPerRead = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowsPerRead = integer(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowsPerRead = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowsPerRead = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, rowsPerRead = c(1, 2)), silent = TRUE)
    
    # split by rows
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "rows", overwrite = TRUE, fill = "right", rowsPerRead = 1L)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    infoBlocks1 <- lapply(outFilePaths, rxGetInfo)
    if (rxIsExpressEdition())
    {
        checkEquals(infoBlocks1[[1L]][["numBlocks"]], 1L)
        checkEquals(infoBlocks1[[2L]][["numBlocks"]], 1L)
    } else
    {
        checkEquals(infoBlocks1[[1L]][["numBlocks"]], 5L)
        checkEquals(infoBlocks1[[2L]][["numBlocks"]], 5L)
    }
    
    
    # split by rows
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "rows", overwrite = TRUE, fill = "right", rowsPerRead = 2L)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    infoBlocks2 <- lapply(outFilePaths, rxGetInfo)
    if (rxIsExpressEdition())
    {
        checkEquals(infoBlocks1[[1L]][["numBlocks"]], 1L)
        checkEquals(infoBlocks1[[2L]][["numBlocks"]], 1L)
    } else
    {    
        checkEquals(infoBlocks2[[1L]][["numBlocks"]], 3L)
        checkEquals(infoBlocks2[[2L]][["numBlocks"]], 3L)
    }

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.blocksPerRead" <- function()
{
    # create a four block data set
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit19", fileext = ".xdf")
    
    #if (rxIsExpressEdition())
    #{
        #rxDataStep(inData = DF, outFile = XDF)
    #} else
    #{
        rxDataStep(DF[1:2,], XDF)
        rxDataStep(DF[3:5,], XDF, append = "rows")
        rxDataStep(DF[6:7,], XDF, append = "rows")
        rxDataStep(DF[8:10,], XDF, append = "rows")
    #}

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, blocksPerRead = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, blocksPerRead = integer(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, blocksPerRead = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, blocksPerRead = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, numOutFiles = 2L, blocksPerRead = c(1, 2)), silent = TRUE)
    
    # split by blocks
    # create output files
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "blocks", overwrite = TRUE, fill = "right", blocksPerRead = 1L)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    infoBlocks1 <- lapply(outFilePaths, rxGetInfo)
    if (rxIsExpressEdition())
    {
        checkEquals(infoBlocks1[[1L]][["numBlocks"]], 1L)
        checkEquals(infoBlocks1[[2L]][["numBlocks"]], 1L)
    } else
    {
        checkEquals(infoBlocks1[[1L]][["numBlocks"]], 2L)
        checkEquals(infoBlocks1[[2L]][["numBlocks"]], 2L)
    }
    
    
    # split by blocks
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "blocks", overwrite = TRUE, fill = "right", blocksPerRead = 2L)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    infoBlocks2 <- lapply(outFilePaths, rxGetInfo)
    checkEquals(infoBlocks2[[1L]][["numBlocks"]], 1L)
    checkEquals(infoBlocks2[[2L]][["numBlocks"]], 1L)

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.updateLowHigh" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit20", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, updateLowHigh = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, updateLowHigh = logical(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, updateLowHigh = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, updateLowHigh = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, updateLowHigh = 1L), silent = TRUE)
    
    # update low-high
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/updateLowHighFalse/foo", sep = "")
    outFilePathsLowHighFalse <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE, updateLowHigh = FALSE)
  outFilePathsLowHighFalse <- sapply(outFilePathsLowHighFalse, slot, "file")
    outFilesBase <- paste(outDir, "/updateLowHighTrue/foo", sep = "")
    outFilePathsLowHighTrue <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, overwrite = TRUE, updateLowHigh = TRUE)
  outFilePathsLowHighTrue <- sapply(outFilePathsLowHighTrue, slot, "file")
    
    infoLowHighFalse <- lapply(outFilePathsLowHighFalse, rxGetVarInfo)
    infoLowHighTrue <- lapply(outFilePathsLowHighTrue, rxGetVarInfo)
    
    xLowHighFalse <- sapply(infoLowHighFalse, function(x) x[["x"]][c("low", "high")])
    yLowHighFalse <- sapply(infoLowHighFalse, function(x) x[["y"]][c("low", "high")])
    xLowHighTrue  <- sapply(infoLowHighTrue, function(x) x[["x"]][c("low", "high")])
    yLowHighTrue  <- sapply(infoLowHighTrue, function(x) x[["y"]][c("low", "high")])
    
    checkEquals(as.vector(unlist(xLowHighFalse["low",])), c(1, 1)) # low for x no update
    checkEquals(as.vector(unlist(xLowHighTrue["low",])), c(1, 6)) # low for x with update
    checkEquals(as.vector(unlist(yLowHighFalse["low",])), c(1, 1)) # low for y no update
    checkEquals(as.vector(unlist(yLowHighTrue["low",])), c(6, 1)) # low for y with update
   
    checkEquals(as.vector(unlist(xLowHighFalse["high",])), c(10, 10)) # high for x no update
    checkEquals(as.vector(unlist(xLowHighTrue["high",])), c(5, 10)) # high for x with update
    checkEquals(as.vector(unlist(yLowHighFalse["high",])), c(10, 10)) # high for y no update
    checkEquals(as.vector(unlist(yLowHighTrue["high",])), c(10, 5)) # high for y with update

    # clean up
    invisible(sapply(c(XDF, outFilePathsLowHighFalse, outFilePathsLowHighTrue), function(x) if (file.exists(x)) unlink(x)))
    outDirTrue <- file.path(outDir, "updateLowHighTrue")
    outDirFalse <- file.path(outDir, "updateLowHighFalse")
    if (file.exists(outDirTrue)) unlink(outDirTrue, recursive = TRUE)
    if (file.exists(outDirFalse)) unlink(outDirFalse, recursive = TRUE)
}

"test.rxSplitXdf.args.reportProgress" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit21", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(suppressWarnings(rxSplitXdf(inFile = XDF, reportProgress = "")), silent = TRUE)
    checkException(suppressWarnings(rxSplitXdf(inFile = XDF, reportProgress = logical(0))), silent = TRUE)
    checkException(suppressWarnings(rxSplitXdf(inFile = XDF, reportProgress = NULL)), silent = TRUE)
    checkException(suppressWarnings(rxSplitXdf(inFile = XDF, reportProgress = pi)), silent = TRUE)
    checkException(suppressWarnings(rxSplitXdf(inFile = XDF, reportProgress = NA)), silent = TRUE)
    
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/verbose/foo", sep = "")
    outVerbose <- capture.output(outFilePathsVerbose <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, 
        numOutFiles = 2L, verbose = 0, overwrite = TRUE, reportProgress = 1L))
    
    outFilesBase <- paste(outDir, "/nonverbose/foo", sep = "")
    outNonVerbose <- capture.output(outFilePathsNonVerbose <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, 
        numOutFiles = 2L, verbose = 0, overwrite = TRUE, reportProgress = 0L))

    checkTrue(length(outNonVerbose) == 0L)
    checkTrue(length(outVerbose) > 0L)
    checkEquals(grep("Rows Processed: 5", outVerbose), 1:2)
    
    # clean up
  outFilePathsVerbose <- sapply(outFilePathsVerbose, slot, "file")
  outFilePathsNonVerbose <- sapply(outFilePathsNonVerbose, slot, "file")
    invisible(sapply(c(XDF, outFilePathsVerbose, outFilePathsNonVerbose), function(x) if (file.exists(x)) unlink(x)))
    
    outPath1 <- file.path(outDir, "verbose")
    if (file.exists(outPath1)) unlink(outPath1, recursive = TRUE)    
    
    outPath2 <- file.path(outDir, "nonverbose")
    if (file.exists(outPath2)) unlink(outPath2, recursive = TRUE)        
}

"test.rxSplitXdf.args.verbose" <- function()
{
    DF <- data.frame(x = 1:10, y = 10:1)
    XDF <- tempfile( pattern = ".rxSplit22", fileext = ".xdf")
    rxDataStep(DF, XDF)

    # check for illegal types
    checkException(rxSplitXdf(inFile = XDF, verbose = ""), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, verbose = logical(0)), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, verbose = NULL), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, verbose = pi), silent = TRUE)
    checkException(rxSplitXdf(inFile = XDF, verbose = NA), silent = TRUE)
    
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/verbose/foo", sep = "")
    outVerbose <- capture.output(outFilePathsVerbose <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, 
        numOutFiles = 2L, verbose = 1, overwrite = TRUE, reportProgress=0))
    
    outFilesBase <- paste(outDir, "/nonverbose/foo", sep = "")
    outNonVerbose <- capture.output(outFilePathsNonVerbose <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, 
        numOutFiles = 2L, verbose = 0, overwrite = TRUE, reportProgress=0))

    checkTrue(length(outNonVerbose) == 0L)
    checkTrue(length(outVerbose) > 0L)
  outFilePathsVerbose <- sapply(outFilePathsVerbose, slot, "file")
  outFilePathsNonVerbose <- sapply(outFilePathsNonVerbose, slot, "file")
    
    # clean up
    invisible(sapply(c(XDF, outFilePathsVerbose, outFilePathsNonVerbose), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.args.xdfCompressionLevel" <- function()
{
    # create a four block data set
    DF <- data.frame(x = 1:100, y = 100:1)
    XDF <- tempfile( pattern = ".rxSplit19", fileext = ".xdf")
    
    #if (rxIsExpressEdition())
    #{
        #rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    #} else
    #{
        rxDataStep(DF[1:20,], XDF)
        rxDataStep(DF[21:50,], XDF, append = "rows")
        rxDataStep(DF[51:70,], XDF, append = "rows")
        rxDataStep(DF[71:100,], XDF, append = "rows")
    #}
    # split by blocks
    # create output files
    outDir <- tempdir()
    outFilesBase <- paste(outDir, "/foo", sep = "")
    outFilePaths <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "blocks", overwrite = TRUE, fill = "right", blocksPerRead = 1L, xdfCompressionLevel = 0)
  outFilePaths <- sapply(outFilePaths, slot, "file")
    infoBlocks1 <- lapply(outFilePaths, file.info)
  
  outFilesBase2 <- paste(outDir, "/foo2", sep = "")
    outFilePaths2 <- rxSplitXdf(inFile = XDF, outFilesBase = outFilesBase, numOutFiles = 2L, 
        splitBy = "blocks", overwrite = TRUE, fill = "right", blocksPerRead = 1L, xdfCompressionLevel = 2)
  outFilePaths2 <- sapply(outFilePaths2, slot, "file")
    infoBlocks2 <- lapply(outFilePaths2, file.info)

  checkTrue(infoBlocks1[[1L]]$size > infoBlocks2[[1L]]$size)
    checkTrue(infoBlocks1[[2L]]$size > infoBlocks2[[2L]]$size)

    # clean up
    invisible(sapply(c(XDF, outFilePaths), function(x) if (file.exists(x)) unlink(x)))
  invisible(sapply(c(XDF, outFilePaths2), function(x) if (file.exists(x)) unlink(x)))
}

"test.rxSplitXdf.no.xdf.extension.repeat" <- function()
{
  XDF <- tempfile(pattern = "irisSplit", fileext = ".xdf") 
  outXDF1 <- tempfile(pattern = "irisSplitOut1", fileext = ".xdf") 
  outXDF2 <- tempfile(pattern = "irisSplitOut2", fileext = ".xdf") 
  rxDataStep(iris, XDF) 
  outFilesBase <- paste(tempdir(), "/", sep = "")
  irisXdfList <- rxSplitXdf(XDF, outFilesBase = outFilesBase, outFileSuffixes = basename(c(outXDF1, outXDF2)), 
      reportProgress = 0, verbose = 0, overwrite = TRUE)
  checkEquals(normalizePath(names(irisXdfList)), normalizePath(paste(tempdir(),"/", basename(c(outXDF1, outXDF2)), sep = "")))

  # clean up
  if (file.exists(XDF)) file.remove(XDF)
  if (file.exists(outXDF1)) file.remove(outXDF1)
  if (file.exists(outXDF2)) file.remove(outXDF2)
}

###
# rxSplit
###

"test.rxSplit.io.df.df" <- function()
{
  # DF -> DF
  DFS <- rxSplit(iris, numOut = 3, reportProgress = 0, verbose = 0)
  checkTrue(is.list(DFS))
  checkEquals(length(DFS), 3L)
  checkTrue(all(sapply(DFS, inherits, "data.frame")))
  checkTrue(all(sapply(DFS, nrow) == 50L))
  checkEquals(names(DFS), paste("iris.rows", 1:3, sep = "."))

  DFS <- rxSplit(iris, splitByFactor = "Species", reportProgress = 0, verbose = 0)
  checkTrue(is.list(DFS))
  checkEquals(length(DFS), 3L)
  checkTrue(all(sapply(DFS, inherits, "data.frame")))
  checkTrue(all(sapply(DFS, nrow) == 50L))
  checkEquals(names(DFS), paste("iris.Species", levels(iris[["Species"]]), sep = "."))
}

"test.rxSplit.io.df.xdf" <- function()
{
  # DF -> XDF
  
  outFiles <- rxSplit(iris, splitByFactor = "Species", outFilesBase = file.path(tempdir(),"irisSplit"), 
            reportProgress = 0, verbose = 0, overwrite = TRUE)
  checkTrue(is.list(outFiles))
  checkEquals(length(outFiles), 3L)
  checkTrue(all(sapply(outFiles, inherits, "RxXdfData")))
  checkEquals(basename(sapply(outFiles, slot, "file")), 
        paste("irisSplit.Species.", levels(iris[["Species"]]), ".xdf", sep = ""))
  # Cleanup
  for (i in seq_along(outFiles))
  {
    file.remove(outFiles[[i]]@file)
  }
  
}
  
"test.rxSplit.io.xdf.df" <- function()
{
  # XDF -> DF
  XDF <- tempfile(pattern = "irisSplitXDF", fileext = ".xdf")
  rxDataStep(iris, XDF)
  DFS <- rxSplit(XDF, splitByFactor = "Species", outFilesBase = "",
    reportProgress = 0, verbose = 0, overwrite = TRUE)
  checkTrue(is.list(DFS))
  checkEquals(length(DFS), 3L)
  checkTrue(all(sapply(DFS, inherits, "data.frame")))
  checkTrue(all(sapply(DFS, nrow) == 50L))
  checkEquals(names(DFS), paste(tools::file_path_sans_ext(basename(XDF)), "Species", levels(iris[["Species"]]), sep = "."))
  file.remove(XDF)
}		

"test.rxSplit.io.xdf.xdf" <- function()
{
  # XDF -> XDF
  XDF <- tempfile(pattern = "irisSplitXX", fileext = ".xdf")
  outXDF1 <- tempfile(pattern = "irisOutXX1", fileext = ".xdf")
  outXDF2 <- tempfile(pattern = "irisOutXX2", fileext = ".xdf")
  rxDataStep(iris, XDF)
  outFiles <- rxSplit(XDF, outFilesBase = tempdir(), outFileSuffixes = basename(c(outXDF1, outXDF2)), 
           reportProgress = 0, verbose = 0, overwrite = TRUE)
  checkTrue(is.list(outFiles))
  checkEquals(length(outFiles), 2L)
  checkTrue(all(sapply(outFiles, inherits, "RxXdfData")))
  checkEqualsNumeric(sapply(outFiles, function(x) rxGetInfo(x)[["numRows"]]), c(75L, 75L))
  
  # Cleanup
  for (i in seq_along(outFiles))
  {
    file.remove(outFiles[[i]]@file)
    }
    file.remove(XDF)
}

"test.rxSplit.args.inData" <- function()
{
  # DS -> DF
  XDF <- tempfile( pattern = ".rxSplitIn1", fileext = ".xdf")
  rxDataStep(iris, XDF)
  DS <- RxXdfData(XDF)
  DFS <- rxSplit(DS, splitByFactor = "Species", outFilesBase = "",
    reportProgress = 0, verbose = 0, overwrite = TRUE)
  checkTrue(is.list(DFS))
  checkEquals(length(DFS), 3L)
  checkTrue(all(sapply(DFS, inherits, "data.frame")))
  checkTrue(all(sapply(DFS, nrow) == 50L))
  file.remove(XDF)
}

"test.rxSplit.args.maxRowsByCols" <- function()
{
  # XDF -> DF
  XDF <- tempfile( pattern = ".rxSplitMaxR", fileext = ".xdf")
  rxDataStep(iris, XDF)
  DFS <- rxSplit(XDF, splitByFactor = "Species", outFilesBase = "",
          reportProgress = 0, verbose = 0, overwrite = TRUE,
           maxRowsByCols = 50L * NCOL(iris) - 1L)
  checkTrue(is.list(DFS))
  checkEquals(length(DFS), 3L)
  checkTrue(all(sapply(DFS, inherits, "data.frame")))
  checkTrue(all(sapply(DFS, nrow) == 49L))
  file.remove(XDF)
}

"test.rxSplit.io.xdf.df.input.remains" <- function()
{
  XDF <- tempfile(pattern = ".rxSplitIR", fileext = ".xdf")
  if (file.exists(XDF)) file.remove(XDF)
  rxDataStep(iris, XDF)
  checkTrue(file.exists(XDF))
  DFS <- rxSplit(XDF, splitByFactor="Species", outFilesBase = "", overwrite = TRUE) 
  checkTrue(file.exists(XDF))
  
  # clean up
  if (file.exists(XDF)) file.remove(XDF)
}

"test.rxSplit.args.outFilesBase.NULL" <- function()
{
  writeDir <- tempdir()
  inFile <- file.path(writeDir, ".rxSplitOutFile.xdf")

  if (file.exists(inFile)) file.remove(inFile)
  rxDataStep(iris, inFile)
  checkTrue(file.exists(inFile))
  
  # This should create xdf files by default
  # Write is by default to working directory, so reset it temporarily
  initialWorkingDir <- getwd()
  setwd(writeDir)
  xdfSplits <- rxSplit(inFile, splitByFactor="Species", overwrite = TRUE) 
  outFilesNames <- sapply(xdfSplits, slot, "file")
    checkEquals(length(outFilesNames), 3)
  dataFramesFromXdf <- lapply(outFilesNames, rxDataStep, reportProgress = 0L)
  
  # Clean up
    sapply(outFilesNames, unlink, recursive = TRUE)
  
  # This should create data frames
  dataFrameSplits <- rxSplit(inFile, splitByFactor = "Species", outFilesBase = "")
  tempNames <- c("setosa", "versicolor", "virginica")
  names(dataFramesFromXdf) <- tempNames
  names(dataFrameSplits) <- tempNames
  checkEquals(dataFramesFromXdf, dataFrameSplits)
    
  # This should also create data frames
  dataFrameToDF <- rxSplit(iris, splitByFactor = "Species")
  names(dataFrameToDF) <- tempNames
  checkEquals(dataFrameToDF, dataFrameSplits)
  
  # This should create .xdf Files
  outFilesBase <- file.path(writeDir, ".rxSplitOutFile")
  dataFrameToXdf <- rxSplit(iris, splitByFactor = "Species", outFilesBase = outFilesBase)
  dfFilesNames <- sapply(dataFrameToXdf, slot, "file")
    checkEquals(length(dfFilesNames), 3)
  dataFramesFromFiles <- lapply(dfFilesNames, rxDataStep, reportProgress = 0L)
  names(dataFramesFromFiles) <- tempNames
  checkEquals(dataFramesFromXdf, dataFramesFromFiles)
  
  # Clean up
    sapply(dfFilesNames, unlink, recursive = TRUE)	
  
  setwd(initialWorkingDir)
  file.remove(inFile)
  
}

