# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxTextToXdf
#
############################################################################################

"test.rxTextToXdf.args.defaultDecimalColType" <- function()
{
    # initialization
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyColMixedDecimalMissing.txt")
    outFile <- 	file.path(tempdir(), ".rxTempTiny.xdf")
    
    # explicit setting to float32
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, defaultDecimalColType = "float32")  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$b$storage, "float32")
    checkEquals(varInfoObj$c$storage, "float32")
    
    # explicit setting to numeric (float64)
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, defaultDecimalColType = "numeric")  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$b$storage, "float64")
    checkEquals(varInfoObj$c$storage, "float64")
    
    # default setting to numeric (float64) via rxOption
    oldDefaultDecimalColType <- rxGetOption("defaultDecimalColType")
  rxOptions(defaultDecimalColType = "numeric")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile)  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$b$storage, "float64")
    checkEquals(varInfoObj$c$storage, "float64")
  rxOptions(defaultDecimalColType = oldDefaultDecimalColType)

    # default setting to float32 via rxOption
    oldDefaultDecimalColType <- rxGetOption("defaultDecimalColType")
  rxOptions(defaultDecimalColType = "float32")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile)  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$b$storage, "float32")
    checkEquals(varInfoObj$c$storage, "float32")
  rxOptions(defaultDecimalColType = oldDefaultDecimalColType)

    # explicitly set an unsupported storage type
    if (file.exists(outFile)) file.remove(outFile)
    checkException(rxTextToXdf(inFile, outFile, defaultDecimalColType = "character"),
        msg = "Caught unsupported default storage type for decimals")  

    # clean up
    file.remove(outFile)
}

"test.rxTextToXdf.args.defaultMissingColType" <- function()
{
    # initialization
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyColAllMissing.txt")
    outFile <- 	file.path(tempdir(), ".rxTempTiny.xdf")
    
    # explicit setting to float32
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, defaultMissingColType = "float32")  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "float32")
    
    # explicit setting to numeric (float64)
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, defaultMissingColType = "numeric")  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "float64")
    
    # explicit setting to character)
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, defaultMissingColType = "character")  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "string")

    # default setting to numeric (float64) via rxOption
    oldDefaultMissingColType <- rxGetOption("defaultMissingColType")
  rxOptions(defaultMissingColType = "numeric")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, overwrite=TRUE)  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "float64")
  rxOptions(defaultMissingColType = oldDefaultMissingColType)

    # default setting to float32 via rxOption
    oldDefaultMissingColType <- rxGetOption("defaultMissingColType")
  rxOptions(defaultMissingColType = "float32")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, overwrite=TRUE)  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "float32")
  rxOptions(defaultMissingColType = oldDefaultMissingColType)

    # default setting to character via rxOption
    oldDefaultMissingColType <- rxGetOption("defaultMissingColType")
  rxOptions(defaultMissingColType = "character")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, overwrite=TRUE)  
  varInfoObj <- rxGetVarInfo(outFile)
    checkEquals(varInfoObj$c$storage, "string")
  rxOptions(defaultMissingColType = oldDefaultMissingColType)

    # explicitly set an unsupported storage type
    if (file.exists(outFile)) file.remove(outFile)
    checkException(rxTextToXdf(inFile, outFile, defaultMissingColType = "unsupported"),
        msg = "Caught unsupported default storage type for decimals")  

    # clean up
    file.remove(outFile)
}
      
"test.rxTextToXdf.args.inFile" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74579: Unit test test.rxDataStep.args.append.differentNumCols fails to return ERROR in output when running inside SPEES")
  
  #supply name of non-existant input file - should give an error
  inFile <- file.path(rxGetOption("unitTestDataDir"), "NonExistent.txt")
  outFile <- file.path(tempdir(),  "NonExistent.xdf")

  if (file.exists(outFile)) file.remove(outFile)

  o <- capture.output(
    expectedError <- tryCatch(
      rxTextToXdf(inFile, outFile),
      error = function(e) e)
  )
  checkTrue(inherits(expectedError,"error")) 
  #checkEquals("", expectedError$message)
  
  #Bug 76414: capture.output fails to capture entire output when run inside SPEES
  if(!(  RevoScaleR:::rxInDatabaseHostPlatform())) checkEquals(2, grep("NonExistent.txt' does not exist",o))
}			

"test.rxTextToXdf.args.outFile" <- function()
{
    # test straightforward conversion
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile)
    infoObj <- rxGetInfo(outFile)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    file.remove(outFile)
  
  # Use capital in .Xdf (RA-2780)
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <-  ".rxTempTiny.Xdf"
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile)
    infoObj <- rxGetInfo(outFile)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    file.remove(outFile)
  
    # don't supply xdf extension - should be added anyway
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <- "rxTempTinyBlearg"
    createdFile <- "rxTempTinyBlearg.xdf"
    if (file.exists(createdFile)) file.remove(createdFile)
    rxTextToXdf(inFile, outFile)
    infoObj <- rxGetInfo(createdFile)
  varInfoObj <- rxGetVarInfo(createdFile)
  checkEquals(infoObj$fileName, normalizePath(createdFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    file.remove(createdFile)
}


"test.rxTextToXdf.args.na" <- function()
{
    set.seed(100)
    myData <- data.frame(x = as.integer(seq(1:26)),y = as.integer(runif(26,0,10)),z=letters)
    myData$x[1:10] <- NA

    xdfFile <- file.path(tempdir(), ".rxTempMissing.xdf")
    txtFile <- file.path(tempdir(), ".rxTempMissing.txt")

    # Create comma-delimited txt file with missing=blank
    if (file.exists(txtFile)) file.remove(txtFile)
    write.table(myData, file=txtFile, quote=FALSE, sep=",", na=" ",row.names=FALSE, col.names=TRUE)
    # Remove xdfFile if it exists so we know we are creating a new file
    if (file.exists(xdfFile)) file.remove(xdfFile)
    # read text data with missing=blank
    rxTextToXdf(inFile=txtFile, outFile=xdfFile)
    infoblank <- rxGetInfo(xdfFile, getVarInfo=TRUE)
    checkEquals(infoblank$varInfo$x$varType,"integer")
    checkEquals(infoblank$varInfo$y$varType,"integer")
    checkEquals(infoblank$varInfo$z$varType,"character")
  datablank <- rxReadXdf( xdfFile )
  checkEquals( datablank$x, myData$x )
  checkEquals( datablank$y, myData$y )
  checkEquals( as.factor(datablank$z), myData$z )

    # create comma-delimited txt file with missing as "NA"
    if (file.exists(txtFile)) file.remove(txtFile)
    write.table(myData, file=txtFile, quote=FALSE, sep=",", na="NA", row.names=FALSE, col.names=TRUE)
    if (file.exists(xdfFile)) file.remove(xdfFile)
    # read text data with missing=NA
    rxTextToXdf(inFile=txtFile, outFile=xdfFile, rowsToSniff=15)
    infoNA <- rxGetInfo(xdfFile, getVarInfo=TRUE)
  
    #BUG RA-1504: x is being imported as character instead of integer.
    checkEquals(infoNA$varInfo$x$varType,"integer")
    checkEquals(infoNA$varInfo$y$varType,"integer")
    checkEquals(infoNA$varInfo$z$varType,"character")
  dataNA <- rxReadXdf( xdfFile )
  checkEquals( dataNA$x, myData$x )
  checkEquals( dataNA$y, myData$y )
  checkEquals( as.factor(dataNA$z), myData$z )

    # using "/t" delimited file:
    # create tab-delimited txt file with missing as blank
    if (file.exists(txtFile)) file.remove(txtFile)
    write.table(myData, file=txtFile, quote=FALSE, sep="\t", na=" ", row.names=FALSE, col.names=TRUE)
  
    # BUG RA-1505: get error that lines are not same length
    if (file.exists(xdfFile)) file.remove(xdfFile)
    rxTextToXdf(inFile=txtFile, outFile=xdfFile)
    infoblank <- rxGetInfo(xdfFile, getVarInfo=TRUE)
    checkEquals(infoblank$varInfo$x$varType,"integer")
    checkEquals(infoblank$varInfo$y$varType,"integer")
    checkEquals(infoblank$varInfo$z$varType,"character")
  datatab <- rxReadXdf( xdfFile )
  checkEquals( datatab$x, myData$x )
  checkEquals( datatab$y, myData$y )
  checkEquals( as.factor(datatab$z), myData$z )
  rm(datatab)

    # create tab-delimited txt file with missing as "NA"
    if (file.exists(txtFile)) file.remove(txtFile)
    write.table(myData, file=txtFile, quote=FALSE, sep="\t", na="NA", row.names=FALSE, col.names=TRUE)
    if (file.exists(xdfFile)) file.remove(xdfFile)
    # read text data with missing=NA
    rxTextToXdf(inFile=txtFile, outFile=xdfFile)
    infoNA <- rxGetInfo(xdfFile, getVarInfo=TRUE)
  
    #BUG RA-1504: x is being imported as character instead of integer.
    checkEquals(infoNA$varInfo$x$varType,"integer")
    checkEquals(infoNA$varInfo$y$varType,"integer")
    checkEquals(infoNA$varInfo$z$varType,"character")
  datatab <- rxReadXdf( xdfFile )
  checkEquals( datatab$x, myData$x )
  checkEquals( datatab$y, myData$y )
  checkEquals( as.factor(datatab$z), myData$z )
  if (file.exists(xdfFile)) file.remove(xdfFile)
  if (file.exists(txtFile)) file.remove(txtFile)
}

"test.rxTextToXdf.args.rowsToSkip" <- function()
{
  # Fix for bug RA-4624
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    inFile <- file.path(rxGetOption("unitTestDataDir"), "Games.dat")	
    outFile <- tempfile(pattern = ".rxTTX1TempTest", fileext = ".xdf")
  if (file.exists(outFile)) file.remove(outFile)
  
  rxTextToXdf(inFile = inFile, outFile = outFile)
  myData1 <- rxReadXdf(outFile)
  
  file.remove(outFile)
  rxTextToXdf(inFile = inFile, outFile = outFile, rowsToSkip = 5)
  myData2 <- rxReadXdf(outFile)
  checkEquals(myData1[6,], myData2[1,], check.attributes=FALSE)
  
  file.remove(outFile)
  
}

"test.rxTextToXdf.args.stringsAsFactors" <- function()
{
    # should bring in column 'c' as factor rather than string
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(
        inFile,
        outFile,
        stringsAsFactors = TRUE)
    varInfo <- rxGetVarInfo(outFile)
    checkEquals(varInfo$c$varType, "factor")
    checkEquals(varInfo$c$levels, c("one", "two", "three"))
    file.remove(outFile)
}

"test.rxTextToXdf.args.colInfo.factor.NAs" <- function()
{
    # Test the import of blanks and NAs for factors, with and without a level
    # Data file looks like this:
    #"RowNum", "MixedNA", "EmptyNA", "NANA", "BlankNA"
#1, "One", "One", "One", "One"
#2, "Two", "Two", "Two", "Two"
#3, "  ", "", NA,
#4, "   ","", NA,
#5, "One", "One", "One", "One"
#6, "   ", "", NA,
#7, ,"", NA,
#8, NA, "", NA, 
#9, "Two", "Two", "Two", "Two"
#10, "", "", NA,

    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyFactorNA.csv")
  outFile <- file.path(tempdir(), ".rxTinyFactorNA.xdf")
   
    rxTextToXdf(inFile=inFile, outFile=outFile, stringsAsFactors=TRUE, overwrite=TRUE)
    varInfo <- rxGetVarInfo(outFile)
    levelsWithNA <- c("One", "Two", "NA")
    levelsWithOther <- c("One", "Two", ".rxOther")
    levelsWithOutOther <- c("One", "Two")
    checkEquals(varInfo$MixedNA$levels, levelsWithNA)
    checkEquals(varInfo$NANA$levels, levelsWithNA)
    checkEquals(varInfo$BlankNA$levels, levelsWithOutOther)
    checkEquals(varInfo$EmptyNA$levels, levelsWithOutOther)
    # NA strings will become level, blanks & empties NA
    sumOut <- rxSummary(~MixedNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 5)
    
    if (file.exists(outFile)) file.remove(outFile)
    
    colInfoNALevel <- list(
        MixedNA=list(type="factor", levels=c("One", "Two", ".rxOther")),
        EmptyNA=list(type="factor", levels=c("One", "Two", ".rxOther")),
        NANA=list(type="factor", levels=c("One", "Two", ".rxOther")),
        BlankNA=list(type="factor", levels=c("One", "Two", ".rxOther")))
    
    rxTextToXdf(inFile=inFile, outFile=outFile, colInfo=colInfoNALevel, overwrite=TRUE)   
    
    varInfo <- rxGetVarInfo(outFile)
    checkEquals(varInfo$MixedNA$levels, levelsWithOther)
    checkEquals(varInfo$NANA$levels, levelsWithOther)
    checkEquals(varInfo$BlankNA$levels, levelsWithOther)
    checkEquals(varInfo$EmptyNA$levels, levelsWithOther) 
    sumOut <- rxSummary(~MixedNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~EmptyNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~NANA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~BlankNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    if (file.exists(outFile)) file.remove(outFile)
    
    colInfoNoNALevel <- list(
        MixedNA=list(type="factor", levels=c("One", "Two")),
        EmptyNA=list(type="factor", levels=c("One", "Two")),
        NANA=list(type="factor", levels=c("One", "Two")),
        BlankNA=list(type="factor", levels=c("One", "Two")))
 
    rxTextToXdf(inFile=inFile, outFile=outFile, colInfo=colInfoNoNALevel, overwrite=TRUE)  
    varInfo <- rxGetVarInfo(outFile)

    checkEquals(varInfo$MixedNA$levels, levelsWithOutOther)
    checkEquals(varInfo$NANA$levels, levelsWithOutOther)
    checkEquals(varInfo$BlankNA$levels, levelsWithOutOther)
    checkEquals(varInfo$EmptyNA$levels, levelsWithOutOther)
    sumOut <- rxSummary(~MixedNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 4)
    sumOut <- rxSummary(~EmptyNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 4)
    sumOut <- rxSummary(~NANA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 4)
    sumOut <- rxSummary(~BlankNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 4)
    if (file.exists(outFile)) file.remove(outFile)
    
    colInfoNoOneLevel <- list(
        MixedNA=list(type="factor", levels=c("Two", ".rxOther")),
        EmptyNA=list(type="factor", levels=c("Two", ".rxOther")),
        NANA=list(type="factor", levels=c("Two", ".rxOther")),
        BlankNA=list(type="factor", levels=c("Two", ".rxOther")))

    rxTextToXdf(inFile=inFile, outFile=outFile, colInfo=colInfoNoOneLevel, overwrite=TRUE)  
    varInfo <- rxGetVarInfo(outFile)
    
    sumOut <- rxSummary(~MixedNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~EmptyNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~NANA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)
    sumOut <- rxSummary(~BlankNA, data=outFile, byTerm=FALSE)
    checkEquals(sumOut$nobs.valid, 10)

    levelsWithOutOne <- c("Two", ".rxOther")
    checkEquals(varInfo$MixedNA$levels, levelsWithOutOne)
    checkEquals(varInfo$NANA$levels, levelsWithOutOne)
    checkEquals(varInfo$BlankNA$levels, levelsWithOutOne)
    checkEquals(varInfo$EmptyNA$levels, levelsWithOutOne)
    
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.colInfo.levels.integer" <- function()
{
  # read in a sample data set
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")	
  outFile <- file.path(tempdir(), ".rxTTX2TempTest100r.xdf")

  if (file.exists(outFile)) file.remove(outFile)
  readLines(inFile, 5)
  
  rxTextToXdf(inFile=inFile, outFile=outFile, 
    colInfo=list(xint1=list(type="integer", levels=as.character(c(1:10)))))
  varInfo <- rxGetVarInfo(outFile)
  
  # Levels should be ignored
  checkEquals(varInfo$xint1$varType, "integer")
  myDF <- rxReadXdf(outFile)
  # xfac1 will be imported as character data by default (it is in quotes)
  checkEquals(myDF$xint1, as.integer(myDF$xfac1), checkNames=FALSE)
  
  if (file.exists(outFile)) file.remove(outFile)
  
  rxTextToXdf(inFile=inFile, outFile=outFile, 
    colInfo=list(xint1=list(type="integer", levels=as.character(c(1:10))),
      xfac1 = list(type="integer"))
    )
  varInfo <- rxGetVarInfo(outFile)
  
  checkEquals(varInfo$xint1$varType, "integer")
  # Both xfac1 and xint1 should be integers (and the same)
  myDF <- rxReadXdf(outFile)
  checkEquals(myDF$xint1, myDF$xfac1, checkNames=FALSE)
  
  if (file.exists(outFile)) file.remove(outFile)
  
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 500
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_1Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall1st10kOf6.csv") # 1st 10,000 of the 600,000 rows in AirlineDemoSmall.csv
    outFile <- file.path(tempdir(), ".rxTempADS.xdf")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
                colInfo=list(DayOfWeek=list(type="factor")))

    # Read 10 rows starting from 500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-2L, -6L, -12L, 5L, -1L, -41L, 2L, 63L, 27L, 65L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(17.5833339691162, 9, 12.3333330154419, 15.6666660308838, 8, 16.5833339691162, 14.6666660308838, 19.5833339691162, 7.3333330154419, 17.25))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -42, high = 329), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 6, high = 22), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 4, levels = c("Monday", "Tuesday", "Wednesday", "Thursday" )), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 1500
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_2Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall2nd10kOf6.csv") # 2nd 10,000 of the 600,000 rows in AirlineDemoSmall.csv
    outFile <- file.path(tempdir(), ".rxTempADS.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor")))
                          
    # Read 10 rows starting from 1500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 1500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10001)
  checkEquals(infoObj$numVars, 3)
      if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 11)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(45L, 2L, -5L, -16L, 11L, NA, 15L, 1L, 2L, -7L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(18.5499992370605, 9.66666603088379, 15.4166669845581, 
        15.8500003814697, 14.5833339691162, 18.9833335876465, 14.2499990463257, 20.4166660308838, 
        16.4166660308838, 17.5833339691162))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(2L, 3L, 1L, 7L, 2L, 2L, 1L, 6L, 1L, 4L), 
        .Label = c("Saturday", "Friday", "Monday", "Wednesday", "Sunday", "Thursday", "Tuesday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -60, high = 728), 
        .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", 
        low = 1.58333301544189, high = 23.9833335876465), .Names = c("varType", "storage", "low", "high"), 
        class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, 
        levels = c("Saturday", "Friday", "Monday", "Wednesday", "Sunday", "Thursday", "Tuesday")), 
        .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 3500
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_3Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall3rd10kOf6.csv") # 3rd 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
  
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor")))
                          
    # Read 10 rows starting from 3500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 3500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-13L, -11L, 5L, -5L, -20L, 8L, -12L, -15L, -10L, 10L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(7L, 1L, 2L, 4L, 5L, 6L, 7L, 1L, 2L, 4L), .Label = c("Thursday", "Friday", "Saturday", "Sunday", "Monday", "Tuesday", "Wednesday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -56, high = 777), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 5.6666669845581, high = 23.6666660308838), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Thursday", "Friday", "Saturday", "Sunday", "Monday", "Tuesday", "Wednesday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 5000
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_4Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall4th10kOf6.csv") # 4th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
  
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor")))
                                                          
    # Read 10 rows starting from 5000.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 5000)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-11L, -10L, -28L, -30L, 96L, 41L, -24L, -11L, 7L, 105L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465, 16.9833335876465))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(2L, 1L, 3L, 4L, 5L, 6L, 2L, 1L, 3L, 4L), .Label = c("Monday", "Sunday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -51, high = 819), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 0.5, high = 23.9166660308838), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Sunday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 7500
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_5Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall5th10kOf6.csv") # 5th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
 
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor")))

    # Read 10 rows starting from 7500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 7500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(5L, 2L, -5L, 41L, -5L, 12L, 2L, 9L, -2L, -7L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(18.4999980926514, 18.4999980926514, 18.4999980926514, 18.4999980926514, 18.4999980926514, 18.4999980926514, 18.4999980926514, 18.4999980926514, 8.89999961853027, 8.89999961853027))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(4L, 5L, 6L, 1L, 2L, 7L, 3L, 4L, 4L, 5L), .Label = c("Thursday", "Friday", "Sunday", "Monday", "Tuesday", "Wednesday", "Saturday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -49, high = 1195), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 0.0166669990867376, high = 23.8333339691162), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Thursday", "Friday", "Sunday", "Monday", "Tuesday", "Wednesday", "Saturday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 9500
"test.rxTextToXdf.args.colInfo.airlineDemoSmall_6Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall6th10kOf6.csv") # 6th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor")))
    
    # Read 10 rows starting from 9500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 9500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(16L, 67L, -2L, 44L, 28L, 5L, -3L, 15L, 195L, 34L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(6L, 7L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 1L), .Label = c("Saturday", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -58, high = 1370), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 5.33333301544189, high = 23.1666679382324), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Saturday", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only explicit factor levels in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 500
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_1Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall1st10kOf6.csv") # 1st 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
   
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor", 
        levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))))

    # Re ad 10 rows starting from 500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
    
  checkEquals(infoObj$data[["ArrDelay"]], c(-2L, -6L, -12L, 5L, -1L, -41L, 2L, 63L, 27L, 65L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(17.5833339691162, 9, 12.3333330154419, 15.6666660308838, 8, 16.5833339691162, 14.6666660308838, 19.5833339691162, 7.3333330154419, 17.25))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -42, high = 329), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 6, high = 22), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 1500
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_2Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall2nd10kOf6.csv") # 2nd 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
    
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor", 
            levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))))

    # Read 10 rows starting from 1500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 1500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10001)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 11)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(45L, 2L, -5L, -16L, 11L, NA, 15L, 1L, 2L, -7L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(18.5499992370605, 9.66666603088379, 15.4166669845581, 15.8500003814697, 14.5833339691162, 18.9833335876465, 14.2499990463257, 20.4166660308838, 16.4166660308838, 17.5833339691162))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(5L, 1L, 6L, 2L, 5L, 5L, 6L, 4L, 6L, 3L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -60, high = 728), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 1.58333301544189, high = 23.9833335876465), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 3500
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_3Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall3rd10kOf6.csv") # 3rd 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor", 
        levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))))

    # Read 10 rows starting from 3500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 3500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-13L, -11L, 5L, -5L, -20L, 8L, -12L, -15L, -10L, 10L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75, 8.75))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(3L, 4L, 5L, 7L, 1L, 2L, 3L, 4L, 5L, 7L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -56, high = 777), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 5.6666669845581, high = 23.6666660308838), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

# Extract and compare values out of 10 rows starting at row# 5000
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_4Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall4th10kOf6.csv") # 4th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
  outFile <- file.path(tempdir(), ".rxTempADS.xdf")
  
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor",
        levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))))

    # Read 10 rows starting from 5000.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 3500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-6L, -3L, -5L, -1L, 35L, -19L, 3L, -28L, -32L, -15L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621, 8.08333396911621))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -51, high = 819), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 0.5, high = 23.9166660308838), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

# Extract and compare values out of 10 rows starting at row# 7500
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_5Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall5th10kOf6.csv") # 5th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
    outFile <- file.path(tempdir(), ".rxTempADS.xdf")
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
        colInfo=list(DayOfWeek=list(type="factor", 
        levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))))

    # Read 10 rows starting from 7500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 3500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(-21L, -23L, 31L, -2L, 21L, -23L, 2L, -1L, -27L, -31L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(9.4166669845581, 9.4166669845581, 9.4166669845581, 18.8666667938232, 18.8666667938232, 18.8666667938232, 18.8666667938232, 18.8666667938232, 18.8666667938232, 18.8666667938232))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(6L, 7L, 1L, 6L, 7L, 1L, 2L, 3L, 4L, 5L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -49, high = 1195), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 0.0166669990867376, high = 23.8333339691162), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

# Convert a 10,000 row CSV text file to XDF using only the column name in 'colInfo' parameter list.
# Extract and compare values out of 10 rows starting at row# 9500
"test.rxTextToXdf.args.colInfo.factor.airlineDemoSmall_6Of6" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall6th10kOf6.csv") # 6th 10,000 of the 600,000 rows in AirlineDemoSmall.csv
    outFile <- file.path(tempdir(), ".rxTempADS.xdf")
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, missingValueString = "M",
                    colInfo=list(DayOfWeek=list(type="factor", 
                            levels=c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"))),
            overwrite=TRUE)
                                                  
    # Read 10 rows starting from 9500.
    infoObj <- rxGetInfo(outFile, numRows = 10, startRow = 9500)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 10000)
  checkEquals(infoObj$numVars, 3)
    if (rxIsExpressEdition())
    {
      checkEquals(infoObj$numBlocks, 1)
    } else
    {
        checkEquals(infoObj$numBlocks, 10)
    }
  checkEquals(infoObj$data[["ArrDelay"]], c(16L, 67L, -2L, 44L, 28L, 5L, -3L, 15L, 195L, 34L))
  checkEquals(infoObj$data[["CRSDepTime"]], c(12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419, 12.8333330154419))
  checkEquals(infoObj$data[["DayOfWeek"]], structure(c(4L, 5L, 6L, 7L, 1L, 2L, 3L, 4L, 5L, 6L), .Label = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"), class = "factor"))
  checkEquals(varInfoObj[["ArrDelay"]], structure(list(varType = "integer", storage = "int32", low = -58, high = 1370), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["CRSDepTime"]], structure(list(varType = "numeric", storage = "float32", low = 5.33333301544189, high = 23.1666679382324), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["DayOfWeek"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 7, levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.firstRowIsColNames" <- function()
{
    # Bug RA-270 
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")

  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE)

    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.colClasses" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
    if (file.exists(outFile)) file.remove(outFile)  

    # try with various data types - check to see that
    # xdf contains the correct equivalent type

    # here, colClasses is telling the conversion to use types
    # that it would use by default anyway... but worth testing that
    # it all works
    rxTextToXdf(inFile, outFile, colClasses=c(a="int32",b="float32",c="string"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
  # Test all of the integer types to make sure the data is valid
  # Regression test for RA-2378
  
  rxTextToXdf(inFile, outFile, colClasses=c(a="integer"), overwrite=TRUE)
  var.int32 <- rxReadXdf(outFile)$a
  rxTextToXdf(inFile, outFile, colClasses=c(a="int16"), overwrite=TRUE)
  var.int16 <- rxReadXdf(outFile)$a
  rxTextToXdf(inFile, outFile, colClasses=c(a="uint16"), overwrite=TRUE)
  var.uint16 <- rxReadXdf(outFile)$a
  
  checkEquals(var.int32, var.int16)
  checkEquals(var.int32, var.uint16)
  if (file.exists(outFile)) file.remove(outFile)
  
    # try all the different target types - verify that we are getting the xdf types we expect
    rxTextToXdf(inFile, outFile, colClasses=c(a="logical",b="integer",c="int32"), overwrite=TRUE)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "logical", storage = "uchar", low = 1, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "integer", storage = "int32", low = 4, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "integer", storage = "int32", low = NA_real_, high = NA_real_), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, colClasses=c(a="numeric",b="float64",c="character"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "numeric", storage = "float64", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float64", low = 4.1, high = 6.3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    rxTextToXdf(inFile, outFile, colClasses=c(a="string",b="raw",c="uchar"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "logical", storage = "uchar", low = as.numeric(1), high = as.numeric(1)), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "logical", storage = "uchar", low = as.numeric(NA), high = as.numeric(NA)), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    rxTextToXdf(inFile, outFile, colClasses=c(a="uint8",b="factor",c="int16"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "logical", storage = "uchar", low = 1, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 3, levels = c("4.1", "5.2", "6.3")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "integer", storage = "int16", low = NA_real_, high = NA_real_), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, colClasses=c(a="uint16",b="uint32",c="float32"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "uint16", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  # Uints are always factors
  checkEquals(varInfoObj[["b"]], structure(list(varType = "factor", storage = "uint32", low = 5, high = 7, levels = character(0)), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "numeric", storage = "float32", low = NA_real_, high = NA_real_), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
      
    # try some 'R' friendly types and type names
    rxTextToXdf(inFile, outFile, colClasses=c(a="numeric",b="numeric",c="character"), overwrite = TRUE)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "numeric", storage = "float64", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float64", low = 4.1, high = 6.3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # verify that colClasses overrides stringsAsFactors
    rxTextToXdf(inFile, outFile, stringsAsFactors = TRUE, 
        colClasses=c(a="numeric",b="numeric",c="character"), overwrite=TRUE)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "numeric", storage = "float64", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float64", low = 4.1, high = 6.3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    #TODO: test some error cases
}

"test.rxTextToXdf.args.colInfo.newName" <- function()
{
  # Multiple blocks
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  outFile <- file.path(tempdir(), ".rxTemp100.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
  testColInfo <- list(ylogical=list(newName="MyLogicalY"), xnum2=list(newName="MyXnum2"))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=25, overwrite=TRUE)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[2], "MyLogicalY")
  checkEquals(varNames[5], "MyXnum2")
  if (file.exists(outFile)) file.remove(outFile)
  rm(varNames)
  
  # Multiple blocks with hidden transform function
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=25, overwrite=TRUE,
     transforms=list(MyNewX = MyXnum2 + 7.0))
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[2], "MyLogicalY")
  checkEquals(varNames[5], "MyXnum2")
  checkEquals(varNames[10], "MyNewX")
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$MyNewX$low, (varInfoObj$MyXnum2$low + 7.0))
  
  # Variations on a small file
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <- ".rxTempTiny.xdf"
    if (file.exists(outFile)) file.remove(outFile)
  
  rxTextToXdf(inFile, outFile)
  varInfoObj <- rxGetVarInfo(outFile)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("a", "b", "c"))
      
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), c=list(newName="Cleveland"))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, overwrite=TRUE)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  rm(varNames)
  
  # Test with additional colInfo
  testColInfo <- list(a=list(newName="Atlanta", type="numeric"), b=list(newName="Boston"), 
    c=list(newName="Cleveland", type="factor", levels=c("One", ".rxOther")))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, overwrite=TRUE)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Atlanta$varType, "numeric")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  rm(varInfoObj)
  
  # Test with colClasses
  testColClasses <- c(a="numeric")
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), 
  c=list(newName="Cleveland", type="factor", levels=c("One", "Two")))
  rxTextToXdf(inFile, outFile, colClasses=testColClasses, colInfo=testColInfo, overwrite=TRUE)
  
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Atlanta$varType, "numeric")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  
  # Test with colClasses - different variable
  testColClasses <- c(b="integer")
  testColInfo <- list(a=list(newName="Atlanta"), b=list(newName="Boston"), 
  c=list(newName="Cleveland", type="factor", levels=c("One", "Two")))
  rxTextToXdf(inFile, outFile, colClasses=testColClasses, colInfo=testColInfo, overwrite=TRUE)
  
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("Atlanta", "Boston", "Cleveland"))
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$Boston$varType, "integer")
  checkEquals(varInfoObj$Cleveland$varType, "factor")
  rm(varNames)
  
  # Test with just one variable
  testColInfo <- list(b=list(newName="Boston"))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, overwrite=TRUE)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames, c("a", "Boston", "c"))
  
  if (file.exists(outFile)) file.remove(outFile)  
}

"test.rxTextToXdf.args.colInfo.newLevels" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  outFile <- file.path(tempdir(), ".rxTemp100.xdf")

    if (file.exists(outFile)) file.remove(outFile)
      
  origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
  newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
  testColInfo <- list(xfac2=list(type="factor",
    levels=origLevels,
        newLevels=newLevels))

  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=50, overwrite=TRUE)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$xfac2$levels, newLevels)
  
  testColInfo <- list(xfac2=list(newName="MyFactor", type="factor",
    levels=origLevels,
        newLevels=newLevels), xfac1=list(newName="MyOtherFactor"))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=50, overwrite=TRUE)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$MyFactor$levels, newLevels)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[8], "MyOtherFactor")
  checkEquals(varNames[9], "MyFactor")
  
  if (file.exists(outFile)) file.remove(outFile) 
  
  # Use newLevels with transformFunc and newName(RA-2777)
  # Do simple import for comparison
  inFile <- file.path(rxGetOption("sampleDataDir"),"claims.txt")
  rxTextToXdf(inFile=inFile, outFile=outFile, 
    overwrite=TRUE, rowsPerRead = 20, stringsAsFactors=TRUE)
  fileInfoSimple <- rxGetInfo( outFile )
  if (file.exists(outFile)) file.remove(outFile) 
  
  xform <- function(data)
  {
    data$TYPENew <- data$TYPE=="aa"
    return(data)
  }

  newLevels <- c("aa","bb","cc", "dd")
    rxTextToXdf(file.path(rxGetOption("sampleDataDir"),"claims.txt"),outFile , 
        colInfo=list(type=list(newName="TYPE", levels = c("A", "B", "C", "D"),
      newLevels=newLevels)),
        transformVars=c("TYPE"),transformFunc=xform,overwrite=TRUE, 
    rowsPerRead = 20, stringsAsFactors=TRUE)

  fileInfo <- rxGetInfo( outFile )
  checkEquals(fileInfo$numRows, fileInfoSimple$numRows)
  checkEquals(fileInfo$numVars, fileInfoSimple$numVars + 1)
  checkEquals(fileInfo$numBlocks, fileInfoSimple$numBlocks)
  varInfo <- rxGetVarInfo(outFile)
  checkEquals(varInfo$TYPE$levels, newLevels)
  checkEquals(varInfo$TYPENew$varType, "logical")
  if (file.exists(outFile)) file.remove(outFile) 

}

"test.rxTextToXdf.args.colInfo.description" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  outFile <- file.path(tempdir(), ".rxTemp100.xdf")

    if (file.exists(outFile)) file.remove(outFile)
      
  origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
  newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
  newDescription <- "This is my favorite variable."
  testColInfo <- list(xfac2=list(type="factor",
    levels = origLevels,
        newLevels = newLevels, 
    description = newDescription))

  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=50, overwrite=TRUE)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$xfac2$levels, newLevels)
  checkEquals(varInfoObj$xfac2$description, newDescription)
  rm(varInfoObj)
  
  newDescription2 = "My second favorite variable."
  testColInfo <- list(xfac2=list(newName="MyFactor", type="factor",
    description = newDescription,
    levels=origLevels,
        newLevels=newLevels), 
      xfac1=list(newName="MyOtherFactor", description = newDescription2))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, rowsPerRead=50, overwrite=TRUE)
  varNames <- rxGetVarNames(outFile)
  checkEquals(varNames[8], "MyOtherFactor")
  checkEquals(varNames[9], "MyFactor")
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$MyFactor$levels, newLevels)
  checkEquals(varInfoObj$MyFactor$description, newDescription)
  checkEquals(varInfoObj$MyOtherFactor$description, newDescription2)

  if (file.exists(outFile)) file.remove(outFile)  
}

"test.rxTextToXdf.args.colInfo.stringsAsFactors" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  outFile <- file.path(tempdir(), ".rxTemp100.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
  # Start with default values
  rxTextToXdf(inFile=inFile, outFile=outFile, rowsPerRead = 50, overwrite=TRUE,
    reportProgress = 0)
  
  varInfo <- rxGetVarInfo( outFile )
  checkEquals(varInfo$ylogical$varType, "character")
  checkEquals(varInfo$xfac1$varType, "character")
  checkEquals(varInfo$xnum1$varType, "numeric")
  checkEquals(varInfo$xint1$varType, "integer")
  checkEquals(varInfo$xfac2$varType, "character")
  
  if (file.exists(outFile)) file.remove(outFile)
  
  # Set stringsAsFactors to TRUE
  rxTextToXdf(inFile=inFile, outFile=outFile, rowsPerRead = 50, 
    stringsAsFactors = TRUE, overwrite=TRUE, reportProgress = 0)
  
  varInfo <- rxGetVarInfo( outFile ) 
  checkEquals(varInfo$ylogical$varType, "factor")
  checkEquals(varInfo$xfac1$varType, "factor")
  checkEquals(varInfo$xnum1$varType, "numeric")
  checkEquals(varInfo$xint1$varType, "integer")
  checkEquals(varInfo$xfac2$varType, "factor")
  
  if (file.exists(outFile)) file.remove(outFile)
  
  # Use colClasses to override stringsAsFactors
  colClasses = c(ylogical = "logical", xfac1 = "character", xint1 = "factor")
  rxTextToXdf(inFile=inFile, outFile=outFile, rowsPerRead = 50,
    colClasses = colClasses,
    stringsAsFactors = TRUE, overwrite=TRUE, reportProgress = 0)
  
  varInfo <- rxGetVarInfo( outFile ) 
  checkEquals(varInfo$ylogical$varType, "logical")
  checkEquals(varInfo$xfac1$varType, "character")
  checkEquals(varInfo$xnum1$varType, "numeric")
  checkEquals(varInfo$xint1$varType, "factor")
  checkEquals(varInfo$xfac2$varType, "factor")
  
  if (file.exists(outFile)) file.remove(outFile)
  
  # Use colInfo to overrised colClasses to override stringsAsFactors
  colClasses = c(ylogical = "logical", xfac1 = "character", xint1 = "factor")
  colInfo = list(xfac1 = list(type = "factor", levels = letters[1:10]))
  rxTextToXdf(inFile=inFile, outFile=outFile, rowsPerRead = 50,
    colClasses = colClasses, colInfo = colInfo,
    stringsAsFactors = TRUE, overwrite=TRUE, reportProgress = 0)
  
  varInfo <- rxGetVarInfo( outFile ) 
  checkEquals(varInfo$ylogical$varType, "logical")
  checkEquals(varInfo$xfac1$varType, "factor")
  checkEquals(varInfo$xfac1$levels, letters[1:10])
  checkEquals(varInfo$xnum1$varType, "numeric")
  checkEquals(varInfo$xint1$varType, "factor")
  checkEquals(varInfo$xfac2$varType, "factor")
  
  if (file.exists(outFile)) file.remove(outFile)
  
}

"test.rxTextToXdf.args.colInfo.type" <- function()
{
  # Regression test for RA-1917
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
    # try with various data types - check to see that
    # xdf contains the correct equivalent type

  rxTextToXdf(inFile, outFile)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$a$varType, "integer")
  checkEquals(varInfoObj$b$varType, "numeric")
  checkEquals(varInfoObj$b$storage, "float32")
  checkEquals(varInfoObj$c$varType, "character")
  
  testColInfo <- list(a=list(type="numeric"), b=list(type="integer"), c=list(type="integer"))
  rxTextToXdf(inFile, outFile, colInfo=testColInfo, overwrite=TRUE)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(varInfoObj$a$varType, "numeric")
  checkEquals(varInfoObj$a$storage, "float64")
  checkEquals(varInfoObj$b$varType, "integer")
  checkEquals(varInfoObj$c$varType, "integer")
  
  if (file.exists(outFile)) file.remove(outFile)  
}

"test.rxTextToXdf.args.colInfo.variety" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
   
    if (file.exists(outFile)) file.remove(outFile)
  
    # try with various data types - check to see that
    # xdf contains the correct equivalent type
  
    # colInfo can used to only to provide factor conversion and levels information
    rxTextToXdf(inFile, outFile, colInfo=list(c=list(type="factor", levels=c("one","two","three"))))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 3, levels = c("one", "two", "three")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)  

    # check that colInfo overrides colClasses
    rxTextToXdf(inFile, outFile, colClasses=c(c="integer"),
        colInfo=list(c=list(type="factor", levels=c("one","two","three"))))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 3, levels = c("one", "two", "three")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # check that missings are inserted in factor column, if string not in supplied levels
    rxTextToXdf(inFile, outFile, colInfo=list(c=list(type="factor", levels=c("one","three"))))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("one", "three")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # check that if no levels are supplied, they are created automatically from column data
    rxTextToXdf(inFile, outFile, colInfo=list(c=list(type="factor")))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 3, levels = c("one", "two", "three")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)  
}

"test.rxTextToXdf.columnDelimiters.Comma" <- function()
{
  # try textType 'Comma'
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt") # comma delimited
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")

  if (file.exists(outFile)) file.remove(outFile)
  rxTextToXdf(inFile,	outFile, columnDelimiters=",")
    
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)
  
  # Do the same, auto-detecting delimiter
  rxTextToXdf(inFile,	outFile)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.columnDelimiters.tab" <- function()
{
    # try textType 'Tab'
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyTab.txt") # tab delimited
  outFile <- file.path(tempdir(), ".rxTempTinyTab.xdf")
  
  if (file.exists(outFile)) file.remove(outFile)
  rxTextToXdf( inFile, outFile, columnDelimiters="\t")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)
  
  # Repeat, auto-detecting column delimiter
  rxTextToXdf(inFile, outFile)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.columnDelimiters.User" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt") # hash delimited
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
  
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile,columnDelimiters = "#")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.columnDelimiters.multiple" <- function()
{
    # providing columnDelimiters should cause
    # them to be used
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt") # hash delimited
  outFile <- file.path(tempdir(), ".rxTempTinyHash.xdf")
  
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, columnDelimiters = "#")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # supplying a few delimiters should work fine, so long as they include the correct one
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt") # hash delimited
    outFile <- ".rxTempTinyHash.xdf"
    if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, columnDelimiters = "(#|")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
 
    # try a mixed delimiter file
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyMixed.txt") # hash and tilde delimited
    outFile <-  ".rxTempTinyMixed.xdf"
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, columnDelimiters = "~#")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # try a mixed whitespace delimiter file 
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyMixedWhite.txt") # space and tab delimited
    outFile <-  ".rxTempTinyMixedWhite.xdf"
    if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, columnDelimiters = " \t")# space and tab
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)     
}

"test.rxTextToXdf.args.colInfo.counties" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "counties.csv")
  outFile <- file.path(tempdir(), ".rxTempCounties.xdf")
  if (file.exists(outFile)) file.remove(outFile)									        
    rxTextToXdf(inFile, outFile, colInfo=
        list(county=list(type="factor",levels=c("Barbour","Bullock","San Bernardino","San Diego","Kitsap")),
            state=list(type="factor",levels=c("AL","CA","WA"))))	
  counties.df <- rxReadXdf(file=outFile)			
    counties.nrow <- dim(counties.df)[1]
  counties.ncol <- dim(counties.df)[2]			
  infoObj <- rxGetInfo(outFile)
  
  # basic checks to ensure correct values have been imported.			
  checkEquals(is.na(counties.df[21,1]),TRUE)												
  checkEquals(as.character(counties.df[224,2]),"CA")																										
  checkEqualsNumeric(counties.df[18,7],-11.2, tolerance=.001)			
  
  # check that XDF file has same dimensions as imported data frame.			
  checkEquals(infoObj$numRows, counties.nrow)
  checkEquals(infoObj$numVars, counties.ncol)
  
  ### Check to verify that the XDF file contains the correct column types.			
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3141)
  checkEquals(infoObj$numVars, 19)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["county"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 5, levels = c("Barbour", "Bullock", "San Bernardino", "San Diego", "Kitsap")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["state"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 3, levels = c("AL", "CA", "WA")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["msa"]], structure(list(varType = "integer", storage = "int32", low = 40, high = 9360), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["pmsa"]], structure(list(varType = "integer", storage = "int32", low = 80, high = 9270), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["pop.density"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 52432), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["pop"]], structure(list(varType = "integer", storage = "int32", low = 52, high = 8863164), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["pop.change"]], structure(list(varType = "numeric", storage = "float32", low = -34.4000015258789, high = 207.699996948242), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["age6574"]], structure(list(varType = "numeric", storage = "float32", low = 0.600000023841858, high = 21.1000003814697), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["age75"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 15.8999996185303), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["crime"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 20179), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["college"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 53.4000015258789), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["income"]], structure(list(varType = "integer", storage = "int32", low = 10903, high = 65201), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["farm"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 67.5999984741211), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["democrat"]], structure(list(varType = "numeric", storage = "float32", low = 6.80000019073486, high = 84.5999984741211), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["republican"]], structure(list(varType = "numeric", storage = "float32", low = 9.10000038146973, high = 75), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["Perot"]], structure(list(varType = "numeric", storage = "float32", low = 3.20000004768372, high = 46.9000015258789), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["white"]], structure(list(varType = "numeric", storage = "float32", low = 5.03938579559326, high = 100), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["black"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 86.2359848022461), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["turnout"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 101.927192687988), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
  if (file.exists(outFile)) file.remove(outFile)			    
}		

"test.rxTextToXdf.args.colClasses.integerToFactor" <- function()
{
  # read in a sample data set
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")	
  outFile <- file.path(tempdir(), ".rxTTX3TempTest100r.xdf")
  
  if (file.exists(outFile)) file.remove(outFile)
  rxTextToXdf(inFile=inFile, outFile=outFile, colClasses=c(xint1="factor"),
    stringsAsFactors=TRUE)
  varInfo <- rxGetVarInfo(outFile)
  checkEquals(varInfo$xint1$varType, "factor")
  myDF <- rxReadXdf(outFile)
  checkEquals(myDF$xint1, myDF$xfac1, checkNames=FALSE)
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.append" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    inFile2 <- file.path(rxGetOption("unitTestDataDir"), "tiny2.txt")
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
    # using default data types
    rxTextToXdf(inFile, outFile, overwrite=TRUE)
    rxTextToXdf(inFile2, outFile, append="rows") # append to output xdf
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.missings" <- function()
{
  # Reactivated RA-1817 fixed Missing data causes wrong import data type with append

  tempCsv1 <- file.path(tempdir(), ".rxTempCsv1.txt")
  tempCsv2 <- file.path(tempdir(), ".rxTempCsv2.txt")
  tempXdf  <- file.path(tempdir(), ".rxTempImport.xdf")
  
  if (file.exists(tempCsv1)) file.remove(tempCsv1)
  if (file.exists(tempCsv2)) file.remove(tempCsv2)
  if (file.exists(tempXdf)) file.remove(tempXdf)

  myDF1 <- data.frame(x=1:20, y=1:20)
  myDF2 <- data.frame(x=21:40, y=c(rep(NA, 10), 31:40))
  write.csv(myDF1, file=tempCsv1, row.names=FALSE)
  write.csv(myDF2, file=tempCsv2, row.names=FALSE)
  
  rxTextToXdf(inFile=tempCsv1, outFile=tempXdf)
  rxTextToXdf(inFile=tempCsv2, outFile=tempXdf, append="rows")
  rxTextToXdf(inFile=tempCsv1, outFile=tempXdf, append="rows")
  myDF <- rxReadXdf(tempXdf)
  
  checkEquals(myDF$x[1:20], myDF$y[1:20])
  checkEquals(myDF$x[31:40], myDF$y[31:40])
  checkEquals(myDF$x[41:60], myDF$y[41:60])
  if (file.exists(tempCsv1)) file.remove(tempCsv1)
  if (file.exists(tempCsv2)) file.remove(tempCsv2)
  if (file.exists(tempXdf)) file.remove(tempXdf)	
}

"test.rxTextToXdf.args.different.types" <- function()
{
  # Reactivated RA-1817 fixed Missing data causes wrong import data type with append
    #DEACTIVATED("Pending fix to RA-1817")
  tempCsv1 <- file.path(tempdir(), ".rxTempCsv1.txt")
  tempCsv2 <- file.path(tempdir(), ".rxTempCsv2.txt")
  tempXdf  <- file.path(tempdir(), ".rxTempImport.xdf")
  
  if (file.exists(tempCsv1)) file.remove(tempCsv1)
  if (file.exists(tempCsv2)) file.remove(tempCsv2)
  if (file.exists(tempXdf)) file.remove(tempXdf)

  myDF1 <- data.frame(x=1:20, y=1:20)
  myDF2 <- data.frame(x=21:40, y= seq(from=21.0, by=.5, length.out=20))
  write.csv(myDF1, file=tempCsv1, row.names=FALSE)
  write.csv(myDF2, file=tempCsv2, row.names=FALSE)
  
  rxTextToXdf(inFile=tempCsv1, outFile=tempXdf)
    
    # Should get error message appending floats to integers
  res <- try(rxTextToXdf(inFile=tempCsv2, outFile=tempXdf, append="rows"), silent=TRUE)
    checkTrue(is(res,"try-error"))
    fileInfo <- rxGetInfo(tempXdf)
    checkEquals(fileInfo$numRows, 20)
    
    # But should work of we are explicit with colInfo
  rxTextToXdf(inFile=tempCsv2, outFile=tempXdf, append="rows", colClasses=c(y="integer"))
    fileInfo <- rxGetInfo(tempXdf)
    checkEquals(fileInfo$numRows, 40)
    
  if (file.exists(tempCsv1)) file.remove(tempCsv1)
  if (file.exists(tempCsv2)) file.remove(tempCsv2)
  if (file.exists(tempXdf)) file.remove(tempXdf)	
}

"test.rxTextToXdf.args.append.factors" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    inFile2 <- file.path(rxGetOption("unitTestDataDir"), "tiny2.txt")
    outFile <-  file.path(tempdir(), ".rxTempTiny.xdf")
    if (file.exists(outFile)) file.remove(outFile)
  
    # using factors with specified levels
    rxTextToXdf(inFile, outFile, 
        colInfo=list(c=list(type="factor", levels=c("one","two","three","four","five","six"))))
    rxTextToXdf(inFile2, outFile, 
        colInfo=list(c=list(type="factor", levels=c("one","two","three","four","five","six"))), 
        append="rows") # append to output xdf
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 6, levels = c("one", "two", "three", "four", "five", "six")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
      
    # using factors w/o specified levels
    rxTextToXdf(inFile, outFile, colClasses=c(c="factor"))
    rxTextToXdf(inFile2, outFile, colClasses=c(c="factor"), append="rows") # append to output xdf
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 6, levels = c("one", "two", "three", "four", "five", "six")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # using factors w non-overlapping levels
    rxTextToXdf(inFile, outFile, colInfo=list(c=list(type="factor", levels=c("one","two","three"))))
    # Temporarilyy turn off warnings
    origOptions <- options(warn=-1)
    rxTextToXdf(inFile2, outFile, 
        colInfo=list(c=list(type="factor", levels=c("four","five","six"))), 
        append="rows") # append to output xdf
    options(origOptions)
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 6, levels = c("one", "two", "three", "four", "five", "six")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)  

    # specify levels first time, but not second
    rxTextToXdf(inFile, outFile, 
        colInfo=list(c=list(type="factor", levels=c("one","two","three"))))
    rxTextToXdf(inFile2, outFile, append="rows") # append to output xdf
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 6, levels = c("one", "two", "three", "four", "five", "six")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
  
    # specify full level set first time, but not second
    rxTextToXdf(inFile, outFile, 
        colInfo=list(c=list(type="factor", levels=c("one","two","three","four","five","six"))))
    rxTextToXdf(inFile2, outFile, append="rows") # append to output xdf
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 6, levels = c("one", "two", "three", "four", "five", "six")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
    
    # using factors w overlapping but not identical specified levels
    rxTextToXdf(inFile, outFile, 
        colInfo=list(c=list(type="factor", levels=c("one","two","three","four"))))
    # Temporarily turn off warnings to supress expected
    # WARNING: Existing factor levels in column c have been extended
    origOptions <- options(warn=-1)
    rxTextToXdf(inFile2, outFile, colInfo=list(c=list(type="factor", levels=c("four","five"))), append="rows")
    options(origOptions)
 
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 6)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 2)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 6), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 7.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 5, levels = c("one", "two", "three", "four", "five" )), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.transformFunc.xdf" <- function()
{
    xform <- function(x)
    {
      # alter a variable
      x$number <- x$number * 10
      
      # remove a variable
      x$cost <- NULL
      
      # return the altered list of variables
      return(x)
    }

    sampleDataDir <- rxGetOption("sampleDataDir")
    infile <- file.path(sampleDataDir, "claims.txt")
    appendFile <- file.path(sampleDataDir, "claimsExtra.txt")
    outFile <- file.path(tempdir(), ".rxTempOutput.xdf")
    if (file.exists(outFile)) file.remove(outFile)
    
    rxTextToXdf(infile, outFile=outFile, colClasses=c(type="factor"), transformFunc=xform, transformVars=c("number","cost"))
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 128)
  checkEquals(infoObj$numVars, 5)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["RowNum"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 128), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["age"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["car.age"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["type"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 4, levels = c("A", "B", "C", "D")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["number"]], structure(list(varType = "numeric", storage = "float32", low = 0, high = 4340), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo")) 
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.columnNames" <- function()
{
    # Check to see that default column names are used if not present in csv file
    fileName <- file.path(rxGetOption("unitTestDataDir"), "tinyNoNames.csv") 
    outFile <- file.path(tempdir(), ".rxTempTinyNoNames.xdf")
    myDF <- read.csv(fileName, header=FALSE) 
    rxTextToXdf(inFile=fileName, outFile=outFile, overwrite=TRUE) 
    varInfo <- rxGetVarInfo(outFile) 
    checkEquals(names(myDF), names(varInfo))
    myDF1 <- rxReadXdf(outFile)
    checkEquals(myDF$V1, myDF1$V1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(myDF1)
    
    # Redo setting rows to read
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead=2, overwrite=TRUE) 
    varInfo <- rxGetVarInfo(outFile) 
    checkEquals(names(myDF), names(varInfo))
    myDF1 <- rxReadXdf(outFile)
    checkEquals(myDF$V1, myDF1$V1)
    
    if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.columnNames.noRowNames" <- function()
{
    # Check to see that the default row name name is used if not present in csv file
    xdfFile <- file.path(tempdir(), ".rxTempTinyNoRowName.xdf")
    csvFile <- file.path(tempdir(), ".rxTempTinyNoRowName.csv")
    
    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(csvFile)) file.remove(csvFile)
    
    myData <- data.frame(x=c(1,1), y=c(2,2))
    write.csv(myData,file=csvFile)
    rxTextToXdf(inFile=csvFile, outFile=xdfFile, overwrite=TRUE)
    varInfo <- rxGetVarInfo(xdfFile)
    checkEquals(names(varInfo)[1], ".rxRowNames")
    
    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(csvFile)) file.remove(csvFile)
}

"test.rxTextToXdf.args.colClasses.dates" <- function()
{
    # Date data type is not supported so should be imported as strings
    fileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.txt") 
    outFile <- file.path(tempdir(), ".rxTempTinyDates.xdf")
    rxTextToXdf(inFile=fileName, outFile=outFile, overwrite=TRUE) 
    varInfo <- rxGetVarInfo(outFile) 
    checkEquals(varInfo$MyDates$varType, "character")
    if (file.exists(outFile)) file.remove(outFile)
}

#"test.rxTextToXdf.args.startRow" <- function()
#{
    #fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt") 
    #outFile1 <- ".rxTempTest300r1.xdf"
  #outFile2 <- ".rxTempTest300r2.xdf"
  #
  #if (file.exists(outFile1)) file.remove(outFile1)
  #if (file.exists(outFile2)) file.remove(outFile2)
    #rxTextToXdf(inFile=fileName, outFile=outFile1, numRows=50, startRow = 51, overwrite=TRUE) 
  #rxTextToXdf(inFile=fileName, outFile=outFile2, numRows=100, startRow = 1, overwrite=TRUE) 
    #fileInfo1 <- rxGetInfo(outFile1) 
  #checkEquals(fileInfo1$numRows, 50)
  #checkEquals(fileInfo1$numBlocks, 1)
  #
  #fileInfo2 <- rxGetInfo(outFile2) 
  #checkEquals(fileInfo2$numRows, 100)
  #checkEquals(fileInfo2$numBlocks, 1)
  #
  #myData1 <- rxReadXdf(file=outFile1, startRow=1, numRows=10)
  #myData2 <- rxReadXdf(file=outFile2, startRow=51, numRows=10)
  #checkEquals(myData1, myData2)
  #
    #if (file.exists(outFile1)) file.remove(outFile1)
  #rm(myData1)
  #
  #rxTextToXdf(inFile=fileName, outFile=outFile1, numRows=50, startRow = 51, rowsPerRead=25, overwrite=TRUE)
  #fileInfo1 <- rxGetInfo(outFile1) 
  #checkEquals(fileInfo1$numRows, 50)
  #checkEquals(fileInfo1$numBlocks, 2)
  #myData1 <- rxReadXdf(file=outFile1, startRow=1, numRows=10)
  #checkEquals(myData1, myData2)
  #
  #if (file.exists(outFile1)) file.remove(outFile1)
  #rm(myData1)
  #
  #rxTextToXdf(inFile=fileName, outFile=outFile1, numRows=50, startRow = 51, rowsPerRead=20, overwrite=TRUE)
  #fileInfo1 <- rxGetInfo(outFile1) 
  #checkEquals(fileInfo1$numRows, 50)
  #checkEquals(fileInfo1$numBlocks, 3)
  #myData1 <- rxReadXdf(file=outFile1, startRow=1, numRows=10)
  #checkEquals(myData1, myData2)
  #
  #if (file.exists(outFile1)) file.remove(outFile1)
  #if (file.exists(outFile2)) file.remove(outFile2)
#}
#

  
"test.rxTextToXdf.args.numRows" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt") 
    outFile <- file.path(tempdir(), ".rxTempTest300r.xdf")
  
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile=fileName, outFile=outFile, numRows=100, rowsPerRead=300, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 100)
  checkEquals(fileInfo$numBlocks, 1)
  
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile=fileName, outFile=outFile, numRows=100, rowsPerRead=50, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 100)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
      checkEquals(fileInfo$numBlocks, 2)
    }
  
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile=fileName, outFile=outFile, numRows=100, rowsPerRead=40, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 100)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
      checkEquals(fileInfo$numBlocks, 3)
    }
  
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile=fileName, outFile=outFile, numRows=3, rowsPerRead=5, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 3)
  checkEquals(fileInfo$numBlocks,1)
  
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile=fileName, outFile=outFile, numRows=5, rowsPerRead=3, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 5)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
      checkEquals(fileInfo$numBlocks, 2)
    }
  
  # With transforms
  rxTextToXdf(inFile=fileName, outFile=outFile, numRows=50, rowsPerRead=100, 
    transforms=list(xnum3=xnum2*xnum1), overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
  checkEquals(fileInfo$numRows, 50)
  checkEquals(fileInfo$numBlocks,1)
  
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.rowsPerRead" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt") 
    outFile <- file.path(tempdir(), ".rxTempTest300r.xdf")
    rxTextToXdf(inFile=fileName, outFile=outFile, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 100, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 3)
    }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 120, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars,9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 3)
    }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 200, 
        defaultReadBufferSize=100, overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
        if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 2)
    }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
   
    # Repeat with different file
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test200rSemiColon.txt") 
    outFile <- file.path(tempdir(), ".rxTempTest200r.xdf")
    rxTextToXdf(inFile=fileName, outFile=outFile, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 20, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 10)
    }    
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 12, 
        columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars,9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 17)
    }    
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 24, 
        defaultReadBufferSize=5, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows,200)
    checkEquals(fileInfo$numVars, 9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 9)
    }    
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 10, 
        defaultReadBufferSize=100, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 20)
    }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 200, 
     defaultReadBufferSize = 100, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
    
     # Repeat with a bigger file
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300rSemiColon.txt") 
    outFile <- file.path(tempdir(), ".rxTempTest300r.xdf")
    rxTextToXdf(inFile=fileName, outFile=outFile, rowsPerRead = 200, 
     defaultReadBufferSize = 100, columnDelimiters=";", overwrite=TRUE) 
    fileInfo <- rxGetInfo(outFile) 
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
      if (rxIsExpressEdition())
    {
        checkEquals(fileInfo$numBlocks, 1)
    } else
    {
        checkEquals(fileInfo$numBlocks, 2)
    }
    # The text file was created by replicated 100 rows 3 times
    # Make sure the first 100 and last 100 match
    myDF1 <- rxReadXdf(file=outFile, numRows=100)
    myDF2 <- rxReadXdf(file=outFile, numRows=100, startRow=201)
    checkEquals(myDF1, myDF2)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
}

"test.rxTextToXdf.args.transforms" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
  if (file.exists(outFile)) file.remove(outFile)

  transforms <- substitute(list(
    bSquared = b^2,
    bHalved = b / 2,
    bIncremented = b + 1,
    c = toupper(c),
    a = rev(a),
    b = NULL))
  
    rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE, transforms=transforms,
        defaultDecimalColType = "numeric", overwrite=TRUE, xdfCompressionLevel = 0)

    infoObj <- rxGetInfo(outFile)
  checkEquals(list(
    fileName = normalizePath(outFile),
    numRows = 3,
        numVars = 5,
    numBlocks = 1, 
        compressionType = "none"), unclass(infoObj))
  
  outFileDF <- rxReadXdf(outFile)
  original <- read.csv(inFile, strip.white=TRUE)
  tolerance <- 1e-7
  
  checkTrue(all(rxGetVarNames(outFile) == c("a","c","bSquared","bHalved","bIncremented")))
  checkEquals(as.character(outFileDF$c), toupper(as.character(original$c)))
  checkEqualsNumeric(outFileDF$bSquared, original$b^2, tolerance=tolerance)
  checkEqualsNumeric(outFileDF$bHalved, original$b/2, tolerance=tolerance)
  checkEqualsNumeric(outFileDF$bIncremented, original$b + 1, tolerance=tolerance)
  checkEquals(as.character(outFileDF$a), rev(as.character(original$a)))

  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.rowSelection" <- function()
{
  # read in a sample data set
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  stringsAsFactors <- TRUE
  original <- read.csv(inFile, strip.white=TRUE, stringsAsFactors=stringsAsFactors)
  original$ylogical <- factor(original$ylogical)
  original$xfac1 <- factor(original$xfac1)
    outFile <- file.path(tempdir(), ".rxTemp.xdf")
  
  # Sample the unique values of y in the data source
  # and use them as selection thresholds. Compare the rxTextToXdf()
  # output to that of subset() using the same filter. Round the filter values so
  # that we don't have any round off issues when comparing to subset().
  set.seed(100)
  yThreshold <- round(sample(original$y, 10, replace=FALSE), 5)
  
  for (thresh in yThreshold)
  {
      if (file.exists(outFile)) file.remove(outFile)
    rowSelection <- substitute(y >= thresh, list(thresh=thresh))
        rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE, rowSelection = rowSelection, 
      stringsAsFactors=stringsAsFactors)
    filteredExportedData <- rxReadXdf(outFile, stringsAsFactors=stringsAsFactors)
    directSubsetData <- subset(original, eval(rowSelection))
    row.names(directSubsetData) <- seq(NROW(directSubsetData))
    checkEquals(filteredExportedData, directSubsetData, check.attributes=FALSE)
    }

  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.args.rowSelection.noRowsSelected" <- function()
{
  #"RA-1916: rxTextToXdf did not throw warning/error message when there is no data to write after rowSelection
  # read in a sample data set
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    original <- read.csv(inFile, strip.white=TRUE, stringsAsFactors=TRUE)
    outFile <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(outFile)) file.remove(outFile)

    thresh <- max(original$b) + .000001
    rowSelection <- substitute(b > thresh, list(thresh=thresh))
    
  # turn all warnings into errors and save old warning level
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)
    checkException(rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE, rowSelection = rowSelection, 
        stringsAsFactors=TRUE))
  
  if (file.exists(outFile)) file.remove(outFile)
}

"test.rxTextToXdf.data.logicals" <- function()
{
    txtFile <- file.path(tempdir(), ".rxTemp.csv")
    xdfFile <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)
    
    myData1 <- data.frame(
        rowNum = 1:4, 
        logicalMiss = rep(NA, times=4),
        logicalStr = c("FALSE", "TRUE", "FALSE", "TRUE"),
        logicalInt = c(0,1,0,1),
        logicalStrShort = c("F", "T", "F", "T"),
        logicalStrM = c("FALSE", NA, "FALSE", "TRUE"),
        logicalIntM = c(0,NA,0,1),
        logicalStrShortM =  c("F", NA, "F", "T"))

    write.csv(myData1, file=txtFile, row.names=FALSE)

    myColClasses=c(logicalMis="logical", logicalStr="logical", logicalInt="logical",
        logicalStrShort = "logical", logicalStrM = "logical", logicalIntM = "logical",
        logicalStrShortM = "logical")
    rxTextToXdf(inFile=txtFile, outFile=xdfFile, 
    colClasses=myColClasses, overwrite=TRUE)
    varSum <- rxSummary(~logicalMiss, data=xdfFile, byTerm=FALSE)
    checkEquals(varSum$nobs.valid, 0)
    varSum <- rxSummary(~logicalStr, data=xdfFile, byTerm=FALSE)
    checkEquals(varSum$nobs.valid, 4)
    myData2 <- rxReadXdf(file=xdfFile)
    checkEquals(myData2$logicalStr, myData2$logicalInt, checkNames=FALSE)
    checkEquals(myData2$logicalStr, myData2$logicalStrShort, checkNames=FALSE)
    checkEquals(myData2$logicalStrM, myData2$logicalIntM, checkNames=FALSE)
    checkEquals(myData2$logicalStrM, myData2$logicalStrShortM, checkNames=FALSE)
    
    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)
}

"test.rxTextToXdf.data.logicals.sniff" <- function()
{
  # RA-1935 regression test
  txtFile <- file.path(tempdir(), ".rxTemp.csv")
    xdfFile <- file.path(tempdir(), ".rxTemp.xdf")
    if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)
  
  set.seed(13)
  ynotlogical <- as.character(runif(26,0,1) < .5)
  ynotlogical[22] <- "Hello"
  data1 <- data.frame(a=letters,b=seq(1:26),ylogical=runif(26,0,1), ynotlogical)
  data1$ylogical<- data1$ylogical < .5

  # Test to make sure logical is sniffed as logical
  write.csv(data1, txtFile, row.names=FALSE, na=" ")
  rxTextToXdf(inFile=txtFile, outFile=xdfFile, stringsAsFactors=TRUE, 
    rowsPerRead = 10, overwrite=TRUE)
  varInfo <-rxGetVarInfo(xdfFile)
  checkEquals(varInfo$ylogical$varType, "logical")
  checkEquals(varInfo$ynotlogical$varType, "factor")
  if (file.exists(xdfFile)) file.remove(xdfFile)
  
  # Make sure that this can be overridden with colClasses
  rxTextToXdf(inFile=txtFile, outFile=xdfFile, stringsAsFactors=TRUE, 
    colClasses = c(ylogical = "character"),
    rowsPerRead = 10, overwrite=TRUE)
  varInfo <-rxGetVarInfo(xdfFile)
  checkEquals(varInfo$ylogical$varType, "character")
  checkEquals(varInfo$ynotlogical$varType, "factor")

  if (file.exists(xdfFile)) file.remove(xdfFile)
    if (file.exists(txtFile)) file.remove(txtFile)
}

"test.rxTextToXdf.args.transformFunc.rxStartRow" <- function()
{
  # Regression test RA-2878
 
  richDatatxt <- file.path(rxGetOption("unitTestDataDir"),"richTest200rComma.txt") 
  richData <- file.path(tempdir(),"richTest200r5b.xdf") 

  transformFunc <- function(datalist)
  { 
        numRowsInChunk <- length(datalist$zfac) 
    datalist$Index <- .rxStartRow:(.rxStartRow + numRowsInChunk - 1) 
    datalist$Chunk <- rep(.rxChunkNum, numRowsInChunk)
    datalist$File <-  rep(.rxReadFileName, numRowsInChunk)
        datalist 
  } 

  rxTextToXdf(richDatatxt, richData, rowsPerRead=40,stringsAsFactors=TRUE,
    transformFunc=transformFunc, 
    transformVars=c("zfac"),colClasses=c(z.char1="character"), reportProgress=0, 
    overwrite=TRUE) 
  myData <- rxReadXdf( richData)
  checkEquals(myData$Index, 1:200)
    if (rxIsExpressEdition())
    {
      checkEquals(myData$Chunk, rep(1, each=200))
    }
    else {   
        checkEquals(myData$Chunk, rep(1:5, each=40))
    }
    richDatatxt <- normalizePath(richDatatxt)
  checkEquals(myData$File, rep(richDatatxt, 200))
  file.remove( richData)
}

"test.rxTextToXdf.args.data.ordered" <- function()
{
    x <- rep(1:3, times=4)
    yfac <- factor(x, labels=c("low", "med", "high"))
  yofac <- factor(x, labels=c("low", "med", "high"), ordered=TRUE)
    myData <- data.frame(x, yfac, yofac)
   
    xdfFile <- file.path(tempdir(), ".rxTest.xdf")
    txtFile <- file.path(tempdir(), ".rxTest.txt")

    # Create comma-delimited txt file with missing=blank
    if (file.exists(txtFile)) file.remove(txtFile)
    write.table(myData, file=txtFile, quote=FALSE, sep=",", na=" ",row.names=FALSE, col.names=TRUE)
    # Remove xdfFile if it exists so we know we are creating a new file
    if (file.exists(xdfFile)) file.remove(xdfFile)
    # read text data with missing=blank
    rxTextToXdf(inFile = txtFile, outFile = xdfFile, colClasses = c(x = "integer", yfac="factor", yofac = "ordered"), overwrite=TRUE)
    xdfVarInfo <- rxGetVarInfo(xdfFile)
    newData <- rxReadXdf( xdfFile )
    checkEquals( newData, myData)
    file.remove(xdfFile)
    file.remove(txtFile)
}

"test.rxTextToXdf.args.data.row.names" <- function()
{
  testDF <- data.frame(x=1:10, y=1:10)
    # Create a csv file with an un-named row names column
    testFileTxt <- tempfile(pattern = ".rxTTX3TempTest1", fileext = ".txt") 
    testFileXdf <- tempfile(pattern = ".rxTTX3TempTest", fileext = ".xdf")     
  write.table(testDF, testFileTxt, sep=",")
  rxTextToXdf(inFile=testFileTxt, outFile=testFileXdf, overwrite=TRUE) 
  obj <- rxGetInfo(testFileXdf, getVarInfo=TRUE, numRows=10)
  checkEquals(names(obj$data), c(".rxRowNames", "x","y"))
    
    file.remove(testFileTxt)
    file.remove(testFileXdf)

}

"test.rxTextToXdf.args.missingValueString" <- function()
{
    testFileTxt <- tempfile(pattern = ".rxTTX4TempTest1", fileext = ".txt") 
    testFileXdf <- tempfile(pattern = ".rxTTX4TempTest", fileext = ".xdf") 
        
    writeLines(c("a,b,c,d",
             "NA, -7, M, -7",
             "-7, NA, -7, M",
             "0.5, 0.5, 0.5, 0.5"), testFileTxt)
    # Use the default of "NA"
    rxTextToXdf(testFileTxt, outFile = testFileXdf, overwrite=TRUE)
    varInfo <- rxGetVarInfo(testFileXdf)
    checkEquals(varInfo$a$varType, "numeric")
    checkEquals(varInfo$b$varType, "numeric")
    checkEquals(varInfo$c$varType, "character")
    checkEquals(varInfo$d$varType, "character")
 
    myData1 <- rxReadXdf(testFileXdf)
    file.remove(testFileXdf)    
    
    # Now set the missingValueString to "M"
    rxTextToXdf(testFileTxt, outFile = testFileXdf, missingValueString = "M", overwrite=TRUE)
    varInfo <- rxGetVarInfo(testFileXdf)
    checkEquals(varInfo$a$varType, "character")
    checkEquals(varInfo$b$varType, "character")    
    checkEquals(varInfo$c$varType, "numeric")
    checkEquals(varInfo$d$varType, "numeric")
    
    myData2 <- rxReadXdf(testFileXdf)
    checkEquals(myData1$a, myData2$c)
    checkEquals(myData1$b, myData2$d)
      
    file.remove(testFileXdf)  
    file.remove(testFileTxt) 

}

"test.rxTextToXdf.args.xdfCompressionLevel" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tiny.txt")
    outFile <- file.path(tempdir(), ".rxTempTiny.xdf")
  if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE,
        defaultDecimalColType = "numeric", overwrite=TRUE, xdfCompressionLevel = 0)
    
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$compressionType, "none")
    file.remove(outFile)
    
    rxTextToXdf(inFile, outFile, firstRowIsColNames = TRUE,
        defaultDecimalColType = "numeric", overwrite=TRUE, xdfCompressionLevel = 1)
    
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$compressionType, "zlib")
    file.remove(outFile)    
}

# Unit test for	RA-12286 
"test.rxTextToXdf.CompositeCsv.rxDataStep" <- function()
{
  claimsCompCsv <- file.path(rxGetOption("sampleDataDir"), "claimsTxt")
  CsvDS <- RxTextData(claimsCompCsv)	
    xdfPath <- file.path(tempdir(), "testCompositeXdf")
  XdfDS <- RxXdfData(file = xdfPath, createCompositeSet = TRUE, blocksPerCompositeFile=1)


  rxDataStep(inData = CsvDS, outFile = XdfDS, overwrite = TRUE)

  csvResult <- rxSummary(formula = ~., data = CsvDS)
  xdfResult <- rxSummary(formula = ~., data = XdfDS)	
  checkEquals(csvResult$sDataFrame, xdfResult$sDataFrame)
  unlink(xdfPath, recursive = TRUE)
  
}
  
"test.rxTextToXdf.CompositeCsv.rxDataStep.emptyfile" <- function()
{
    #library(RevoScaleR)

    ###################################
    ## composite csv with no empty files
    compCsvPathNoEmpty <- file.path(rxGetOption("sampleDataDir"), "claimsTxt")
    dataColInfo <- list(
        RowNum =list(type = "integer"),
        age = list( type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")))

    compCsvDSNoEmpty <- RxTextData(compCsvPathNoEmpty, colInfo = dataColInfo)
    #list.files(compCsvPathNoEmpty)
    #rxGetInfo(compCsvDSNoEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)

    compXdfPathNoEmpty <- tempfile(pattern = "compXdfPathNoEmpty")
    on.exit(unlink(compXdfPathNoEmpty, recursive = TRUE, force = TRUE), add = TRUE)
    compXdfDSNoEmpty <- RxXdfData(file = compXdfPathNoEmpty, createCompositeSet = TRUE, blocksPerCompositeFile = 2)

    rxDataStep(inData = compCsvDSNoEmpty, outFile = compXdfDSNoEmpty, overwrite = TRUE, rowsPerRead = 32)
    #rxGetInfo(compXdfDSNoEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)
    csvResult <- rxSummary(formula = ~., data = compCsvDSNoEmpty)
    xdfResult <- rxSummary(formula = ~., data = compXdfDSNoEmpty)	
    checkEquals(csvResult$sDataFrame, xdfResult$sDataFrame)

    rxImport(inData = compCsvDSNoEmpty, outFile = compXdfDSNoEmpty, overwrite = TRUE, rowsPerRead = 32)
    #rxGetInfo(compXdfDSNoEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)
    csvResult <- rxSummary(formula = ~., data = compCsvDSNoEmpty)
    xdfResult <- rxSummary(formula = ~., data = compXdfDSNoEmpty)	
    checkEquals(csvResult$sDataFrame, xdfResult$sDataFrame)

    ###################################
    ## composite csv with some empty files
    compCsvPathSomeEmpty <- file.path(tempdir(), "compCsvPathSomeEmpty")
    on.exit(unlink(compCsvPathSomeEmpty, recursive = TRUE, force = TRUE), add = TRUE)
    dir.create(compCsvPathSomeEmpty)
    file.copy(compCsvPathNoEmpty, compCsvPathSomeEmpty, recursive = TRUE)
    file.create(file.path(compCsvPathSomeEmpty, "claimsTxt", c("AAA.txt", "zzz.txt")))

    compCsvDSSomeEmpty <- RxTextData(file.path(compCsvPathSomeEmpty, "claimsTxt"), colInfo = dataColInfo)
    #list.files(compCsvPathSomeEmpty)
    #rxGetInfo(compCsvDSSomeEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)

    compXdfPathSomeEmpty <- tempfile(pattern = "compXdfPathSomeEmpty")
    on.exit(unlink(compXdfPathSomeEmpty, recursive = TRUE, force = TRUE), add = TRUE)
    compXdfDSSomeEmpty <- RxXdfData(file = compXdfPathSomeEmpty, createCompositeSet = TRUE, blocksPerCompositeFile = 2)

    rxDataStep(inData = compCsvDSSomeEmpty, outFile = compXdfDSSomeEmpty, overwrite = TRUE, rowsPerRead = 32)
    #rxGetInfo(compXdfDSSomeEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)
    csvResult <- rxSummary(formula = ~., data = compCsvDSSomeEmpty)
    xdfResult <- rxSummary(formula = ~., data = compXdfDSSomeEmpty)	
    checkEquals(csvResult$sDataFrame, xdfResult$sDataFrame)

    rxImport(inData = compCsvDSSomeEmpty, outFile = compXdfDSSomeEmpty, overwrite = TRUE, rowsPerRead = 32)
    #rxGetInfo(compXdfDSSomeEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)
    csvResult <- rxSummary(formula = ~., data = compCsvDSSomeEmpty)
    xdfResult <- rxSummary(formula = ~., data = compXdfDSSomeEmpty)	
    checkEquals(csvResult$sDataFrame, xdfResult$sDataFrame)

    ###################################
    ## path with no file or composite csv with all empty files
    compCsvPathAllEmpty <- file.path(tempdir(), "compCsvPathAllEmpty")
    on.exit(unlink(compCsvPathAllEmpty, recursive = TRUE, force = TRUE), add = TRUE)
    dir.create(compCsvPathAllEmpty)
    dir.create(file.path(compCsvPathAllEmpty, "claimsTxt"))

    compCsvDSAllEmpty <- RxTextData(file.path(compCsvPathAllEmpty, "claimsTxt"), colInfo = dataColInfo)
    #list.files(compCsvPathAllEmpty)
    #rxGetInfo(compCsvDSAllEmpty, getVarInfo = TRUE, getBlockSizes = TRUE)

    compXdfPathAllEmpty <- tempfile(pattern = "compXdfPathAllEmpty")
    on.exit(unlink(compXdfPathAllEmpty, recursive = TRUE, force = TRUE), add = TRUE)
    compXdfDSAllEmpty <- RxXdfData(file = compXdfPathAllEmpty, createCompositeSet = TRUE, blocksPerCompositeFile = 2)

    ## path with no files
    checkException(rxDataStep(inData = compCsvDSAllEmpty, outFile = compXdfDSAllEmpty, overwrite = TRUE, rowsPerRead = 32))
    checkException(rxImport(inData = compCsvDSAllEmpty, outFile = compXdfDSAllEmpty, overwrite = TRUE, rowsPerRead = 32))

    ## composite csv with all empty files
    file.create(file.path(compCsvPathAllEmpty, "claimsTxt", c("AAA.txt", "zzz.txt")))
    checkException(rxDataStep(inData = compCsvDSAllEmpty, outFile = compXdfDSAllEmpty, overwrite = TRUE, rowsPerRead = 32))
    checkException(rxImport(inData = compCsvDSAllEmpty, outFile = compXdfDSAllEmpty, overwrite = TRUE, rowsPerRead = 32))
}
