# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxDataStep, testing for deprecated rxXdfToDataFrame functionality
#
############################################################################################

"test.rxXdfToDataFrame.args.file" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    myDF1 <- rxDataStep(inData = test100r5b)
    myDF2 <- rxDataStep(inData =test100r1b)
    checkEquals(myDF1, myDF2)
}

"test.rxXdfToDataFrame.args.rowSelection" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    myDF1 <- rxDataStep(inData =test100r5b, rowSelection = xint1 > 7)
    myDF2 <- rxDataStep(inData =test100r1b)
    myDF2 <- subset(myDF2, xint1 > 7)
    # Row numbers will be different, so don't check attributes
    checkEquals(myDF1, myDF2, check.attributes=FALSE) 
    
    outFile <- tempfile(pattern = ".rxTDF1TempTest", fileext = ".xdf")
  if (file.exists(outFile)) file.remove(outFile)
  myData <- data.frame(x=1:10, y=c(1:5, rep(NA, times=5)))
  rxDataStep(inData=myData, outFile = outFile, overwrite=TRUE)
  myDataSubset <- subset(myData, y > 3)
  myDataSelXdf <- rxDataStep(inData = outFile, rowSelection = y > 3)
  # They will have different row names
  checkEquals(myDataSubset, myDataSelXdf, check.attributes=FALSE)
  if (file.exists(outFile)) file.remove(outFile)
    
    #RA-3674
    # Check for warning if no rows are returned
    # turn all warnings into errors and save old warning level
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)
    res <- try(
        myData <- rxDataStep(inData = test100r5b, rowSelection = xint1 > 20)
        ) 
    checkTrue(inherits(res,"try-error"))
    #warnRegExpr <- "No data was written to the output data source"
    warnRegExpr <- "There are no rows"
    checkTrue(length(grep(warnRegExpr, as.character(res))) > 0)
    options(oldWarn)
}

"test.rxXdfToDataFrame.args.varsToKeep" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
    # With rowSelection
    myDF1 <- rxDataStep(inData=test100r5b, rowSelection = xint1 > 7, varsToKeep = c("xnum1", "xint1"))
    myDF2 <- rxDataStep(inData=test100r1b)
    myDF2 <- subset(myDF2, xint1 > 7, select=c(xnum1, xint1))
    # Row numbers will be different, so don't check attributes
    checkEquals(myDF1, myDF2, check.attributes=FALSE)
    
    # Without rowSelection
    myDF3 <- rxDataStep(inData=test100r5b, varsToKeep = c("xnum1", "xint1"))
    myDF4 <- rxDataStep(inData=test100r1b, varsToKeep = c("xnum1", "xint1"))
    checkEquals(myDF3, myDF4, check.attributes=FALSE)
}

"test.rxXdfToDataFrame.args.varsToDrop" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
    # With rowSelection
    myDF1 <- rxDataStep(inData =test100r5b, rowSelection = xint1 > 7, varsToDrop = c("xnum1", "xint1"))
    myDF2 <- rxDataStep(inData =test100r1b)
    myDF2 <- subset(myDF2, xint1 > 7, select=-c(xnum1, xint1))
    # Row numbers will be different, so don't check attributes
    checkEquals(myDF1, myDF2, check.attributes=FALSE)
    
    # Without rowSelection
    myDF3 <- rxDataStep(inData=test100r5b, varsToDrop = c("xnum1", "xint1"))
    myDF4 <- rxDataStep(inData=test100r1b, varsToDrop = c("xnum1", "xint1"))
    checkEquals(myDF3, myDF4, check.attributes=FALSE)
}

"test.rxXdfToDataFrame.args.transforms" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
    # With rowSelection
    myDF1 <- rxDataStep(inData =test100r5b, rowSelection = xint1 > 7, transforms=list(xnew=xnum1+xnum2))
    myDF2 <- rxDataStep(inData =test100r1b)
    myDF2 <- subset(myDF2, xint1 > 7)
    myDF2$xnew <- myDF2$xnum1 + myDF2$xnum2

    # Columns will be in different orders
    checkEquals(myDF1$y, myDF2$y)
    checkEquals(myDF1$xnew, myDF2$xnew)
    checkEquals(myDF1$xfac1, myDF2$xfac1)
    checkEquals(myDF1$xnum2miss, myDF2$xnum2miss)
    
    # Without rowSelection, variables still reordered
    myDF3 <- rxDataStep(inData =test100r5b, transforms=list(xnew=xnum1+xnum2))
    myDF4 <- rxDataStep(inData =test100r1b)
    myDF4$xnew <- myDF4$xnum1 + myDF4$xnum2
    checkEquals(myDF3$y, myDF4$y)
    checkEquals(myDF3$xnew, myDF4$xnew)
    checkEquals(myDF3$xfac1, myDF4$xfac1)
    checkEquals(myDF3$xnum2miss, myDF4$xnum2miss)
}

"test.rxXdfToDataFrame.args.transformFunc" <- function()
{
   
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf") 
    
    # Use transformation function with rowSelection and transforms
    myFun <- function(data)
    {
        data$xnew2 <- data$xnum1*2
        data
    }

    myDF1 <- rxDataStep(inData =test100r5b, rowSelection = xint1 > 7, transforms=list(xnew=xnum1+xnum2),
        transformFunc = myFun, transformVars="xnum1")
    myDF2 <- rxDataStep(inData =test100r1b, returnDataFrame=FALSE)
    myDF2 <- as.data.frame(myFun(myDF2))
    myDF2 <- subset(myDF2, xint1 > 7)
    myDF2$xnew <- myDF2$xnum1 + myDF2$xnum2
    
    checkEquals(myDF1$y, myDF2$y)
    checkEquals(myDF1$xnew, myDF2$xnew)
    checkEquals(myDF1$xnew2, myDF2$xnew2)
    checkEquals(myDF1$xfac1, myDF2$xfac1)
    checkEquals(myDF1$xnum2miss, myDF2$xnum2miss)
    

}

"test.rxXdfToDataFrame.args.stringsAsFactors" <- function()
{	
  
  outFile <- tempfile(pattern = ".rxTempStrings", fileext = ".xdf")
  if (file.exists(outFile)) file.remove(outFile)
  
  myData <- data.frame(xchar= c("One", "Two", "One", "Two", "Three", "One", "Three", "One"),
                     xint=c(1,2,1,2,3,1,3,1),
             xfac = as.factor(c("One", "Two", "One", "Two", "Three", "One", "Three", "One")),
             stringsAsFactors=FALSE)
  rxDataStep(inData=myData, outFile=outFile)
  
  myDF1 <- rxDataStep(inData = outFile, stringsAsFactors=FALSE)
  checkEquals(myDF1, myData)
  
  myDF2 <- rxDataStep(inData = outFile, stringsAsFactors=TRUE)
  checkEquals(myDF2$xchar, myDF2$xfac)
  
  checkEquals(class(myDF1$xchar), "character")
  checkEquals(class(myDF2$xchar), "factor")
  
  # If there is rowSelection, it will go through a different code path
  
  rm(myDF1)
  rm(myDF2)
  myDF1 <- rxDataStep(inData =outFile, stringsAsFactors=FALSE, rowSelection = xint > 1)
  myDF2 <- rxDataStep(inData =outFile, stringsAsFactors=TRUE, rowSelection = xint > 1)
    
  checkEquals(class(myDF1$xchar), "character")
  checkEquals(class(myDF2$xchar), "factor")
  
  if (file.exists(outFile)) file.remove(outFile)
   
}

"test.rxXdfToDataFrame.args.maxRowsByCols" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    fileInfo <- rxGetInfo(test100r5b)
  
  myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*fileInfo$numVars))
  checkEquals(fileInfo$numRows, nrow(myDF))
  checkEquals(fileInfo$numVars, ncol(myDF))
    rm(myDF)
  
    # Note: warnings from C++ code are not converted to errors with options(), so
    # there is slightly different behavior between rxXdfToDataFrame and rxDataStep
  # This should fail if all warnings into errors and save old warning level
    #oldWarn <- options(warn=2)
    #on.exit(options(oldWarn), add = TRUE)
  #res <- try(myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*fileInfo$numVars)-1)))
  # check for error encountered during execution
  #checkTrue(is(res, "try-error"))
    
  res <- capture.output(
        myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*fileInfo$numVars)-1),
          reportProgress = 0))
    checkTrue(grepl("WARNING: The number of rows", res[1]))
    # If there are transforms, the code goes through a different path
  # This should fail if warnings are errors because we're creating an extra variable
  #res <- try(myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*fileInfo$numVars), 
    #transforms=list(xnum3=xnum1+xnum2)))
  #checkTrue(is(res, "try-error"))
    
  res <- capture.output(myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*fileInfo$numVars), 
    transforms=list(xnum3=xnum1+xnum2), reportProgress = 0))    
    checkTrue(grepl("WARNING: The number of rows", res[1]))  
    
  res <- capture.output(myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*2)-1),
    varsToKeep=c("xnum1", "xfac1"), reportProgress = 0))    
    checkTrue(grepl("WARNING: The number of rows", res[1]))  
       
    # varsToDrop: This should fail if warnings are errors
  res <- capture.output(myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*(fileInfo$numVars-2))-1),
    varsToDrop=c("xnum1", "xfac1"), reportProgress = 0))    
     checkTrue(grepl("WARNING: The number of rows", res[1]))   
    
    # If warnings aren't errors, these should return data frames
    myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*fileInfo$numVars)-1))
  checkEquals(nrow(myDF1), fileInfo$numRows - 1)
  rm(myDF1)
    
    myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*fileInfo$numVars), 
    transforms=list(xnum3=xnum1+xnum2))
    numRowsShouldBeRead = (fileInfo$numRows*fileInfo$numVars)/(fileInfo$numVars+1)
    checkEquals(nrow(myDF1), numRowsShouldBeRead)

  # Bumping up the number of variables allowed by one should work
  myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*(fileInfo$numVars+1)), 
    transforms=list(xnum3=xnum1+xnum2))
  checkEquals(fileInfo$numRows, nrow(myDF))
  checkEquals(fileInfo$numVars+1, ncol(myDF))
  rm(myDF)

  # varsToKeep / varsToDrop should alter the calculation
  
  myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*2), varsToKeep=c("xnum1", "xfac1"))
  checkEquals(fileInfo$numRows, nrow(myDF))
  checkEquals(2, ncol(myDF))
    rm(myDF)
    
  # varsToKeep: This should give a warning
    myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=((fileInfo$numRows*2)-1),
    varsToKeep=c("xnum1", "xfac1"))
    rowsShouldBeRead = as.integer(((fileInfo$numRows*2)-1)/2)
    checkEquals(nrow(myDF1), rowsShouldBeRead)
    rm(myDF1)
    
  # Use varsToDrop
  myDF <- rxDataStep(inData =test100r5b, maxRowsByCols=(fileInfo$numRows*(fileInfo$numVars-2)), varsToDrop=c("xnum1", "xfac1"))
  checkEquals(fileInfo$numRows, nrow(myDF))
  checkEquals(fileInfo$numVars-2, ncol(myDF))
    rm(myDF)
  
  # varsToDrop: This should give a warning
    maxRowsByCols=((fileInfo$numRows*(fileInfo$numVars-2))-1)
  myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=maxRowsByCols,
    varsToDrop=c("xnum1", "xfac1"))
    rowsShouldBeRead = as.integer(maxRowsByCols/7)
    checkEquals(nrow(myDF1), rowsShouldBeRead)
    rm(myDF1)
    
    # This fails in rxXdfToDataFrame, but will give warning in rxDataStep
    res <- try(myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=4,
  varsToDrop=c("xnum1", "xfac1")))
  # check for error encountered during execution
  # checkTrue(is(res, "try-error"))
    checkEquals(nrow(myDF1), 0)
    
    # This fails in rxXdfToDataFrame, but will give warning in rxDataStep
    res <- try(myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=9,
      transforms=list(xnum3=xnum1+xnum2)))
  # check for error encountered during execution
  #checkTrue(is(res, "try-error"))
     checkEquals(nrow(myDF1), 0)
    
    # Row selection
    myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=522,
      rowSelection=ylogical==TRUE)
    checkEquals(nrow(myDF1), 58)
    
    # This should give a warning
    myDF1 <- rxDataStep(inData =test100r5b, maxRowsByCols=500,
      rowSelection=ylogical==TRUE)
    checkEquals(nrow(myDF1), 55) 
}

"test.rxXdfToDataFrame.args.removeMissings" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  test100r5bStartMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bStartMiss")
    test100r5bEndMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bEndMiss")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
  
  # Simple case
  myDF <- rxDataStep(inData =test100r5b, removeMissings=TRUE)
  myDF2 <- rxDataStep(inData =test77r1bNoMiss)
  checkEquals(myDF, myDF2, check.attributes=FALSE)
  rm(myDF)
  rm(myDF2)
  
  # With tranforms  
  myDF <- rxDataStep(inData =test100r5b, transforms=list(xint2=xint1*2), removeMissings=TRUE)
  myDF2 <- rxDataStep(inData =test77r1bNoMiss)
  myDF2$xint2 <- myDF2$xint1 * 2
  checkEquals(myDF, myDF2, check.attributes=FALSE)
}

"test.rxXdfToDataFrame.args.blocksPerRead" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  myDF <- rxDataStep(inData =test100r5b)
  myDF1 <- rxDataStep(inData =test100r5b, blocksPerRead=2)
  checkEquals(myDF, myDF1)
  rm(myDF)
  rm(myDF1)
  
  myDF <- rxDataStep(inData =test100r5b, transforms=list(xint2=xint1*2))
  myDF1 <- rxDataStep(inData =test100r5b, transforms=list(xint2=xint1*2), blocksPerRead=2)
  checkEquals(myDF, myDF1)
}

"test.rxXdfToDataFrame.args.reportProgress" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  output0 <- capture.output(myDF <- rxDataStep(inData =test100r5b, reportProgress=0))
  output1 <- capture.output(myDF <- rxDataStep(inData =test100r5b, reportProgress=1))
  output2 <- capture.output(myDF <- rxDataStep(inData =test100r5b, reportProgress=2))
  output3 <- capture.output(myDF <- rxDataStep(inData =test100r5b, reportProgress=3))
  checkEquals(length(output0), 0)
  checkTrue(length(output1) > 0)
  
  output0 <- capture.output(myDF <- rxDataStep(inData =test100r5b, rowSelection=xnum1 > 0.0, reportProgress=0))
  output1 <- capture.output(myDF <- rxDataStep(inData =test100r5b, rowSelection=xnum1 > 0.0, reportProgress=1))
  output2 <- capture.output(myDF <- rxDataStep(inData =test100r5b, rowSelection=xnum1 > 0.0, reportProgress=2))
  output3 <- capture.output(myDF <- rxDataStep(inData =test100r5b, rowSelection=xnum1 > 0.0, reportProgress=3))
}
