# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxXdfToText
#
############################################################################################


test.rxXdfToText.data.multiBlocks <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    testTxt <-  tempfile(pattern = ".rxTempXdfToText", fileext = ".txt")
    testXdfNew <- tempfile(pattern = ".rxTempXdfToText", fileext = ".xdf")
    if (file.exists(testTxt)) file.remove(testTxt)
    if (file.exists(testXdfNew)) file.remove(testXdfNew)

    # Test basic function, reading in each of the 5 blocks 1 at a time
    rxXdfToText(inFile = testXdf, outFile = testTxt, blocksPerRead = 1,
                reportProgress = 0)
    # Read it back in and compare
    rxTextToXdf(inFile = testTxt, outFile = testXdfNew, stringsAsFactors = TRUE,
                reportProgress = 0)
            
    infoObj <- rxGetInfo(testXdf, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdf)
  checkEquals(infoObj$fileName, normalizePath(testXdf))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 5)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(testXdfNew, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdfNew)
  checkEquals(infoObj$fileName, normalizePath(testXdfNew))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    # Cleanup
    file.remove(testTxt)
    file.remove(testXdfNew)

    # Redo, reading in all 5 blocks at once
    rxXdfToText(inFile = testXdf, outFile = testTxt, blocksPerRead = 5,
                reportProgress = 0)
    # Read it back in and compare
    rxTextToXdf(inFile = testTxt, outFile = testXdfNew, stringsAsFactors = TRUE,
                reportProgress = 0)
    infoObj <- rxGetInfo(testXdf, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdf)
  checkEquals(infoObj$fileName, normalizePath(testXdf))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 5)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(testXdfNew, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdfNew)
  checkEquals(infoObj$fileName, normalizePath(testXdfNew))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    # Cleanup
    file.remove(testTxt)
    file.remove(testXdfNew)
}

test.rxXdfToText.args.varsToDrop <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    testTxt <-  tempfile(pattern = ".rxTempXdfToText", fileext = ".txt")
    testXdfNew <- tempfile(pattern = ".rxTempXdfToText", fileext = ".xdf")    

    if (file.exists(testTxt)) file.remove(testTxt)
    if (file.exists(testXdfNew)) file.remove(testXdfNew)

    # Argument Test: varsToDrop
    rxXdfToText(inFile = testXdf, outFile = testTxt, varsToDrop = "xfac2",
                blocksPerRead = 1, reportProgress = 0)
    # Read it back in and compare
    rxTextToXdf(inFile = testTxt, outFile = testXdfNew, stringsAsFactors = TRUE,
                reportProgress = 0)
    infoObj <- rxGetInfo(testXdf, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdf)
  checkEquals(infoObj$fileName, normalizePath(testXdf))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 5)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(testXdfNew, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdfNew)
  checkEquals(infoObj$fileName, normalizePath(testXdfNew))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 8)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    # Cleanup
    file.remove(testTxt)
    file.remove(testXdfNew)
}

"test.rxXdfToText.args.varsToKeep" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    testTxt <-  tempfile(pattern = ".rxTempXdfToText", fileext = ".txt")
    testXdfNew <- tempfile(pattern = ".rxTempXdfToText", fileext = ".xdf")
    testTxt <- ".rxTempXdfToText.txt"

    if (file.exists(testTxt)) file.remove(testTxt)
    if (file.exists(testXdfNew)) file.remove(testXdfNew)
    
    # Test keeping first, middle, and last variable
    rxXdfToText(inFile = testXdf, outFile = testTxt,
                varsToKeep = c("y", "xnum2", "xfac2"), blocksPerRead = 1,
                reportProgress = 0)
    # Read it back in and compare
    rxTextToXdf(inFile = testTxt, outFile = testXdfNew, stringsAsFactors = TRUE,
                reportProgress = 0)

    infoObj <- rxGetInfo(testXdf, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdf)
  checkEquals(infoObj$fileName, normalizePath(testXdf))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 9)
  checkEquals(infoObj$numBlocks, 5)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ylogical"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 2, levels = c("TRUE", "FALSE")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["ybinint"]], structure(list(varType = "integer", storage = "int32", low = 0, high = 1), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum1"]], structure(list(varType = "numeric", storage = "float32", low = -2.27192544937134, high = 2.58195900917053), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2miss"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 0.456121057271957), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xint1"]], structure(list(varType = "integer", storage = "int32", low = 5, high = 14), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac1"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("10", "7", "8", "9", "13", "11", "6", "14", "5", "12")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    infoObj <- rxGetInfo(testXdfNew, numRows = 0, startRow = 1, getBlockSizes = FALSE)
  varInfoObj <- rxGetVarInfo(testXdfNew)
  checkEquals(infoObj$fileName, normalizePath(testXdfNew))
  checkEquals(infoObj$numRows, 100)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["y"]], structure(list(varType = "numeric", storage = "float32", low = 15.3615589141846, high = 64.3370056152344), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xnum2"]], structure(list(varType = "numeric", storage = "float32", low = -2.13649392127991, high = 2.16860032081604), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["xfac2"]], structure(list(varType = "factor", storage = "uint32", low = 1, high = 10, levels = c("f", "c", "d", "e", "i", "g", "b", "j", "a", "h")), .Names = c("varType", "storage", "low", "high", "levels"), class = "rxVarInfo")) 

    # Cleanup
    file.remove(testTxt)
    file.remove(testXdfNew) 
}

"test.rxXdfToText.args.reportProgress" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    testTxt <-  tempfile(pattern = ".rxTempXdfToText", fileext = ".txt")

    if (file.exists(testTxt)) file.remove(testTxt)

    # Write to file with reportProgress set to FALSE
    writeOutput1 <-
      capture.output(rxXdfToText(inFile = testXdf, outFile = testTxt,
                                 blocksPerRead = 1, reportProgress = 0))
    checkEquals(length(writeOutput1), 0)
    writeOutput2 <-
      capture.output(rxXdfToText(inFile = testXdf, outFile = testTxt,
                                 blocksPerRead = 1, overwrite = TRUE,
                                 reportProgress = 2))
    if (rxIsExpressEdition())
    {
        checkEquals(length(writeOutput2), 3)
    } else
    {
        checkEquals(length(writeOutput2), 7)
    }
    

    # Cleanup
    file.remove(testTxt)
}

"test.rxXdfToText.multiBlocks.na" <- function()
{
    testXdfOrig <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testXdf <- tempfile(pattern = ".rxXdfToText2", fileext = ".xdf")
  rxFactors(inData = testXdfOrig, sortLevels = TRUE, outFile = testXdf)
    testTxt <-  tempfile(pattern = ".rxXdfToText2", fileext = ".txt")
  myDataOrig <- rxReadXdf(testXdf)
  myDataOrig$ylogical <- as.logical(myDataOrig$ylogical)
  myDataOrig$xfac1 <- as.integer(as.character(myDataOrig$xfac1))

  #Work in progress
  rxXdfToText(inFile=testXdf, outFile=testTxt,  blocksPerRead=1, reportProgress=0)
  myDataTest <- read.csv(file=testTxt, stringsAsFactors=TRUE)
  checkEquals(myDataOrig, myDataTest)

    ### Write to file with missing values written as 'M'
  rxXdfToText(inFile=testXdf, outFile=testTxt,  blocksPerRead=1, na="M", 
    overwrite = TRUE, reportProgress=0)
  myDataTest <- read.csv(file=testTxt, na.strings="M")
  checkEquals(myDataOrig, myDataTest)

  # Cleanup
    file.remove(testTxt)
  file.remove(testXdf)
}

"test.rxXdfToText.args.col.names" <- function()
{
  inXdfFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCsvFile <- tempfile(pattern = "xdfToTextTest",  fileext = ".csv")
  #testXdfFile <- tempfile(pattern = "xdfToText",  fileext = ".xdf")
  # Simple case
  rxXdfToText(inFile = inXdfFile, outFile = testCsvFile, 
    overwrite = TRUE, col.names = FALSE)
  
  testDF <- rxImport(inData = testCsvFile, stringsAsFactors = TRUE)
  # Check to make sure column names were removed
  checkEquals(names(testDF), paste("V", 1:9, sep = ""))
  # Check to make sure data is the same
  testOrigDF <- rxReadXdf(file= inXdfFile)
  names(testOrigDF) <- names(testDF)
  checkEquals(testOrigDF, testDF)

  # More complicated case
  rxXdfToText(inFile = inXdfFile, outFile = testCsvFile,
    varsToKeep = c("y", "ylogical", "xnum1"),
    rowSelection = xnum1 > 0,
    transforms = list(xnum3 = xnum1*3),
    overwrite = TRUE, col.names = FALSE )
  
  testDF <- rxImport(inData = testCsvFile, stringsAsFactors = TRUE, defaultDecimalColType = "numeric")
  # Check to make sure column names were removed
  checkEquals(names(testDF), paste("V", 1:4, sep = ""))
  # Check to make sure data is the same
  testOrigDF <- rxReadXdf(file= inXdfFile)
  testOrigDF <- testOrigDF[testOrigDF$xnum1 > 0, c("y", "ylogical", "xnum1")]
  testOrigDF$xnum3 <- testOrigDF$xnum1*3
  testDF$V2 <- factor(as.character(testDF$V2), levels = c("TRUE", "FALSE"))
  row.names(testOrigDF) <- NULL
  names(testOrigDF) <- names(testDF)
  checkEquals(testOrigDF, testDF)
  
  file.remove(testCsvFile)
}
