# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Functions to create test data
#
############################################################################################
## This file, when comments are removed, creates the following test xdf files: 
##  test100r5b.xdf  An xdf with 100 rows and 5 blocks.  
##  test100r1b.xdf  The xdf file with 100 rows and 1 block
##  test100r5bStartMiss.xdf The xdf file with the missing values at the beginning
##  test100r5bEndMiss.xdf The xdf file with the missing values at the end
##  test77r5bNoMiss.xdf  The xdf file with missings removed resulting in 5 unequal-sized blocks
##  test77r1bNoMiss.xdf  The xdf file with missings removed in 1 block
## All data sets have the same variables
## xnum1: numeric, 100 random normal
## xnum2: numeric, 100 random normal
## xint1: integer, 100 random uniform between 5 and 15
## xfac1: factor version of xint1
## xfac2: factor version of xint1 with factor levels "a" through "j"
## y: a numeric variable created by the expression 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)
## ylogical: a logical created by the expression y > 35
## ybinint: a 0,1 integer version of ylogical
## xnum2miss: numeric version of xnum2 where values greater than .5 are missing
## 
## Make sure RevoScaleR is loaded
# require(RevoScaleR)
# Set directory below to the location you want to create the data sets
#setwd("C:/Users/Sue/bigAnalytics/trunk/revoAnalytics/inst/unitTestData")
## Create a data frame with a variety of variables
#set.seed(100)
#xnum1 <- rnorm(100)
#xnum2 <- rnorm(100)
#xint1 <- as.integer(runif(100, 5,15))
#xfac1 <- as.factor(xint1)
#xfac2 <- xfac1
#levels(xfac2) <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
#y <- 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)
#ylogical <- y > 35
#ybinint <- as.integer(ylogical)
#xnum2miss <- xnum2
#xnum2miss[which(xnum2 > .5)] <- NA
#myDataFrame <- data.frame(y=y, ylogical=ylogical, ybinint=ybinint, xnum1=xnum1, xnum2=xnum2, 
    #xnum2miss=xnum2miss, xint1=xint1, xfac1=xfac1, xfac2=xfac2)
  #
##Write out to a text file removing row names
    #
#write.csv(x = myDataFrame, file = "test100r.txt", row.names = FALSE)
#
## Create xdf file with 5 blocks
#rxTextToXdf(inFile="test100r.txt",outFile="test100r5b", rowsPerRead=20, stringsAsFactors=TRUE)
#rxGetInfo("test100r5b")
    #
## Create xdf file with 1 block
#rxTextToXdf(inFile="test100r.txt", outFile="test100r1b", stringsAsFactors=TRUE)
#rxGetInfo("test100r1b")
    #
## Modify data frame to change order of missing values    
#myDataFrameNoMiss <- myDataFrame[which(!is.na(myDataFrame$xnum2miss)),]
#myDataFrameMiss <- myDataFrame[which(is.na(myDataFrame$xnum2miss)),]
    #
## Create xdf file with 5 blocks, all missings in first block
#myDataFrameStartMiss <- rbind(myDataFrameMiss, myDataFrameNoMiss)
#write.csv(x = myDataFrameStartMiss, file = "test100r.txt", row.names = FALSE)
#rxTextToXdf(inFile="test100r.txt", outFile="test100r5bStartMiss", stringsAsFactors=TRUE, rowsPerRead=20,
    #colClasses=c(xnum2miss="float32"))
#rxGetInfo("test100r5bStartMiss")
#
## Create xdf file with all missings at end    
#myDataFrameEndMiss <- rbind(myDataFrameNoMiss, myDataFrameMiss)
#write.csv(x = myDataFrameEndMiss, file = "test100r.txt", row.names = FALSE)
#rxTextToXdf(inFile="test100r.txt", outFile="test100r5bEndMiss", stringsAsFactors=TRUE, rowsPerRead=20)
#rxGetInfo("test100r5bEndMiss") 
#
## Remove missings to create unequal block size
#rxDataStep(inFile="test100r5b", outFile="test77r5bNoMiss", removeMissings=TRUE)
#rxGetInfo("test77r5bNoMiss", getBlockSizes=TRUE) 
#
## Xdf file with no missings and one block
#write.csv(x = myDataFrameNoMiss, file = "test100r.txt", row.names = FALSE)
#rxTextToXdf(inFile="test100r.txt", outFile="test77r1bNoMiss", stringsAsFactors=TRUE)
#rxGetInfo("test77r1bNoMiss", getBlockSizes=TRUE) 