# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Functions to create test data
#
############################################################################################
# This file creates the following test xdf and text files:
# This file is an extension of the test100r series, but adds: numeric with  missing, character and factor with missings
#
#  richTest100r5b.xdf  An xdf with 100 rows and 5 blocks. 
#  richTest100r1b.xdf  The xdf file with 100 rows and 1 block
#  test61r5bNoMiss.xdf  The xdf file with missings removed resulting in 5 unequal-sized blocks
#  test61r1bNoMiss.xdf  The xdf file with missings removed in 1 block
#  richTest100rComma.txt Comma delimited text file of all 100 rows of data
#  richTest100rCommaNA.txt Comma delimited text file, 100 rows, missing Value= "NA"
#  richTest100rSemiColon.txt semi-colon delimited text file of all 100 rows of data
#  richTest100rTab.txt tab delimited text file of all 100 rows of data
###-- NOTE: need gtools and gdata installed for next one -- code section commented out below
#  richTest100rFW.dat The text file in fixed width - 100 rows of data; accompanying .sts file 

# Outside this script, statTransfer is used to create .sas7bdat file from richTest100rComma.txt
# richTest100rComma.sas7bdat

# All data sets have the same variables
# xnum1: numeric, 100 random normal
# xnum2: numeric, 100 random normal
# xint1: integer, 100 random uniform between 5 and 15
# xfac1: factor version of xint1
# xfac2: factor version of xint1 with factor levels "a" through "j"
# y: a numeric variable created by the expression 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)
# y2: a numeric variable created by the expression 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)+I(zfac=="Washington")+2*I(zfac=="California")+25*I(zfac=="Alaska")
# ylogical: a logical created by the expression y > 35
# ybinint: a 0,1 integer version of ylogical
# xnum2miss: numeric version of xnum2 where values greater than .5 are missing
# zchar1: character of state names sampled from ("Washington","California","Alaska") with probabilities:(.5,.3,.1)
# zfac: factor of zchar1; with NA's at: [c(3:12,34,54,65,77,66,88:90,100)]
#
# Make sure RevoScaleR is loaded
 require(RevoScaleR)

#Set directory below to the location you want to create the data sets
#setwd("C:/Users/Sue/bigAnalytics/trunk/revoAnalytics/inst/unitTestData")
# Create a data frame with a variety of variables
set.seed(100)
xnum1 <- rnorm(100)
xnum2 <- rnorm(100)

zchar1 <- sample(c("Washington","North Carolina","Alaska"),100,TRUE,c(.5,.3,.1))
zfac <- factor(zchar1)
zfac[c(3:12,34,54,65,77,66,88:90,100)] <- NA

xint1 <- as.integer(runif(100, 5,15))
xfac1 <- as.factor(xint1)
xfac2 <- xfac1
levels(xfac2) <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
y <- 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)
y2 <- 1 + xnum1*2 + xnum2*3  + xint1*4 + rnorm(100)+I(zfac=="Washington")+2*I(zfac=="California")+25*I(zfac=="Alaska")
ylogical <- y > 35
ybinint <- as.integer(ylogical)
xnum2miss <- xnum2
xnum2miss[which(xnum2 > .5)] <- NA
myDataFrame <- data.frame(y=y, y2=y2,ylogical=ylogical, ybinint=ybinint, xnum1=xnum1, xnum2=xnum2,
    xnum2miss=xnum2miss, xint1=xint1, xfac1=xfac1, xfac2=xfac2,z.char1=zchar1,zfac=zfac)

# write fixed format file - You  need to install gtools and gdata from CRAN and these only seem to work on 64-bit version
#library(gdata)
#library(gtools)
#write.fwf(myDataFrame,"richTest100rFW.dat",na=" ",colnames=FALSE)
 
#Write out to a text file removing row names
write.table(x = myDataFrame, file = "richTest100rComma.txt", row.names = FALSE,sep=",",na=" " )
write.table(x = myDataFrame, file = "richTest100rCommaNA.txt", row.names = FALSE,sep=",",na="NA" )
write.table(x = myDataFrame, file = "richTest100rSemiColon.txt", row.names = FALSE,sep=";",na=" ")
write.table(x = myDataFrame, file = "richTest100rTab.txt", row.names = FALSE,sep="\t",na=" ")

write.table(x = myDataFrame, file = "richTest200rComma.txt", row.names = FALSE,sep=",",na=" ")
write.table(x = myDataFrame, file = "richTest200rSemiColon.txt", row.names = FALSE,sep=";",na=" ")
write.table(x = myDataFrame, file = "richTest200rComma.txt", row.names = FALSE,sep=",",na=" ",col.names=FALSE,append=TRUE )
write.table(x = myDataFrame, file = "richTest200rSemiColon.txt", row.names = FALSE,sep=";",na=" ",col.names=FALSE,append=TRUE)

write.table(x = myDataFrame, file = "richTest300rComma.txt", row.names = FALSE,sep=",",na=" " )
write.table(x = myDataFrame, file = "richTest300rSemiColon.txt", row.names = FALSE,sep=";",na=" ")
write.table(x = myDataFrame, file = "richTest300rComma.txt", row.names = FALSE,sep=",",na=" ",col.names=FALSE,append=TRUE )
write.table(x = myDataFrame, file = "richTest300rSemiColon.txt", row.names = FALSE,sep=";",na=" ",col.names=FALSE,append=TRUE)
write.table(x = myDataFrame, file = "richTest300rComma.txt", row.names = FALSE,sep=",",na=" ",col.names=FALSE,append=TRUE )
write.table(x = myDataFrame, file = "richTest300rSemiColon.txt", row.names = FALSE,sep=";",na=" ",col.names=FALSE,append=TRUE)

# Create xdf file with 5 blocks
rxTextToXdf(inFile="richTest100rComma.txt",outFile="richTest100r5b", rowsPerRead=20, stringsAsFactors=TRUE,colClasses=c(z.char1="character"))
rxGetInfo("richTest100r5b",numRows=10)
   
# Create xdf file with 1 block
rxTextToXdf(inFile="richTest100rComma.txt", outFile="richTest100r1b", stringsAsFactors=TRUE,colClasses=c(z.char1="character"))
rxGetInfo("richTest100r1b",getVarInfo=TRUE)
   
## Modify data frame to change order of missing values   
myDataFrameNoMiss <- myDataFrame[which(!is.na(myDataFrame$xnum2miss)&!is.na(myDataFrame$zfac)),]
## choose out data where xnum2miss is missing
#myDataFrameMiss <- myDataFrame[which(is.na(myDataFrame$xnum2miss)),]
   #
## Create xdf file with 5 blocks, all missings in xnum2miss in first block
#myDataFrameStartMiss <- rbind(myDataFrameMiss, myDataFrameNoMiss)
#file.remove("richTest100r5bStartMiss.xdf")
#write.csv(x = myDataFrameStartMiss, file = "richTest100rComma.txt", row.names = FALSE,na=" ")
#rxTextToXdf(inFile="richTest100rComma.txt", outFile="richTest100r5bStartMiss", stringsAsFactors=TRUE, rowsPerRead=20,
    #colClasses=c(xnum2miss="float32"))
#rxGetInfo("richTest100r5bStartMiss",getVarInfo=TRUE,numRows=100)
#
## Create xdf file with all missings in xnum2miss at end   
#myDataFrameEndMiss <- rbind(myDataFrameNoMiss, myDataFrameMiss)
#write.csv(x = myDataFrameEndMiss, file = "richTest100rComma.txt", row.names = FALSE,na=" " )
#rxTextToXdf(inFile="richTest100rComma.txt", outFile="richTest100r5bEndMiss", stringsAsFactors=TRUE, rowsPerRead=20)
#
#rxGetInfo("richTest100r5bEndMiss")


# Remove missings to create unequal block size
rxDataStep(inData="richTest100r5b", outFile="richTest61r5bNoMiss", removeMissings=TRUE)
rxGetInfo("richTest61r5bNoMiss", getBlockSizes=TRUE)

# Xdf file with no missings and one block
write.csv(x = myDataFrameNoMiss, file = "TmprichTest100r.txt", row.names = FALSE,na=" ")
rxTextToXdf(inFile="TmprichTest100r.txt", outFile="richTest61r1bNoMiss", stringsAsFactors=TRUE,colClasses=c(z.char1="character"))
rxGetInfo("richTest61r1bNoMiss", getBlockSizes=TRUE,getVarInfo=TRUE)


file.remove("TmprichTest100r.txt")
