#!/bin/bash


# --- defaults ---------------------------------------------------------
CLUSTER_GUID_DIR="%clusterGUIDDir%"
HPC_SERVER_CLASS="%hpcServerClass%"
LOAD_SITE_FILE="%loadSiteFile%"
MRS_HADOOP_PATH="%mrsHadoopPath%"
REVO_PATH="%revoPath%"
DFS_GUID_DIR="%dfsGUIDDir%"
HADOOP_SWITCHES="%hadoopSwitches%"
RUN_AS_USER_MODE="%runAsUserMode%"


# --- command-line options ---------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    -clusterGUIDDir)  CLUSTER_GUID_DIR="$2"; shift 2;;
    -hpcServerClass)  HPC_SERVER_CLASS="$2"; shift 2;;
    -loadSiteFile)    LOAD_SITE_FILE="$2"; shift 2;;
    -mrsHadoopPath)   MRS_HADOOP_PATH="$2"; shift 2;;
    -revoPath)        REVO_PATH="$2"; shift 2;;
    -dfsGUIDDir)      DFS_GUID_DIR="$2"; shift 2;;
    -hadoopSwitches)  HADOOP_SWITCHES="$2"; shift 2;;
    -runAsUserMode)   RUN_AS_USER_MODE="$2"; shift 2;;
    *) echo "WARNING! unknown option $1"; shift;;
  esac
done

# --- script logic -----------------------------------------------------


#Cannot use vanilla R here, since we need some options from Rprofile
$("${REVO_PATH}" --vanilla --slave -e 'cat(system.file("utils/spark/SafeGuard.out", package="RevoScaleR"))') "${REVO_PATH}" --slave -e "library(RevoScaleR); RevoScaleR:::rxLogHadoopMRJob('${CLUSTER_GUID_DIR}', '${HPC_SERVER_CLASS}')" 2>&1 > /dev/null
touch ${CLUSTER_GUID_DIR}/lastOutputBlock.txt
echo "uninitialized" > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
MYUUID=$(cat /proc/sys/kernel/random/uuid)
if ${LOAD_SITE_FILE} ; then
  # load environment file
  SITE_PATH=$(dirname "$(readlink -nf ${MRS_HADOOP_PATH})")/RevoHadoopEnvVars.site
  if [ ! -e "${SITE_PATH}" ] ; then
    SITE_PATH="${HOME}/.RevoHadoopEnvVars.site"
  fi
  if [ -e "${SITE_PATH}" ] ; then
    while read LINE ; do
      LINE="${LINE/export /}"
      VAR_NAME=${LINE%%=*}
      VAR_VALUE=$(eval "echo \${${VAR_NAME}}")
      if [ -z "${VAR_VALUE}" ] ; then
        export ${LINE}
      fi
    done < "${SITE_PATH}"
  else
    echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    echo "cannot find RevoHadoopEnvVars.site, aborting" >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
    exit 1
  fi
fi
echo "queued" > ${CLUSTER_GUID_DIR}/$MYUUID
mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
touch ${CLUSTER_GUID_DIR}/MergedOut0.txt
if [ -z "$HADOOP_STREAMING" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    echo "HADOOP_STREAMING environment variable must be set.  Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
    exit
fi
if [ ! -f "$HADOOP_STREAMING" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    echo "HADOOP_STREAMING environment variable must point to a valid file.  Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
    exit
fi
MKDIR_OUTPUT=`${MRS_HADOOP_PATH} fs -mkdir ${DFS_GUID_DIR} 2>&1`
MKDIR_VALUE=$?
#echo $?
if [ ${MKDIR_VALUE} -eq 0 ]
then
    if ${RUN_AS_USER_MODE} ; then
      ${MRS_HADOOP_PATH} fs -chmod a+rwx ${DFS_GUID_DIR} 2>&1 > /dev/null
    fi
else
    GREP_TEST=`echo ${MKDIR_OUTPUT} | grep -i 'Permission denied' | wc -l`
    if [ ${GREP_TEST} -gt 0 ]
    then
        MYUUID=$(cat /proc/sys/kernel/random/uuid)
        echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
        mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
        echo "No permissions to create ${DFS_GUID_DIR} Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
        exit
    fi
    DIR_EXISTANCE_TEST=`${MRS_HADOOP_PATH} fs -ls ${DFS_GUID_DIR} 2>&1`
    GREP_DIR_EXISTANCE_TEST=`echo $DIR_EXISTANCE_TEST | grep -i 'No such file or directory' | wc -l`
    if [ ${GREP_TEST} -gt 0 ]
    then
        MYUUID=$(cat /proc/sys/kernel/random/uuid)
        echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
        mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
        echo "${DFS_GUID_DIR} does not exist, and unable to create.  Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
        exit
    fi
fi
(${MRS_HADOOP_PATH}  jar ${HADOOP_STREAMING} -files ${CLUSTER_GUID_DIR}/input.R,${CLUSTER_GUID_DIR}/00.RInData,${CLUSTER_GUID_DIR}/scaleR.sh -Dmapred.job.map.memory.mb=2500 -Dmapred.task.timeout=3600000 -Dmapred.reduce.tasks=0 ${REVOHADOOPSWITCHES} ${HADOOP_SWITCHES} -inputformat org.apache.hadoop.mapred.lib.NLineInputFormat -input ${DFS_GUID_DIR}/input -output ${DFS_GUID_DIR}/output -mapper scaleR.sh  >  ${CLUSTER_GUID_DIR}/MergedOut0.txt  2>&1;
>/dev/null 2>&1)>/dev/null 2>&1 &
COMMANDPID=$!
COUNTER=0
while [ `ps -o pid= -p $COMMANDPID` ] && [ $COUNTER -lt 90 ]; do
    JOBIDLINE=$(cat ${CLUSTER_GUID_DIR}/MergedOut0.txt | grep -m 1 ' Running job:')
    JOBID=$(echo "$JOBIDLINE" | sed -n -e 's/^.* Running job: //p')
    echo $JOBID > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
    sleep 1s
    if [ ! -z "$JOBID" ]; then
        echo $JOBID
        exit
    fi
    let COUNTER=COUNTER+1
done
