#!/bin/bash

# --- defaults ---------------------------------------------------------
CLUSTER_GUID_DIR="%clusterGUIDDir%"
MRS_HADOOP_PATH="%mrsHadoopPath%"


# --- command-line options ---------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    -clusterGUIDDir) CLUSTER_GUID_DIR="$2"; shift 2;;
    -mrsHadoopPath)   MRS_HADOOP_PATH="$2"; shift 2;;
    *) echo "WARNING! unknown option $1"; shift;;
  esac
done

# --- script logic -----------------------------------------------------

function get_error_message {
    local JOBID="$1"
    local APPID="application${JOBID##job}"
    yarn application -status "${APPID}" 2>/dev/null | grep Diagnostics | sed 's/^.*Diagnostics[^:]://'
}

JOBID=$(cat ${CLUSTER_GUID_DIR}/jobIdentifier.txt)

if [ -f ${CLUSTER_GUID_DIR}/jobStatus.txt ]
then
    set -- $(<${CLUSTER_GUID_DIR}/jobStatus.txt)
    MYSTATE=$1
    if [ $MYSTATE == "canceled" ] || [ $MYSTATE == "finished" ] || [ $MYSTATE == "failed" ]
    then
        get_error_message "${JOBID}" > ${CLUSTER_GUID_DIR}/yarn-error-message.txt
        exit
    fi
fi

MYUUID=$(cat /proc/sys/kernel/random/uuid)
echo "undetermined" > ${CLUSTER_GUID_DIR}/$MYUUID
mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
JOBSTATE=$(${MRS_HADOOP_PATH} job -list all | grep $JOBID | cut -f2)
if [ $JOBSTATE == "1" -o $JOBSTATE == "RUNNING" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "running" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
fi
if [ $JOBSTATE == "2" -o $JOBSTATE == "SUCCEEDED" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "finished" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
fi
if [ $JOBSTATE == "3" -o $JOBSTATE == "FAILED" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    get_error_message "${JOBID}" > ${CLUSTER_GUID_DIR}/yarn-error-message.txt
fi
if [ $JOBSTATE == "4" -o $JOBSTATE == "PREP" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "queued" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
fi
if [ $JOBSTATE == "5" -o $JOBSTATE == "KILLED" ]
then
    MYUUID=$(cat /proc/sys/kernel/random/uuid)
    echo "canceled" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    get_error_message "${JOBID}" > ${CLUSTER_GUID_DIR}/yarn-error-message.txt
fi
