#!/bin/bash


# --- defaults ---------------------------------------------------------
HADOOP_R_PATH="%hadoopRPath%"
HADOOP_R_PATH_ARGS="%hadoopRPathArgs%"
TRACE_LEVEL="%traceLevel%"
SAFE_GUARD="%SafeGuard%"
MRS_HADOOP_PATH="%mrsHadoopPath%"
RUN_AS_USER_MODE="%runAsUserMode%"


# --- command-line options ---------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    -hadoopRPath)     HADOOP_R_PATH="$2"; shift 2;;
    -traceLevel)      TRACE_LEVEL="$2"; shift 2;;
    -hadoopRPathArgs) HADOOP_R_PATH_ARGS="$2"; shift 2;;
    -safeGuard)       SAFE_GUARD="$2"; shift 2;;
    -mrsHadoopPath)   MRS_HADOOP_PATH="$2"; shift 2;;
    -runAsUserMode)   RUN_AS_USER_MODE="$2"; shift 2;;
    *) echo "WARNING! unknown option $1"; shift;;
  esac
done


# --- script logic -----------------------------------------------------


set -x
set -v
set > /dev/stderr
# exit on error
set -e
if [ -z "$mapred_task_partition" ]; then
	# mapred_task_partition is unset - we must be running YARN/MR2
    echo 'mapred_task_partition is unset - we must be running YARN/MR2'
	export mapred_task_partition=$mapreduce_task_partition
fi
MY_RANK=`expr ${mapred_task_partition} + 1`
echo "MY_RANK='${MY_RANK}'"
ls -lR > /dev/stderr
${SAFE_GUARD} ${HADOOP_R_PATH} ${HADOOP_R_PATH_ARGS} "rxCommArgs(rxNetworkShareJobDirectory='${PWD}',rxComputeWhere='manyNodes',rxParametricRank='${MY_RANK}',rxClusterTarget='hadoopmr',rxTraceLevel=${TRACE_LEVEL})"
ROUT_FILE=`ls rxElem*.ROutData`
# the next lines are for debugging
echo "ROUT_FILE='${ROUT_FILE}'"
ls -l rxElem*.ROutData > /dev/stderr
if [ -z "$mapred_output_dir" ]; then
	# mapred_output_dir is unset - we must be running YARN/MR2
    echo 'mapred_output_dir is unset - we must be running YARN/MR2'
	export mapred_output_dir=$mapreduce_output_fileoutputformat_outputdir
fi
echo $mapred_output_dir > /dev/stderr
# ${MRS_HADOOP_PATH} fs -ls  ${mapred_output_dir} > /dev/stderr
whoami > /dev/stderr
# Ideally, we would write the outputs to ${mapred_work_output_dir}, which is ${mapred.output.dir}/_temporary/${mapred.task.id},
# but there are permission issues.
# The following work-around runs the risk of two task attempts copying to the same file,
# but we handle that case below.
ROUTDATA_DIR=`echo ${mapred_output_dir} | sed s/\\\\/output/-output/`
# ROUTDATA_DIR=${mapred_work_output_dir}
echo $ROUTDATA_DIR > /dev/stderr
# ${MRS_HADOOP_PATH} fs -ls ${mapred_output_dir} > /dev/stderr
# ${MRS_HADOOP_PATH} fs -ls ${mapred_output_dir}/_temporary > /dev/stderr
# ${MRS_HADOOP_PATH} fs -ls ${mapred_work_output_dir} > /dev/stderr
if ${MRS_HADOOP_PATH} fs -mkdir $ROUTDATA_DIR; then
	# the hadoop mkdir returned success
    echo 'mkdir was successful'
    if ! ${RUN_AS_USER_MODE} ; then
      ${MRS_HADOOP_PATH}  fs -chmod a+rwx ${ROUTDATA_DIR}
    fi
else
	# the hadoop mkdir returned failure - assume the directory already exists -
	# if the directory does not exist, the subsequent copyFromLocal will fail,
	# and this attempt will exit as failed
    echo '${MRS_HADOOP_PATH} mkdir failed - assume the dir already exists'
fi # 2> /dev/null
# This logic handles the case where another attempt of this task
# has already copied ${ROUT_FILE} to $ROUTDATA_DIR
if ${MRS_HADOOP_PATH} fs -copyFromLocal ${ROUT_FILE} $ROUTDATA_DIR; then
	# the ${MRS_HADOOP_PATH} copyFromLocal returned success
    echo 'copyFromLocal was successful'
else
	# the hadoop copyFromLocal returned failure - check if the file already exists
	if ${MRS_HADOOP_PATH} fs -test -e $ROUTDATA_DIR/${ROUT_FILE}; then
		echo 'the output file exists in HDFS - another task attempt must have already generated it'
	else
		echo 'the output file does not exist in HDFS - we must exit'
		exit 255 # exit as failed
	fi
fi # 2> /dev/null
if ! ${RUN_AS_USER_MODE} ; then
  ${MRS_HADOOP_PATH} fs -chmod a+rw ${ROUTDATA_DIR}/${ROUT_FILE}
fi
# ${MRS_HADOOP_PATH} fs -ls ${ROUTDATA_DIR} > /dev/stderr
# ${MRS_HADOOP_PATH} fs -ls ${mapred_output_dir}/_temporary > /dev/stderr
