#!/bin/bash


# --- defaults ---------------------------------------------------------
CLUSTER_GUID_DIR="%clusterGUIDDir%"
HPC_SERVER_CLASS="%hpcServerClass%"
LOAD_SITE_FILE="%loadSiteFile%"
MRS_HADOOP_PATH="%mrsHadoopPath%"
REVO_PATH="%revoPath%"
RUN_AS_USER_MODE="%runAsUserMode%"
DFS_GUID_DIR="%dfsGUIDDir%"
JOB_GUID="%jobGUID%"
HADOOP_R_PATH="%hadoopRPath%"
SHARE_DIR="%shareDir%"
USER="%user%"
R_SESSION_PID="%rSessionPid%"
APP_ID="%appId%"
WAIT="%wait%"
NAME_NODE="%nameNode%"
PORT="%port%"
IDLE_TIMEOUT="%idleTimeout%"
EXECUTOR_CORES="%executorCores%"
EXECUTOR_MEM="%executorMem%"
DRIVER_MEM="%driverMem%"
OVERHEAD_MEM="%overheadMem%"
NUM_EXECUTORS="%numExecutors%"
EXTRA_CONFIG="%extraConfig%"
PERSISTENT_RUN="%persistentRun%"


# --- command-line options ---------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    -clusterGUIDDir)  CLUSTER_GUID_DIR="$2"; shift 2;;
    -hpcServerClass)  HPC_SERVER_CLASS="$2"; shift 2;;
    -loadSiteFile)    LOAD_SITE_FILE="$2"; shift 2;;
    -mrsHadoopPath)   MRS_HADOOP_PATH="$2"; shift 2;;
    -revoPath)        REVO_PATH="$2"; shift 2;;
    -runAsUserMode)   RUN_AS_USER_MODE="$2"; shift 2;;
    -dfsGUIDDir)      DFS_GUID_DIR="$2"; shift 2;;
    -jobGUID)         JOB_GUID="$2"; shift 2;;
    -hadoopRPath)     HADOOP_R_PATH="$2"; shift 2;;
    -shareDir)        SHARE_DIR="$2"; shift 2;;
    -user)            USER="$2"; shift 2;;
    -rSessionPid)     R_SESSION_PID="$2"; shift 2;;
    -appId)           APP_ID="$2"; shift 2;;
    -wait)            WAIT="$2"; shift 2;;
    -nameNode)        NAME_NODE="$2"; shift 2;;
    -port)            PORT="$2"; shift 2;;
    -idleTimeout)     IDLE_TIMEOUT="$2"; shift 2;;
    -executorCores)   EXECUTOR_CORES="$2"; shift 2;;
    -executorMem)     EXECUTOR_MEM="$2"; shift 2;;
    -driverMem)       DRIVER_MEM="$2"; shift 2;;
    -overheadMem)     OVERHEAD_MEM="$2"; shift 2;;
    -numExecutors)    NUM_EXECUTORS="$2"; shift 2;;
    -extraConfig)     EXTRA_CONFIG="$2"; shift 2;;
    -persistentRun)   PERSISTENT_RUN="$2"; shift 2;;
    *) echo "WARNING! unknown option $1"; shift;;
  esac
done

# --- script logic -----------------------------------------------------


#Cannot use vanilla R here, since we need some options from Rprofile
"$("${REVO_PATH}" --vanilla --slave -e 'cat(system.file("utils/spark/SafeGuard.out", package="RevoScaleR"))')" "${REVO_PATH}" --slave -e "library(RevoScaleR); RevoScaleR:::rxLogHadoopMRJob('${CLUSTER_GUID_DIR}', '${HPC_SERVER_CLASS}')" 2>&1 > /dev/null
touch ${CLUSTER_GUID_DIR}/lastOutputBlock.txt
echo "uninitialized" > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
MYUUID=$(cat /proc/sys/kernel/random/uuid)
if ${LOAD_SITE_FILE} ; then
  # load environment file
  SITE_PATH=$(dirname "$(readlink -nf ${MRS_HADOOP_PATH})")/RevoHadoopEnvVars.site
  if [ ! -e "${SITE_PATH}" ] ; then
    SITE_PATH="${HOME}/.RevoHadoopEnvVars.site"
  fi
  if [ -e "${SITE_PATH}" ] ; then
    while read LINE ; do
      LINE="${LINE/export /}"
      VAR_NAME=${LINE%%=*}
      VAR_VALUE=$(eval "echo \${${VAR_NAME}}")
      if [ -z "${VAR_VALUE}" ] ; then
        export ${LINE}
      fi
    done < "${SITE_PATH}"
  else
    echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
    mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
    echo "cannot find RevoHadoopEnvVars.site, aborting" >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
    exit 1
  fi
fi
echo "queued" > ${CLUSTER_GUID_DIR}/$MYUUID
mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
touch ${CLUSTER_GUID_DIR}/MergedOut0.txt
MKDIR_OUTPUT=`${MRS_HADOOP_PATH} fs -mkdir ${DFS_GUID_DIR} 2>&1`
MKDIR_VALUE=$?
#echo $?
if [ ${MKDIR_VALUE} -eq 0 ]
then
    if ${RUN_AS_USER_MODE} ; then
      ${MRS_HADOOP_PATH} fs -chmod a+rwx ${DFS_GUID_DIR} 2>&1 > /dev/null
    fi
else
    GREP_TEST=`echo ${MKDIR_OUTPUT} | grep -i 'Permission denied' | wc -l`
    if [ ${GREP_TEST} -gt 0 ]
    then
        MYUUID=$(cat /proc/sys/kernel/random/uuid)
        echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
        mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
        echo "No permissions to create ${DFS_GUID_DIR} Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
        exit
    fi
    DIR_EXISTANCE_TEST=`${MRS_HADOOP_PATH} fs -ls ${DFS_GUID_DIR} 2>&1`
    GREP_DIR_EXISTANCE_TEST=`echo $DIR_EXISTANCE_TEST | grep -i 'No such file or directory' | wc -l`
    if [ ${GREP_TEST} -gt 0 ]
    then
        MYUUID=$(cat /proc/sys/kernel/random/uuid)
        echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
        mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
        echo "${DFS_GUID_DIR} does not exist, and unable to create.  Exiting." >> ${CLUSTER_GUID_DIR}/MergedOut0.txt
        exit
    fi
fi
FIFO_TO_SPARK=${SHARE_DIR}/RevoSparkFifoToSpark-${USER}-${R_SESSION_PID}-${APP_ID}
FIFO_FROM_SPARK=${SHARE_DIR}/RevoSparkFifoFromSpark-${USER}-${R_SESSION_PID}-${APP_ID}
# In non-blocking mode, multi jobs can run simultaneously in one R session, append job GUID to make each job unique.
if ! ${WAIT} ; then
  FIFO_TO_SPARK=${FIFO_TO_SPARK}-${JOB_GUID}
  FIFO_FROM_SPARK=${FIFO_FROM_SPARK}-${JOB_GUID}
fi
# Only clean Spark app associated with current R session in blocking mode.
# In non-blocking mode, we allow multi Spark apps within same R session.
if ${WAIT} ; then
    $(${REVO_PATH} --vanilla --slave -e 'cat(system.file("utils/spark/RevoSparkSubmitKiller", package="RevoScaleR"))') ${USER} ${R_SESSION_PID} ${APP_ID} ${JOB_GUID} ${WAIT} clean session
fi
($("${REVO_PATH}" --vanilla --slave -e 'cat(system.file("utils/spark/RevoSparkHPCLauncher", package="RevoScaleR"))') \
 "${FIFO_TO_SPARK}" \
 "${FIFO_FROM_SPARK}" \
 $(wc -l < "${CLUSTER_GUID_DIR}/input/scaleR-input.txt") \
 "${DFS_GUID_DIR}/output" \
 "${CLUSTER_GUID_DIR}/input.R" \
 "${CLUSTER_GUID_DIR}/00.RInData" \
 "${CLUSTER_GUID_DIR}/scaleR.sh" \
 "${HADOOP_R_PATH}" \
 "${NAME_NODE}" \
 "${PORT}" \
 "${USER}" \
 "${R_SESSION_PID}" \
 "${APP_ID}" \
 "${JOB_GUID}" \
 "${WAIT}" \
 "${IDLE_TIMEOUT}" \
 "${EXECUTOR_CORES}" \
 "${EXECUTOR_MEM}" \
 "${DRIVER_MEM}" \
 "${OVERHEAD_MEM}" \
 "${NUM_EXECUTORS}" \
 "${CLUSTER_GUID_DIR}" \
 "${EXTRA_CONFIG}" > "${CLUSTER_GUID_DIR}/MergedOut0.txt" 2>&1;
 # Send stop to spark if we are in non-persist mode
 if ! ${PERSISTENT_RUN} ; then
     $(${REVO_PATH} --vanilla --slave -e 'cat(system.file("utils/spark/RevoSparkSubmitKiller", package="RevoScaleR"))') ${USER} ${R_SESSION_PID} ${APP_ID} ${JOB_GUID} ${WAIT} stop ${SHARE_DIR}/RevoSparkFifoToSpark-${USER}-${R_SESSION_PID}-${APP_ID}-${JOB_GUID}>/dev/null 2>&1
 fi)>/dev/null 2>&1 &
COMMANDPID=$!
COUNTER=0
while [ $(ps -o pid= -p $COMMANDPID) ] && [ $COUNTER -lt 90 ]; do
    JOBIDLINE=$(cat ${CLUSTER_GUID_DIR}/MergedOut0.txt | grep -m 1 ' Running job:')
    JOBID=$(echo "$JOBIDLINE" | sed -n -e 's/^.* Running job: //p')
    echo $JOBID > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
    sleep 1s
    if [ ! -z "$JOBID" ]; then
        echo $JOBID
        exit
    fi
    let COUNTER=COUNTER+1
done
