require(RevoScaleR)

args <- commandArgs(trailingOnly = TRUE)

workDir <- args[1]
jobRDataPath <- args[2]
numTasks <- as.numeric(args[3])

objectName <- load(jobRDataPath)
jobRData <- get(objectName)$JOB$RData

for (taskId in seq(0, numTasks - 1))
{
    # build input data source
    xdfName <- paste0("PXDF", taskId)
    inDataPath <- paste(workDir, xdfName, sep = "/")
    inData <- RxXdfData(inDataPath)

    # deserialize binary key file and get key data.frame
    keyPath <- paste(workDir, xdfName, "metadata", paste0(xdfName, ".xdfk"), sep = "/")
    rxUtilParams <- list(USE = 'DeserializeSparkDataSet', inputObjectPath = keyPath)
    keyDataSet <- .Call('RxUtil', rxUtilParams)

    # convert keys from data.frame to list
    keys <- as.list(keyDataSet[1,])

    # invoke UDF
    out.data <- NULL
    status <- NULL
    eMsg <- NULL
    wMsgs <- NULL

    status <- tryCatch(
    {
        withCallingHandlers(
        {
            args <- c(keys = list(keys), data = inData, jobRData$funcParams)
            out.data <- do.call(what = jobRData$func, args = args)
            "OK"
        }, warning = function(wMsg)
        {
            wMsgs <- get("wMsgs", pos = -1)
            wMsgs <<- c(wMsgs, conditionMessage(wMsg))
            invokeRestart("muffleWarning")
        })
    }, error = function(eMsg)
    {
        eMsg <<- conditionMessage(eMsg)
        "Error"
    })

    # construct status
    status <- list(status, eMsg, wMsgs)

    # remove column names from keys to reduce list size
    names(keys) <- NULL

    # construct outData
    outData <- list(list(keys = keys, result = out.data, status = status))

    # save outData
    outDataPath <- paste0(workDir, "/iroFile", taskId, ".iro")
    save(outData, file = outDataPath)
}
