// Copyright (c) Microsoft Corporation. All rights reserved.
// Author: Lukasz Bartnik

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

int main (int argc, char ** argv) {


    pid_t pid = fork();

    if (pid < 0) {
        perror("SafeGuard: error in fork()");
        exit(EXIT_FAILURE);
    }

    int child_exit_status = 0;

    // child
    if (pid == 0) {
        // create a new group; R will be the leader
        pid_t new_group_pid = setsid();
        if (new_group_pid < 0) {
            perror("SafeGuard: setsid() error");
            exit(EXIT_FAILURE);
        }

        // execute R; R will start BxlServer in the same group
        // printf("[%d] SafeGuard: new group id = %d\n", getpid(), new_group_pid);
        // printf("[%d] SafeGuard: exec %s\n", getpid(), argv[1]);
        execv(argv[1], argv+1); /* replace that with R */

        // printf("[%d] SafeGuard: exec didn't work\n", getpid());
        exit(EXIT_FAILURE);
    }
    else {
        // fclose(stdin); TODO do we need it at all?
        // printf("[%d] SafeGuard: waiting for R process %d\n", getpid(), pid);

        // wait for R to exit normally
        waitpid(pid, &child_exit_status, 0);
        // R has exited, give BxlServer time to exit gracefully
        // TODO should we spend some time to wait?
        //sleep(2);

        // just in case BxlServer is in a deadlock, send SIGKILL
        // printf("[%d] SafeGuard: sending kill to group %d\n", getpid(), pid);
        killpg(pid, SIGKILL);
    }

    return (child_exit_status != 0);
}



