
test.doRSR.stress <- function()
{    
	library(doRSR)
	library(RUnit)

    DEACTIVATED("Test fails and Rich C should look at it")

	registerDoRSR()
	# Be sure we're NOT using doSMP
	currSMPopt <- rxGetOption("useDoSMP")
	rxOptions(useDoSMP=FALSE)
	on.exit(rxOptions(useDoSMP=currSMPopt))
	
	# equivalent to rnorm(3)
	set.seed(14)
	rnormCheck <- rnorm(3)
	set.seed(14)
	checkEquals(unlist(times(3) %dopar% rnorm(1)), rnormCheck) 

	# equivalent to lapply(1:3, sqrt)
	checkEquals(foreach(i=1:3) %dopar%  sqrt(i), lapply(1:3, sqrt))

	# equivalent to colMeans(m)
	m <- matrix(rnorm(9), 3, 3)
	checkEquals(foreach(i=1:ncol(m), .combine=c) %dopar%
  		mean(m[,i]), colMeans(m))

	# normalize the rows of a matrix in parallel, with parenthesis used to
	# force proper operator precedence
	# Need to register a parallel backend before this example will run
	# in parallel
	m1 <- foreach(i=1:nrow(m), .combine=rbind) %dopar%(m[i,] / mean(m[i,]))
	dimnames(m1) <- NULL
	m2 <- t(apply(m, 1, function(x) x/mean(x)))
	checkEquals(m1, m2)

	# simple (and inefficient) parallel matrix multiply
	a <- matrix(1:16, 4, 4)
	b <- t(a)
	b1 <- foreach(b=iter(b, by='col'), .combine=cbind) %dopar%
		(a %*% b)
	b2 <- a %*% b
	checkEquals(b1, b2)

	# split a data frame by row, and put them back together again without
	# changing anything
	d <- data.frame(x=1:10, y=rnorm(10))
	s <- foreach(d=iter(d, by='row'), .combine=rbind) %dopar% d
	checkTrue(identical(s, d))

	# a quick sort function
	qsort <- function(x) {
	  n <- length(x)
	  if (n == 0) {
		x
	  } else {
		p <- sample(n, 1)
		smaller <- foreach(y=x[-p], .combine=c) %:% when(y <= x[p]) %dopar% y
		larger  <- foreach(y=x[-p], .combine=c) %:% when(y >  x[p]) %dopar% y
		c(qsort(smaller), x[p], qsort(larger))
	  }
	}
	sortOut <- qsort(runif(12))
	checkTrue(sortOut[1] <= sortOut[2] && sortOut[2] <= sortOut[3] &&
	          sortOut[3] <= sortOut[4] && sortOut[4] <= sortOut[5] &&
			  sortOut[5] <= sortOut[6] && sortOut[6] <= sortOut[7] &&
			  sortOut[7] <= sortOut[8] && sortOut[8] <= sortOut[9] &&
			  sortOut[9] <= sortOut[10] && sortOut[10] <= sortOut[11] &&
			  sortOut[11] <= sortOut[12])
}


